
***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    TEST_SET_MOD_COMP
**  SUPPORT: sasrdw, Russ Wolfinger
**  DATE:    09/05/07
**  PURPOSE: Test set model comparison 
**  HISTORY: action                                        date     who
**           Created.                                      09/05/07 rdw 
** 
**  MACRO PARAMETERS:
**  TESTSET_PATH    Library path of test data sets
**  TESTSET_NAMES:  Names of test data sets, space delimited
**  TRAIT:          Trait variable name
**  TRAIT_TYPE:     Trait variable type: 
**                  qualitative(categorical), 
**                  quantitative(interval)
**  NCLASS          For qualitative traits, the number of levels
**  MODEL_NAMES:    List of model names, space delimited
**                  (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  OUTPUT_PATH:    Library path of output data sets 
**  OUTPUT_COMP:    Models comparison fit statistics output
**  OUTPUT_EFFECTS: Models effects frequencies output 
**
**  NOTES:
************************************************************************;
%macro test_set_mod_comp(
	testset_path=,
	testset_names=,
	trait=,
	trait_type=,
	numclass=,
	model_names=,
	model_labels=,
	model_label_list=,
	output_path=,
	plot_output_path=,
	output_comp=,
	output_effects=,
	chart_by_testset=,
	chart_by_model=,
	prevalences=proportional
);
 
%global nclasses;
%let nclasses = &numclass;
* DETERMINE WHETHER TEST PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&testset_path) %then %do;
    libname ts "&testset_path";
    %let tslib = ts;
%end;
%else %do;
    %let tslib = work;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&output_path) > 0 and "&output_path" ne '' %then %do;
    libname output "&output_path";
    %let outlib = output;
%end;
%else %do;
    %let outlib = work;
%end;

* RETRIEVE TEST SET NAMES, ASSUME THEY ARE UNIQUE;
%put Test Set Names = &testset_names;
%let i=1;
%do %while(%length(%scan(&testset_names,&i,' ')));
    %let i = %eval(&i+1);
%end;
%let n_ts = %eval(&i-1);
%do i=1 %to &n_ts;
    %global tsname&i;
%end;
%do i=1 %to &n_ts;
    %let tsname&i = %scan(&testset_names,&i,' ');
%end;

* RETRIEVE MODEL NAMES, REMOVE DUPLICATES;
%let umodnames = ;
%let n_models = 0;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
	%let nmi = %scan(&model_names,&i," ");
	%let j=1;
	%let foundit = 0;
	%do %while(%length(%scan(&umodnames,&j," ")));
		%if &nmi = %scan(&umodnames,&j," ") %then %do;
			%let foundit = 1;
			%let j = 100000;
		%end;
		%let j = %eval(&j+1);
	%end;
	%if ^&foundit %then %do;
		%let umodnames = &umodnames &nmi;
		%let n_models = %eval(&n_models+1);
		%global mdlname&n_models;
		%let mdlname&n_models = &nmi;
		%let mdllabel&n_models = %scan(&model_labels,&i,"|");
		%put Model Name &i = &&mdlname&i;
		%put Model Label &i = &&mdllabel&i;
	%end;
	%let i = %eval(&i+1);
%end;

* ASSIGN GLOBALS;
%global GENOMICS_HOME BPV mann auc TSMC;

%let TSMC = 1;
%let got1 = 0;
%let DropMissing = 0;

* LOOP OVER TEST SETS;
%do iter=1 %to &n_ts;

	%put;
	%put Evaluating test set &&tsname&iter, number &iter of &n_ts.;
	%put;

	data cvtest;
		set &tslib..&&tsname&iter;
		if missing(&trait) then delete;
	run;
	
	%NObsVars(cvtest);
	%if ^&nobs %then %do;
		%put ERROR: Test set &&tsname&iter has no nonmissing values for the dependent variable &trait and so;
		%put cannot be used to compare models for prediction performance.   Please add this variable to this data set.;
	%end;
	%else %do;
	
		* add missing trait values back in to the test set;
		data cvtest;
			set &tslib..&&tsname&iter;
		run;
		
		/*	
		COMPUTE MODEL PREDICTIONS,
		TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
		cv_<MODELNAME>_PREDS AND cv_<MODELNAME>_VARS, 
		THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED
		*/	
		%fitmodels(test_dsname=cvtest);

		* LOOP OVER MODELS;
		%do nm=1 %to &n_models;

			%if (&&FM_Error&nm) %then %do;
				%put Results not included for model &nm because errors detected.;
			%end;
			%else %do;
				%let got1 = 1;
				%let mdlname = &&mdlname&nm;
				%let mdllabel=&&mdllabel&nm;

				data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					length Test_Set $32 Model $64 Method $64;
					set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					Test_Set = "&&tsname&iter";
					Model = "&mdllabel";
					Method = "&mdlname";
				run;

				data ts_m&nm._pcum;
					set
					%if (&iter > 1) %then %do;
						ts_m&nm._pcum
					%end;
						cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
				run;

				data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
					length Test_Set $32 Model $64 Method $64;
					set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
					Test_Set = "&&tsname&iter";
					Model = "&mdllabel";
					Method = "&mdlname";
				run;

				data ts_m&nm._vcum;
					set
					%if (&iter > 1) %then %do;
						ts_m&nm._vcum
					%end;
						cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
				run;
			%end;
		%end;
	%end;
%end;

%let DropMissing = 1;
%put got1 = &got1;
%if ^&got1 %then %goto ts_exit;

* SAVE OVERALL TS PREDICTIONS AND EFFECTS;
%let oc = &output_comp;
%let lc = %length(&oc);
%if (&lc > 28) %then %let oc=%substr(&oc,1,2);

data &outlib..&oc._all;
	set 
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
		ts_m&nm._pcum
	%end;
	;
run;

data &outlib..&output_effects;
	set 
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
		ts_m&nm._vcum
	%end;
	;
run;

/*
%if (&CV_ListAll = Yes) %then %do;
    %do nm=1 %to &n_models;
        %let mdlname=&&mdlname&nm;
        proc contents data=ts_m&nm._pcum;
        run;
    %end;
%end;
*/

* COMPUTE ASE, ACC, AUC FOR EACH TEST SET AND EACH MODEL;
%do iter=1 %to &n_ts;
	%do nm=1 %to &n_models;
   	%let mdlname=&&mdlname&nm;
   	%let mdllabel=&&mdllabel&nm;
		data p;
			set ts_m&nm._pcum;
			where Test_Set="&&tsname&iter";
		run;
		
	   %if %upcase(&trait_type)=QUALITATIVE %then %do;	   
	   
	   	* COMPUTE ASE;
			%compute_ase_nominal(data=p,
									  output=%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_ase,
									  actual=actualp,
									  nclasses=&nclasses,
									  mdlname=&mdlname,
									  prevalences=&prevalences,
									  trait=&trait
									  );

			* COMPUTE ACC;
			%compute_acc_nominal(data=p,
									  output=%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_acc,
									  trait=&trait,
									  correct=correct,
									  mdlname=&mdlname,
									  prevalences=&prevalences
									  );

			data %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_acc;
				length Test_Set $32 Model $64 Method $64;
				set %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_acc;
				Test_Set = "&&tsname&iter";
				Model = "&mdllabel";
				Method = "&mdlname";
			run; 

			* APPEND ACC;
			data ts_acc;
				set 
				%if &iter ne 1 or &nm ne 1 %then %do;
	            	ts_acc
				%end;
		            %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_acc;
		   run;

			* COMPUTE MANN-WHITNEY AUC;
			%if &nclasses = 2 %then %do;
				%locate_pevent_isevent(data=p,
											 trait=&trait,
											 event=&_lev1,
											 actual=actualp);

				%let mann = .;
				%compute_mann(data=p,
								 pevent=_pevent,
								 isevent=_isevent);
				%put mann=&mann;

				data %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_mann;
					length Test_Set $32 Model $64 Method $64;
					Test_Set = "&&tsname&iter";
					Model = "&mdllabel";
					Method = "&mdlname";
					AUC=&mann;
				run;

				* APPEND AUC;
				data ts_mann;
					set 
					%if &iter ne 1 or &nm ne 1 %then %do;
						ts_mann
					%end;
						%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_mann;
				run;

		    %end;

	    %end;
	    %else %do;
	        %compute_ase_interval(data=p,
	                              output=%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_ase,
	                              y=&trait,
	                              predict=Predicted_Value,
	                              mdlname=&mdlname);
	                              
		     %compute_harrellc(data=p,
							   	  output=%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_hac,
							 		  TimeVar=&trait,
									  CenVar=,
									  Predict=Predicted_Value);

		     %compute_corr_interval(data=p,
		                           output=%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_corr,
		                           y=&trait,
		                           predict=Predicted_Value,
		                           mdlname=&mdlname);
				  
			  data %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_hac;
					length Test_Set $32 Model $64 Method $64;
			      set %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_hac;
					Test_Set = "&&tsname&iter";
					Model = "&mdllabel";
					Method = "&mdlname";
		     run;

			  data %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_corr;
					length Test_Set $32 Model $64 Method $64;
			      set %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_corr;
					Test_Set = "&&tsname&iter";
					Model = "&mdllabel";
					Method = "&mdlname";
		     run;

			  data ts_hac;
			  		set 
					%if &iter ne 1 or &nm ne 1 %then %do;
						ts_hac
					%end;
						%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_hac;
			  run;

			  data ts_corr;
			  		set 
					%if &iter ne 1 or &nm ne 1 %then %do;
						ts_corr
					%end;
						%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_corr;
			  run;  
			  
	    %end;

		data %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_ase;
			length Test_Set $32 Model $64 Method $64;
			set %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_ase;
			Test_Set = "&&tsname&iter";
			Model = "&mdllabel";
			Method = "&mdlname";
		run;

		* APPEND ASE;
	   data ts_ase;
	      set
			%if &iter ne 1 or &nm ne 1 %then %do;
         	ts_ase
        	%end;
	      	%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),27)))_ase;
	   run;
    %end; * END OF N_MODELS LOOP;
%end; * END OF TESTSET ITERATION LOOP;

* ASE;
data &outlib..&output_comp;
	set ts_ase;
	RMSE = sqrt(ASE);
run;

* MERGE AUC and ACC;
%if %upcase(&trait_type)=QUALITATIVE %then %do;

	%if &nclasses = 2 %then %do;
		data &outlib..&output_comp ;
		  merge &outlib..&output_comp ts_mann;
		run;
	%end;
	
	data &outlib..&output_comp;
		merge &outlib..&output_comp ts_acc;
	run;

%end;
%else %do;
	
	data &outlib..&output_comp;
		merge &outlib..&output_comp ts_hac ts_corr;
	run;

%end;

%let trTestSet = 			%nrbquote(%LocaleGet(TestSet_Text));
%let trAverageSquareError = %nrbquote(%LocaleGet(AverageSquareError_Text));
%let trRootMeanSquareError = %nrbquote(%LocaleGet(RootMeanSquareError_Text));
%let trMeanAbsoluteError = 	%nrbquote(%LocaleGet(MeanAbsoluteError_Text));
%let trAreaUnderROCCurve = 	%nrbquote(%LocaleGet(AreaUnderROCCurve_Text));
%let trHarrellsC = 			%nrbquote(%LocaleGet(HarrellsC_Text));
%let trCorrelation = 		%nrbquote(%LocaleGet(Correlation_Text));
%let trModel =				%nrbquote(%LocaleGet(Model_Text));
%let trAccuracy = 			%nrbquote(%LocaleGet(Accuracy_Text));

data &outlib..&output_comp;
	set &outlib..&output_comp;
	label Test_Set="&trTestSet" ASE="&trAverageSquareError" RMSE="&trRootMeanSquareError" MAE="&trMeanAbsoluteError"
		AUC="&trAreaUnderROCCurve" HARRELL="&trHarrellsC" CORR="&trCorrelation"
		Model="&trModel" Accuracy="&trAccuracy";
	/*S1459553: Fixing error when model labels have blank spaces*/
	drop /*method*/ k iter;
run;
	

* JSL PLOTTING CODE, launched from ExitAndRetrieveLogLstFromJMP in UtilityMacros.sas;
%if ^%length(&plot_output_path) %then %let plot_output_path = &output_path;
filename jslfile "&output_path.&tsPrefix._tsmc.jsl";
data _null_;
	file jslfile encoding="utf-8";
	put "//!";
	put "Names Default to Here(1);";
	put "outputpath = ""&plot_output_path"";";
	put "outdata = ""&output_comp"";";
	put "jg:outname = substr(outdata,1,length(outdata)-4);";
	put "chart_by_testset = ""&chart_by_testset"";";
	put "chart_by_model = ""&chart_by_model"";";
	put "trait_type = ""&trait_type"";";
	%if %upcase(&trait_type)=QUALITATIVE %then %do;
		put "nclasses = &nclasses;";
		put "event = ""&event"";";
		put "trait = ""&trait"";";
	%end;
	put "if(Host Is(Windows),";
	put "   jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
	put "   macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
	put "   jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
	put "   macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"";";
	put ");";
	put "model_names = ""&model_names"";";
	put "model_label_list = &model_label_list;";
   put "include(jslpath||""Test Set Model Comparison Plots.jsl"");";
run;

%ts_exit:;
%mend;
