/*
JMP Genomics Analytical Process 

Name: CrossCorrelation
Author: Russ Wolfinger, SAS Institute Inc.
History:
   sasrdw 15Dec08 pulled code from DataCorrelation

Description: 
	Cross Correlation
	computes all pairwise correlations between two sets of numeric variables,
	tests their significance, and optionally depicts them using a heat map.

	The default type of correlation is
	Pearson product-moment correlation, but
	you may optionally specify Hoeffding D,
	Kendall tau-b, or Spearman rank-order
	correlation.
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData Variables BaseMergeVars CovVars ByVars WeightVar ListVars WhereStmt
WithInData WithVars ListWithVars ListCovVars WhereStmt2
OutType Covariances HeatMap ChunkSize1 ChunkSize2 MultipleTestingMethod Alpha NegLog10pCutoff
ProcCorrOptions CrossCorrelation_OutData CrossCorrelation_OutDataTest
AnnoInData AnnoLabel AnnoGroup AnnoKeepVars AnnoWhereStmt
AnnoInData2 AnnoLabel2 AnnoKeepVars2 AnnoWhereStmt2
OutName OutPath ServerOutPath MacroPath CrossCorrelationScript;

*ProcessBody;

%macro CrossCorrelation;

options nolabel;
* initialization and argument checking;
%let ProcessName = CrossCorrelation;

* Path for SAS macros.;
%include "&MacroPath./UtilityMacros.sas" / nosource;

%if %length(&WeightVar) and %substr(&OutType,1,1) ne P %then %do;
	%put ERROR: You must use Pearson correlation with a Weight Variable.;
	%goto exit;
%end;

%PathName(&InData);
%GetOrigFileName(InData,OrigInData);

%CheckInData(_ExpFlag=0);
%if &exiterror %then %goto exit;

%if ^%length(&Variables) & ^%length(&ListVars) %then %do;
	%put ERROR: The Primary Set of Variables or List-Style Variables must be specified.;
	%let exiterror = 1;
	%goto exit;
%end;
%if ^%length(&WithVars) & ^%length(&ListWithVars) %then %do;
	%put ERROR: The Secondary Set of Variables or List-Style Variables must be specified.;
	%let exiterror = 1;
	%goto exit;
%end;

%if %length(&CovVars &ListCovVars) & (&OutType=Hoeffding or &OutType=Kendall) %then %do;
	%put ERROR: Specifying Covariates for partial correlations may only be done for Pearson and Spearman correlation types.  Please change the correlation type or remove the covariates.;
	%let exiterror = 1;
	%gotot exit;
%end;

libname OutLib "&OutPath"; 

%PathName(&WithInData);
%let WithPath = &TmpPath;
%let WithInData = &TmpName;
libname WithIn "&WithPath";

data ind1;
	set InLib.&InData;
	%if %length(&WhereStmt) %then %do;
		where &WhereStmt;
	%end;
	keep &Variables &BaseMergeVars &ByVars &CovVars &WeightVar &ListVars &ListCovVars;
run;
%if &syserr>4 %then %do;
	%let exiterror=1;
	%goto exit;
%end;

data ind2;
	set WithIn.&WithInData;
	%if %length(&WhereStmt2) %then %do;
		where &WhereStmt2;
	%end;	
	keep &BaseMergeVars &WithVars &ListWithVars;
run;
%if &syserr>4 %then %do;
	%let exiterror=1;
	%goto exit;
%end;

/*
%if ^%length(&BaseMergeVars) %then %do;
	%Nobsvars(ind1);
	%let nobs1=&nobs;
	%Nobsvars(ind2);
	%if &nobs1 ne &nobs %then %do;
		%put ERROR: There are an unequal number of observations between the Primary and Secondary Input Data Sets.;
		%put Merge Key Variables must be specified to avoid inaccurate merging of the data sets.;
		%let exiterror = 1;
		%goto exit;
	%end;
%end;
*/

%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trWith = %nrbquote(%LocaleGet(With_Text));

/* Annotation data */
%let avlen = 0;
%if %length(&AnnoInData) %then %do;
	%if &SASMode ne SDD %then %do;
		%if %sysfunc(fileexist(&AnnoInData)) = 0 %then %do;
			%let exiterror = 1;
			%put ERROR: Specified AnnoInData does not exist.;
			%goto exit;
		%end;
	%end;
	%PathName(&AnnoInData);
	%if &exiterror %then %do;
		%put ERROR: Annotation data set specification is invalid.;
		%goto exit;
	%end;
	%let AnnoData = &TmpName;
	%let AnnoPath = &TmpPath;
	libname AnnoLib "&AnnoPath";
	
	data annodata;
		set AnnoLib.&AnnoData;
		Variable=strip(&AnnoLabel);
		%if %length(&AnnoWhereStmt) %then %do;
			where %unquote(&AnnoWhereStmt);
		%end;
		%if %length(&AnnoKeepVars) %then %do;
			keep Variable &AnnoGroup &AnnoKeepVars;
		%end;
		%else %do;
			drop &AnnoLabel;
		%end;
		label Variable = "&trVariable";
	run;
	
	%let anid=%sysfunc(open(annodata,i));
   %let avlen=%sysfunc(varlen(&anid,%sysfunc(varnum(&anid,Variable))));
   %let rc=%sysfunc(close(&anid));
   	
	%if %length(&AnnoWhereStmt) %then %do;
	
		%global AlleleVars;
		%let AlleleVars = &Variables &ListVars;
		%include "&MacroPath./Reorder.sas" / nosource;		
		%Reorder(ind1,&Variables &ListVars,annodata,Variable,ind1,1);
		%let Variables = &AlleleVars;
		%let ListVars = ;
		%if &exiterror=1 %then %goto exit;
	%end;
	
	proc sort data=annodata;
		by Variable;
	run;	
%end;
/* %if %length(&AnnoGroup)=0 %then %let JournalChrom = No; */

/* Secondary Annotation data */
%if %length(&AnnoInData2) %then %do;
	%if &SASMode ne SDD %then %do;
		%if %sysfunc(fileexist(&AnnoInData2)) = 0 %then %do;
			%let exiterror = 1;
			%put ERROR: Specified AnnoInData2 does not exist.;
			%goto exit;
		%end;
	%end;
	%PathName(&AnnoInData2);
	%if &exiterror %then %do;
		%put ERROR: Secondary annotation data set specification is invalid.;
		%goto exit;
	%end;
	%let AnnoData2 = &TmpName;
	%let AnnoPath2 = &TmpPath;
	libname AnnoLib2 "&AnnoPath2";
	
	data annodata2;
		set AnnoLib2.&AnnoData2;
		With=strip(&AnnoLabel2);
		%if %length(&AnnoWhereStmt2) %then %do;
			where %unquote(&AnnoWhereStmt2);
		%end;
		%if %length(&AnnoKeepVars2) %then %do;
			keep With &AnnoKeepVars2;
		%end;
		%else %do;
			drop &AnnoLabel2;
		%end;
		label With = "&trWith";
	run;
	
	/* Rename With anno vars if there is an anno1 data set */
	%if %length(&AnnoInData) %then %do;
		%let renamestmt=;
		%let _danno2_ = %sysfunc(open(annodata2));
		%let nv2 = %sysfunc(attrn(&_danno2_,NVARS));
		%do v=1 %to &nv2;
			%let vname_&v=%sysfunc(varname(&_danno2_,&v));
			%if (%length(&&vname_&v) > 27) %then %let vname1_&v = %substr(&&vname_&v,1,27);
			%else %let vname1_&v = &&vname_&v;
		%end;
		%let rc=%sysfunc(close(&_danno2_));		
		
		proc datasets nolist;
			modify annodata2;
			rename %do v=1 %to &nv2; %if &&vname_&v ne With %then %quote(&&vname_&v=&&vname1_&v.._With); %end;;
		quit;
	%end;
	
	%if %length(&AnnoWhereStmt2) %then %do;
	
		%let AlleleVars = &WithVars &ListWithVars;	
		%include "&MacroPath./Reorder.sas" / nosource;		
		%Reorder(ind2,&WithVars &ListWithVars,annodata2,With,ind2,1);
		%let WithVars = &AlleleVars;
		%let ListWithVars = ;
		%if &exiterror=1 %then %goto exit;		
		
	%end;
	
	proc sort data=annodata2;
		by With;
	run;	
%end; /* end secondary anno */

%if %upcase(&Covariances)=YES and &OutType^=Pearson %then %do;
	%let Covariances = no;
%end;

%if %upcase(&Covariances)=YES %then %do;
	%let ProcCorrOptions = &ProcCorrOptions COV;
	%let csuff = _cov;
	%let ctype = COV;
	%let cvar = &OutType._Covariance;
	%let clabel = %nrbquote(%LocaleEZX(%nrbquote(&OutType), EIA=1)) %nrbquote(%LocaleGet(Covariance_Text));
	%let cstring = %nrbquote(%LocaleGet(Covariances_Text));
%end;
%else %do;
	%let csuff = _cor;
	%let ctype = CORR;
	%let cvar = &OutType._Correlation;
	%let clabel = %nrbquote(%LocaleEZX(%nrbquote(&OutType), EIA=1)) %nrbquote(%LocaleGet(Correlation_Text));
	%let cstring = %nrbquote(%LocaleGet(Correlations_Text));
%end;

%if ^%length(&NegLog10pCutoff) %then %let NegLog10pCutoff = 0;

/*
* check name for OutData;
%if %length(&OrigInData) < 29 %then %let OID = &OrigInData;
%else %let OID = %substr(&OrigInData,1,28);
%if %upcase(&HeatMap)=YES %then %PrepOutDS(CrossCorrelation_OutData,CrossCorrelation_OutDataName,&OID,&csuff);
%PrepOutDS(CrossCorrelation_OutDataTest,CrossCorrelation_OutDataTestName,&OID,_tst);
*/

* check name for OutName;
%let maxnamelength= 22;
%CheckDataName(_name=OutName,_default=&OrigInData);
%if %length(&OutName)>&maxnamelength %then %let OutName=%substr(&OutName,1,&maxnamelength);

%if %upcase(&HeatMap)=YES %then %PrepOutDS(CrossCorrelation_OutData,CrossCorrelation_OutDataName,&OutName,&csuff);
%PrepOutDS(CrossCorrelation_OutDataTest,CrossCorrelation_OutDataTestName,&OutName,_tst);
%let OID= &OutName;

%let fl=%substr(&OutType,1,1);
%if &fl = F %then %let fl = P;

* use chunking of variables to budget memory for large data sets;
%let ChunkSize1 = %sysevalf(10**&ChunkSize1,integer);
%let ChunkSize2 = %sysevalf(10**&ChunkSize2,integer);
%put ChunkSize1=&ChunkSize1  ChunkSize2=&ChunkSize2;
data vars1;
	set ind1;
	keep &Variables &ListVars;
	if (0);
run;
%let _dsix1_ = %sysfunc(open(vars1));
%let nix1 = %sysfunc(attrn(&_dsix1_,NVARS));
/* Check that valid var names were entered */
%if &nix1=0 %then %do;
	%put ERROR: No variables were found in the Primary Input Data Set with the names specified for the Primary Set of Variables.;
	%let exiterror=1;
	%goto exit;
%end;

/* wehass - get var names and lengths here? */
%let maxvlen=1;
%put nix1=&nix1;
%do v=1 %to &nix1;
	%let vname=%sysfunc(varname(&_dsix1_,&v));
	%if %length(&vname) > &maxvlen %then %let maxvlen = %length(&vname);
	%if %upcase(&HeatMap)=YES %then %let var&v = &vname;
%end;

%let startv1 = 1;
%let endv1 = %eval(&startv1 + &ChunkSize1 - 1);
%if (&endv1 > &nix1) %then %let endv1 = &nix1;
%let morevars1 = 1;

data vars2;
	set ind2;
	keep &WithVars &ListWithVars;
	if (0);
run;
%let _dsix2_ = %sysfunc(open(vars2));
%let nix2 = %sysfunc(attrn(&_dsix2_,NVARS));
/* Check that valid var names were entered */
%if &nix2=0 %then %do;
	%put ERROR: No variables were found in the Secondary Input Data Set with the names specified for the Secondary Set of Variables.;
	%let exiterror=1;
	%goto exit;
%end;

%let maxwlen=1;
%do v=1 %to &nix2;
	%let vname=%sysfunc(varname(&_dsix2_,&v));
	%if %length(&vname) > &maxwlen %then %let maxwlen = %length(&vname);
%end;
%if &maxvlen < &maxwlen %then %let maxvlen = &maxwlen;
%if &maxvlen < &avlen %then %let maxvlen = &avlen;

%if %upcase(&HeatMap)=YES and (&nix2 = 1) %then %do;
	%put NOTE: A clustered heat map will not be displayed because only one secondary variable is specified.;
	%let HeatMap = No;
%end;

%let firstone1 = 1;
%let firstone2 = 1;

%let trNegLog10p = %nrbquote(%LocaleGet(Neglog10p_Text));
%let trNObs = %nrbquote(%LocaleGet(NObs_Text));

%if %length(&CovVars &ListCovVars) %then %do;
	data covars1;
		set ind1;
		keep &CovVars &ListCovVars;
		if (0);
	run;
	%let _dsiz1_ = %sysfunc(open(covars1));
	%let niz1 = %sysfunc(attrn(&_dsiz1_,NVARS));
	/* Check that valid var names were entered */
	%if &niz1=0 %then %do;
		%put ERROR: No variables were found in the Primary Input Data Set with the names specified for the Covariates for Partial Correlations.;
		%let exiterror=1;
		%goto exit;
	%end;
%end;

%do %while(&morevars1);

	data ind1sub;
		set ind1;
		keep &BaseMergeVars &ByVars &WeightVar 
			%do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i)) %end; 
			%if %length(&CovVars &ListCovVars) %then %do; 
				%do i=1 %to &niz1; %sysfunc(varname(&_dsiz1_,&i)) %end; 
			%end;
		;
	run;

	proc sort data=ind1sub;
		by &BaseMergeVars;
	run;

	%let startv2 = 1;
	%let endv2 = %eval(&startv2 + &ChunkSize2 - 1);
	%if (&endv2 > &nix2) %then %let endv2 = &nix2;
	%let morevars2 = 1;
	%let firstone3 = 1;

	%do %while(&morevars2);

		%put startv1=&startv1 endv1=&endv1;
		%put startv2=&startv2 endv1=&endv2;
		
		data ind2sub;
			set ind2;
			keep &BaseMergeVars %do i=&startv2 %to &endv2; %sysfunc(varname(&_dsix2_,&i)) %end; ;
		run;

		* merge the two data sets;	
		proc sort data=ind2sub;
			by &BaseMergeVars;
		run;
		data indat;
			merge ind1sub(in=in_p) ind2sub(in=in_s);
			by &BaseMergeVars;
			if in_p & in_s;
		run;

		%NobsVars(indat);
		%if &nobs=0 %then %do;
			%put ERROR: There are no observations in common in the Primary and Secondary data sets according the the Merge Key Variables and Filtering Settings.;
			%let exiterror=1;
			%goto exit;
		%end;

		%if &syserr > 4 %then %do;
			%let exiterror = 1;
			%goto exit;
		%end;

		%if %length(&ByVars) %then %do;
			proc sort data=indat;
				by &ByVars;
			run;
		%end;

		%if %length(&CovVars &ListCovVars) %then %let tablePrefix = Partial;
		%else %let tablePrefix =; 
		* compute correlations;
		ods exclude all;
		ods noresults;
		proc corr data=indat out&fl=thecorrs &OutType &ProcCorrOptions;
			%if %length(&ByVars) %then %do;
				by &ByVars;
			%end;
			var  %do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i)) %end; ;
			with %do i=&startv2 %to &endv2; %sysfunc(varname(&_dsix2_,&i)) %end; ;
			%if %length(&CovVars &ListCovVars) %then %do;
				partial %do i=1 %to &niz1; %sysfunc(varname(&_dsiz1_,&i)) %end; ;
			%end;
			%if %length(&WeightVar) %then %do;
				weight &WeightVar;
			%end;
			ods output 
				%if &ctype=CORR %then %do;
					&tablePrefix.&OutType.Corr=morecorrs(compress=yes)
				%end;
				%else %do;
					&tablePrefix.COV=morecorrs(compress=yes)
				%end;
			;
		run;
		
		%if &syserr > 4 %then %do;
			%let exiterror = 1;
			%goto exit;
		%end;		
		
		ods exclude none;
		ods results;

		/*
		title "corrs";
		proc print data=thecorrs(obs=10);
		run;

		title "more corrs";
		proc print data=morecorrs(obs=10);
		run;
		*/
		
		%if %upcase(&HeatMap)=YES %then %do;
		
			* main output data set in rectangular format;
			data thecorrs1;
				set thecorrs;
				if _type_="&ctype";
				label _name_ = '_NAME_'
					%do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i))=' ' %end; 
				;
			run;

			%if (&firstone3) %then %do;
				data allthecorrs;
					set thecorrs1;
				run;
				%let firstone3 = 0;
			%end;
			%else %do;
				proc append base=allthecorrs data=thecorrs1;
				run;
			%end;
		%end;

		* stack output data set in order to compute testing statistics;
		* proc corr stores p values under variable names with a P prefix;
		%if &ctype = CORR %then %do;
			data tpn;
				set morecorrs;
				drop &ByVars %do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i)) %end; ;
				if (0);
			run;

			data tp;
				set tpn;
				keep P:;
			run;

			%let _tpid_  = %sysfunc(open(tp));
			%let nv = %sysfunc(attrn(&_tpid_, NVARS));
			%put nv = &nv;

			data tn;
				set tpn;
				keep N:;
			run;

			%let _tnid_  = %sysfunc(open(tn));
			%let nn = %sysfunc(attrn(&_tnid_, NVARS));
			%put nn = &nn;

			* if &nn is zero then there are no missing data, so get count from thecorrs;
			%if ^&nn %then %do;
				data thecorrs2;
					set thecorrs;
					if _type_ = "N";
				run;
				%let firstvar = %sysfunc(varname(&_dsix1_,&startv1));
				%put firstvar = &firstvar;
				%if %length(&ByVars) %then %do;
					data nobsby;
						set thecorrs2;
						NObs = &firstvar;
						label NObs = "&trNObs";
						keep &ByVars NObs;
					run;
				%end;
				%else %do;
					data _null_;
						set thecorrs2;
						call symputx("nobsall",&firstvar);
					run;
					%put nobsall = &nobsall;
				%end;
			%end;

			data odt(compress=yes);
				length Variable $&maxvlen With $&maxwlen;
				set morecorrs;
				With = strip(Variable);
				array corrs{*} %do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i)) %end; ;
				array pvals{*} %do i=1 %to &nv; %sysfunc(varname(&_tpid_,&i)) %end; ;
				%if (&nn) %then %do;
					array nvals{*} %do i=1 %to &nn; %sysfunc(varname(&_tnid_,&i)) %end; ;
				%end;		
				%let j = &startv1;
				%do i=1 %to &nv;
					%let vn = %sysfunc(varname(&_dsix1_,&j));
					%let len = %length(&vn);
					Variable = strip("&vn");
					&cvar = corrs[&i];
					%if (&nn) %then %do;
						NObs = %sysfunc(varname(&_tnid_,&i));
					%end;
					%else %if ^%length(&ByVars) %then %do;
						NObs = &nobsall;
					%end;
					Raw_p = pvals[&i];
					if missing(Raw_p) then NegLog10p = .;
					else if (Raw_p <= 10**(-312)) then NegLog10p = 312;
					else NegLog10p = -log10(Raw_p);
					if not missing(NegLog10p) and NegLog10p ge &NegLog10pCutoff then output;
					%let j = %eval(&j+1);
				%end;
				label &cvar="&clabel" NegLog10p = "&trNegLog10p" Variable = "&trVariable" With = "&trWith" NObs = "&trNObs";
				keep &ByVars Variable With &cvar NObs 
					%if %length(&MultipleTestingMethod) %then %do; Raw_p %end;
					NegLog10p
				; 
			run;
		%end;
		%else %do;
			%let nv = %eval(&endv1-&startv1+1);
			%let nn = 0;
			data odt(compress=yes);
				length Variable $&maxvlen With $&maxwlen;
				set morecorrs;
				With = strip(Variable);
				array covs{*} %do i=&startv1 %to &endv1; %sysfunc(varname(&_dsix1_,&i)) %end; ;
				%let j = &startv1;
				%do i=1 %to &nv;
					%let vn = %sysfunc(varname(&_dsix1_,&j));
					%let len = %length(&vn);
					Variable = strip("&vn");
					&cvar = covs[&i];
					if not missing(&cvar) then output;
					%let j = %eval(&j+1);
				%end;
				label &cvar="&clabel" Variable = "&trVariable" With = "&trWith";
				keep &ByVars Variable With &cvar; 
			run;
		%end;
		
		%if ^&nn and %length(&ByVars) %then %do;
			data odt(compress=yes);
				merge odt nobsby;
				by &ByVars;
			run;
		%end;
		
		%if &ctype = CORR %then %do;
			%let _tpid_ = %sysfunc(close(&_tpid_));
			%let _tnid_ = %sysfunc(close(&_tnid_));
		%end;
		
		%if (&firstone2) %then %do;
			data &CrossCorrelation_OutDataTest(compress=yes);
				set odt;
			run;
			%let firstone2 = 0;
		%end;
		%else %do;
			proc append base=&CrossCorrelation_OutDataTest(compress=yes) data=odt;
			run;
		%end;

		%if &endv2 = &nix2 %then %do;
			%let morevars2 = 0;
		%end;
		%else %do;
			%let startv2 = %eval(&endv2+1);
			%let endv2 = %eval(&startv2 + &ChunkSize2 - 1);
			%if (&endv2 > &nix2) %then %let endv2 = &nix2;
		%end;	

	%end;
	
	%if %upcase(&HeatMap)=YES %then %do;

		%if (&firstone1) %then %do;
			data &CrossCorrelation_OutData;
				set allthecorrs;
			run;
			%let firstone1 = 0;
		%end;
		%else %do;
			data &CrossCorrelation_OutData;
				set &CrossCorrelation_OutData;
				set allthecorrs;
			run;
		%end;
	%end;

	%if &endv1 = &nix1 %then %do;
		%let morevars1 = 0;
	%end;
	%else %do;
		%let startv1 = %eval(&endv1+1);
		%let endv1 = %eval(&startv1 + &ChunkSize1 - 1);
		%if (&endv1 > &nix1) %then %let endv1 = &nix1;
	%end;

%end;

%let _rc_ = %sysfunc(close(&_dsix1_));
%let _rc_ = %sysfunc(close(&_dsix2_));
%if %length(&CovVars &ListCovVars) %then %do;
	%let _rc_ = %sysfunc(close(&_dsiz1_));
%end;

%if %length(&MultipleTestingMethod) %then %do;
	%include "&MacroPath.\MultipleTestingAdjustment.sas";
	%AdjPValueName(method=&MultipleTestingMethod);
   %MultipleTestingAdjustment(data=&CrossCorrelation_OutDataTest,method=&MultipleTestingMethod,out=&CrossCorrelation_OutDataTest);
   %if ^%length(&Alpha) %then %let Alpha = 0;
   data &CrossCorrelation_OutDataTest;
   	set &CrossCorrelation_OutDataTest;
   	if &mtpvar <= &Alpha;
   run;   	
%end;

%*PrepOutFile(CrossCorrelationScript,CrossCorrelationScriptName,&ProcessName Script for &InData..jsl);
%PrepOutFile(CrossCorrelationScript,CrossCorrelationScriptName,&ProcessName Script for &OutName..jsl);

%let labvar = _NAME_;

%let nv = &nix1;

%if %length(&ByVars) %then %do;
	%let i = 1;
	%do %while (%length(%scan(&ByVars,&i,' ')));
		%let bvi = %scan(&ByVars,&i,' ');
		%let bv&i = &bvi;
		*%CheckLabel(&CrossCorrelation_OutData,bvi);	
		%let byv&i = &bvi;
		%let i=%eval(&i+1);
	%end;
	%let nbyv = %eval(&i-1);
%end;

%if %upcase(&HeatMap)=YES %then %do;

	%macro HeatMapMacro;
		put "Here:CCHEATfn = Function({},{default local}," ;
		put "If(IsEmpty(Here:corrdata),";
		put "		Here:corrdata=open(""&ClientOutpath.&CrossCorrelation_OutDataName..sas7bdat"",invisible);";
		%if %length(&ByVars) %then %do;
			%do i=1 %to &nbyv;
				put "		%str(column(%"&&byv&i%") << label(1);)";
			%end;
		%end;
		put "		%str(column(%"&labvar%") << label(1);)";
		put ","; /*Is empty else stmt*/
		put "   Current Data Table(Here:corrdata);";
		put ");";
		put "Vlistbox(";	
		put "   Hierarchical Cluster(";	
		%if %length(&ByVars) %then %do;
			put "      By(" / 
			%do i=1 %to %eval(&nbyv-1);
				"         :&&byv&i," /
			%end;
			"         :&&byv&nbyv" /
			"      ),";        
		%end;
		put "      Y(";
		%do i = 1 %to %eval(&nv-1);
			put "      Name(""&&var&i""), ";
		%end;
		put "      Name(""&&var&i"")), " /
			 "      Method(Ward), Standardize(0), Geometric X Scale(1), ColorClusters(1), " /
			 "      Two way clustering, Number of Clusters(10), Color Map(Blue to Gray to Red), Legend(1)," /
			 "      Dendrogram Scale( Distance Scale )";
		put "   ),";
		put "); ";
		put "); "; 
		put " ";	
		
		put "InsertInto(DataList,{{""Here:corrdata""}});";
		put "InsertInto(TabButtonList, Expr(locale:fill(locale:get(""XHeatMap_Text""), {""&clabel""})));";
		put "InsertInto(TabCodeList,expr(Here:CCHeatfn()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";	
		put "  ";		
		
	%mend;
%end;

%Nobsvars(&CrossCorrelation_OutDataTest);
%if &nobs=0 %then %do;
	%put ERROR: There are no correlations with p-values below the cutoff.  The Output Test Data Set is not created.;
	proc datasets nolist library=OutLib;
		delete &CrossCorrelation_OutDataTestName;
	quit;
	
	%let OutTest=0;
	%let AnnoInData=;
	%let AnnoInData2=;
	%let JournalChrom = No;
	%if %upcase(&HeatMap)=NO %then %goto exit;
%end;
%else %let OutTest=1;

%if %length(&AnnoInData2) %then %do;

	proc sort data=&CrossCorrelation_OutDataTest;
		by With;
	run;
	
	data &CrossCorrelation_OutDataTest;
		length Variable $&maxvlen With $&maxwlen;
		merge annodata2 &CrossCorrelation_OutDataTest(in=stay);
		by With;
		if stay;
	run;
%end;

%if %length(&AnnoInData) %then %do;

	proc sort data=&CrossCorrelation_OutDataTest;
		by Variable;
	run;
	
	data &CrossCorrelation_OutDataTest;
		length Variable $&maxvlen With $&maxwlen;
		merge annodata &CrossCorrelation_OutDataTest(in=stay);
		by Variable;
		if stay;
	run;
	
	proc sort data=&CrossCorrelation_OutDataTest;
		by descending &cvar;
	run;	
	
	%if %length(&AnnoGroup) %then %do;
		proc sort data=&CrossCorrelation_OutDataTest out=countgps nodupkey;
			by &AnnoGroup;
		run;
		
		%Nobsvars(countgps);
		%global NumGps;
		%let NumGps=&nobs;
		%if &NumGps=1 %then %do;
			%goto onegroup;
		%end;
		
		%let dsid=%sysfunc(open(annodata));
			%let GpPos=%sysfunc(varnum(&dsid,&AnnoGroup));
			%global GpType GpLen;
			%let GpType=%sysfunc(vartype(&dsid,&GpPos)); 
			%let GpLen=%sysfunc(varlen(&dsid,&GpPos)); 
		%let rc1=%sysfunc(close(&dsid)); 		
		
		data _null_;
			set countgps end=last_one;
			retain maxlenG 0;
			lenG = lengthn(strip(&AnnoGroup));
			if (lenG > maxlenG) then maxlenG = lenG;
			%if &GpType=C %then %do;
				call symput('Groupn'||left(_n_),trimn(&AnnoGroup));
			 	_sasag=trimn(&AnnoGroup);
			 	j=0;
			 	do until(j=0);  
					j=notname(trimn(_sasag));  
					if j > 0 then substr(_sasag,j,1)="_";  
			 	end;
				call symput('SGroupn'||left(_n_),lowcase(trimn(_sasag)));
			%end;
			%else %do;
			call symput('Groupn'||left(_n_),trimn(left(&AnnoGroup)));
			if &AnnoGroup ne . then call symput('SGroupn'||left(_n_),trimn(left(&AnnoGroup)));
			else call symput('SGroupn'||left(_n_),"_");
			%end;
			if last_one then do;
				call symputx("maxlenAG",trim(left(maxlenG)));
			end;
		run;	
		
		%*global OutName;
		%*let OutName=&OID;
			
		/****************************************************/
		/**** Taken from PrepAGJournal in GenMacros.sas *****/
		/****************************************************/
		
		%let maxon = %eval(31-&maxlenAG);
		%let __OutDataName = &CrossCorrelation_OutDataTestName;
		
		%if %length(%trim(&__OutDataName)) > &maxon %then 
			%let _name = %substr(&__OutDataName,1,&maxon);
		%else %let _name = &__OutDataName;
	
		options label;
	
		/*	data %do i=1 %to &NumGps; OutLib.&_name._&&SGroupn&i %end;;
			set OutLib.&__OutDataName;
			%if (&GpType = N) %then %do;
				if &AnnoGroup = &Groupn1 then output OutLib.&_name._&SGroupn1;
			%end;
			%else %do;
				if trimn(&AnnoGroup) = "&Groupn1" then output OutLib.&_name._&SGroupn1;
			%end;				
			%do i=2 %to &NumGps;
				%if (&GpType = N) %then %do;
					else if &AnnoGroup = &&Groupn&i then output OutLib.&_name._&&SGroupn&i;
				%end;
				%else %do;
					else if trimn(&AnnoGroup) = "&&Groupn&i" then output OutLib.&_name._&&SGroupn&i;
				%end;			
			%end;
		run;
	
	
		* main script needs tabs for each AnnoGroup;
		data _null_;
			file "&__JSLFile";
			put "//!";
			put "Names Default to Here(1);";
			%if %upcase(&HeatMap)=YES %then %do;
				%HeatMapMacro;
			%end;			
			put "open("".\&jrnfilename"");";
		run;
		
		* construct journal;
		data _null_;
			file "&JRNFile";
			put "//!";
			put "Names Default to Here(1);";
			put "List Box(";
			put "   vertical,";
			put "   near,";
			put "   sub(";
			put "      Outline Box( Open(1),";
			put "         title(""&ProcessName Results""),";
			put "         sub(";
			%do i=1 %to &NumGps;
				put "            Button Box(title(""&AnnoGroup &&Groupn&i""),";
				put "               Script Text("" ";
				put "                  _ag_ = \!""&&Groupn&i\!"";";			
				put "                  outds = \!""&_name._&&SGroupn&i...sas7bdat\!"";";
				put "						  OutTestSub= \!""&_name._&&SGroupn&i.._sub\!"";";
				put "                  include(\!"".\&jslfile2name\!"");";
				put "               "")";
				put "            ),";
			%end;
			put "            Button Box(title(""Combined Results""),";
			put "               Script Text("" ";
			put "                  outds = \!""&__OutDataName..sas7bdat\!"";";
			put "                  include(\!"".\&jslfile2name\!"");";

			put "               "")";
			put "            )";
			put "         )";
			put "      )";
			put "   )";
			put ");";
		run;
		*/
		/****************************************************/	
	%end;
	%else %let NumGps = 1;
%end;
%else %do;
	%if &OutTest=1 %then %do;
		proc sort data=&CrossCorrelation_OutDataTest;
			by descending &cvar;
		run;
	%end;
	%let NumGps =1;
%end;

%onegroup:;
      
* create JSL code;
%let ByVarsJSL = ;
%if %length(&ByVars) %then %do;
	%let i = 1;
	%do %while (%length(%scan(&ByVars,&i,' ')));
		%let bvi = %scan(&ByVars,&i,' ');
		%CheckLabel(&CrossCorrelation_OutData,bvi);	
		%let byv&i = &bvi;
		%if (&i>1) %then %do;
			%let ByVarsJSL = &ByVarsJSL.,;
		%end;
		%let ByVarsJSL = &ByVarsJSL :&bvi;
		%let i=%eval(&i+1);
	%end;
	%let nbyv = %eval(&i-1);
%end;

%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = %quote(&CrossCorrelationScript);
%let FullAPName = %str(Cross Correlation);

data _null_;
	file "&CrossCorrelationScript" encoding="utf-8";
	put '//!';
	put "Names Default to Here(1);";
	
	/* kemicl - Put Needed Macros for Drill Downs	*/
	put "CorrExpr = expr(Continuous Distribution(Column(:&clabel),Outlier Box Plot( 0 ) ));";
	put "CVar = ""&CVar"";";
	put "CLabel = "":&CLabel"";";
	put "CType = ""&ctype"";";
	put "CString = ""&cstring"";";
	put "InPath = ""&InPath"";";
	put "WithPath = ""&WithPath"";";
	put "InData = ""&InData"";";
	put "WhereStmt = ""&WhereStmt"";";
	put "WithInData = ""&WithInData"";";
	put "WhereStmt2 = ""&WhereStmt2"";";
	put "BaseMergeVars = ""&BaseMergeVars"";";
	put "ByVars = ""&ByVarsJSL"";";
	put "ByVarsSAS = ""&ByVars"";";
	put "WeightVar = ""&WeightVar"";";
	put "OutPath= ""&OutPath"";";
	put "ClientOutPath=""&ClientOutPath"";";
	put "MacroPath = ""&MacroPath"";";
	

	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";		
	
	/* Overall group */
	%if %upcase(&HeatMap)=YES %then %do;
		%HeatMapMacro;	
	%end;	
	
	/* kemicl - name for drilldown to name subset*/

	%if &OutTest %then %do;
		put "OutTestSub= ""&CrossCorrelation_OutDataTestName._sub"";";	
		put "Here:CCfcn =Function({},{default local}," ;
		put "   If(IsEmpty(Here:pdata),";
		put "      opened=Try(Here:pdata=open(""&ClientOutPath.&CrossCorrelation_OutDataTestName..sas7bdat"", invisible);1,0);";
		put "   );";
		put "   Current Data Table(Here:pdata);";
		put "   include(""$LS_HOME\JSLFiles\Cross Correlation Drill Down.JSL"");";
		put ");";
	
		put "InsertInto(DataList,{{""Here:pdata""}});";
		put "InsertInto(TabButtonList, Expr(locale:get(""TestResults_Text"")));";
		put "InsertInto(TabCodeList,expr(Here:CCfcn()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
	%end;
	%if &NumGps > 1 %then %do;
		put "Here:TabsSectionFlag = 1;";
		%do i=1 %to &NumGps;
			put "CCfcn_&i =Function({},{default local}," ;
			put "   if(isEmpty(Here:pdata_&i),";
			/* build subset dataset if not already existing */
			put '   SASCode = EvalInsert("\[';
			put "   libname outlib ""&OutPath"";";
			put "   data OutLib.&_name._&&SGroupn&i;";
			put "     set OutLib.&CrossCorrelation_OutDataTestName;";
			%if (&GpType = N) %then %do;
				put "    if &AnnoGroup = &&Groupn&i;";
			%end;
			%else %do;
				put "    if trimn(&AnnoGroup) = ""&&Groupn&i"";";
			%end;		  
			put "  run;";
			put " ]\"");";

			put "   jg:CheckSASConnection;";
			put "   SASSubmit(SASCode, NoOutputWindow);";
			put "   SASLog=SASGetLog();";
			put "   if(jg:sasserver!=""Local"",";
			put "     SASGetFile2(jg:serveroutpath||""&_name._&&SGroupn&i...sas7bdat"",";
			put "       clientoutpath||""&_name._&&SGroupn&i...sas7bdat"");";
			put "   );";	
			put "   opened=Try(Here:pdata_&i=open(""&ClientOutPath.&_name._&&SGroupn&i...sas7bdat"", invisible);1,0);";
			put "	 );";
			put "  CurrentDataTable(Here:pdata_&i);";
			put "  include(""$LS_HOME\JSLFiles\Cross Correlation Drill Down.JSL"");";
			put ");";   /* close function*/
			put "_ag_=""&&Groupn&i"";";
			put "InsertInto(DataList,{{""Here:pdata_&i""}});";
			/*put "InsertInto(TabButtonList, ""&AnnoGroup "" || _ag_ || "" "" || Expr(locale:get(""TestResults_Text"")));";*/
			put "InsertInto(TabButtonList, ""&AnnoGroup "" || _ag_ || "" "" || locale:get(""TestResults_Text""));";
			put "InsertInto(TabCodeList,expr(CCfcn_&i()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,0);";		
		%end;
	%end;							
run; 

%if %upcase(&HeatMap)=YES %then %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CrossCorrelationScriptName, 
	"&CrossCorrelation_OutDataName..sas7bdat, &CrossCorrelation_OutDataTestName..sas7bdat", 
	text/plain, yes);
%end;
%else %PackageInsertFile(jsl code from &ProcessName, &OutPath, &CrossCorrelationScriptName, 
	"&CrossCorrelation_OutDataTestName..sas7bdat", text/plain, yes);

/* copy over journal files if in C-S mode 
%if &Server^=Local %then %do;
	%if %upcase(&JournalChrom) = YES %then %do;
		%if &NumGps>1 %then %do;
			%PackageInsertFile(&ProcessName Journal for &OutName, &OutPath, &jrnfilename, , text/plain, yes, no);
			%PackageInsertFile(&ProcessName &OutName by &AnnoGroup, &OutPath, &jslfile2name, , text/plain, yes, no);
			%do i=1 %to &NumGps; 
				%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(XdatafromX_Text)),
					%nrbquote(&AnnoGroup &&SGroupn&i),
					%nrbquote(%LocaleGet(&ProcessName._Label)))), outlib, &_name._&&SGroupn&i, yes, no);
			%end;
		%end;
	%end;
%end;
*/

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CrossCorrelation;

* execute macro;
%CrossCorrelation;
