/*
JMP Genomics Analytical Process

Name: Case-Control Association
Author: Wendy Czika

Description:
Case-Control Association tests for association between
markers and a binary trait using case-control
data.  P-values from these tests, with 
adjustments applied if requested, are plotted
along the marker map.

See the CASECONTROL procedure in the SAS/Genetics
User's Guide for more information.
*/


/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------*/

*ProcessVariables;

%global StudyName InData AlleleVars MarkerFmt GenoDelim TraitVar StrataVars ByVars ListMarkerVars ListTraitVars WhereStmt 
AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoMajAllele AnnoAccession AnnoWhereStmt AnnoMName
AllSNPs CCTests TestType ApplyVIF NullSNPs OROption Perms NSim Seed Conv MTC PValAdj Tau Bandwidth Alpha OnlySig 
OutPath ServerOutPath OutName MacroPath CaseControlAssoc_OutData CaseControlAssocScript;

*ProcessBody;

%macro CaseControlAssociation;

%include "&MacroPath./GenMacros.sas" / nosource;

%GenInit(CaseControlAssociation,28);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Case-Control Association);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;
%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%global OldAlleleVars;
%let OldAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%if %length(&Seed)=0 %then %let Seed=0;

%PrepOutDS(CaseControlAssoc_OutData,CaseControlAssoc_OutDataName,&OutName._cca);
%PrepOutFile(CaseControlAssocScript,CaseControlAssocScriptName,&ProcessName Script for &OutName..jsl);
%RemoveData(outo outcc fe_all);

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

data a1;
	set &ds(obs=1);
run;

proc transpose data=a1 out=at;
	var &TraitVar;
run;

%if &syserr %then %goto exit;

%NObsVars(at);
%global NTrait;
%let Ntrait = &nobs;

%do i=1 %to &Ntrait; 
	%global _tv&i;
%end; 

proc sort data=at;
	by _name_;
run;

data _null_;
	set at;
	length is$ 16;
	is = "_tv" || strip(_n_);
	call symput(is,strip(_name_));
run;

%global Fisher Nonfisher;
%let NonFisher = 1;
%if %index(%quote(&TestType),%str(Fisher))>0 %then %do;
	%let Fisher=1;
	%if %index(%quote(&TestType),%str(Asymptotic))=0 %then %let Nonfisher=0;
%end;
%else %let Fisher=0;


%if %qupcase(&CCTests)=%str(TREND) or %qupcase(&CCTests)=%str(MAX) or %qupcase(&CCTests)=%str(TREND MAX) %then %do;
	%if &NonFisher=0 %then %do;
		%put %str(ERROR: Fisher%'s exact test cannot be performed for the Trend and MAX tests.);
		%let exiterror=1;
		%goto exit;
	%end;
	%else %if &Fisher=1 %then %let Fisher=0;
%end;


%if %length(&Perms)=0 %then %let Perms=0;

%if %length(&ByVars)=0 %then %do;
	%if %length(&StrataVars)=0 %then %do;
		%if &NTrait=1 %then %do;
			%if %upcase(&ApplyVIF) ne YES %then %do;
				%if &Perms=0 %then %do;
					%if %length(&GenoCol)>0 %then %do;
						%if %upcase(&AllSNPs)=YES or &MarkerFmt=%str(Numeric Genotypes) %then %do;
							/*******************************************************************/
							/**** Run FastCC for faster processing given criteria above met ****/
							/*******************************************************************/					
							%include "&MacroPath./FastCC.sas" / nosource;
							%if &runstdcc=0 %then %goto exit; %*return;
							/*******************************************************************/
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
%end;					

%if %upcase(&ApplyVIF)=YES %then %do;
 %let Vif=vif;
 %if %length(&NullSNPs) %then %do;
 	%if %length(&GenoCol)=0 %then %do;
 		%let countns=1;
 		%do %while(%length(%scan(&NullSNPs,&countns)));
 			%let countns=%eval(&countns+1);
 		%end;
 		%put countns=&countns;
 		%let countns=%eval(&countns-1);
 		%if %sysfunc(mod(&countns,2)) ne 0 %then %do;
 			%put ERROR: There must be an even number of Null SNP variables selected when the Allele format is used.;
 			%goto exit;
 		%end;
 	%end;
  %let Vif=%str(vif Nullsnps=(&nullsnps));
 %end;
 %else %do;
  %let Vif=%str(vif Nullsnps=(&AlleleVars));
 %end;
%end;
%else %let Vif = ;

data outcc;
 delete;
run;

%if %upcase(&OROption)=YES %then %let OROption=%quote(or alpha=&alpha);
%else %let OROption=;

%let t=1;
%let CCTests_temp=;
%let OtherTests=;
%do %while (%length(%scan(&CCTests,&t)));
	%let test=%scan(&CCTests,&t);
	%if &test=Trend or &Test=Genotype or &Test=Allele %then %let CCTests_temp = &CCTests_temp &test;
	%else %let OtherTests=&OtherTests %substr(&test,1,3);
	%let t=%eval(&t+1);
%end;
%let CCTests = &CCTests_temp;
%if %length(&CCTests &OtherTests)=0 %then %do;
	%put ERROR: You must select at least one association test.;
	%goto exit;
%end;
%if &NonFisher %then %let TestCols = &CCTests &OtherTests;
%else %let TestCols=;

%if %length(&WhereStmt) %then %do;
 data countobs; 
 	set &ds;
 	_i=0;
  where &WhereStmt;
  keep _i;
 run;
 %Nobsvars(countobs);
%end;
%else %Nobsvars(&ds);

%if &nobs=0 %then %do;
	%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
	%goto exit;
%end;

/* Need smaller block size for Proc Allele that is run in CreateStackedNumericDS*/
%if %length(&OtherTests) or &Fisher %then %do;
	%let BlockSize = %sysfunc(min(100000,%sysevalf(30000000/&nobs,floor)));
%end;
%else %do;
	%let BlockSize = %sysfunc(min(100000,%sysevalf(100000000/&nobs,floor)));
%end;


/*begin: S1055811 - Luciano;
The ndata= option of PROC CASECONTROL requires a data set that has a variable named name. 
In the code, mapdata is assigned to ndata, but prior to that, mapdata has its variable 
&AnnoLabel renamed to name (i.e., rename=(&AnnoLabel=name)). Therefore, it turned out 
that if by chance the mapdata had already a variable named name, the renaming operation 
would cause an error.*/
%let foundname = 0;
%if %sysfunc(exist(mapdata)) %then %do;
	data mapnames; 
		set mapdata;
		where 0;
	run;
			
	%let dsn = %sysfunc(open(mapnames));
	%let foundname = %sysfunc(varnum(&dsn,name));
	%let dsn = %sysfunc(close(&dsn));
	
	%if &foundname > 0 %then %do;
		proc datasets nolist;
			modify mapdata;
			rename name=_nametmp;
		quit;
	%end;
%end;
/*end: S1055811 - Luciano*/

%if &nummark > &BlockSize or %length(&OtherTests)>0 or &Fisher %then %do;
	%let first = 1;
	%let BlockSize=%sysfunc(min(&BlockSize,&nummark));
	%let last = &BlockSize;
	
	data othervars(keep=&ByVars &TraitVar &StrataVars) traits(keep=&ByVars &TraitVar &StrataVars) avars(keep=&AlleleVars);
		set &ds;
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;		
		array getname{*} &AlleleVars;
		%let Block=1;
		%do %until (&first > &nummark);	
			%if %length(&GenoCol) %then %do;
				_f = vname(getname(&first));
				_l = vname(getname(&last));
			%end;
			%else %do;
				_f = vname(getname(%eval(2*&first-1)));
				_l = vname(getname(%eval(2*&last)));
			%end;
						
			call symput("firstvar&Block",_f);
			call symput("lastvar&Block",_l);
			
			%global BlockStart&Block BlockEnd&Block;
			%let BlockStart&Block = &first;
			%let BlockEnd&Block = &last;			
			
			%let first = %eval(&last + 1);
			%let last = %eval(&first + &BlockSize - 1);
			%let Block = %eval(&Block + 1);
			%if &last > &nummark %then %let last = &nummark;
		%end;
	run;
	
	%if %length(&OtherTests)>0 or &Fisher %then %do;
		/* Same code in FastCC.sas */
		options nonotes;
		proc datasets nolist;
			%do i=2 %to &BlockSize;
				append base=traits data=othervars;
			%end;
		quit;
		options notes;	
	%end;
	
	%let NBlocks = %eval(&Block-1);
	%let NChunks=%eval((&NBlocks-1)/100 + 1);
	%let ChunkSize = %sysfunc(min(100,&NBlocks)); /* Number of blocks in chunk */

	%let first = 1; /* first marker in block */
	%let last=&BlockSize; /* last marker in block */
	%let Block = 1;		
	
	%do c=1 %to &NChunks;

		/* Last block in current chunk */
		%let BinC = %sysfunc(min(%eval(&Block + &ChunkSize -1),&NBlocks));	
			
		/* Need to make sure AlleleVars are in consecutive variables */
		/*
		data %do b=&Block %to &BinC; avars&b(keep=&ByVars &TraitVar &StrataVars &&firstvar&b--&&lastvar&b) %end;;
			set othervars; set avars;
		run;
		*/
		
		/*begin: Defect S1046837. Luciano;*/
		%do b=&Block %to &BinC; 	
			data avars&b(keep=&ByVars &TraitVar &StrataVars &&firstvar&b--&&lastvar&b) _tmp_&b(keep=&&firstvar&b--&&lastvar&b);
				set othervars; set avars;
			run;
		
			%NObsVars(_tmp_&b);
			%if &nvars < 1 %then %do;
				%let string1= Order of marker variables specified in the Marker Variables field do not match order of marker variables in the list of Available Variables field.;
				%let string2= Please, goto the General tab and change the order of marker variables in the Marker Variables field.;
				%put ERROR: &string1 &string2; 
				%let exiterror=1;
			 	%goto exit;
			%end;
			
			proc datasets nolist;
				delete _tmp_&b;
			quit;
		%end;
		/*end: Luciano;*/
		
		%if &c=&NChunks %then %do;
			proc datasets nolist;
				delete avars othervars;
			quit;
		%end;
		
		%do %while (&Block le &BinC);
		
			%if %length(&CCTests) and &Nonfisher %then %do;
				/* Note: the following line can be removed when running with SAS 9.2 */
				%if %length(&Vif)>0 and %length(&NullSNPs)=0 %then %let Vif = %str(vif nullsnps=(&&firstvar&Block--&&lastvar&Block));

				%do i=1 %to &Ntrait;

					%put Fitting trait &i of &Ntrait: &&_tv&i;
					proc casecontrol data=avars&Block ndata=mapdata(rename=(&AnnoLabel=name) firstobs=&first obs=&last) &CCTests &Vif outstat=outcct &GenoCol &OROption
										 perms=&Perms %if %length(&Seed) %then seed=&Seed; ; 
						%if %length(&ByVars) %then %do; 
							by &ByVars;
						%end;
						var &&firstvar&Block--&&lastvar&Block;  
						trait &&_tv&i;
						%if %length(&StrataVars) %then %do;
							strata &StrataVars %if &Block=1 %then / info;;
						%end;
						%if %upcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
							format &&firstvar&Block--&&lastvar&Block numtochar.;
						%end;							
					run;

					%if &Perms>0 %then %let Seed=&sysranend;

					data outcc;
					 length Trait $32;
					 set outcc outcct(in=new);
					 if new then do;
							TraitNum = &i;
							Trait="&&_tv&i";
							/* Workaround for bug in PROC CASECONTROL */
							%if %index(&CCTests,Genotype)>0 %then %do;
								if dfGenotype < 1 then ProbGenotype=.;
							%end;	
							%if %length(&OROption) %then %do;
								if not missing(AlleleOddsRatio) then do;
									Log_OddsRatio = log(AlleleOddsRatio);
									StdErr_OddsRatio = (log(LowerCL) - log(AlleleOddsRatio))/probit(&Alpha/2);
								end;
								else do;
									Log_OddsRatio = .;
									StdErr_OddsRatio = .;
								end;	
							%end;
					 end;
					run;		

				%end;
			%end;

			%if %length(&OtherTests) or &Fisher %then %do;
			
				%let OldTrtVar = &TraitVar;	

				/**** For MAX, rec, dom tests - need stacked data ***/
				%CreateStackedNumericDS(avars&Block);
				%if &exiterr=1 %then %goto exit;
								
				* do we need this?;
				%if %length(&ByVars) %then %do;
					proc sort data=genotypes;
						by _order_ &ByVars;
					run;
				%end;
				
				%if &Fisher %then %do;
				/* Similar code in FastCC.sas */
					%let datastmt = ;
					%let Alleletest=0;
					%let DomTest=0;
					%let RecTest=0;
					%let GenoTest=0;
					%if %index(%quote(&CCTests),%str(Allele))>0 %then %do;
						%let datastmt = %quote(&datastmt alleles);
						%let Alleletest=1;
					%end;					
					%if %index(%quote(&CCTests),%str(Genotype))>0 %then %do;
						%let Genotest=1;
					%end;					
					%if %index(%quote(&OtherTests),%str(Rec))>0 %then %do;
						%let datastmt = %quote(&datastmt genotypes_r); 
						%let RecTest=1;
					%end;
					%if %index(%quote(&OtherTests),%str(Dom))>0 %then %do;
						%let datastmt = %quote(&datastmt genotypes_d);
						%let DomTest=1;
					%end;
					
					%if %length(&datastmt) %then %do;
						data &datastmt;
							set genotypes;
							if missing(genotype)=0 then do;
								if genotype=2 then do;
									allele=1;
									%if &AlleleTest %then output alleles;;
									%if &RecTest %then output genotypes_r;;
									%if &DomTest %then output genotypes_d;;
									allele=1;
									%if &AlleleTest %then output alleles;;
								end;
								else if genotype=1 then do;
									allele=0;
									%if &AlleleTest %then output alleles;;
									%if &RecTest %then output genotypes_r;;
									allele=1;
									%if &AlleleTest %then output alleles;;
									%if &DomTest %then output genotypes_d;;
								end;
								else do;
									allele=0;
									%if &AlleleTest %then output alleles;;
									%if &RecTest %then output genotypes_r;;
									%if &DomTest %then output genotypes_d;;									
									allele=0;
									%if &AlleleTest %then output alleles;;
								end;		
							end;
							else do;
								allele=.;
								%if &AlleleTest %then output alleles;;
								%if &RecTest %then output genotypes_r;;
								%if &DomTest %then output genotypes_d;;								
								allele=.;
								%if &AlleleTest %then output alleles;;
							end;							
						run;
					%end;
					
					%let feaexist=0;
					%let fegexist=0;
					%let fedexist=0;
					%let ferexist=0;

					%if &AlleleTest %then %do;
					
						ods output fishersexact=fea(where=(Name1="XP2_FISH") keep=_order_ &ByVars Name1 NValue1 Table rename=(NValue1=ProbAllele_Fisher));
						ods listing close;
						proc freq data=alleles;
							tables allele*(&TraitVar) / chisq;
							by _order_ &ByVars;
							output out=nobs n;
						run;
						
						/*
						data outlib.fea;
							set fea;
						run;
						
						data outlib.nobs;
							set nobs;
						run;
						*/
						
						%if %sysfunc(exist(fea)) %then %do;
							%let feaexist=1;

							data fea;
								length Trait $32;
								merge fea nobs;
								by _order_;
								Trait = strip(scan(Table,2,'*'));
								SampleSize = N/2;
								drop Name1 Table N;
							run;

							%if &NTrait>1 %then %do;
								proc sort data=fea;
									by _order_ Trait;
								run;
							%end;
						%end;
					%end;
					
					%if &RecTest %then %do;
					
						ods output fishersexact=fer(where=(Name1="XP2_FISH") keep=_order_ &ByVars Name1 NValue1 Table rename=(NValue1=ProbRec_Fisher));
						ods listing close;
						proc freq data=genotypes_r;
							tables allele*(&TraitVar) / chisq;
							by _order_ &ByVars;
							output out=nobs n;							
						run;
						
						%if %sysfunc(exist(fer)) %then %do;
							%let ferexist=1;						

							data fer;
								length Trait $32;
								merge fer nobs;
								by _order_;
								Trait = strip(scan(Table,2,'*'));
								SampleSize = N;
								drop Name1 Table N;								
							run;

							%if &NTrait>1 %then %do;
								proc sort data=fer;
									by _order_ Trait;
								run;
							%end;		
						%end;
					%end;		
					
					%if &DomTest %then %do;
					
						ods output fishersexact=fed(where=(Name1="XP2_FISH") keep=_order_ &ByVars Name1 NValue1 Table rename=(NValue1=ProbDom_Fisher));
						ods listing close;
						proc freq data=genotypes_d;
							tables allele*(&TraitVar) / chisq;
							by _order_ &ByVars;
							output out=nobs n;														
						run;

						%if %sysfunc(exist(fed)) %then %do;
							%let fedexist=1;
							
							data fed;
								length Trait $32;
								merge fed nobs;
								by _order_;
								Trait = strip(scan(Table,2,'*'));
								SampleSize = N;
								drop Name1 Table N;
							run;

							%if &NTrait>1 %then %do;
								proc sort data=fed;
									by _order_ Trait;
								run;
							%end;		
						%end;
					%end;		
					
					%if &GenoTest %then %do;
					
						ods output fishersexact=feg(where=(Name1="XP2_FISH") keep=_order_ &ByVars Name1 NValue1 Table rename=(NValue1=ProbGeno_Fisher));
						ods listing close;
						proc freq data=genotypes;
							tables genotype*(&TraitVar) / fisher;
							by _order_ &ByVars;
							output out=nobs n;																					
						run;
						
						%if %sysfunc(exist(feg)) %then %do;
							%let fegexist=1;						

							data feg;
								length Trait $32;
								merge feg nobs;
								by _order_;
								Trait = strip(scan(Table,2,'*'));
								SampleSize = N;
								drop Name1 Table N;
							run;

							%if &NTrait>1 %then %do;
								proc sort data=feg;
									by _order_ Trait;
								run;
							%end;			
						%end;
					%end;							
									
					%let anyfe = %eval(&fegexist+&feaexist+&ferexist+&fedexist);

					%if &anyfe > 0 %then %do;
						data fe;
							merge %if &fegexist %then feg; %if &feaexist %then fea; %if &ferexist %then fer; %if &fedexist %then fed;;
							by _order_ Trait;
						run;					

						proc datasets nolist;
							%if %sysfunc(exist(fe_all)) %then %do;
								append base=fe_all data=fe force;
							%end;
							%else %do;
								change fe = fe_all;
							%end;
							delete %if &fegexist %then feg; %if &feaexist %then fea; %if &ferexist %then fer; %if &fedexist %then fed;;
						quit;		
					%end;				
					
				%end;
			
				%if %length(&OtherTests) and &Nonfisher %then %do;
					/* If there's just 1 trait var and it's character & binary, convert it to 0/1 */
					%if &NTrait=1 %then %do;
						%let dsid=%sysfunc(open(genotypes,i));
						%let trnum=%sysfunc(varnum(&dsid,&TraitVar));
						%let tlen=%sysfunc(varlen(&dsid,&trnum));
						%let ttype=%sysfunc(vartype(&dsid,&trnum));
						%let rc=%sysfunc(close(&dsid));

						%if &ttype=C %then %do;
							%CheckBinaryChar(genotypes,&TraitVar,_len=&tlen);

							%if &nvalue=2 %then %do;
								%let tnum=%substr(%trim(&TraitVar),1,28)_num;
								%put Converting &TraitVar to numeric 0/1 variable &tnum.;
								data genotypes;
									set genotypes;
									if &TraitVar="&cval1" then &tnum=0;
									else if &TraitVar="&cval2" then &tnum=1;
									else &tnum=.;
									%let TraitVar=&tnum;
								run;
							%end;
							%else %do;
								%put ERROR: Trait Variable must be a numeric 0-1 variable.  The Dominant, Recessive, and/or MAX tests are not performed.;
								%let OtherTests=;
								%goto nextblock;
							%end;
						%end;
						%else %do;
							%CheckBinaryChar(genotypes,&TraitVar,_len=8);
							%if (&cval1 ne 0 and &cval1 ne 1) or (&cval2 ne 0 and &cval2 ne 1) %then %do;
								%put ERROR: Trait Variable must have values 0 and 1.  The Dominant, Recessive, and/or MAX tests are not performed.;
								%let OtherTests=;
								%goto nextblock;
							%end;
						%end;
					%end;	
					
					/*
					ods listing close;
					ods output discrete=counts;
					proc multtest data=genotypes out=o %if &Perms>0 and %quote(&OtherTests) ne %str(MAX) %then perm n=&Perms seed=&Seed ;;
						class genotype;
						test ca(&TraitVar / binomial );
						contrast 'add' 2 1 0;
						contrast 'dom' 1 0 0;
						contrast 'rec' 1 1 0;
						%if %length(&StrataVars) %then %do;
							strata &StrataVars;
						%end;
						by _order_ &ByVars;
						where genotype ne .;
					run;
					%if &syserr>4 %then %do;
						%let exiterror=1;
						%let OtherTests=;
						%goto nextblock;
					%end;
					
					ods listing;
					*/
					
					/*begin: S1046828. Luciano*/
					
					ods listing close;
					ods output onewayfreqs=owfgeno nlevels= nlgeno;
					proc freq data=genotypes nlevels;
						tables genotype;
						by _order_ &ByVars;
						where genotype ne .;
					run;

					proc means data=nlgeno noprint;
						var nlevels;
						output out=maxnlgeno max=mxnlgeno;
					run;

					data _null_;
						set maxnlgeno;
						call symputx("maxnl",mxnlgeno,'L');
					run;
					%put maxnl= &maxnl;

					proc sort data=owfgeno;
						by _order_ &ByVars genotype;
					run;

					proc transpose data=owfgeno(keep= _order_ &ByVars table genotype) out=owfgenotrans;
						by _order_ &ByVars table;
					run;
					
					/*data outlib.owfgeno;set owfgeno;run;*/
					
					data owfgenotrans(drop=i);
						set owfgenotrans;
						length case $10;
						array a{*} col1-col&maxnl;
						do i=1 to dim(a);
							if ^missing(a{i}) then
								case=catx('-', case, trim(left(a{i})));
						end;
					run;
					
					/*data outlib.owfgenotrans;set owfgenotrans;run;*/

					ods output onewayfreqs=owfcase nlevels=nlcase;
					proc freq data=owfgenotrans nlevels;
						table case;
						by _order_ &ByVars;
					run;

					proc sort data=owfcase out=owfcase_s nodupkey;
						by case;
					run;

					%let allcases=;
					data _null_;
					     set owfcase_s(keep=case) End=last;
					     Length combcase $ 500;
					     retain combcase;
					     combcase=Catx(' ',combcase,case);
					     if last=1 then 
							call symputx('allcases',trim(left(combcase)),'L');
					run;
					%put &allcases;

					proc sort data=owfgenotrans;
						by _order_ &ByVars;
					run;

					data genotypes;
						merge genotypes owfgenotrans(keep= _order_ &ByVars case);
						by _order_ &ByVars;
					run;

					%let k_=1;
					%let clevel= %scan(&allcases, &k_, ' ');
					ods listing close;
					%do %while(%length(&clevel));
						/*when all coefficient for a contrast are zero, proc multtest issues an error. Thefore,
						I assumed a coefficient 9 when all coefficients are zero to avoid this error. Then,
						in after the proc multtest, I put missing values for outcome of the procedure.*/
						%if "&clevel"="0" %then
							%let contrasts= %str(contrast 'add' 2; contrast 'dom' 1; contrast 'rec' 1;);
						%else %if "&clevel"="1" %then
							%let contrasts= %str(contrast 'add' 1; contrast 'dom' 9; contrast 'rec' 1;);
							/*%let contrasts= %str(contrast 'add' 1; contrast 'dom' 0; contrast 'rec' 1;);*/
						%else %if "&clevel"="2" %then
							%let contrasts= %str(contrast 'add' 9; contrast 'dom' 9; contrast 'rec' 9;);
							/*%let contrasts= %str(contrast 'add' 0; contrast 'dom' 0; contrast 'rec' 0;);*/
						%else %if "&clevel"="0-1" %then
							%let contrasts= %str(contrast 'add' 2 1; contrast 'dom' 1 0; contrast 'rec' 1 1;);
						%else %if "&clevel"="0-2" %then
							%let contrasts= %str(contrast 'add' 2 0; contrast 'dom' 1 0; contrast 'rec' 1 0;);
						%else %if "&clevel"="1-2" %then
							%let contrasts= %str(contrast 'add' 1 0; contrast 'dom' 9 9; contrast 'rec' 1 0;);
							/*%let contrasts= %str(contrast 'add' 1 0; contrast 'dom' 0 0; contrast 'rec' 1 0;);*/
						%else %if "&clevel"="0-1-2" %then
							%let contrasts= %str(contrast 'add' 2 1 0; contrast 'dom' 1 0 0; contrast 'rec' 1 1 0;);
						%else %do;
							%let clevel=;
							%let exiterror=1;
							%let OtherTests=;
							%goto nextblock;
						%end;
						%put contrasts= &contrasts;
						
						data genotypes_sub;
							set genotypes;
							where case eq "&clevel";
						run;
						
						/*data outlib.genotypes_sub;set genotypes_sub;run;*/

						%*put Seed1= &Seed;
						
						ods output discrete=counts_sub;
						proc multtest data=genotypes_sub out=o_sub %if &Perms>0 and %quote(&OtherTests) ne %str(MAX) %then perm n=&Perms seed=&Seed ;;
							class genotype;
							test ca(&TraitVar / binomial );
								&contrasts;
							%if %length(&StrataVars) %then %do;
									strata &StrataVars;
							%end;
							by _order_ &ByVars;
							where genotype ne .;
						run;
						%if &syserr>4 %then %do;
							%let clevel=;
							%let exiterror=1;
							%let OtherTests=;
							%goto nextblock;
						%end;
						
						%*put Seed1_1= &Seed;
						
						/*
						data outlib.o_sub&k_;set o_sub;run;
						data outlib.counts_sub&k_;set counts_sub;run;
						*/
						
						%if "&clevel"="1" or "&clevel"="1-2" %then %do;
							/*when &clevel=1 or 1-2, then dom is not estimable*/
							data o_sub;
								set o_sub;
								if _contrast_ = "dom" then do;
									_value_= .;
									_exp_= .;
									_se_= .;
									raw_p= 1;	
								end;
							run;				
						%end;
						%else %if "&clevel"="2" %then %do;
							/*when &clevel=2, add, dom, and rec are not estimable*/
							data o_sub;
								set o_sub;
								if _contrast_ = "add" or _contrast_ = "dom" or _contrast_ = "rec" then do;
									_value_= .;
									_exp_= .;
									_se_= .;
									raw_p= 1;	
								end;
							run;													
						%end;
					
						%if &k_=1 %then %do;
							data counts;
								set counts_sub;
							run;

							data o;
								set o_sub;
							run;
						%end;
						%else %do;
							data counts;
								set counts counts_sub;
							run;

							data o;
								set o o_sub;
							run;
						%end;

						%let k_=%sysevalf(&k_ + 1);
						%let clevel= %scan(&allcases, &k_, ' ');
					%end;
					ods listing;

					proc sort data=counts;
						by _order_ &ByVars;
					run;

					proc sort data=o;
						by _order_ &ByVars;
					run;
					
					/*end: S1046828. Luciano*/					
									
					%if &Perms>0 and %quote(&OtherTests) ne %str(MAX) %then %do;
						%if %symexist(sysranend) %then %let Seed=&sysranend;
						%*put Seed4= &Seed;
					%end;
					
					%if %index(%quote(&OtherTests),%str(MAX))>0 %then %do;

						data counts;
							set counts;
							%if &NTrait>1 %then %do;
								by _order_ &ByVars variable notsorted;
								retain p0 p1 p2;
								if first.variable then do;	
							%end;
							%else %do;
								by variable _order_ &ByVars;
								retain p0 p1 p2;
								%if %length(&ByVars) %then %do;
									if first.&lastby then do;
								%end;
								%else %do;
									if first._order_ then do;
								%end;
							%end;
								p0=0;
								p2=0;
								p1=0;
								_tot_=numobs;
							end;
							else _tot_+numobs;
							/* Need to do + below for strata vars */
							if Genotype="0" then p0+numobs;
							else if Genotype="1" then p1+numobs;	
							else if Genotype="2" then p2+numobs;
							if last._order_ or last.variable %if %length(&ByVars) %then or last.&lastby; then do;
								p0=p0/_tot_;
								p1=p1/_tot_;
								p2=p2/_tot_;
								var_r=p0*(1-p0);
								var_d=p2*(1-p2);
								var_a=(p1+2*p2)*p0 + (p1+2*p0)*p2;
								if p0>0 and p2>0 then rho_rd=p0*p2/(sqrt(p0*(1-p0)*p2*(1-p2)));
								else rho_rd=0;
								output;
							end;
							keep _order_ &ByVars variable _t var_a var_r var_d rho_rd _tot_;
							rename variable=Trait _tot_=SampleSize;
						run;		
					%end;

					%if %length(&StrataVars) %then %do;
						data o;
							set o;
							where _strat_=".";
						run;
					%end;
					
					/*data outlib.io;set o;run;*/
					
					data o ;
						set o;
						%if &NTrait>1 %then %do;
							by _order_ &ByVars _var_ notsorted;
						%end;
						%else %do;
							by _var_ _order_ &ByVars;
						%end;
						if _se_>0 then Z=abs((_value_-_exp_)/_se_);
						else Z=0;
						%if %index(%quote(&OtherTests),%str(Dom))>0 %then %do;
							retain ZDom ProbDom;
							if _contrast_="dom" then do;
								ZDom=Z;
								ProbDom=%if &Perms>0 %then perm_p; %else raw_p;;
							end;
							keep ZDom ProbDom;
						%end;
						%if %index(%quote(&OtherTests),%str(Rec))>0 %then %do;
							retain ZRec ProbRec;
							if _contrast_="rec" then do;
								ZRec = Z;
								ProbRec=%if &Perms>0 %then perm_p; %else raw_p;;
							end;
							keep ZRec ProbRec;
						%end;
						/*
						retain ZAdd ProbAdd;
						if _contrast_="add" then do;
							ZAdd = Z;
							ProbAdd=%if &Perms>0 %then perm_p; %else raw_p;;
						end;
						keep ZAdd ProbAdd;
						*/					
						%if %index(%quote(&OtherTests),%str(MAX))>0 %then %do;
							retain ZMax;
							if first._order_ or first._var_ %if %length(&ByVars) %then or first.&lastby; then ZMax=Z;
							else if Z>ZMax then ZMax=Z;
							keep ZMax;
						%end;
						if last._order_ or last._var_ %if %length(&ByVars) %then or last.&lastby; then output;
						keep _order_ &ByVars _var_;
						rename _var_=Trait;
					run;
					
					/*data outlib.fo;set o;run;*/

					%if %index(%quote(&OtherTests),%str(MAX))>0 %then %do;
						%if &NTrait>1 %then %do;
								proc sort data=o;
									by _order_ &ByVars Trait;
								run;

								proc sort data=counts;
									by _order_ &ByVars Trait;
								run;
						%end;
						
						/*data OutLib.case_counts; set counts; run;*/
						/*data OutLib.case_o0; set o; run;*/
						%*put Seed2= &Seed;
						
						*S1418044: Begin - Function normal has a limit on the seed, which is 2^31 - 1*;
						%let SeedNorm= &Seed;
						%if &Seed > %sysevalf(2**31 - 1) %then %do;
							%let SeedNorm= %sysfunc(floor(&Seed/1000000));;
						%end;
						%*put SeedNorm= &Seed;
						*S1418044: End;
					
						data o;
							merge counts o;
							%if &NTrait>1 %then %do;
								by _order_ &ByVars Trait;
							%end;
							%else %do;
								by _order_ &ByVars;
							%end;
							seed=&SeedNorm;
							/* From 4.42, p.196 in Casella and Berger */
							ax=sqrt((1+rho_rd)/2);
							if rho_rd < 1 then bx=sqrt((1-rho_rd)/2);
							else bx=0;
							ay=sqrt((1+rho_rd)/2);
							if rho_rd < 1 then by=-sqrt((1-rho_rd)/2);	
							else by=0;
							ProbMaxZ=0;
							do i=1 to &NSim;
								z1=normal(seed);
								z2=normal(seed);
								zr=ax*z1 + bx*z2;
								zd=ay*z1 + by*z2;
								if var_a>0 then za=(sqrt(var_r)*zr + sqrt(var_d)*zd)/sqrt(var_a);
								else za=0;
								if max(abs(zr),abs(zd),abs(za))>=ZMax then probmaxZ+1;	
								 *output;
							end;
							ProbMAX=probmaxz/&NSim; 
							output;
							drop ax bx ay by seed i z1 z2 zr zd za var_r var_a var_d rho_rd probmaxz;
						run;
						
						/*data OutLib.case_o1; set o; run;*/
						
						%*let exiterror=1;
						%*goto exit;
					%end;		

					proc datasets nolist;
						%if &Block>1 %then %do;
							append base=outo data=o force;
						%end;
						%else %do;
							change o = outo;
						%end;
					quit;
						
					/*data outlib.outoi;set outo;run;*/
					
				%end;
				
				%nextblock:
				
				%let TraitVar = &OldTrtVar;								

			%end;/**** END: For MAX, rec, dom tests ***/
		
			proc datasets nolist;
				delete avars&Block;
			quit;

			%let first = %eval(&last + 1);
			%let last = %eval(&first + &BlockSize - 1);
			%let Block = %eval(&Block + 1);
			%if &last > &nummark %then %let last = &nummark;
		%end;

	%end; /* end chunk */
	
%end;
%else %do;

	%do i=1 %to &Ntrait;
						
		%put Fitting trait &i of &Ntrait: &&_tv&i;
		proc casecontrol data=&ds ndata=mapdata(rename=(&AnnoLabel=name)) &CCTests &Vif outstat=outcct &GenoCol &OROption
							 perms=&Perms %if %length(&Seed) %then seed=&Seed; ; 
			%if %length(&ByVars) %then %do; 
				by &ByVars;
			%end;
			var &AlleleVars;  
			%if %upcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
				format &AlleleVars numtochar.;
			%end;	   			
			trait &&_tv&i;
			%if %length(&WhereStmt) %then %do;
			 where &WhereStmt;
			%end;
			%if %length(&StrataVars) %then %do;
			  strata &StrataVars / info;
			%end;
		run;
			
		data outcc;
		 length Trait $32;
		 set outcc outcct(in=new);
		 if new then do;
	 		TraitNum = &i;
	 		Trait="&&_tv&i";
			/* Workaround for bug in PROC CASECONTROL */
			%if %index(&CCTests,Genotype)>0 %then %do;
				if dfGenotype < 1 then ProbGenotype=.;
			%end;
			%if %length(&OROption) %then %do;
				if not missing(AlleleOddsRatio) then do;
					Log_OddsRatio = log(AlleleOddsRatio);
					StdErr_OddsRatio = (log(LowerCL) - log(AlleleOddsRatio))/probit(&Alpha/2);
				end;
				else do;
					Log_OddsRatio = .;
					StdErr_OddsRatio = .;
				end;
			%end;
		 end;
		run;		
	%end;
%end;

/*begin: S1055811 - Luciano;
Renaming _nametmp to its original name.*/
%if &foundname > 0 %then %do;
	proc datasets nolist;
		modify mapdata;
		rename _nametmp= name;
	quit;
%end;
/*end: S1055811 - Luciano*/

%if &NTrait>1 %then %do;
	%if %length(&CCTests)>0 %then %do;
		%let TraitBy = Trait;
		%let JTraitBy = Trait;
	%end;
	%else %do;
		%let TraitBy = Trait;
		%let JTraitBy = Trait;
	%end;
	%if %length(&ByVars)>0 %then %do;
	 %let ByVars = &TraitBy &ByVars;
	 %let ByVarsOnly = &TraitBy &ByVarsOnly;
	 %let JBy = %quote(&JTraitBy,&JBy);
	%end;
	%else %do;
	 %let ByVars = &TraitBy;
	 %let ByVarsOnly = &TraitBy;
	 %let JBy = %quote(&JTraitBy);
	%end;
   %let Nbyval = %eval(&Nbyval*&Ntrait);
%end;

%if (&syserr>4) %then %goto exit;

%if %length(&CCTests)>0 and &Nonfisher %then %do;
	proc sort data=outcc;
	 by Locus;
	run;
	
	%if (&syserr>4) %then %goto exit;

	proc sort data=mapdata;
	 by &AnnoLabel;
	run;

	/* Get variables in order to place LogOR and StdErr next to OR */
	%if %length(&OROption) %then %do;
		data ccnames; 
			set outcc; 
			where 0; 
		run;
		
		data mapnames; 
			set mapdata;
			where 0;
		run;
		
		%let dsn = %sysfunc(open(ccnames));
		%let ORvnum = %sysfunc(varnum(&dsn,AlleleOddsRatio));
		%let dsn2 = %sysfunc(open(mapnames));
		%let nvars = %sysfunc(attrn(&dsn2,NVARS));
	%end;
	
	data temp;
	 %if %length(&OROption) %then %do;
	 	retain 
	 		%do _i_=1 %to &nvars; %sysfunc(varname(&dsn2,&_i_)) %end;
	 		%do _j_=1 %to &ORvnum; %sysfunc(varname(&dsn,&_j_)) %end;
	 		Log_OddsRatio StdErr_OddsRatio;
	 %end;		
	 merge mapdata outcc(in=k rename=(Locus=&AnnoLabel));
	 by &AnnoLabel;
	 label &AnnoLoc= ' ';
	 if k;
	 SampleSize = sum(of NumTrait:);
	run;
	%let mgdata=temp;
	
	%if %length(&OROption) %then %do;
		%let _rc_ = %sysfunc(close(&dsn));
		%let _rc_ = %sysfunc(close(&dsn2));
	%end;
%end;
%else %let mgdata=mapdata;

%if %length(&OtherTests)>0 and &Nonfisher %then %do;
	proc sort data=&mgdata;
		by _order_ ;
	run;

	proc sort data=outo;
		by _order_;
	run;
	
	data temp;
		merge &mgdata outo;
		by _order_;
	run;
	
	%let mgdata=temp;
%end;

%if &Fisher %then %do;

	%if %sysfunc(exist(fe_all))=0 %then %do;
		%put ERROR: Could not perform the Fisher Exact tests.  See the SAS log for more information.;
		%if &NonFisher %then %let TestCols = &CCTests &OtherTests;
		%else %goto exit;
	%end;

	%else %do;
		%let dsf = %sysfunc(open(fe_all));
		%if %sysfunc(varnum(&dsf,ProbAllele_Fisher)) %then %let TestCols = &TestCols Allele_Fisher;
		%if %sysfunc(varnum(&dsf,ProbGeno_Fisher)) %then %let TestCols = &TestCols Geno_Fisher;
		%if %sysfunc(varnum(&dsf,ProbRec_Fisher)) %then %let Testcols = &TestCols Rec_Fisher;
		%if %sysfunc(varnum(&dsf,ProbDom_Fisher)) %then %let Testcols = &TestCols Dom_Fisher;
		%let rc = %sysfunc(close(&dsf));

		proc sort data=&mgdata;
			by _order_ ;
		run;

		data temp;
			merge &mgdata  fe_all;
			by _order_;
		run;
	%end;
%end;
	
proc sort data=temp out=&CaseControlAssoc_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;

%*put TestCols= &TestCols;

%Psmooth(&TestCols, &CaseControlAssoc_OutData, &CaseControlAssoc_OutDataName);
%if (&syserr>4) %then %goto exit;

%PackageInsertDataset(P-value data from &ProcessName, outlib, &OutName._cca, yes, yes, _type=anno);

%if &NByVal > 1 %then %let ssr=0;
%else %let ssr=1;

%if &MarkerFmt = %str(Numeric Genotypes) %then %let _pi = 1;
%else %let _pi = 0;
%global TraitType;
%let TraitType = Binary;
%JSLPvalMap(Case-Control, &CaseControlAssocScript, &CaseControlAssocScriptName, &CaseControlAssoc_OutDataName,_subsetdd=&ssr,_plotinput=&_pi);
%AddDatatoStudy(&CaseControlAssoc_OutDataName,_type=anno);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CaseControlAssociation;

* run the macro;
%CaseControlAssociation;
