/*
JMP Genomics Analytical Process

Name: CrossEvaluation
Author: Russ Wolfinger, SAS Institute Inc.
History:
   sasrdw 08Dec13 initial coding
   sbrlsi 28Jan16 updating code to include topcross, backcross, map info, 
   		multiple generations, optimize cross, and error checking.
   sbrlsi 10Oct18 including automated progeny simulation for outcross
   		and threshold method for progeny selection

Description:
	Cross Evaluation evaluates all 
	crosses among rows of a data set
	containing biallelic numeric genotypes using scoring code for traits.  It outputs
	a model-based mean, max, min, and range for each cross and trait.
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData IDVars SexVar TesterVar ParentalVar WhereStmt ScoringFolder ScoringFilter ScoringCode
	ListMarkerVars MarkerVars AnnoInData ChromNum ChromLab MarkerLab MarkerPos MarkerPos_BP AnnoWhereStmt
	CInData EvalLines MatingType Maternal P1Value P2Value Simulate SimMarkersInScore UseMapInfo MapFunc NSim Seed 
	ComputeSelIndex IndexWeights
	SimMultiGen MultiGenType NumbGen OptimizeCrosses ProgSelectionMethod
	OptSense ThresholdSense ThresholdValues   
	IndexSense IndexCutOff PercentCutOff PercentSense SimRil SimRilMethod
	OutData OutPath ServerOutPath MacroPath Compress CE_OutData CrossEvaluationScript; 

*ProcessBody;

%macro CrossEvaluation;

/* 
The code used to require that chromosome variable be numeric and in ascending order. 
The new code relax that requirement by allowing numeric or character chromosome variable
with levels grouped, not necessarely sorted. In this code, a new numeric chromosome
variable, TmpChromNum, is created such that if the check for grouping of chromosome 
levels in the code below passes, this variable is in ascending order as needed. 
In addition, a new variable, TmpAnnoInOrd, is created to make sure that the chromosome 
and marker order in the input annotation data set is kept the same if sorting is ever
needed in the code.
*/

%global ChromID InAnnoOrd;
%let ChromID= &ChromNum;
%let ChromNum= TmpChromNum;
%let InAnnoOrd= TmpAnnoInOrd;

%let line_length = 256;
%let trimvarlen= 25;

%if %length(&EvalLines)= 0 %then %let EvalLines= No;

%global OptimizeSelection ThresholdSelection IndexSelection PercentSelection;
%let OptimizeSelection= No;
%let ThresholdSelection= No;
%let IndexSelection= No;
%let PercentSelection= No;

%if %index(&ProgSelectionMethod, Optimized) %then %let OptimizeSelection= Yes;
%else %if %index(&ProgSelectionMethod, Threshold) %then %let ThresholdSelection= Yes;
%else %if %index(&ProgSelectionMethod, Index) %then %let IndexSelection= Yes;
%else %if %index(&ProgSelectionMethod, Percent) %then %let PercentSelection= Yes;

/*
SimRilSSD uses the single seed descent method on multiple generations, 
which more realistic, however, it is very intensive. 
SimRilFast is an one-shot simulation strategy that is less 
realistic (but still used in a lot of softwares), however, it is very fast
*/
%*let SimRilFast= &SimRil; 
%*let SimRilSSD= No;

%if %upcase(&SimRil)= YES %then %do;
	%if %upcase(&SimRilMethod) = %upcase(Fast Simulation Method) %then %do;
		%let SimRilFast= Yes;
		%let SimRilSSD= No;
	%end;
	%else %do;
		%let SimRilFast= No;
		%let SimRilSSD= Yes;	
	%end;
%end;
%else %do;
	%let SimRilFast= No;
	%let SimRilSSD= No;
%end;

%if %length(&MapFunc) = 0 %then %do;
	%let MapFunc= Haldane;
%end;

%let ProcessName = CrossEvaluation;
%let FullAPName = %str(Cross Evaluation); 

%include "&MacroPath./UtilityMacros.sas" / nosource;

%GetOrigFileName(InData,OrigInData);

%let ntrait= 0;
%let InData0 = &InData;
%PathName(&InData);
%if &exiterror %then %do;
   %put ERROR: Input data set specification is invalid.;
   %let exiterror= 1;
   %goto exit;
%end;
%let InData = &TmpName;
libname InLib "&TmpPath";

%if %length(&AnnoInData) %then %do;
	%PathName(&AnnoInData);
	%if &exiterror %then %do;
		%put ERROR: Annotation data set (map information) specification is invalid.;
		%let exiterror=1;
		%goto exit;
	%end;

	%let AnnoData = &TmpName;
	libname AnnoLib "&TmpPath";	
%end;

libname OutLib "&OutPath";
%if &exiterror %then %goto exit;

%if %length(&CInData) %then %do;

	%if %length(&IDVars) = 0 %then %do;
		%put ERROR: The usage of the SAS Data Set Indicating Which Crosses to Simulate option requires the specification of ID Variables in the General tab.;
		%let exiterror= 1;
		%goto exit;
	%end;
	
	%PathName(&CInData);
	%if &exiterror %then %do;
		%*put ERROR: Cross data set specification is invalid.;
		%put ERROR: The SAS Data Set Indicating Which Crosses to Simulate from the Options tab is invalid.;
		%let exiterror= 1;
		%goto exit;
	%end;
	%let CInData = &TmpName;
	libname CInLib "&TmpPath";
	
	%let _dsid_ = %sysfunc(open(CInLib.&CInData));
	%let m_n = %sysfunc(varnum(&_dsid_,m_id));
	%let p_n = %sysfunc(varnum(&_dsid_,p_id));
	%let m_type = %sysfunc(vartype(&_dsid_,&m_n));
	%let p_type = %sysfunc(vartype(&_dsid_,&p_n));
	%let rc=%sysfunc(close(&_dsid_));
	
	%put m_n= &m_n;
	%put p_n= &p_n;
	
	%if &m_n = 0 or &p_n = 0 %then %do;
		%if &m_n = 0 and &p_n = 1 %then %do;
			%put ERROR: The SAS Data Set Indicating Which Crosses to Simulate from the Options tab is missing the required variable m_id.;				
		%end;
		%else %if &m_n = 1 and &p_n = 0 %then %do;
			%put ERROR: The SAS Data Set Indicating Which Crosses to Simulate from the Options tab is missing the required variable p_id.;				
		%end;		
		%else %do;
			%put ERROR: The SAS Data Set Indicating Which Crosses to Simulate from the Options tab is missing the required variables m_id and p_id.;						
		%end;
		%let exiterror= 1;
		%goto exit;
	%end;	

	data cin;
		set CInLib.&CInData;
		if m_id = p_id then _self_=1;
		else _self_=0;
		keep m_id p_id _self_;
	run;
	%if &syserr > 4 %then %goto exit;

	proc means data=cin min max noprint;
		var _self_;
		output out= matype min= _min_ max= _max_;
	run;

	data _null_;
		set matype;
		call symputx("mintype", _min_);
		call symputx("maxtype", _max_);
	run;

	%if &mintype = &maxtype %then %do;
		%if &mintype = 1 %then %do;
			%let MatingType= Include self only;
		%end;
		%else %do;
			%let MatingType= Include outcross only;
		%end;
	%end;
	%else %do;
		%let MatingType= Include self and outcross;
	%end;
	
	%*let Orig_SexVar= &SexVar;
	%*let SexVar=;
%end;

* check name for OutData;
/*
%if ^%length(&OutData) %then %AddSuffixToDataName(_ce,&OrigInData,OutData,_SDS=1);
%else %CheckDataName(_name=OutData,_default=,_SDS=1);
*/
%let maxnamelength= 28;
%CheckDataName(_name=OutData,_default=&OrigInData,_SDS=1);
%if %length(&OutData)>&maxnamelength %then %let OutData=%substr(&OutData,1,&maxnamelength);
%PrepOutDS(CE_OutData,CE_OutDataName,&OutData._ce);

* create name for jsl file;
%if ^%symexist(ClientScoringFolder) %then %let ClientScoringFolder = &ScoringFolder;
%PrepOutFile(CrossEvaluationScript,CrossEvaluationScriptName,&ProcessName Script for &OutData..jsl);
%global _mainjslfile_;
%let _mainjslfile_ = %quote(&CrossEvaluationScript);

%if %length(&ListMarkerVars &MarkerVars)=0 %then %do;
	%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
	%let exiterror=1;
	%goto exit;
%end;
%let OrigMarkerVars = &MarkerVars;
%let MarkerVars = &MarkerVars &ListMarkerVars;

%if %length(&TesterVar) and %length(&ParentalVar) %then %do;
	%put ERROR: Both Tester Lines Variable and Parental Lines Variable were specified in the General Tab. Please, specify just one of them.;
	%let exiterror= 1;
	%goto exit;
%end;

%if (%length(&SexVar) and 
	(%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) or
	%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only))
	)) %then %do;
	%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) %then %do;
		%put ERROR: Include self and outcross mating type cannot be specified when a Sex Variable is specified. Please, choose Include outcross Only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
	%else %if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only)) %then %do;
		%put ERROR: Include self only mating type cannot be specified when a Sex Variable is specified. Please, choose Include outcross only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
%end;

%if %length(&TesterVar) or %length(&ParentalVar) %then %do;
	**MatingType option is unavailable when either TesterVar or ParentalVar is specified in the General tab**;
	%let MatingType= Include outcross only;
%end;

%if (%length(&TesterVar) and 
	(%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) or
	%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only))
	)) %then %do;
	%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) %then %do;
		%put ERROR: Include self and outcross mating type cannot be specified when a Top Cross Tester Lines Indicator Variable is specified. Please, choose Include outcross only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
	%else %if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only)) %then %do;
		%put ERROR: Include self only mating type cannot be specified when a Top Cross Tester Lines Indicator Variable is specified. Please, choose Include outcross only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
%end;

%if (%length(&ParentalVar) and 
	(%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) or
	%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only))
	)) %then %do;
	%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) %then %do;
		%put ERROR: Include self and outcross mating type cannot be specified when a Backcross Parental Lines Variable is specified. Please, choose Include outcross only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
	%else %if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only)) %then %do;
		%put ERROR: Include self only mating type cannot be specified when a Backcross Parental Lines Variable is specified. Please, choose Include outcross only in the Options tab.;
		%let exiterror= 1;
		%goto exit;	
	%end;
%end;

%let Selfs= No;
%if (%length(&SexVar &TesterVar &ParentalVar) = 0) and 
	(%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) or
	%nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only))) %then %do;
	%let Selfs= Yes;
%end;

%if %upcase(&SimMultiGen)= YES and %length(&ParentalVar) = 0 and %length(&SexVar) > 0 %then %do;
	%put ERROR: When Sex Variable is specified, the option to simulate mulitple generations is available only when a Backcross Parental Lines Variable is also specified.;
	%let exiterror= 1;
	%goto exit;
%end;

%if %upcase(&SimMultiGen)= YES and %nrbquote(%upcase(&MultiGenType)) = BACKCROSS and %length(&ParentalVar) = 0 %then %do;
	%put ERROR: The backcross mating type cannot be specified for multiple generations unless a variable is specified in the Backcross Parental Lines Variable.;
	%let exiterror= 1;
	%goto exit;
%end; 

%if %upcase(&SimMultiGen)= YES and %nrbquote(%upcase(&MultiGenType)) = SELF and (%length(&ParentalVar) or %length(&SexVar) or %length(&TesterVar)) %then %do;
	%put ERROR: The self mating type cannot be specified for multiple generations when either a Sex Variable or Top Cross Tester Lines Indicator Variable or Backcross Parental Lines Variable is specified.;
	%let exiterror= 1;
	%goto exit;
%end; 

%put SimMultiGen= SimMultiGen;
%put MultiGenType= &MultiGenType;
%if %upcase(&SimMultiGen)= YES and %nrbquote(%upcase(&MultiGenType)) = %nrbquote(OUTCROSS) and (%length(&ParentalVar) or %length(&SexVar) or %length(&TesterVar)) %then %do;
	%put ERROR: The outcross mating type cannot be specified for multiple generations when either a Sex Variable or Top Cross Tester Lines Indicator Variable or Backcross Parental Lines Variable is specified.;
	%let exiterror= 1;
	%goto exit;
%end; 

%if %upcase(&IndexSelection) = YES and %upcase(&ComputeSelIndex) ^= YES %then %do;
	%put ERROR: The Index Selection option requires that the Compute Selection Index in the Analysis tab must be checked.; 
	%let exiterror= 1;	
	%goto exit;
%end;

%if %upcase(&ComputeSelIndex) = YES and %upcase(&EvalLines) ^= YES and &Simulate ^= Yes %then %do;
	%put ERROR: The option to compute selection index is only available when either the option to Evaluate individuals in the Input SAS Data Set without making crosses is checked or Simulate progenies is checked.; 
	%let exiterror= 1;	
	%goto exit;
%end;

%put NumbGen= &NumbGen;
%let NSelf= 0;
%let NBack= 0;
%let NnonSelf= 0; 
%if %nrbquote(%upcase(&MultiGenType)) = SELF %then %let NSelf= &NumbGen;
%else %if %nrbquote(%upcase(&MultiGenType)) = BACKCROSS %then %let NBack= &NumbGen;
%else %let NnonSelf= &NumbGen; 

*S1240091: preventing problem if code ends without closing files*;
%if %symexist(_dsix_in) %then %do;
	%let _dsix_in = %sysfunc(close(&_dsix_in));
%end;

**name of markers in the input data set**;
data MkInFile;
	set InLib.&InData(keep = &MarkerVars obs=0);
run;

%let _dsix_in = %sysfunc(open(MkInFile));
%let nm_in = %sysfunc(attrn(&_dsix_in,NVARS));
%put total number of markers in the input file= &nm_in;

*checking if the input data set has variables with suffix _a or _d, which indicates that the input data has been recoded to additive and dominance variables*;
%do i=1 %to %sysfunc(min(&nm_in,100)); 
	%let AorDvar= %sysfunc(substr(%sysfunc(left(%sysfunc(reverse(%sysfunc(varname(&_dsix_in,&i)))))),1,2));
	%put AorDvar= &AorDvar; 
	*S1486026: Changing _A and _D to lowercase to avoid error with marker variables that have this same sufixes*;
	%if &AorDvar= a_ or &AorDvar= d_ %then %do;
		%let String1= The input data set contains marker variables with either suffix "_a" or "_d", which indicates the presence of recoded additive and dominance variables.;
		%let String2= The input data must not contain recoded marker variables. Please, use an input data set with numeric genotypes coded as 0, 1, and 2.;
		%put ERROR: &String1 &String2;
		%let exiterror= 1;
		%goto exit;
	%end;
%end;	

proc transpose data=MkInFile out=MkInFileT(rename=(_name_=_MarkerInFile_));
	var 	
	%do i=1 %to &nm_in; 
		%sysfunc(varname(&_dsix_in,&i))
	%end;	
	;
run;
/*data OutLib.c_MkInFileT; set MkInFileT; run;*/

* parse scoring code, assuming a linear function with each term on a new row;
%let itree= ;
%let FlagDepVarChar= 0;
%let k = 1;
libname ScoreLib "&ScoringFolder.\";		
%let i = 1;
%do %while(%length(%scan(&ScoringCode,&i,'|')));
	%let Code = %scan(&ScoringCode,&i,'|');
	%if ^%sysfunc(find(&Code, .sas7bdat)) %then %do; 
		**score file NOT FROM Partition Trees**;
		%let itree= &itree 0;
	
		data score&i;
			length instring $ 32767 _name_ $50 _label_ $50;
			infile "&ScoringFolder.\&Code" truncover lrecl=32767;
			input instring $1-32767;
			if _n_ = 1 then do;
				nidx = trim(left(_n_));
				* remove "_score" from the end of the trait name;
				call symputx("trait&i",substr(scan(instring,1," *=+;"),1,length(scan(instring,1," *=+;"))-6));
				call symputx("trait2&i",scan(instring,1," *=+;"));
				call symputx("t&i.intercept",scan(instring,2," *=+;"));
			end;
			else do;
				nidx = trim(left(_n_-1));
				/*
				if lengthn(scan(instring,1," *=+;")) then do;
					_label_ = scan(instring,1," *=+;");
					_name_ = scan(instring,2," *=+;");		
					output;
					call symputx("nm"||"&i",nidx);
				end;
				*/
				if lengthn(scan(instring,1," *+;")) then do;
					_label_ = scan(instring,1," *+;");
					_name_ = trim(left(scan(instring,2," *+;")));
					*the if statements are needed when score comes from Logistic Regression*;
					if length(scan(instring,3," *+;")) then _name_ = trim(left(_name_))||trim(left(scan(instring,3," *+;")));
					if length(scan(instring,4," *+;")) then _name_ = trim(left(_name_))||trim(left(scan(instring,4," *+;")));
					if length(scan(instring,5," *+;")) then _name_ = trim(left(_name_))||trim(left(scan(instring,5," *+;")));
					*if(index(substr(_name_,1,1),'=')) then _name_= substr(trim(left(_name_)),2);
					output;
					*call symputx("nm"||"&i",nidx);
				end;
			end;
			keep _name_ _label_;
		run;	
		/*data OutLib.c_score&i; set score&i; run;*/

		%put trait&i= &&trait&i;
		%put trait2&i= &&trait2&i;
		%put t&i.intercept= &&t&i.intercept;
	%end;
	%else %do;
		**score file FROM Partition Trees**;
		%let itree= &itree 1;
		
		%let ScoreFile&k= %substr(&Code, 1, %eval(%sysfunc(find(&Code, .)) - 1));
		%put ScoreFile&k= &&ScoreFile&k;
		
		data score&i(keep= _name_ _label_) levels&i;
			length _name_ $50 _label_ $50;
			set ScoreLib.&&ScoreFile&k;
			_lag1lab_= lag(label);
			*_lag2lab_= lag2(label);
			if _n_ = 6 then do;
				call symputx("trait&i",label);
			end;		
			if _n_ = 7 then do; 
				if upcase(label) = "INTERVAL" then do;
					call symputx("DepVarType&i", "CONTINUOUS");
				end;
				else do;
					call symputx("DepVarType&i", "DISCRETE");
				end;
				call symputx("NumbLevels&i", x);
			end;	
			if _n_ = 8 then do;
				if find(label,"$") then do;
					call symputx("DepVarStyle&i", "CHARACTER");
					call symputx("FlagDepVarChar", 1);					
				end;
				else do;
					call symputx("DepVarStyle&i", "NUMERIC");
				end;
			end;
			if upcase(_lag1lab_) = "I VALUE" and upcase(label) ^= "END" then do;
				_name_ = label;
				output score&i;
			end;
			if ID = -1 then do;
				output levels&i;
			end;
		run;
		%put trait&i = &&trait&i;

		%let trait_pt&k = &&trait&i;
		%put trait_pt&k = &&trait_pt&k;
	
		%let DepVarType3&k = &&DepVarType&i;
		%put DepVarType3&k = &&DepVarType3&k;
	
		%let NumbLevels3&k = &&NumbLevels&i;
		%put NumbLevels3&k = &&NumbLevels3&k;
		
		%if &&NumbLevels3&k > 0 %then %do;		
			* binary and nominal traits*;
		
			data levels&i;
				set levels&i;
				if _n_ > 1 and _n_ <= &&NumbLevels3&k + 1 then output;
			run;
			
			proc sort data=levels&i;
				by label;
			run;
			
			/*data OutLib.c_levels&i; set levels&i; run;*/
			
			data _null_;
				set levels&i;
				call symputx("vlevel&i"||trim(left(_n_)),label);
			run;
			
			%do z=1 %to &&NumbLevels3&k;
				%let PTvlevel&k&z= &&vlevel&i&z;
				%put PTvlevel&k&z= &&PTvlevel&k&z;
			%end;
		%end;
		
		%if &&NumbLevels3&k = 2 %then %do;
			* binary trait*;
			%let trait2&i = &&trait&i..&&vlevel&i.1._score;
		%end;
		%else %do;
			* nominal and continuous traits*;
			%let trait2&i = &&trait&i.._score;				
		%end;
		%put trait2&i= &&trait2&i;		
	
		%let trait2_pt&k = &&trait2&i;
		%put trait2_pt&k = &&trait2_pt&k;
	
		%let k= %eval(&k + 1);
	%end;
	
	/*data OutLib.c_score&i; set score&i; run;*/
	
	%let i = %eval(&i+1);
%end;
%let ntrait = %eval(&i-1);
%let ntree = %eval(&k-1);

%if &ntrait < 2 and %upcase(&ComputeSelIndex) = YES %then %do;
	%put ERROR: The option to compute selection index is only available when two or more score files are specified in the Scoring Code Files from the General tab.;
	%let exiterror = 1;
	%goto exit;
%end;

%if &ntrait > 1 and %upcase(&PercentSelection) = YES and %upcase(&ComputeSelIndex) = YES %then %do;
	%let String1= The percent progeny selection method is only available for univariate selection.; 
	%let String2= Because, there are more than one score file specified in the Scoring Code Files from the General tab, and the option to compute selection index is checked,; 
	%let String3= the top p-percent progenies will be selected based on the values for the selection index.;
	%put WARNING: &String1 &String2 &String3;
%end;

%if &ntrait > 1 and %upcase(&PercentSelection) = YES and %upcase(&ComputeSelIndex) ^= YES %then %do;
	%let String1= The percent progeny selection method is only available for univariate selection.; 
	%let String2= Because, there are more than one score file specified in the Scoring Code Files from the General tab, and the option to compute selection index is NOT checked,; 
	%let String3= the analysis was halted. Please, either specify only one score file or check the option to compute selection index.;	
	%put ERROR: &String1 &String2 &String3;
	%let exiterror = 1;
	%goto exit;
%end;

%let PercentDirection = ;
%let ValueDirection = %str(<);
%if %index(&PercentSense, High) %then %do;
	%let PercentDirection = DESCENDING;
	%let ValueDirection = %str(>);
%end;

%if &ntree > 0 and &FlagDepVarChar = 1 %then %do;
	%put ERROR: There is at least one tree model file among the score files for which the response variable is not numeric.;
	%let exiterror= 1;
	%goto exit;
%end;

%if (&ntree > 0 and &Simulate ^= Yes) and (%upcase(&EvalLines) ^= YES) %then %do;
	%if &ntree = 1 %then %do;
		%put ERROR: There is a tree model file among the score files. Therefore, the Simulate Progenies option in the Option tab must be checked.;
	%end;
	%else %do;
		%put ERROR: There are some tree model files among the score files. Therefore, the Simulate Progenies option in the Option tab must be checked.;	
	%end;
	%let exiterror= 1;
	%goto exit;
%end;

%*goto exit;

* save all of the markers and coefficients in data set names and labels for speed;
%let ReCodeScores = ;
%do i=1 %to &ntrait;	
	%let Flag_AD&i= 0;
	%let Flag_EQ&i= 0;
	data flagdat&i;
		set score&i;
		*_finda= find(upcase(_name_), "_A");
		*_findd= find(upcase(_name_), "_D");
		/*
		if find(upcase(_name_), "_A") or find(upcase(_name_), "_D") then do;
			call symputx("Flag_AD"||"&i",1);
		end;
		*/
		if find((_name_), "_a") or find((_name_), "_d") then do;
			call symputx("Flag_AD"||"&i",1);
		end;		
		if find(upcase(_name_), "=") and ^find(upcase(_name_), "_SCORE") then do;
			call symputx("Flag_EQ"||"&i",1);
		end;		
	run;
	%put Flag_AD&i= &&Flag_AD&i;
	%put Flag_EQ&i= &&Flag_EQ&i;	

	/*data outlib.c_flagdat&i; set flagdat&i; run;*/

	%if ^%scan(&itree, &i) %then %do;
		%if &&Flag_AD&i or &&Flag_EQ&i %then %do;

			%let ReCode0= ReCodeCE&i;
			%let ReCodeScores = &ReCodeScores &Recode0..sas;
			%put ReCodeScores = &ReCodeScores;

			data score&i;	
				%if &&Flag_AD&i %then %do;
					*S1313458: insufficient authorization to access c:\Windows\System32\ **;
					file "&OutPath.&ReCode0..sas"  encoding="utf-8";
					*file "&ReCode0..sas"  encoding="utf-8";
				%end;
				set score&i(rename= (_name_= _nameold_)) end=_eof_;
				_ord_ = _n_;
				_findeq_= find(upcase(_nameold_), "="); 
				/*
				_findA_= find(upcase(_nameold_), "_A");
				_findD_= find(upcase(_nameold_), "_D");
				*/
				_findA_= find((_nameold_), "_a");
				_findD_= find((_nameold_), "_d");				
				if _findeq_ then do;
					_nametmp0_= substr(_nameold_, 1, _findeq_ - 1);
					_nametmp0_= compress(upcase(_nametmp0_), "()");
					if _findA_ or _findD_ then do;	
						_nametmp_= substr(_nametmp0_, 1, length(_nametmp0_)-2);
						_suffix_= substr(_nametmp0_, length(_nametmp0_)-1, length(_nametmp0_));
					end;
					else do;
						_nametmp_= _nametmp0_;
						_suffix_= "";
					end;
				end; 
				else do;
					_nametmp_= substr(_nameold_, 1, length(_nameold_)-2);
					_suffix_= substr(_nameold_, length(_nameold_)-1, length(_nameold_));
				end;
				if _n_ = 1 then put "&&trait&i.._Score = &&t&i.intercept";
				if (_suffix_) = "_a" /*upcase(_suffix_) = "_A"*/ then do;
					*_addvar_= -1;
					_name_= _nametmp_;
					if ^_findeq_ then do; 
						put "+" _label_ "*(" _name_ "-1)";
					end;
					else do;
						if find(upcase(_nameold_), "-1") then do;
							put "+" _label_ "*(" _name_ "=0)";
						end;
						else if find(upcase(_nameold_), "0") then do; 
							put "+" _label_ "*(" _name_ "=1)";
						end;
						else if find(upcase(_nameold_), "1") then do;
							put "+" _label_ "*(" _name_ "=2)";
						end;
					end;
				end;
				else if (_suffix_) = "_d" /*upcase(_suffix_) = "_D"*/ then do;
					*_domvar_= 1;
					_name_= _nametmp_;
					if ^_findeq_ then do; 
						put "+" _label_ "*(" _name_ "=1)";
					end;
					else do;
						if find(upcase(_nameold_), "0") then do;
							put "+" _label_ "*(" _name_ "=1)";
							put "+" _label_ "*(" _name_ "=2)";
						end;
						else if find(upcase(_nameold_), "1") then do; 
							put "+" _label_ "*(" _name_ "=1)";
						end;
					end;
				end;
				else do;
					if ^_findeq_ then do;
						_name_= _nameold_;
						if ^find(upcase(_label_),"CALL") and ^find(upcase(_label_), "_SCORE") then do;
							put "+" _label_ "*" _name_;
						end;
						else do;
							put ";";
							put _label_ _name_;
						end;
					end;
					else do;
						_name_= _nametmp0_;
						if ^find(upcase(_label_),"CALL") and ^find(upcase(_label_), "_SCORE") then do;
							put "+" _label_ "*" _nameold_;
						end;
						else do;
							put ";";
							put _label_ _nameold_;
						end;
					end;				
				end;
				
				if _eof_ then do;
					put ";";
				end;
				*drop _nameold_ _nametmp_ _nametmp0_ _suffix_;
			run;	

			proc sort data=score&i out=score&i nodupkey;
				by _name_;
			run;

			proc sort data=score&i out=score&i(drop= _ord_);
				by _ord_;
			run;

			/*data OutLib.c2_score&i; set score&i; run;*/					
		%end;
	%end;
	%else %do;
		%if &&Flag_AD&i %then %do;
			%put ERROR: Score files from partition tree models cannot be processed when they have additive and dominance genetic marker variables.;
			%let exiterror=1;
			%goto exit;
		%end;
	%end;

	data score&i;
		set score&i;
		if ^find(upcase(_label_), "CALL") and ^find(upcase(_label_), "_SCORE") then do; 
			output;
		end;
	run;	
	/*data OutLib.c3_score&i; set score&i; run;*/

	%NObsVars(score&i);
	%let nm&i= &nobs;

	*S1240091: preventing problem if code ends without closing files*;
	%if %symexist(_dsix_&i) %then %do;
		%let _dsix_&i = %sysfunc(close(&&_dsix_&i)); 
	%end;

	proc transpose data=score&i out=scoret&i(drop=_name_);
		var _name_;
	run;
	/*data OutLib.c_scoret&i; set scoret&i; run;*/

	%let _dsix_&i = %sysfunc(open(scoret&i));
%end;

%*goto exit;

/*
*S1240091: preventing problem if code ends without closing files*;
%if %symexist(_dsix_) %then %do;
	%let _dsix_ = %sysfunc(close(&_dsix_)); 
%end;

* save all referenced marker names in the input data set;
data mva;
	set InLib.&InData(obs=0);
	keep 
		%do i=1 %to &ntrait;
			%do j=1 %to &&nm&i;
				%sysfunc(varname(&&_dsix_&i,&j))
			%end;
		%end;
		&MarkerVars
	;
run;

%let _dsix_ = %sysfunc(open(mva));
%let nm = %sysfunc(attrn(&_dsix_,NVARS));
%put total number of markers in the score files= &nm;
*/
/*data OutLib.c_mva; set mva; run;*/

* print out scoring function to log;
%let FlagDupTrait= 0;
%let DupTraitList= ;
%do i=1 %to &ntrait;
	%let nt&i= 0;
%end;

%do i=1 %to &ntrait;
	%if ^%scan(&itree, &i) %then %do;
		%put;
		%put &&trait&i = &&t&i.intercept;
		%do j=1 %to &&nm&i;
			%put + %sysfunc(varlabel(&&_dsix_&i,&j)) * %sysfunc(varname(&&_dsix_&i,&j));
		%end;
	%end;

	%do k=1 %to &ntrait;
		%if &&trait&k = &&trait&i and &&nt&k < 2 %then %do;
			%let nt&i= %sysevalf(&&nt&i + 1);
		%end;
	%end;
	
	%if &&nt&i > 1 %then %do;
		%let FlagDupTrait= %sysevalf(&FlagDupTrait + 1);
		%let DupTraitList= &DupTraitList &&trait&i;
	%end;
%end;

%if &FlagDupTrait %then %do;
	%if &FlagDupTrait < 2 %then %do;
		%put ERROR: There is a duplicated trait name in the Scoring Code Files. Each scoring code file must have a unique trait name that does not appear in any other scoring code file. Please check all scoring code files. The duplicate trait is: &DupTraitList..;
	%end;
	%else %do;
		%let DupTraitList= %sysfunc(tranwrd(&DupTraitList, %str( ), %str(, )));
		%put ERROR: There are duplicated trait names in the Scoring Code Files. Each scoring code file must have a unique trait name that does not appear in any other scoring code file. Please check all scoring code files. The duplicated traits are: &DupTraitList..;
	%end;
	%let exiterror= 1;
	%goto exit;
%end;

* save markers in the score files in a data set*;
data MkInScore;
	length _MarkerInScore_ $50;
	%do i=1 %to &ntrait;
		%do j=1 %to &&nm&i;
			_MarkerInScore_= "%sysfunc(varname(&&_dsix_&i,&j))";
			output;
		%end;
	%end;
run;
/*data OutLib.c_MkInScore0; set MkInScore; run;*/

%if %sysevalf(&ntrait > 1) %then %do;
	proc sort data=MkInScore out=MkInScore nodupkey;
		by _MarkerInScore_;
	run;
	/*data OutLib.c_MkInScore1; set MkInScore; run;*/
%end;

**checking if all markers in the score files are present in the input data set**;
proc sql UNDO_POLICY= NONE;
	create table MkInScFile as
	select * 
	from 
	MkInScore as a left join MkInFileT as b
	on upcase(a._MarkerInScore_)= upcase(b._MarkerInFile_)
	;
quit;
/*data OutLib.c_MkInScFile; set MkInScFile; run;*/

data miss_MkInFile;
	set MkInScFile end=_of_;
	if _n_ = 1 then _i_= 1;
	if missing(_MarkerInFile_) then do;
		call symputx("mismk"||trim(left(_i_)), _MarkerInScore_);
		_i_= _i_ + 1;
		output;
	end;
	retain _i_;
run;
/*data OutLib.c_miss_MkInFile; set miss_MkInFile; run;*/

%NObsVars(miss_MkInFile);
%if &nobs > 0 %then %do;
	%put mismk1= &mismk1;

	%let MissMkList= ;
	%do i=1 %to &nobs;
		%let MissMkList= &MissMkList &&mismk&i;
	%end;

	%put ERROR: Variables listed bellow were found in the Score Files but not found among marker variables in the Input Data Set. Only an intercept and marker variables are allowed in the score files.;
	%put Variable list: &MissMkList..;
	%let exiterror= 1;
	%goto exit;
%end;

%*goto exit;

**when annotation data set is avalaible, check its properties**;
%if %length(&AnnoInData) %then %do;

	%global VType;

	/*** 1. Deal with map data set - AnnoLib.&AnnoData ***/
	data nmapfile;
	 	set AnnoLib.&AnnoData;
		&InAnnoOrd= _n_;
		by &ChromID notsorted;
		if _n_= 1 then _count_= 0;
		if first.&ChromID and not missing(&ChromID) then do;
			_count_= _count_ + 1;
		end;
		else do;
			_count_= _count_;
		end;
		retain _count_;
		if not missing(&ChromID) then &ChromNum=_count_;
		else &ChromNum= .;
		drop _count_;
	run;
	%let MapFile = nmapfile;
	
	%let ListMapVars = &ChromNum &ChromLab &MarkerLab &MarkerPos &MarkerPos_BP;
	%if &ChromNum = &ChromLab %then %let ChromLab =;
	/* Test the type of chromNum etc. */
	%VarAttributes(_Ds=&MapFile,_Var=&ChromNum,_Len=,_Type=VType);
	%if &VType ^= N %then %do;
		%put ERROR: Variable (&ChromNum) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%if %length(&ChromLab)^=0 %then %do;
		%VarAttributes(_Ds=&MapFile,_Var=&ChromLab,_Len=,_Type=ChLab_Type);
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&MarkerLab,_Len=,_Type=VType);
	%if &VType ^= C %then %do;
		%put ERROR: Variable (&MarkerLab) is not a character variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos,_Len=,_Type=VType);
	%if &VType ^= N %then %do;
		%put ERROR: Variable (&MarkerPos) is not a numeric variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	%if %length(&MarkerPos_BP)^=0 %then %do;
		%VarAttributes(_Ds=&MapFile,_Var=&MarkerPos_BP,_Len=,_Type=MkBP_Type);
	%end;

	*checking if there are markers with missing chromosome ID*;	
	data miss_chr;
		set &MapFile;
		where missing(&ChromNum);
	run;

	%NObsVars(miss_chr);
	%if &nobs %then %do;
		%if &nobs = 1 %then %do;
			%put ERROR: There is &nobs marker with missing chromosome ID.;
		%end;
		%else %do;
			%put ERROR: There are &nobs markers with missing chromosome ID.;
		%end;
		%let exiterror= 1;
		%goto exit;
	%end;

	*checking if there are markers with missing position*;		
	data miss_pos;
		set &MapFile;
		where missing(&MarkerPos);
	run;

	%NObsVars(miss_pos);
	%if &nobs %then %do;
		%if &nobs = 1 %then %do;
			%put ERROR: There is &nobs marker with missing position.;
		%end;
		%else %do;
			%put ERROR: There are &nobs markers with missing position.;
		%end;
		%let exiterror= 1;
		%goto exit;
	%end; 
	
	* Check if levels of ChromID variable are grouped properly*;	
	proc sort data=&MapFile out=Tmp_Map;
		by &ChromID;
	run;
	
	%let FlagChromOutOfOrder= 0;
	data _null_;
		set Tmp_Map;
		by &ChromID;
		_lagord_= lag(&InAnnoOrd);
		if first.&ChromID then _diff_= 0;
		else _diff_= &InAnnoOrd - _lagord_;
		if _diff_ > 1 then do;
			_out_of_order_= 1;
			call symputx("FlagChromOutOfOrder", 1);
		end;
	run;
	%put FlagChromOutOfOrder= &FlagChromOutOfOrder;
	
	proc datasets nolist;
		delete Tmp_Map;
	quit;
	
	%if &FlagChromOutOfOrder %then %do;
		%put ERROR: &ChromID variable is not grouped properly in the annotation data set (map information). The levels of the &&ChromID variable must be grouped, however, they do not need to be sorted.;
		%let exiterror= 1;
		%goto exit;
	%end;
	
	* Check if MarkerPos is in ascending order *;	
	%let FlagMarkerOutOfOrder= 0;
	data _null_;
		set &MapFile;
		by &ChromID notsorted;
		_lagpos_= lag(&MarkerPos);
		if first.&ChromID then _diff_= 0;
		else _diff_= &MarkerPos - _lagpos_;
		if _diff_ < 0 then do;
			_out_of_order_= 1;
			call symputx("FlagMarkerOutOfOrder", 1);
		end;
	run;
	%put FlagMarkerOutOfOrder= &FlagMarkerOutOfOrder;
	
	%if &FlagMarkerOutOfOrder %then %do;
		%put ERROR: &MarkerPos variable is not sorted properly in the annotation data set (map information). &MarkerPos must be in ascending order within each level of the &ChromID variable.;
		%let exiterror= 1;
		%goto exit;
	%end;	
	
	* Check ChromNum, MarkerPos is in ascending order or not *;
	%let _tmp_ = 1;
	data _null_;
		set &MapFile;
		if (&ChromNum*1000000000+&MarkerPos/100)<(lag(&ChromNum)*1000000000+lag(&MarkerPos)/100) then do;
			call symput('_tmp_',"-1");
		end;
	run;
	%if &_tmp_ = -1 %then %do;
		%put ERROR: Either &ChromNum or &MarkerPos variables are not sorted properly in the annotation data set (map information). The &&ChromNum variable must be in ascending order, and &MarkerPos must be in ascending order within each level of the &ChromNum variable.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	*** 3. Combine map (MapFile) and genotype (InData) information *;
	data _null_;
		set &MapFile end=last;
		call symputx("annomrk"||strip(_n_),&MarkerLab);
		if last then call symputx("nanno",_n_);
	run;
	
	%let allmatch=1;
	%put nm_in= &nm_in;
	%put nanno= &nanno;
	%if &nanno < &nm_in %then %do;
		%let allmatch=0;
		%let allmatch_string= There are markers in the Input SAS Data Set that are not present in the Annotation SAS Data Set.;
		%put WARNING: &allmatch_string;
		%goto subset;
	%end;
		
	%do i=1 %to &nm_in; 
		%let varj= %sysfunc(varname(&_dsix_in,&i));
		
		%if %upcase(&&annomrk&i) ne %upcase(&varj) %then %do;
			%let allmatch=0;
			%let allmatch_string= For Variable &i, Marker Variable is &Varj and Marker Label in Annotation is &&annomrk&i..;
			%put WARNING: &allmatch_string;
			%goto subset;
		%end;
	%end; 	
	
	%subset:;
	%if &allmatch=0 %then %do;  
	
		%let MarkerVars= ;
		%do i=1 %to &nm_in; 
			%let MarkerVars= &MarkerVars %sysfunc(varname(&_dsix_in,&i));
		%end;	
	
		%put ERROR: The markers in the input data set do not match the annotation data set according to the variable selected as Marker Label Variable.;
		%put Please check that the Marker Label exactly matches the input data columns or run the Subset and Reorder Genetic Data AP to order these data sets correctly.;
		%put &allmatch_string;

		* create JSL file;
		%global SRButtonsScript;
		%PrepOutFile(SRButtonsScript,SRButtonsScriptName,launchAP_buttons_SR.jsl);
		filename jslbtns "&SRButtonsScript";

		options noquotelenmax;
		data _null_;
			file jslbtns lrecl=32767;
			put "//!";
			put "Names Default to Here(1);";
			put "wait(0.5);";
			put "if(host is(Windows),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
			put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
			put "	";
			put "srbuttonswindow=New Window(""Launch AP to Reorder Markers"",";
			put "		VListBox(";
			put "			TextBox(""Click on the button below to launch the Subset and Reorder Genetic Data process to reorder marker columns properly.""),";
			put "			TextBox(""Use the output data sets from that process as the new input and annotation data sets to .""),";

			put "			TextBox("" ""),";
			put "			ButtonBox(""Subset and Reorder Genetic Data"",";
			put "				if(host is(Windows),";
			put "					Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, windows)),";
			put "					Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/SubsetandReorderGeneticData.jsl"", absolute, POSIX) ));";
			put "	";

			/* Set input data set and Output Folder */
			put "				jg:setDefaultParmValues(2, ""SubsetandReorderGeneticData"");";				
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""InData""),""%sysfunc(pathname(InLib))\&InData..sas7bdat"");";
			put "				jg:SubsetandReorderGeneticData_updateDeps;";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""ListMarkerVars""),""&MarkerVars"");";	
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MarkerFmt""),""Numeric Genotypes"");";
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""MinPercent""),""0"");";		
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""OutPath""),""&ClientOutPath"");";					
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoInData""),""%sysfunc(pathname(AnnoLib))\&AnnoData..sas7bdat"");";					
			put "				jg:SubsetandReorderGeneticData_updateDeps;";				
			put "				jg:setCurrentParmValue(2, ""SubsetandReorderGeneticData"", jg:getParmNum(""SubsetandReorderGeneticData"",""AnnoMName""),""&MarkerLab"");";	
			put "			)";;

			put "		)";
			put ");";
		run;

		%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SRButtonsScriptName, , text/plain, yes);		
		%let exiterror=1;
		%goto exit;
	%end;
%end;

*Keep only markers present in the score files to speed up simulations*;	
%if %upcase(&SimMarkersInScore)= YES or (%length(&AnnoInData) and %length(&AnnoWhereStmt)) %then %do;
	data NonMarkersOnly;
		set InLib.&InData(drop = %do i=1 %to &nm_in; %sysfunc(varname(&_dsix_in,&i)) %end;);
	run;
%end;
%let _dsix_in = %sysfunc(close(&_dsix_in));

%let MkInFileT= MkInFileT; 
%let GenData= InLib.&InData;
%if %upcase(&SimMarkersInScore)= YES %then %do;
	proc sql UNDO_POLICY= NONE;
		create table SubMkInFileT as
		select * 
		from 
		MkInFileT as b, MkInScore as a
		where upcase(a._MarkerInScore_)= upcase(b._MarkerInFile_)
		;
	quit;
	%let MkInFileT= SubMkInFileT;
	/*data OutLib.c_SubMkInFileT; set SubMkInFileT; run;*/

	proc transpose data=SubMkInFileT out=SubMkInFile(obs=0 drop= _name_ _label_);
		var _MarkerInFile_;
		id _MarkerInFile_;
	run;
	/*data OutLib.c_SubMkInFile; set SubMkInFile; run;*/

	%let _dsix_insub = %sysfunc(open(SubMkInFile));
	%let nm_insub = %sysfunc(attrn(&_dsix_insub,NVARS));
	%put total number of markers in the subset input file= &nm_insub;	

	data SubMarkersOnly;
		set InLib.&InData;
		keep 
			%do i=1 %to &nm_insub; 
				%sysfunc(varname(&_dsix_insub,&i)) 
			%end;
		;
	run;		

	/*
	%if (%length(&AnnoInData) and %length(&AnnoWhereStmt)) %then %do; 
		proc datasets nolist library=work;
			change SubMarkersOnly= SubInData;
		run;
		quit;	
	%end;
	%else %do;
	*/
		data SubIndata;
			merge NonMarkersOnly SubMarkersOnly;
		run;	
	/*%end;*/	
	%let GenData= SubInData;
	/*data OutLib.c_SubInData0; set SubInData; run;*/

	%let _dsix_insub = %sysfunc(close(&_dsix_insub));
%end;

%let MarkerPos_Orig= &MarkerPos;

%if %length(&AnnoInData) %then %do;
	**Removing markers by the AnnoWhereStmt**;
	%if %length(&AnnoWhereStmt) and %upcase(&SimMarkersInScore) ^= YES %then %do;
		data &MapFile; 
			set &MapFile;
			where &AnnoWhereStmt;
		run;
		
		/*data OutLib.c_MapFile; set &MapFile; run;*/
		
		%NObsVars(&MapFile);
		%if &nobs = 0 %then %do;
			%put ERROR: The statement entered in the Filter to Include Markers option from the Annotation tab removed all markers from the analysis. Please, review the statement.;
			%let exiterror= 1;
			%goto exit;
		%end;
		
		**checking if all markers in the score files are present in the anno data set after filtering**;
		proc sql UNDO_POLICY= NONE;
			create table MkInAnnFile as
			select * 
			from 
			MkInScore as a left join &MapFile(rename=(&MarkerLab= _MarkerAnnInFile_)) as b
			on upcase(a._MarkerInScore_)= upcase(b._MarkerAnnInFile_)
			;
		quit;	
		
		data miss_MkAnnInFile;
			set MkInAnnFile end=_of_;
			if _n_ = 1 then _i_= 1;
			if missing(_MarkerAnnInFile_) then do;
				call symputx("misannmk"||trim(left(_i_)), _MarkerInScore_);
				_i_= _i_ + 1;
				output;
			end;
			retain _i_;
		run;
		/*data OutLib.c_miss_MkAnnInFile; set miss_MkAnnInFile; run;*/

		%NObsVars(miss_MkAnnInFile);
		%if &nobs > 0 %then %do;
			%put mismk1= &mismk1;

			%let MissAnnMkList= ;
			%do i=1 %to &nobs;
				%let MissAnnMkList= &MissAnnMkList &&misannmk&i;
			%end;
	
			%put ERROR: Markers listed bellow were found in the Score Files but not found in the Annotation Data Set after filtering markers.;
			%put Marker list: &MissAnnMkList..;
			%let exiterror= 1;
			%goto exit;
		%end;		
		
		proc transpose data=&MapFile out=SubMapFileT(drop = _name_);
			var &MarkerLab;
			id &MarkerLab;
		run;	
		
		%let _dsix_annoin = %sysfunc(open(SubMapFileT));
		%let nm_annoin = %sysfunc(attrn(&_dsix_annoin,NVARS));
				
		data 
			SubMarkersOnly(keep = %do i=1 %to &nm_annoin; %sysfunc(varname(&_dsix_annoin,&i)) %end;)
			/*NonMarkersOnly(drop = %do i=1 %to &nm_in; %sysfunc(varname(&_dsix_in,&i)) %end;)*/
			;
			*set InLib.&InData;
			set &GenData;
		run;
		
		data SubInData;
			 merge NonMarkersOnly SubMarkersOnly;
		run;
		%let GenData= SubInData;

		/*data OutLib.c_SubInData1; set SubInData; run;*/
		
		proc transpose data=SubMarkersOnly(obs=0) out=SubMkInFileT(rename=(_name_=_MarkerInFile_));
			var _all_;
		run;	
		%let MkInFileT= SubMkInFileT;
		
		%let _dsix_annoin = %sysfunc(close(&_dsix_annoin));
	%end;
	/*data OutLib.c_mapfile0; set &MapFile; run;*/
	
	**transforming SF2 recombination into RILself recombination**;
	%let RIL_type= Self;
	%*let RIL_type= Sib;
	%if %upcase(&SimRilFast)= YES %then %do;
		data &MapFile;
			set &MapFile;
			by &ChromNum &MarkerPos;
			_cmlag_f2_= lag(&MarkerPos);
			if first.&ChromNum then do;
				_cmstep_f2_= .;
				_rfstep_f2_= .;
				_rfstep_ri_= .;
				_cmstep_ri_= .;
				_cumdist_ri_= &MarkerPos; 
			end;
			else do;
				**get SF2-based adjacent distance from the SF2-based cumulative distance**;
				_cmstep_f2_= &MarkerPos - _cmlag_f2_;
				
				**transform SF2-based distance into recombination for an SF2-based population**;
				%if %upcase(&MapFunc)= HALDANE %then %do;
					_rfstep_f2_= 0.5*(1.0-exp(-2.0*_cmstep_f2_/100));
				%end;
				%else %if %upcase(&MapFunc)= KOSAMBI %then %do;
					_rfstep_f2_= (1-exp(-4*_cmstep_f2_/100))/(2*(1+exp(-4*_cmstep_f2_/100)));
				%end;			
				
				**transform SF2-based recombination into RIL-based recombination**;
				%if %upcase(&RIL_type)= SELF %then %do;
					_rfstep_ri_= 2*_rfstep_f2_ /(1 + 2*_rfstep_f2_);
				%end;
				%else %if %upcase(&RIL_type)= SELF %then %do;
					_rfstep_ri_= 4*_rfstep_f2_ /(1 + 6*_rfstep_f2_);
				%end;

				**transform RIL-based recombination into RIL-based adjacent distance**;
				%if %upcase(&MapFunc)= HALDANE %then %do;
					_cmstep_ri_= 100*(-(1/2)*log(1-2*_rfstep_ri_));
				%end;
				%else %if %upcase(&MapFunc)= KOSAMBI %then %do;
					_cmstep_ri_= 100*((1/4)*log((1+2*_rfstep_ri_)/(1-2*_rfstep_ri_)));
				%end;
				
				**get RIL-based cumulative distance from the RIL-based adjacent distance**;
				_cumdist_ri_= _cumdist_ri_ + _cmstep_ri_;
			end;
			retain _cumdist_ri_;
		run;			
		
		%let MarkerPos= _cumdist_ri_;
	%end;
	/*data OutLib.c_mapfile1; set &MapFile; run;*/
	
	**estimating lower and upper bound of each chromosome, as well as their respective lengths*;	
	data &MapFile;
		set &MapFile;
		by &ChromNum &MarkerPos;
		if first.&ChromNum then do;
			ch_lb= &MarkerPos;
			_order_= 1;
		end;
		else do;
			_order_= _order_ + 1;
		end;
		if last.&ChromNum then do;
			ch_ub= &MarkerPos;
		end;
		ch_len= ch_ub - ch_lb;
		retain ch_lb _order_;
	run;
	/*data OutLib.c_mapfile3; set &MapFile; run;*/

	proc sort data=&MapFile;
		by &ChromNum descending _order_ &MarkerPos &MarkerLab ;
	run;
	/*data OutLib.c_mapfile4; set &MapFile; run;*/

	data &MapFile;
		set &MapFile;
		by &ChromNum descending &MarkerPos;
		if first.&ChromNum then do;
			chr_lb = ch_lb;
			chr_ub = ch_ub;
			chr_len= ch_len;
		end;
		retain chr_lb chr_ub chr_len;
	run;	
	/*data OutLib.c_mapfile5; set &MapFile; run;*/

	proc sort data=&MapFile;
		by &ChromNum _order_ &MarkerPos &MarkerLab ;
	run;
	/*data OutLib.c_mapfile6; set &MapFile; run;*/
	
	proc sql UNDO_POLICY=NONE;
		create table submap as 
		select *
		from &MkInFileT as a left join &MapFile as b
		on a._MarkerInFile_= b.&MarkerLab
		order by &ChromNum, &MarkerPos, &MarkerLab
		;
	quit;	
	/*data OutLib.c_submap; set submap; run;*/
	
	data _null_;
		set submap end= _eof_;
		by &ChromNum &MarkerPos;
		if _n_ = 1 then nchr= 0;
		if first.&ChromNum then nchr= nchr + 1;
		if _eof_ then do;
			call symputx("nm_sub", trim(left(_n_)));
			call symputx("nc", trim(left(nchr)));
		end;
		retain nchr;
		*call symputx("mk"||trim(left(_n_)), &MarkerLab);
	run;
	%put nc= &nc;
	%put total number of markers in the submap= &nm_sub;

	proc means data=submap max noprint;
		var chr_len;
		output out=maxchrlen max=_maxchrlen_;
	run;

	data _null_;
		set maxchrlen;
		call symputx('maxchrlen', trim(left(_maxchrlen_)));
	run;
	%put maxchrlen= &maxchrlen;

	proc means data=submap n noprint;
		var &MarkerPos;
		by &ChromNum;
		output out=nMrkByChr n=_nmrk_;
	run;

	proc transpose data=nMrkByChr out=subnmrk(drop=_name_) prefix= _nmk_;
		var _nmrk_;
		id &ChromNum;
	run;
	/*data OutLib.c_submap; set submap; run;*/

	proc transpose data=submap out=subpos(drop=_name_) prefix= _pos_;
		var &MarkerPos;
		id &MarkerLab;
	run;

	proc sort data=submap out=submap_nodup nodupkey;
		by &ChromNum;
	run;
	/*data OutLib.c_submap_nodup; set submap_nodup; run;*/

	proc transpose data=submap_nodup out=subchr(drop=_name_) prefix= _chr_;
		var &ChromNum;
		id &ChromNum;
	run;

	proc transpose data=submap_nodup out=sublen(drop=_name_) prefix= _len_;
		var chr_len;
		id &ChromNum;
	run;
	/*data OutLib.c_sublen; set sublen; run;*/

	proc transpose data=submap_nodup out=sublb(drop=_name_) prefix= _lbo_;
		var chr_lb;
		id &ChromNum;
	run;

	proc transpose data=submap_nodup out=subub(drop=_name_) prefix= _ubo_;
		var chr_ub;
		id &ChromNum;
	run;

	/**
	maxr is the upper bound on number of breaks for any given chromosome.
	Since the mean and the variance of a Poisson process are equal, I'm using
	an ad-hoc upper bound of lambda + 10*sqrt(lambda).
	**/
	%let maxr= %sysfunc(ceil(%sysevalf(&maxchrlen/100 + 10*%sysfunc(sqrt(&maxchrlen/100)))));
	%put maxr= &maxr; 
%end;
%*let _dsix_in = %sysfunc(close(&_dsix_in));

*S1240091: preventing problem if code ends without closing files*;
%if %symexist(_dsix_) %then %do;
	%let _dsix_ = %sysfunc(close(&_dsix_)); 
%end;

* save all referenced marker names in the input data set;
data mva;
	/*
	%if %length(&AnnoInData) and %length(&AnnoWhereStmt) %then %do;
		set SubInData(obs=0);
	%end;
	%else %do;
		set InLib.&InData(obs=0);
	%end;
	*/
	set &GenData;
	keep 
		%do i=1 %to &ntrait;
			%do j=1 %to &&nm&i;
				%sysfunc(varname(&&_dsix_&i,&j))
			%end;
		%end;
		&MarkerVars
	;
run;
/*data OutLib.c_mva; set mva; run;*/

%let _dsix_ = %sysfunc(open(mva));
%let nm = %sysfunc(attrn(&_dsix_,NVARS));
%put total number of markers in the score files= &nm;

%let OrigNBack= &NBack;
%if &NBack > 0 %then %do;
	%if %length(&ParentalVar) = 0 /*or %upcase(&Simulate) ^= YES or %upcase(SimMultiGen) ^= YES*/ %then %do;
		%put ERROR: Number of Backcross Generations cannot be specified unless a variable is specified in the Backcross Parental Lines Variable.;
		%let exiterror=1;
		%goto exit;
	%end;
%end;
%else %do;
	%let NBack= 1;
%end;
%put NBack= &NBack;

%let OrigNSelf= &NSelf;
%let NSimRIL= 0;
%if %upcase(&SimRilSSD)= YES %then %do;
	*producing RILs via the single seed descent method*;
	*need to set the number of selfing generations (NSelf) and the number of progenies simulated on each generation (NSimRIL)*;
	%let NSelf= 8;
	%let NSimRIL= 1; 	
%end;

%if &NSelf > 0 %then %do;
	%if %length(&SexVar) or 
		%length(&TesterVar) or 
		%length(&ParentalVar) /*or
		%nrbquote(%upcase(&MatingType)) ^= %nrbquote(%upcase(Include self only))*/ %then %do;
			%put ERROR: Number of Selfing Generations cannot be specified when a variable is specified either in the Sex, Top Cross Tester Lines Indicator, or Backcross Parental Lines Variables. /*Also, Number of Selfing Generations cannot be specified when mating type is not Include Self Only.*/;
			%let exiterror=1;
			%goto exit;	
	%end;
%end;
%else %do;
	%let NSelf= 1;
%end;
%put NSelf= &NSelf;

%let OrigNnonSelf= &NnonSelf;
%if &NnonSelf > 0 %then %do;
	%if %length(&SexVar) or 
		%length(&TesterVar) or 
		%length(&ParentalVar) /*or
		%nrbquote(%upcase(&MatingType)) ^= %nrbquote(%upcase(Include self only))*/ %then %do;
			%put ERROR: Number of Generations cannot be specified when a variable is specified either in the Sex, Top Cross Tester Lines Indicator, or Backcross Parental Lines Variables. /*Also, Number of Selfing Generations cannot be specified when mating type is not Include Self Only.*/;
			%let exiterror=1;
			%goto exit;	
	%end;
%end;
%else %do;
	%let NnonSelf= 1;
%end;
%put NnonSelf= &NnonSelf;

%put OptimizeCrosses= &OptimizeCrosses;
%put NBack= &NBack;
%put NSelf= &NSelf;
%put NnonSelf= &NnonSelf;
%if %upcase(&OptimizeCrosses)= YES and &NBack < 2 and &NSelf < 2 and &NnonSelf < 2 %then %do;
	%put ERROR: The option to select best simulated progenies to cross is not available when number of generations is less than 2.;
	%let exiterror= 1;
	%goto exit;
%end;
%else %if %upcase(&OptimizeCrosses)= YES and (&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) %then %do;
	
	%if %upcase(&OptimizeSelection) = YES %then %do;
	
		%let nsense= 0;
		%let nMaxOrMim= 0;
		%let k=1;
		%let Sense= %scan(&OptSense, &k, ' ');
		%do %while(%length(&Sense));
			%put Sense= &Sense;

			%if %nrbquote(&Sense) = -1 or %nrbquote(&Sense) = 0 or %nrbquote(&Sense) = 1 %then %do;	

				%if %nrbquote(&Sense) = -1 or %nrbquote(&Sense) = 1 %then %do;
					%put k= &k;
					%let nMaxOrMim= %eval(&nMaxOrMim + 1);
				%end;

				%let k= %eval(&k + 1);
				%let Sense= %scan(&OptSense, &k, ' ');
			%end;
			%else %do;
				%put ERROR: The value &Sense appears in the list of optimization direction for progeny selection. However, all values in the list must be either numeral -1, 0, or 1, indicating whether to minimize, maximize, or ignore the trait scoring, respectively.;
				%let Sense= ;
				%let exiterror= 1;
				%goto exit;
			%end;
		%end;
		%let nsense= %eval(&k - 1);

		%if &ntrait ^= &nsense %then %do;
			%if &nsense = 1 %then %let string0= item;
			%else %let string0= items;
			%put ERROR: The list of optimization direction for progeny selection has &nsense &string0 indicating whether to minimize, maximize, or ignore the trait scorings. However, there are &ntrait traits to be optimizized. There must be a numeral for each trait scoring. Please, review the list of optimization direction for progeny selection.;
			%let exiterror = 1;
			%goto exit;
		%end;

		%put nMaxOrMim= &nMaxOrMim;
		%if &nMaxOrMim = 0 %then %do;
			%put ERROR: The list of optimization direction for progeny selection has all values equal to the numeral 0. However, at least one value must be either numeral -1 or 1, indicating whether to minimize or maximize the trait scoring, respectively.;
			%let exiterror= 1;
			%goto exit;
		%end;

		options mstored sasmstore=orlib;
		%if %length(%sysfunc(pathname(orlib))) = 0 %then %do;
			libname orlib "&MacroPath.";
		%end;

		%let foundcatmacro=%sysfunc(fileexist(&MacroPath.\sasmacr.sas7bcat));
		%put foundcatmacro= &foundcatmacro;

		%if &foundcatmacro = 0 %then %do;
			%put ERROR: The stored macro catalog "sasmacr.sas7bcat" was not found in "&MacroPath.".;
			%put This file should be copied from;
			%put "&MacroPath.\Stored32" or;
			%put "&MacroPath.\Stored64" depending on your OS.;
			%let exiterror=1;
			%goto exit;
		%end;

		/*
		%macro FindBestProgenies(indat_=, optvars_=, optsense_=);

			%put optvars_= &optvars_;
			%put optsense_= &optsense_;

			data indata_tmp;
				set &indat_(keep= &OptVars_ _orig_order_);
				%let k=1;
				%let _ovar_= %scan(&OptVars_, &k, ' ');
				%do %while(%length(&_ovar_));
					%if %index(%scan(&OptSense_, &k, ' '), max) %then %do;
						x&k= -1*&_ovar_;
					%end;
					%else %do;
						x&k= &_ovar_;
					%end;
					%let k= %eval(&k + 1);
					%let _ovar_= %scan(&OptVars_, &k, ' ');
				%end;
			run;

			data OutLib.c_indata_tmp; set indata_tmp; run;

			%*goto exitbest;

			%FindParetoPoints(indata=indata_tmp, outdata=outdata_tmp, outorder= outorder_tmp);

			proc sql UNDO_POLICY= NONE;
				create table outdata_all as
				select *
				from outorder_tmp as a left join outdata_tmp(drop= x:) as b
				on a._sort_order_ = b.pareto_points
				;

				create table outdata_all as
				select *
				from &indat_ as a left join outdata_all as b
				on a._orig_order_ = b._orig_order_
				order by _orig_order_;
				;
			quit;

			data Outdata_all;
				set outdata_all;
				if not missing(pareto_points) then do;
					%let k=1;
					%let _ovar_= %scan(&OptVars_, &k, ' ');
					%do %while(%length(&_ovar_));
						x&k= &_ovar_;
						%let k= %eval(&k + 1);
						%let _ovar_= %scan(&OptVars_, &k, ' ');
					%end;
				end;
			run;

			*proc gplot data=outdata_all;
			*	symbol1 value=dot pointlabel=none;
			*	plot %scan(&OptVars_, 1, ' ')*%scan(&OptVars_, 2, ' ') x1*x2 / overlay ;
			*run;
			*quit; 

			%exitbest:;

		%mend FindBestProgenies;
		*/
	%end;
	%else %if %upcase(&ThresholdSelection) = YES %then %do;
		
		*Checking threshold direction list*;
		%put ThresholdSense= &ThresholdSense; 
		%let nsense= 0;
		%let nMaxOrMim= 0;
		%let k=1;
		%let Sense= %scan(&ThresholdSense, &k, ' ');
		%do %while(%length(&Sense));
			%put Sense= &Sense;

			%if %nrbquote(&Sense) = -1 or %nrbquote(&Sense) = 0 or %nrbquote(&Sense) = 1 %then %do;	

				%if %nrbquote(&Sense) = -1 or %nrbquote(&Sense) = 1 %then %do;
					%put k= &k;
					%let nMaxOrMim= %eval(&nMaxOrMim + 1);
				%end;

				%let k= %eval(&k + 1);
				%let Sense= %scan(&ThresholdSense, &k, ' ');
			%end;
			%else %do;
				%put ERROR: The value &Sense appears in the list of threshold direction for progeny selection. However, all values in the list must be either numeral -1, 0, or 1, indicating whether to minimize, maximize, or ignore the trait scoring, respectively.;
				%let Sense= ;
				%let exiterror= 1;
				%goto exit;
			%end;
		%end;
		%let nsense= %eval(&k - 1);

		%if &ntrait ^= &nsense %then %do;
			%if &nsense = 1 %then %let string0= item;
			%else %let string0= items;
			%put ERROR: The list of threshold direction for progeny selection has &nsense &string0 indicating whether to minimize, maximize, or ignore the trait scorings. However, there are &ntrait traits to be optimizized. There must be a numeral for each trait scoring. Please, review the list of threshold direction for progeny selection.;
			%let exiterror = 1;
			%goto exit;
		%end;

		%put nMaxOrMim= &nMaxOrMim;
		%if &nMaxOrMim = 0 %then %do;
			%put ERROR: The list of threshold direction for progeny selection has all values equal to the numeral 0. However, at least one value must be either numeral -1 or 1, indicating whether to minimize or maximize the trait scoring, respectively.;
			%let exiterror= 1;
			%goto exit;
		%end;		
		
		*Checking threshold list*
		%let nvalues= 0;
		%let numvals= 0;
		%let k=1;
		%let Threshold= %scan(&ThresholdValues, &k, ' ');
		%do %while(%length(&Threshold));
			%put Threshold= &Threshold;

			%if %DataTyp(&Threshold) ^= CHAR or (%DataTyp(&Threshold) = CHAR and %nrbquote(&Threshold) = .) %then %do;	

				%if %nrbquote(&Threshold) ^= . %then %do;
					%put k= &k;
					%let numvals= %eval(&numvals + 1);
				%end;

				%let k= %eval(&k + 1);
				%let Threshold= %scan(&ThresholdValues, &k, ' ');
			%end;
			%else %do;
				%put ERROR: The value &Threshold appears in the list of threshold values for progeny selection. However, all values in the list must be either numeral or dot ("."), indicating the cut-off value for the trait, or indicating that the trait is not be used for selection respectively.;
				%let Threshold= ;
				%let exiterror= 1;
				%goto exit;
			%end;
		%end;
		%let nvalues= %eval(&k - 1);

		%if &ntrait ^= &nvalues %then %do;
			%if &nvalues = 1 %then %let string0= item;
			%else %let string0= items;
			%put ERROR: The list of threshold values for progeny selection has &nvalues &string0 indicating cut-off values for the traits. However, there are &ntrait traits. There must be a numeral or a dot (".") for each trait. Please, review the list of threshold direction for progeny selection.;
			%let exiterror = 1;
			%goto exit;
		%end;

		%put numvals= &numvals;
		%if &numvals = 0 %then %do;
			%put ERROR: The list of threshold values for progeny selection has all values equal to dot ("."). However, at least one value must be a non-missing numeric value, indicating the cutt-of value for the trait.;
			%let exiterror= 1;
			%goto exit;
		%end;
	%end;
%end;

%let ntraitix= &ntrait;
%if %upcase(&ComputeSelIndex) = YES %then %do;
	*Index selection*;
	%put IndexWeights= &IndexWeights;
	%let SelIndexFormula= ;
	%let _Plus_ = ;
	%let nvalues= 0;
	%let numvals= 0;
	%let k=1;
	%let IndexW= %scan(&IndexWeights, &k, ' ');
	%do %while(%length(&IndexW));
		%put IndexW= &IndexW;

		%if %DataTyp(&IndexW) ^= CHAR %then %do;	

			%put k= &k;
			
			%let SelIndexFormula= &SelIndexFormula &_Plus_ &IndexW %str(*) &&trait2&k;
			%let _plus_ = %str(+);
			%let numvals= %eval(&numvals + 1);
			%let k= %eval(&k + 1);
			%let IndexW= %scan(&IndexWeights, &k, ' ');
		%end;
		%else %do;
			%put ERROR: The value &IndexW appears in the list of weight values for the selection index. However, all values in the list must be either numeral or zero, indicating the cut-off value for the trait, or indicating that the trait is not be used for selection respectively.;
			%let Threshold= ;
			%let exiterror= 1;
			%goto exit;
		%end;
	%end;
	%let nvalues= %eval(&k - 1);
	%put SelIndexFormula= &SelIndexFormula;

	%if &ntrait ^= &nvalues %then %do;
		%if &nvalues = 1 %then %let string0= item;
		%else %let string0= items;
		%put ERROR: The list of weight values for the selection index has &nvalues &string0 indicating weight values for the traits. However, there are &ntrait traits. There must be a numeral for each trait. Please, review the list of weight values for the selection index.;
		%let exiterror = 1;
		%goto exit;
	%end;

	%put numvals= &numvals;
	%if &numvals = 0 %then %do;
		%put ERROR: The list of weight values for the selection index has all values equal to zero. However, at least one value must be a non-zero numeric value, indicating the weight for a trait.;
		%let exiterror= 1;
		%goto exit;
	%end;	
	
	%let IndexDirection = %str(>);
	%if %index(&IndexSense, Low) %then %let IndexDirection = %str(<);
	
	%let SelIndex= SelectionIndex;	
	%let SelIndex2= &SelIndex._Score;
	
	%let ntraitix= %eval(&ntrait + 1);
	%let trait&ntraitix= &SelIndex;	
	%let trait2&ntraitix= &SelIndex2;
%end;

%if %upcase(&PercentSelection) = YES and %upcase(&ComputeSelIndex) = YES %then %do;
	%let PercentTrait = &SelIndex2; 
%end;
%else %do;
	%let j= 1;
	%let PercentTrait = &&trait2&j; 
%end;	

%*goto exit;

%let NGenerations= 1;
%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
	/*
	data SimStByLines_All;
		length 
			%if &NBack > 1 %then %do;
				_backgeneration_ _orig_m_lines_ _orig_p_lines_
			%end;
			%else %if &NSelf > 1 %then _selfgeneration_; 
			_orig_m_ID_ _orig_p_ID_ $ &line_length;
		delete;
	run;
	
	data SimStByCross_All;
		length 
			%if &NBack > 1 %then %do;
				_backgeneration_ _orig_m_lines_ _orig_p_lines_
			%end;
			%else %if &NSelf > 1 %then _selfgeneration_; 
			_orig_m_ID_ _orig_p_ID_ $ &line_length;
		delete;
	run;
	*/
	
	%if &NBack > 1 %then %do;
		%let NGenerations= &NBack;
	%end;
	%else %if &NSelf > 1 %then %do;
		%let NGenerations= &NSelf;
		%let OrigMatingType= &MatingType;
	%end;
	%else %if &NnonSelf > 1 %then %do;
		%let NGenerations= &NnonSelf;
		%let OrigMatingType= &MatingType;
	%end;
%end;
%put NGenerations= &NGenerations;

%if (&Compress = Yes) %then %let dsoptions = compress=yes /*(compress=yes)*/;
%else %let dsoptions = ;

%if %upcase(&EvalLines) ^= YES %then %do;
**begin generations loop**;
%let StopMultGen= 0;
%let gen= 1;
%do %while(&gen <= &NGenerations and &StopMultGen = 0);
	%put Processing generation &gen.;

	%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
		%if &gen = 1 %then %do;
			%let OrigIDVars= &IDVars;
			%let OrigParentalVar= &ParentalVar;
		%end;
		%else %if &gen = 2 %then %do;
			%let IDVars= m_id p_id sim;
			%if &NBack > 1 %then %do;
				%let ParentalVar= _orig_m_lines_;
			%end;
			%else %if &NSelf > 1 %then %do;
				**after the initial crosses, allow only selfing**;
				%let MatingType= Include self only;
				%if %length(&SexVar &TesterVar &ParentalVar) = 0 %then
					%let Selfs= Yes;
			%end;
			%else %if &NnonSelf > 1 %then %do;
				**after the initial crosses, allow only outcross**;
				%let MatingType= Include outcross only;
				%let Selfs= No;
			%end;			
		%end;
	%end;
	%put MatingType= &MatingType;

	data g;
		%if &gen= 1 %then %do;
			/*
			%if %length(&AnnoInData) and %length(&AnnoWhereStmt) %then %do;
				set SubInData;
			%end;
			%else %do;
				set InLib.&InData;
			%end;
			*/
			set &GenData;
		%end;
		%else %do;
			set simdat;
		%end;
		%if &gen= 1 %then %do;
			%if %length(&WhereStmt) %then %do;
				where &WhereStmt;
			%end;
		%end;
		%if ^%length(&IDVars) %then %do;
			length Line_ID $ &line_length;
			Line_ID = trim(left(_n_));
			keep Line_ID;
		%end;
		%if %length(&SexVar) %then %do;
			if not missing(&SexVar);
		%end;
		%if %length(&TesterVar) %then %do;
			if not missing(&TesterVar);
		%end;
		%if %length(&ParentalVar) %then %do;
			if not missing(&ParentalVar);
		%end;
		keep &IDVars &SexVar &TesterVar &ParentalVar 
			%do i=1 %to &nm; 
				%sysfunc(varname(&_dsix_,&i)) 
			%end; 
			%if &gen > 1 %then %do;
				%if &NBack > 1 %then %do;
					_orig_p_lines_
				%end;
				_orig_m_ID_
				_orig_p_ID_
			%end;
		;
	run;
	/*data OutLib.c_g0_&gen; set g; run;*/

	%NObsVars(g);
	%if &nobs = 0 %then %do;
		%if %length(&WhereStmt) and %length(&SexVar) %then %do; 
			%put ERROR: Either the where statement specified in the Filter to Include Observations produced a data set without observations or the specified Sex Variable has all missing values.;
		%end;
		%else %if %length(&WhereStmt) and %length(&SexVar) = 0 %then %do; 
			%put ERROR: The where statment specified in the Filter to Include Observations from the General tab produced a data set without observations.;
		%end;
		%else %if %length(&WhereStmt)= 0 and %length(&SexVar) %then %do; 
			%put ERROR: The specified Sex Variable has all missing values.;
		%end;
		%let exiterror = 1;
		%goto exit;
	%end;
	%else %if &nobs = 1 and &gen= 1 %then %do;
		%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include outcross only)) %then %do;
			%put ERROR: Include self only option for the first-generation mating type requires 2 or more observations in the input data set.;
			%let exiterror = 1;
			%goto exit;
		%end;
	%end;

	%if %length(&IDVars) %then %do;

		%put IDVars= &IDVars;
		/*data OutLib.c_g1_&gen; set g; run;*/

		%NObsVars(g);
		%let nobs0 = &nobs;
		proc sort data=g nodupkey;
			by &IDVars;
		run;
		%NObsVars(g);
		%if &nobs ne &nobs0 %then %do;
			%put ERROR: ID Variables must uniquely define each row in the input data set.  Use different variables or remove them altogether.;
			%let exiterror = 1;
			%goto exit;
		%end;
		data g;
			length Line_ID $ &line_length;
			set g;
			Line_ID = trim(left(%scan(&IDVars,1,' '))) 
				%let i=2;
				/*
				%do %while(%length(%scan(&IDVars,&i,' ')));
					|| " " || trim(left(%scan(&IDVars,&i,' ')))
					%let i=%eval(&i+1);
				%end;
				*/
				%do %while(%length(%scan(&IDVars,&i,' ')));
					|| "-" || trim(left(%scan(&IDVars,&i,' ')))
					%let i=%eval(&i+1);
				%end;
			;
		run;
	%end;
	/*data OutLib.c_g1; set g; run;*/

	%if %length(&SexVar) %then %do;
		proc sort data=g out=gs nodupkey;
			by &SexVar;
		run;
		%NObsVars(gs);
		%if &nobs ne 2 %then %do;
			%put ERROR: Sex variable must have 2 nonmissing levels. The current variable &SexVar has &nobs levels.;
			%let exiterror = 1;
			%goto exit;
		%end;
		data _null_;
			set gs;
			call symputx("sex"||compress(_n_),compress(&SexVar));
		run;
		%let gid = %sysfunc(open(g));
		%let svn = %sysfunc(varnum(&gid,&SexVar));
		%let svt = %sysfunc(vartype(&gid,&svn));
		%let svl = %sysfunc(varlen(&gid,&svn));
		%let gid = %sysfunc(close(&gid));
		%if %length(&Maternal) and &sex1 ne &Maternal %then %do;
			%if &sex2 ne &Maternal %then %do;
				%put ERROR: The specified maternal value &Maternal does not match either value of &SexVar, &sex1 and &sex2.. Please specify one of these two values.;
				%let exiterror = 1;
				%goto exit;
			%end;
			%let temp = &sex1;
			%let sex1 = &sex2;
			%let sex2 = &temp;
		%end;
		%put sex1 = &sex1;
		%put sex2 = &sex2;
		%put svt = &svt;
		%let Selfs = No;
	%end;

	%if %length(&TesterVar) %then %do;
		proc sort data=g out=gs nodupkey;
			by &TesterVar;
		run;
		%NObsVars(gs);
		%if &nobs ne 2 %then %do;
			%put ERROR: Tester Lines variable must have 2 nonmissing levels. The current variable &TesterVar has &nobs levels.;
			%let exiterror = 1;
			%goto exit;
		%end;
		/*
		data _null_;
			set gs;
			call symputx("top"||compress(_n_),compress(&TesterVar));
		run;
		%let gid = %sysfunc(open(g));
		%let tvn = %sysfunc(varnum(&gid,&TesterVar));
		%let tvt = %sysfunc(vartype(&gid,&tvn));
		%let gid = %sysfunc(close(&gid));
		%if %length(&TesterValue) and &top1 ne &TesterValue %then %do;
			%if &top2 ne &TesterValue %then %do;
				%put ERROR: The specified tester value &TesterValue does not match either value of &TesterVar, &top1 and &top2.. Please specify one of these two values.;
				%let exiterror = 1;
				%goto exit;
			%end;
			%let temp = &top1;
			%let top1 = &top2;
			%let top2 = &temp;
		%end;
		%put top1 = &top1;
		%put top2 = &top2;
		%put tvt = &tvt;
		*/
	%end;

	%put ParentalVar= &ParentalVar;
	%if %length(&ParentalVar) %then %do;

		%put P1Value= &P1Value;
		%put P2Value= &P2Value;

		proc sort data=g out=gs nodupkey;
			by &ParentalVar;
		run;

		%NObsVars(gs);
		%if %length(&P2Value) %then %do;
			%if &nobs < 3 %then %do;
				%put ERROR: Parental Lines variable must have 3 (one for each parental and another for offsprings) nonmissing levels. The current variable &ParentalVar has &nobs levels.;
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
		%else %do;
			%if &nobs < 2 %then %do;
				%put ERROR: Parental Lines variable must have 2 (one for a parental and another for offsprings) nonmissing levels. The current variable &ParentalVar has &nobs levels.;
				%let exiterror = 1;
				%goto exit;
			%end;	
		%end;

		%let gid = %sysfunc(open(g));
		%let pvn = %sysfunc(varnum(&gid,&ParentalVar));
		%let pvt = %sysfunc(vartype(&gid,&pvn));
		%let gid = %sysfunc(close(&gid));

		%if &gen = 1 %then %do;
			%if &pvt = C %then %do;
				%let LValue= "_L_";
				%let P1Value= %trim(%left("&P1Value"));
				%if %length(&P2Value) %then 
					%let P2Value= %trim(%left("&P2Value"));
			%end;
			%else %do;
				%let LValue= -9;
			%end;

			%let P1Value= %upcase(&P1Value);
			%let P2Value= %upcase(&P2Value);

			%put P1Value= &P1Value;
			%put P2Value= &P2Value;	
		%end;

		%put pvt= &pvt;
		data g;
			length ParentalVarNew %if &pvt = C %then $32; %else 8;;
			set g;
			%if %length(&P2Value) %then %do;
				if upcase(&ParentalVar) = &P1Value then ParentalVarNew= &P1Value; 
				else if upcase(&ParentalVar) = &P2Value then ParentalVarNew= &P2Value;
				else ParentalVarNew= &LValue;
			%end;
			%else %do;
				if upcase(&ParentalVar) = &P1Value then ParentalVarNew= &P1Value; 
				else ParentalVarNew= &LValue;
			%end;
		run;
		/*data OutLib.c_g2; set g; run;*/

		proc freq data=g noprint;
			table ParentalVarNew / out=ParFreq;
			where not missing(ParentalVarNew); 
		run;
		/*data OutLib.c_parfreq; set parfreq; run;*/

		%let nP1= 0;
		%let nP2= 0;
		data _null_;
			set ParFreq;
			if upcase(ParentalVarNew) = &P1Value then call symputx("nP1",compress(count));		
			%if %length(&P2Value) %then %do;
				if upcase(ParentalVarNew) = &P2Value then call symputx("nP2",compress(count));
			%end;		
		run;
		%put nP1= &nP1;
		%put nP2= &nP2;

		%if &nP1 ne 1 %then %do;
			%if &nP1 = 0 %then %do;
				%put ERROR: The specified value of &P1Value for Parental Line 1 does not match any value of &ParentalVar variable.;
			%end;
			%else %if &nP1 > 1 %then %do;
				%put ERROR: The specified value of &P1Value for Parental Line 1 matches &nP1 rows of &ParentalVar variable. The value of Parental Line 1 must appear exactly once in the rows of &ParentalVar variable. Please check that the value of Parental Line 1 appears only once in the rows of &ParentalVar variable.;
			%end;
			%let exiterror = 1;
			%goto exit;	
		%end;

		%if %length(&P2Value) %then %do; 
			%if &nP2 ne 1 %then %do;
				%if &nP2 = 0 %then %do;
					%put ERROR: The specified value of &P2Value for Parental Line 2 does not match any value of &ParentalVar.;
				%end;
				%else %if &nP2 > 1 %then %do;
					%put ERROR: The specified value of &P2Value for Parental Line 2 matches &nP2 rows of &ParentalVar variable. The value of Parental Line 2 must appear exactly once in the rows of &ParentalVar variable. Please check that the value of Parental Line 2 appears only once in the rows of &ParentalVar variable.;
				%end;
				%let exiterror = 1;
				%goto exit;	
			%end;	
		%end;

		%if &gen= 1 and &NBack > 1 %then %do;		
			data backparents;
				set g;
				where 
					%if &pvt = C %then %do;
						upcase(&ParentalVar) = &P1Value 
						%if %length(&P2Value) %then %do; 
							or upcase(&ParentalVar) = &P2Value
						%end;
					%end;
					%else %do;
						&ParentalVar = &P1Value 
						%if %length(&P2Value) %then %do; 
							or &ParentalVar = &P2Value
						%end;					
					%end;
				;
			run;
			/*data OutLib.c_backparents; set backparents; run;*/
		%end;
	%end;

	*S1216215: begin - adding this new chunck of code to fix the referred defect;
	%if &gen= 1 %then %do;
		%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
			data SimStByLines_All;
				length 
					%if &NBack > 1 %then %do;
						%if &pvt= C %then %do;
							_backgeneration_ _orig_m_lines_ _orig_p_lines_
						%end;
						%else %do;
							_backgeneration_ $ &line_length _orig_m_lines_ _orig_p_lines_ 8
						%end;
					%end;
					%else %if &NSelf > 1 %then _selfgeneration_; 
					%else %if &NnonSelf > 1 %then _generation_; 
					_orig_m_ID_ _orig_p_ID_ $ &line_length;
				delete;
			run;

			data SimStByCross_All;
				length 
					%if &NBack > 1 %then %do;
						%if &pvt= C %then %do;
							_backgeneration_ _orig_m_lines_ _orig_p_lines_
						%end;
						%else %do;
							_backgeneration_ $ &line_length _orig_m_lines_ _orig_p_lines_ 8
						%end;
					%end;
					%else %if &NSelf > 1 %then _selfgeneration_;
					%else %if &NnonSelf > 1 %then _generation_; 
					_orig_m_ID_ _orig_p_ID_ $ &line_length;
				delete;
			run;
		%end;

		%if %length(&CInData) %then %do;
			/*
			%PathName(&CInData);
			%if &exiterror %then %do;
				%*put ERROR: Cross data set specification is invalid.;
				%put ERROR: The SAS Data Set Indicating Which Crosses to Simulate from the Options tab is invalid.;
				%goto exit;
			%end;
			%let CInData = &TmpName;
			libname CInLib "&TmpPath";
			data cin;
				set CInLib.&CInData;
				if m_id = p_id then _self_=1;
				else _self_=0;
				keep m_id p_id _self_;
			run;
			%if &syserr > 4 %then %goto exit;

			proc means data=cin min max noprint;
				var _self_;
				output out= matype min= _min_ max= _max_;
			run;

			data _null_;
				set matype;
				call symputx("mintype", _min_);
				call symputx("maxtype", _max_);
			run;

			%if &mintype = &maxtype %then %do;
				%if &mintype = 1 %then %do;
					%let MatingType= Include self only;
				%end;
				%else %do;
					%let MatingType= Include outcross only;
				%end;
			%end;
			%else %do;
				%let MatingType= Include self and outcross;
			%end;
			*/

			%if %upcase(&m_type)= N or %upcase(&m_type)= N %then %do;
				data cin;
					length %if %upcase(&m_type)= N %then m_id; %if %upcase(&p_type)= N %then p_id; $ &line_length;
					set cin(rename=(%if %upcase(&m_type)= N %then m_id=m_id_num; %if %upcase(&p_type)= N %then p_id=p_id_num;));
					%if %upcase(&m_type)= N %then m_id= strip(put(m_id_num, best32.));;
					%if %upcase(&p_type)= N %then p_id= strip(put(p_id_num, best32.));;
					keep m_id p_id %if %upcase(&m_type)= N %then m_id_num; %if %upcase(&p_type)= N %then p_id_num;;
				run;
				%if &syserr > 4 %then %goto exit;
			%end;

			/*data OutLib.c_cin; set cin; run;*/

			* find unique ids and subset m and p;	
			data cm_id;
				set cin;
				keep m_id;
			run;

			data cp_id; 
				set cin;
				keep p_id;
			run;

			data cmp_id;
				length _id_ $ &line_length;
				set cm_id(in=a) cp_id(in=b);
				if a then _id_= m_id;
				else if b then _id_= p_id;
				drop m_id p_id;
			run;	

			proc sort data=cmp_id nodupkey;
				by _id_;
			run;
			/*data OutLib.c_cmp_id; set cmp_id; run;*/

			* find unique ids and subset m and p;
			proc sql;
				create table gsub(drop=_id_) as
				select * 
				from cmp_id as d1 inner /*left*/ join g as d2
					on d1._id_=d2.line_id 
				order by line_id;
			quit;
			/*data OutLib.c_gsub; set gsub; run;*/

			%let gdata=gsub;
		%end;
		%else %do;
			%let gdata=g;
		%end;
	%end;
	%else %do;
		%let gdata=g;
	%end;
	*S1216215: end - adding this new chunck of code to fix the referred defect;
	/*data OutLib.c_gdata&gen; set &gdata; run;*/

	*creating parent 1 data set*;
	data m;
		length m_id $ &line_length;
		set g;
		%if %length(&SexVar) %then %do;
			%if &svt = C %then %do;
				where &SexVar = "&sex1";
			%end;
			%else %do;
				where &SexVar = &sex1;
			%end;
		%end;
		array mm{&nm} %do i=1 %to &nm; m_%sysfunc(varname(&_dsix_,&i)) %end; ;
		array m{&nm} %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end; ;
		m_id = line_id;
		do i=1 to &nm;
			mm[i] = m[i];
			if missing(mm[i]) then mm[i] = 1;
		end;
		keep m_id &TesterVar 
			%if %length(&ParentalVar) %then &ParentalVar ParentalVarNew; 
			%do i=1 %to &nm; 
				m_%sysfunc(varname(&_dsix_,&i)) 
			%end;
		;
		%if %length(&TesterVar) %then %do;
			if &TesterVar = 1 then output;
			rename &TesterVar= m_tester_indicator;
		%end;
		%if %length(&ParentalVar) %then %do;
			rename &ParentalVar= m_parental ParentalVarNew= m_parentalnew;
		%end;
		%if &gen > 1 %then %do;
			rename 
				%if &gen > 1 and &NBack > 1 %then %do;
					_orig_p_lines_= m_orig_p_lines_
				%end;
				_orig_m_ID_= m_orig_m_ID_
				_orig_p_ID_= m_orig_p_ID_
			;
			keep %if &gen > 1 and &NBack > 1 %then _orig_p_lines_; _orig_m_ID_ _orig_p_ID_;
		%end;
	run;
	/*data OutLib.c_m&gen; set m; run;*/

	*creating parent 2 data set;
	data p;
		length p_id $ &line_length;
		set g;
		%if %length(&SexVar) %then %do;
			%if &svt = C %then %do;
				where &SexVar = "&sex2";
			%end;
			%else %do;
				where &SexVar = &sex2;
			%end;
		%end;
		array pm{&nm} %do i=1 %to &nm; p_%sysfunc(varname(&_dsix_,&i)) %end; ;
		array m{&nm} %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end; ;
		p_id = line_id;
		do i=1 to &nm;
			pm[i] = m[i];
			if missing(pm[i]) then pm[i] = 1;
		end;
		keep p_id &TesterVar 
			%if %length(&ParentalVar) %then &ParentalVar ParentalVarNew; 
			%do i=1 %to &nm;
				p_%sysfunc(varname(&_dsix_,&i)) 
			%end;
		;
		%if %length(&TesterVar) %then %do;
			if &TesterVar ^= 1 then output;
			rename &TesterVar= p_tester_indicator;
		%end;
		%if %length(&ParentalVar) %then %do;
			rename &ParentalVar= p_parental ParentalVarNew= p_parentalnew;
		%end;
		%if &gen > 1 %then %do;
			rename 
				%if &gen > 1 and &NBack > 1 %then %do;
					_orig_p_lines_= p_orig_p_lines_
				%end;
				_orig_m_ID_= p_orig_m_ID_
				_orig_p_ID_= p_orig_p_ID_
			;
			keep %if &gen > 1 and &NBack > 1 %then _orig_p_lines_; _orig_m_ID_ _orig_p_ID_;
		%end;
	run;
	/*data OutLib.c_p&gen; set p; run;*/

	*creating crossdat data set;
	/*
	proc sql;
		create table crossdat as
		select * 
		from m as d1 full outer join p as d2
		%if %length(&SexVar) %then %do;
			on d1.m_id <> d2.p_id
		%end;
		%else %do;
			%if &Selfs = Yes %then %do;
				on d1.m_id <= d2.p_id
			%end;
			%else %do;
				on d1.m_id < d2.p_id	
			%end;
		%end;
	*/
		/*
		order by m_id,p_id
		*/
	/*
		;
	quit;
	*/

	%put Selfs= &Selfs;
	%put TesterVar= &TesterVar;
	%put ParentalVar= &ParentalVar;
	%if %length(&TesterVar) %then %do;
		%let TesterOrParental= &TesterVar;
		%let p_var= p_tester_indicator;
		%let m_var= m_tester_indicator;
	%end;
	%else %if %length(&ParentalVar) %then %do;
		%let TesterOrParental= &ParentalVar;
		%let p_var= p_parentalnew;
		%let m_var= m_parentalnew;
	%end;
	%else %do;
		%let TesterOrParental=;
		%let p_var=;
		%let m_var=;
	%end;

	proc sql;
		create table crossdat as
		select * 
		from m as d1 cross join p as d2
		%if &Selfs = No %then %do;
			/*if Selfs=No, then SexVar may be present*/
			%if %length(&SexVar) and %length(&TesterOrParental) %then %do;
				where 
					d1.m_id <> d2.p_id 
					and 
					d1.&m_var <> d2.&p_var
			%end;
			%else %if %length(&SexVar) and %length(&TesterOrParental)=0 %then %do;
				where d1.m_id <> d2.p_id
			%end;
			%else %if %length(&SexVar)=0 and %length(&TesterOrParental) %then %do;
				/*d1.m_id < d2.p_id avoids the reciprocal cross*/
				where 
					%if %length(&ParentalVar) %then %do;
						d1.m_id < d2.p_id
						and 
					%end;
					d1.&m_var <> d2.&p_var
			%end;
			%else %do;
				where d1.m_id < d2.p_id	
			%end;
		%end;
		%else %do;
			/*if Selfs=Yes, then SexVar is NOT present*/
			%if %length(&TesterOrParental) %then %do;
				%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) %then %do;
					where 
						((d1.m_id = d2.p_id and d1.&m_var = d2.&p_var) 
						or
						(d1.m_id < d2.p_id and d1.&m_var <> d2.&p_var))
				%end;
				%else %if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include outcross only)) %then %do;
					where (d1.m_id < d2.p_id and d1.&m_var <> d2.&p_var)	
				%end;
				%else %do;
					%put ERROR: Mating Type error.;
					%let exiterror= 1;
					%goto exit;
				%end;
			%end;
			%else %do;
				%if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self and outcross)) %then %do;
					where d1.m_id <= d2.p_id
				%end;
				%else %if %nrbquote(%upcase(&MatingType))= %nrbquote(%upcase(Include self only)) %then %do;
					where d1.m_id = d2.p_id
				%end;
				%else %do;
					%put ERROR: Mating Type error.;
					%let exiterror= 1;
					%goto exit;
				%end;
			%end;
		%end;
		%if &gen > 1 and &NBack > 1 %then %do;
			and (d1.m_orig_p_lines_ = d2.&p_var or d1.&m_var = d2.p_orig_p_lines_) 
		%end;
		/*
		order by m_id, p_id
		*/
		;
	quit;

	%if &syserr > 4 %then %goto exit;
	/*data OutLib.c_crossdat0_&gen; set crossdat; run;*/

	%if %length(&ParentalVar) %then %do;
		data crossdat;
			length 
				%if &NBack > 1 %then %do;
					%if &pvt= C %then %do;
						_backgeneration_ _orig_m_lines_ _orig_p_lines_
					%end;
					%else %do;
						_backgeneration_ $ &line_length _orig_m_lines_ _orig_p_lines_ 8
					%end;
				%end;
				_orig_m_ID_ _orig_p_ID_ $ &line_length;
			set crossdat;
			%if %length(&P2Value) %then %do;
				if (m_parentalnew = &P1Value and p_parentalnew = &P2Value) or
					(m_parentalnew = &P2Value and p_parentalnew = &P1Value) then delete;
			%end;
			_backgeneration_= "BC_&gen";
			%if %length(&P2Value) %then %do;
				if m_parental ^= &P1Value and m_parental ^= &P2Value then do;
					_orig_m_lines_= m_parental; 
					%if &gen=1 %then %do;
						_orig_m_ID_= m_id;
					%end;
					%else %do;
						_orig_m_ID_= m_orig_m_ID_;
					%end;
				end;
				else if p_parental ^= &P1Value and p_parental ^= &P1Value then do;
					_orig_m_lines_= p_parental;
					%if &gen=1 %then %do;
						_orig_m_ID_= p_id; 
					%end;
					%else %do;
						_orig_m_ID_= p_orig_m_ID_;
					%end;
				end;

				if m_parental = &P1Value or m_parental = &P2Value then do;
					_orig_p_lines_= m_parental;
					%if &gen=1 %then %do;
						_orig_p_ID_= m_id;
					%end;
					%else %do;
						_orig_p_ID_= m_orig_p_ID_;
					%end;
				end;
				else if p_parental = &P1Value or p_parental = &P2Value then do;
					_orig_p_lines_= p_parental;
					%if &gen=1 %then %do;
						_orig_p_ID_= p_id;
					%end;
					%else %do;
						_orig_p_ID_= p_orig_p_ID_;
					%end;
				end;
			%end;
			%else %do;
				if m_parental ^= &P1Value then do;
					_orig_m_lines_= m_parental; 
					%if &gen=1 %then %do;
						_orig_m_ID_= m_id; 
					%end;
					%else %do;
						_orig_m_ID_= m_orig_m_ID_;
					%end;
				end;
				else if p_parental ^= &P1Value then do;
					_orig_m_lines_= p_parental; 
					%if &gen=1 %then %do;
						_orig_m_ID_= p_id; 
					%end;
					%else %do;
						_orig_m_ID_= p_orig_m_ID_;
					%end;
				end;

				if m_parental = &P1Value then do;
					_orig_p_lines_= m_parental;
					%if &gen=1 %then %do;
						_orig_p_ID_= m_id; 
					%end;
					%else %do;
						_orig_p_ID_= m_orig_p_ID_;
					%end;
				end;
				else if p_parental = &P1Value then do;
					_orig_p_lines_= p_parental;
					%if &gen=1 %then %do;
						_orig_p_ID_= p_id;
					%end;
					%else %do;
						_orig_p_ID_= p_orig_p_ID_;
					%end;
				end;
			%end;
			if &gen= 1 then do;
				m_id= _orig_m_ID_;
				p_id= _orig_p_ID_;
			end;
			drop m_parental p_parental;
		run;
	%end;
	%else %if %length(&TesterVar) %then %do;
		data crossdat;
			length m_tester_indicator p_tester_indicator 8 m_id p_id $ &line_length;
			set crossdat;
			if m_tester_indicator = 1 then do;
				tester_id= m_id;
				nontester_id= p_id;
			end;
			else if p_tester_indicator = 1 then do;
				tester_id= p_id;
				nontester_id= m_id;
			end;
		run;
	%end;
	%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then %do;
		data crossdat;
			length _selfgeneration_ /*_orig_m_lines_ _orig_p_lines_*/ 
				_orig_m_ID_ _orig_p_ID_ $ &line_length;
			set crossdat;
			%if &gen = 1 %then %do;
				_selfgeneration_= "F_&gen";
			%end;
			%else %do;
				_selfgeneration_= "SF_&gen";
			%end;			

			*_orig_m_lines_= m_parental;
			*_orig_p_lines_= p_parental;
			%if &gen=1 %then %do;
				_orig_m_ID_= m_id;
				_orig_p_ID_= p_id; 
			%end;
			%else %do;
				_orig_m_ID_= m_orig_m_ID_;
				_orig_p_ID_= p_orig_p_ID_;
			%end;
		run;
	%end;
	%else %if &NnonSelf > 1 %then %do;
		data crossdat;
			length _generation_ /*_orig_m_lines_ _orig_p_lines_*/ 
				_orig_m_ID_ _orig_p_ID_ $ &line_length;
			set crossdat;
			%if &gen = 1 %then %do;
				_generation_= "F_&gen";
			%end;
			%else %do;
				_generation_= "F_&gen";
			%end;			

			*_orig_m_lines_= m_parental;
			*_orig_p_lines_= p_parental;
			%if &gen=1 %then %do;
				_orig_m_ID_= m_id;
				_orig_p_ID_= p_id; 
			%end;
			%else %do;
				_orig_m_ID_= m_orig_m_ID_;
				_orig_p_ID_= p_orig_p_ID_;
			%end;
		run;
	%end;	

	/*data OutLib.c_crossdat1_&gen; set crossdat; run;*/

	%if &gen = 1 %then %do;
		%if %length(&CInData) %then %do;
			/*
			*adding selfing when a data set with specific crosses is specified;
			%if %length(&SexVar)=0 and &Selfs = Yes %then %do;
				data cmp_id;
					merge cmp_id(rename=(_id_= m_id)) cmp_id(rename=(_id_=p_id));
				run;

				data cin;
					length m_id p_id $32;
					set cin cmp_id;
				run;

				prod sort data=c nodukey;
					by m_id p_id;
				run;

				data OutLib.c_c1; set cin; run;
			%end;
			*/

			proc sql;
				create table cross1 as
				select * 
				from cin as d1 inner /*left*/ join crossdat as d2
					on (
						(d1.p_id=d2.p_id and d1.m_id=d2.m_id) or
						(d1.p_id=d2.m_id and d1.m_id=d2.p_id)
					)
				/*order by p_id,m_id;*/
				order by m_id, p_id;
			quit;
			
			%NOBsVars(cross1);
			%if &nobs =0 %then %do;
				%put ERROR: None of the crosses specified in the SAS Data Set Indicating Which Crosses to Simulate option in the Analysis tab could be made from the available parents in the Input SAS Data Set in the General Tab.;
				%let exiterror= 1;
				%goto exit;
			%end;			

			proc datasets library= work nolist;
				delete crossdat;
				change cross1= crossdat;
			quit;
		%end;
	%end;	

	%if &Simulate = Yes %then %do;
		proc sort data=crossdat;
			by m_id p_id;
		run;
		/*data OutLib.c_crossdat2_&gen; set crossdat; run;*/
	%end;

	%if (&gen = 1 and &ntrait ^= &ntree) %then %do;
		data OutLib.&CE_OutDataName.(&dsoptions);
		*data OutLib.&CE_OutDataName;
			set crossdat;
			* compute max and min traits via optimal genotypes;
			* also compute trait variance assuming genotypes are independent binomials;
			%do i=1 %to &ntrait;
				%if ^%scan(&itree, &i) %then %do;
					length &&trait&i.._Mean &&trait&i.._Max &&trait&i.._Min &&trait&i.._Range &&trait&i.._StdDev 8;
					array m_geno&i{*} %do j=1 %to &&nm&i; m_%sysfunc(varname(&&_dsix_&i,&j)) %end; ;
					array p_geno&i{*} %do j=1 %to &&nm&i; p_%sysfunc(varname(&&_dsix_&i,&j)) %end; ;
					array coef&i{*} coef&i._1-coef&i._&&nm&i;
					%do j=1 %to &&nm&i;
						coef&i[&j] = 0 + %sysfunc(varlabel(&&_dsix_&i,&j));
					%end;
					tmax = &&t&i.intercept;
					tmin = &&t&i.intercept;
					tv = 0;
					do j=1 to dim(coef&i);
						maxm = max(p_geno&i[j],m_geno&i[j]);
						minm = min(p_geno&i[j],m_geno&i[j]);
						if (coef&i[j] >= 0) then do;
							if minm = 0 and maxm = 0 then maxgeno = 0;
							else if minm = 0 then maxgeno = 1;
							else maxgeno = 2;
							if minm = 2 and maxm = 2 then mingeno = 2;
							else if maxm = 2 then  mingeno = 1;
							else mingeno = 0;
						end;
						else do;
							if minm = 2 and maxm = 2 then maxgeno = 2;
							else if maxm = 2 then maxgeno = 1;
							else maxgeno = 0;
							if minm = 0 and maxm = 0 then mingeno = 0;
							else if minm = 0 then mingeno = 1;
							else mingeno = 2;
						end;
						tmax = tmax + coef&i[j] * maxgeno;
						tmin = tmin + coef&i[j] * mingeno;

						if minm = 1 and maxm = 1 then tvj = 0.5;
						else if (maxm - minm) = 1 then tvj = 0.25;
						else tvj = 0;			
						if tvj > 0 then do;
							tv = tv + tvj * coef&i[j]**2;
						end;
					end;
					&&trait&i.._Max = tmax;
					&&trait&i.._Min = tmin;
					&&trait&i.._StdDev = sqrt(tv);

					* compute mean and range from max and min;
					* the mean is the average of the max and min because the distribution is symmetric;
					&&trait&i.._Mean = (&&trait&i.._Max + &&trait&i.._Min)/2;
					&&trait&i.._Range = &&trait&i.._Max - &&trait&i.._Min;
				%end;
			%end;
			keep m_id p_id 	
				%do i=1 %to &ntrait;
					%if ^%scan(&itree, &i) %then %do;
						&&trait&i.._Mean
						&&trait&i.._Max
						&&trait&i.._Min
						&&trait&i.._Range
						&&trait&i.._StdDev
					%end;
				%end;
				%if %length(&TesterVar) %then m_tester_indicator p_tester_indicator tester_id nontester_id;
				%if %length(&ParentalVar) %then /*m_parental p_parental*/ _orig_m_lines_ _orig_p_lines_;
				%if &NBack > 1 %then _backgeneration_;
				%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then _selfgeneration_;
				%else %if &NnonSelf > 1 %then _generation_;
			;
			/*
			%if %length(&ParentalVar) %then %do;
				rename 
					m_parental= _orig_m_lines_ 
					p_parental= _orig_p_lines_
				;
			%end;
			*/
		run;

		%if &syserr > 4 %then %goto exit;
		/*data OutLib.c_OutDataName0_&gen; set OutLib.&CE_OutDataName; run;*/
	%end;

	%if &Simulate = Yes %then %do;

		%put nm= &nm;

		%NObsVars(crossdat);
		%let ncrosses= &nobs;
		%put ncrosses= &ncrosses;

		data simdat(&dsoptions);
			%if %length(&AnnoInData) and %upcase(&UseMapInfo)= YES %then %do;
				merge crossdat subchr subnmrk subpos sublb sublen end=_eof_;
				seed= &seed + &gen + _n_;
			%end;
			%else %do;
				set crossdat end=_eof_;			
			%end;		
			%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
				length sim 8;  
				%if %length(&SexVar) %then %do;
					length &SexVar %if &svt=C %then %do; $ %end; &svl;
				%end;
			%end;
			array pm{&nm} %do i=1 %to &nm; p_%sysfunc(varname(&_dsix_,&i)) %end; ;;
			array mm{&nm} %do i=1 %to &nm; m_%sysfunc(varname(&_dsix_,&i)) %end; ;;
			array m{&nm} %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end; ; /**simulated marker scores**/

			%if %length(&AnnoInData) and %upcase(&UseMapInfo)= YES %then %do;
				array _chr{*} _chr_:; /**input chromosome ID**/
				array _nmk{*} _nmk_:; /**input number of markers on each chromosome**/
				array _pos{*} _pos_:; /**input position on each chromosome**/
				array _lbo{*} _lbo_:; /**input lower bound of chromosomes**/
				array _len{*} _len_:; /**lenght of chromosomes**/
				array chr{&nc}; /**chromosome ID**/
				array nmk{&nc}; /**number of markers on each chromosome**/
				array pos{&nm}; /**marker position on each chromosome**/
				array lbo{&nc}; /**lower bound of chromosomes**/
				array len{&nc}; /**length of chromosomes**/
				array pnr{&nc}; /**number of breaks on each chromosome from father**/
				array prc{&maxr}; /**place of breaks on each chromosome from father**/
				array p1h{&nm}; /**haplotype 1 for parent father**/;
				array p2h{&nm}; /**haplotype 2 for parent father**/;
				array pht{&nm}; /**temporary haplotype for parent father**/;
				%if %upcase(&SimRilFast) ^= YES %then %do;
					array mnr{&nc}; /**number of breaks on each chromosome from mather**/
					array mrc{&maxr}; /**place of breaks on each chromosome from mather**/
					array m1h{&nm}; /**haplotype 1 for parent mather**/;
					array m2h{&nm}; /**haplotype 2 for parent mather**/;
					array mht{&nm}; /**temporary haplotype for parent mather**/;
				%end;

				**put chromosome and position in new arrays**;
				if _n_ = 1 then do;
					do i= 1 to &nm;
						pos[i]= _pos[i];
					end;
					do i= 1 to &nc;
						chr[i]= _chr[i];
						nmk[i]= _nmk[i];
						lbo[i]= _lbo[i];
						len[i]= _len[i];
					end;
				end;
				retain chr: nmk: pos: lbo: len:;				
			%end;			

			%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 and &ntree = 0 %then %do;
				%do i=1 %to &ntrait;
				%*if ^%scan(&itree, &i) %then %do;
					&&trait&i.._Sim_Mean = 0;
					&&trait&i.._Sim_SS = 0;
					&&trait&i.._Sim_Max = -1e8;
					&&trait&i.._Sim_Min = 1e8;
				%*end;
				%end;
				%if %upcase(&ComputeSelIndex) = YES %then %do;
					&SelIndex._Sim_Mean = 0;
					&SelIndex._Sim_SS = 0;
					&SelIndex._Sim_Max = -1e8;
					&SelIndex._Sim_Min = 1e8;				
				%end;
			%end;

			do sim = 1 to &nsim;
				%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
					%if %length(&SexVar) %then %do;
						u = ranuni(&seed + &gen);
						if u <= 0.5 then do;
							%if &svt = C %then %do;
								&SexVar = "&sex1";
							%end;
							%else %do;
								&SexVar = &sex1;
							%end;
						end;
						else do;
							%if &svt = C %then %do;
								&SexVar = "&sex2";
							%end;
							%else %do;
								&SexVar = &sex2;
							%end;			
						end;
					%end;
				%end;

				%if %length(&AnnoInData) and %upcase(&UseMapInfo)= YES %then %do;
					**simulate number of crossovers (breaks) within each chromosome**;
					do i= 1 to dim(pnr);
						if i = 1 then do; 
							call ranpoi(seed, len[i]/100, pnr[i]); **father**;
							%if %upcase(&SimRilFast) ^= YES %then %do;
								call ranpoi(seed, len[i]/100, mnr[i]); **mather**;
							%end;
						end;
						else do;
							if chr[i] ^= chr[i - 1] then do;
								call ranpoi(seed, len[i]/100, pnr[i]); **father**;
								%if %upcase(&SimRilFast) ^= YES %then %do;
									call ranpoi(seed, len[i]/100, mnr[i]); **mather**;
								%end;
							end;
							else do;
								pnr[i] = pnr[i - 1]; **father**;
								%if %upcase(&SimRilFast) ^= YES %then %do;
									mnr[i] = mnr[i - 1]; **mather**;
								%end;
							end; 
						end; 
					end; 

					v0= 1;
					v1= 1;
					v2= 1;
					v3= 1;
					do c= 1 to &nc;
						**simulate places of recombination within each chromosome from father**;
						maxprc= -1e8;
						do i= 1 to pnr[c];
							prc[i]= len[c]*ranuni(&seed + &gen) + lbo[c]; 
							*put 'sim c i maxprc' sim c i maxprc;
							if prc[i] < maxprc then do;
								do k= i to 2 by -1;
									*put sim k prc[k] prc[k - 1] @;
									if prc[k] < prc[k - 1] then do;
										prctmp= prc[k];
										prc[k]= prc[k - 1];
										prc[k - 1]= prctmp;
									end;
								end;
							end;
							maxprc= prc[i];
						end;

						** begin: checking code only - father**;
						*do i= 1 to pnr[c];
						*	put 'sim= ' sim 'c= ' c 'i= ' i  'pnr= ' pnr[c] 'prc= ' prc[i] @;
						*end;
						*put;
						** end: checking code only - father**;

						%if %upcase(&SimRilFast) ^= YES %then %do;
							**simulate places of recombination within each chromosome from mather**;
							maxmrc= -1e8;
							do i= 1 to mnr[c];
								mrc[i]= len[c]*ranuni(&seed + &gen) + lbo[c]; 
								*put 'sim c i maxmrc ' sim c i maxmrc;
								if mrc[i] < maxmrc then do;
									do k= i to 2 by -1;
										*put sim k mrc[k] mrc[k - 1] @;
										if mrc[k] < mrc[k - 1] then do;
											mrctmp= mrc[k];
											mrc[k]= mrc[k - 1];
											mrc[k - 1]= mrctmp;
										end;
									end;
								end;
								maxmrc= mrc[i];
							end;

							** begin: checking code only - mather**;
							*do i= 1 to mnr[c];
							*	put 'sim= ' sim 'c= ' c 'i= ' i 'mnr= ' mnr[c] 'mrc= ' mrc[i] @;
							*end;
							*put;
							** end: checking code only - mather**;
						%end;

						**creating haplotypes of parents based on marker scores of 0, 1 and 2**;
						%if %upcase(&SimRilFast) ^= YES %then %do;
							do i= 1 to nmk[c];
								**father**;
								if pm[v0]= 0 then do;
									p1h[v0]= 0;
									p2h[v0]= 0;
								end;
								else if pm[v0]= 1 then do;
									**phase unknown, assign dominant allele to haplotype 1**;
									**phase unknown, randomly assign dominant allele to haplotypes**;
									*u = ranuni(&seed + &gen);
									*if u <= 0.5 then do;
										p1h[v0]= 1;
										p2h[v0]= 0;
									*end;
									*else do;
									*	p1h[v0]= 0;
									*	p2h[v0]= 1;								
									*end;
								end;
								else if pm[v0]= 2 then do;
									p1h[v0]= 1;
									p2h[v0]= 1;
								end;
								else do;
									if not missing(pm[v0]) then do;
										**imputed data with non-integer values**;
										pm[v0]= round(pm[v0],1);
										*put sim pm[v0];
										if pm[v0]= 0 then do;
											p1h[v0]= 0;
											p2h[v0]= 0;
										end;
										else if pm[v0]= 1 then do;
											**phase unknown, assign dominant allele to haplotype 1**;
											**phase unknown, randomly assign dominant allele to haplotypes**;
											*u = ranuni(&seed + &gen);
											*if u <= 0.5 then do;
												p1h[v0]= 1;
												p2h[v0]= 0;
											*end;
											*else do;
											*	p1h[v0]= 0;
											*	p2h[v0]= 1;								
											*end;
										end;
										else if pm[v0]= 2 then do;
											p1h[v0]= 1;
											p2h[v0]= 1;
										end;
										else do;
											p1h[v0]= .;
											p2h[v0]= .;
										end;
									end;
									else do;
										p1h[v0]= .;
										p2h[v0]= .;
									end;								
								end;

								**mather**;
								if mm[v0]= 0 then do;
									m1h[v0]= 0;
									m2h[v0]= 0;
								end;
								else if mm[v0]= 1 then do;
									**phase unknown, assign dominant allele to haplotype 1**;
									**phase unknown, randomly assign dominant allele to haplotypes**;
									*u = ranuni(&seed + &gen);
									*if u <= 0.5 then do;
										m1h[v0]= 1;
										m2h[v0]= 0;
									*end;
									*else do;
									*	m1h[v0]= 0;
									*	m2h[v0]= 1;								
									*end;
								end;
								else if mm[v0]= 2 then do;
									m1h[v0]= 1;
									m2h[v0]= 1;
								end;
								else do;
									if not missing(mm[v0]) then do;
										**imputed data with non-integer values**;
										mm[v0]= round(mm[v0],1);
										*put sim mm[v0];
										if mm[v0]= 0 then do;
											m1h[v0]= 0;
											m2h[v0]= 0;
										end;
										else if mm[v0]= 1 then do;
											**phase unknown, assign dominant allele to haplotype 1**;
											**phase unknown, randomly assign dominant allele to haplotypes**;
											*u = ranuni(&seed + &gen);
											*if u <= 0.5 then do;
												m1h[v0]= 1;
												m2h[v0]= 0;
											*end;
											*else do;
											*	m1h[v0]= 0;
											*	m2h[v0]= 1;								
											*end;
										end;
										else if mm[v0]= 2 then do;
											m1h[v0]= 1;
											m2h[v0]= 1;
										end;
										else do;
											m1h[v0]= .;
											m2h[v0]= .;
										end;
									end;
									else do;
										m1h[v0]= .;
										m2h[v0]= .;
									end;								
								end;
								v0= v0 + 1;
							end;
						%end;
						%else %do;
							do i= 1 to nmk[c];
								**father**;
								p1h[v0]= 0;
								p2h[v0]= 2;
								v0= v0 + 1;
							end;
						%end;						

						**checking for breaks (crossovers) on haplotype from father**;			
						if pnr[c] > 0 then do;
							b0= 1;
							v1= v1 + 1;
							**loop over markers, starting from the second marker on each chromosome**;
							do i= 2 to nmk[c];
								*put 'sim= ' sim 'c= ' c 'i= ' i 'pnr= ' pnr[c] 'b0= ' b0 'pos= ' pos[v1];
								*do while(b0 <= pnr[c] and prc[b0] <= pos[v1]);
								do while(b0 <= pnr[c]);
									if prc[b0] > pos[v1] then goto endploop;
									*put 'Recombine ' 'sim= ' sim 'c= ' c 'i= ' i 'pnr= ' pnr[c] 'b0= ' b0 'pos= ' pos[v1] 'prc= ' prc[b0];
									b2= v1;
									do i2= i to nmk[c];
										*put 'B b2 p1h p2h ' b2 p1h[b2] p2h[b2];
										pht[b2]= p1h[b2];
										p1h[b2]= p2h[b2];
										p2h[b2]= pht[b2];
										*put 'A b2 p1h p2h ' b2 p1h[b2] p2h[b2];
										b2= b2 + 1;
									end;
									b0= b0 + 1;
								end; **end breaks loop**;
								endploop:;
								v1= v1 + 1;
							end; 
							**end marker loop on each chromosome**;
						**end if breaks > 0 for father**;
						end; 
						else do;
							v1= v1 + nmk[c];
						end; 
						**end if breaks = 0 for father**;

						%if %upcase(&SimRilFast) ^= YES %then %do;
							**checking for breaks (crossovers) on haplotype from mather**;			
							if mnr[c] > 0 then do;
								b0= 1;
								v2= v2 + 1;
								**loop over markers, starting from the second marker on each chromosome**;
								do i= 2 to nmk[c];
									*do while(b0 <= mnr[c] and mrc[b0] <= pos[v2]);
									do while(b0 <= mnr[c]);
										if mrc[b0] > pos[v2] then goto endmloop;
										*put 'Recombine ' 'sim= ' sim 'c= ' c 'i= ' i 'mnr= ' mnr[c] 'b0= ' b0 'pos= ' pos[v2] 'mrc= ' mrc[b0];
										b2= v2;
										do i2= i to nmk[c];
											*put 'B b2 m1h m2h ' b2 m1h[b2] m2h[b2];
											mht[b2]= m1h[b2];
											m1h[b2]= m2h[b2];
											m2h[b2]= mht[b2];
											*put 'A b2 m1h m2h ' b2 m1h[b2] m2h[b2];
											b2= b2 + 1;
										end;
										b0= b0 + 1;
									end; **end breaks loop**;
									endmloop:;
									v2= v2 + 1;
								end; 
								**end marker loop (i) on each chromosome**;
							end; **end if breaks > 0 for mather**;
							else do;
								v2= v2 + nmk[c];
							end; **end if breaks = 0 for mather**;
						%end;

						u= ranuni(&seed + &gen);
						b3= v3;
						do i2= 1 to nmk[c];
							%if %upcase(&SimRilFast) ^= YES %then %do;
								if u <= 0.25 then do;
									m[b3]= p1h[b3] + m1h[b3];
								end;
								else if u <= .50 then do;
									m[b3]= p1h[b3] + m2h[b3];
								end;
								else if u <= .75 then do;
									m[b3]= p2h[b3] + m1h[b3];
								end;
								else do;
									m[b3]= p2h[b3] + m2h[b3];
								end;
							%end;
							%else %do;
								if u <= 0.50 then do;
									if p1h[b3] = 2 then m[b3]= pm[b3];
									else m[b3]= mm[b3];
								end;
								else do;
									if p2h[b3] = 2 then m[b3]= pm[b3];
									else m[b3]= mm[b3];
								end; 
							%end;

							*check for the presence of heterozygous, and set them to one of the homozygous genotypes*;
							*NSelf has been set to a large value to guarantee a very low amount of heterzygous in RILs*;
							%if (%upcase(&SimRilSSD)= YES and &gen= &NSelf) or (%upcase(&SimRilFast)= YES) %then %do;
								if m[b3] = 1 then do;
									u = ranuni(&seed + &gen);
									if u < 0.5 then m[b3]= 0;
									else m[b3] = 2;
								end;
							%end;							

							b3= b3 + 1;
						end;
						v3= v3 + nmk[c];
					end; 
					**end chromosome loop (c)**;
				%end;
				%else %do;
					*independent markers assumption*;
					do i=1 to &nm;
						/*
						if pm[i] = 0 then do;
							if mm[i] = 0 then m[i] = 0;
							else if mm[i] = 2 then m[i] = 1;
							else do;
								u = ranuni(&seed + &gen);
								if (u < 0.5) then m[i] = 0;
								else m[i] = 1;
							end;
						end;
						else if pm[i] = 2 then do;
							if mm[i] = 0 then m[i] = 1;
							else if mm[i] = 2 then m[i] = 2;
							else do;
								u = ranuni(&seed + &gen);
								if (u < 0.5) then m[i] = 1;
								else m[i] = 2;
							end;
						end;
						else do;
							u = ranuni(&seed + &gen);
							if mm[i] = 0 then do;
								if (u < 0.5) then m[i] = 0;
								else m[i] = 1;
							end;
							else if mm[i] = 2 then do;
								if (u < 0.5) then m[i] = 1;
								else m[i] = 2;
							end;
							else do;
								u2 = ranuni(&seed + &gen);
								if (u < 0.5) and (u2 < 0.5) then m[i] = 0;
								else if (u >= 0.5) and (u2 >= 0.5) then m[i] = 2;
								else m[i] = 1;
							end;
						end;
						*/

						if pm[i] = 0 then do;
							if mm[i] = 0 then m[i] = 0;
							else if mm[i] = 2 then m[i] = 1;
							else if mm[i] = 1 then do;
								u = ranuni(&seed + &gen);
								if (u < 0.5) then m[i] = 0;
								else m[i] = 1;
							end;
							else do;
								if not missing(mm[i]) then do;
									**imputed data with non-integer values**;
									mm[i]= round(mm[i],1);
									*put sim mm[i];
									if mm[i] = 0 then m[i] = 0;
									else if mm[i] = 2 then m[i] = 1;
									else if mm[i] = 1 then do;
										u = ranuni(&seed + &gen);
										if (u < 0.5) then m[i] = 0;
										else m[i] = 1;
									end;																
								end;	
								else do;
									*missing mm data*;
									m[i]=.;
								end;
							end;
						end;
						else if pm[i] = 2 then do;
							if mm[i] = 0 then m[i] = 1;
							else if mm[i] = 2 then m[i] = 2;
							else if mm[i] = 1 then do;
								u = ranuni(&seed + &gen);
								if (u < 0.5) then m[i] = 1;
								else m[i] = 2;
							end;
							else do;
								if not missing(mm[i]) then do;
									**imputed data with non-integer values**;
									mm[i]= round(mm[i],1);
									*put sim mm[i];
									if mm[i] = 0 then m[i] = 1;
									else if mm[i] = 2 then m[i] = 2;
									else if mm[i] = 1 then do;
										u = ranuni(&seed + &gen);
										if (u < 0.5) then m[i] = 1;
										else m[i] = 2;
									end;
								end;	
								else do;
									*missing mm data*;
									m[i]=.;
								end;						
							end;
						end;
						else if pm[i] = 1 then do;
							u = ranuni(&seed + &gen);
							if mm[i] = 0 then do;
								if (u < 0.5) then m[i] = 0;
								else m[i] = 1;
							end;
							else if mm[i] = 2 then do;
								if (u < 0.5) then m[i] = 1;
								else m[i] = 2;
							end;
							else if mm[i] = 1 then do;
								u2 = ranuni(&seed + &gen);
								if (u < 0.5) and (u2 < 0.5) then m[i] = 0;
								else if (u >= 0.5) and (u2 >= 0.5) then m[i] = 2;
								else m[i] = 1;
							end;
							else do;
								if not missing(mm[i]) then do;
									**imputed data with non-integer values**;
									mm[i]= round(mm[i],1);
									*put sim mm[i];
									u = ranuni(&seed + &gen);
									if mm[i] = 0 then do;
										if (u < 0.5) then m[i] = 0;
										else m[i] = 1;
									end;
									else if mm[i] = 2 then do;
										if (u < 0.5) then m[i] = 1;
										else m[i] = 2;
									end;
									else if mm[i] = 1 then do;
										u2 = ranuni(&seed + &gen);
										if (u < 0.5) and (u2 < 0.5) then m[i] = 0;
										else if (u >= 0.5) and (u2 >= 0.5) then m[i] = 2;
										else m[i] = 1;
									end;
								end;
								else do;
									*missing mm data*;
									m[i]=.;
								end;
							end;
						end;
						else do;
							if not missing(pm[i]) then do;
								**imputed data with non-integer values**;
								pm[i]= round(pm[i],1);
								*put sim pm[i];
								if pm[i] = 0 then do;
									if mm[i] = 0 then m[i] = 0;
									else if mm[i] = 2 then m[i] = 1;
									else if mm[i] = 1 then do;
										u = ranuni(&seed + &gen);
										if (u < 0.5) then m[i] = 0;
										else m[i] = 1;
									end;
									else do;
										if not missing(mm[i]) then do;
											**imputed data with non-integer values**;
											mm[i]= round(mm[i],1);
											*put sim mm[i];
											if mm[i] = 0 then m[i] = 0;
											else if mm[i] = 2 then m[i] = 1;
											else if mm[i] = 1 then do;
												u = ranuni(&seed + &gen);
												if (u < 0.5) then m[i] = 0;
												else m[i] = 1;
											end;																
										end;	
										else do;
											*missing mm data*;
											m[i]=.;
										end;
									end;
								end;
								else if pm[i] = 2 then do;
									if mm[i] = 0 then m[i] = 1;
									else if mm[i] = 2 then m[i] = 2;
									else if mm[i] = 1 then do;
										u = ranuni(&seed + &gen);
										if (u < 0.5) then m[i] = 1;
										else m[i] = 2;
									end;
									else do;
										if not missing(mm[i]) then do;
											**imputed data with non-integer values**;
											mm[i]= round(mm[i],1);
											*put sim mm[i];
											if mm[i] = 0 then m[i] = 1;
											else if mm[i] = 2 then m[i] = 2;
											else if mm[i] = 1 then do;
												u = ranuni(&seed + &gen);
												if (u < 0.5) then m[i] = 1;
												else m[i] = 2;
											end;
										end;	
										else do;
											*missing mm data*;
											m[i]=.;
										end;						
									end;
								end;
								else if pm[i] = 1 then do;
									u = ranuni(&seed + &gen);
									if mm[i] = 0 then do;
										if (u < 0.5) then m[i] = 0;
										else m[i] = 1;
									end;
									else if mm[i] = 2 then do;
										if (u < 0.5) then m[i] = 1;
										else m[i] = 2;
									end;
									else if mm[i] = 1 then do;
										u2 = ranuni(&seed + &gen);
										if (u < 0.5) and (u2 < 0.5) then m[i] = 0;
										else if (u >= 0.5) and (u2 >= 0.5) then m[i] = 2;
										else m[i] = 1;
									end;
									else do;
										if not missing(mm[i]) then do;
											**imputed data with non-integer values**;
											mm[i]= round(mm[i],1);
											*put sim mm[i];
											u = ranuni(&seed + &gen);
											if mm[i] = 0 then do;
												if (u < 0.5) then m[i] = 0;
												else m[i] = 1;
											end;
											else if mm[i] = 2 then do;
												if (u < 0.5) then m[i] = 1;
												else m[i] = 2;
											end;
											else if mm[i] = 1 then do;
												u2 = ranuni(&seed + &gen);
												if (u < 0.5) and (u2 < 0.5) then m[i] = 0;
												else if (u >= 0.5) and (u2 >= 0.5) then m[i] = 2;
												else m[i] = 1;
											end;
										end;
										else do;
											*missing mm data*;
											m[i]=.;
										end;
									end;
								end;
							end;
							else do;
								*missing pm data*;
								m[i]=.;		
							end;
						end;						

						*check for the presence of heterozygous, and set them to one of the homozygous genotypes*;
						*NSelf has been set to a large value to guarantee a very low amount of heterzygous in RILs*;
						%if (%upcase(&SimRilSSD)= YES and &gen= &NSelf) or (%upcase(&SimRilFast)= YES) %then %do;
							if m[i] = 1 then do;
								u = ranuni(&seed + &gen);
								if u < 0.5 then m[i]= 0;
								else m[i] = 2;
							end;
						%end;							
					end;
				%end;

				%do i=1 %to &ntrait;
					%if ^%scan(&itree, &i) %then %do;
						%if ^&&Flag_AD&i %then %do;
							%let Code = %scan(&ScoringCode,&i,'|');
							**using double semicolon to avoid error when the score file does not end with a semicolon**;
							%include "&ScoringFolder.\&Code";; 
						%end;
						%else %do;
							%let ReCode = %scan(&ReCodeScores,&i," ");
							**using double semicolon to avoid error when the score file does not end with a semicolon**;
							%include "&OutPath.\&ReCode";;
							%*include "&ReCode";;
						%end;

						%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 and &ntree = 0 %then %do;
							&&trait&i.._Sim_Mean = &&trait&i.._Sim_Mean + &&trait2&i;
							&&trait&i.._Sim_SS = &&trait&i.._Sim_SS + &&trait2&i * &&trait2&i;
							&&trait&i.._Sim_Max = max(&&trait&i.._Sim_Max,&&trait2&i);
							&&trait&i.._Sim_Min = min(&&trait&i.._Sim_Min,&&trait2&i);
						%end;
					%end;
				%end;

				%if %upcase(&ComputeSelIndex) = YES and &ntree = 0 %then %do;
					&SelIndex2= &SelIndexFormula;
					%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 %then %do;
						&SelIndex._Sim_Mean = &SelIndex._Sim_Mean + &SelIndex2;
						&SelIndex._Sim_SS = &SelIndex._Sim_SS + &SelIndex2 * &SelIndex2;
						&SelIndex._Sim_Max = max(&SelIndex._Sim_Max,&SelIndex2);
						&SelIndex._Sim_Min = min(&SelIndex._Sim_Min,&SelIndex2);
					%end;
				%end;

				%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
					/*
					if sim = &nsim then do; 
						%do i=1 %to &ntrait;
							%if ^%scan(&itree, &i) %then %do;
							&&trait&i.._Sim_Mean = &&trait&i.._Sim_Mean / &nsim;
							&&trait&i.._Sim_Range = &&trait&i.._Sim_Max - &&trait&i.._Sim_Min;
							%if %upcase(&NSim) > 1 %then %do;
								&&trait&i.._Sim_StdDev = sqrt((&&trait&i.._Sim_SS / &nsim - &&trait&i.._Sim_Mean * &&trait&i.._Sim_Mean) * (&nsim / (&nsim - 1)));
							%end;
							%else %do;
								&&trait&i.._Sim_StdDev = sqrt((&&trait&i.._Sim_SS / &nsim - &&trait&i.._Sim_Mean * &&trait&i.._Sim_Mean) * (&nsim / (&nsim)));				
							%end;
							%end;
						%end;
					end;
					*/
					output;					
				%end;
			end; 
			**end simulation loop**;

			%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 and &ntree = 0 %then %do;
				%do i=1 %to &ntrait;
					%*if ^%scan(&itree, &i) %then %do;
						&&trait&i.._Sim_Mean = &&trait&i.._Sim_Mean / &nsim;
						&&trait&i.._Sim_Range = &&trait&i.._Sim_Max - &&trait&i.._Sim_Min;
						%if %upcase(&NSim) > 1 %then %do;
							&&trait&i.._Sim_StdDev = sqrt((&&trait&i.._Sim_SS / &nsim - &&trait&i.._Sim_Mean * &&trait&i.._Sim_Mean) * (&nsim / (&nsim - 1)));
						%end;
						%else %do;
							&&trait&i.._Sim_StdDev = sqrt((&&trait&i.._Sim_SS / &nsim - &&trait&i.._Sim_Mean * &&trait&i.._Sim_Mean) * (&nsim / (&nsim)));				
						%end;
					%*end;
				%end;
				%if %upcase(&ComputeSelIndex) = YES %then %do;
						&SelIndex._Sim_Mean = &SelIndex._Sim_Mean / &nsim;
						&SelIndex._Sim_Range = &SelIndex._Sim_Max - &SelIndex._Sim_Min;
						%if %upcase(&NSim) > 1 %then %do;
							&SelIndex._Sim_StdDev = sqrt((&SelIndex._Sim_SS / &nsim - &SelIndex._Sim_Mean * &SelIndex._Sim_Mean) * (&nsim / (&nsim - 1)));
						%end;
						%else %do;
							&SelIndex._Sim_StdDev = sqrt((&SelIndex._Sim_SS / &nsim - &SelIndex._Sim_Mean * &SelIndex._Sim_Mean) * (&nsim / (&nsim)));				
						%end;
				%end;
			%end;

			keep m_id p_id 

				%do i=1 %to &ntrait;
					%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 and &ntree = 0 %then %do;
						&&trait&i.._Sim_Mean
						&&trait&i.._Sim_Max
						&&trait&i.._Sim_Min
						&&trait&i.._Sim_Range
						&&trait&i.._Sim_StdDev
						%if %upcase(&ComputeSelIndex) = YES %then %do;
							&SelIndex._Sim_Mean
							&SelIndex._Sim_Max
							&SelIndex._Sim_Min
							&SelIndex._Sim_Range
							&SelIndex._Sim_StdDev
						%end;					
					%end;
					%else %if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
						%if ^%scan(&itree, &i) %then %do;
							&&trait2&i
						%end;						
					%end;
				%end;	

				%if %upcase(&ComputeSelIndex) = YES and (&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &ntree = 0 %then %do;
					&SelIndex2 
				%end;				

				%if %length(&TesterVar) %then m_tester_indicator p_tester_indicator tester_id nontester_id;
				%if %length(&ParentalVar) %then _orig_m_lines_ _orig_p_lines_;
				%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;
					sim &SexVar
					%do i=1 %to &nm; 
						%sysfunc(varname(&_dsix_,&i)) 
					%end;
				%end;
				%if &NBack > 1 %then %do;
					_backgeneration_
					_orig_p_lines_
					_orig_m_lines_
					_orig_m_ID_
					_orig_p_ID_
				%end;
				%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then %do;
					_selfgeneration_
					_orig_m_ID_
					_orig_p_ID_
				%end;
				%else %if &NnonSelf > 1 %then %do;
					_generation_
					_orig_m_ID_
					_orig_p_ID_
				%end;				
			;
		run;

		%if &syserr > 4 %then %goto exit;		
		/*data OutLib.c_simdat0_&gen; set simdat; run;*/

		proc datasets library=work nolist;
			delete crossdat;
		quit;

		%if &ntree %then %do;
			%do k=1 %to &ntree;
				proc geneselect inmodel=ScoreLib.&&ScoreFile&k;;
					score data=simdat (keep= m_id p_id sim &SexVar %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end;) 
						role=SCORE out=ts2o (drop= _WARN_ &SexVar %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end;) 
						outfit=_null_;
				run;
				quit;

				/*data OutLib.c_ts2o&k; set ts2o; run;*/				

				%if &&DepVarType3&k ^= CONTINUOUS %then %do; 
					%put NumbLevels3&k= &&NumbLevels3&k;

					%if &&NumbLevels3&k = 2 %then %do;
						* binary trait*;
						%if %length(&&trait_pt&k) >= 10 %then %do;
							%let DepVarTrunc = %substr(&&trait_pt&k,1,7);
						%end;
						%else %do;
							%let DepVarTrunc = &&trait_pt&k;
						%end;
					%end;
					%else %do;
						* nominal trait*;
						%if %length(&&trait_pt&k) > 8 %then %do;
							%let DepVarTrunc = %substr(&&trait_pt&k,1,8);
						%end;
						%else %do;
							%let DepVarTrunc = &&trait_pt&k;
						%end;
					%end;					
				%end;
				%else %do;
					%if %length(&&trait_pt&k) > 30 %then %do;
						%let DepVarTrunc = %substr(&&trait_pt&k,1,30);
					%end;
					%else %do;
						%let DepVarTrunc = &&trait_pt&k;
					%end;
				%end;

				%if &&DepVarType3&k ^= CONTINUOUS %then %do;
					* binary or nominal trait;
					* construct _INTO_ and Predicted_Class variable;

					%if &&NumbLevels3&k = 2 %then %do;
						* binary trait*;
						%let into = P_&DepVarTrunc.&&PTvlevel&k.1;	 
					%end;
					%else %do;
						* nominal trait*;
						%let into = U_&DepVarTrunc;				
					%end;

					data ts2o;
						set ts2o;
						/*
						%if &into ^= _INTO_ %then %do;
							_INTO_ = &into;
						%end;
						%if &_dtype_ = C %then %do;
							Predicted_Class = &into;
						%end;
						%else %do;
							Predicted_Class = 0 + &into;
						%end;
						*drop &into;
						*/
						&&trait2_pt&k = &into;
					run;
				%end;
				%else %do;
					* continuous trait;

					%let pred = P_&DepVarTrunc;

					data ts2o;
						set ts2o;
						*Predicted_Value = &pred;
						&&trait2_pt&k = &pred;
					run;
				%end;
				/*data OutLib.c_ts2o&k; set ts2o; run;*/

				data simdat;
					merge simdat ts2o;
					%if %upcase(&ComputeSelIndex) = YES %then %do;									
						&SelIndex2= &SelIndexFormula; 									
					%end;
				run;
			%end;
		%end;

		/*data OutLib.c_simdat1_&gen; set simdat; run;*/

		%if &gen= 1 %then %do;
			%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 or &ntree > 0 %then %do;

				%if &NBack = 1 and &NSelf = 1 and &NnonSelf = 1 and &ntree > 0 %then %do;
					%let SortedBy= m_id p_id;
				%end;
				%else %do;
					%let SortedBy= _orig_m_ID_ _orig_p_ID_;
				%end;

				proc means data=simdat mean max min range n stddev noprint;
					var 
						%do i=1 %to &ntrait; 
							&&trait2&i
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %do;
							&SelIndex2
						%end;
					;
					*by _orig_m_ID_ _orig_p_ID_ NOTSORTED;
					by &SortedBy NOTSORTED; 
					id m_id p_id;
					output out=simdatst(drop= _type_ _freq_) 
						/*
						%do i=1 %to &ntrait;
							mean= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
							min= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
							max= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
							range= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
							stddev= %substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
							n= %substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;
						*/

						mean= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Mean;
						min= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Min;
						max= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Max;
						range= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Range;
						stddev= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_stddev;
						n= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_N;

					;
				run;

				%let summdat= simdatst;			
			%end;
			%else %do;
				%let summdat= simdat;
			%end;
			/*data OutLib.c_summdat_&gen; set &summdat; run;*/

			%if %upcase(&SimRil) ^= YES and (&ntrait ^= &ntree) %then %do;

				proc datasets lib=OutLib memtype=data noprint;
					*modify &CE_OutDataName(sortedby= p_id m_id);
					modify &CE_OutDataName(sortedby= m_id p_id);
				run;		

				data OutData_tmp; 
					set OutLib.&CE_OutDataName; 
				run;

				proc sql UNDO_POLICY= NONE;
					create table OutLib.&CE_OutDataName.(&dsoptions 
						drop= m_id_ p_id_ 
							%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
								/*
								sim &SexVar
								%do i=1 %to &nm; 
									%sysfunc(varname(&_dsix_,&i)) 
								%end;
								*/
								%*if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
									/*
									%if &NBack > 1 %then %do;
										_orig_m_lines_ _orig_p_lines_ 
									%end;
									*/
									_orig_m_ID_ _orig_p_ID_ 
								%*end;
							%end;

						) as
					select *
					from 
					OutData_tmp as a, &summdat(rename=(m_id= m_id_ p_id= p_id_)) as b
					where a.m_id= b.m_id_ and a.p_id= b.p_id_  
					/*order by p_id, m_id*/
					order by %if %length(&TesterVar) %then %do; tester_id, nontester_id, %end; m_id, p_id
					;
				quit;
				/*data OutLib.c_OutDataName1_&gen; set OutLib.&CE_OutDataName; run;*/

				proc datasets nolist;
					delete &summdat outdata_tmp;
				quit;
			%end;
		%end;

		*creating summary stats for the current generation and preparing progenies data for the next generation of crosses*;
		%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;	
			%if &NSelf > 1 and %upcase(&SimRilSSD) = YES and &gen > 1 and &NSimRIL > 1 %then %do;
				*selecting one progeny (producing RILs via the single seed desdent method) from each cross to selfcross in the next generation*;

				proc surveyselect data=simdat
					method=srs n=1 out=simdat seed=%sysevalf(&Seed + &gen) noprint;
					strata m_id p_id;
				run;				

				/*data OutLib.c_simdat3_&gen; set simdat; run;*/
			%end;

			%if &NSelf > 1 and %upcase(&SimRilSSD) = YES and &gen= 1 %then %do;
				*reset number of simulated progenies when producing RILs via the single seed descent method*;				
				%let NSim= &NSimRIL;
			%end;		

			%if (%upcase(&SimRilSSD) ^= YES) or (%upcase(&SimRilSSD) = YES and &gen=1) or (%upcase(&SimRilSSD) = YES and &gen= &NSelf) %then %do;
				**summary statistics across simulated replicates for each line within cross by generation**;		
				proc sort data=simdat out=simdat;
					*by _orig_m_ID_ _orig_p_ID_ p_id m_id;
					by _orig_m_ID_ _orig_p_ID_ m_id p_id;
				run;

				proc means data=simdat mean max min range n stddev noprint;
					var 
						%do i=1 %to &ntrait; 
							&&trait2&i
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %do;
							&SelIndex2
						%end;
					;
					by _orig_m_ID_ _orig_p_ID_ m_id p_id NOTSORTED;
					output out=SimStByLines&gen(drop= _type_ _freq_) 
						/*
						%do i=1 %to &ntrait;
							mean= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
							min= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
							max= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
							range= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
							stddev= %substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
							n= %substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;	
						*/
						mean= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Mean;
						min= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Min;
						max= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Max;
						range= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Range;
						stddev= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_stddev;
						n= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_N;
					;
				run;	
				/*data OutLib.c_SimStByLines0_&gen; set SimStByLines&gen; run;*/

				data MergeColsByLines;
					set simdat;
					keep 
						%if &NBack > 1 %then _backgeneration_ _orig_m_lines_ _orig_p_lines_ /*m_parental p_parental*/; 
						%else %if &NSelf > 1 %then _selfgeneration_; 
						%else %if &NnonSelf > 1 %then _generation_; 
						_orig_m_ID_ _orig_p_ID_ m_id p_id;
					by _orig_m_ID_ _orig_p_ID_ m_id p_id;
					if first.p_id;
				run;
				/*data OutLib.c_MergeColsByLines&gen; set MergeColsByLines; run;*/ 

				data SimStByLines&gen;
					merge MergeColsByLines SimStByLines&gen;
					by _orig_m_ID_ _orig_p_ID_ m_id p_id;
				run;
				/*data OutLib.c_SimStByLines1_&gen; set SimStByLines&gen; run;*/

				data SimStByLines_All; 
					set SimStByLines_All SimStByLines&gen;
				run;

				**summary statistics across simulated replicates for each initial cross by generation**;
				proc means data=simdat mean max min range n stddev noprint;
					var 
						%do i=1 %to &ntrait; 
							&&trait2&i
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %do;
							&SelIndex2
						%end;						
					;
					by _orig_m_ID_ _orig_p_ID_ NOTSORTED;
					output out=SimStByCross&gen(drop= _type_ _freq_) 
						/*
						%do i=1 %to &ntrait;
							mean= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
							min= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
							max= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
							range= %substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
							stddev= %substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
							n= %substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;
						*/

						mean= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Mean;						
						min=
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Min
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Min;
						max= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Max
						%end;	
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Max;
						range= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_Range
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_Range;
						stddev= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_StdDev
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_stddev;
						n= 
						%do i=1 %to &ntrait;
							%substr(&&trait&i.._Sim, 1, &trimvarlen)_N
						%end;
						%if %upcase(&ComputeSelIndex)= YES %then %substr(&SelIndex._Sim, 1, &trimvarlen)_N;
					;
				run;
				/*data OutLib.c_SimStByCross0_&gen; set SimStByCross&gen; run;*/

				data MergeColsByCross;
					set simdat;
					keep 
						%if &NBack > 1 %then _backgeneration_ _orig_m_lines_ _orig_p_lines_; 
						%else %if &NSelf > 1 %then _selfgeneration_;
						%else %if &NnonSelf > 1 %then _generation_;
						_orig_m_ID_ _orig_p_ID_;
					by _orig_m_ID_ _orig_p_ID_;
					if first._orig_p_ID_;
				run;
				/*data OutLib.c_MergeColsByCross&gen; set MergeColsByCross; run;*/ 

				data SimStByCross&gen;
					merge MergeColsByCross SimStByCross&gen;
					by _orig_m_ID_ _orig_p_ID_;
				run;
				/*data OutLib.c_SimStByCross1_&gen; set SimStByCross&gen; run;*/

				data SimStByCross_All; 
					set SimStByCross_All SimStByCross&gen;
				run;

				*selecting and preparing progenies data for next generation of crosses*;
				%if &gen < &NGenerations %then %do;
					*selecting progenies to cross in the next generation of crosses*;
					%if %upcase(&OptimizeCrosses) = YES %then %do;

						/*data OutLib.c_simdat_bs_&gen; set simdat; run;*/

						%if %upcase(&OptimizeSelection) = YES %then %do;

							data simdat;
								set simdat;
								if _n_ = 1 then _order_= 1;
								else _order_ = _order_ + 1;
								retain _order_;
							run;						

							%if &gen = 1 %then %do;					
								%let OptVars= ;
								%do i=1 %to &ntrait; 
									%let Sense= %scan(&OptSense, &i, ' '); 
									%if %nrbquote(&Sense) = 1 or %nrbquote(&Sense) = -1 %then %do;
										**  1 = maximize **;
										** -1 = minimize **;
										%let OptVars= &OptVars &&trait2&i;
									%end;
								%end;
							%end;

							proc sort data= simdat out=unique_mp nodupkey;
								by m_id p_id;
							run;

							data _null_;
								set unique_mp end=_eof_;
								call symputx("u_m"||trim(left(_n_)), m_id);
								call symputx("u_p"||trim(left(_n_)), p_id);				
								if _eof_ then call symputx("n_unique", trim(left(_n_)));
							run;
							%put n_unique= &n_unique;
							%put u_m= &u_m1;
							%put u_p= &u_p1;

							data outdata_all;
								delete;
							run;

							%do j= 1 %to &n_unique;
								data indata_tmp;
									set simdat(keep=
											%do i=1 %to &ntrait; 
												&&trait2&i
											%end;
											_order_ m_id p_id
										);
									where m_id= "&&u_m&j" and p_id= "&&u_p&j";
									%let k= 1;
									%do i=1 %to &ntrait;
										%let Sense= %scan(&OptSense, &i, ' ');
										%if %nrbquote(&Sense) = 1 %then %do;
											**  1 = maximize **;
											** Must negate the variable because the Pareto code finds the minimum**;
											x&k= -1 * &&trait2&i;
											%let k= %eval(&k + 1);
										%end;
										%else %if %nrbquote(&Sense) = -1 %then %do;
											** -1 = minimize **;
											x&k= &&trait2&i;
											%let k= %eval(&k + 1);
										%end;
									%end;
									/*
									%let k=1;
									%let _ovar_= %scan(&OptVars, &k, ' ');
									%do %while(%length(&_ovar_));
										%let Sense= %scan(&OptSense, &k, ' '); 
										%if %nrbquote(&Sense) = 1 %then %do;
											**  1 = maximize. Must negate the variable because the Pareto code finds the minimum**;
											x&k= -1 * &_ovar_;
										%end;
										%else %if %nrbquote(&Sense) = -1 %then %do;
											** -1 = minimize **;
											x&k= &_ovar_;
										%end;
										%let k= %eval(&k + 1);
										%let _ovar_= %scan(&OptVars, &k, ' ');
									%end;
									*/
								run;
								/*data OutLib.c_indata_tmp; set indata_tmp; run;*/

								%FindParetoPoints(indata=indata_tmp, outdata=outdata_tmp, outorder= outorder_tmp);

								/*data OutLib.c_outorder_tmp; set outorder_tmp; run;*/
								/*data OutLib.c_outdata_tmp; set outdata_tmp; run;*/

								proc sql UNDO_POLICY= NONE;
									create table outdata_s as
									select *
									from outorder_tmp as a left join outdata_tmp(drop= x:) as b
									on a._sort_order_ = b.pareto_points
									order by _orig_order_;
									;
								quit;

								data outdata_s;
									merge indata_tmp outdata_s;
								run;

								data Outdata_s;
									set outdata_s;
									if not missing(pareto_points) then do;
										%let k=1;
										%let _ovar_= %scan(&OptVars, &k, ' ');
										%do %while(%length(&_ovar_));
											x&k= &_ovar_;
											%let k= %eval(&k + 1);
											%let _ovar_= %scan(&OptVars, &k, ' ');
										%end;
									end;
								run;
								/*data OutLib.c_outdata_s&gen; set outdata_s; run;*/

								data outdata_all;
									set outdata_all outdata_s(keep= _order_ pareto_points);
								run;			
							%end;

							/*data OutLib.c_outdata_all_&gen; set outdata_all; run;*/

							proc sql UNDO_POLICY= NONE;
								create table simdat as
								select *
								from simdat as a, outdata_all as b
								where a._order_= b._order_ and not missing(b.pareto_points)
								order by _order_
								;
							quit;

							/*
							data simdat(where= (not missing(pareto_points)));
								merge simdat outdata_all(in=b);
								by _order_;
							run;
							*/
						%end;
						%else %if %upcase(&ThresholdSelection) = YES or %upcase(&IndexSelection) = YES or %upcase(&PercentSelection) = YES %then %do;

							%let SelWhereStmt = ;
							%if %upcase(&ThresholdSelection) = YES or %upcase(&IndexSelection) = YES %then %do;
								%let _Sep_= ;
								%if %upcase(&ThresholdSelection) = YES %then %do;
									%do i=1 %to &ntrait; 
										%let Sense= %scan(&ThresholdSense, &i, ' ');
										%put Sense= &Sense;
										%if %nrbquote(&Sense) = -1 or %nrbquote(&Sense) = 1 %then %do; 
											%put Sense= &Sense;
											%let Threshold= %scan(&ThresholdValues, &i, ' ');
											%if %nrbquote(&Sense) = -1 %then %let SelWhereStmt = &SelWhereStmt &_Sep_ &&trait2&i %str(<) &Threshold; 
											%else %let SelWhereStmt = &SelWhereStmt &_Sep_ &&trait2&i %str(>) &Threshold; 
											%let _Sep_= and;
										%end;
									%end;
								%end;

								%if %upcase(&IndexSelection) = YES %then %do;
									%let SelWhereStmt= &SelWhereStmt &_Sep_ &SelIndex2 &IndexDirection &IndexCutOff; 
								%end;

								%put SelWhereStmt= &SelWhereStmt; 
								data simdat;
									set simdat;
									where &SelWhereStmt;
								run;

							%end;
							%else %do;

								/*														
								proc sort data=simdat out=simdatsorted;
									by &PercentDirection &PercentTrait;
								run;

								%NObsVars(simdatsorted);
								%let NCutOff= %sysfunc(ceil(%sysevalf(&nobs * &PercentCutOff/100)));								

								%let SelIfStmt= %str(_n_ <= &NCutOff);
								%put SelIfStmt= &SelIfStmt;	

								data simdat;
									set simdat;
									if &SelIfStmt;
								run;
								*/

								proc sort data=simdat(keep= &PercentTrait) out=simdatsorted;
									by &PercentDirection &PercentTrait;
								run;

								%NObsVars(simdatsorted);
								%let NCutOff= %sysfunc(ceil(%sysevalf(&nobs * &PercentCutOff/100)));								

								data _null_;
									set simdatsorted(obs=&NCutOff);
									call symputx("ValueCutOff", &PercentTrait);
								run;
								%put ValueCutOff= &ValueCutOff;								

								%let SelWhereStmt= &PercentTrait &ValueDirection &ValueCutOff;
								%put SelWhereStmt= &SelWhereStmt;
								data simdat;
									set simdat;
									where &SelWhereStmt;
								run;
							%end;

							%NObsVars(simdat);
							%if &nobs = 0 %then %do; 

								%let String1= The specified threshold to select progenies values were too stringent at generation &gen..; 
								%let String2= Therefore, all progenies failed to pass the threshold criteria and the simulation stoped at generation &gen..;								

								%global jsl_after_tabbedreport;
								%let jsl_after_tabbedreport = &clientoutpath.&ProcessName Message for &OutData..jsl;
								%global SummaryScript;			
								%PrepOutFile(SummaryScript,SummaryScriptName,&ProcessName Message for &OutData..jsl);

								 * put JSL statements;
								 data _null_;
								   file "&SummaryScript";
								   put '//!';
								   put "Names Default to Here(1);";
								   put 'wait(0.5);';
								   put 'Here:cewarn=New Window("WARNING running &ProcessName.",showmenu(0),showtoolbars(0),';
								   put 'Vlistbox(';
								   put "   Textbox(""WARNING: &String1 &String2   "", Set Font Size(10), Font Color(5)),";
								   put "   ));";
								   put "if(!IsEmpty(Here:TabWinList),";
								   put "    InsertInto(Here:TabWinList,0);";
								   put "    Here:TabWinList[Here:nwcounter]=Here:cewarn;";
								   put "    Here:nwcounter++; ";		
								   put "	);";
								run;								

								%put WARNING: &String1 &String2;
								%let StopMultGen= 1;
								%goto StopGenLoop;
							%end;							
						%end;

						/*data OutLib.c_simdat_af_&gen; set simdat; run;*/

					%end; /*end if OptimizeCrosses*/

					%if &NBack > 1 %then %do;
						data simdat;
							length m_id p_id $ &line_length;
							set 
								simdat( /*
									drop= *m_parental p_parental*
									*/
									in=a
								)
								backparents(
									drop= ParentalVarNew /*&OrigIDVars*/
									rename= (&OrigParentalVar= _orig_m_lines_ Line_ID= m_id)
									in=b
								)
							;
							if b then do;
								p_id= m_id;
								_orig_p_lines_= _orig_m_lines_;
								_orig_p_ID_= m_id;
								_orig_m_ID_= m_id;
							end;
						run;

						/*data OutLib.c_simdat3_&gen; set simdat; run;*/

					%end; /*end if NBack > 1*/
				%end; /*end if gen < NGenerations*/
			%end;
		%end; /*end if &NGenerations > 1*/
	%end; /*end if Simulate*/
	%StopGenLoop:;
	%let gen= %eval(&gen + 1);		
%end; /**end generations loop**/
%let LastGen= %eval(&gen - 1);
%put LastGen= &LastGen;

%global CEByLines_OutData CEByCross_OutData;
%PrepOutDS(CEByLines_OutData,CEByLines_OutDataName,&OutData._cel);
%PrepOutDS(CEByCross_OutData,CEByCross_OutDataName,&OutData._cec);

%if (&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) %then %do;
	%*global CEByLines_OutData CEByCross_OutData;
	%*PrepOutDS(CEByLines_OutData,CEByLines_OutDataName,&OutData._cel);
	%*PrepOutDS(CEByCross_OutData,CEByCross_OutDataName,&OutData._cec);

	data OutLib.&CEByLines_OutDataName; 
		length _Cross_ $ &line_length;
		set SimStByLines_All;
		_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
	run;

	data OutLib.&CEByCross_OutDataName; 
		length _Cross_
			%if &NBack > 1 %then %do;
				%if &pvt= C %then %do;
					_backgeneration_ _orig_m_lines_ _orig_p_lines_
				%end;
				%else %do;
					_backgeneration_ $ &line_length _orig_m_lines_ _orig_p_lines_ 8
				%end;
			%end;			
			%else %if &NSelf > 1 %then _selfgeneration_;
			%else %if &NnonSelf > 1 %then _generation_;
			_orig_m_ID_ _orig_p_ID_ m_id p_id $ &line_length
		;	
		set SimStByCross_All; 
		%if %upcase(&SimRilSSD) = YES %then %do;
			where _selfgeneration_= "SF_"||trim(left("&NSelf")); 
			_selfgeneration_= "RIL";
		%end;
		/*
		rename 
			_orig_m_ID_= m_id 
			_orig_p_ID_= p_id
		;
		*/
		m_id= _orig_m_ID_; 
		p_id= _orig_p_ID_;
		_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
	run;

	%let IDVars= &OrigIDVars;
	%let ParentalVar= &OrigParentalVar;

	%if &NSelf > 1 or &NnonSelf > 1 %then %do;
		%let MatingType= &OrigMatingType;
	%end;	
%end;
%else %if %upcase(&SimRilFast) = YES %then %do;
	%global CEByCross_OutData;
	%PrepOutDS(CEByCross_OutData,CEByCross_OutDataName,&OutData._cec);	

	/*data OutLib.simdat; set simdat; run;*/

	%if &ntree = 0 %then %do;
		data OutLib.&CEByCross_OutDataName; 
			length _Cross_ $ &line_length;
			*length _selfgeneration_ _orig_m_ID_ _orig_p_ID_ m_id p_id $ &line_length;	
			set simdat; 
			_selfgeneration_= "RIL";
			*_orig_m_ID_=m_id; 
			*_orig_p_ID_=p_id;
			_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
		run;
	%end;
	%else %do;
		data OutLib.&CEByCross_OutDataName; 
			*length _Cross_ _selfgeneration_ $ &line_length;
			length _Cross_ _selfgeneration_ _orig_m_ID_ _orig_p_ID_ $ &line_length;	
			set &summdat; 
			_selfgeneration_= "RIL";
			_orig_m_ID_=m_id; 
			_orig_p_ID_=p_id;
			_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
		run;	
	%end;
%end;
%else %if &ntrait = &ntree %then %do;
	data OutLib.&CE_OutDataName;
		length _Cross_ _orig_m_ID_ _orig_p_ID_ $ &line_length;	
		set &summdat; 
		_orig_m_ID_=m_id; 
		_orig_p_ID_=p_id;
		_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
	run;
%end;

%if (&ntrait ^= &ntree) %then %do;
	data OutLib.&CE_OutDataName;
		length _Cross_ $ &line_length;
		set OutLib.&CE_OutDataName;
		_Cross_= trim(left(m_id))||"_x_"||trim(left(p_id));
	run;
%end;

%if %length(&AnnoWhereStmt) %then %do;
	*replace double string with single string to avoind error when passing variable to drill down process*;
	%*let AnnoWhereStmt= %nrbquote(&AnnoWhereStmt);
	%*put &AnnoWhereStmt;
	%let AnnoWhereStmt= %sysfunc(tranwrd(&AnnoWhereStmt, %str(%"), %str(%')));
	%*put &AnnoWhereStmt;
%end;

%let nblocks= 1;
%if (%length(&OrigMarkerVars) > 32000) %then %do;
	%let TmpVars= &OrigMarkerVars;
	%let i=1;
	%let ub= 1000;
	%let flagstop= 1;
	%do %while(&flagstop ^= 0);
		%let sc&i= %scan(&TmpVars, &ub," ");
		%put sc&i= &&sc&i;
		%let fd&i= %sysfunc(find(&TmpVars, &&sc&i));
		%put fd&i= &&fd&i;
		%let sb&i= %substr(&TmpVars, 1, %sysevalf(&&fd&i - 1));
		%put sb&i= &&sb&i;
		%let TmpVars= %sysfunc(substr(&TmpVars, %sysevalf(&&fd&i - 1)));
		%put TmpVars= &TmpVars;
		%let flagstop= %length(%scan(&TmpVars, &ub, " "));
		%put flagstop= &flagstop;
		%let i= %sysevalf(&i + 1);
	%end;
	%if %length(&TmpVars) %then %do;
		%let sb&i= &TmpVars;
		%put sb&i= &&sb&i;
		%let nblocks= &i;
	%end;
	%else %do;
		%let nblocks= %sysevalf(&i - 1);
	%end;
%end;
%put nblocks= &nblocks;


* create JSL code;
data _null_;
	file "&CrossEvaluationScript" lrecl=32767;
	put '//!';
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";

	%if &ntree = 0 %then %do;
		put "Here:mult = Function({},{default local}," ;
		put "If(IsEmpty(Here:cedata),";	
		put "   Here:cedata = open(""&ClientOutPath.&CE_OutDataName..sas7bdat"",invisible);";
		put "   Column(""m_id"") << label(1);";
		put "   Column(""p_id"") << label(1);";
		put "   ,"; 
		put "   Current Data Table(Here:cedata);";
		put ");";
		put "HListBox( ";
		%if %length(&TesterVar) or %length(&ParentalVar) %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";	
			%if %length(&TesterVar) %then %do;
				put "			columns(";
				%do i=1 %to &ntrait; 
					put "         		:&&trait&i.._Mean,";
				%end;		
				put "				:tester_id";
				put "),";
			%end;
			%else %do;
				put "			columns(";
				%do i=1 %to &ntrait; 
					put "         		:&&trait&i.._Mean,";
				%end;
				put "				:_orig_p_lines_";
				put "),";
			%end;
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		%else %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";			
			put "			columns("; 
			%do i=1 %to &ntrait; 
				put "         		:&&trait&i.._Mean,";
			%end;
			put "				:p_ID, :m_ID),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		put "   VListBox( ";
		put "      Multivariate(";
		put "         Y( ";
		%do i=1 %to &ntrait; 
			put "         Column(""&&trait&i.._Mean""),";
			put "         Column(""&&trait&i.._StdDev""),";
			put "         Column(""&&trait&i.._Max""),";
			put "         Column(""&&trait&i.._Min""),";
			put "         Column(""&&trait&i.._Range""),";
		%end;
		put "         ),";
		put "         Scatterplot Matrix(Density Ellipses(0), Horizontal(1)),";
		%if &LastGen > 1 %then %do;
			%if &NBack > 1 %then %do;
				put "         By( :_backgeneration_),";
			%end;
			%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then %do;
				put "         By( :_selfgeneration_),";
			%end;
			%else %if &NnonSelf > 1 %then %do;
				put "         By( :_generation_),";
			%end;	
		%end;
		put "      )";
		put "   )";
		put "));";
		put "InsertInto(DataList,{{""Here:cedata""}});";
		put "InsertInto(TabButtonList, ""First Generation Expected Trait Statistics"");";
		/*put "InsertInto(TabButtonList, ""Expected Trait Statistics"");";*/
		put "InsertInto(TabCodeList,expr(Here:mult()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		%if &Simulate = Yes %then %do;
			put "InsertInto(TabOpenList,0);";
		%end;
		%else %do;
			put "InsertInto(TabOpenList,1);";
		%end;
		put " ";
	%end;

	%if &Simulate = Yes %then %do;
		put "Here:mult2 = Function({},{default local}," ;
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "If(IsEmpty(Here:cedata_bycross),";
			put "   Here:cedata_bycross = open(""&ClientOutPath.&CEByCross_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata_bycross);";
			put ");";
		%end;
		%else %do;
			put "If(IsEmpty(Here:cedata),";
			put "   Here:cedata = open(""&ClientOutPath.&CE_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata);";
			put ");";
		%end;
		put "HListBox( ";
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			put "			columns( ";
			%do i=1 %to &ntraitix; 
				put "         		:&&trait&i.._Sim_Mean,";
			%end;
			put "				:_orig_p_ID_, :_orig_m_ID_";
			put "	),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		%else %if %length(&TesterVar) or %length(&ParentalVar) %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			%if %length(&TesterVar) %then %do;
				put "			columns( ";
				%do i=1 %to &ntraitix; 
					put "         		:&&trait&i.._Sim_Mean,";
				%end;				
				put "				:tester_id";
				put "),";
			%end;
			%else %do;
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "         		:&&trait&i.._Sim_Mean,";
				%end;
				put "				:_orig_p_lines_";
				put "),";
			%end;
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		%else %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			put "				columns(";
			%do i=1 %to &ntraitix; 
				put " 	        		:&&trait&i.._Sim_Mean,";
			%end;
			put "					:p_ID, :m_ID";
			put "),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";			
		%end;
		put "   VListBox( ";
		put "      Multivariate(";
		put "         Y( ";
		%do i=1 %to &ntraitix; 
			put "         Column(""&&trait&i.._Sim_Mean""),";
			put "         Column(""&&trait&i.._Sim_StdDev""),";
			put "         Column(""&&trait&i.._Sim_Max""),";
			put "         Column(""&&trait&i.._Sim_Min""),";
			put "         Column(""&&trait&i.._Sim_Range""),";
		%end;
		put "         ),";
		put "         Scatterplot Matrix(Density Ellipses(0), Horizontal(1)),";
		%if &LastGen > 1 %then %do;
			%if &NBack > 1 %then %do;
				put "         By( :_backgeneration_),";
			%end;
			%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then %do;
				put "         By( :_selfgeneration_),";
			%end;
			%else %if &NnonSelf > 1 %then %do;
				put "         By( :_generation_),";
			%end;
		%end;
		put "      )";
		put "   )";
		put "));";
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "InsertInto(DataList,{{""Here:cedata_bycross""}});";
		%end;
		%else %do;
			put "InsertInto(DataList,{{""Here:cedata""}});";
		%end;
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			%if %upcase(&SimRilSSD) = YES or %upcase(&SimRilFast) = YES %then %do;
				put "InsertInto(TabButtonList, ""RILs Simulated Trait Statistics"");";	
			%end;
			%else %do;
				put "InsertInto(TabButtonList, ""Multi-Generations Simulated Trait Statistics"");";
			%end;
		%end;
		%else %do;
			put "InsertInto(TabButtonList, ""Simulated Trait Statistics"");";
		%end;
		put "InsertInto(TabCodeList,expr(Here:mult2()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		put " ";

		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;

			%goto skipind;

			put "Here:Fn_bylines = Function({},{default local}," ;
			put "	If(IsEmpty(Here:cedata_bylines),";	
			put "	   Here:cedata_bylines = open(""&ClientOutPath.&CEByLines_OutDataName..sas7bdat"",invisible);";
			put "	   ,"; 
			put "	   Current Data Table(Here:cedata_bylines);";
			put "	);";	
			put "	HListBox( ";
			put "	   VListBox( ";
			/*
			put "		gb_stbylines= Graph Builder(";
			put "			Size( 589, 525 ),";
			put "			Show Control Panel( 0 ),";
			put "			Variables(";
			put "				X( :_orig_m_ID_ ),";
			put "				X( :_orig_p_ID_, Position( 1 ) ),";
			%if &NBack > 1 %then %do;
				put "				X( :_backgeneration_, Position( 1 ) ),";
				put "				X( :p_id, Position( 1 ) ),";
			%end;
			%else %if &NSelf > 1 %then %do;
				put "				X( :_selfgeneration_, Position( 1 ) ),";
			%end;
			%do i=1 %to &ntraitix; 
				put "        		 Y( :&&trait&i.._Sim_Mean),";
			%end;
			put "				Color( :_orig_p_ID_ ),";
			put "			),";
			%do i=1 %to &ntraitix; 
				put "			Elements(";
				put "				Position( 1, &i ),";
				put "				Points( X( 1 ), X( 2 ), X( 3 ), X( 4 ), Y, Legend( &i ) )";
				put "			),";
			%end;
			put "			SendToReport(";
			put "			Dispatch(";
			put "				{},";
			put "				""Graph Builder"",";
			put "				OutlineBox,";
			put "				{Set Title( ""Trait= %upcase(&&trait&i.._Sim_Mean)"" )}";
			put "			),";
			put "			Dispatch(";
			put "				{},";
			put "				""graph title"",";
			put "				TextEditBox,";
			put "				{Set Text( ""%upcase(&&trait&i.._Sim_Mean) by lines through multi-generations"" )}";
			put "			)";
			put "		)";
			put "		);";
			*/
			%do i=1 %to &ntraitix;
				put "		gb_stbylines&i= Graph Builder(";
				put "			Size( 589, 525 ),";
				put "			Show Control Panel( 0 ),";
				put "			Variables(";
				put "				X( :_orig_m_ID_ ),";
				put "				X( :_orig_p_ID_, Position( 1 ) ),";
				%if (&NBack > 1 and &LastGen > 1) %then %do;
					put "				X( :_backgeneration_, Position( 1 ) ),";
					put "				X( :p_id, Position( 1 ) ),";
				%end;
				%else %if (&NSelf > 1 and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
					put "				X( :_selfgeneration_, Position( 1 ) ),";
				%end;
				put "        		 	Y( :&&trait&i.._Sim_Mean),";
				put "				Color( :_cross_ ),";
				put "				Size( :&&trait&i.._Sim_StdDev ),";
				put "			),"; 
				put "			Elements(";
				put "				Position( 1, 1 ),";
				put "				Points( X( 1 ), X( 2 ), X( 3 ), X( 4 ), Y, Legend( &i ) )";
				put "			),";
				put "			SendToReport(";
				put "			Dispatch(";
				put "				{},";
				put "				""Graph Builder"",";
				put "				OutlineBox,";
				put "				{Set Title( ""Trait= %upcase(&&trait&i.._Sim_Mean)"" )}";
				put "			),";
				put "			Dispatch(";
				put "				{},";
				put "				""graph title"",";
				put "				TextEditBox,";
				put "				{Set Text( ""%upcase(&&trait&i.._Sim_Mean) by individual lines through multi-generations"" )}";
				put "			)";
				put "		),";
				put "	),";			
			%end;
			put "	);";
			put "	);";
			put ");";
			put "InsertInto(DataList,{{""Here:cedata_bylines""}});";
			%if %upcase(&SimRilSSD) = YES or %upcase(&SimRilFast) = YES %then %do;
				put "InsertInto(TabButtonList, ""RILs Simulated Trait Means By Individual Lines"");";	
			%end;
			%else %do;
				put "InsertInto(TabButtonList, ""Multi-Generations Simulated Trait Means By Individual Lines"");";
			%end;	
			put "InsertInto(TabCodeList,expr(Here:Fn_bylines()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
			put " ";

			%skipind:;

			put "Here:Fn_bycross = Function({},{default local}," ;
			put "If(IsEmpty(Here:cedata_bycross),";
			put "   Here:cedata_bycross = open(""&ClientOutPath.&CEByCross_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata_bycross);";
			put ");";	
			put "	HListBox( ";
			%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
				put "		Current Data Table() << Data Filter(";
				put "			Location( {120, 18} ),";
				put "			Add Filter(";
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "         		:&&trait&i.._Sim_Mean,";
				%end;
				put "				:_orig_p_ID_, :_orig_m_ID_";
				put "),";
				put "			),";
				/*put "			Animation(),";*/
				put "		),";	
			%end;
			put "	   VListBox( ";
			/*
			put "		gb_stbylines= Graph Builder(";
			put "			Size( 589, 525 ),";
			put "			Show Control Panel( 0 ),";
			put "			Variables(";
			put "				X( :m_id ),";
			put "				X( :p_id, Position( 1 ) ),";
			%if &LastGen > 1 %then %do;
				%if &NBack > 1 %then %do;			
					put "				X( :_backgeneration_, Position( 1 ) ),";
				%end;
				%else %if &NSelf > 1 %then %do;
					put "				X( :_selfgeneration_, Position( 1 ) ),";
				%end;
				%else %if &NnonSelf > 1 %then %do;
					put "				X( :_generation_, Position( 1 ) ),";
				%end;
				%do i=1 %to &ntraitix; 
					put "        		 Y( :&&trait&i.._Sim_Mean),";
				%end;
			%end;
			put "				Color( :p_id ),";
			put "			),";
			%do i=1 %to &ntraitix; 
				put "			Elements(";
				put "				Position( 1, &i ),";
				put "				Points( X( 1 ), X( 2 ), X( 3 ), X( 4 ), Y, Legend( &i ) )";
				put "			),";
			%end;
				put "			SendToReport(";
			%do i=1 %to &ntraitix; 
				put "			Dispatch(";
				put "				{},";
				put "				""Graph Builder"",";
				put "				OutlineBox,";
				put "				{Set Title( ""Trait= %upcase(&&trait&i.._Sim_Mean)"" )}";
				put "			),";
				put "			Dispatch(";
				put "				{},";
				put "				""graph title"",";
				put "				TextEditBox,";
				put "				{Set Text( ""%upcase(&&trait&i.._Sim_Mean) by lines through multi-generations"" )}";
				put "			)";
			%end;
			put "		)";
			put "		);";			
			*/
			%do i=1 %to &ntraitix;
				put "		gb_stbycross&i= Graph Builder(";
				put "			Size( 589, 525 ),";
				put "			Show Control Panel( 0 ),";
				put "			Variables(";
				put "				X( :m_id),";
				put "				X( :p_id, Position( 1 ) ),";
				%if &NBack > 1 and &LastGen > 1 %then %do;
					put "				X( :_backgeneration_, Position( 1 ) ),";
				%end;
				%else %if (&NSelf > 1 and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
					put "				X( :_selfgeneration_, Position( 1 ) ),";
				%end;
				%else %if &NnonSelf > 1 and &LastGen > 1 %then %do;
					put "				X( :_generation_, Position( 1 ) ),";
				%end;
				put "        		 	Y( :&&trait&i.._Sim_Mean),";
				put "				Color( :_cross_ ),";
				put "				Size( :&&trait&i.._Sim_StdDev ),";
				put "			),"; 
				put "			Elements(";
				put "				Position( 1, 1 ),";
				put "				Points( X( 1 ), X( 2 ), X( 3 ), X( 4 ), Y, Legend( &i ) )";
				put "			),";
				put "			SendToReport(";
				put "			Dispatch(";
				put "				{},";
				put "				""Graph Builder"",";
				put "				OutlineBox,";
				put "				{Set Title( ""Trait= %upcase(&&trait&i.._Sim_Mean)"" )}";
				put "			),";
				put "			Dispatch(";
				put "				{},";
				put "				""graph title"",";
				put "				TextEditBox,";
				put "				{Set Text( ""%upcase(&&trait&i.._Sim_Mean) by generations"" )}";
				put "			)";
				put "		),";
				put "	),";			
			%end;
			put "	);";
			put "	);";
			put ");";
			put "InsertInto(DataList,{{""Here:cedata_bycross""}});";
			/*
			%if %upcase(&SimRilSSD) = YES or %upcase(&SimRilFast) = YES %then %do;
				put "InsertInto(TabButtonList, ""RILs Simulated Trait Means"");";
			%end;
			%else %do;
				put "InsertInto(TabButtonList, ""Multi-Generations Simulated Trait Means"");";			
			%end;
			*/
			put "InsertInto(TabButtonList, ""Simulated Trait Means"");";
			put "InsertInto(TabCodeList,expr(Here:Fn_bycross()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
			put " ";			
		%end;

		********Progeny Trait Means Heat Map*******;
		put "Here:meansheatmap = Function({},{default local}," ;
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "If(IsEmpty(Here:cedata_bycross),";
			put "   Here:cedata_bycross = open(""&ClientOutPath.&CEByCross_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata_bycross);";
			put ");";
		%end;
		%else %do;
			put "If(IsEmpty(Here:cedata),";		
			put "   Here:cedata = open(""&ClientOutPath.&CE_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata);";
			put ");";				
		%end;
		put "	HListBox( ";
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			put "				columns(";
			%do i=1 %to &ntraitix; 
				put "         				:%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean,";
			%end;
			put "					:_orig_p_ID_, :_orig_m_ID_";
			put "),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		%else %if %length(&TesterVar) or %length(&ParentalVar) %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			%if %length(&TesterVar) %then %do;
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "  				:%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean,";
				%end;
				put "				:tester_id";
				put "),";
			%end;
			%else %do;
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "  				:%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean,";
				%end;
				put "				:_orig_p_lines_";
				put "),";
			%end;
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		%end;
		%else %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";
			put "			columns(";
			%do i=1 %to &ntraitix; 
				put "  				:%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean,";
			%end;			
			put "				:p_ID, :m_ID";
			put "),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";			
		%end;
		put "	   VListBox( ";
		%do i=1 %to &ntraitix;
			put " Here:mhlb1&i = HListBox(";
			put " Here:molb1&i = OutLineBox("""",";
			/*start: By Chris Kirchberg and Luciano Silva, May 24th, 2017 */
			put " mgb_heat1&i= Graph Builder(";
			put "	Show Control Panel( 0 ),";
			put "	Variables(";
			put "		X(";
			put "			:_Cross_,";
			put "			Order By( Column(""%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean""), Ascending, Order Statistic( ""Mean"" ) )";
			put "		),";
			put "		Y( Column(""%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean"") ),";
			put "		Color(:_Cross_)";
			put "	),";
			%if &NBack > 1 and &LastGen > 1 %then %do;
				put "	Where( :_backgeneration_ == ""BC_1"" ),";
			%end;
			%else %if &NSelf > 1 and %upcase(&SimRilSSD) ^= YES and &LastGen > 1 %then %do;
				put "	Where( :_selfgeneration_ == ""F_1"" ),";
			%end;
			%else %if &NnonSelf > 1 and &LastGen > 1 %then %do;
				put "	Where( :_generation_ == ""F_1"" ),";
			%end;			
			put "	Elements( Points( X, Y, Legend( 4 ) ) ),";
			put "	SendToReport(";
			put "		Dispatch(";
			put "			{},";
			put "			""Graph Builder"",";
			put "			OutlineBox,";
			%if &NBack > 1 and &LastGen > 1 %then %do;
				put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean / Generation= BC_1"" )}";
			%end;
			%else %if ((&NSelf > 1 and %upcase(&SimRilSSD) ^= YES) or &NnonSelf > 1) and &LastGen > 1 %then %do;
				put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean / Generation= F_1"" )}";
			%end;
			%else %do;
				put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean"" )}";
			%end;
			put "		),";
			put "		Dispatch(";
			put "			{},";
			put "			""graph title"",";
			put "			TextEditBox,";
			put "			{Set Text( ""Cross ID ordered by %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean (Ascending)"" )}";
			put "		),";
			put "		Dispatch(";
			put "			{},";
			put "			""400"",";
			put "			ScaleBox,";
			put "			{Legend Model(";
			put "				4,";
			put "				Properties(";
			put "					0,";
			put "					{gradient(";
			put "						{Color Theme( ""Spectral"" ), Scale Values( [0 .] ),";
			put "						Range Type( ""Exact Data Range"" )}";
			put "					)}";
			put "				)";
			put "			)}";
			put "		)";
			put "	)";
			put "),";
			put "),";
			%if (&NBack > 1 or (&NSelf > 1 and %upcase(&SimRilSSD) ^= YES) or &NnonSelf > 1) and &LastGen > 1 %then %do;
				put "try(Here:molb1&i[Text Box( 1 )] << delete;);";
			%end;
			put "),";		
			/*end: By Chris Kirchberg and Luciano Silva, May 24th, 2017 */
		%end;
		put "	);";
		put "	);";
		put ");";
		put "Here:meansheatmap_append = Function({},{default local},";
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "If(IsEmpty(Here:cedata_bycross),";
			put "   Here:cedata_bycross = open(""&ClientOutPath.&CEByCross_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata_bycross);";
			put ");";
		%end;
		%else %do;
			put "If(IsEmpty(Here:cedata),";		
			put "   Here:cedata = open(""&ClientOutPath.&CE_OutDataName..sas7bdat"",invisible);";
			put "   Column(""m_id"") << label(1);";
			put "   Column(""p_id"") << label(1);";
			put "   ,"; 
			put "   Current Data Table(Here:cedata);";
			put ");";				
		%end;
		%if (&NBack > 1 or (&NSelf > 1 and %upcase(&SimRilSSD) ^= YES) or &NnonSelf > 1) and &LastGen > 1 %then %do;
			%do j=2 %to &NGenerations;
				%do i=1 %to &ntraitix;
					put "		Here:mhlb&j.&i= HListBox(";
					put " 		Here:molb&j.&i = OutLineBox("""",";
					put "		mgb_heat&j.&i= Graph Builder(";
					put "			Show Control Panel( 0 ),";
					put "			Variables(";
					put "				X(";
					put "				:_Cross_,";
					put "				Order By( Column(""%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean""), Ascending, Order Statistic( ""Mean"" ) )";
					put "			),";
					put "			Y( Column(""%substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean"") ),";
					put "			Color(:_Cross_)";
					put "		),";
					%if &NBack > 1 %then %do;
						put "	Where( :_backgeneration_ == ""BC_&j"" ),";
					%end;
					%else %if &NSelf > 1 %then %do;
						put "	Where( :_selfgeneration_ == ""SF_&j"" ),";
					%end;
					%else %if &NnonSelf > 1 %then %do;
						put "	Where( :_generation_ == ""F_&j"" ),";
					%end;
					put "		Elements( Points( X, Y, Legend( 4 ) ) ),";
					put "		SendToReport(";
					put "			Dispatch(";
					put "				{},";
					put "				""Graph Builder"",";
					put "				OutlineBox,";
					%if &NBack > 1 %then %do;
						put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean / Generation= BC_&j"" )}";
					%end;
					%else %if &NSelf > 1 %then %do;
						put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean / Generation= SF_&j"" )}";
					%end;
					%else %if &NnonSelf > 1 %then %do;
						put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean / Generation= F_&j"" )}";
					%end;
					%else %do;
						put "			{Set Title( ""Graph Builder = %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean"" )}";
					%end;
					put "			),";
					put "			Dispatch(";
					put "				{},";
					put "				""graph title"",";
					put "				TextEditBox,";
					put "				{Set Text( ""Cross ID ordered by %substr(&&trait&i.._Sim, 1, &trimvarlen)_Mean (Ascending)"" )}";
					put "			),";
					put "			Dispatch(";
					put "				{},";
					put "				""400"",";
					put "				ScaleBox,";
					put "				{Legend Model(";
					put "					4,";
					put "					Properties(";
					put "						0,";
					put "						{gradient(";
					put "							{Color Theme( ""Spectral"" ), Scale Values( [0 .] ),";
					put "							Range Type( ""Exact Data Range"" )}";
					put "						)}";
					put "					)";
					put "				)}";
					put "			)";
					put "		)";
					put "	),";
					put "	);";
					put "	);";
					%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
						put "	try(Here:molb&j&i[Text Box( 1 )] << delete;);";
					%end;
					put "	Here:mhlb1&i << Append(Here:mhlb&j.&i);";
				%end;
			%end;
		%end;
		put ");";
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			put "InsertInto(DataList,{{""Here:cedata_bycross""}});";
		%end;
		%else %do;
			put "InsertInto(DataList,{{""Here:cedata""}});";
		%end;
		%if ((&NBack > 1 or &NSelf > 1 or &NnonSelf > 1) and &LastGen > 1) or %upcase(&SimRilFast) = YES %then %do;
			%if %upcase(&SimRilSSD) = YES or %upcase(&SimRilFast) = YES %then %do;
				put "InsertInto(TabButtonList, ""RILs Simulated Trait Means Ordered"");";	
			%end;
			%else %do;
				put "InsertInto(TabButtonList, ""Multi-Generations Simulated Trait Means Ordered"");";
			%end;
		%end;
		%else %do;
			put "InsertInto(TabButtonList, ""Simulated Trait Means Ordered"");";
		%end;		
		put "InsertInto(TabCodeList,expr(Here:meansheatmap()));";
		put "InsertInto(TabAfterCodeList, expr(Here:meansheatmap_append()));";
		put "InsertInto(TabOpenList,1);";
		put " ";
	%end;
	put "ProgenySimulationExpr = Expr(";
	put "   cdata = current data table();";
	put "   nsel = NRows(cdata<<GetSelectedRows);";
	put "   if(nsel==0,";
	put "      Dialog(""No rows are selected in the currently active JMP table.  Make sure "" ||";
	put "		     ""a table with cross evaluations is active and then select points in associated graphs or rows "" ||";
	put "			  ""from the table.""),";
	put "		  cdata << Clear Column Selection();";
	put "		  selcross = cdata << Subset(output table name(""selected_crosses""));";
	put "		  selcross << Sort( By(:m_id, :p_id), Order( Ascending ), Replace table);";
	put "		  selcross << New Column( ""unique"",";
	put "		  	Numeric,";
	put "		  	Continuous,";
	put "		  	Formula(if(Lag(:m_id, 1)==:m_id & Lag(:p_id, 1)==:p_id, Row())),";
	put "		  );";
	put "		  selcross << Select Where( !ismissing(:unique));";
	put "   	  ndel = NRows(selcross<<GetSelectedRows);";
	put "   	  if(ndel > 0,";
	put "			  selcross << Delete Rows(),";
	put "   	  );";
	put "	          selcross << Delete Column(""unique"");";
	put "		  selcross << Save(""&ClientOutPath.selected_crosses.sas7bdat"",SAS);";	
	put "      if(host is(Windows),";
	put "         Include(ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ProgenySimulation.jsl"", absolute, windows)),";
	put "         Include( ConvertFilePath(""$HOME/JMPG/ProcessLibrary/ProgenySimulation.jsl"", absolute, POSIX))";
	put "      );";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""StudyName""),""&StudyName"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""InData""),""&InData0"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ScoringFolder""),""&ClientScoringFolder"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""CInData""),""&ClientOutPath.selected_crosses.sas7bdat"");";
	put "      jg:ProgenySimulation_updateDeps;";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""IDVars""),""&IDVars"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""SexVar""),""&SexVar"");";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""Maternal""),""&Maternal"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MatingType""),""&MatingType"");";	
	/*put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""TesterVar""),""&TesterVar"");";*/	
	put "      jg:ProgenySimulation_updateDeps;";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""WhereStmt""),"""");";	
	%if &nblocks = 1 %then %do;
		put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AlleleVars""),""&OrigMarkerVars"");";
	%end;
	%else %do;
		put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AlleleVars""),""&sb1";	%do k=2 %to &nblocks; put " &&sb&k"; %end;put '");';
	%end;
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ListMarkerVars""),""&ListMarkerVars"");";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ScoringCode""),""&ScoringCode"");";		
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""OutPath""),""&ClientOutPath"");";
	put "      jg:ProgenySimulation_updateDeps;";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AnnoInData""),""&AnnoInData"");";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ChromNum""),""&ChromID"");";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""ChromLab""),""&ChromLab"");";	
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerLab""),""&MarkerLab"");";		
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerPos""),""&MarkerPos_Orig"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""MarkerPos_BP""),""&MarkerPos_BP"");";
	put "      jg:setCurrentParmValue(1,""ProgenySimulation"", jg:getParmNum(""ProgenySimulation"",""AnnoWhereStmt""),""&AnnoWhereStmt"");";
	put "      jg:ProgenySimulation_updateDeps;";
	/*
	put "      jg:runProcess( 1,""ProgenySimulation"" );";
	*/
	put "   )";
	put ");";
	put "Here:ActionButtonDesc = {};";
	put "Here:ActionButtonList={{}};";
	put "Here:ActionCodeList ={{}};";
	put "InsertInto(Here:ActionButtonDesc,{expr(""Select crosses then click"")});";
	put "InsertInto(ActionButtonList[1],{expr(""Simulate Progeny"")});";
	put "InsertInto(ActionCodeList[1],expr(ProgenySimulationExpr));";

run;

%if &NBack > 1 or &NSelf > 1 or &NnonSelf > 1 %then %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CrossEvaluationScriptName, "&CE_OutDataName..sas7bdat, &CEByLines_OutDataName..sas7bdat, &CEByCross_OutDataName..sas7bdat", text/plain, yes);
%end;
%else %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CrossEvaluationScriptName, "&CE_OutDataName..sas7bdat", text/plain, yes);
%end;

%put StopMultGen= &StopMultGen;
%if &StopMultGen %then %do;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SummaryScriptName, , text/plain, yes, no);
%end;

%end; /*End EvalLines= No*/
%else %do;

	data linesdat(&dsoptions);
		set &GenData;
		%do i=1 %to &ntrait;
			%if ^%scan(&itree, &i) %then %do;
				%if ^&&Flag_AD&i %then %do;
					%let Code = %scan(&ScoringCode,&i,'|');
					**using double semicolon to avoid error when the score file does not end with a semicolon**;
					%include "&ScoringFolder.\&Code";; 
				%end;
				%else %do;
					%let ReCode = %scan(&ReCodeScores,&i," ");
					**using double semicolon to avoid error when the score file does not end with a semicolon**;
					%include "&OutPath.\&ReCode";;
					%*include "&ReCode";;
				%end;
			%end;
		%end;
			
		%if %upcase(&ComputeSelIndex) = YES and &ntree = 0 %then %do;
			&SelIndex2= &SelIndexFormula;
		%end;
	run;
	
	%if &ntree %then %do;
		%do k=1 %to &ntree;
			proc geneselect inmodel=ScoreLib.&&ScoreFile&k;;
				score data=&GenData (keep= &SexVar %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end;) 
					role=SCORE out=ts2o (drop= _WARN_ &SexVar %do i=1 %to &nm; %sysfunc(varname(&_dsix_,&i)) %end;) 
					outfit=_null_;
			run;
			quit;
						
			/*data OutLib.c_ts2o&k; set ts2o; run;*/				
			
			%if &&DepVarType3&k ^= CONTINUOUS %then %do; 
				%put NumbLevels3&k= &&NumbLevels3&k;
			
				%if &&NumbLevels3&k = 2 %then %do;
					* binary trait*;
					%if %length(&&trait_pt&k) >= 10 %then %do;
						%let DepVarTrunc = %substr(&&trait_pt&k,1,7);
					%end;
					%else %do;
						%let DepVarTrunc = &&trait_pt&k;
					%end;
				%end;
				%else %do;
					* nominal trait*;
					%if %length(&&trait_pt&k) > 8 %then %do;
						%let DepVarTrunc = %substr(&&trait_pt&k,1,8);
					%end;
					%else %do;
						%let DepVarTrunc = &&trait_pt&k;
					%end;
				%end;					
			%end;
			%else %do;
				%if %length(&&trait_pt&k) > 30 %then %do;
					%let DepVarTrunc = %substr(&&trait_pt&k,1,30);
				%end;
				%else %do;
					%let DepVarTrunc = &&trait_pt&k;
				%end;
			%end;
				
			%if &&DepVarType3&k ^= CONTINUOUS %then %do;
				* binary or nominal trait;
				* construct _INTO_ and Predicted_Class variable;
				
				%if &&NumbLevels3&k = 2 %then %do;
					* binary trait*;
					%let into = P_&DepVarTrunc.&&PTvlevel&k.1;	 
				%end;
				%else %do;
					* nominal trait*;
					%let into = U_&DepVarTrunc;				
				%end;
				
				data ts2o;
					set ts2o;
					/*
					%if &into ^= _INTO_ %then %do;
						_INTO_ = &into;
					%end;
					%if &_dtype_ = C %then %do;
						Predicted_Class = &into;
					%end;
					%else %do;
						Predicted_Class = 0 + &into;
					%end;
					*drop &into;
					*/
					&&trait2_pt&k = &into;
				run;
			%end;
			%else %do;
				* continuous trait;
				
				%let pred = P_&DepVarTrunc;
				
				data ts2o;
					set ts2o;
					*Predicted_Value = &pred;
					&&trait2_pt&k = &pred;
				run;
			%end;
			/*data OutLib.c_ts2o&k; set ts2o; run;*/
			
			data linesdat;
				merge linesdat ts2o;
				%if %upcase(&ComputeSelIndex) = YES %then %do;									
					&SelIndex2= &SelIndexFormula; 									
				%end;
			run;
		%end;
	%end;	
	
	/*data OutLib.c_linesdat; set linesdat; run;*/

	data OutLib.&CE_OutDataName(&dsoptions); 
		set linesdat;
	run;

	* create JSL code;
	data _null_;
	   	file "&CrossEvaluationScript" lrecl=32767;
		put '//!';
		put "Names Default to Here(1);";
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";	

		put "Here:mult = Function({},{default local}," ;
		put "If(IsEmpty(Here:cedata),";	
		put "   Here:cedata = open(""&ClientOutPath.&CE_OutDataName..sas7bdat"",invisible);";
		/*put "   Column(""m_id"") << label(1);";*/
		/*put "   Column(""p_id"") << label(1);";*/
		put "   ,"; 
		put "   Current Data Table(Here:cedata);";
		put ");";
		put "HListBox( ";
		/*
		%if %length(&TesterVar) or %length(&ParentalVar) %then %do;
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";	
			%if %length(&TesterVar) %then %do;
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "         		:&&trait&i.._Mean,";
				%end;		
				put "				:tester_id";
				put "),";
			%end;
			%else %do;
				put "			columns(";
				%do i=1 %to &ntraitix; 
					put "         		:&&trait&i.._Mean,";
				%end;
				put "				:_orig_p_lines_";
				put "),";
			%end;
			put "			),";
			put "		),";	
		%end;
		*/
		/*%else %do;*/
			put "		Current Data Table() << Data Filter(";
			put "			Location( {120, 18} ),";
			put "			Add Filter(";			
			put "			columns("; 
			%do i=1 %to &ntraitix; 
				/*put "         		:&&trait&i.._Mean,";*/
				put "         		:&&trait2&i,";
			%end;
			/*put "				:p_ID, :m_ID";*/
			put "				),";
			put "			),";
			/*put "			Animation(),";*/
			put "		),";	
		/*%end;*/
		put "   VListBox( ";
		put "      Multivariate(";
		put "         Y( ";
		%do i=1 %to &ntraitix; 
			put "         Column(""&&trait2&i""),";
			/*
			put "         Column(""&&trait&i.._Mean""),";
			put "         Column(""&&trait&i.._StdDev""),";
			put "         Column(""&&trait&i.._Max""),";
			put "         Column(""&&trait&i.._Min""),";
			put "         Column(""&&trait&i.._Range""),";
			*/
		%end;
		put "         ),";
		put "         Scatterplot Matrix(Density Ellipses(0), Horizontal(1)),";
		/*
		%if &LastGen > 1 %then %do;
			%if &NBack > 1 %then %do;
				put "         By( :_backgeneration_),";
			%end;
			%else %if &NSelf > 1 or %upcase(&SimRilFast) = YES %then %do;
				put "         By( :_selfgeneration_),";
			%end;
			%else %if &NnonSelf > 1 %then %do;
				put "         By( :_generation_),";
			%end;	
		%end;
		*/
		put "      )";
		put "   )";
		put "));";
		put "InsertInto(DataList,{{""Here:cedata""}});";
		put "InsertInto(TabButtonList, ""Individuals Predicted Trait Statistics"");";
		put "InsertInto(TabCodeList,expr(Here:mult()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		/*%if &Simulate = Yes %then %do;
			put "InsertInto(TabOpenList,0);";
		%end;
		%else %do;*/
			put "InsertInto(TabOpenList,1);";
		/*%end;*/
		put " ";
		
	run;
	
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CrossEvaluationScriptName, "&CE_OutDataName..sas7bdat", text/plain, yes);
	
%end;

%exit:;

%if %symexist(_dsix_in) %then %do;
	%let _dsix_in = %sysfunc(close(&_dsix_in));
%end;
%do i=1 %to &ntrait;
	%if %symexist(_dsix_&i) %then %do;
		%let _dsix_&i = %sysfunc(close(&&_dsix_&i));
	%end;
%end;
%if %symexist(_dsix_) %then %do;
	%let _dsix_ = %sysfunc(close(&_dsix_));
%end;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend;

* run the macro;
%CrossEvaluation;

