************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:      CrossValidationModelComparisonMacros
**  AUTHOR:    saspyt, Pei-Yi Tan
**  SUPPORT:   saspyt, Pei-Yi Tan
**  DATE:      02/19/07
**  PURPOSE:   Macros for JMP Life Sciences Cross Validation Model Comparison
**  HISTORY:   action                                            date who
**             Created.                                      02/19/07 pyt
**  
**  MACRO NAMES: 
**  APPLYCOSTS: Apply cost functions in model prediction
**  BASESCORINT: Create interval trait baseline scored data set
**  BASESCOR: Create categorical trait baseline scored data set
**  BIAS_ESTIMATES: Bias test estimate
**  COMPUTE_ACC_NOMINAL: Compute accuracy for nominal trait
**  COMPUTE_AEC_NOMINAL: Compute average expected cost for nominal trait
**  COMPUTE_ASE_INTERVAL: Compute interval trait model average square error
**  COMPUTE_ASE_NOMINAL: Compute categorical trait model average square error
**  COMPUTE_AUC: Compute area-under-roc
**  COMPUTE_CORR_INTERVAL: Compute interval trait model correlation
**  COMPUTE_EFFECTS: Compute predictors effect frequency and order 
**  COMPUTE_HARRELLC: Compute Harrell concordance index of survival model
**  COMPUTE_MANN: Compute Mann-Whitney statistic
**  COMPUTE_MEAN_SURVFUNC: Compute average survival function estimates 
**  COMPUTE_PVALUE: Compute pevent or predicted trait values when 
**           		an importance variable is removed
**  COMPUTE_SURVIVAL_AUC: Compute area under survival curve
**  COMPUTE_WHISKERS: Compute box plot upper and lower whiskers
**  CLASSES_EVENT: Retrieve categorical trait classes and event
**  CROSS_VALIDATION: Cross validation
**  CV_ITERATIONS: Cross validation iterations 
**  CV_ITERATION_LOOP: Cross validation hold-out iterations
*   CV_LEARNING_CURVE_ESTIMATES: Cross validation learning curve  
**  CV_MODEL_PVALUE_DATA: Generate a cv model pvalues plot data 
**  CV_MODEL_ROC_DATA: Generate cv model roc plot data 
**  DUPOBS: Inflat data based on the interval trait distribution
**  ENSEMBLE: Forward Stepwise with-replacement ensemble
**  ENSEMBLE_CV: Forward Stepwise with-replacement ensemble
**  GET_PRIORS: Get prior probabilities 
**  GENERATE_CV_VARS: Generate cross validation variables
**  GRIDSEQUENCE: Generate grid computing distribution
**  LC_BIAS_PLOTS_DATA: Generate learning curve bias estimates plots data
**  LC_CV_ITERATION_LOOP: Learning curve hold-out iterations 
**  LC_STANDARD_ESTIMATES: Learning curve with fixed test data set 
**  LC_STANDARD_ITERATION_LOOP: Learning curve with fixed test set iterations 
**  LC_ITERATION_LOOP: Loop over learning curve outer and inner loops 
**  LC_PLOTS_DATA: Generate learning curve estimates plots data
**  LEARNING_CURVE: General learning curve 
**  LEAVE_L_OUT: Leave l out cross validation method 
**  LEAVE_L_OUT_ESTIMATES: Fit leave l out cross validation models,  
**           	 append models predictions and selected predictors
**  LO_FITSTATS: Macro statement to insert in the if statement for 
**               appending leave_l_out output
**  LOCATE_PEVENT_ISEVENT: Locate pevent corresponds to the actual event
**  LOOP_PREDS: Loop over predictors to drop one at a time from fit model
**  MERGE_CV_RESULTS: Merge cross validation models ase, acc, auc results
**  MERGE_CV_MULTI_RESULTS: Merge multiple cross validation results
**  MERGE_LC_RESULTS: Merge learning curve models ase, acc, auc results
**  MERGE_LC_MULTI_RESULTS: Merge multiple learning curves results
**  MERGE_LLO_RESULTS: Merge leave-l-out models ase, acc, auc results
**  PRESELECT_PREDICTORS: Preselect predictors
**  PSEUDO: Creates pseudo variables with values 1, 2, ...., kfold
**  RANDOM_SAMPLE: Generate random sample
**  RELIABILITY_DIAGRAM: Generate Reliability Diagram  
**  STRATIFIED_SAMPLE: Generate stratified random sample 
**  SURVIVAL_PLOT_DATA: Generate survival plot data 
**  TRANSCORINT: Convert interval trait scored data created from 
**               PROC GENESELECT to cv wrapper form of scored data set
**  TRANSCOR: Convert categorical trait scored data created from 
**            PROC GENESELECT to cv wrapper form of scored data set
************************************************************************;

/*% example costfunction="1x 2 as 0 y 3=0.3, 0 y 3 AS 1x 3 = 1.0, 1x 2 As 1x 3 = -0.5" */
%macro ApplyCosts(data=, costfunction=, intovar=);                                                                                               

%put Calling ApplyCosts with data=&data, costfunction=&costfunction, intovar=&intovar;

* RETRIEVE COST FUNCTIONS;

data cs;
   length word $32767;
   drop string count;
   string = "&costfunction";
   do until(word=' ');
      count+1;
      word = upcase(compress(scan(string, count, ",=")));
	  if word ne '' then output;
   end;
run;

%do r=1 %to &nclasses;
	%let classr = &&class&r;
	data _null_;
		class = compress("&classr");
		call symput('classr', trim(left(class)));
	run;
	%do c=1 %to &nclasses;
		%let classc = &&class&c;
		data _null_;
			class = compress("&classc");
			call symput('classc', trim(left(class)));
		run;
		%let cdef_&r._&c=%upcase(&classr.AS&classc);
		%let cost_&r._&c=;
	%end;
%end;

%let _csid_ = %sysfunc(open(cs));
%let _ncs_ = %sysfunc(attrn(&_csid_, NOBS));
%do w=1 %to &_ncs_;
	%let rc = %sysfunc(fetchobs(&_csid_, &w));
	%let word = %sysfunc(getvarc(&_csid_, 1));
	%put word = &word;
	%let w = %sysevalf(&w + 1, int);
	%do r=1 %to &nclasses;
		%do c=1 %to &nclasses;
			%let cdef = &&cdef_&r._&c;
			%put cdef=&cdef; 
			%if %trim(%left(&word)) = %trim(%left(&cdef)) %then %do;
				%let rc = %sysfunc(fetchobs(&_csid_, &w));
				%let cost = %sysfunc(getvarc(&_csid_, 1));
				%let cost_&r._&c = &cost;
			%end;
		%end;
	%end;
%end;
%let _csid_ = %sysfunc(close(&_csid_));

* CREATE COST MATRIX DATA SET
  UNLESS SPECIFIC COST IS SPECIFIED, OTHERWISE USE DEFAULT  
  DEFAULT COST FOR CORRECTLY PREDICTED = 0, ELSE = 1;
data costmtx;
	length trait $512 class
	%do c=1 %to &nclasses;
		class&c
	%end; 
	8 ;
	%do r=1 %to &nclasses;
		class=&r;
		%let class=&&class&r;
		label class&r="&class";
		trait="&class";
		%do c=1 %to &nclasses;
			%let cost = &&cost_&r._&c;
			%if %length(&cost) > 0 %then %do;
				class&c = &cost;
			%end;
			%else %do;
				%if &r=&c %then %do;
					class&c=0;
				%end;
				%else %do;
					class&c=1;
				%end;
			%end;
		%end;
		output;
	%end;
run;

%if &_dtype_ = N %then %do;
	data costmtx;
		set costmtx;
		length trait_n 8;
		drop trait;
		trait_n = 0 + trait;
		rename trait_n = trait;
	run;
%end;

data cspred ;                                                                                                                            
   keep class Expected_Cost probability;                                                                                                   
   array costmatrix(&nclasses, &nclasses);                                                                  
   retain costmatrix;                                                                                                                   
   set costmtx end=gotcosts;                                                                                                          
                                                                                                                                        
   array tt                                                                                                                             
        %do c=1 %to &nclasses;                                                                                                          
            class&c                                                                                                                      
        %end;                                                                                                                           
        ;                                                                                                                               
                                                                                                                                        
   do col=1 to &nclasses;                                                                             
      costmatrix(_n_, col) = tt(col);                                                                                                   
   end;                                                                                                                                 
                                                                                                                                        
   if gotcosts ne 0 then do;                                                                                                            
       do i = 1 to nobs;                                                                                                                
           set &data. point=i nobs=nobs;                                                                                                
           array pp p_:;                                                                                                                
           isbest    = 1;            * first class is best until not;                                                                   
           do row=1 to &nclasses;                                                                           
              Expected_Cost = 0;                                                                                                                 
              do col=1 to &nclasses;                                                                   
                 Expected_Cost = Expected_Cost + pp[col]* costmatrix(col, row);                                                                           
              end;                                                                                                                      
              if row > 1 then do;                                                                                                       
                 if Expected_Cost < bestcost then isbest = 1;                                                                                    
              end;                                                                                                                      
              if isbest ne 0 then do;                                                                                                   
                 bestrow   = row;                                                                                                       
                 bestcost  = Expected_Cost;                                                                                                      
              end;                                                                                                                      
              * put row= Expected_Cost= bestcost=;                                                                                               
              isbest=0;                                                                                                                 
           end;                                                                                                                         
		   class=bestrow; 
           Expected_Cost = bestcost;                                                                                                             
           probability= pp[class];                                                                                                      
           output;                                                                                                                      
       end;                                                                                                                             
   end;                                                                                                                                 
run;     

data cspred;
	set cspred;
	__n = _n_;
run;

proc sort data=cspred;
	by class;
run;
data cspred;
	merge cspred costmtx;
   by class;
run;
proc sort data=cspred;
	by __n;
run; 
* REPLACE &intovar WITH THE new_into IN cspred DATA SET;
%let _dsid_ = %sysfunc(open(&data));
%let intovarnum = %sysfunc(varnum(&_dsid_, &intovar));
%let intovarlabel = %sysfunc(varlabel(&_dsid_, &intovarnum));
%let _dsid_ = %sysfunc(close(&_dsid_));

data cspred;
	set cspred;
	keep trait Expected_Cost;
	rename trait=new_into;
run;
data outlib.costmtx;
	set costmtx;
run;

data &data;
	merge &data cspred;
run;
data &data;
	set &data;
	drop &intovar;
	rename new_into = &intovar;
	%if %length(&intovarlabel) > 0 and "&intovarlabel" ne "" %then %do;
		label new_into="&intovarlabel";
	%end;
run;
%mend; 

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:     BASESCORINT
**  AUTHOR:   saspyt, Pei-Yi Tan
**  SUPPORT:  saspyt, Pei-Yi Tan
**  DATE:     02/19/07
**  PURPOSE:  Create interval trait baseline scored data set. 
**  HISTORY:  action                                            date who
**            Created.                                      02/19/07 pyt 
**
**  MACRO PARAMETERS:
**  TRAIN_DSNAME: Training data set name
**  TEST_DSNAME:  Test data set name
**  TRAIT:        Trait variable name
**
**  NOTES: Create interval trait baseline(without a model) scored data.
************************************************************************;
%macro basescorint(train_dsname=, 
                   test_dsname=,
                   trait= );

%if %sysfunc(exist(&train_dsname)) = 1 %then %do;
	%NObsVars(&train_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &train_dsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &train_dsname DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(&test_dsname)) = 1 %then %do;
	%NObsVars(&test_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &test_dsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &test_dsname DOES NOT EXIST.;
	%goto exit;
%end;

*  PREDICTION EQUALS THE AVERAGE TRAIT AMONG TRAINING OBSERVATIONS;
proc means data=&train_dsname noprint;
	var &trait;
	output out=outmean mean=Predicted_Value;
run;

data baseline_preds;
	set &test_dsname;
	retain Predicted_Value;
	keep &trait Predicted_Value;
	if _n_ eq 1 then do;
		set outmean;
		call symput("BPV",trim(left(Predicted_Value)));
	end;
	if not missing(&trait) then output;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:     BASESCOR
**  AUTHOR:   saspyt, Pei-Yi Tan
**  SUPPORT:  saspyt, Pei-Yi Tan
**  DATE:     02/19/07
**  PURPOSE:  Create categorical trait baseline scored data set. 
**  HISTORY:  action                                            date who
**            Created.                                      02/19/07 pyt 
**
**  MACRO PARAMETERS:
**  TRAIN_DSNAME: Training data set name
**  TEST_DSNAME:  Test data set name
**  TRAIT:        Trait variable name
**  PREVALENCES:  Prior probabilities (EQUAL or PROPORTIONAL)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**
**  NOTES: Create categorical trait baseline(without a model) scored data
************************************************************************;
%macro basescor(train_dsname=, 
                test_dsname=,
                trait=,
                prevalences=proportional,
				cv_custompriors=
                );

data baseline_preds;
	delete;
run;

%if %sysfunc(exist(&train_dsname)) = 1 %then %do;
	%NObsVars(&train_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &train_dsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &train_dsname DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(&test_dsname)) = 1 %then %do;
	%NObsVars(&test_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &test_dsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &test_dsname DOES NOT EXIST.;
	%goto exit;
%end;

*  POSTERIOR PROBABILITIES ARE THE TRAINING DATA CLASS PROPORTIONS;
*  CREATE DATA ACTUALP WITH TWO VARIABLES: TRAIT AND CLASS PROPORTION;
proc freq data=&train_dsname noprint;
	tables &trait/out=f;
run;

data actualp; 
	set f;
	keep &trait actualp;
	if missing(&trait) then delete;
	actualp = percent / 100;
run;

%NObsVars(actualp);
%if ^&nObs or ^&nVars %then %do;
	%goto exit;
%end; 
%global nclasses;
%let nclasses = &nobs;
%put nclasses = &nclasses;
%do i=1 %to &nclasses;
	%global class&i;
%end;
proc sort data=actualp;
	by &trait;
run;
data _null_;
	set actualp;
	call symput("class"||trim(left(_n_)),trim(left(&trait)));
run;
%do i=1 %to &nclasses;
	%put class&i = &&class&i;
%end;

%let _apid_ = %sysfunc(open(actualp));
%let traitnum = %sysfunc(varnum(&_apid_, &trait)); 
%let traittype = %sysfunc(vartype(&_apid_, &traitnum));
%let _apid_ = %sysfunc(close(&_apid_));

* IF EQUAL PREVALENCES, SET ACTUALP = 0.5;
%if %upcase(&prevalences)=EQUAL %then %do;
	data actualp;
		set actualp;
		actualp = 0.5;
	run;
%end;  
%else %if &prevalences=CUSTOM and %length(&cv_custompriors) > 0 and 
	&cv_custompriors ne "" %then %do;
	* SET CUSTOM PRIORS AS ACTUALP;
	%get_priors(priors_string=%nrbquote(&cv_custompriors), traittype=&traittype);
					
	proc sort data=adjust_pr;
		by &trait;
	run;
	data actualp;
		merge actualp adjst_pr;
		by &trait;
	run;
	data actualp;
		set actualp;
		drop actualp;
		rename priors=actualp;
	run;
%end;

* SAVE PROPORTIONS;
data _null_;
	set actualp;
	call symputx("actualp"||trim(left(_n_)),trim(left(actualp)));
run;

*  SAVE MOST FREQUENT TRAIT VALUE AS MACRO VARIABLE BPV;
proc sort data=actualp;
	by actualp;
run;
data _null_;
	set actualp end=the_end;
	if the_end then do;
		call symput("BPV",trim(left(&trait)));
	end;
run;
%put BPV = &BPV;


*  DATA SET T WILL HAVE A VARIABLE Pk FOR TRAIT K;
*  1 OBSERVATION;                                   
proc transpose data=actualp(keep=actualp) out=t prefix=p_; 
run;

*  NOW THE OBSERVATIONS TO SCORE. ONLY NEED TRAIT VALUE;
data testcases;
	set &test_dsname;
	keep &trait;
	if not missing(&trait) then output;
run;
proc sort data=testcases; 
 	by &trait;
run;
proc sort data=actualp; 
	by &trait;
run;
data almost_done; 
	merge testcases actualp; 
	by &trait;
run;

data baseline_preds; 
	set almost_done;
	if _n_ eq 1 then set t(keep= p_:);  
	%if &traittype = C %then %do;
		Predicted_Class = "&BPV";
	%end;
	%else %do;
		Predicted_Class = &BPV;
	%end;
	if (trim(left(&trait)) = "&BPV") then Correct = 1;
	else Correct = 0;
run;

%if %length(&cv_customcosts) %then %do;
	%let _dtype_ = &traittype;
	%ApplyCosts(data=baseline_preds,costfunction=&cv_customcosts,intovar=Predicted_Class);
%end;
  
%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    BIAS_ESTIMATES
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/12/07
**  PURPOSE: Bias estimate 
**  HISTORY: action                                        date     who
**           Created.                                      06/15/07 pyt 
**
**  MACRO PARAMETERS:
**  OUTPUTPATH: Library path of output data sets 
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODESL: Number of models
**  CVLC_ESTIMATES_OUTPUT: Cross validation learning curve estimates 
**  LC_FIXTEST_ESTIMATES_OUTPUT: Fixed test learning curve estimates
**	BIAS_ESTIMATES_OUTPUT: Bias estimates 
**
**  NOTES:
**	   Formula:
**     Bias estimate = cv estimate - fixed test estimate  
**	or Bias estimate = cv estimate - generalized test estimate
**  
**  1. Fixed test: test set is generated by random or stratified sample
**	2. Generalized test: user specifies a test set
***********************************************************************;
%macro bias_estimates(outputpath=,
					  model_names=,
					  model_labels=,
					  n_models=,
					  cvlc_estimates_output=,
					  lc_fixtest_estimates_output=,
					  bias_estimates_output=);

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%if %sysfunc(exist(&cvoutlib..&cvlc_estimates_output)) = 1 %then %do;
	%NObsVars(&cvoutlib..&cvlc_estimates_output);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&cvlc_estimates_output CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&cvlc_estimates_output DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(&cvoutlib..&lc_fixtest_estimates_output)) = 1 %then %do;
	%NObsVars(&cvoutlib..&lc_fixtest_estimates_output);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&lc_fixtest_estimates_output CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&lc_fixtest_estimates_output DOES NOT EXIST.;
	%goto exit;
%end;

* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
   %let mdlname&i = %scan(&model_names,&i," ");
   %let mdllabel&i = %scan(&model_labels,&i,"|");
%end;

data t;
	set &cvoutlib..&lc_fixtest_estimates_output;
	%do nm=1 %to &n_models;
		rename ASE_m&nm=ASE_m&nm._T;
		rename RMSE_m&nm=RMSE_m&nm._T;
		rename ACC_m&nm=ACC_m&nm._T;
		rename AUC_m&nm=AUC_m&nm._T;
	%end;
	drop TRAIN_SIZE;
run;

proc sort data=t;
	by outer_loop inner_loop;
run;

data &cvoutlib..&bias_estimates_output;
	merge &cvoutlib..&cvlc_estimates_output t;
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
		%let mdllabel=&&mdllabel&nm;
		BIAS_ASE_m&nm=ASE_m&nm - ASE_m&nm._T;
		BIAS_RMSE_m&nm=RMSE_m&nm - RMSE_m&nm._T;
		BIAS_ACC_m&nm=ACC_m&nm - ACC_m&nm._T;
		BIAS_AUC_m&nm=AUC_m&nm - AUC_m&nm._T;
		ABIAS_ASE_m&nm = abs(BIAS_ASE_m&nm);
		ABIAS_RMSE_m&nm = abs(BIAS_RMSE_m&nm);
		ABIAS_AUC_m&nm = abs(BIAS_AUC_m&nm);
		ABIAS_ACC_m&nm = abs(BIAS_ACC_m&nm);
		SBIAS_ASE_m&nm = BIAS_ASE_m&nm * BIAS_ASE_m&nm;
		SBIAS_RMSE_m&nm = BIAS_RMSE_m&nm * BIAS_RMSE_m&nm;
		SBIAS_AUC_m&nm = BIAS_AUC_m&nm * BIAS_AUC_m&nm;
		SBIAS_ACC_m&nm = BIAS_ACC_m&nm * BIAS_ACC_m&nm;
		keep BIAS_ASE_m&nm 
			 BIAS_RMSE_m&nm
			 BIAS_ACC_m&nm
			 BIAS_AUC_m&nm
			 ABIAS_ASE_m&nm 
			 ABIAS_RMSE_m&nm
			 ABIAS_ACC_m&nm
			 ABIAS_AUC_m&nm
			 SBIAS_ASE_m&nm 
			 SBIAS_RMSE_m&nm
			 SBIAS_ACC_m&nm
			 SBIAS_AUC_m&nm
		;
		label BIAS_ASE_m&nm="BIAS ASE &mdllabel";
		label BIAS_RMSE_m&nm="BIAS RMSE &mdllabel";
		label BIAS_ACC_m&nm="BIAS ACC &mdllabel";
		label BIAS_AUC_m&nm="BIAS AUC &mdllabel";
		label ABIAS_ASE_m&nm="ABIAS ASE &mdllabel";
		label ABIAS_RMSE_m&nm="ABIAS RMSE &mdllabel";
		label ABIAS_ACC_m&nm="ABIAS ACC &mdllabel";
		label ABIAS_AUC_m&nm="ABIAS AUC &mdllabel";
		label SBIAS_ASE_m&nm="SBIAS ASE &mdllabel"; 
		label SBIAS_RMSE_m&nm="SBIAS RMSE &mdllabel";
		label SBIAS_ACC_m&nm="SBIAS ACC &mdllabel";
		label SBIAS_AUC_m&nm="SBIAS AUC &mdllabel";
	%end;
	keep TRAIN_SIZE OUTER_LOOP INNER_LOOP;
	by outer_loop inner_loop /* train_size*/;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_ACC_NOMINAL
**  AUTHOR:  sasrdw, Russ Wolfinger
**  SUPPORT: sasrdw, Russ Wolfinger
**  DATE:    04/12/07
**  PURPOSE: Compute accuracy for nominal trait 
**            
**  HISTORY: action                                            date who
**           Created.	                                   04/12/07 rdw 
**
**  MACRO PARAMETERS:
**  DATA:    Scored data set    
**  OUTPUT:  ACC output data set
**  TRAIT:   Name of trait/response/dependent variable that has been predicted
**  CORRECT: Name of 0-1 variable whose value   
**           indicates if the prediction of the response is correct     
**  MDLNAME: Model method name                                           
**  FREQ:    Name of the variable in the DATA= data set whose value   
**           is the frequency of observation (Optional)      
**  PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM) 
**
**  NOTES: Accuracy is computed as the overall mean of CORRECT.  
**  	   The output also contains the mean for each distinct
**         level of the response.  For binary responses, this
**         corresponds to specificity and sensitivity.
************************************************************************;
%macro compute_acc_nominal(data=, 
						   output=, 
						   trait=, 
						   correct=, 
						   mdlname=, 
						   freq=,
						   prevalences=proportional,
						   cv_custompriors=
						   ); 

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

%if %upcase(&prevalences)=PROPORTIONAL %then %do;
	proc means data=&data noprint;
		 var &correct;
		 %if %length(&freq) %then %do;
			  freq &freq;
		 %end;
		 output out=acc(keep=Accuracy) mean=Accuracy;
	run;
%end;
    
proc sort data=&data out=sorted;
	by &trait;
run;

proc means data=sorted noprint;
	by &trait;
	var &correct;
	%if %length(&freq) %then %do;
		freq &freq;
	%end;
	output out=acc1 n=n1 mean=acc1;
run;

data acc1;
	set acc1;
	length _trid $ 23;
	if missing(&trait) then delete;
	* make sure id value produces a valid sas name;
	_trid = trim(left(&trait));
	lent = min(length(_trid),23);
	do i=1 to lent;
		_check2 = verify(upcase(substr(_trid,i,1)),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
		if _check2 then do;
			substr(_trid,i,1) = "_";
		end;
	end;
run;

proc transpose data=acc1 out=nby(drop=_name_) Prefix=NOBS_;
	var n1;
	id _trid;
run;

proc transpose data=acc1 out=accby(drop=_name_) Prefix=Accuracy_;
	var acc1;
	id _trid;
run;

* assume equal prevalance if not proportional, add custom later;
%if %upcase(&prevalences)^=PROPORTIONAL %then %do;

	%if &prevalences=CUSTOM and %length(&cv_custompriors) > 0 
		and &cv_custompriors ne "" %then %do;
		%let _aseid_ = %sysfunc(open(ase2));
		%let traitnum = %sysfunc(varnum(&_aseid_, &trait)); 
		%let traittype = %sysfunc(vartype(&_aseid_, &traitnum));
		%let _aseid_ = %sysfunc(close(&_aseid_));
		
		%get_priors(priors_string=%nrbquote(&cv_custompriors), traittype=&traittype);

		data acc1;
			merge acc adjust_pr;
			by &trait;
		run;
	%end;
	
	proc means data=acc1 noprint;
		%if &prevalences=CUSTOM and %length(&cv_custompriors) > 0 and &cv_custompriors ne "" %then %do;
			weight priors;
		%end;
		 var acc1;
		 output out=acc(keep=Accuracy) mean=Accuracy;
	run;
%end;

data method;
	length METHOD $64;
	METHOD  = "&mdlname";
run;

data &output;
	merge method acc nby accby;
run;
    
%exit:;
%mend;


************************************************************************
**  Copyright (c) 2011 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_AEC_NOMINAL
**  AUTHOR:  sasrdw, Russ Wolfinger
**  SUPPORT: sasrdw, Russ Wolfinger
**  DATE:    04/28/11
**  PURPOSE: Compute average expected cost for nominal trait 
**            
**  HISTORY: action                                            date who
**           Created.	                                   04/28/11 rdw 
**
**  MACRO PARAMETERS:
**  DATA:    Scored data set    
**  OUTPUT:  AEC output data set
**  TRAIT:   Name of trait/response/dependent variable that has been predicted
**  EXPCOST: Name of variable whose value is the expected cost for that observation  
**  MDLNAME: Model method name                                           
**  FREQ:    Name of the variable in the DATA= data set whose value   
**           is the frequency of observation (Optional)      
**
**  NOTES:  AEC is computed as the overall mean of EXPCOST.  
**  	   The output also contains the mean for each distinct
**         level of the trait.  
************************************************************************;
%macro compute_aec_nominal(data=, 
						   output=, 
						   trait=, 
						   expcost=, 
						   mdlname=,
						   freq=
						   ); 

						   
%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

proc means data=&data noprint;
	var &expcost;
	%if %length(&freq) %then %do;
		freq &freq;
	%end;
	output out=aec(keep=AEC) mean=AEC;
run;
    
proc sort data=&data out=sorted;
	by &trait;
run;

proc means data=sorted noprint;
	by &trait;
	var &expcost;
	%if %length(&freq) %then %do;
		freq &freq;
	%end;
	output out=aec1 mean=aec1;
run;

data aec1;
	set aec1;
	length _trid $ 23;
	if missing(&trait) then delete;
	* make sure id value produces a valid sas name;
	_trid = trim(left(&trait));
	lent = min(length(_trid),23);
	do i=1 to lent;
		_check2 = verify(upcase(substr(_trid,i,1)),"_ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
		if _check2 then do;
			substr(_trid,i,1) = "_";
		end;
	end;
run;

proc transpose data=aec1 out=aecby(drop=_name_) Prefix=AEC_;
	var aec1;
	id _trid;
run;

data method;
	length METHOD $64;
	METHOD  = "&mdlname";
run;

data &output;
	merge method aec nby aecby;
run;
    
%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_ASE_INTERVAL
**  AUTHOR:  saspyt, Pei-Yi Tan
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Compute interval trait model average square error.
**            
**  HISTORY: action                                            date who
**           Created.                                      02/19/07 pyt 
**
**  MACRO PARAMETERS:
**  DATA:    Scored data set         
**  OUTPUT:  ASE output Data Set
**  Y:       Name of the response variable in the DATA= data set      
**  PREDICT: Name of the variable in the DATA= data set whose value   
**           is the prediction of the observed response               
**  MDLNAME: Model method name                                           
**  AWEIGHT: Asymmetric weight (Optional)                      
**  FREQ:    Frequency of observation (Optional)                      
**
**  NOTES:
************************************************************************;
%macro compute_ase_interval(data=, 
							output=, 
							y=, 
							predict=,
							aweight=,
							freq=,
							mdlname= 
							);  

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

data &output;
	length METHOD $64 ITER 8 K 8;
	retain sse 0 sae 0 nobs 0;
	set &data end=the_end;

	if not missing(&predict) and not missing(&y) then do;
		s = (&y - &predict)**2;
		sa = abs(&y - &predict);
		%if %length(&aweight) > 0 and &aweight ne "" %then %do;
			s = s * &aweight;
			sa = sa * &aweight;
		%end;
		%if %length(&freq) > 0 and &freq ne "" %then %do;
			s = s * &freq;
			sa = sa * &freq;
			nobs = nobs + &freq;
		%end;
		%else %do;
			nobs = nobs + 1;
		%end;
		sse = sse + s;
		sae = sae + sa;
	end;

	if the_end then do;
		METHOD  = "&mdlname";
		ASE = sse / nobs;
		MAE = sae / nobs;
		output;
	end;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_ASE_NOMINAL
**  AUTHOR:  saspyt, Pei-Yi Tan
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Compute categorical trait model average square error.
**            
**  HISTORY: action                                            date who
**			 Created.									   02/19/07 pyt 
**           Changed denominator to scalar level   		   05/21/07 rdw
**           Added prevalences and trait parms			   04/03/08 rdw
**
**  MACRO PARAMETERS:
**  DATA:     Scored data set    
**  OUTPUT:   ASE output Data Set
**  ACTUAL:   Name of the variable in the DATA= data set whose value   
**            is the posterior probality of the observed response.     
**  NCLASSES: The number of classes                                    
**  MDLNAME:  Model method name                                           
**  FREQ:     Name of the variable in the DATA= data set whose value   
**            is the frequency of observation (Optional)               
** 	PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities
**
**  NOTES:
**  FORMULA                                                                
**     ASE = (1 - actual)^2 + sum over classes of posteriors - actual^2.   
**     note (1 - p)^2 - p^2 = 1 - 2 p   
**	     Also compute MAE = sum of (1 - actual)                                    
************************************************************************;
%macro compute_ase_nominal(data=, 
					       output=, 
						   actual=, 
						   nclasses=, 
						   mdlname=, 
						   freq=,
						   prevalences=proportional,
						   cv_custompriors=,
						   trait=
						   ); 

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

%let prevalences = %upcase(&prevalences);
%if &prevalences=PROPORTIONAL %then %do;
	data &output;
		length METHOD $64 ITER 8 K 8;
		keep METHOD ITER K ASE MAE NOBS NOBSALL;
		retain sse 0 sae 0 sse2 0 NOBS 0 NOBSALL 0;
		set &data end=the_end;
		array _p p_:;

		if not missing(&actual) then do;
			s = 1.0 - 2 * &actual;
			sa = 1.0 - &actual;
			sa2 = sa*sa;
			do i = 1 to dim(_p);
				* regard missing _p(i) as 0;
				* They occur when class i is not present in the training data;
				if not missing( _p(i)) then s = s + _p(i)**2;
			end;
			%if %length(&freq) > 0 and &freq ne "" %then %do;
				s = s * &freq;
				sa = sa * &freq;
				sa2 = sa2 * &freq;
				nobsall = nobsall + &freq * dim(_p);
			%end;
			%else %do;
				nobsall = nobsall + dim(_p);
			%end;
			sse = sse + s;
			sae = sae + sa;
			sse2 = sse2 + sa2;
			nobs = nobs + 1;
		end;

		if the_end then do;
			METHOD  = "&mdlname";
			if nobsall gt 0 then do;
				 MAE = sae / nobs;
				 ASE = sse2 / nobs;
			end;
			else do;
				put "WARNING: COMPUTE_NOMINAL_ASE SET ASE = . BECAUSE NO OBS";
				ASE = .;
				MAE = .;
			end;
			output;
		end;
	run;
%end;
%else %do;
	data ase;
		length METHOD $64 ITER 8 K 8;
		keep METHOD &trait SA SA2;
		set &data;
		&trait = upcase(&trait);
		METHOD  = "&mdlname";
		sa = 1.0 - &actual;
		sa2 = sa*sa;
	run;
	proc sort data=ase;
		by METHOD &trait;
	run;
	proc means data=ase noprint;
		by METHOD &trait;
		var sa sa2;
		output out=ase2 mean=sa_mean sa2_mean;
	run;

	%if &prevalences=CUSTOM and %length(&cv_custompriors) > 0 and &cv_custompriors ne "" %then %do;
 		%let _aseid_ = %sysfunc(open(ase2));
		%let traitnum = %sysfunc(varnum(&_aseid_, &trait)); 
		%let traittype = %sysfunc(vartype(&_aseid_, &traitnum));
		%let _aseid_ = %sysfunc(close(&_aseid_));
				
		%get_priors(priors_string=%nrbquote(&cv_custompriors), traittype=&traittype);
		
		proc sort data=adjust_pr;
			by &trait;
		run;

		data ase2;
			merge ase2 adjust_pr;
			by &trait;
		run;
	%end;

	proc sort data=ase2 out=ase2;
		by METHOD;
	run;
	proc means data=ase2 noprint;
		by METHOD;
		%if &prevalences=CUSTOM and %length(&cv_custompriors) > 0 and &cv_custompriors ne "" %then %do;
			weight priors;
		%end;
		var sa_mean sa2_mean;
		output out=&output mean=MAE ASE;
	run;	

%end;

%exit:;
%mend;

********************  MACRO COMPUTE_AUC **********************************
** Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
** NAME:    COMPUTE_AUC
** AUTHOR:  Padraic Neville
** SUPPORT: Padraic Neville
** DATE:    12APR07
** PURPOSE: COMPUTE AREA-UNDER-ROC 
**
** ARGUMENTS:
**          DATA= SAS data set containing posterior probability
**          PEVENT= name of variable in &data containing event probability
**          ISEVENT= name of variable whose value = 0 unless obs is event.
**
** OUTPUT:  The macro creates Macro variable AUC 
**          containing a number between 0 and 1.
**
** BACKGROUND:
** A ROC curve is a plot of sensitivity versus 1-specificity,
** where the sensitivity is defined as the probability that a 
** test result is positive given the subject is truly diseased,
** and the specificity is defined as the probability 
** that the test result is negative given the subject is truly non-diseased. 
** Let Pevent in [0, 1] denote the predicted probability of diseased.
** Let D denode an indicator of diseased ( D =1 ) or 
** nondiseased ( D = 0 ) status. 
** Let c denode a threshold that Pevent >= considered to be diseased.
** Let F_D(c)    = Pr( Pevent <= c | D =1) 
** and F_NotD(c) = Pr( Pevent <= c | D = 0)
** denote the cumulative distribution function of Pevent for 
** diseased and non-diseased subjects. 
** Then 1- F_D(c) and F_NotD(c) are the sensitivity and specificity, 
** respectively.
** [prose adapted from Honghu Liu and Tongtong Wu]
**
** ALGORITHM:
** Notation is inspired by the SAS/STAT LOGISTIC procedure.
**
**    Sort by increasing Pevent
**
**    Loop over ordered observations creating two variables:
**       _FALNEG_  = number deceased obs before now
**       _NEG_     = number nondeceased before now
**
**    Loop over ordered observations creating four more variables:
**       _POS_     = number of deceased now and after
**       _FALPOS_  = number of nondeceased now and after
**       _SENSIT_  = _POS_/N_D          (=sensitivity)
**       _1MSPEC_  = _FALPOS_/N_NOTD    (=specificity)
**    
**    Sort by increasing _1MSPEC_
**
**    Set AUC=0.
**    Loop over ordered observations.
**       When _1MSPEC_ increases,  increase AUC by
**            _SENSIT_ times difference in _1MSPEC_ 
**    Finish by increasing AUC by 1 times (1- max _1MSPEC_).
***************************************************************************;
%macro compute_auc(data=, 
				   pevent=, 
				   isevent=);

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

proc sort data=&data out=sorted; 
	by &pevent ;
run;

***************************************************
*   Output one row for each observed pevent value,
*   plus a row of 0s at the beginning              
***************************************************;
data two;
	retain _FALNEG_ _NEG_ n_events n_not 0;
	retain is p 0;
	keep is p _FALNEG_ _NEG_;
	label _NEG_    = "No. of Correctly Predicted Nonevents";
	label _FALNEG_ = "No. of Events Predicted as Nonevents";

	set sorted end = the_end ;
	if _n_ eq 0 then output; 
	else do;
		if &pevent  ne p  then output;
	end;
	is = &isevent;
	p  = &pevent;

	if &isevent. eq 0 then do;
		n_not = n_not         + 1;
		_NEG_ = _NEG_         + 1;
	end;
	else do;
		n_events   = n_events + 1;
		_FALNEG_   = _FALNEG_ + 1;
	end;

	if the_end ne 0 then do;
		output;
		call symput("n_events", put(n_events, 8.0));
		call symput("n_not", put(n_not, 8.0));
	end;
run;

%let trSensitivity =				%nrbquote(%LocaleGet(Sensitivity_Text));
%let trSpecificity =				%nrbquote(%LocaleGet(Specificity_Text));
%let trTotalAccuracy =				%nrbquote(%LocaleGet(TotalAccuracy_Text));
%let trPositivePredictiveValue =	%nrbquote(%LocaleGet(PositivePredictiveValue_Text));
%let trNegativePredictiveValue =	%nrbquote(%LocaleGet(NegativePredictiveValue_Text));
%let trMatthewsCorrelationCoeff =	%nrbquote(%LocaleGet(MatthewsCorrelationCoefficient_Text));
%let trYoudenIndex =				%nrbquote(%LocaleGet(YoudenIndex_Text));
%let trZeroOneIndex =				%nrbquote(%LocaleGet(ZeroOneIndex_Text));	

data three;
	keep is p 
	  _POS_ _NEG_ _FALPOS_ _FALNEG_ _SENSIT_ _1MSPEC_;

	label _POS_    = "No. of Correctly Predicted Events";
	label _FALPOS_ = "No. of Nonevents Predicted as Events";
	label _SENSIT_ = "&trSensitivity";
	label _1MSPEC_ = "1 - &trSpecificity";

	set two;
	_POS_    = &n_events - _FALNEG_;
	_FALPOS_ = &n_not    - _NEG_;
	_SENSIT_ = _POS_    / &n_events;
	_1MSPEC_ = _FALPOS_ / &n_not;
run;

proc sort data=three out=four; 
     by _1MSPEC_;
run;

data auc;
	retain auc 0;
	retain prev_SENSIT_ prev_1MSPEC_ 0;
	drop   prev_SENSIT_ prev_1MSPEC_ increase;

	set four end = the_end;

	if _1MSPEC_ gt prev_1MSPEC_ then do;
		output;
		increase = _SENSIT_ - prev_SENSIT_;
		auc = auc + (prev_SENSIT_  + 0.5 * increase) 
		         * (_1MSPEC_ - prev_1MSPEC_); 
		prev_SENSIT_ = _SENSIT_;
		prev_1MSPEC_ = _1MSPEC_;
	end;

	if the_end ne 0 then do;
		put auc=;
		call symput("auc", put(auc, 12.10));
	end;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2013 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_CORR_INTERVAL
**  AUTHOR:  sasrdw, Russ Wolfinger
**  PURPOSE: Compute correlation between prediction and quantitative 
**           response
**       
**  MACRO PARAMETERS:
**  DATA:    Scored data set         
**  OUTPUT:  CORR output Data Set
**  Y:       Name of the response variable in the DATA= data set      
**  PREDICT: Name of the variable in the DATA= data set whose value   
**           is the prediction of the observed response               
**  MDLNAME: Model method name                                           
**  FREQ:    Frequency of observation (Optional)                      
**
**  NOTES:
************************************************************************;
%macro compute_corr_interval(data=, 
							output=, 
							y=, 
							predict=, 
							mdlname=, 
							freq=,
							weight=
							);  

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

proc corr data=&data out=cout noprint;
	var &y &predict;
	%if %length(&freq) %then %do;
		weight &freq;
	%end;
	%else %if %length(&weight) %then %do;
		weight &weight;
	%end;
run;

data _null_;
	set cout end=_last;
	if _last then call symputx('corr',&y);
run;

%if ^%length(%trim(&corr)) %then %let corr = .;

data &output;
	length METHOD $64 ITER 8 K 8;
	METHOD = "&mdlname";
	CORR = &corr;
run;

%put Correlation = &corr;

%exit:;
%mend;





%macro compute_effects(outputpath=,
					   cv_method=,
					   leave_l_out=,
					   models_comparison_output=,
					   models_effects_output=);

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

/*
data &cvoutlib..&prefix._cve_all;
	set &cvoutlib..&prefix._cve_all;
	if NAME = "" then delete;
run;
*/

%NObsVars(&cvoutlib..&prefix._cve_all);
%if &nObs and &nVars %then %do;
	/*
	proc freq data=&cvoutlib..&prefix._cve_all noprint;
		tables METHOD/out=fmethod(rename=(COUNT=OVERALL_COUNT));    
	run;

	%let _fmid_ = %sysfunc(open(work.fmethod));
	%let n_models = %sysfunc(attrn(&_fmid_, NOBS));
	%let mthnum = %sysfunc(varnum(&_fmid_, METHOD));
	*/
	%do nm=1 %to &n_models;
		/*
		%let rc = %sysfunc(fetchobs(&_fmid_, &nm));
		%let mdlname&nm = %sysfunc(getvarc(&_fmid_, &mthnum));
		*/
		%let mdlname = &&mdlname&nm;
		data &cvoutlib..cv_m&nm._vcum;
			set &cvoutlib..&prefix._cve_all;
			where METHOD="&mdlname";
		run;
	%end;
	/*
	%let _fmid_ = %sysfunc(close(&_fmid_));
	*/
	
	%if &cv_method = LEAVEOUT %then %do;
		proc freq data=&cvoutlib..&prefix._cve_all noprint;
			tables METHOD/out=fmethod(rename=(COUNT=OVERALL_COUNT));                                                                           
			tables NAME/out=fname(keep=NAME COUNT);    
		run; 
		%if &leave_l_out = 1 %then %do;
			%let lo_count = &__nobs;
		%end;
		%else %if &leave_l_out = 2 %then %do;
			%let lo_count = %sysevalf(&__nobs * (&__nobs-1), int);
		%end;
		%else %if &leave_l_out = 3 %then %do;
			%let lo_count = %sysevalf(&__nobs * (&__nobs-1) * (&__nobs-2), int);
		%end;
		data fname;
		    set fname;
		    OVERALL=COUNT/(&lo_count. * &n_models.);
		    drop COUNT;
		run;
	%end;
	%else %do;
		proc freq data=&cvoutlib..&prefix._cve_all noprint;
			tables METHOD/out=fmethod(rename=(COUNT=OVERALL_COUNT));                                                                           
			tables NAME/out=fname(keep=NAME COUNT);    
			tables METHOD*K*ITER/out=namedim;
			tables K*ITER/out=methdim;
		run; 
		%let _ndimid_ = %sysfunc(open(work.namedim));
		%let _ndimObs_ = %sysfunc(attrn(&_ndimid_, NOBS));
		%let _ndimid_ = %sysfunc(close(&_ndimid_));
		data fname;
		    set fname;
		    OVERALL=COUNT/&_ndimObs_;
		    drop COUNT;
		run;
		%let _mdimid_ = %sysfunc(open(work.methdim));
		%let _mdimObs_ = %sysfunc(attrn(&_mdimid_, NOBS));
		%let _mdimid_ = %sysfunc(close(&_mdimid_));
	%end;

	%global AllMdlImp_Exist;
	%let AllMdlImp_Exist = 1;

	%do nm=1 %to &n_models;
		%global ImpExist_M&nm;
		%let mdlname=&&mdlname&nm;
	    data c;
	        set fmethod;        
	        keep OVERALL_COUNT METHOD;
	        where METHOD="&mdlname";
	    run;

		%NObsVars(c);
		%let _cObs_ = &nObs;
		%if &_cObs_ %then %do;

			%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
				%NObsVars(&cvoutlib..cv_m&nm._vcum);
				%let _vcumObs_ = &nObs;
				%if &_vcumObs_ %then %do;
				    proc freq data=&cvoutlib..cv_m&nm._vcum noprint;
				        tables NAME/out=f;
				    run;
				    data f;
				        length METHOD $64;
				        set f;
				        METHOD="&mdlname";
				    run;
				    data m;
				        keep NAME AVECOUNT_M&nm COUNT;
				        merge f c;
				        by METHOD;
						%if &cv_method = LEAVEOUT %then %do;
							AVECOUNT_M&nm=COUNT/(&lo_count.);
							label AVECOUNT_M&nm="&mdlname Model Count/&lo_count";
						%end;
						%else %do;
							AVECOUNT_M&nm=COUNT/&_mdimObs_;
							label AVECOUNT_M&nm="&mdlname Model Count/&_mdimObs_";
						%end;
						rename COUNT=COUNT_M&nm;
						label COUNT="&mdlname Model Count";
				    run;

					**************************************************;
					data cv_m&nm._vcum;
						set &cvoutlib..cv_m&nm._vcum;
						if IMPORTANCE ne . then output;
					run;
					%nObsVars(cv_m&nm._vcum);

					%if &nObs %then %do;
						%let ImpExist_M&nm = 1;
						proc means data=&cvoutlib..cv_m&nm._vcum noprint;
							class NAME;
							var IMPORTANCE;
							output out=si sum=SUMIMP;
						run;
						
						data i;
							keep NAME AVEIMP_M&nm;
							merge si f;
							by NAME;
							if NAME = "" then delete;
							%if &cv_method = LEAVEOUT %then %do;
								AVEIMP_M&nm = SUMIMP/(&lo_count.);
							%end;
							%else %do;
								AVEIMP_M&nm = SUMIMP/&_mdimObs_;
							%end;
							label AVEIMP_M&nm = "&mdlname Average Importance";
						run;				 

						data m;
							keep NAME AVECOUNT_M&nm COUNT_M&nm AVEIMP_M&nm;
							merge m i;
							by NAME;
						run;
					%end;
					%else %do;
						%let AllMdlImp_Exist = 0;
						%let ImpExist_M&nm = 0;
					%end;
					**************************************************;
				%end;
			%end;
		%end;
		
		%if %sysfunc(exist(m)) = 1 %then %do;
			%NObsVars(m);
			%let _mObs_ = &nObs;
			%if &_mObs_ %then %do;
				%if %sysfunc(exist(&cvoutlib..&models_effects_output)) = 1 %then %do;
					%NObsVars(&cvoutlib..&models_effects_output);	
					%let _effObs_ = &nObs;
					%if &_effObs_ %then %do;
						proc sort data=&cvoutlib..&models_effects_output;
							by NAME;
						run;
						proc sort data=m;
							by NAME;
						run;
						data &cvoutlib..&models_effects_output;
				        	merge 	
								&cvoutlib..&models_effects_output
								m;
							by NAME;
						run;
					%end;
					%else %do;
						proc sort data=fname;
							by NAME;
						run;
						proc sort data=m;
							by NAME;
						run;
						%NObsVars(fname);
						%if &nObs and &nVars %then %do;
							data &cvoutlib..&models_effects_output;
								merge fname m;
								by NAME;
							run;
						%end;
						%else %do;
							data &cvoutlib..&models_effects_output;
								set m;
							run;
						%end;
					%end;
				%end;
				%else %do;
					proc sort data=fname;
						by NAME;
					run;
					proc sort data=m;
						by NAME;
					run;
					%NObsVars(fname);
					%if &nObs and &nVars %then %do;
						data &cvoutlib..&models_effects_output;
							merge fname m;
							by NAME;
						run;
					%end;
					%else %do;
						data &cvoutlib..&models_effects_output;
							set m;
						run;
					%end;
				%end; 
			%end;
		%end;
	%end;

	* REPLACE MISSING VALUE WITH 0;
	%if %sysfunc(exist(&cvoutlib..&models_effects_output)) = 1 %then %do;
		%NObsVars(&cvoutlib..&models_effects_output);	
		%if &nObs and &nVars %then %do;
			data &cvoutlib..&models_effects_output;
			   set &cvoutlib..&models_effects_output;
			   %do nm=1 %to &n_models;
			    	%let mdlname = &&mdlname&nm;
			      %let mdltrunc = %substr(&mdlname,1,%sysfunc(min(%length(&mdlname),32)));
			      %let mdllabel = &&mdllabel&nm;
			      if AVECOUNT_M&nm = . then AVECOUNT_M&nm=0;
					if COUNT_M&nm = . then COUNT_M&nm=0;
					%if &&ImpExist_M&nm = 1 %then %do;
						if AVEIMP_M&nm=. then AVEIMP_M&nm=0;
					%end;
					%if &AllMdlImp_Exist = 1 %then %do;
						&mdltrunc = AVEIMP_M&nm;
					%end;
					%else %do;
						&mdltrunc = AVECOUNT_M&nm;
					%end;
					label &mdltrunc="&mdllabel";
			   %end; 
				%if &AllMdlImp_Exist = 1 %then %do;
					OVERALL = ( AVEIMP_M1 
					%do nm=2 %to &n_models;
						%str( + AVEIMP_M&nm )
					%end;
					)/&n_models;
				%end;
			   * CREATE JMP HIERARCHICAL CLUSTERING ASENDING ORDER VARIABLE; 
			   ORDER=-OVERALL;
			run;
			proc sort data=&cvoutlib..&models_effects_output;
			    by descending OVERALL;
			run;
		%end;
	%end;

%end;

%mend;

%macro compute_survival_auc(data=, 
							output=,
							TimeVar=,
							CenVar=);

proc sort data=&data;
	by descending _obs;
run;

%if %upcase(&TimeVar)=SURVIVAL %then %let SurvivalVar = Survival2;
%else %let SurvivalVar = Survival;

data &output; 
	set &data; 
	by descending _obs;
	retain _AUC_ _MedSurTime_ _mIndex_ ;
	_dT_ = &TimeVar - lag(&TimeVar);
	_PreSurvival_= lag(&SurvivalVar);
	_Area_ = _dT_*_PreSurvival_;
	if first._obs then do;
		_mIndex_ = 0;
		_MedSurTime_ = .;
		_AUC_ = 0;
	end;
	else do;
		if ^_mIndex_ then do;
			if &SurvivalVar <= 0.5 | last._obs then do;
				* all survival rate above 0.5 then assign the maximum time to be med-time;
				_MedSurTime_ = &TimeVar;
				_mIndex_ = 1;
			end;
		end; 
		_AUC_ = _AUC_ + _Area_;
	end;
	if last._obs;
	label _MedSurTime_="Median Survival Time" _AUC_="Area Under Survival Curve";
	/* keep descending _obs _MedSurTime_ _AUC_ 
		 Org_&TimeVar &CenVar; */
	rename Org_&TimeVar = &TimeVar;
run;

%mend;


%macro compute_harrellc(data=,
						output=,
						TimeVar=,
						CenVar=,
						Predict=,
						Freq=
						);

	* Harrell C-statistics using _AUC_ as the predictive value;
	proc harrell data=&data out=&output;
		time &TimeVar;
		%if %length(&CenVar) %then %do;
			censor &CenVar;
		%end;
		* predict _AUC_;
		predict &Predict;
		%if %length(&Freq) %then %do;
			freq &Freq;
		%end;
	run;

%mend;


********************  MACRO COMPUTE_MANN**********************************
** Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
** NAME:    COMPUTE_MANN
** AUTHOR:  Padraic Neville
** SUPPORT: Padraic Neville
** DATE:    13APR07
** PURPOSE: COMPUTE MANN-WHITNEY STATISTIC
**
** STATUS:  Use for comparing AUC stats.
**
** ARGUMENTS:
**          DATA= SAS data set containing posterior probability
**          PEVENT= name of variable in &data containing event probability
**          ISEVENT= name of variable whose value = 0 unless obs is event.
**
** OUTPUT:  The macro creates Macro variable mann 
**          containing a number between 0 and 1.
**
** BACKGROUND:
** 
** Given Xi, i = 1, ..., Nx,
**   and Yj, j = 1, ..., Ny,
**
** Define MANN = sum over i  sum over j of  I( Xi < Yj) /  (Nx * Ny)
**                             plus 1/2 of  I( Xi = Yj) /  (Nx * Ny)
** 
** where I(true) = 1, I(false) = 0.
**
** 0 <= MANN <= 1.
** MANN is interpreted as the probability of X < Y,
** and is equivalent to the area-under-the ROC curve.
** Ties are subtle.
** 
***************************************************************************;
%macro compute_mann(data=, 
					pevent=, 
					isevent=);

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data DOES NOT EXIST.;
	%goto exit;
%end;

data sorted;
	set &data;
	if missing(&pevent) or missing(&isevent) then delete;
run;

%NObsVars(sorted);
%if &nobs=0 %then %do;
	%let mann = .;
	%goto exit;
%end;

proc sort data=sorted nodupkey;
	by &isevent;
run;

%NObsVars(sorted);
%if &nobs^=2 %then %do;
	%let mann = .;
	%goto exit;
%end;

ods exclude all;
ods noresults;

proc logistic data=&data;
	model &isevent(event='1') = ;
	roc "m_1" pred=&pevent;
	ods output rocassociation=ra;
	where not missing(&pevent);
run;

/*
ods exclude none;
ods results;
*/

data _null_;
	set ra;
	if rocmodel="m_1" then call symputx("mann",area);
run;

/*
proc sort data=sorted; 
	by &pevent &isevent;
run;

proc freq data=sorted noprint; 
	tables &isevent / out=f;
run;

data one;
	retain not_now events_now p 0;

	set sorted end=the_end;
	keep not_now events_now p;

	if _n_  > 1 then do;
		if &pevent > p then do;
			output;
			not_now    = 0;
			events_now = 0;
		end;
	end;
	if &isevent. eq 0 then do
		not_now = not_now  + 1;
	end;
	else do;
		events_now = events_now + 1;
	end;

	p = &pevent;

	if the_end ne 0 then do;
		output;
	end;
run;

data three;
	retain n_not n_events mm mann 0;
	retain total_not total_events 0;
	keep n_not n_events mann;
	set f  one end = the_end ;
	if _n_ eq 1 then total_not = count;
	else if _n_ eq 2 then do;
        total_events = count;
        mm = total_not * total_events;
    end;
	else do;
		if events_now > 0 then do;
			mann = mann + ((n_not + 0.5*not_now)*events_now)/mm;
		end;
		n_not    = n_not    + not_now;
		n_events = n_events + events_now;
		output;
	end;

	if the_end ne 0 then do;
		put mann=;
		call symput("mann", put(mann, 12.10)); 
	end;
run;
*/

%exit:;

%mend;

/* COMPUTE AVERAGE Survival Distribution Function, 
		           SDF Lower 95.00% confidence, 
				   SDF Upper 95.00% Confidence	*/
%macro compute_mean_survfunc(cvoutlib=,
							 trait=,
							 cenvar=,
							 cenval=,
							 models_comparison_output=);

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));
%put prefix=&prefix;

%if %sysfunc(exist(&cvoutlib..&prefix._cvr_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cvr_all);
	%if ^&nObs or ^&nVars %then %do;
		%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS.;
		%let exiterror = 1;
		%goto exit;		
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._cvr_all DOES NOT EXIST.;
	%goto exit;
%end;

* RETRIEVE TEST SETS PREDS;
data cvr_test;
	set &cvoutlib..&prefix._cvr_all;
	if _train_ = 0 then output;
run;

%NObsVars(cvr_test);
%if ^&nObs or ^&nVars %then %do;
	%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS. FOR TEST PREDITIONS;
	%let exiterror = 1;
	%goto exit;		
%end;

* RETRIEVE MODEL NAMES, ITERATION NUMBERS;
proc freq data=cvr_test noprint;
	tables METHOD/out=_nmdl;
	tables ITER/out=_niter;
	tables METHOD*ITER/out=_nmi;
run;

%NObsVars(_nmdl);
%let n_models=&nObs;
%let _mid_ = %sysfunc(open(_nmdl));
%let mthnum = %sysfunc(varnum(&_mid_, METHOD));
%do m=1 %to &n_models;
	%let rc = %sysfunc(fetchobs(&_mid_, &m));
	%let mdlname&m = %sysfunc(getvarc(&_mid_, &mthnum));
%end;
%let _mid_ = %sysfunc(close(&_mid_));

%NObsVars(_niter);
%let n_iters=&nObs;
%let _iid_ = %sysfunc(open(_niter));
%let inum = %sysfunc(varnum(&_iid_, ITER));
%do i=1 %to &n_iters;
	%let rc = %sysfunc(fetchobs(&_iid_, &i));
	%let iter&i = %sysfunc(getvarn(&_iid_, &inum));
%end;
%let _iid_ = %sysfunc(close(&_iid_));

* REPLACE CHARACTER CENSOR VARIABLE WITH NUMERIC; 
%let ctid = %sysfunc(open(cvr_test));
%let cenum = %sysfunc(varnum(&ctid, &cenvar));
%let centype = %sysfunc(vartype(&ctid, &cenum));
%let ctid = %sysfunc(close(&ctid));
%if &centype = C %then %do;     
	data cvr_test;               
		set cvr_test;             
		length &cenvar.n 8;       
		&cenvar.n=&cenvar;        
		drop &cenvar;             
		rename &cenvar.n=&cenvar; 
	run;
%end;

* COMPUTE BASELINE KAPLAN-MEIER ESTIMATE;   
proc lifetest data=cvr_test outsurv=basesurf noprint; 
	 %if %length(%trim(%left(&CenVar))) %then %do;   
    	 time Org_&trait * &cenvar(&cenval);  
    %end;
    %else %do;
    	 time Org_&trait;
    %end;
	 where trim(left(METHOD)) = trim(left("&mdlname1"));
run;    
data basesurf;                                            
	set basesurf;                                         
	if _CENSOR_ = 1 then delete;                          
	drop _CENSOR_ ;     
	rename SURVIVAL = BASELINE_SURVIVAL;
	label SURVIVAL = "Baseline Survival Function"; 
	rename SDF_LCL = BASELINE_SDF_LCL;
	label SDF_LCL = "Baseline SDF Lower 95.00% Confidence";
	rename SDF_UCL = BASELINE_SDF_UCL;
	label SDF_UCL = "Baseline SDF Upper 95.00% Confidence";
	rename Org_&trait = &trait;
	label Org_&trait = "&trait";
run;                                                          
 
* COMPUTE AVERAGE SURVIVAL FUNCTION FOR EACH MODEL;           
proc means data=cvr_test noprint;              
   var Survival stdErrSurvival;                
   by METHOD;
	class &trait;                               
   output out=meansurf(drop=_TYPE_ _FREQ_)     
   mean(Survival)=Survival                     
	mean(Survival)=MeanSurvival
	mean(StdErrSurvival)=MeanStdErrSurvival;    
run;                              
data meansurf;
	set meansurf;
	if &trait=. then delete;
run; 
%do m=1 %to &n_models;
	%let mdlname = &&mdlname&m;
	data meansurf_m&m;
		set meansurf;
		drop METHOD;
		if trim(left(METHOD))=trim(left("&mdlname")) then output;
		rename MeanSurvival = M&m._Survival;
		label MeanSurvival = "&mdlname Survival Function";
		rename MeanStdErrSurvival = M&m._StdErrSurvival;
		label MeanStdErrSurvival = "&mdlname Survival Standard Error";
	run;
%end;
 
* TO DO: COMPUTE SDF_LCL, SDF_UCL LATER;
* VARIABLE NAMES, LABELS CONSISTENT;
data &cvoutlib..&prefix.meansurf;
	merge basesurf 
	%do m=1 %to &n_models;
		meansurf_m&m
	%end;
	;
	by &trait;
	drop Survival;
run;

/* 
* COMPUTE BASELINE KAPLAN-MEIER ESTIMATE FOR EACH ITERATION; 
proc sort data=cvr_test out=cvr;                             
    by ITER;                                                 
run;   

 *REPLACE CHARACTER CENSOR VARIABLE TO NUMERIC; 
%let cvrid = %sysfunc(open(cvr));
%let cenum = %sysfunc(varnum(&cvrid, &cenvar));
%let centype = %sysfunc(vartype(&cvrid, &cenum));
%let cvrid = %sysfunc(close(&cvrid));
%if &centype = C %then %do;                                  
	data cvr;                                                 
		set cvr;                                               
		length &cenvar.n 8;                                    
		&cenvar.n=&cenvar;                                     
		drop &cenvar;                                          
		rename &cenvar.n=&cenvar;                              
	run;                                                      
%end;

proc lifetest data=cvr outsurv=basesurf noprint;    
	by ITER; 
    time Org_&trait * &cenvar(&cenval);                      
run;    
data basesurf;
	set basesurf;
	if _CENSOR_ = 1 then delete;  
run;
%do i=1 %to &n_iters;
	%let iter = &&iter&i;
	%put iter=&iter;
	data basesurf_i&i;
		set basesurf;
		if _CENSOR_ = 1 then delete;                                      
		if ITER=&iter then output;
		drop _CENSOR_;
		rename SURVIVAL = BASELINE_I&i._SURVIVAL;
	    label SURVIVAL = "Baseline_I&i Survival Function";
		rename SDF_LCL = BASELINE_I&i._SDF_LCL;
		label SDF_LCL = "Baseline_I&i SDF Lower 95.00% Confidence";
		rename SDF_UCL = BASELINE_I&i._SDF_UCL;
		label SDF_UCL = "Baseline_I&i SDF Upper 95.00% Confidence";
		rename Org_&trait = &trait;
		label Org_&trait = "&trait";
	run;  
%end;
 
* COMPUTE AVERAGE SURVIVAL FUNCTION FOR EACH MODEL;        
proc means data=cvr_test noprint;                          
	var Survival stdErrSurvival;                            
	by METHOD ITER;
	class &trait;                                           
	output out=meansurf(drop=_TYPE_ _FREQ_)                 
	mean(Survival)=Survival                                 
	mean(Survival)=MeanSurvival
	* CHANGE AVE StdErrSurvival COMPUTATION LATER;
	mean(StdErrSurvival)=MeanStdErrSurvival;                
run;                              
data meansurf;
	set meansurf;
	if &trait=. then delete;
run; 
%do m=1 %to &n_models;
	%let mdlname = &&mdlname&m;
	%do i=1 %to &n_iters;
		%let iter = &&iter&i;
		data meansurf_m&m._i&i;
			set meansurf;
			drop METHOD;
			if trim(left(METHOD))=trim(left("&&mdlname&m")) and 
			   ITER=&iter then output;
			rename MeanSurvival = M&m._I&i._Survival;
			label MeanSurvival = "&mdlname._&i Survival Function";
			rename MeanStdErrSurvival = M&m._I&i._StdErrSurvival;
			label MeanStdErrSurvival = "&mdlname._&i Survival Standard Error";
		run;
	%end;
%end;

* TO DO: COMPUTE SDF_LCL, SDF_UCL LATER;
* VARIABLE NAMES, LABELS CONSISTENT;
data &cvoutlib..&prefix.meansurf;
	merge 
	%do i=1 %to &n_iters;
		basesurf_i&i
	%end;
	%do m=1 %to &n_models;
		%do i=1 %to &n_iters;
			meansurf_m&m._i&i
		%end;
	%end;
	;
	by &trait;
	drop Survival ITER;
run;
*/

%exit:;
%mend; 

********************  MACRO COMPUTE_ROC**********************************
** Copyright (c) 2009 by SAS Institute Inc., Cary, NC USA 27513  
** NAME:    COMPUTE_ROC
** AUTHOR:  Russ Wolfinger
** SUPPORT: Russ Wolfinger
** DATE:    08AUG09
** PURPOSE: COMPUTE ROC STATISTICS
**
** STATUS:  Intend to use for comparing ROC stats.
**
** ARGUMENTS:
**          DATA= SAS data set containing posterior probabilities
**          PEVENT= name of variable in &data containing event probability
**          ISEVENT= name of variable whose value = 0 unless obs is event.
**
** OUTPUT: The macro creates an output data set containing various stats.
** 
***************************************************************************;
%macro compute_roc(data=, isevent=, pevent=, outdata=roc);

	data sorted;
		set &data;
		_one = 1;
		if missing(&pevent) or missing(&isevent) then delete;
	run;

	%NObsVars(sorted);
	%if (&nobs = 0) %then %do;
		%put ROC Statistics are not computed because input data set has no nonmissing observations.;
		data &outdata;
			set sorted;
		run;
	%end;
	%else %do;
		%let trSensitivity =				%nrbquote(%LocaleGet(Sensitivity_Text));
		%let trSpecificity =				%nrbquote(%LocaleGet(Specificity_Text));
		%let trTotalAccuracy =				%nrbquote(%LocaleGet(TotalAccuracy_Text));
		%let trPositivePredictiveValue =	%nrbquote(%LocaleGet(PositivePredictiveValue_Text));
		%let trNegativePredictiveValue =	%nrbquote(%LocaleGet(NegativePredictiveValue_Text));
		%let trMatthewsCorrelationCoeff =	%nrbquote(%LocaleGet(MatthewsCorrelationCoefficient_Text));
		%let trYoudenIndex =				%nrbquote(%LocaleGet(YoudenIndex_Text));
		%let trZeroOneIndex =				%nrbquote(%LocaleGet(ZeroOneIndex_Text));
	
		 /*
	 	 ods exclude all;
	 	 ods noresults;
	 	 proc logistic data=sorted;
			model &isevent (event='1') = &pevent / outroc=outroc;
		 run;
		 ods exclude none;
		 ods results;

		 * get totals;
		 data _null_;
		 	set outroc(obs=1);
			call symput('n_events', trim(left(_falneg_+1)));
			call symput('n_not', trim(left(_neg_)));
			call symput('n_total',trim(left(_neg_+_falneg_+1)));
		 run;

		 * compute additional stats;
		 data &outdata;
			retain maxacc 0;
			label _NEG_    = 'True Negatives';
			label _FALNEG_ = 'False Negatives';
			label _POS_    = 'True Positives';
			label _FALPOS_ = 'False Positives';
			label _SENSIT_ = "&trSensitivity";
			label _SPEC_   = "&trSpecificity";
			label _1MSPEC_ = "1 - &trSpecificity";
			label _ACC_    = "&trTotalAccuracy";
			label _PPV_    = "&trPositivePredictiveValue";
			label _NPV_    = "&trNegativePredictiveValue";
			label _MCC_    = "&trMatthewsCorrelationCoeff";
			label _J_      = "&trYoudenIndex";
			label _01_     = "&trZeroOneIndex";
			label _PROB_   = 'P_Event';

			set outroc;
			_SPEC_   = 1 - _1MSPEC_;
			_ACC_    = (_POS_ + _NEG_)/(&n_events + &n_not);
			if (_ACC_ > maxacc) then do;
				maxacc = _ACC_;
				call symput('maxacc_n_',trim(left(_n_)));
			end;
			_PPV_    = _POS_ / (_POS_ + _FALPOS_);
			_NPV_    = _NEG_ / (_NEG_ + _FALNEG_);
			_J_      = _SENSIT_ + _SPEC_ - 1;
			_01_     = 1 - sqrt((1-_SENSIT_)**2 + (1-_SPEC_)**2);
			mccden = (_POS_+_FALNEG_)*(_POS_+_FALPOS_)*(_NEG_+_FALPOS_)*(_NEG_+_FALNEG_);
			if (mccden > 0) then _MCC_ = ((_POS_*_NEG_)-(_FALPOS_*_FALNEG_))/sqrt(mccden);
			else _MCC_ = .;
			drop maxacc mccden;
			rename _PROB_ = P_Event;
		 run;
		 */
	 	 	
		 proc sort data=sorted; 
			by &pevent &isevent;
		 run;
		 		 
		 * take means and transpose to handle ties;
		 proc means data=sorted noprint;
		 	by &pevent &isevent;
		 	var _one;
		 	output out=sorted1 n=n1;
		 run;
		 
		 proc transpose data=sorted1 out=sorted2 prefix=freq;
		 	by &pevent;
		 	var _freq_;
		 	id &isevent;
		 run;
		 
		 data sorted1;
		 	set sorted2;
		 	if missing(freq0) then freq0 = 0;
		 	if missing(freq1) then freq1 = 0;
		 	drop _name_;
		 run;
		 			 
		 * first pass to compute negatives and total;
		 data sorted1;
			retain _FALNEG_ _NEG_ n_events n_not n_total 0;

			label _NEG_    = 'True Negatives';
			label _FALNEG_ = 'False Negatives';

			set sorted1 end=the_end;
			
			if _n_ = 1 then output;

			n_not = n_not + freq0;
			_NEG_ = _NEG_ + freq0;
			n_events = n_events + freq1;
			_FALNEG_ = _FALNEG_ + freq1;
			n_total = n_total + freq0 + freq1;
			output;
			
			if the_end ne 0 then do;
				call symput('n_events', trim(left(n_events)));
				call symput('n_not', trim(left(n_not)));
				call symput('n_total',trim(left(n_total)));
			end;
		 run;

		 * second pass, each value is computed assuming the cutoff equals pevent at the current observation;
		 data &outdata;
			retain maxacc 0;
			label _POS_    = 'True Positives';
			label _FALPOS_ = 'False Positives';
			label _SENSIT_ = "&trSensitivity";
			label _SPEC_   = "&trSpecificity";
			label _1MSPEC_ = "1 - &trSpecificity";
			label _ACC_    = "&trTotalAccuracy";
			label _PPV_    = "&trPositivePredictiveValue";
			label _NPV_    = "&trNegativePredictiveValue";
			label _MCC_    = "&trMatthewsCorrelationCoeff";
			label _J_      = "&trYoudenIndex";
			label _01_     = "&trZeroOneIndex";

			set sorted1;
			_POS_    = &n_events - _FALNEG_;
			_FALPOS_ = &n_not - _NEG_;
			_SENSIT_ = _POS_ / &n_events;
			_1MSPEC_ = _FALPOS_ / &n_not;
			_SPEC_   = 1 - _1MSPEC_;
			_ACC_    = (_POS_ + _NEG_)/(&n_events + &n_not);
			if (_ACC_ > maxacc) then do;
				maxacc = _ACC_;
				call symput('maxacc_n_',trim(left(_n_)));
			end;
			_PPV_    = _POS_ / (_POS_ + _FALPOS_);
			_NPV_    = _NEG_ / (_NEG_ + _FALNEG_);
			_J_      = _SENSIT_ + _SPEC_ - 1;
			_01_     = 1 - sqrt((1-_SENSIT_)**2 + (1-_SPEC_)**2);
			mccden = (_POS_+_FALNEG_)*(_POS_+_FALPOS_)*(_NEG_+_FALPOS_)*(_NEG_+_FALNEG_);
			if (mccden > 0) then _MCC_ = ((_POS_*_NEG_)-(_FALPOS_*_FALNEG_))/sqrt(mccden);
			else _MCC_ = .;
			drop maxacc mccden;
		 run;

		 proc sort data=&outdata;
			* by &pevent _SPEC_;
			by _1MSPEC_ _SENSIT_ descending &pevent;
		 run;

	%end;

%mend;


***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    COMPUTE_PVALUE
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    12/13/07
**  PURPOSE: Compute pevent or predicted trait values when an 
**           importance variable is removed. 
**  HISTORY: action                                        date     who
**           Created.                                      12/13/07 pyt 
** 
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  WHERE_CLAUSE: Where clause
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  EVENT: Trait event level
**  IMPORTANCE_VAR: An importance variable name 
**  MODEL_NAMES:  List of model names
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**	VARIMPORTANCE_OUTPUT: Pevent or predicted trait values output:
**				  Pvalue or DiffPvalue
**
**  NOTES: 
**		Single model_names and importance_var is required. 
************************************************************************;
%macro compute_pvalue(datapath=,
					  outputpath=,
					  dsname=,
					  where_clause=,
					  trait=,
					  trait_type=,
					  event=,
					  importance_var=,
					  model_names=,
					  models_comparison_output=,
					  importance_output=);

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* CHECK FOR &cvoutlib..&prefix._cvr_all DATA SET EXIST;
%if %sysfunc(exist(&cvoutlib..&prefix._cvr_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cvr_all);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._cvr_all DOES NOT EXIST.;
	%goto exit;
%end;

* CHECK FOR &cvoutlib..metadata_variables DATA SET EXIST;
%if %sysfunc(exist(&cvoutlib..metadata_variables)) = 1 %then %do;
	%NObsVars(&cvoutlib..metadata_variables);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..metadata_variables CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..metadata_variables DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
    libname datalib "&datapath";
   %let datalib = datalib;
%end;
%else %do;
    %let datalib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cpdsname = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

* CREATE MODELS FROM ENTIRE TRAINING DATA SET,
  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH METHOD: 
  CV_<METHODNAME>_PREDS AND CV_<METHODNAME>_VARS, 
  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
%let wc=%superq(where_clause);
data train;
    set
    %if %length(&wc) > 0 %then %do;
        &__cpdsname(where=(%unquote(&wc)))
    %end;
    %else %do;
        &__cpdsname
    %end;
    ;
    _Obs = _n_;
    %if &trait_type = qualitative %then %do;
        %str(if &trait ne " " then output;);
	%end;
    %else %do;
        %str(if &trait ne . then output;);
	%end;
run;
%NObsVars(train);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
	%if %length(&wc) > 0 %then %do;
		%put ERROR: &__cpdsname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do;
		%put ERROR: &__cpdsname DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
	%end;
	%goto exit;
%end;

%if %sysfunc(exist(&cvoutlib..&prefix._vir_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._vir_all);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._vir_all CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._vir_all DOES NOT EXIST.;
	%goto exit;
%end;

data pvar;
	set &cvoutlib..&prefix._cvr_all;
	keep P_:;
	if _n =1 then output;
run;	
%let _dsid_ = %sysfunc(open(pvar));
%let _nclasses_ = %sysfunc(attrn(&_dsid_, NVARS));
%if %upcase(&trait_type) = QUALITATIVE %then %do; 

	* RETRIEVE P_: VARIABLE NAMES AND MAPPED TRAIT CLASSES;	
	%do v=1 %to &_nclasses_;
		%let p_var&v = %sysfunc(varname(&_dsid_, &v));
		%let p_label&v = %sysfunc(varlabel(&_dsid_, &v));
		%let class&v = %scan("&&p_var&v",-1,"_");
	%end;

	* RETRIEVE P_BASELINE: VALUES WHICH CORRECTLY MAPPED WITH 
	  ASSOCIATED TRAIT CLASSES;	
	proc freq data=train noprint;
		 tables &trait/out=ft;
	run;
	proc sort data=ft;
	    by PERCENT;
	run;	
	%let _fid_ = %sysfunc(open(ft));
	%let _fnobs_ = %sysfunc(attrn(&_fid_, NOBS));
	%let traitnum=%sysfunc(varnum(&_fid_, &trait));
	%let traitype=%sysfunc(vartype(&_fid_, &traitnum));
	%let pctnum=%sysfunc(varnum(&_fid_, PERCENT));
	%do v=1 %to &_nclasses_; 
		%let class=&&class&v;
		%do i=1 %to &_fnobs_;
			%let rc= %sysfunc(fetchobs(&_fid_, &i));
			%if &traitype = C %then %do;
				%let tmpc=%sysfunc(getvarc(&_fid_,&traitnum)); 
			%end;
			%else %do;
				%let tmpc=%sysfunc(getvarn(&_fid_,&traitnum)); 
			%end;
			%if %trim(%left("&tmpc"))=%trim(%left("&class")) %then %do;
				%let p_baseline&v=%sysevalf(%sysfunc(getvarn(&_fid_,&pctnum))/100);
			%end;
		%end;
	%end;
	%let _fid_ = %sysfunc(close(&_fid_));
%end;
%else %do;
	%let p_var = %sysfunc(varname(&_dsid_, 1));
	%let p_label = %sysfunc(varlabel(&_dsid_, 1));
	%let nclasses = 0;
	proc means data=train mean;
		var &trait;
		output out=mean;
	run;
	data _null_;
		set mean;
		if _STAT_ = "MEAN" then 
			call symput("predicted_value_baseline", trim(left(put(&trait, best12.))));
	run; 
%end;
%let _dsid_ = %sysfunc(close(&_dsid_));

* RETRIEVE MODEL NAMES;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);
%do i=1 %to &n_models;
    %global mdlname&i;
%end;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
%end;
%let mdlname=&mdlname1;

data _null_;
	set &cvoutlib..metadata_variables;
	if NAME="&importance_var" or LABEL="&importance_var" then do;
		call symput("DROP_NAME", trim(left(NAME)));
		call symput("DROP_LABEL", trim(left(LABEL)));
		call symput("INPUT_TYPE", trim(left(MODEL)));
	end;
run;

data preds_vir;
	set &cvoutlib..&prefix._vir_all;
	where DROPVAR="&DROP_NAME";
run;
%if %sysfunc(exist(preds_vir)) = 1 %then %do;
	%NObsVars(preds_vir);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._vir_all DOES NOT CONTAIN DROPVAR=&DROP_NAME.;
		%goto exit;
	%end;
%end;
%else %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._vir_all DOES NOT CONTAIN DROPVAR=&DROP_NAME.;
		%goto exit;
%end;
proc sort data=preds_vir out=preds_vir;
	by _Obs;
run;

data var;
	set train;
	keep _Obs &DROP_NAME;
run;

%if &importance_output = Pvalue %then %do;
	data &cvoutlib..&importance_output;
		merge preds_vir var;
		by _Obs;
		keep &DROP_NAME	P_: &trait _Obs METHOD ITER K;
		%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
			label &DROP_NAME = "&DROP_LABEL";
		%end;
	run;
%end;
%else %if &importance_output ne Pvalue %then %do;
	proc sort data=&cvoutlib..&prefix._cvr_all out=preds_full;
		by _Obs;
	run;
	
	data preds_full;
		merge preds_full var;
		by _Obs;
		keep &DROP_NAME P_: _Obs ITER K ;
		%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
			label &DROP_NAME = "&DROP_LABEL";
		%end;
		%if %upcase(&trait_type) = QUALITATIVE %then %do; 
			%do v=1 %to &_nclasses_;
				%let pvar=&&p_var&v;
				%let plabel=&&p_label&v;
				rename &pvar=&pvar._FULL;
				label &pvar="FULL &plabel";
			%end;
		%end;
		%else %do;
			rename &p_var=&p_var._FULL;
			label &p_var="FULL &p_label";
		%end;
	run;  

	proc sort data=preds_full out=preds_full;
		by ITER K _Obs;
	run;

	data &cvoutlib..&importance_output;
		merge preds_vir preds_full;
		%if %upcase(&trait_type) = QUALITATIVE %then %do; 
			%do v=1 %to &_nclasses_;
				%let pvar=&&p_var&v;
				%let plabel=&&p_label&v;
				D_&pvar = Abs(&pvar - &pvar._FULL);
				label D_&pvar = "AbsDiff &plabel";
			%end;
		%end;
		%else %do;
			D_&p_var = Abs(&p_var - &p_var._FULL);
			label D_&p_var = "AbsDiff &p_label";
		%end;
	run;
%end; 

filename jslfile "&outputpath.imp_plot.jsl";
data _null_;
	file jslfile encoding="utf-8";
    put "//!";
    put "Names Default to Here(1);";
    put "// parameters for variable importance plots;";
	%if %upcase(&trait_type) = QUALITATIVE %then %do; 
		put "nclasses = &_nclasses_;";
		%do c=1 %to &_nclasses_;
			%if &c=1 %then %do;
				%let trait_classes = &&class&c;
				%let p_baselines = &&p_baseline&c;
				%let p_vars = &&p_var&c;
				%let p_labels = &&p_label&c;
			%end;
			%else %do;
				%let trait_classes = &trait_classes,&&class&c;
				%let p_baselines = &p_baselines,&&p_baseline&c;
				%let p_vars = &p_vars,&&p_var&c;
				%let p_labels = &p_labels,&&p_label&c;
			%end;
		%end;
		put "trait_classes = ""&&trait_classes"";";
		put "p_baselines = ""&&p_baselines"";";
		put "p_vars = ""&&p_vars"";";
		put "p_labels = ""&&p_labels"";";
	%end;
	%else %do;
		put "nclasses = .;";
		put "p_var = ""&p_var"";";
		put "p_label = ""&p_label"";";
		put "predicted_value_baseline = &predicted_value_baseline;";
	%end;
	put "input_type = ""&input_type"";";
	put "model_name = ""&mdlname"";";
	put "if(Host Is(Windows),";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
run;

%exit:;
%mend;

%macro compute_whiskers(data=,
				        output=,
						method=,
					    freq=,
						varlist=);
			
* RETRIEVE VARIABLE NAMES;
%let i=1;
%do %while(%length(%scan(&varlist,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_vars = %eval(&i-1);
%do i=1 %to &n_vars;
    %global varname&i;
%end;
%do i=1 %to &n_vars;
    %let varname&i = %scan(&varlist,&i," ");
%end;
	
proc means data=&data noprint;
	freq &freq;
	var &varlist;
	output out=&output
	%do i=1 %to &n_vars;
		%let varname = &&varname&i;
		   median(&varname)=&varname._median 
	       q1(&varname)=&varname._p25 
   	       q3(&varname)=&varname._p75 
           qrange(&varname)=&varname._qrange
	%end;
	;
run;

%do i=1 %to &n_vars;

	%let varname = &&varname&i;
	%put varname = &varname;

	data _null_;
		set &output;
		call symputx("p25", &varname._p25);
		call symputx("p75", &varname._p75);
		call symputx("uwhisker", put(&varname._p75 + 1.5 * &varname._qrange, best12.));
		call symputx("lwhisker", put(&varname._p25 - 1.5 * &varname._qrange, best12.));
	run;
	
	%if ^%length(&p25) %then %let p25 = .;
	%if ^%length(&p75) %then %let p75 = .;
	%if ^%length(&uwhisker) %then %let uwhisker = .;
	%if ^%length(&lwhisker) %then %let lwhisker = .;
	
	%put p25 = &p25;
	%put p75 = &p75;
	%put uwhisker = &uwhisker;
	%put lwhisker = &lwhisker;

	proc sort data=&data;
		by &varname;
	run;

	data _null_;
		set &data end=end;
		retain uw &p75;
		retain lw &p25;
		if &lwhisker <= &varname and &varname <= &p25 then do;
			if &varname < lw then 
				lw = &varname;
		end;
		if &p75 <= &varname and &varname <= &uwhisker then do;
			if &varname > uw then 
				uw = &varname;
		end; 
		if end ne 0 then do;
			call symputx("uw", uw);
			call symputx("lw", lw);
		end; 
	run;
	
	%if ^%length(&uw) %then %let uw = .;
	%if ^%length(&lw) %then %let lw = .;
	
	%put uw = &uw;
	%put lw = &lw;

	data &output;
		set &output;
		length METHOD $64;
		&varname._UWHISKER = &uw;
		&varname._LWHISKER = &lw;
		%if &i = 1  %then %do;
			METHOD="&method";
			drop _TYPE_;
			rename _FREQ_ = NOBS;
		%end;
		drop &varname._p25 &varname._p75 &varname._qrange; 
	run;
%end;

%mend;

%macro classes_event(namedat=);

%let _nid_=%sysfunc(open(&namedat));
%let nclasses=%sysfunc(attrn(&_nid_, NOBS));
%if &nclasses > 0 %then %do;
	%let classnum=%sysfunc(varnum(&_nid_, CLASS));
	%let classtype=%sysfunc(vartype(&_nid_, &classnum));
	%let eventnum=%sysfunc(varnum(&_nid_, EVENT));
	%let eventype=%sysfunc(vartype(&_nid_, &eventnum));
	%do c=1 %to &nclasses;
        %let rc=%sysfunc(fetchobs(&_nid_, &c));
		%if &c=1 %then %do;
			%if &eventype=C %then %do;
				%let event=%sysfunc(getvarc(&_nid_, &eventnum)); 
			%end;
			%else %do;
				%let event=%sysfunc(getvarn(&_nid_, &eventnum));
			%end;	
		%end;
		%global class&c;
		%if &classtype=C %then %do;
			%let class&c=%sysfunc(getvarc(&_nid_, &classnum)); 
		%end;
		%else %do;
			%let class&c=%sysfunc(getvarn(&_nid_, &classnum));
		%end;
	%end;
%end;
%let _nid_ = %sysfunc(close(&_nid_));

%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: CROSS_VALIDATION  
**  AUTHOR:  saspyt, Pei-Yi Tan 
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/20/07
**  PURPOSE: Cross validation 
**  HISTORY: action                                        date     who
**           Created.                                      02/20/07 pyt 
**			 Added leave-l-out cv method				   06/22/07 pyt
**           Replace cv_seed with seedsequence, seedget    06/15/07 pyt
** 
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  WHERE_CLAUSE: Where clause
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**				  survival (only applicable for survival data)
**  CENVAR:	  	  Censor variable (only applicable for survival data)
**  CENVAL:		  Censor value indicates if the record is censored	
**  ID:			  Survival data id variable
**  GROUPVAR:	  Survival data group variable for coloring survival curves
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  CV_HOLDOUT_ITERATION_START:	Random hold-out iteration starting index
**  CV_HOLDOUT_ITERATION_END: Random hold-out iteration ending index
**  CV_PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities
**  CV_CUSTOMCOSTS: Custom cost functions
**  CV_ALOSSPROP: Asymmetic loss evaluation proportion
**  CV_SEED: Random number seed
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  KFOLD: K-fold cross validation 
**  BLOCKNAME: K-fold block partition variable name 
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  MODELS_EFFECTS_OUTPUT: Models effects frequencies output 
**  VARIMPORTANCE: Flag to indicate performing variable importance  
**	 VARIMPORTANCE_OUTPUT: Variable importance results output
**  PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Shared grid folder - client version
**  SERVERGSHARE: Shared grid folder - server version
**
**  NOTES:
*   Define random seeds sequence:
*      CV HOLDOUT LOOP of &cv_holdout_iteration ITERATIONS. USES THREAD=1; 
************************************************************************;
%macro cross_validation(datapath=,
						outputpath=,
						plot_outputpath=,
						dsname=,
						where_clause=,
						trait=,
						trait_type=,
						cenvar=,
						cenval=,
						id=,
						groupvar=,
						freq=,
						model_names=,
						model_labels=,
						model_label_list=,
						cv_method=,      
						cv_holdout_percent=,
						cv_holdout_iteration_start=,
						cv_holdout_iteration_end=,
						cv_prevalences=,
						cv_custompriors=,
						cv_customcosts=,
						cv_alossprop=,
						cv_seed=,
						cv_vars_outds=,     
						cvname=,         
						kfold=,   
						groupname=,
						blockname=,
						leave_l_out=, 
						models_comparison_output=,
						models_effects_output=,
						varimportance=,
						varimportance_output=,
					   	prevalences=proportional,
						ngridnodes=,
						clientgshare=,
						servergshare=
						);

* options nonotes;
%put START CV MODEL COMPARISON;
%put cross_validation groupname = &groupname;

* ASSIGN GLOBALS;
%global LS_HOME BPV;

* ERROR CHECKING;
%let cv_method = %upcase(&cv_method);

/*
%if %upcase(&trait_type)=QUANTITATIVE and &cv_method=STRATIFIED %then %do; 
    %put ERROR: Stratified Random hold outs are not available with a continuous dependent variable.;
    %let exiterror = 1;
    %goto cv_exit;
%end;
*/
        
* RETRIEVE MODEL NAMES;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);
%do i=1 %to &n_models;
    %global mdlname&i;
%end;
%let PropHazardsFlag= 0;
%let OtherModelsFlag= 0;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
    %if %index(&&mdlname&i, Proportional_Hazards) %then %do;
    	%let PropHazardsFlag= 1;
    %end;
    %else %do;
    	%let OtherModelsFlag= 1;
    %end;
%end;

%put PropHazardsFlag= &PropHazardsFlag;
%put OtherModelsFlag= &OtherModelsFlag;
%if &PropHazardsFlag and &OtherModelsFlag %then %do;
	%put ERROR: Cross validation cannot be run when Proportional Hazards Regression is specified among another other models in the Models tab.;		
    	%let exiterror= 1;
    	%goto cv_exit;
%end;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT;
* AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&clientgshare) %then %do;
	%let cv_datapath = &datapath;
	%let datapath = &clientgshare;
%end;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cvdsname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto cv_exit;
%end;

%if %sysfunc(exist(&__cvdsname)) = 1 %then %do;
	%NObsVars(&__cvdsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__cvdsname CONTAINS 0 OBSERVATIONS.;
		%goto cv_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__cvdsname DOES NOT EXIST.;
	%goto cv_exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&clientgshare) %then %do;
	%let cv_outputpath = &outputpath;
	%let outputpath = &clientgshare;
%end;

%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%let cv_customcosts = %trim(&cv_customcosts);
%let cv_custompriors = %trim(&cv_custompriors);

* CREATE MODELS FROM ENTIRE TRAINING DATA SET,
  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH METHOD: 
  CV_<METHODNAME>_PREDS AND CV_<METHODNAME>_VARS, 
  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;

* KEEP TRAIT WITH NON-MISSING VALUES;  
%let _tid_ = %sysfunc(open(&__cvdsname));
%let _tObs_ = %sysfunc(attrn(&_tid_, NOBS));
%if &_tObs_ %then %do;
	%let traitnum=%sysfunc(varnum(&_tid_, &trait));
	%if ^&traitnum %then %do;
		%put ERROR: Variable &trait not found in cross validation input data set &__cvdsname..;
		%let exiterror = 1;
		%goto cv_exit;
	%end;
	%let traitype=%sysfunc(vartype(&_tid_, &traitnum));
%end;
%let _tid_ = %sysfunc(close(&_tid_));

* KEEP TRAIT WITH NON-MISSING VALUES AND APPLY SPECIFIED WHERE_CLAUSE; 
%let wc=%superq(where_clause); 
data train;
	set &__cvdsname;
	__id = _n_;
	%if &traitype = C %then %do; 
		if &trait ne " " 
		%if %length(&wc) > 0 %then %do;
			and %unquote(&wc)
		%end;
		then output;
	%end;
	%else %do;
		if &trait ne . 
		%if %length(&wc) > 0 %then %do;
			and %unquote(&wc)
		%end;
		then output;
	%end;
run;

* FOR SURVIVAL DATA; 
%put cenval=&cenval;
%if %upcase(&trait_type)=SURVIVAL %then %do;
	* IF CENSOR VALUE IS NOT SPECIFIED, SET DEFAULT =1;
	%if ^%length(&cenval) %then %do;
		%let cenval = 1;
	%end;
	%else %do;
		* IF CENSOR VALUE CONTAINS QUOTES, REMOVE QUOTES;
		%let char1 = %bquote(%substr(&cenval,1,1));
		%if %bquote(&char1) = %str(%") %then %do;
			%let qslen=%length(&cenval);
			%let cenval=%bquote(%substr(&cenval,2,%eval(&qslen-2)));
			%put cenval=&cenval;
		%end;
	%end;
%end;


%NObsVars(train);
%let _trainObs_ = &nObs;
%if ^&_trainObs_ %then %do;
	%let exiterror = 1;
	%if %length(&wc) > 0 %then %do;
		%put ERROR: &__cvdsname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do;
		%put ERROR: &__cvdsname DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
	%end;
	%goto cv_exit;
%end;

%if %upcase(&trait_type) = QUALITATIVE %then %do; 
	proc freq data=train noprint;
	    tables &trait/out=ft;
	run;
    proc sort data=ft;
        by PERCENT;
    run;
	
	* RETRIEVE NUMBER OF TRAIT CLASSES AND TRAIT EVENT;
	%let _fid_ = %sysfunc(open(ft));
	%let _nclasses_ = %sysfunc(attrn(&_fid_, NOBS));
	%let _fid_ = %sysfunc(close(&_fid_));
    %do c=1 %to &_nclasses_;
        %global actualp&c;
    %end;

    %global mann auc;
    
    %if ^%length(&cv_method) or &cv_method = "" %then %do;
		%let cv_method=STRATIFIED;
	%end;
%end;
%else %do;
    %if ^%length(&cv_method) or &cv_method = "" %then %do;
		%let cv_method=RANDOM;
	%end;
%end;

%if &cv_method = LEAVEOUT %then %do;

	data &cvoutlib..train;
		set train;
	run;

	%put Starting leave_l_out;

	%leave_l_out(datapath=&outputpath,
				 outputpath=&outputpath,
				 dsname=train,
				 model_names=&model_names,
				 trait=&trait,
				 trait_type=&trait_type,
				 leave_l_out=&leave_l_out,
				 prevalences=&cv_prevalences,
				 cv_custompriors=%nrbquote(&cv_custompriors),
				 cv_customcosts=%nrbquote(&cv_customcosts),
				 cv_alossprop=&cv_alossprop,
				 ngridnodes=&ngridnodes,
				 clientgshare=&clientgshare,
				 servergshare=&servergshare
				 );

	proc datasets library=&cvoutlib nolist;
		delete train;
	quit;

	%if %length(&clientgshare) %then %do;
  	    %let datapath = &cv_datapath;
  	    %let outputpath = &cv_outputpath;
  	%end;
  	
  	%put Starting merge_llo_results;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%if &ngridnodes > 0 %then %do;
			data &cvoutlib.._namedat;
				set &cvoutlib.._namedat1;
			run;
			proc datasets library=&cvoutlib nolist;
				delete
				%do node=1 %to &ngridnodes;
					_namedat&node
				%end;
				;
			quit;

			%let _nid_=%sysfunc(open(&cvoutlib.._namedat));
			%let nclasses=%sysfunc(attrn(&_nid_, NOBS));
			%if &nclasses > 0 %then %do;
				%let classnum=%sysfunc(varnum(&_nid_, CLASS));
				%let classtype=%sysfunc(vartype(&_nid_, &classnum));
				%let eventnum=%sysfunc(varnum(&_nid_, EVENT));
				%let eventype=%sysfunc(vartype(&_nid_, &eventnum));
				%do c=1 %to &nclasses;
			        %let rc=%sysfunc(fetchobs(&_nid_, &c));
					%if &c=1 %then %do;
						%if &eventype=C %then %do;
							%let event=%sysfunc(getvarc(&_nid_, &eventnum)); 
						%end;
						%else %do;
							%let event=%sysfunc(getvarn(&_nid_, &eventnum));
						%end;	
					%end;
					%if &classtype=C %then %do;
						%let class&c=%sysfunc(getvarc(&_nid_, &classnum)); 
					%end;
					%else %do;
						%let class&c=%sysfunc(getvarn(&_nid_, &classnum));
					%end;
				%end;
			%end;
			%let _nid_ = %sysfunc(close(&_nid_));
		%end;
	%end;
	%else %do;
		%let nclasses=;
		%let event=;
	%end;

	%merge_llo_results(outputpath=&outputpath,
					  __nobs=&_trainObs_,
					   trait=&trait,
					   trait_type=&trait_type,
					   nclasses=&nclasses,
					   event=&event,
					   freq=&freq,
					   model_names=&model_names,
					   model_labels=&model_labels,
					   n_models=&n_models,
					   leave_l_out=&leave_l_out,
					   models_comparison_output=&models_comparison_output,
					   models_effects_output=&models_effects_output,
				  	   prevalences=&cv_prevalences,
					   cv_custompriors=%nrbquote(&cv_custompriors),
				      gshare=&clientgshare
				  	   );
%end;
%else %do;
	*VALIDATION CV PARAMETERS;
	%if %length(&cv_holdout_iteration_start) = 0 %then %do;
		%let cv_holdout_iteration_start = 1;
	%end;
	%if %length(&cv_holdout_iteration_end) = 0 %then %do;
		%put ERROR: NUMBER OF RANDOM CV HOLD-OUT ITERATIONS IS UNKNOWN.;
		%let exiterror = 1;
		%goto cv_exit;
	%end;
	%if %length(&cv_method) = 0 or &cv_method = "" %then %do;
	    %put ERROR: CV_METHOD IS UNKNOWN.;
		 %let exiterror = 1;
	    %goto cv_exit;
	%end;
	%if &cv_method=RANDOM or &cv_method=STRATIFIED or &cv_method= "" %then %do;
		%if (&cv_holdout_percent <= 0) %then %do;
			%put WARNING: CV_HOLDOUT_PERCENT < or = 0.;
		        %*put ERROR: CV_HOLDOUT_PERCENT < or = 0.;
		        %*let exiterror = 1;
        		%*goto cv_exit;
    		%end;
		%if &kfold ne 1 %then %do;
			%put WARNING: SIMPLE RANDOM AND STRATIFIED RANDOM HOLD-OUT METHODS K-FOLD SHOULD ALWAYS BE 1.;
			%put REPLACE K-FOLD FROM &kfold TO 1.;
			%let kfold = 1;
		%end;
	%end;
	%else %if &cv_method=BLOCK or &cv_method=SPLIT %then %do;
		/*
	    * THESE MEAN NOT DOING RANDOM CROSS VALIDATION, THEREFORE CV ITERATION SHOULD ALWAYS = 1; 
		%if &cv_holdout_iteration_end ne 1 %then %do;
			%put WARNING: SPLIT PARTITION AND BLOCK PARTITION HOLD-OUT METHODS CV ITERATION SHOULD ALWAYS BE 1.;
			%put REPLACE CV ITERATION FROM &cv_holdout_iteration_end TO 1.;
		%end;
		*/
	    %let cv_holdout_iteration_start = 1;
	    %if %length(&blockname) > 0 and &blockname ne "" %then %do;
	    	proc freq data=&__cvdsname noprint;
			tables &blockname/out=Blocks(keep=&blockname);
		run;
		
		%NObsVars(Blocks);
	    	%let cv_holdout_iteration_end = &nobs;
	    %end;
	    %else %do;
	    	%let cv_holdout_iteration_end = &kfold;
	    %end;
	    %let kfold = 1;
	%end;

	* CHECK FOR THE SIZE OF KFOLD, IF KFOLD > NOBS, REDUCE IT TO NOBS;
	%if &kfold > &_trainObs_ %then %do;
		%put WARNING: SIZE OF KFOLD > TRAINING DATA SET SIZE;
		%put RESETTING kfold TO &_trainObs_; 
	    %let kfold = &_trainObs_;
	%end;

	* DEFINE SEEDS FOR;
    *     A CV HOLDOUT LOOP of &cv_holdout_iteration ITERATIONS. USES THREAD=1; 
    %global randseed;
	data _null_;
		n=&cv_holdout_iteration_end;
		call symput("nseed", n);
	run;
	
	%seedsequence(n=&nseed, seedds=&cvoutlib..seedseq, nthreads=1, seqseed=&cv_seed);

	data &cvoutlib..train;
		set train;
	run;
	
	%put Starting cv_iterations;
	%put iter_start = &cv_holdout_iteration_start;
	%put iter_end = &cv_holdout_iteration_end;
	%put;
	
	%cv_iterations(datapath=&outputpath,
				   outputpath=&outputpath,
				   dsname=train,
				   model_names=&model_names,
				   n_models=&n_models,
				   trait=&trait,
				   trait_type=&trait_type,
				   cv_method=&cv_method,
				   cv_holdout_percent=&cv_holdout_percent,
				   cv_holdout_iteration_start=&cv_holdout_iteration_start,
				   cv_holdout_iteration_end=&cv_holdout_iteration_end,
				   cv_seed=&cv_seed,
				   cv_vars_outds=&cv_vars_outds,
				   cvname=&cvname,
				   kfold=&kfold,
				   groupname=&groupname,
				   blockname=&blockname,
				   varimportance=&varimportance,
				   prevalences=&cv_prevalences,
				   cv_custompriors=%nrbquote(&cv_custompriors),
				   cv_customcosts=%nrbquote(&cv_customcosts),
				   cv_alossprop=&cv_alossprop,
				   ngridnodes=&ngridnodes,
				   clientgshare=&clientgshare,
				   servergshare=&servergshare
				   );

	proc datasets library=&cvoutlib nolist;
		delete train;
	quit;
	
	%if %length(&clientgshare) %then %do;
		%let datapath = &cv_datapath;
		%let outputpath = &cv_outputpath;
	%end;

	%put Starting merge_cv_results;
	%put iter_start = &cv_holdout_iteration_start;
	%put iter_end = &cv_holdout_iteration_end;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%if &ngridnodes > 0 %then %do;
			data &cvoutlib.._namedat;
				set &cvoutlib.._namedat1;
			run;
			proc datasets library=&cvoutlib nolist;
				delete
				%do node=1 %to &ngridnodes;
					_namedat&node
				%end;
				;
			quit;

			%let _nid_=%sysfunc(open(&cvoutlib.._namedat));
			%let nclasses=%sysfunc(attrn(&_nid_, NOBS));
			%if &nclasses > 0 %then %do;
				%let classnum=%sysfunc(varnum(&_nid_, CLASS));
				%let classtype=%sysfunc(vartype(&_nid_, &classnum));
				%let eventnum=%sysfunc(varnum(&_nid_, EVENT));
				%let eventype=%sysfunc(vartype(&_nid_, &eventnum));
				%do c=1 %to &nclasses;
			        %let rc=%sysfunc(fetchobs(&_nid_, &c));
					%if &c=1 %then %do;
						%if &eventype=C %then %do;
							%let event=%sysfunc(getvarc(&_nid_, &eventnum)); 
						%end;
						%else %do;
							%let event=%sysfunc(getvarn(&_nid_, &eventnum));
						%end;	
					%end;
					%if &classtype=C %then %do;
						%let class&c=%sysfunc(getvarc(&_nid_, &classnum)); 
					%end;
					%else %do;
						%let class&c=%sysfunc(getvarn(&_nid_, &classnum));
					%end;
				%end;
			%end;
			%let _nid_ = %sysfunc(close(&_nid_));
		%end;
	%end;
	%else %do;
		%let nclasses=;
		%let event=;
	%end;
	
	* options notes;

	* EXECUTE CODE BELOW AFTER ALL ITERATIONS ARE RUN;
	* 1. COMPUTE EFFECTS FREQUENCIES;
	* 2. COMPUTE OVERALL CV ITERATIONS MODELS FIT STATISTICS: ASE, ACC, AUC;
	* 	 MERGE ALL MODELS OVERALL ASE, ACC, AND AUC;
	* 3. COMPUTE EACH CV ITERATION MODELS ASE, ACC, AUC;
	*    MERGE ALL CV ITERATIONS MODELS AUC, ACC, ASE, AND OVERALL BASELINE ASE;
	%merge_cv_results(outputpath=&outputpath,
					  n_models=&n_models,
					  model_names=&model_names,
					  model_labels=&model_labels,
					  trait=&trait,
					  trait_type=&trait_type,
					  nclasses=&nclasses,
					  event=&event,
					  freq=&freq, 
					  cenvar=&cenvar,
					  cv_method=&cv_method,
				     	  cv_holdout_iteration_start=&cv_holdout_iteration_start,
					  cv_holdout_iteration_end=&cv_holdout_iteration_end,
					  kfold=&kfold,
					  models_comparison_output=&models_comparison_output,
					  models_effects_output=&models_effects_output,
					  varimportance=&varimportance,
					  varimportance_output=&varimportance_output,
					  prevalences=&cv_prevalences,
					  cv_custompriors=%nrbquote(&cv_custompriors),
					  gshare=&clientgshare
					  );
	%if &exiterror = 1 %then 
		%goto cv_exit;
	
	* options notes;
	
	%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;
		* SAVE BASELINE ESTIMATES MEDIAN, UPPER AND LOWER WHISKERS IN GLOBAL MACRO VARIABLES;
		%global baseline_rmse_median baseline_rmse_uwhisker baseline_rmse_lwhisker
			    baseline_accuracy_median baseline_accuracy_uwhisker baseline_accuracy_lwhisker
			    baseline_aec_median baseline_aec_uwhisker baseline_aec_lwhisker
				baseline_auc_median baseline_auc_uwhisker baseline_auc_lwhisker
				baseline_harrell_median baseline_harrell_uwhisker baseline_harrell_lwhisker
				baseline_corr_median baseline_corr_uwhisker baseline_corr_lwhisker
				;
		%if %sysfunc(exist(&cvoutlib..baseline_pctl)) = 1 %then %do;
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				%if &nclasses = 2 %then %do;
					data _null_;
						set &cvoutlib..baseline_pctl;
						call symput("baseline_auc_median", trim(left(put(AUC_MEDIAN, best12.))));
						call symput("baseline_auc_uwhisker", trim(left(put(AUC_UWHISKER, best12.))));
						call symput("baseline_auc_lwhisker", trim(left(put(AUC_LWHISKER, best12.))));
					run;
				%end;
				data _null_;
					set &cvoutlib..baseline_pctl;
					call symput("baseline_accuracy_median", trim(left(put(Accuracy_MEDIAN, best12.))));
					call symput("baseline_accuracy_uwhisker", trim(left(put(Accuracy_UWHISKER, best12.))));
					call symput("baseline_accuracy_lwhisker", trim(left(put(Accuracy_LWHISKER, best12.))));
				run;
				%if %length(&cv_customcosts) %then %do;
					data _null_;
						set &cvoutlib..baseline_pctl;
						call symput("baseline_aec_median", trim(left(put(AEC_MEDIAN, best12.))));
						call symput("baseline_aec_uwhisker", trim(left(put(AEC_UWHISKER, best12.))));
						call symput("baseline_aec_lwhisker", trim(left(put(AEC_LWHISKER, best12.))));
					run;
				%end;
			%end;
			%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
				data _null_;
					set &cvoutlib..baseline_pctl;
					call symput("baseline_harrell_median", trim(left(put(HARRELL_MEDIAN, best12.))));
					call symput("baseline_harrell_uwhisker", trim(left(put(HARRELL_UWHISKER, best12.))));
					call symput("baseline_harrell_lwhisker", trim(left(put(HARRELL_LWHISKER, best12.))));
					call symput("baseline_corr_median", trim(left(put(CORR_MEDIAN, best12.))));
					call symput("baseline_corr_uwhisker", trim(left(put(CORR_UWHISKER, best12.))));
					call symput("baseline_corr_lwhisker", trim(left(put(CORR_LWHISKER, best12.))));
				run;
				%if ^%length(&baseline_corr_median) %then %let baseline_corr_median = .;
				%if ^%length(&baseline_corr_uwhisker) %then %let baseline_corr_uwhisker = .;
				%if ^%length(&baseline_corr_lwhisker) %then %let baseline_corr_lwhisker = .;
			%end;
			data _null_;
				set &cvoutlib..baseline_pctl;
				call symput("baseline_rmse_median", trim(left(put(RMSE_MEDIAN, best12.))));
				call symput("baseline_rmse_uwhisker", trim(left(put(RMSE_UWHISKER, best12.))));
				call symput("baseline_rmse_lwhisker", trim(left(put(RMSE_LWHISKER, best12.))));
			run;
		%end;
	%end;
%end;

* RETRIEVE NUMBER OF ITERATIONS;
proc freq data=&cvoutlib..&models_comparison_output	noprint;
	tables ITER/out=iter(keep=ITER);
run;
%NObsVars(iter);
%let cv_holdout_iterations=&nObs;  
%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* GENERATE RELIABILITY DIAGRAM RESULTS TABLE;
%if %upcase(&trait_type)=SURVIVAL %then %do;
	%reliability_diagram(cvoutlib=&cvoutlib, 
			  			 trait=&trait,
			  		     trait_type=&trait_type,
						 cenvar=&cenvar,
						 cenval=&cenval,
			  			 models_comparison_output=&models_comparison_output);
	
	%compute_mean_survfunc(cvoutlib=&cvoutlib, 
			  			   trait=&trait,
						   cenvar=&cenvar,
						   cenval=&cenval,
			  			   models_comparison_output=&models_comparison_output);
	%let idlabel = ;
	%if %length(&id) %then %do;
		%CheckLabel(&cvoutlib..&prefix._spm3all,id,_new=idlabel);
		%put id = &id;
		%put idlabel = &idlabel;
	%end;
	%if %length(&groupvar) %then %do;
		%CheckLabel(&cvoutlib..&prefix._spm3all,groupvar);
		%put groupvar = &groupvar;
	%end;
%end;
%else %do;
	%reliability_diagram(cvoutlib=&cvoutlib, 
			  			 trait=&trait,
			  		     trait_type=&trait_type,
			  			 models_comparison_output=&models_comparison_output);
%end;

* save _namedat for drill down;
%if %upcase(&trait_type)=QUALITATIVE %then %do;
	/*S1460881: Saving _namedat with unique suffix for each input data and trait to avoid 
	error when analyzing multiple traits in the Predictive Modeling Review*/
	/*data &cvoutlib.._jl_namedat;*/
	data &cvoutlib..&prefix._cvc;
		set &cvoutlib.._namedat;
	run;
%end;

* JSL PLOTTING CODE, launched from ExitAndRetrieveLogLstFromJMP in UtilityMacros.sas;
%if ^%length(&plot_outputpath) %then %let plot_outputpath = &outputpath;
%if ^%symexist(LabelVar) %then %let LabelVar = ; 
filename jslfile "&outputpath.&mcPrefix._cvmc.jsl";
data _null_;
	file jslfile encoding="utf-8";
	put "//!";
	put "Names Default to Here(1);";
	put "// parameters for cross-validation model comparison plots;";
	put "datapath=""&datapath"";";
	put "outputpath=""&plot_outputpath"";";
	put "dsname=""&dsname"";";
	put "where_clause=""&where_clause"";";
	put "models_comparison_output=""&models_comparison_output"";";
	put "models_effects_output = ""&models_effects_output"";";
	put "cv_method = ""&cv_method"";";
	put "cv_holdout_iterations = &cv_holdout_iterations;"; 
	put "cv_holdout_iteration_start = &cv_holdout_iteration_start;";
	put "cv_holdout_iteration_end = &cv_holdout_iteration_end;";
	put "cv_prevalences = ""&cv_prevalences"";";
	put "cv_custompriors = ""&cv_custompriors"";";
	put "cv_customcosts = ""&cv_customcosts"";";
	put "cv_alossprop = ""&cv_alossprop"";";
	put "kfold = &kfold;";
	put "leave_l_out = &leave_l_out;";
	put "cvname = ""_CV_"";";
	put "model_names = ""&model_names"";";
	put "model_label_list = &model_label_list;";
	put "labelvar = ""&LabelVar"";";
	put "trait = ""&trait"";";
	put "trait_type = ""&trait_type"";";
	%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;
		%if &cv_method ne LEAVEOUT %then %do;
			put "baseline_rmse_median = &baseline_rmse_median;";
			put "baseline_rmse_uwhisker = &baseline_rmse_uwhisker;";
			put "baseline_rmse_lwhisker = &baseline_rmse_lwhisker;";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				%if &nclasses = 2 %then %do;
					put "baseline_auc_median = &baseline_auc_median;";
					put "baseline_auc_uwhisker = &baseline_auc_uwhisker;";
					put "baseline_auc_lwhisker = &baseline_auc_lwhisker;";
				%end;
				put "baseline_accuracy_median = &baseline_accuracy_median;";
				put "baseline_accuracy_uwhisker = &baseline_accuracy_uwhisker;";
				put "baseline_accuracy_lwhisker = &baseline_accuracy_lwhisker;";
				%if %length(&cv_customcosts) %then %do;
					put "baseline_aec_median = &baseline_aec_median;";
					put "baseline_aec_uwhisker = &baseline_aec_uwhisker;";
					put "baseline_aec_lwhisker = &baseline_aec_lwhisker;";
				%end;
			%end;
			%else %do;
				put "baseline_harrell_median = &baseline_harrell_median;";
				put "baseline_harrell_uwhisker = &baseline_harrell_uwhisker;";
				put "baseline_harrell_lwhisker = &baseline_harrell_lwhisker;";
				put "baseline_corr_median = &baseline_corr_median;";
				put "baseline_corr_uwhisker = &baseline_corr_uwhisker;";
				put "baseline_corr_lwhisker = &baseline_corr_lwhisker;";
			%end;
		%end;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
	    	put "nclasses = &nclasses;";
			put "event = ""&event"";";
			%do c=1 %to &nclasses;
				%if &c=1 %then %do;
					%let trait_classes = &&class&c;
				%end;
				%else %do;
					%let trait_classes = &trait_classes,&&class&c;
				%end;
			%end;
			put "trait_classes = ""&&trait_classes"";";
			/*put "_namedat = ""_jl_namedat"";";*/
			put "_namedat = ""&prefix._cvc"";";
		%end;
		%else %do;
			put "nclasses = .;";
			put "event = "" "";";
			put "trait_classes= "" "";";
			put "_namedat = "" "";";
		%end;
		put "cenvar = "" "";";
		put "cenval= "" "";"; 
		put "id = "" "";";
		put "groupvar = "" "";";
	%end;
	%else %do;
		put "nclasses = .;";
		put "event = "" "";";
		put "trait_classes= "" "";";
		put "_namedat = "" "";";
		put "cenvar = ""&cenvar"";";
		put "cenval = ""&cenval"";"; 
		put "id = ""&id"";";
		put "idlabel = ""&idlabel"";";
		put "groupvar = ""&groupvar"";";
	%end;
	put "prevalences = ""&prevalences"";";
	put "rdiagram_output = ""&prefix._rdiagram"";";
    put "varimportance = ""&varimportance"";";
    put "varimportance_output=""&varimportance_output"";";
    put "if(Host Is(Windows),";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
    %if &varimportance = YES %then %do;
        put "include(jslpath||""Variable Importance Fitsdiff.jsl"");"; 
	%end;
    %else %do;
    	put "include(jslpath||""Cross Validation Model Comparison.jsl"");";
	%end;
run;

%cv_exit:;

%if &exiterror = 1 %then %do;
	%if ^%length(&plot_outputpath) %then %let plot_outputpath = &outputpath;
	filename jslfile "&outputpath.&mcPrefix._cvmc.jsl";
	data _null_;
		file jslfile encoding="utf-8";
		put "//!";
		put "Names Default to Here(1);";
	run;
%end;

%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    CV_ITERATIONS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/30/08
**  PURPOSE: Cross validation iterations thru grid computing which is 
**		     called from both cross_validation and 
**           cv_learning_curve_estimates macros  
**  HISTORY: action                                       date     who
**           Created.                                     06/15/07 pyt 
**			 Grid computing						          07/30/08 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:   Library path of training data set 
**  OUTPUTPATH: Library path of output data sets 
**  DSNAME:     Input data set name
**  MODEL_NAMES:
**  N_MODELS:   Number of modeling mthods
**  TRAIT:      Trait variable name
**  TRAIT_TYPE: Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  CV_HOLDOUT_ITERATION_START:	Random hold-out iteration starting index
**  CV_HOLDOUT_ITERATION_END: Random hold-out iteration ending index
**  CV_SEED: Random number seed
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  KFOLD: K-fold cross validation 
**  GROUPNAME: Grouping variable
**  BLOCKNAME: K-fold block partition variable name 
**  VARIMPORTANCE: Flag to launch variable importance processor
**  PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**  CV_CUSTOMCOSTS: Custom cost functions
**  CV_ALOSSPROP: Asymmetric loss evaluation proportion
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: client grid share folder
**  SERVERGSHARE: server grid share folder
**
**  NOTES: It is invoked from cross_validation
**		   Invokes SEEDGET( ,THREAD=1);
**
************************************************************************;
%macro cv_iterations(datapath=,
				     outputpath=,
				     dsname=, 
				     model_names=,
				     n_models=,
				     trait=,
				     trait_type=,
					  cv_method=,
				     cv_holdout_percent=,
				     cv_holdout_iteration_start=, 
			   	  cv_holdout_iteration_end=, 
					  cv_seed=,
				     cv_vars_outds=,
					  cvname=,
				     kfold=,
				     groupname=,
					  blockname=, 
				     varimportance=, 	
					  prevalences=,
					  cv_custompriors=,
					  cv_customcosts=,
					  cv_alossprop=,
					  ngridnodes=,
					  clientgshare=,
					  servergshare=
					 );

%put Entering cv_iterations;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cvitname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__cvitname)) = 1 %then %do;
	%NObsVars(&__cvitname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__cvitname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__cvitname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;
%if %length(&clientgshare) %then %do;
	libname gout "&clientgshare";
	%let itoutlib = gout;
%end;
%else %do;
	%let itoutlib = &cvoutlib;
%end;

* RETRIEVE MODEL NAMES;
%put n_models=&n_models;
%do m=1 %to &n_models;
    %let mdlname&m = %scan(&model_names,&m," ");
%end;
%put mdlname1 = &mdlname1;

%if %upcase(&varimportance) = YES %then %do;

	%if &n_models ne 1 %then %do;
		%put ERROR: MORE THAN ONE MODEL IS SPECIFIED.;
		%let exiterror = 1;
		%goto cv_exit;
	%end;

	%let iter=0;
	%preselect_predictors(datapath=,
						  outputpath=&outputpath,
						  dsname=train,
						  model_name=&mdlname1,
						  pred_vars_dsname=vi_&mdlname1._vars
						  );
%end;

* CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
	data &itoutlib..cv_baseline_preds&iter;
		delete;
	run;
	%do nm=1 %to &n_models;
		data &itoutlib..cv_m&nm._pcum&iter;
			delete;
		run;
		data &itoutlib..cv_m&nm._vcum&iter;
			delete;
		run;
	%end;
	%if %upcase(&varimportance) = YES %then %do;
		data &itoutlib..cv_vipreds_pcum&iter;
			delete;
		run;
		data &itoutlib..cv_vivars_pcum&iter;
			delete;
		run;
		%do k=1 %to &kfold;
			data &itoutlib..importance&iter&k;
				delete;
			run;
			data &itoutlib..impfit&iter&k;
				delete;
			run;
		%end;
	%end;
%end;

%if &ngridnodes > 0 %then %do;
	
	* SET UP GRID SEQUENCE;
	%gridsequence(process=CVMC,
				  cv_holdout_iteration_start=&cv_holdout_iteration_start,
			 	  cv_holdout_iteration_end=&cv_holdout_iteration_end,
			      innerloop_start=,
			      innerloop_end=,
			      outerloop_start=,
			      outerloop_end=,
				  leave_l_out=,
				  nnodes=&ngridnodes,
		          gridds=&cvoutlib..cvgridseq);
	
	%NObsVars(&cvoutlib..cvgridseq);
	%let ngridnodes = &nObs;
	%do node=1 %to &nnodes;
		%global node_name&node;
	%end;
	%global rungridseq;
	%let rungridseq=&ngridnodes;

	%UpdateFitModels(nnodes=&ngridnodes, outputpath=&outputpath);

	%do %while (&rungridseq > 0);

		%let max_tries = 100;
		%let waitlist = ;
		%let ng = 0;

		* CHECK GRID NODES RUN_STATUS IN cvgridseq.sas7bdat;
		* FOR ANY RUN_STATUS ne 0, RSUBMIT THE CODE TO GENERATE INTERMEDIATE RESULTS;
		%do gridseq=1 %to &ngridnodes;
			%let _gseq_ = %sysfunc(open(&cvoutlib..cvgridseq));
			%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
			%let status_num = %sysfunc(varnum(&_gseq_, run_status));
			%let gndone = %sysfunc(getvarn(&_gseq_, &status_num));	
			%let _gseq_ = %sysfunc(close(&_gseq_));
			%if &gndone=0 %then %do;
	            %put REMOTE SUBMIT COMPLETE ON GNODE&gridseq;
		    %end;
 			%else %if &gndone = . or &gndone = 1 %then %do;
				* KEEP THE RERUN RECORD IN cvgrideq.sas7bdat data set;
				%if &gndone = 1 %then %do;
		        	%put REMOTE SUBMIT FAILED TO EXECUTE ON GNODE&gridseq;
				%end;
				%let ng=%eval(&ng+1);
				%let node=&ng;
				%goto run_gridseq;
			%end;
		    %else %if &gndone = 2 %then %do;
		        %put REMOTE SUBMIT IS STILL IN PROGRESS ON GNODE&gridseq;
			%end;

			%run_gridseq:
				* RETRIEVE ITER_START, ITER_END FROM cvgridseq.sas7bdat OF EACH NODE; 
				%let _gseq_ = %sysfunc(open(&cvoutlib..cvgridseq));
				%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
				%let itstart_num = %sysfunc(varnum(&_gseq_, iter_start));
				%let iter_start =  %sysfunc(getvarn(&_gseq_, &itstart_num));
				%let itend_num = %sysfunc(varnum(&_gseq_, iter_end));
				%let iter_end =  %sysfunc(getvarn(&_gseq_, &itend_num));
				%let _gseq_ = %sysfunc(close(&_gseq_));

				data &cvoutlib..cvgridseq;
					set &cvoutlib..cvgridseq;
					if grid_node = &gridseq then do;
						run_status=2;
					end;
				run;

				%let waitlist=&waitlist gnode&node;
				%let tries = 1;
				%let finished = 0;
				%do %while(^&finished);
					%put SIGN ON ATTEMPT &tries OF &max_tries TO GRID NODE &node....;
					signon gnode&node;
					%let node_name = ;
					%let node_name&gridseq = %sysfunc(grdsvc_getname(gnode&node));
					%let node_name = &&node_name&gridseq;
					%if %length(&node_name) %then %do;
						%put SUCCESSFUL SIGN ON TO GNODE&node = &node_name;
						%put RUNNING ITERATION &iter_start TO &iter_end ON GNODE&node = &node_name;
						%let finished = 1;
						* UPDATE NODE_NAME IN cvgridseq.sas7bdat;
						data &cvoutlib..cvgridseq;
							set &cvoutlib..cvgridseq;
							if grid_node = &gridseq then do;
								node_name="&node_name";
							end;
						run;		
					%end;
					%else %do;
						%let tries = %eval(&tries+1);
						%if &tries > &max_tries %then %do;
							%put ERROR: GRID SIGN ON TO GNODE&node FAILED AFTER &max_tries TRIES.;
							%let exiterror = 1;
							%let finished = 1;
							signoff _all_;
							%goto cv_exit;
						%end;
					%end;
				%end;

				%if %sysfunc(sysprod(prodnum007)) ne 0 %then %do; 
					*S1313431: SAS/CONNECT be licensed*;
					%macro ConnectVars01;
						* SYSLPUT CREATES A NEW MACRO VARIABLE OR MODIFIES THE VALUE OF  
						  AN EXISTING MACRO VARIABLE ON A REMOTE HOST OR SERVER;
						%syslput GSHARE=&servergshare;
						%syslput LS_HOME=&LS_HOME;
						%syslput GHOME=&servergshare.gnode&gridseq.%str(/);
						%syslput CLIENTGHOME=&clientgshare.gnode&gridseq.%str(/);
						%syslput USER_HOME=&servergshare.gnode&gridseq.%str(/);
						%syslput ProcessName=CVMC&gridseq; 
						%syslput datalib=datalib;
						%syslput cvoutlib=cvoutlib;
						%syslput iter_start=&iter_start;
						%syslput iter_end=&iter_end;
						%syslput datapath=&datapath;
						%syslput outpath=&servergshare;
						%syslput outputpath=&servergshare;
						%syslput dsname=&dsname;
						%syslput model_names=&model_names;
						%syslput n_models=&n_models;
						%do i=1 %to &n_models;
							%syslput mdlname&i = &&mdlname&i;
							%syslput FM_Error&i = 0;
						%end;
						%syslput trait=&trait;
						%syslput trait_type=&trait_type;
						%syslput cv_method=&cv_method;
						%syslput cv_holdout_percent=&cv_holdout_percent;
						%syslput mdlname = &mdlname1;
						%syslput cv_vars_outds=&cv_vars_outds;
						%syslput cvname=&cvname;
						%syslput kfold=&kfold;
						%syslput varimportance=&varimportance;
						%syslput groupname=&groupname;
						%syslput blockname=&blockname;
						%syslput prevalences=&prevalences;
						%syslput cv_custompriors=%nrbquote(&cv_custompriors);
						%syslput cv_customcosts=%nrbquote(&cv_customcosts);
						%syslput cv_alossprop=&cv_alossprop;
						%syslput randseed=&cv_seed;
						%if %upcase(&varimportance) = YES %then %do;
							%syslput NewPredictorVars=&NewPredictorVars;
							%syslput NewClassPredVars=&NewClassPredVars;
							%syslput NewListContPredVars=&NewListContPredVars;
							%syslput NewListClassPredVars=&NewListClassPredVars;
						%end;
						%syslput node_name=&node_name;
						%syslput node=&node;
						%syslput gridseq=&gridseq;
						%syslput gndone&gridseq = 2;
						%syslput UseGrid = 1;
					%mend;
				%end;
				%else %do;
					%macro ConnectVars01;
						%put EMPTY.;	
					%mend;								
				%end;
				%ConnectVars01;

				* RSUBMIT BLOCK OF WORK TO BE SUBMITTED TO GRID NODE;
				rsubmit cwait=no persist=no cmacvar=gndone&gridseq;

				%macro ITERATION_LOOP;

				%put rsubmit for &ProcessName;
				%put LS_HOME = &LS_HOME;
				%put gshare = &gshare;
				%put ghome = &GHOME;
				%put node number = &node;
				%put node name = &node_name;
				
				%global workpath;
				data _null_;
					call symput('workpath',pathname("work"));
				run;
				%put workpath = &workpath;
				
				* COPY DATA SETS TO WORK;
				proc upload data=inlib.&dsname out=&dsname;
				run;
				proc upload data=inlib.seedseq1 out=seedseq1;
				run;

				* CREATE SUBFOLDERS FROM JMP LIFE SCIENCES;
				%let rc = %sysfunc(dcreate(MacroLib,&workpath));
				%let rc = %sysfunc(dcreate(ProcessLibrary,&workpath));

				* COPY MACROLIB FILES;
				proc upload infile="&CLIENTGHOME.FitModels.sas" 
					outfile="&workpath./MacroLib/FitModels.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\BinaryResponseEffectSelectionMacros.sas" 
					outfile="&workpath./MacroLib/BinaryResponseEffectSelectionMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\ClassVars.sas" 
					outfile="&workpath./MacroLib/ClassVars.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\CrossValidationModelComparisonMacros.sas" 
					outfile="&workpath./MacroLib/CrossValidationModelComparisonMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\GAMacros.sas" 
					outfile="&workpath./MacroLib/GAMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\MixedModelMacros.sas" 
					outfile="&workpath./MacroLib/MixedModelMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\PredMacros.sas" 
					outfile="&workpath./MacroLib/PredMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\SeedMacros.sas" 
					outfile="&workpath./MacroLib/SeedMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\UtilityMacros.sas" 
					outfile="&workpath./MacroLib/UtilityMacros.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.MacroLib\MultipleTestingAdjustment.sas" 
					outfile="&workpath./MacroLib/MultipleTestingAdjustment.sas" connectstatus=no;
				run;

				* COPY PROCESSLIBRARY FILES;
				proc upload infile="&LS_HOME.ProcessLibrary\DiscriminantAnalysis.sas" 
					outfile="&workpath./ProcessLibrary/DiscriminantAnalysis.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\DistanceScoring.sas" 
					outfile="&workpath./ProcessLibrary/DistanceScoring.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\GLMSelect.sas" 
					outfile="&workpath./ProcessLibrary/GLMSelect.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\KNearestNeighbors.sas" 
					outfile="&workpath./ProcessLibrary/KNearestNeighbors.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\LogisticRegression.sas" 
					outfile="&workpath./ProcessLibrary/LogisticRegression.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\PartitionTrees.sas" 
					outfile="&workpath./ProcessLibrary/PartitionTrees.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\PartialLeastSquares.sas" 
					outfile="&workpath./ProcessLibrary/PartialLeastSquares.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\RadialBasisMachine.sas" 
					outfile="&workpath./ProcessLibrary/RadialBasisMachine.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\RidgeRegression.sas" 
					outfile="&workpath./ProcessLibrary/RidgeRegression.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\SurvivalPredictiveModeling.sas" 
					outfile="&workpath./ProcessLibrary/SurvivalPredictiveModeling.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\GenomicBLUP.sas" 
					outfile="&workpath./ProcessLibrary/GenomicBLUP.sas" connectstatus=no;
				run;
				proc upload infile="&LS_HOME.ProcessLibrary\GenomicBayesianRegression.sas" 
					outfile="&workpath./ProcessLibrary/GenomicBayesianRegression.sas" connectstatus=no;
				run;				

				* LOAD MACROS DEFINED WORK FOLDER;
				filename gmacro "&workpath./MacroLib";  
				%if ^%sysfunc(fexist(gmacro)) %then %do;
					%put ERROR: Unable to load grid macros from &workpath on &ghome.. ;
				  	%put The disk may be full or permissions may not be set appropriately.;
					%let exiterror = 1;
					%goto iter_exit;
				%end;
				%inc gmacro(
					UtilityMacros.sas,
					CrossValidationModelComparisonMacros.sas,
					FitModels.sas,
					BinaryResponseEffectSelectionMacros.sas,
					SeedMacros.sas
				);
				filename gmacro clear;

				%put cv iter_start = &iter_start;
				%put cv iter_end = &iter_end;

				* LOOP;
				%cv_iteration_loop(datapath=,
							      outputpath=&outputpath,
								   dsname=&dsname,
								   n_models=&n_models,
								   trait=&trait,
								   trait_type=&trait_type,
								   cv_method=&cv_method,
								   iter_start=&iter_start,
								   iter_end=&iter_end,
								   kfold=&kfold,
								   groupname=&groupname,
								   cv_vars_outds=&cv_vars_outds,
								   cvname=&cvname,
								   cv_holdout_percent=&cv_holdout_percent,
								   varimportance=&varimportance,
								   prevalences=&prevalences,
								   cv_custompriors=%nrbquote(&cv_custompriors),
								   cv_customcosts=%nrbquote(&cv_customcosts),
								   cv_alossprop=&cv_alossprop
								   );

				* SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
				%if %upcase(&trait_type) = QUALITATIVE %then %do;	
					%do c=1 %to &nclasses;
						%let class=&&class&c;
						%let p_prob=&&p_prob&c;
						data _namedat;
							length CLASS P_PROB $32 ;
						 	CLASS="&class";
							P_PROB="&p_prob";
							EVENT="&event";
						run;
						%if &c=1 %then %do;
							%if &UseGrid %then %do;
								proc download data=_namedat out=&cvoutlib.._namedat&gridseq;
								run;
							%end;
							%else %do;
								data &cvoutlib.._namedat&gridseq;
									set _namedat;
								run;
							%end;
						%end;
						%else %do;
							%if &UseGrid %then %do;
								proc upload data=&cvoutlib.._namedat&gridseq out=temp;
								run;
								data temp;
									set temp _namedat;
								run;
								proc download data=temp out=&cvoutlib.._namedat&gridseq;
								run;
							%end;
							%else %do;
								data &cvoutlib.._namedat&gridseq;
									set &cvoutlib.._namedat&gridseq
										_namedat;
								run;
							%end;
						%end;
					%end;
				%end;

				%iter_exit:;
				%mend;

				%ITERATION_LOOP;

				endrsubmit;
			;
		%end;

		* WAIT FOR ALL RSUBMITS TO FINISH;
		%put waitlist = &waitlist;
		waitfor _ALL_ &waitlist;

		* STOP SAS RUNNING ON THE GRID NODES;                                 
		* signoff _all_;

		* UPDATE GRID NODES RUN_STATUS IN cvgridseq.sas7bdat;
		data &cvoutlib..cvgridseq;
			set &cvoutlib..cvgridseq;
			%do node=1 %to &ngridnodes;
				%let gndone=&&gndone&node;
				if grid_node = &node and run_status ne 0 then do; 
					run_status=&gndone;
				end;
			%end;
		run;

		data rungrid;
			set &cvoutlib..cvgridseq;
			if run_status ne 0 then output;
		run;

		%NObsVars(rungrid);
		%let rungridseq=&nObs;

	%end;

%end;
%else %do;

	%let iter_start=&cv_holdout_iteration_start;
	%let iter_end=&cv_holdout_iteration_end;

	%cv_iteration_loop(datapath=&datapath,
				       outputpath=&outputpath,
					   dsname=&dsname,
					   n_models=&n_models,
					   trait=&trait,
					   trait_type=&trait_type,
					   cv_method=&cv_method,
					   iter_start=&iter_start,
				   	iter_end=&iter_end,
					   kfold=&kfold,
					   groupname=&groupname,
					   blockname=&blockname,
					   cv_vars_outds=&cv_vars_outds,
					   cvname=&cvname,
					   cv_holdout_percent=&cv_holdout_percent,
					   varimportance=&varimportance,
					   prevalences=&prevalences,
					   cv_custompriors=%nrbquote(&cv_custompriors),
					   cv_customcosts=%nrbquote(&cv_customcosts),
					   cv_alossprop=&cv_alossprop
					   );

	*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%do c=1 %to &nclasses;
			%let class=&&class&c;
			%let p_prob=&&p_prob&c;
			%put c=&c class&c=&class p_prob&c=&p_prob event=&event;
			data nd1;
				length CLASS P_PROB $32 ;
			 	CLASS="&class";
				P_PROB="&p_prob";
				EVENT="&event";
			run;
			%if &c=1 %then %do;
				data &cvoutlib.._namedat;
					set nd1;
				run;
			%end;
			%else %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat
						nd1;
				run;
			%end;
		%end;
	%end;

%end;

%cv_exit:
;
%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    CV_ITERATION_LOOP
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/30/08
**  PURPOSE: Cross validation iteration loops
**  HISTORY: action                                       date     who
**           Created.                                     06/15/07 pyt 
**			 Grid computing						          07/30/08 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:   Library path of training data set 
**  OUTPUTPATH: Library path of output data sets 
**  DSNAME:     Input data set name
**  N_MODELS:   Number of modeling mthods
**  TRAIT:      Trait variable name
**  TRAIT_TYPE: Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  CV_METHOD:  Cross validation method
**              Various methods are: 
**              RANDOM     (RANDOM SIMPLE)        
**              STRATIFIED (RANDOM STRATIFIED)
**              BLOCK      (K-FOLD BLOCK)
**              SPLIT      (K-FOLD SPLIT)
**              KFOLD      (K-FOLD RANDOM PARTITION)
**              LEAVEOUT	(LEAVE L OUT)
**  ITER_START: Iteration starting point
**  ITER_END: 	Iteration ending point
**  KFOLD: 		K-fold cross validation 
**  GROUPNAME:  Grouping variable
**  BLOCKNAME:  K-fold block partition variable name 
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  VARIMPORTANCE: Flag to launch variable importance processor
** 	PREVALENCES:   Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**  CV_CUSTOMCOSTS: Custom cost functions
**  CV_ALOSSPROP: Asymmetric loss evaluation proportion
**
**  NOTES: It is called from cv_iterations
**
************************************************************************;
%macro cv_iteration_loop(datapath=,
				         outputpath=,
						 dsname=,
						 n_models=,
						 trait=,
						 trait_type=,
						 cv_method=,
 						 iter_start=,
				   	 iter_end=,
						 kfold=,
						 groupname=,
						 blockname=,
						 cv_vars_outds=,
						 cvname=,
						 cv_holdout_percent=,
						 varimportance=,
						 prevalences=,
						 cv_custompriors=,
						 cv_customcosts=,
						 cv_alossprop=
						 );

%put IN CV_ITERATION_LOOP kfold=&kfold cv_holdout_percent=&cv_holdout_percent;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname datalib "&datapath";
	%let datalib = datalib;
%end;
%else %do;
    %let datalib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cvlpname = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__cvlpname)) = 1 %then %do;
	%NObsVars(&__cvlpname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__cvlpname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__cvlpname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;
%if ^%symexist(UseGrid) %then %let UseGrid = 0;

* MAIN ITERATION LOOP;
%do iter = &iter_start %to &iter_end;

	%put STARTING CV ITERATION &iter. of &iter_end;

	* GENERATE MACRO RANDSEED FOR THIS CROSS-VALIDATION ITERATION;
	* KFOLD BASIC uses seed to create cv_var= 1, ..., KFOLD      ;
	* RANDOM, STRATIFIED uses seed to draw random train sample   ; 
  	* BLOCK, SPLIT, LEAVEOUT do not use a seed            	     ;
  	%if &UseGrid %then %do;
 		%seedget(i=&iter, seedds=seedseq, thread=1);
  	%end;
  	%else %do;
  		%seedget(i=&iter, seedds=&cvoutlib..seedseq, thread=1);
  	%end;
	%put dsname=&dsname;
	
	%NObsVars(&__cvlpname);
	%let _trainObs_ = &nObs;

	%if &cv_method=BLOCK %then %do;
		%put CV_METHOD: K-FOLD BLOCK;
		%if %length(&blockname) > 0 and &blockname ne "" %then %do;
			%let block_var = &blockname;
			%put block_var = &block_var;
		%end;
		%else %do;
			%let block_var=;
			data _null_;
				 call symput("blocksize", trim(left(put(&_trainObs_/&iter_end, best12.))));
			run;
			%put blocksize = &blocksize;
		%end;
	%end;
	%else %if &cv_method=SPLIT %then %do;
		%put CV_METHOD: K-FOLD SPLIT;
	%end;
	%else %if &cv_method=KFOLD %then %do;
		%put CV_METHOD: K-FOLD RANDOM PARTITION.;
		%if &kfold > 1 %then %do;
			%if %length(&cv_vars_outds) > 0 and &cv_vars_outds ne "" %then %do;
				 %let cv_outds=&cv_vars_outds;
			%end;
			%else %do;
				 %let cv_outds=&dsname._CV;
			%end;
			%if %length(&cvname) > 0 and &cvname ne "" %then %do;
				 %let cv_var=&cvname;
			%end;
			%else %do;
				 %let cv_var=_CV_;
			%end;
			* GENERATE KFOLD _CV_ VARIABLE;
			* SAVE THE CV_OUTDS DATA SET TO SASUSER ON EACH GRIDNODE;
			%let ntries = 0;
			%let isok = 0;
			%do %while(&isok = 0 and &ntries < 10);
				%let ntries = %eval(&ntries + 1);
				%generate_cv_vars(datapath=&datapath,
					     		  outputpath=,
							      dsname=&dsname,
								  trait=&trait,
								  trait_type=&trait_type,
								  kfold=&kfold,
								  groupname=&groupname,
								  cvname=&cv_var, 
								  cv_seed=&randseed,
								  cv_vars_outds=&cv_outds,
								  iteration=&iter
				);
				* UP TO 10 TRIES UNTIL ALL KFOLD ARE CREATED;  
				proc freq data=&cv_outds.&iter noprint;
					tables &cv_var/out=cvfreq;
				run;
				%NObsVars(work.cvfreq);
				%if &nObs = &kfold %then %do;
					%let isok = 1;
				%end; 
				%else %do;
				 	%let randseed = %eval(&randseed+7);
				%end;
			%end;
		%end;
	%end;

	* MAIN KFOLD LOOP;
	%do k=1 %to &kfold;

		%put STARTING K-FOLD ITERATION &k OF &kfold.;

		data cvtrain;
			delete;
		run;
		data cvtest;
			delete;
		run;

		%if &cv_holdout_percent = 0 %then %do;
			* FOR KFOLD=1 USE ALL OF THE DATA FOR BOTH TRAINING AND TEST;
			%put CV_METHOD: USING ALL OF THE DATA FOR BOTH TRAINING AND TESTING, 
				 RESULTS ARE LIKELY OPTIMISTIC.;
			data cvtrain;
				set &__cvlpname;
			run;
			data cvtest;
				set &__cvlpname;
			run;
		%end;
		%else %if &cv_method=RANDOM or &cv_method=STRATIFIED %then %do;
			* 10 TRIES UNTIL CVTRAIN DOES NOT HAVE CONSTANT TRAIT,
				 TEST SET HAS AT LEAST ONE OBSERVATION;
			%let ntrain = 0;
			%let ntest = 0;
			%let ntries = 0;
		   %let isconstant = 0;

			%let isok = 0;
			%if &cv_method=RANDOM %then %do;
			 	%put CV_METHOD: RANDOM SIMPLE.;
			%end;
			%else %if &cv_method=STRATIFIED	%then %do;
				%put CV_METHOD: RANDOM STRATIFIED.;
			%end;
			%do %while(&isok = 0 and &ntries < 10);
				%let ntries = %eval(&ntries + 1);
				%put ntries = &ntries;
				%if &cv_method=RANDOM %then %do;
					data cvtrain cvtest;
						set &__cvlpname;
						_I = &iter;
						_K = &k;
						_U = ranuni(&randseed);
						if (_U < &cv_holdout_percent/100) then output cvtest;
						else output cvtrain;
					run;
				%end; 
				%else %if &cv_method=STRATIFIED	%then %do;
					%stratified_sample(datapath=&datapath,
					  			     outputpath=,
								     dsname=&dsname, 
								     trait=&trait, 
								     trait_type=&trait_type, 
								     holdout_percent=&cv_holdout_percent,
								     groupname=&groupname,
								     train_dsname=cvtrain,
								     test_dsname=cvtest,
								     seed=&randseed,
									 iter=&iter,
								     k=&k);
				%end;
				%NObsVars(work.cvtrain);
				%let ntrain = &nObs;
				%if &ntrain > 0 %then %do;
				  %if &trait_type = qualitative %then %do;
				     proc freq data=cvtrain noprint;
					 	tables &trait/out=tfreq;
					 run;
					 %NObsVars(work.tfreq)
					 %if &nObs < 2 %then %do;
					 	%let isconstant = 1;
						%if &ntries > 10 %then %do;
							%put CV_ITERATION &iter KFOLD &k CVTRAIN CONTAINS CONSTANT TRAIT;
							%put ERROR: UNABLE TO OBTAIN TRAINING SET WITH NONCONSTANT TRAIT AFTER 10 TRIES.;
							%goto iter_end;
						%end;
					%end;
					%else %do;
						%let isconstant = 0;
					%end;
				   %end;
				%end;
				%NObsVars(work.cvtest);
				%let ntest = &nObs;
				%put ntest = &ntest;
				%put ntrain = &ntrain;
				%put isconstant = &isconstant;
				%if ^&ntest or ^&ntrain or &isconstant = 1 %then %do;
				  	%let isok = 0;
				 	%let randseed = %eval(&randseed+7);
					%put IN RETRY;
					%put randseed = &randseed;
				%end;
				%else %do;
					%let isok = 1;
				%end;
			%end;

			* CONFIRM CVTEST IS NOT EMPTY;
			%if %sysfunc(exist(cvtest)) = 1 %then %do;
				%NObsVars(cvtest);
				%if ^&nObs or ^&nVars %then %do;
					%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
					%goto iter_end;
				%end; 
			%end;
			%else %do;
				%put ERROR: CVTEST DOES NOT EXIST.;
				%goto iter_end;
			%end;

		%end;
		%else %if &cv_method=KFOLD or &cv_method=BLOCK or &cv_method=SPLIT %then %do;
			 %if &cv_method=KFOLD %then %do;
				 %put CV_METHOD: K-FOLD RANDOM.;
				 %put iter = &iter;
				 %put k = &k;
				 %if &kfold > 1 %then %do;
				 	 proc sort data=&__cvlpname;
					 	by __id;
					 run;
					 proc sort data=&cv_outds.&iter;
					 	by __id;
					 run;
					 data ds;
						merge &__cvlpname &cv_outds.&iter;
						by __id;
					 run;
					 data cvtrain cvtest;
						  set ds;
						  _I = &iter;
						  _K = &k;
						  if &cv_var = &k then output cvtest;
						  else output cvtrain;
					 run;
				 %end;
				 %else %do;
					 data cvtrain;
					 	set train;
					 run;
				 %end;
			 %end;
			 %else %if &cv_method=BLOCK %then %do;
				 %put CV_METHOD: K-FOLD BLOCK;
				 %put iter=&iter  block_var=&block_var;
				 data outlib.both;
				 	set &__cvlpname;
				 run;
				 %if %length(&block_var) > 0 and &block_var ne "" %then %do;
					data cvtrain cvtest;
						set &__cvlpname;
						_I = &iter;
						_K = &k;
						if &block_var = &iter then output cvtest;
						else output cvtrain;
					run;
				 %end;
				 %else %do;
					 data cvtrain cvtest;
						  set &__cvlpname;
						  _I = &iter;
						  _K = &k;
						  if (&iter-1)*&blocksize < _n_ <= &iter*&blocksize then output cvtest;
						  else output cvtrain;
					 run;
				 %end;
			 %end;
			 %else %if &cv_method=SPLIT %then %do;
				 %put CV_METHOD: K-FOLD SPLIT.;
				 data cvtrain cvtest;
					  set &__cvlpname;
					  _I = &iter;
					  _K = &k;
					  if mod(_n_, &iter_end) = (&iter-1) then output cvtest;
					  else output cvtrain;
				 run;
			 %end;
			 %if &trait_type = qualitative %then %do;
				%NObsVars(cvtrain);
				%if ^&nObs or ^&nVars %then %do;
					%put ERROR: CVTRAIN CONTAINS 0 OBSERVATIONS.;
					%goto iter_end;
				%end;
				proc freq data=cvtrain noprint;
					tables &trait/out=tfreq;
				run;
				%NObsVars(work.tfreq);
				%let _ftnObs_ = &nObs;
				%if &_ftnObs_ < 2 %then %do;
					%put CV_ITERATION &iter KFOLD &k CVTRAIN CONTAINS CONSTANT TRAIT;
					%goto iter_end;
				%end;
			 %end;

			 %if &kfold > 1 %then %do;
				* CONFIRM CVTEST IS NOT EMPTY;
				%if %sysfunc(exist(cvtest)) = 1 %then %do;
					%NObsVars(cvtest);
					%if ^&nObs or ^&nVars %then %do;
						%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
						%goto iter_end;
					%end; 
				%end;
				%else %do;
					%put ERROR: CVTEST DOES NOT EXIST.;
					%goto iter_end;
				%end;
			%end;

		%end;

		%if (&CV_ListAll = Yes) %then %do;
			 data &cvoutlib..cvtrain_&iter._&k;
				  set cvtrain;
			 run;
			 data &cvoutlib..cvtest_&iter._&k;
				  set cvtest;
			 run;
		%end;

		* CREATE BASELINE SCORED DATA SET;
		* COMPUTE FIT STATISTICS: ASE, AUC FROM EACH CV RUN;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			 %basescor(train_dsname=cvtrain,
					   test_dsname=cvtest,
					   trait=&trait,
					   prevalences=&prevalences,
					   cv_custompriors=%nrbquote(&cv_custompriors)
						);
		%end;
		%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
			 %basescorint(train_dsname=cvtrain,
				          test_dsname=cvtest,
			    	      trait=&trait);
		%end;

		* APPEND BASELINE SCORED DATA SETS;
		%if %sysfunc(exist(baseline_preds)) = 1 %then %do;
			%NObsVars(baseline_preds);
			%let _basepObs_ = &nObs;
			%if &_basepObs_ > 0 %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_baseline_preds&iter)) = 1 %then %do;
					%if &UseGrid %then %do;
						%let bd = cv_baseline_preds&iter;
						proc upload data=&cvoutlib..cv_baseline_preds&iter out=&bd;
						run;
					%end;
					%else %do;
						%let bd = &cvoutlib..cv_baseline_preds&iter;
					%end;
					%NObsVars(&bd);
					%let _cvbpObs_ = &nObs;
					data &bd;
						 length METHOD $64;
						 set 
						 %if &_cvbpObs_ %then %do;
							  &bd
						 %end;
							  baseline_preds;
						 METHOD = "BASELINE";
					run;
					%if &UseGrid %then %do;
						proc download data=&bd out=&cvoutlib..cv_baseline_preds&iter;
						run;
					%end;
				%end;
				%else %do;
					data &cvoutlib..cv_baseline_preds&iter;
						 length METHOD $64;
						 set baseline_preds;
						 METHOD = "BASELINE";
					run;
				%end;
			%end;
		%end;

		* CREATE CROSS VALIDATION MODELS PREDICTIONS,
		  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
		  CV_<MODELNAME>_PREDS AND CV_<MODELNAME>_VARS, 
		  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
		%fitmodels(train_dsname=cvtrain,
				   test_dsname=cvtest);

		* APPEND CV ITERATION MODELS PREDICTIONS AND VARS;
		%do nm=1 %to &n_models;
			%let FM_Error = &&FM_Error&nm;
			%put FM_ERROR&nm = &FM_Error;
			* ERROR CHECKING;
			%if &FM_Error %then %do;
				data &cvoutlib..cvtrain_m&nm._i&iter._k&k._error;
					set cvtrain;
				run;
				data &cvoutlib..cvtest_m&nm._i&iter._k&k._error;
					set cvtest;
				run;
			%end;
			%else %do; 
				%let mdlname=&&mdlname&nm;
				%*put mdlname= &mdlname;
				
				* APPEND PREDICTIONS;
				%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds)) = 1 %then %do;
					%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds);
					%*put Name0nObs= &nObs;
					%*put Name0nVars= &nVars;
					%if &nObs and &nVars %then %do;

						%if %upcase(&trait_type) = SURVIVAL and 
							%sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm1)) = 1%then %do;

							* tall sorted (by &_tVar) survival training data;
							data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm1;
								length METHOD $64 ITER 8 K 8;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm1;
								ITER=&iter;
								K=&k; 
								RANDSEED=&randseed;
								METHOD = "&mdlname";
							run;

							%if %sysfunc(exist(&cvoutlib..cv_m&nm._s1cum&iter)) = 1 %then %do;
								%NObsVars(&cvoutlib..cv_m&nm._s1cum&iter);
								%let _cvs1cumObs_ = &nObs;
								data &cvoutlib..cv_m&nm._s1cum&iter;
									set 
									%if &_cvs1cumObs_ %then %do;
										&cvoutlib..cv_m&nm._s1cum&iter
									%end;
										cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm1;
								run;
							%end;
							%else %do;
								data &cvoutlib..cv_m&nm._s1cum&iter;
									set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm1;
								run;
							%end;

							* tall sorted (by &_tVar) survival test data;
							data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm2;
								length METHOD $64 ITER 8 K 8;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm2;
								ITER=&iter;
								K=&k; 
								RANDSEED=&randseed;
								METHOD = "&mdlname";
							run;

							%if %sysfunc(exist(&cvoutlib..cv_m&nm._s2cum&iter)) = 1 %then %do;
								%NObsVars(&cvoutlib..cv_m&nm._s2cum&iter);
								%let _cvs2cumObs_ = &nObs;
								data &cvoutlib..cv_m&nm._s2cum&iter;
									set 
									%if &_cvs2cumObs_ %then %do;
										&cvoutlib..cv_m&nm._s2cum&iter
									%end;
										cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm2;
								run;
							%end;
							%else %do;
								data &cvoutlib..cv_m&nm._s2cum&iter;
									set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm2;
								run;
							%end;

							* training and test Data combined containing experimental factors and predictors to be;
							* merged after transpose;
							data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm3;
								length METHOD $64 ITER 8 K 8;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm3;
								ITER=&iter;
								K=&k; 
								RANDSEED=&randseed;
								METHOD = "&mdlname";
							run;

							%if %sysfunc(exist(&cvoutlib..cv_m&nm._s3cum&iter)) = 1 %then %do;
								%NObsVars(&cvoutlib..cv_m&nm._s3cum&iter);
								%let _cvs3cumObs_ = &nObs;
								data &cvoutlib..cv_m&nm._s3cum&iter;
									set 
									%if &_cvs3cumObs_ %then %do;
										&cvoutlib..cv_m&nm._s3cum&iter
									%end;
										cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm3;
								run;
							%end;
							%else %do;
								data &cvoutlib..cv_m&nm._s3cum&iter;
									set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_spm3;
								run;
							%end;

							data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
								drop EventTime_:; 
							run;
						%end;

						data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							length METHOD $64 ITER 8 K 8;
							set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							ITER=&iter;
							K=&k; 
							RANDSEED=&randseed;
							METHOD = "&mdlname";
						run;

						%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum&iter)) = 1 %then %do;
							%if &UseGrid %then %do;
								%let bd = cv_m&nm._pcum&iter;
								proc upload data=&cvoutlib..cv_m&nm._pcum&iter out=&bd;
								run;
							%end;
							%else %do;
								%let bd = &cvoutlib..cv_m&nm._pcum&iter;
							%end;
							%NObsVars(&bd);
							%let _cvpcumObs_ = &nObs;
							data &bd;
								set 
								%if &_cvpcumObs_ %then %do;
									&bd
								%end;
									cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							run;
							%if &UseGrid %then %do;
								proc download data=&bd out=&cvoutlib..cv_m&nm._pcum&iter;
								run;
							%end;
						%end;
						%else %do;
							data &cvoutlib..cv_m&nm._pcum&iter;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							run;
						%end;
					%end;
				%end;
				
				* APPEND PREDICTOR VARS SELECTED DATA SETS;
				%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars)) = 1 %then %do;
					%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars);
					%if &nObs and &nVars %then %do;
						data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
							length METHOD $64 ITER 8 K 8;
							set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
							ITER=&iter;
							K=&k;
							METHOD = "&mdlname";
						run;
						%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum&iter)) = 1 %then %do;
							%if &UseGrid %then %do;
								%let bd = cv_m&nm._vcum&iter;
								proc upload data=&cvoutlib..cv_m&nm._vcum&iter out=&bd;
								run;
							%end;
							%else %do;
								%let bd = &cvoutlib..cv_m&nm._vcum&iter;
							%end;
							%NObsVars(&bd);
							%let _cvvcumObs_ = &nObs;
							data &bd;
								set 
								%if &_cvvcumObs_ %then %do;
									&bd
								%end; 
									cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
							run;
							%if &UseGrid %then %do;
								proc download data=&bd out=&cvoutlib..cv_m&nm._vcum&iter;
								run;
							%end;
						%end;
						%else %do;
							data &cvoutlib..cv_m&nm._vcum&iter;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
							run;
						%end;
					%end;
				%end;

				%if %upcase(&varimportance) = YES %then %do;
					%if %sysfunc(exist(work.impfit)) = 1 %then %do;
						%NObsVars(work.impfit);
						%if &nObs and &nVars %then %do;
							proc sort data=impfit;
								by _INPUT1_;
							run;
							data &cvoutlib..impfit&iter&k;
								set impfit;
								rename _TASE_ = _TASE_&iter&k;
							run;
						%end;
					%end;
				%end;
			%end; 
		%end; * END OF N_MODELS LOOP;
        
		%if %upcase(&varimportance) = YES %then %do;
			%if &n_models ne 1 %then %do;
				%put ERROR: MORE THAN ONE MODEL IS SPECIFIED.;
				%let exiterror = 1;
				%goto cv_exit;
			%end;

			%let mdlname=&mdlname1;

			* CHECK FOR PRESELECT_PREDICTORS OUTPUT EXIST;	
			%if %sysfunc(exist(&cvoutlib..vi_&mdlname._vars)) = 1 %then %do;
				%NObsVars(&cvoutlib..vi_&mdlname._vars);
				%if &nObs and &nVars %then %do;
					%loop_preds(effects_dsname=&cvoutlib..vi_&mdlname._vars,
								train_dsname=cvtrain,
								test_dsname=cvtest,
								model_name=&mdlname);

					* ERROR CHECKING;
					%if ^&FM_Error1 %then %do; 
						*APPEND PREDICTIONS;
					 	%if %sysfunc(exist(cv_vipreds)) = 1 %then %do;
							%NObsVars(cv_vipreds);
							%if &nObs and &nVars %then %do;
								data cv_vipreds;
									length METHOD $64 ITER 8 K 8;
									set cv_vipreds;
									ITER=&iter;
									K=&k;
									RANDSEED=&randseed;
									METHOD = "&mdlname";
								run;

					 			%if %sysfunc(exist(&cvoutlib..cv_vipreds_pcum&iter)) = 1 %then %do;
									%NObsVars(&cvoutlib..cv_vipreds_pcum&iter);
									%let _cvpcumObs_ = &nObs;
									data &cvoutlib..cv_vipreds_pcum&iter;
										set 
										%if &_cvpcumObs_ %then %do;
											&cvoutlib..cv_vipreds_pcum&iter
										%end;
										cv_vipreds;
									run;
								%end;
								%else %do;
									data &cvoutlib..cv_vipreds_pcum&iter;
										set cv_vipreds;
									run;
								%end;
							%end;
						%end;

						*APPEND EFFECTS;
						%if %sysfunc(exist(cv_vivars)) = 1 %then %do;
							%NObsVars(cv_vivars);
							%if &nObs and &nVars %then %do;
								data cv_vivars;
									length METHOD $64 ITER 8 K 8;
									set cv_vivars;
									ITER=&iter;
									K=&k;
									RANDSEED=&randseed;
									METHOD = "&mdlname";
								run; 

					 			%if %sysfunc(exist(&cvoutlib..cv_vivars_pcum&iter)) = 1 %then %do;
									%NObsVars(&cvoutlib..cv_vivars_pcum&iter);
									%let _cvvarsObs_ = &nObs;
									data &cvoutlib..cv_vivars_pcum&iter;
										set 
										%if &_cvvarsObs_ %then %do;
											&cvoutlib..cv_vivars_pcum&iter
										%end;
											cv_vivars;
									run;
								%end;
								%else %do;
									data &cvoutlib..cv_vivars_pcum&iter;
										set cv_vivars;
									run;
								%end;
							%end;
						%end; * END OF APPEND EFFECT;
					%end; * LOOP_PREDS NO ERROR;
				%end; * cvoutlib..vi_&mdlname._vars nObs nVars;
			%end; * cvoutlib..vi_&mdlname._vars EXIST;
		%end; * varimportance YES;
	%end; * END OF KFOLD LOOP; 
	%iter_end:;
%end; * END OF MAIN ITERATION;
%mend;

* create metadata for learning curves;
%macro lc_create_metadat;

	* RETRIEVE MODEL NAMES;
	%do m=1 %to &n_models;
		%global mdlname&m;
		%let mdlname&m = %scan(&model_names,&m," ");
	%end;

	* CHECK FOR N_INNER_LOOP SIZE,
	  BECAUSE INNERLOOP SIZES ARE EQUALLY SPACED IN FLOOR(log(n)), 
	  THIS WILL MAP SEVERAL INTEGERS TO THE SAME INTEGER.   
	  MAX_INNERLOOP IS THE MAXIMUM NUMBER OF DISTINCT INTEGERS AFTER THE FLOOR(log(n)) TRANFORMATION. 
	  IF NMIN EXISTS, N_INNERLOOP <=FLOOR((&_tnObs_-&nmin)/2.7182818281)
	  ELSE N_INNERLOOP <= FLOOR((0.9*&_tnObs_)/2.7182818281);
	%if &nmin > 0 %then %do;
		%let max_innerloop = %sysevalf((&_tnObs_-&nmin)/2.7182818281, int);
	%end;
	%else %do; 
		%let max_innerloop = %sysevalf((0.9*&_tnObs_)/2.7182818281, int);
	%end;

	%if &n_innerloop > &max_innerloop %then %do;
		%put WARNING: NUMBER OF TRAINING SIZES FOR EACH LEARNING CURVE > &max_innerloop;
		%put RESETTING n_innerloop TO &max_innerloop;
		%let n_innerloop = &max_innerloop;
	%end;

	%if ^%length(&nmax) %then %do;
		 %if %upcase(&testset_option) = HOLDOUT %then %do;
			%let nmax = %sysevalf(&_tnObs_ * 0.9, int);
		%end;
		%else %do;
			%let nmax = &_tnObs_;
		%end;
	%end;

	data _null_;
		%if &nmax > 0 %then %do;
			  %if &nmax > &_tnObs_ %then %do;
				  %put ERROR: MAXIMUM SIZE OF TRAINING SIZE > NUMBER OF OBSERVATIONS IN THE TRAINING DATA SET.; 
				  %let exiterror = 1;
				%goto lc_exit;
			%end;
			%str(all_train_size=&nmax;);
		%end;
		%else %do;
			%put ERROR: nmax MUST BE GREATER THAN 0.; 
			%let exiterror = 1;
			%goto lc_exit;
		%end;
		 %if &outer_method = BLOCK %then %do;
			%if ^%length(&n_outerloop) or &n_outerloop < 1 %then %do;
				%put n_outerloop IS NOT SPECIFIED OR IS < 1 FOR BLOCK OUTER_METHOD.;
				%let exiterror = 1;
				%goto lc_exit;
			%end;
			block_size = floor(all_train_size/&n_outerloop);
			  max_inner_size = block_size;
		 %end;
		 %else %do;
			block_size = &_tnObs_;
			  max_inner_size = &nmax;
		 %end;

		%if &nmin > 0 %then %do;
			%if &nmin < 8 %then %do;
				  %put ERROR: MINIMUM SIZE OF TRAINING SIZE MUST BE >= 8.; 
				  %let exiterror = 1; 
				%goto lc_exit;
			%end;
			%else %do;		
				%if %upcase(&cv_method) ne LEAVEOUT %then %do;
					%if &nmin < &kfold %then %do;
						%put ERROR: MINIMUM SIZE OF TRAINING SIZE &nmin < K FOR K-FOLD SIZE &kfold;
							%let exiterror = 1; 
						 %goto lc_exit;
					%end; 
				%end;
				%str(min_inner_size =&nmin;);
			%end;
		%end;
		%else %do;
			n = ceil(max_inner_size/10);
			%if %upcase(&cv_method) ne LEAVEOUT %then %do;
				min_inner_size=max(n, 8, &kfold);
			%end;
			%else %do;
				min_inner_size=max(n, 8);
			%end;
		%end;

		 %if &n_innerloop = 1 %then %do;
			  %str(delta = 0;);
		%end;
		 %else %do;
			  %str(delta = (log(max_inner_size) - log(min_inner_size))/(&n_innerloop-1););
		%end;
		 call symputx("delta", delta);
		call symputx("min_inner_size", put(min_inner_size, 12.0));
		call symputx("max_inner_size", put(max_inner_size, 12.0));
		 call symputx("block_size", put(block_size, 12.0));
		 call symputx("all_train_size", put(all_train_size, 12.0));
	run;

	%put delta = &delta;
	%put block_size = &block_size (ONLY USED WHEN OUTER_METHOD=BLOCK);
	%put all_train_size = &all_train_size;
	%put _tnObs_ = &_tnObs_;
	%put min_inner_size = &min_inner_size;
	%put max_inner_size = &max_inner_size;

	* SAVE MODEL NAMES, OUTER_LOOP, INNER_LOOP, TRAIN_SIZE INFORMATION IN _METADATA;
	* USE IT AS A GUIDE FOR MERGING RESULTS;
	data &cvoutlib.._metadat;
		 length METHOD $64 METHOD_I OUTER_LOOP INNER_LOOP TRAIN_SIZE 8;   
		%do m=1 %to &n_models;
			%let mdlname =&&mdlname&m;
			METHOD="&mdlname";
			METHOD_I=&m;
			%do outer_loop = &outerloop_start %to &outerloop_end;  
				OUTER_LOOP=&outer_loop; 
				%do inner_loop = 1 %to &n_innerloop;                                                                                       
						INNER_LOOP=&inner_loop;                                                                                                           
						TRAIN_SIZE = put(&min_inner_size * exp((&inner_loop-1)*&delta), 12.0);  
					output;
				 %end;                                                                                                                               
			%end;         
		%end;	
	run; 
	%lc_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    CV_LEARNING_CURVE_ESTIMATES
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/08/07
**  PURPOSE: Cross validation learning curve  
**  HISTORY: action                                       date     who
**           Created.                                     06/08/07 pyt 
** 			 Grid computing 							  08/11/08 pyt
**			 Added innerloop_option and testset_option	  11/03/09 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:     Number of models
**  NMIN: Min size of available data (default = Nmax / 10)
**  NMAX: Max size of available data (All by default)
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  N_OUTERLOOP:  Maximum number of total LC outer loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  OUTER_METHOD: Method to prepare inner_train training data set 
**				  from outer loop iteration
**				  Methods are: BLOCK or RANDOM	
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  CV_HOLDOUT_ITERATIONS: Number of total random hold-out iterations
**  CV_SEED: Random number seed
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  KFOLD: K-fold cross validation 
**  BLOCKNAME: K-fold block partition variable name 
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  LCR_PREFIX: Prefix of LC prediction results data set name
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Client shared grid folder 
**  SERVERGSHARE: Server shared grid folder
*
**  NOTES:
**  Do not collect selected predictor variables for learning curve as
**  it is done for cross validation.
**  Invokes lc_iteration_loop
************************************************************************;
%macro cv_learning_curve_estimates(datapath=,
								   outputpath=,
								   dsname=,
								   innerloop_option=,
								   testset_option=,
								   trait=,
								   trait_type=,
								   freq=,
								   model_names=,
								   n_models=,
								   nmin=,
								   nmax=,
								   n_innerloop=,
								   n_outerloop=,
								   outerloop_start=,
								   outerloop_end=,
								   outer_method=,
								   cv_method=,
								   cv_holdout_percent=,
								   cv_holdout_iterations=,
								   cv_seed=,
								   cv_vars_outds=,
								   cvname=,
								   kfold=,
								   groupname=,
								   blockname=,
								   leave_l_out=,
								   lcr_prefix=,
								   models_comparison_output=,
				  				   prevalences=proportional,
								   ngridnodes=,
								   clientgshare=,
								   servergshare=
				  				   );

%put START CV LEARNING CURVE;

* ERROR CHECKING;
%let cv_method = %upcase(&cv_method);

/*
%if %upcase(&trait_type)=QUANTITATIVE and &cv_method=STRATIFIED %then %do; 
    %put ERROR: Stratified Random hold outs are not available with a continuous dependent variable.;
    %let exiterror = 1;
    %goto lc_exit;
%end;
*/

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABEL ACCORDINGLY;
%if %length(&datapath) %then %do;
    libname inlib "&datapath";
   %let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) > 0 %then %do; 
    %let __cvlcname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto lc_exit;
%end;

%if %sysfunc(exist(&__cvlcname)) = 1 %then %do;
	%NObsVars(&__cvlcname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__cvlcname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
	%else %do;
		%let _tnObs_ = &nObs;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__cvlcname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* VALIDATE CV PARAMETERS;
%if %length(&cv_holdout_iterations) = 0 %then %do;
	%let exiterror = 1;
	%put CV_HOLDOUT_ITERATIONS IS UNKNOWN.;
	%goto lc_exit;
%end;
%if %length(&cv_method) = 0 or &cv_method = "" %then %do;
	%let exiterror = 1;
    %put CV_METHOD IS UNKNOWN.;
    %goto lc_exit;
%end;
%if &cv_method=RANDOM or &cv_method=STRATIFIED or &cv_method= "" %then %do;
    %if (&cv_holdout_percent <= 0) %then %do;
    	%put WARNING: CV_HOLDOUT_PERCENT < or = 0.;
        %*put ERROR: CV_HOLDOUT_PERCENT < or = 0.;
        %*let exiterror = 1;
        %*goto lc_exit;
    %end;
    %let kfold = 1;
%end;
%else %if &cv_method=BLOCK or &cv_method=SPLIT %then %do;
	* THESE MEAN NOT DOING RANDOM CROSS VALIDATION, 
	  THEREFORE CV ITERATION SHOULD ALWAYS = 1; 
	%if &cv_holdout_iterations ne 1 %then %do;
		%put WARNING: SPLIT AND BLOCK HOLD-OUT METHOD CV ITERATION SHOULD ALWAYS BE 1.;
		%put REPLACE CV ITERATION FROM &cv_holdout_iterations WITH 1.; 
	%end;
	%let cv_holdout_iterations = 1;
%end;
* CHECK FOR THE SIZE OF KFOLD, IF KFOLD > NOBS, REDUCE IT TO NOBS;
%if &kfold > &_tnObs_ %then %do;
	%put WARNING: SIZE OF KFOLD > TRAINING DATA SET SIZE;
	%put RESETTING kfold TO &_tnObs_; 
    %let kfold = &_tnObs_;
%end;

%let delta=;
%let block_size=;
%let all_train_size=;
%let min_inner_size=;
%let max_inner_size=;

%lc_create_metadat;
%if &exiterror %then %goto lc_exit;

%if &ngridnodes > 0 %then %do;

	* DETERMINE GRID SEQUENCE;
	%gridsequence(process=LCMC,
				  cv_holdout_iteration_start=1,
			 	  cv_holdout_iteration_end=&cv_holdout_iterations,
				  innerloop_start=1,
				  innerloop_end=&n_innerloop,
			      outerloop_start=&outerloop_start,
				  outerloop_end=&outerloop_end,
				  leave_l_out=,
				  nnodes=&ngridnodes,
		          gridds=&cvoutlib..lcgridseq);

	%NObsVars(&cvoutlib..lcgridseq);
	%let ngridnodes = &nObs;
	%do node=1 %to &nnodes;
		%global node_name&node;
	%end;
	%global rungridseq;
	%let rungridseq=&ngridnodes;

	%UpdateFitModels(nnodes=&ngridnodes,outputpath=&outputpath);

	%do %while (&rungridseq > 0);

		%let max_tries = 100;
		%let waitlist = ;
		%let ng = 0;

		* CHECK GRID NODES RUN_STATUS IN cvgridseq.sas7bdat
		  FOR ANY RUN_STATUS ne 0, RSUBMIT THE CODE TO GENERATE INTER-MEDIA RESULTS;
		%do gridseq=1 %to &ngridnodes;
			%let _gseq_ = %sysfunc(open(&cvoutlib..lcgridseq));
			%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
			%let status_num = %sysfunc(varnum(&_gseq_, run_status));
			%let gndone = %sysfunc(getvarn(&_gseq_, &status_num));
			%let _gseq_ = %sysfunc(close(&_gseq_));
			%if &gndone=0 %then %do;
	            %put REMOTE SUBMIT COMPLETE ON GNODE&gridseq;
		    %end;
 			%else %if &gndone = . or &gndone = 1 %then %do;
				* KEEP THE RERUN RECORD IN gridseq data set;
				%if &gndone = 1 %then %do;
		        	%put REMOTE SUBMIT FAILED TO EXECUTE ON GNODE&gridseq;
				%end;
				%let ng=%eval(&ng+1);
				%let node=&ng;
				%goto run_gridseq;
		    %end;
		    %else %if &gndone = 2 %then %do;
		        %put REMOTE SUBMIT IS STILL IN PROGRESS ON GNODE&gridseq;
			%end;

			%run_gridseq:
				* RETRIEVE OUTER_START, OUTER_END, INNER_START, INNER_END, 
				* ITER_START, ITER_END FROM lcgridseq OF EACH NODE; 
				%let _gseq_ = %sysfunc(open(&cvoutlib..lcgridseq));
				%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
				%let outstart_num = %sysfunc(varnum(&_gseq_, outer_start));
				%let outer_start = %sysfunc(getvarn(&_gseq_, &outstart_num));
				%let outend_num	= %sysfunc(varnum(&_gseq_, outer_end));
				%let outer_end = %sysfunc(getvarn(&_gseq_, &outend_num));
				%let instart_num = %sysfunc(varnum(&_gseq_, inner_start));
				%let inner_start =  %sysfunc(getvarn(&_gseq_, &instart_num));
				%let inend_num = %sysfunc(varnum(&_gseq_, inner_end));
				%let inner_end =  %sysfunc(getvarn(&_gseq_, &inend_num));
				%let itstart_num = %sysfunc(varnum(&_gseq_, iter_start));
				%let iter_start =  %sysfunc(getvarn(&_gseq_, &itstart_num));
				%let itend_num = %sysfunc(varnum(&_gseq_, iter_end));
				%let iter_end =  %sysfunc(getvarn(&_gseq_, &itend_num));
				%let _gseq_ = %sysfunc(close(&_gseq_));

				data &cvoutlib..lcgridseq;
					set &cvoutlib..lcgridseq;
					if grid_node = &gridseq then do;
						run_status=2;
					end;
				run;
					
				%put In RUN_GRIDSEQ node=&node;
				%let waitlist=&waitlist gnode&node;
				%put waitlist = &waitlist;
				%let tries = 1;
				%let finished = 0;
				%do %while(^&finished);
					%put Sign on attempt &tries of &max_tries to grid node &node....;
					signon gnode&node;
					%let node_name = ;
					%let node_name&gridseq = %sysfunc(grdsvc_getname(gnode&node));
					%let node_name = &&node_name&gridseq;
					%if %length(&node_name) %then %do;
						%put SUCCESSFUL SIGN ON TO GNODE&node = &node_name;
						%put RUNNING OUTER_LOOP &outer_start TO &outer_end, INNER_LOOP &inner_start TO &inner_end,
							 ITERATION &iter_start TO &iter_end ON GNODE&node = &node_name;
						%let finished = 1;
						* UPDATE NODE_NAME IN lcgridseq.sas7bdat;
						data &cvoutlib..lcgridseq;
							set &cvoutlib..lcgridseq;
							if grid_node = &gridseq then do;
								node_name="&node_name";
							end;
						run;
					%end;
					%else %do;
						%let tries = %eval(&tries+1);
						%if &tries > &max_tries %then %do;
							%put ERROR: GRID SIGN ON TO GNODE&node FAILED AFTER &max_tries TRIES.;
							%let exiterror = 1;
							%let finished = 1;
							signoff _all_;
							%goto cv_exit;
						%end;
					%end;
				%end;

				%if %sysfunc(sysprod(prodnum007)) ne 0 %then %do; 
					*S1313431: SAS/CONNECT be licensed*;
					%macro ConnectVars02;
						* SYSLPUT CREATES A NEW MACRO VARIABLE OR MODIFIES THE VALUE OF 
						  AN EXISTING MACRO VARIABLE ON A REMOTE HOST OR SERVER;
						%syslput GSHARE=&gshare;
						%syslput LS_HOME=&LS_HOME;
						%syslput GHOME=&servergshare.gnode&gridseq.%str(/);
						%syslput USER_HOME=&servergshare.gnode&gridseq.%str(/);
						%syslput ProcessName=LCMC&gridseq; 
						%syslput inlib=inlib;
						%syslput cvoutlib=cvoutlib;
						%syslput datapath=&datapath;
						%syslput outpath=&servergshare;
						%syslput outputpath=&servergshare;
						%syslput dsname=&dsname;
						%syslput innerloop_option=&innerloop_option;
						%syslput testset_option=&testset_option;
						%syslput model_names=&model_names;
						%syslput n_models=&n_models;
						%do i=1 %to &n_models;
							%syslput mdlname&i = &&mdlname&i;
							%syslput FM_Error&i = 0;
						%end;
						%syslput mdlname = &mdlname1;
						%syslput trait=&trait;
						%syslput trait_type=&trait_type;
						%syslput n_innerloop=&n_innerloop;
						%syslput n_outerloop=&n_outerloop;
						%syslput outer_start=&outer_start;
						%syslput outer_end=&outer_end;
						%syslput inner_start=&inner_start;
						%syslput inner_end=&inner_end;
						%syslput outer_method=&outer_method;
						%syslput cv_method=&cv_method;
						%syslput cv_holdout_percent=&cv_holdout_percent;
						%syslput cv_holdout_iterations=&cv_holdout_iterations;
						%syslput iter_start=&iter_start;
						%syslput iter_end=&iter_end;
						%syslput cv_vars_outds=&cv_vars_outds;
						%syslput cvname=&cvname;
						%syslput kfold=&kfold;
						%syslput blockname=&blockname;
						%syslput leave_l_out=&leave_l_out;
						%syslput prevalences=&prevalences;
						%syslput randseed=&cv_seed;
						%syslput delta=&delta;
						%syslput min_inner_size=&min_inner_size;
						%syslput max_inner_size=&max_inner_size;
						%syslput block_size=&block_size;
						%syslput lcr_prefix=&lcr_prefix;
						%syslput node_name=&node_name;
						%syslput node=&node;
						%syslput gridseq=&gridseq;
						%syslput gndone&gridseq = 2;
					%mend;
				%end;
				%else %do;
					%macro ConnectVars02;
						%put EMPTY.;	
					%mend;								
				%end;
				%ConnectVars02;

				* RSUBMIT BLOCK OF WORK TO BE SUBMITTED TO GRID NODE;
				rsubmit cwait=no persist=no cmacvar=gndone&gridseq;

				%macro ITERATION_LOOP;

				%put rsubmit for &ProcessName;
				%put gshare = &gshare;
				%put ghome = &GHOME;
				%put node number = &node;
				%put node name = &node_name;
				
				* MACROS DEFINED IN GRID SHARE FOLDER;
				filename gmacro "&GHOME.MacroLib";  
				%if ^%sysfunc(fexist(gmacro)) %then %do;
					%put ERROR: Unable to find grid share folder &GHOME.MacroLib. ;
				  	%put Make sure the server version of your grid share path is correct using ;
					%put File > Configure Life Sciences Settings.;
					%let exiterror = 1;
					%goto iter_exit;
				%end;
				%inc gmacro(
					UtilityMacros.sas,
					CrossValidationModelComparisonMacros.sas,
					FitModels.sas,
					BinaryResponseEffectSelectionMacros.sas,
					SeedMacros.sas
				);
				filename gmacro clear;

				%put outer_start = &outer_start;
				%put outer_end = &outer_end;
				%put inner_start = &inner_start;
				%put inner_end = &inner_end;
				%put iter_start = &iter_start;
				%put iter_end = &iter_end;

				%lc_iteration_loop(datapath=&datapath,
								   outputpath=&outputpath,
								   dsname=&dsname,
								   innerloop_option=&innerloop_option,
								   testset_option=&testset_option,
								   model_names=&model_names,
								   n_models=&n_models,
								   trait=&trait,
								   trait_type=&trait_type,
								   n_outerloop=&n_outerloop,
								   outer_start=&outer_start,
								   outer_end=&outer_end,
								   n_innerloop=&n_innerloop,
								   inner_start=&inner_start,
								   inner_end=&inner_end,
								   outer_method=&outer_method,
								   cv_method=&cv_method,
								   cv_holdout_percent=&cv_holdout_percent,
								   iter_start=&iter_start,
								   iter_end=&iter_end,
								   cv_vars_outds=&cv_vars_outds,
								   cvname=&cvname,
								   kfold=&kfold,
								   groupname=&groupname,
								   blockname=&blockname,
								   leave_l_out=&leave_l_out,
								   prevalences=&prevalences,
								   delta=&delta,
								   min_inner_size=&min_inner_size,
								   max_inner_size=&max_inner_size,
								   block_size=&block_size,
								   lcr_prefix=&lcr_prefix
								   );

				%put nclasses=&nclasses;
				*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
				%if %upcase(&trait_type) = QUALITATIVE %then %do;	
					%do c=1 %to &nclasses;
						%let class=&&class&c;
						%let p_prob=&&p_prob&c;
						data _namedat;
							length CLASS P_PROB $32 ;
						 	CLASS="&class";
							P_PROB="&p_prob";
							EVENT="&event";
						run;
						%if &c=1 %then %do;
							data &cvoutlib.._namedat&gridseq;
								set _namedat;
							run;
						%end;
						%else %do;
							data &cvoutlib.._namedat&gridseq;
								set &cvoutlib.._namedat&gridseq
									_namedat;
							run;
						%end;
					%end;
				%end;

				%iter_exit:;
				%mend;

				%ITERATION_LOOP;

				endrsubmit;
			;

		%end;

		* WAIT FOR ALL RSUBMITS TO FINISH;
		%put waitlist = &waitlist;
		waitfor _ALL_ &waitlist;

		* STOP SAS RUNNING ON THE GRID NODES;                                 
		* signoff _all_;

		* UPDATE GRID NODES RUN_STATUS IN lcgridseq.sas7bdat;
		data &cvoutlib..lcgridseq;
			set &cvoutlib..lcgridseq;
			%do node=1 %to &ngridnodes;
				%let gndone=&&gndone&node;
				if grid_node = &node and run_status ne 0 then do;
					run_status=&gndone;
				end;
			%end;
		run;

		data rungrid;
			set &cvoutlib..lcgridseq;
			if run_status ne 0 then output;
		run;

		%NObsVars(rungrid);
		%let rungridseq=&nObs;

	%end;

%end;
%else %do;

	%let outer_start = &outerloop_start;
	%let outer_end = &outerloop_end;
	%let inner_start = &n_innerloop;
	%let inner_end = 1;
	%let iter_start = 1;
	%let iter_end = &cv_holdout_iterations;

	%lc_iteration_loop(datapath=&datapath,
					   outputpath=&outputpath,
					   dsname=&dsname,
					   innerloop_option=&innerloop_option,
					   testset_option=&testset_option,
					   model_names=&model_names,
					   n_models=&n_models,
					   trait=&trait,
					   trait_type=&trait_type,
					   n_outerloop=&n_outerloop,
					   outer_start=&outer_start,
					   outer_end=&outer_end,
					   n_innerloop=&n_innerloop,
					   inner_start=&inner_start,
					   inner_end=&inner_end,
					   outer_method=&outer_method,
					   cv_method=&cv_method,
					   cv_holdout_percent=&cv_holdout_percent,
					   iter_start=&iter_start,
					   iter_end=&iter_end,
					   cv_vars_outds=&cv_vars_outds,
					   cvname=&cvname,
					   kfold=&kfold,
					   leave_l_out=&leave_l_out,
					   prevalences=&prevalences,
					   groupname=&groupname,
					   blockname=&blockname,
					   delta=&delta,
					   min_inner_size=&min_inner_size,
					   max_inner_size=&max_inner_size,
					   block_size=&block_size,
					   lcr_prefix=&lcr_prefix
					   );

	*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%do c=1 %to &nclasses;
			%let class=&&class&c;
			%let p_prob=&&p_prob&c;
			data _namedat;
				length CLASS P_PROB $32 ;
			 	CLASS="&class";
				P_PROB="&p_prob";
				EVENT="&event";
			run;
			%if &c=1 %then %do;
				data &cvoutlib.._namedat;
					set _namedat;
				run;
			%end;
			%else %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat
						_namedat;
				run;
			%end;
		%end;
	%end;
			
%end;

%lc_exit:;
%mend;
	
***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: CV_MODEL_PVALUE_DATA  
**  AUTHOR:  saspyt, Pei-Yi Tan 
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    03/28/08
**  PURPOSE: Generate a cv model pvalues or predicted values plot data  
**  HISTORY: action                                        date     who
**           Created.                                      03/28/08 pyt 
** 
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  WHERE_CLAUSE: Where clause
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  MODELS_EFFECTS_OUTPUT: Models effects frequencies output 
**
**  NOTES:
**  The output data name for quantitative trait is named PVALUES.
**  The output data names for qualitative trait have prefix PVALUES, 
**  follow by each trait class value, for example 0, 1.
************************************************************************;
%macro cv_model_pvalue_data(datapath=,
							outputpath=,
							dsname=,
							where_clause=,
							trait=,
							trait_type=,
							freq=,
							_namedat=,
							model_names=,
							model_labels=,
							models_comparison_output=
							);

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));
%put prefix=&prefix;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname datalib "&datapath";
	%let datalib = datalib;
%end;
%else %do;
    %let datalib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cvpname = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%if %sysfunc(exist(&cvoutlib..&prefix._cvr_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cvr_all);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end; 
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._cvr_all DOES NOT EXIST.;
	%goto exit;
%end;

%let wc=%superq(where_clause);
data train;
    set
    %if %length(&wc) > 0 %then %do;
        &__cvpname(where=(%unquote(&wc)))
    %end;
    %else %do;
        &__cvpname
    %end;
    ;
	__id = _n_;
	keep &trait __id;
	%if &trait_type = qualitative %then %do;
        %str(if &trait ne " " then output;);
	%end;
    %else %do;
        %str(if &trait ne . then output;);
	%end;
run;

%NObsVars(train);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
    %if %length(&wc) > 0 %then %do;
		%put ERROR: &__cvpname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do;
		%put ERROR: &__cvpname DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
	%end;
	%goto exit;
%end;

* RETRIEVE MODEL NAMES AND LABELS;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);
%do i=1 %to &n_models;
   %let mdlname&i = %scan(&model_names,&i," ");
   %let mdllabel&i = %scan(&model_labels,&i,"|");
	%put mdlname&i=&&mdlname&i;
	%put mdllabel&i=&&mdllabel&i;
%end;

data preds;
	set &cvoutlib..&prefix._cvr_all;
	where 
	%do i=1 %to &n_models;
		%let mdlname = &&mdlname&i;
		%if &i > 1 %then %do;
			or METHOD="&mdlname"
		%end;
		%else %do;
			METHOD="&mdlname"
		%end;
	%end;
	;
run;

%NObsVars(preds);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
	%put ERROR: MODEL &model_name DOES NOT EXIST IN &cvoutlib..&prefix._cvr_all.;
	%goto exit;
%end; 

* SINCE DIFFERENT MODELS CAN HAVE DIFFERENT SUFFICES FOR p_, 
  REMOVE THOSE THAT ALL MISSING; 
/*
ods exclude none;
ods results;
*/
proc means data=preds noprint;
  	var p_:;
  	output out=opm;
run;
  
data opm;
 	set opm;
  	if _stat_ = "N";
run;
  
proc transpose data=opm out=opmt;
  	var p_:;
run;
 
data _null_;
  	set opmt end=the_end;
  	if (the_end) then call symputx("nobs",_n_);
run;
  
%put nobs = &nobs;
%do i=1 %to &nobs;
  	%let drop&i = ;
%end;
  
data _null_;
  	set opmt;
  	if col1 = 0 then do;
  		call symputx("drop"||trim(left(_n_)),_name_);
  	end;
run;
  
%let dropem = ;
%do i=1 %to &nobs;
	%if %length(&dropem) > 0 and &dropem ne "" %then %do;
		%let dropem = &dropem; 
	%end;
	%if %length(&&drop&i) > 0 and &&drop&i ne "" %then %do;
		%let dropem = &dropem &&drop&i;
	%end;
%end;
  
%if %length(&dropem) > 0 and &dropem ne "" %then %do;
	%put Dropping predicted probability variables &dropem because all values are missing for this model.;
	data preds;
	  	set preds;
	  	drop &dropem;
	run;
%end;
%NObsVars(preds);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
	%put ERROR: PVALES CONTAINS 0 OBSERVATIONS OR NO VARIABLES.;
	%goto exit;
%end; 
proc sort data=preds;
	by __id;
run;

%let trPredictedProbabilityofX = %nrbquote(%LocaleGet(PredictedProbabilityofX_Text));
%let trObsRankedbyPredAvgdoverCVIts = %nrbquote(%LocaleGet(ObservationsRankedbyPredAvgdoverCVIterations_Text));

%if %upcase(&trait_type)=QUALITATIVE %then %do;

	* RETRIEVE TRAIT LEVELS, P_ NAMES, AND CORRESPONDING PROPORTIONS;
	proc freq data=train noprint;
		tables &trait/out=ft;
	run;
	
	proc sort data=ft;
		by PERCENT;
	run;

	%let _fid_ = %sysfunc(open(ft));
	%let traitnum=%sysfunc(varnum(&_fid_, &trait));
	%let traitype=%sysfunc(vartype(&_fid_, &traitnum));
	%let percentnum = %sysfunc(varnum(&_fid_, PERCENT));	
		
	%let _nid_ = %sysfunc(open(&cvoutlib..&_namedat));
	%let nclasses = %sysfunc(attrn(&_nid_, NOBS));
	%put nclasses=&nclasses;
	%if &nclasses > 0 %then %do;
		%let classnum=%sysfunc(varnum(&_nid_, CLASS));
		%let pvarnum=%sysfunc(varnum(&_nid_, P_PROB));
		%let classtype=%sysfunc(vartype(&_nid_, &classnum));
		%do c=1 %to &nclasses;
			%let rc=%sysfunc(fetchobs(&_nid_, &c));
			%if &classtype=C %then %do;
				%let class&c=%sysfunc(getvarc(&_nid_, &classnum)); 
			%end;
			%else %do;
				%let class&c=%sysfunc(getvarn(&_nid_, &classnum));
			%end;
			%let pvar&c=%sysfunc(getvarc(&_nid_, &pvarnum));	
			%do i=1 %to &nclasses;
				%let rc=%sysfunc(fetchobs(&_fid_, &i));
				%if &traitype=C %then %do;
					%let class=%sysfunc(getvarc(&_fid_,&traitnum)); 
				%end;
				%else %do;
					%let class=%sysfunc(getvarn(&_fid_,&traitnum)); 
				%end;
				%if &class=&&class&c %then %do;
					%let percent&c=%sysfunc(getvarn(&_fid_,&percentnum));
				%end;
			%end; 
		%end;
	%end;
	%let _nid_ = %sysfunc(close(&_nid_));
	%let _fid_ = %sysfunc(close(&_fid_));

	data preds;
		merge preds train;
		by __id;
	run;	

	data preds;
		set preds;
		%do c=1 %to &nclasses;
			%let trPredictedProbabilityofTrait = %nrbquote(%LocaleFill(%nrbquote(&trPredictedProbabilityofX), f1=%nrbquote(&trait)));
			label &&pvar&c="&trPredictedProbabilityofTrait=&&class&c";
		%end;
	run;

	* LOOP OVER _NCLASSES_ TO GENERATE PVALUESxx OUTPUT FOR EACH TRAIT CLASS;
	%do c=1 %to &nclasses;
		%let pname = pred_prob&c;
		%let pvname = &&pvar&c;
		%let lp = %length(&&pvar&c);
		%if &lp > 28 %then %do;
			%let pvname = %substr(&&pvar&c,1,14)%substr(&&pvar&c,%eval(&lp-13),14);
		%end;
		
		data &pname;
			set preds;
			%if &traitype=N %then %do;
				if trim(left(&trait)) = &&class&c then output;
			%end;
			%else %do;
				if trim(left(&trait)) = "&&class&c" then output;
			%end;
		run;

		%NObsVars(&pname);
		%let _pObs_ = &nObs;
		%if &_pObs_ %then %do;
			proc sort data=&pname;
				by __id;
			run;

			proc means data=&pname noprint;
				class __id;
				var &&pvar&c;
				output out=sumstat mean=Ave_&pvname;
			run;

			data sumstat;
				set sumstat;
				keep __id Ave_&&pvar&c;
				label Ave_&pvname ="Average Predicted Probability of &trait=&&class&c";
				where __id ne .;
			run;
			
			%NObsVars(sumstat);
			%if &nObs and &nVars %then %do;
				proc sort data=sumstat out=sumstat;
					by descending Ave_&pvname;
				run;

				data sumstat;
					set sumstat;
					order = _n_;
					label order = "&trObsRankedbyPredAvgdoverCVIts";
				run;
					
				proc sort data=sumstat out=sumstat;
					by __id;
				run;

				data &pname;
					merge &pname sumstat;
					by __id;
					if missing(actualp) then delete;
				run;

				proc sort data=&pname;
					by order;
				run;
			%end;
		%end;
	
		%if &c=1 %then %do;
			data &cvoutlib..&prefix._pprob;
				set &pname;
			run;
		%end;
		%else %do;
			data &cvoutlib..&prefix._pprob;
				set &cvoutlib..&prefix._pprob
					&pname;
			run;
		%end;
	%end;
%end;
%else %do;
	%let tname = &trait;
	%let lt = %length(&trait);
	%if &lt > 28 %then %do;
		%let pvname = %substr(&trait,1,14)%substr(&trait,%eval(&lt-13),14);
	%end;
	
	proc means data=train noprint;
		var &trait;
		output out=avep mean=Ave_p_&tname;
	run;
	data _null_;
		set avep;
		call symput("average_pvalues", put(Ave_p_&tname, best12.));
	run;

	proc means data=preds noprint;
		class __id;
		/* var p_&trait; */
		/* FIX S0760403 */
		var Predicted_value;
		output out=sumstat mean=Ave_p_&tname;
	run;

	data sumstat;
		set sumstat;
		keep __id Ave_p_&tname;
		label Ave_p_&tname = "Average Predicted: &trait";
	    where __id ne .;
	run; 
	
	%NObsVars(sumstat);
	%if &nObs and &nVars %then %do;
		proc sort data=sumstat out=sumstat;
			by descending Ave_p_&tname;
		run;

		data sumstat; 
			set sumstat; 
			order = _n_;
			label order="&trObsRankedbyPredAvgdoverCVIts";
		run;

		proc sort data=sumstat out=sumstat;
			by __id;
		run;

		data &cvoutlib..&prefix._pprob;
			merge preds sumstat;
			by __id;
		run;
		/*
		proc sort data=&cvoutlib..&prefix._pprob;
			by order;
		run;
		*/
	%end;
%end;

* merge in model labels;
data lab;
	length Model $ 64 Method $ 32;
	%do i=1 %to &n_models;
		Model = "&&mdllabel&i";
		Method = "&&mdlname&i";
		output;
	%end;
run;
proc sort data=&cvoutlib..&prefix._pprob;
	by Method;
run;
data &cvoutlib..&prefix._pprob;
	merge lab &cvoutlib..&prefix._pprob;
	by Method;
run;

%let traitLabel =;
%CheckLabel(&cvoutlib..&prefix._pprob, trait, _New=traitLabel);

* JSL PLOTTING CODE, launched from ExitAndRetrieveLogLstFromJMP in UtilityMacros.sas;
filename jslfile "&outputpath.&prefix._pprob.jsl";
data _null_;
	file jslfile encoding="utf-8";
    put "//!";
    put "Names Default to Here(1);";
    put "// parameters for cross-validation model comparison pvalues plots;";
	put "trait = ""&trait"";";
	put "traitlabel = ""&traitLabel"";";
	%if %upcase(&trait_type)=QUALITATIVE %then %do;
		%do c=1 %to &nclasses;
			%if &c=1 %then %do;
				%let average_pvalues = &&percent&c;
				%let trait_classes = &&class&c;
			%end;
			%else %do;
				%let average_pvalues = &average_pvalues,&&percent&c;
				%let trait_classes = &trait_classes,&&class&c;
			%end;
		%end;
		put "trait_classes = ""&trait_classes"";";
	%end;
	%else %do;
		put "plabels = ""Predicted_Value"";";
	%end;
	put "pvalues_output = ""&prefix._pprob"";";
	put "average_pvalues = ""&average_pvalues"";";
	put "if(Host Is(Windows),";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
run;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2009 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: CV_MODEL_ROC_DATA  
**  AUTHOR:  saspyt, Pei-Yi Tan 
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    05/01/09
**  PURPOSE: Generate data set to plot ROC curves  
**  HISTORY: action                                        date     who
**           Created.                                      05/01/09 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:   Library path of output data sets 
**  TRAIT:        Trait variable name
**  EVENT:   		Event indicator
**  MODEL_NAMES:  List of model names
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**
**  NOTES:
**  Assumes _cvr_all data set 
************************************************************************;
%macro cv_model_roc_data(outputpath=,
						 trait=,
						 event=,
						 model_names=,
						 model_labels=,
						 models_comparison_output=
						 );

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));
%put prefix=&prefix;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%if %sysfunc(exist(&cvoutlib..&prefix._cvr_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cvr_all);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._cvr_all DOES NOT EXIST.;
	%goto exit;
%end;

* RETRIEVE MODEL NAMES;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
    %let mdllabel&i = %scan(&model_labels,&i,"|");
%end;

data preds;
	set &cvoutlib..&prefix._cvr_all;
	keep METHOD;
	where 
	%do i=1 %to &n_models;
		%let mdlname = &&mdlname&i;
		%if &i > 1 %then %do;
			or METHOD="&mdlname"
		%end;
		%else %do;
			METHOD="&mdlname"
		%end;
	%end;
	;
run;

* USE PROC FREQ TO DETERMINE UNIQUE MODEL NAMES;
proc freq data=preds noprint;
	tables METHOD/out=fmethod;
run;
%let _fmid_ = %sysfunc(open(work.fmethod));
%let n_models = %sysfunc(attrn(&_fmid_, NOBS));
%let mthnum = %sysfunc(varnum(&_fmid_, METHOD));
%do nm=1 %to &n_models;
	%let rc = %sysfunc(fetchobs(&_fmid_, &nm));
	%let mdlname&nm = %sysfunc(getvarc(&_fmid_, &mthnum));
	%let mdlname = &&mdlname&nm;
	%put roc mdlname &nm = = &mdlname;
	data preds;
		set &cvoutlib..&prefix._cvr_all;
		where METHOD="&mdlname";
		_i = iter;
		keep method __id _i _k &trait actualp p_:;
	run;

	%locate_pevent_isevent(data=preds,
	                       trait=&trait,
	                       event=&event,
	                       actual=actualp);	
	                       
	* COMPUTE ROC STATISTICS;
	%compute_roc(data=preds,
				 pevent=_pevent,
	             isevent=_isevent,
	             outdata=auc);

	/*
	data outlib.preds;
		set preds;
	run;
	*/

	data auc;
		METHOD="&mdlname";
		set auc;
	run;
	
	proc sort data=auc;
		by _1MSPEC_ _SENSIT_;
	run;
	
	/*
	data outlib.auc;
		set auc;
	run;
	*/

	%if &nm=1 %then %do;
		data &cvoutlib..&prefix._roc;
			set auc;	
		run;
		data predsall;
			set preds;
		run;
	%end;
	%else %do;
		data &cvoutlib..&prefix._roc;
			set &cvoutlib..&prefix._roc auc;
		run;
		data predsall;
			set predsall preds;
		run;
	%end;
	
	/*data OutLib.predsall; set predsall; run;*/
%end;
%let _fmid_ = %sysfunc(close(&_fmid_));

* merge in model labels;
data lab;
	length Model $ 64 Method $ 32;
	%do i=1 %to &n_models;
		Model = "&&mdllabel&i";
		Method = "&&mdlname&i";
		output;
	%end;
run;
proc sort data=&cvoutlib..&prefix._roc;
	by Method;
run;
data &cvoutlib..&prefix._roc;
	merge lab &cvoutlib..&prefix._roc;
	by Method;
run;

* USE PROC LOGISTIC TO COMPARE AUC VALUES;
data predsall;
	set predsall;
	if missing(__id) then delete;
run;
proc sort data=predsall;
	by __id _i _k _isevent;
run;

/*data OutLib.predsall02; set predsall; run;*/

proc transpose data=predsall out=&cvoutlib..&prefix._rocin(drop=_name_ _label_);
	by __id _i _k _isevent;
	var _pevent;
	id method;
run;
* set all missing values to 0.5;
data &cvoutlib..&prefix._rocin;
	set &cvoutlib..&prefix._rocin;
	array m{*} M:;
	do i=1 to dim(m);
		if missing(m[i]) then m[i] = 0.5;
	end;
run;
* options notes;
proc logistic data=&cvoutlib..&prefix._rocin;
	Null_Model: model _isevent(event='1') = ;
	%do i=1 %to &n_models; 
		roc "&&mdllabel&i" pred=&&mdlname&i;
	%end;
	roccontrast / estimate=allpairs;
	ods output 
		ROCAssociation=&cvoutlib..&prefix._rocas 
		ROCContrastEstimate=&cvoutlib..&prefix._rocce
		ROCContrastTest=&cvoutlib..&prefix._rocct
	;
run;

* WRITE OUT MODEL NAMES AND LABELS TO .jsl FILE;
filename jslfile "&outputpath.&prefix._model_names.jsl";
data _null_;
	file jslfile encoding="utf-8";
	put "selected_model_names = {"@@;
	%do i=1 %to &n_models; 
		put """&&mdlname&i"""@@;
		%if &i < &n_models %then %do;
			put ","@@;
		%end;
	%end;
	put "};";
	put "selected_model_labels = {"@@;
	%do i=1 %to &n_models; 
		put """&&mdllabel&i"""@@;
		%if &i < &n_models %then %do;
			put ","@@;
		%end;
	%end;
	put "};";
run;

%exit:;
%mend;

%macro dupobs(indata=,
              outdata=,
              var=,
              nbins=,
              infreq=,
              outfreq=);

%if %sysfunc(exist(&indata)) = 1 %then %do;
	%NObsVars(&indata);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &indata CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &indata DOES NOT EXIST.;
	%goto exit;
%end;

data inputdsx;
	set &indata;
	%if %length(&infreq) > 0 and &infreq ne "" %then %do;
		if &infreq > 0 and &var ne . then output;
	%end;
	%else %do;
		infreq = 1;
		if &var ne . then output;
	%end;
run;

%if %sysfunc(exist(inputdsx)) = 1 %then %do;
	%NObsVars(inputdsx);
	%if &nObs and &nVars %then %do;
		proc means data=inputdsx noprint;
			var &var;
			output out=mean min=minvar max=maxvar;
		run;
		data _null_;
			set mean;
			call symput("minvar", minvar);
			call symput("maxvar", maxvar);
			call symput("sum", _FREQ_);
		run;

		data bin;
			set inputdsx;
			bin = floor(((&var - &minvar)/(&maxvar - &minvar))*&nbins) + 1;
			if bin >= &nbins then bin = &nbins;
		run;
		proc freq data=bin noprint;
			tables bin/out=counted;
		run;

		proc means data=counted noprint;
			output out=binsum sum=binsum;
		run;
		data _null_;
			set binsum;
			call symput("binsum", binsum);
		run;

		* NORMALIZE COUNTED SO BIN SUM TO 1;
		data counted;
			set counted;
			normbin=bin/&binsum;
		run;

		%let _cid_ = %sysfunc(open(counted));
		%let _ncounted_ = %sysfunc(attrn(&_cid_, NOBS));
		%if &_ncounted_ > 0 %then %do;
			%do i=1 %to &_ncounted_;
				%let normbinnum=%sysfunc(varnum
				                      (&_cid_,normbin));
				%let binnum=%sysfunc(varnum
				                      (&_cid_,bin));
				%let rc=%sysfunc(fetchobs(&_cid_,&i));
				%let bin&i=%sysfunc(getvarn
				                   (&_cid_,&binnum));
				%let counted&i=%sysfunc(getvarn
				                   (&_cid_,&normbinnum));
			%end;
		%end;
		%let _cid_ = %sysfunc(close(&_cid_));

		%if &_ncounted_ > 0 %then %do;
			data &outdata;
				set inputdsx;
				%do idsx=1 %to &_ncounted_;
				    bin=&&bin&idsx;
					outfreq=&&counted&idsx;
					&var=(bin-1)*(&maxvar-&minvar)/&nbins + &minvar;
					output;
				%end;
			run;
		%end;
	%end;
%end;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: ENSEMBLE  
**  AUTHOR:  saspyt, Pei-Yi Tan 
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    03/13/08
**  PURPOSE: Forward Stepwise with-replacement ensemble 
**  HISTORY: action                                        date     who
**           Created.                                      03/13/08 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:	Library path of output data sets 
**  TRAIT:      Trait variable name
**  TRAIT_TYPE:	Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  NCLASSES:	Number of qualitative trait variable category
**  FREQ: 		Frequency variable name (optional)
**  CVRALL_DSNAME: Models scored data set 
**  MODEL_NAMES:   List of model names
**                 (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  CV_HOLDOUT_ITERATION_START:	Random hold-out iteration starting index
**  CV_HOLDOUT_ITERATION_END: Random hold-out iteration ending index
**  STARTSTEP:  ensemble forward selection starting step
**  MAXSTEPS: 	Maximum number of ensemble forward selection steps 		
** 	PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)	
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**
**  NOTES: Compare the ensemble models created from ensemble and 
**         ensemble_cv macros, the results indicate both macros 
**         are not selecting the correct best cvmc model at the 1st step 
**         due to data was not partitioned consistently between 
**         cvmc k-fold random partition and the ensemble macros. 
**         Need more work on this area. 
************************************************************************;
%macro ensemble(outputpath=,
			    trait=,
				trait_type=,
				nclasses=,
				freq=,
			    cvrall_dsname=, 
			    model_names=,
				cv_holdout_iteration_start=,
				cv_holdout_iteration_end=,
				startstep=,
				maxsteps=,
				prevalences=proportional,
				cv_custompriors=
			    );

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%let m=1;
%do %while(%length(%scan(&model_names,&m," ")));
    %let m = %eval(&m+1);
%end;
%let n_models = %eval(&m-1);
%do m=1 %to &n_models;
    %global mdlname&m;
    %let mdlname&m = %scan(&model_names,&m," ");
%end;

%if %sysfunc(exist(&cvoutlib..&cvrall_dsname)) = 1 %then %do;
	%NObsVars(&cvoutlib..&cvrall_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&cvrall_dsname CONTAINS 0 OBSERVATIONS.;
		%goto ensemble_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&cvrall_dsname DOES NOT EXIST.;
	%goto ensemble_exit;
%end;

* RETRIEVE NOBS IN THE ORIGINAL INPUT DATA;
proc freq data=&cvoutlib..&cvrall_dsname noprint;
	tables METHOD*ITER/out=d;
run;
data _null_;
	set d;
	if _n_ = 1 then 
		call symput("_NOBS_", put(COUNT, 12.0));
run;

* SUBSET INDIVIDUAL MODEL PREDICTIONS;
%do m=1 %to &n_models;
	%let mdlname = &&mdlname&m;
	%if %upcase(&trait_type)=QUALITATIVE %then %do; 
		* RENAME ACTUALP AND P_ VARIABLE NAMES;  
		data &mdlname;
			set &cvoutlib..&cvrall_dsname;
			array rp{&nclasses} 
			%do c=1 %to &nclasses;
				rp&c._m&m
			%end;
			;
			array _p p_:;
			keep METHOD ITER _Obs actualp rp: &trait;
			ri = 0;
			if METHOD = "&mdlname" then do;
				do i=1 to dim(_p);
					if not missing(_p(i)) then do;
						ri = ri + 1;
						rp(ri) = _p(i);
					end;
				end;
				output;
			end;
		run;
	%end;
	%else %do;
		data &mdlname;
			set &cvoutlib..&cvrall_dsname;
			keep METHOD ITER _Obs Predicted_Value &trait;
			if METHOD = "&mdlname" then output;
		run;
	%end;

	%if %sysfunc(exist(&mdlname)) = 1 %then %do;
		%NObsVars(&mdlname);
		%if &nObs and &nVars %then %do;
			proc sort data=&mdlname;
				by ITER _Obs;
			run;
		%end;
	%end;
%end;

%global bestm best_model;

%do s=&startstep %to &maxsteps;
	%put;
	%put Starting ensemble step &s of &maxsteps;

	%if %upcase(&trait_type)=QUALITATIVE %then %do; 
		%if &s = &startstep %then %do;
			* INITIALIZE ENSEMBLE DATA;
			data &cvoutlib..ensemble_accum;
				keep sum_actualp sum_rp: ITER _Obs;
				retain sum_actualp 0;
				%do c=1 %to &nclasses;
					retain sum_rp&c 0;
				%end;
				do iter=&cv_holdout_iteration_start to &cv_holdout_iteration_end;
					do _Obs = 1 to &_NOBS_;
						output;
					end;
				end;
			run;

			* INITIALIZE STEP HISTORY TO CONTAIN (AT LEAST) NAME OF MODEL ENTERED AT STEP S.;                                                   
			data  &cvoutlib..ensemble_steps;                                                                                                                   
				length METHOD $64 step median_ase 8;                                                                                                           
				keep method step median_ase;                                                                                                        
			run;     
		%end;
		
	    data median_ase;
			length METHOD $64 m median_ase 8;                                                                                                           
			keep method m median_ase;                                                                                                        
		run;     

		* CREATE CANDIDATE ENSEMBLES FOR EACH ITERATION;
		%do m=1 %to &n_models;

			%let mdlname = &&mdlname&m;

			%if %sysfunc(exist(&mdlname)) = 1 %then %do;
				%NObsVars(&mdlname);
				%if &nObs and &nVars %then %do;
					data candidate&m;
						merge &cvoutlib..ensemble_accum &mdlname;
						if actualp=. or sum_actualp=. then do;
							meanp = .;
							correct = .;
							%do c=1 %to &nclasses;
								meanrp&c = .;
							%end;
						end;
						else do;
							meanp = (sum_actualp + actualp)/&s;
							if meanp > 0.5 then 
								correct = 1;
							else 
								correct = 0;
							%do c=1 %to &nclasses;	
								meanrp&c = (sum_rp&c + rp&c._m&m)/&s;
							%end;
						end;
					run;

					* FOR EACH MODEL, COMPUTE ASE FOR EACH ITERATION THEN TAKE MEDIAN; 
					* SAVE MEDIAN_ASE FOR EACH CANDIDATE_M IN A SINGLE DATA SET;                                                                
					%if %sysfunc(exist(candidate_ase)) = 1 %then %do;
						%NObsVars(candidate_ase);
						%if &nObs or &nVars %then %do;
							proc datasets library=work nolist;
								delete candidate_ase;
							quit; 
						%end;
					%end;

					* CREATE CANDIDATE_ASE DATA SET;
					* NROWS = MAXITER, 1 COLUMN FOR EACH FIT STATISTICS (ASE, ACC);                               
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end; 
						%if %sysfunc(exist(candidate&m)) = 1 %then %do;
							%NObsVars(candidate&m);
							%let _candmObs_ = &nObs;
			 				%if &_candmObs_ %then %do;
								data p;
									set candidate&m;
									where ITER=&iter;
									%do c=1 %to &nclasses;	
										rename meanrp&c = p_&c;
									%end;
									if meanp ne . then output;
								run;

								%NObsVars(p);
								%let _pObs_ = &nObs;
								%if &_pObs_ %then %do;
									* ONLY USE EVERY OTHER OBSERVATION TO TRAIN THE ENSEMBLE;
									data p1;
										set p;
										if mod(_n_,2) = 1;
									run;

									* COMPUTE MODEL ASE;
									%compute_ase_nominal(data=p1,
									                     output=ase&iter,
									                     actual=meanp,
									                     nclasses=&nclasses,
									                     mdlname=&mdlname,
									                     freq=&freq,
									                     prevalences=&prevalences,
														 cv_custompriors=%nrbquote(&cv_custompriors),
									                     trait=&trait
									                     );

									* APPEND ASE TO CANDIDATE_ASE DATA SET;
									data ase&iter;	
										set ase&iter;
										ITER = &iter;
										keep ITER ASE;
									run;

									* COMPUTE MODEL Accuracy;
									%compute_acc_nominal(data=p1,
														 output=acc&iter,
														 trait=&trait,
														 correct=correct,
														 mdlname=&mdlname,
														 freq=&freq,
									                     prevalences=&prevalences,
														 cv_custompriors=%nrbquote(&cv_custompriors)
									                     );
										
									* APPEND ACC TO CANDIDATE_ASE DATA SET;
									data acc&iter;	
										set acc&iter;
										ITER = &iter;
										keep ITER Accuracy;
									run;

									* COMPUTE MODEL AEC;
									%if %length(&cv_customcosts) %then %do;
										%compute_aec_nominal(
											 data=p1,
											 output=aec&iter,
											 trait=&trait,
											 expcost=expected_cost,
											 mdlname=&mdlname,
											 freq=&freq
										);

										* APPEND AEC TO CANDIDATE_ASE DATA SET;
										data aec&iter;	
											set acc&iter;
											ITER = &iter;
											keep ITER AEC;
										run;
										%let aec = aec&iter;
									%end;
									%else %do;
										%let aec = ;
									%end;

									data aseacc&iter;
										merge ase&iter acc&iter &aec;
									run;

									proc append base=candidate_ase data=aseacc&iter;
									run;
								%end; 
							%end;
						%end;
					%end;                                                                                                                         
				  
					%NObsVars(candidate_ase);
					%if ^&nObs or ^&nVars %then %do;
						%put MODEL &mdlname FAILED IN ALL ITERATIONS, IT SHOULD BE REMOVED.;
					%end;
					%else %do;
						* COMPUTE MEDIAN FIT STATISTICS AND APPEND TO MEDIAN_ASE;
						proc means data=candidate_ase noprint;
							var ASE Accuracy %if %length(&cv_customcosts) %then %do; AEC %end; ;
							output out=median 
								   median(ase)=median_ase 
								   median(Accuracy)=median_acc
								   %if %length(&cv_customcosts) %then %do;
								   	median(aec)=median_aec
								   %end;
							;
						run;
						
						data median;
							set median;
							length METHOD $64;
							keep method m median_ase median_acc %if %length(&cv_customcosts) %then %do; median_aec %end; ;
							method = "&mdlname";
							m = &m;
							call symputx("median_ase",median_ase);
							call symputx("median_acc",median_acc);
							%if %length(&cv_customcosts) %then %do;
								call symputx("median_aec",median_aec);
							%end;
						run;
						%put Step &s of &maxsteps, Model &m of &n_models, median_ase=&median_ase median_acc=&median_acc;
						%if %length(&cv_customcosts) %then %do;
							%put median_aec=&median_aec;
						%end;

						%if %sysfunc(exist(median)) = 1 %then %do;
							%NObsVars(median);
							%let _mObs_ = &nObs;
							%if &_mObs_ %then %do;
								%if %sysfunc(exist(median_ase)) = 1 %then %do;
									%NObsVars(median_ase);
									%let _aObs_ = &nObs; 
									data median_ase;
										set 
										%if &_aObs_ %then %do;
											median_ase	
										%end;
										median;
									run;
								%end;	
								%else %do;
									data median_ase;
										set median;
									run;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;

	 	* FIND THE BEST MODEL IN DATA SET MEDIAN_ASE;
		%if %sysfunc(exist(median_ase)) = 1 %then %do;
			%NObsVars(median_ase);
			%if &nObs and &nVars %then %do;
				proc sort data=median_ase;
					by descending median_acc median_ase;
				run; 
			                                                                                                                                        
			    data best;
					set median_ase;
					step = &s;
					if _n_ = 1 then do;
						call symput("best_model", trim(left(method)));
						call symput("bestm", trim(left(put(m, best12.))));
						output;
					end;
				 run;
				%put bestm=&bestm best_model=&best_model;
			%end;
		%end;

		* FOR BEST MODEL, COMPUTE METRICS ON OTHER HALF OF HOLDOUT SETS;
		%let first = 1;
		%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end; 
			%if %sysfunc(exist(candidate&bestm)) = 1 %then %do;
				%NObsVars(candidate&bestm);
				%let _bestObs_ = &nObs;
				%if &_bestObs_ %then %do; 
					data p;
						set candidate&bestm;
						where ITER=&iter;
						%do c=1 %to &nclasses;	
							rename meanrp&c = p_&c;
						%end;
						if meanp ne . then output;
					run;

					%NObsVars(p);
					%let _pObs_ = &nObs;
					%if &_pObs_ %then %do;

						* USE OBSERVATIONS NOT USED ABOVE;
						data p1;
							set p;
							if mod(_n_,2) = 0;
						run;

						* COMPUTE MODEL ASE;
						%compute_ase_nominal(data=p1,
											 output=ase&iter,
											 actual=meanp,
											 nclasses=&nclasses,
											 mdlname=&best_model,
											 freq=&freq,
											 prevalences=&prevalences,
											 cv_custompriors=%nrbquote(&cv_custompriors),
							                 trait=&trait
											 );

						* APPEND ASE TO BEST_ASE DATA SET;
						data ase&iter;	
							set ase&iter;
							ITER = &iter;
							keep ITER ASE;
						run;

						* COMPUTE MODEL Accuracy;
						%compute_acc_nominal(data=p1,
											 output=acc&iter,
											 trait=&trait,
											 correct=correct,
											 mdlname=&best_model,
											 freq=&freq,
							                 prevalences=&prevalences,
											 cv_custompriors=%nrbquote(&cv_custompriors)
							                 );

						* APPEND ACC TO BEST_ASE DATA SET;
						data acc&iter;	
							set acc&iter;
							ITER = &iter;
							keep ITER Accuracy;
						run;

						* COMPUTE MODEL AEC;
						%if %length(&cv_customcosts) %then %do;
							%compute_aec_nominal(
								 data=p1,
								 output=aec&iter,
								 trait=&trait,
								 expcost=expected_cost,
								 mdlname=&mdlname,
								 freq=&freq
							);

							* APPEND AEC TO CANDIDATE_ASE DATA SET;
							data aec&iter;	
								set acc&iter;
								ITER = &iter;
								keep ITER AEC;
							run;
							%let aec = aec&iter;
						%end;
						%else %do;
							%let aec = ;
						%end;
						
						data aseacc&iter;
							merge ase&iter acc&iter &aec;
						run;
						
						%if &first = 1 %then %do;
							data best_ase;
								set aseacc&iter;
							run;
							%let first = 0;
						%end;
						%else %do;
							data best_ase;
								set best_ase aseacc&iter;
							run;
						%end;	
					%end;
				%end;
			%end;
		%end;                                                                                                                         

		* COMPUTE MEDIAN FIT STATISTICS;
		%if %sysfunc(exist(best_ase)) = 1 %then %do;
			%NObsVars(best_ase);
			%if &nObs and &nVars %then %do;
				proc means data=best_ase noprint;
					var ASE Accuracy %if %length(&cv_customcosts) %then %do; AEC %end; ;
					output out=median 
							median(ase)=median_ase 
							median(Accuracy)=median_acc
							%if %length(&cv_customcosts) %then %do;
								median(aec)=median_aec
							%end;
					;
				run;
				data median;
					set median;
					length METHOD $64;
					keep method m step median_ase median_acc 
						%if %length(&cv_customcosts) %then %do; median_aec %end; median_rmse 
						one_minus_median_ase one_minus_median_rmse;
					method = "&best_model";
					m = &bestm;
					step = &s;
					median_rmse = sqrt(median_ase);
					one_minus_median_ase = 1 - median_ase;
					one_minus_median_rmse = 1 - median_rmse;
					call symputx("best_median_ase",median_ase);
					call symputx("best_median_acc",median_acc);
					%if %length(&cv_customcosts) %then %do; 
						call symputx("best_median_aec",median_aec)
					%end;
				run;
				%put hold_out_best_median_ase=&best_median_ase hold_out_best_median_acc=&best_median_acc;
				%if %length(&cv_customcosts) %then %do; 
					%put hold_out_best_median_aec=&best_median_aec;
				%end;

			  	* UPDATE ENSEMBLE_STEP BY APPENDING MODEL NAME AND MEDIAN_ASE;  
				%if %sysfunc(exist(&cvoutlib..ensemble_steps)) = 1 %then %do;
					%NObsVars(&cvoutlib..ensemble_steps);
					%let _estpObs_ = &nObs;
					data &cvoutlib..ensemble_steps;  
						set 
						%if &_estpObs_ %then %do;
							&cvoutlib..ensemble_steps
						%end; 
							median;
					run; 
				%end;
				%else %do;
					data &cvoutlib..ensemble_steps;  
						set median;
					run; 
				%end;
			  	
			 	* UPDATE ENSEMBLE_ACCUM WITH MODEL BEST_M;   
				%if %sysfunc(exist(&best_model)) = 1 %then %do;
					%NObsVars(&best_model);
					%if &nObs and &nVars %then %do;
					    data &cvoutlib..ensemble_accum;                                                                                                           
							merge &cvoutlib..ensemble_accum &best_model;                                                                                        
							keep iter _obs sum:;                                                                                              
							sum_actualp  = sum_actualp + actualp;
							%do c=1 %to &nclasses;
								sum_rp&c = sum_rp&c + rp&c._m&bestm; 
							%end; 
						run;      
					%end;
				%end;
			%end;
		%end;
	%end;
	%else %if %upcase(&trait_type)=QUANTITATIVE %then %do; 

		%if &s = &startstep %then %do;
			* INITIALIZE ENSEMBLE DATA;
			data &cvoutlib..ensemble_accum;
				keep sum_pred ITER _Obs;
				retain sum_pred 0;
				do iter = &cv_holdout_iteration_start to &cv_holdout_iteration_end;
					do _Obs = 1 to &_NOBS_;
						output;
					end;
				end;
			run;
			
			* INITIALIZE STEP HISTORY TO CONTAIN (AT LEAST) NAME OF MODEL ENTERED AT STEP S.;                                                   
			data  &cvoutlib..ensemble_steps;                                                                                                                   
				length METHOD $64 step median_ase 8;                                                                                                           
				keep method step median_ase;                                                                                                        
			run;     
		%end;

	   data median_ase;
			length METHOD $64 m median_ase 8;                                                                                                           
			keep method m median_ase;                                                                                                        
		run;     

		%if &cv_alossprop ne 0.5 %then %do;
			%let cv_alossweight = Asymmetric;
		%end;
		%else %do;
			%let cv_alossweight = ;
		%end;

		* CREATE CANDIDATE ENSEMBLES FOR EACH ITERATION;
		%do m=1 %to &n_models;
			%let mdlname = &&mdlname&m;
			%if %sysfunc(exist(&mdlname)) = 1 %then %do;
				%NObsVars(&mdlname);
				%if &nObs and &nVars %then %do;
					data candidate&m;
						merge &cvoutlib..ensemble_accum &mdlname;
						if Predicted_Value = . or sum_pred = . then do;
							meanpred = .;
						end;
						else do;
							meanpred = (sum_pred + Predicted_Value)/&s;
						end;
					run;

					* FOR EACH MODEL, COMPUTE ASE FOR EACH ITERATION THEN TAKE MEDIAN; 
					* SAVE MEDIAN_ASE FOR EACH CANDIDATE_M IN A SINGLE DATA SET;                                                                
					%if %sysfunc(exist(candidate_ase)) = 1 %then %do;
						%NObsVars(candidate_ase);
						%if &nObs or &nVars %then %do;
							proc datasets library=work nolist;
								delete candidate_ase;
							quit; 
						%end;
					%end;

					* CREATE CANDIDATE_ASE DATA SET;
					* NROWS = MAXITER, 1 COLUMN FOR PREDICTED_VALUE;   
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
						%if %sysfunc(exist(candidate&m)) = 1 %then %do;
							%NObsVars(candidate&m);
							%if &nObs and &nVars %then %do;
							
								data p;
									set candidate&m;
									where ITER=&iter;
									%if &cv_alossprop ne 0.5 %then %do;
										_resid = &trait - meanpred;
										if missing(_resid) then &cv_alossweight = .;
										else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
										else &cv_alossweight = 2 * (1 - &cv_alossprop);
									%end;
									
									* ONLY USE EVERY OTHER OBSERVATION TO TRAIN THE ENSEMBLE;
									if mod(_n_, 2) = 1 and meanpred ne . then output;
								run;

								%nObsVars(p);
								%if &nObs and &nVars %then %do;
									* COMPUTE MODEL ASE;
									%compute_ase_interval(data=p,
								                          output=ase&iter,
								                          y=&trait,
								                          predict=meanpred,
								                          mdlname=&mdlname,
								                          freq=&freq,
								                          aweight=&cv_alossweight
								                          );

									* APPEND ASE TO CANDIDATE_ASE DATA SET;
									data ase&iter;	
										set ase&iter;
										ITER = &iter;
										keep ITER ASE;
									run;

									proc append base=candidate_ase data=ase&iter;
									run;
									/*
									*COMPUTE MODEL HARREL_C;
									%compute_harrellc(data=p,
													  output=hac&iter,
													  TimeVar=&trait,
													  CenVar=,
													  Predict=meanpred,
													  Freq=&cv_alossweight);

									*TO DO APPEND HARREL_C TO CANDIDATE_HAC DATA SET;
									data hac&iter;
										set hac&iter;
										ITER = &iter;
										keep ITER HARRELL;
									run;

									proc append base=candidate_hac data=hac&iter;
									run;
									*/
								%end;
							%end;
						%end;
					%end;

					%if %sysfunc(exist(candidate_ase)) = 1 %then %do;
						%NObsVars(candidate_ase);
						%if ^&nObs or ^&nVars %then %do;
							%put MODEL &mdlname FAILED IN ALL ITERATIONS, IT SHOULD BE REMOVED.;
						%end;
						%else %do;
							* COMPUTE MEDIAN FIT STATISTICS AND APPEND TO MEDIAN_ASE;
							proc means data=candidate_ase noprint;
								var ASE;
								output out=median 
									   median(ase)=median_ase;
							run;
							data median;
								set median;
								length METHOD $64;
								keep method m median_ase;
								method = "&mdlname";
								m = &m;
								call symputx("median_ase",median_ase);
							run;
							%put Step &s of &maxsteps, Model &m of &n_models, median_ase=&median_ase;

							%NObsVars(median);
							%let _mObs_ = &nObs;
							%if &_mObs_ %then %do;
								%if %sysfunc(exist(median_ase)) = 1 %then %do;
									%NObsVars(median_ase);
									%let _aObs_ = &nObs; 
									data median_ase;
										set 
										%if &_aObs_ %then %do;
											median_ase	
										%end;
										median;
									run;
								%end;	
								%else %do;
									data median_ase;
										set median;
									run;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;

		* FIND THE BEST MODEL IN DATA SET MEDIAN_ASE;
		%if %sysfunc(exist(median_ase)) = 1 %then %do;
			%NObsVars(median_ase);
			%if &nObs and &nVars %then %do;
				proc sort data=median_ase;
					by descending median_ase;
				run; 
			                                                                                                                                        
			    data best;
					set median_ase;
					step = &s;
					if _n_ = 1 then do;
						call symput("best_model", trim(left(method)));
						call symput("bestm", trim(left(put(m, best12.))));
						output;
					end;
				run;
				%put bestm=&bestm best_model=&best_model;
			%end;
		%end;

		* FOR BEST MODEL, COMPUTE METRICS ON OTHER HALF OF HOLDOUT SETS;
		%let first = 1;
		%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end; 
			%if %sysfunc(exist(candidate&bestm)) = 1 %then %do;
				%NObsVars(candidate&bestm);
				%let _bestObs_ = &nObs;
				%if &_bestObs_ %then %do; 



					data p;
						set candidate&bestm;
						where ITER=&iter;
						%if &cv_alossprop ne 0.5 %then %do;
							_resid = &trait - meanpred;
							if missing(_resid) then &cv_alossweight = .;
							else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
							else &cv_alossweight = 2 * (1 - &cv_alossprop);
						%end;
						* USE OBSERVATIONS NOT USED ABOVE;
						if mod(_n_, 2) = 0 and meanpred ne . then output;
					run;

					%NObsVars(p);
					%let _pObs_ = &nObs;
					%if &_pObs_ %then %do;
						* COMPUTE MODEL ASE;
						%compute_ase_interval(data=p,
					                          output=ase&iter,
					                          y=&trait,
					                          predict=meanpred,
					                          mdlname=&mdlname,
					                          freq=&freq,
					                          aweight=&cv_alossweight
					                          );

						* APPEND ASE TO BEST_ASE DATA SET;
						data ase&iter;	
							set ase&iter;
							ITER = &iter;
							keep ITER ASE;
						run;

						%if &first = 1 %then %do;
							data best_ase;
								set ase&iter;
							run;
							%let first = 0;
						%end;
						%else %do;
							data best_ase;
								set best_ase ase&iter;
							run;
						%end;
						/*
						* COMPUTE MODEL HARREL_C;
						%compute_harrellc(data=p,
									      output=hac&iter,
										  TimeVar=&trait,
										  CenVar=,
										  Predict=meanpred,
										  Freq=&cv_alossweight);

						* APPEND HARRELC TO BEST_HAC DATA SET;
						data hac&iter;	
							set hac&iter;
							ITER = &iter;
							keep ITER HARRELL;
						run;

						%if &first = 1 %then %do;
							data best_hac;
								set hac&iter;
							run;
							%let first = 0;
						%end;
						%else %do;
							data best_hac;
								set best_hac hac&iter;
							run;
						%end;
						*/
					%end;
				%end;
			%end;
		%end;

		* COMPUTE MEDIAN FIT STATISTICS;
		%if %sysfunc(exist(best_ase)) = 1 %then %do;
			%NObsVars(best_ase);
			%if &nObs and &nVars %then %do;
				proc means data=best_ase noprint;
					var ASE;
					output out=median 
						   median(ase)=median_ase;
				run;
				data median;
					set median;
					length METHOD $64;
					keep method m step median_ase median_rmse;
					method = "&best_model";
					m = &bestm;
					step = &s;
					median_rmse = sqrt(median_ase);
					call symputx("best_median_ase",median_ase);
				run;
				%put hold_out_best_median_ase=&best_median_ase;

				* UPDATE ENSEMBLE_STEP BY APPENDING MODEL NAME AND MEDIAN_ASE;  
				%if %sysfunc(exist(&cvoutlib..ensemble_steps)) = 1 %then %do;
					%NObsVars(&cvoutlib..ensemble_steps);
					%let _estpObs_ = &nObs;
					data &cvoutlib..ensemble_steps;  
						set 
						%if &_estpObs_ %then %do;
							&cvoutlib..ensemble_steps
						%end; 
							median;
					run; 
				%end;
				%else %do;
					data &cvoutlib..ensemble_steps;  
						set median;
					run; 
				%end;
			  			 
			 	* UPDATE ENSEMBLE_ACCUM WITH MODEL BEST_M;  
  				%if %sysfunc(exist(&best_model)) = 1 %then %do;
					%NObsVars(&best_model);
					%if &nObs and &nVars %then %do;
					  	data &cvoutlib..ensemble_accum;                                                                                                           
							merge &cvoutlib..ensemble_accum &best_model;                                                                                        
							keep iter _obs sum:;                                                                                              
							sum_pred  = sum_pred + Predicted_Value;
						run;
		  			%end;
				%end;
			%end;
		%end;
   	%end;
%end;

%ensemble_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: ENSEMBLE_CV  
**  AUTHOR:  saspyt, Pei-Yi Tan and Russ Wolfinger
**  SUPPORT: saspyt, Pei-Yi Tan and Russ Wolfinger
**  DATE:    03/13/08
**  PURPOSE: Forward Stepwise with-replacement ensemble 
**  HISTORY: action                                        date     who
**           Created.                                      03/13/08 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:	Library path of output data sets 
**  TRAIT:      Trait variable name
**  TRAIT_TYPE:	Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  NCLASSES:	Number of qualitative trait variable category
**  FREQ: 		Frequency variable name (optional)
**  CVRALL_DSNAME: Models scored data set 
**  MODEL_NAMES:   List of model names
**                 (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  CV_HOLDOUT_ITERATION_START:	Random hold-out iteration starting index
**  CV_HOLDOUT_ITERATION_END: Random hold-out iteration ending index
**  STARTSTEP:  ensemble forward selection starting step
**  MAXSTEPS: 	Maximum number of ensemble forward selection steps 		
** 	PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCE = CUSTOM)
**
**  NOTES: Compare the ensemble models created from ensemble and 
**         ensemble_cv macros, the results indicate both macros 
**         are not selecting the correct best cvmc model at the 1st step 
**         due to data was not partitioned consistently between 
**         cvmc k-fold random partition and the ensemble macros. 
**         Need more work on this area. 
************************************************************************;
%macro ensemble_cv(outputpath=,
					trait=,
					trait_type=,
					nclasses=,
					freq=,
					cvrall_dsname=, 
					model_names=,
					cv_holdout_iteration_start=,
					cv_holdout_iteration_end=,
					startstep=,
					maxsteps=,
					prevalences=proportional,
					cv_custompriors=
					);

%put Starting macro ENSEMBLE_CV;
* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%put Determining number of observations;

%if %sysfunc(exist(&cvoutlib..&cvrall_dsname)) = 1 %then %do;
	%NObsVars(&cvoutlib..&cvrall_dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put &cvoutlib..&cvrall_dsname CONTAINS 0 OBSERVATIONS.;
		%goto ensemble_cv_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put &cvoutlib..&cvrall_dsname DOES NOT EXIST.;
	%goto ensemble_cv_exit;
%end;

* RETRIEVE NOBS IN THE ORIGINAL INPUT DATA;
proc freq data=&cvoutlib..&cvrall_dsname noprint;
	tables METHOD*ITER/out=d;
run;
data _null_;
	set d;
	if _n_ = 1 then 
		call symput("_NOBS_", put(COUNT, 12.0));
run;

* RETRIEVE MODEL NAMES;
%let m=1;
%do %while(%length(%scan(&model_names,&m," ")));
    %let m = %eval(&m+1);
%end;
%let n_models = %eval(&m-1);
%do m=1 %to &n_models;
    %global mdlname&m;
    %let mdlname&m = %scan(&model_names,&m," ");
%end;

* SUBSET INDIVIDUAL MODEL PREDICTIONS;
%do m=1 %to &n_models;
	%let mdlname = &&mdlname&m;
	%put Extracting data for model &mdlname;
	%if %upcase(&trait_type)=QUALITATIVE %then %do; 
		* RENAME ACTUALP AND P_ VARIABLE NAMES;  
		data &mdlname;
			set &cvoutlib..&cvrall_dsname;
			array rp{&nclasses} 
			%do c=1 %to &nclasses;
				rp&c._m&m
			%end;
			;
			array _p p_:;
			keep METHOD ITER _Obs actualp rp: &trait;
			ri = 0;
			if METHOD = "&mdlname" then do;
				do i=1 to dim(_p);
					if not missing(_p(i)) then do;
						ri = ri + 1;
						rp(ri) = _p(i);
					end;
				end;
				output;
			end;
		run;
	%end;
	%else %do;
		data &mdlname;
			set &cvoutlib..&cvrall_dsname;
			keep METHOD ITER _Obs Predicted_Value &trait;
			if METHOD = "&mdlname" then output;
		run;
	%end;

	%if %sysfunc(exist(&mdlname)) = 1 %then %do;
		%NObsVars(&mdlname);
		%if &nObs and &nVars %then %do;
			proc sort data=&mdlname;
				by ITER _Obs;
			run;
		%end;
	%end;
%end;

* INITIALIZE STEP HISTORY;                                                   
data  &cvoutlib..ensemble_steps;                                                                                                                   
	length iter 8 METHOD $64 step 8 ase 8 acc 8;                                                                                                           
	keep iter method step ase acc;                                                                                                        
run;     

%global bestm best_model;

* LOOP OVER PRECOMPUTED CV ITERATIONS;
%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end; 

	%put;
	%put Processing CV iteration &iter of &cv_holdout_iteration_end;

	%do s=&startstep %to &maxsteps;

		%put;
		%put Starting ensemble step &s of &maxsteps;

		%if %upcase(&trait_type)=QUALITATIVE %then %do; 
			%if &s = &startstep %then %do;
				* INITIALIZE ENSEMBLE DATA;
				data &cvoutlib..ensemble_accum;
					keep sum_actualp sum_rp: _Obs;
					retain sum_actualp 0;
					%do c=1 %to &nclasses;
						retain sum_rp&c 0;
					%end;
					do _Obs = 1 to &_NOBS_;
						output;
					end;
				run;
			%end;

			* CREATE CANDIDATE ENSEMBLES;
			%do m=1 %to &n_models;
				%let mdlname = &&mdlname&m;
				%if %sysfunc(exist(&mdlname)) = 1 %then %do;
					%NObsVars(&mdlname);
					%let _mdlObs_ = &nObs;
					%if &_mdlObs_ %then %do;
						data &mdlname.i;
							set &mdlname;
							where ITER = &iter;
						run;
						
						%if %sysfunc(exist(&mdlname.i)) = 1 %then %do;
							%NObsVars(&mdlname.i);
							%let _mdliObs_ = &nObs;
							%if &_mdliObs_ %then %do;
								data candidate&m;
									merge &cvoutlib..ensemble_accum &mdlname.i;
									if actualp=. or sum_actualp=. then do;
										meanp = .;
										correct = .;
										%do c=1 %to &nclasses;
											meanrp&c = .;
										%end;
									end;
									else do;
										meanp = (sum_actualp + actualp)/&s;
										if meanp > 0.5 then 
											correct = 1;
										else 
											correct = 0;
										%do c=1 %to &nclasses;	
											meanrp&c = (sum_rp&c + rp&c._m&m)/&s;
										%end;
									end;
								run;

								* FOR EACH MODEL, COMPUTE ASE and ACC AND CREATE CANDIDATE_ASE DATA SET;
								%if %sysfunc(exist(candidate&m)) = 1 %then %do;
									%NObsVars(candidate&m);
									%let _candmObs_ = &nObs;
									%if &_candmObs_ %then %do;
										data p;
											set candidate&m;
											%do c=1 %to &nclasses;	
												rename meanrp&c = p_&c;
											%end;
											if meanp ne . then output;
										run;

										%NObsVars(p);
										%let _pObs_ = &nObs;
										%if &_pObs_ %then %do;
											* USE ODD OBSERVATIONS;
											data p1;
												set p;
												if mod(_n_,2) = 1;
											run;

											%NObsVars(p1);
											%if &nObs and &nVars %then %do;
												* COMPUTE MODEL ASE;
												%compute_ase_nominal(data=p1,
																	 output=ase,
																	 actual=meanp,
																	 nclasses=&nclasses,
																	 mdlname=&mdlname,
																	 freq=&freq,
																	 prevalences=&prevalences,
																	 cv_custompriors=%nrbquote(&cv_custompriors),
										                     		 trait=&trait
																	 );

												* COMPUTE MODEL Accuracy;
												%compute_acc_nominal(data=p1,
																	 output=acc,
																	 trait=&trait,
																	 correct=correct,
																	 mdlname=&mdlname,
																	 freq=&freq,
																	 prevalences=&prevalences,
																	 cv_custompriors=%nrbquote(&cv_custompriors)
																	 );
												
												* COMPUTE MODEL AEC;
												%if %length(&cv_customcosts) %then %do;
													%compute_aec_nominal(
														 data=p1,
														 output=aec,
														 trait=&trait,
														 expcost=expected_cost,
														 mdlname=&mdlname,
														 freq=&freq
													);
													%let aec = aec;
												%end;
												%else %do;
													%let aec = ;
												%end;

												data candidate_ase;
													merge ase acc &aec;
												run;

												data candidate_ase;
													set candidate_ase;
													keep method m ase accuracy &aec;
													method = "&mdlname";
													m = &m;
													call symputx("ase",ase);
													call symputx("acc",accuracy);
													%if %length(&cv_customcosts) %then %do;
														call symputx("aec",aec);
													%end;
												run;
												%put Iter &iter of &cv_holdout_iteration_end, Step &s of &maxsteps; 
												%put Model &m of &n_models, ase=&ase accuracy=&acc;
												%if %length(&cv_customcosts) %then %do;
													%put aec=&aec;
												%end;
												
												%if %sysfunc(exist(model_ase)) = 1 %then %do;
													%NObsVars(model_ase);
													%let _aObs_ = &nObs; 
													data model_ase;
														set 
														%if &_aObs_ %then %do;
															model_ase
														%end;	
														candidate_ase;
													run;
												%end;
												%else %do;
													data model_ase;
														set candidate_ase;
													run;
												%end;
											%end;
										%end;
									%end;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;

			* FIND THE BEST MODEL IN DATA SET ASE;
			%if %sysfunc(exist(model_ase)) = 1 %then %do;
				%NObsVars(model_ase);
				%if &nObs and &nVars %then %do;
					proc sort data=model_ase;
						by descending accuracy ase;
					run; 

					data best;
						set model_ase;
						if _n_ = 1 then do;
							call symput("best_model", trim(left(method)));
							call symput("bestm", trim(left(put(m, best12.))));
							output;
						end;
					run;
					%put bestm=&bestm best_model=&best_model;
				%end;
			%end;

			* FOR BEST MODEL, COMPUTE METRICS ON OTHER HALF OF HOLDOUT SETS;
			%if %sysfunc(exist(candidate&bestm)) = 1 %then %do;
				%NObsVars(candidate&bestm);
				%let _bestObs_ = &nObs;
				%if &_bestObs_ %then %do; 
			
					data p;
						set candidate&bestm;
						%do c=1 %to &nclasses;	
							rename meanrp&c = p_&c;
						%end;
						if meanp ne . then output;
					run;

					%NObsVars(p)
					%let _pObs_ = &nObs;
					%if &_pObs_ %then %do;

						* USE OBSERVATIONS NOT USED ABOVE;
						data p1;
							set p;
							if mod(_n_,2) = 0;
						run;

						* COMPUTE MODEL ASE;
						%compute_ase_nominal(data=p1,
											 output=ase,
											 actual=meanp,
											 nclasses=&nclasses,
											 mdlname=&best_model,
											 freq=&freq,
											 prevalences=&prevalences,
											 cv_custompriors=%nrbquote(&cv_custompriors),
							             	 trait=&trait
											 );

						* COMPUTE MODEL Accuracy;
						%compute_acc_nominal(data=p1,
											 output=acc,
											 trait=&trait,
											 correct=correct,
											 mdlname=&best_model,
											 freq=&freq,
											 prevalences=&prevalences,
											 cv_custompriors=%nrbquote(&cv_custompriors),
											 );

						* COMPUTE MODEL AEC;
						%if %length(&cv_customcosts) %then %do;
							%compute_aec_nominal(
								 data=p1,
								 output=aec,
								 trait=&trait,
								 expcost=expected_cost,
								 mdlname=&mdlname,
								 freq=&freq
							);
							%let aec = aec;
						%end;
						%else %do;
							%let aec = ;
						%end;
						
						data best_ase;
							merge ase acc &aec;
						run;
						
						data best_ase;
							length iter 8;
							set best_ase;
							keep iter method m step ase accuracy &aec rmse one_minus_ase one_minus_rmse;
							iter = &iter;
							method = "&best_model";
							m = &bestm;
							step = &s;
							rmse = sqrt(ase);
							one_minus_ase = 1 - ase;
							one_minus_rmse = 1 - rmse;
							call symputx("best_ase",ase);
							call symputx("best_accuracy",accuracy);
						run;
						%put hold_out_best_ase=&best_ase hold_out_best_accuracy=&best_accuracy;

						* UPDATE ENSEMBLE_STEP BY APPENDING BEST_ASE;  
						%if %sysfunc(exist(&cvoutlib..ensemble_steps)) = 1 %then %do;
							%NObsVars(&cvoutlib..ensemble_steps);
							%let _estpObs_ = &nObs;
							data &cvoutlib..ensemble_steps;  
								set 
								%if &_estpObs_ %then %do;
									&cvoutlib..ensemble_steps
								%end; 
									best_ase;
							run; 
						%end;
						%else %do;
							data &cvoutlib..ensemble_steps;  
								set best_ase;
							run; 
						%end;
					%end;                                                                                                                         
				%end;
			%end;

			* UPDATE ENSEMBLE_ACCUM WITH BEST_MODEL;  
			%if %sysfunc(exist(&best_model.i)) = 1 %then %do;
				%NObsVars(&best_model.i);
				%if &nObs %then %do;
					data &cvoutlib..ensemble_accum;                                                                                                           
						merge &cvoutlib..ensemble_accum &best_model.i;                                                                                        
						keep iter _obs sum:;                                                                                              
						sum_actualp  = sum_actualp + actualp;
						%do c=1 %to &nclasses;
							sum_rp&c = sum_rp&c + rp&c._m&bestm; 
						%end; 
					run;      
				%end;
			%end;
		%end;
		%else %if %upcase(&trait_type)=QUANTITATIVE %then %do; 

			* QUANTITATIVE TRAITS;
			%if &s = &startstep %then %do;
				* INITIALIZE ENSEMBLE DATA;
				data &cvoutlib..ensemble_accum;
					keep sum_pred ITER _Obs;
					retain sum_pred 0;
					do iter=&cv_holdout_iteration_start to &cv_holdout_iteration_end;
						do _Obs = 1 to &_NOBS_;
							output;
						end;
					end;
				run;

			%end;

			* CREATE CANDIDATE ENSEMBLES FOR EACH ITERATION;
			%do m=1 %to &n_models;

				%let mdlname = &&mdlname&m;
				
				%if %sysfunc(exist(&mdlname)) = 1 %then %do;
					%NObsVars(&mdlname);
					%let _mdlObs_ = &nObs;
					%if &_mdlObs_ %then %do;
						data &mdlname.i;
							set &mdlname;
							where ITER = &iter;
						run;

						%if %sysfunc(exist(&mdlname.i)) = 1 %then %do;
							%NObsVars(&mdlname.i);
							%let _mdliObs_ = &nObs;
							%if &_mdliObs_ %then %do;
								data candidate&m;
									merge &cvoutlib..ensemble_accum &mdlname.i;
									if Predicted_Value = . or sum_pred = . then do;
										meanpred = .;
									end;
									else do;
										meanpred = (sum_pred + Predicted_Value)/&s;
									end;
								run;

								* FOR EACH MODEL, COMPUTE ASE FOR EACH ITERATION THEN TAKE MEDIAN; 
								%if %sysfunc(exist(candidate&m)) = 1 %then %do;
									%NObsVars(candidate&m);
									%let _candmObs_ = &nObs;
									%if &_candmObs_ %then %do;
										data p;
											set candidate&m;
											where ITER=&iter;
											%if &cv_alossprop ne 0.5 %then %do;
												_resid = &trait - meanpred;
												if missing(_resid) then &cv_alossweight = .;
												else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
												else &cv_alossweight = 2 * (1 - &cv_alossprop);
											%end;
											* ONLY USE EVERY OTHER OBSERVATION TO TRAIN THE ENSEMBLE;
											if mod(_n_, 2) = 1 and meanpred ne . then output;
										run;

										%if %sysfunc(exist(p)) = 1 %then %do;
											%nObsVars(p);
											%let _pObs_ = &nObs;
											%if &_pObs_ %then %do;
												* COMPUTE MODEL ASE;
												%compute_ase_interval(data=p,
																	  output=candidate_ase,
																	  y=&trait,
																	  predict=meanpred,
																	  mdlname=&mdlname,
																	  freq=&freq,
																	  aweight=&cv_alossweight
																	  );
												
												* APPEND ASE TO CANDIDATE_ASE DATA SET;
												data candidate_ase;	
													set candidate_ase;
													keep method m ase;
													method = "mdlname";
													m = &m;
													call symput("ase", ase);
												run;
												%put Iter &iter of &cv_holdout_iteration_end, Step &s of &maxsteps;
												%put Model &m of &n_models, ase=&ase accuracy=&acc;
											
												%if %sysfunc(exist(model_ase)) = 1 %then %do;
													%NObsVars(model_ase);
													%let _aObs_ = &nObs; 
													data model_ase;
														set 
														%if &_aObs_ %then %do;
															model_ase
														%end;	
														candidate_ase;
													run;
												%end;
												%else %do;
													data model_ase;
														set candidate_ase;
													run;
												%end;
												/*
												* COMPUTE MODEL HARREL_C;
												%compute_harrellc(data=p,
																  output=candidate_hac,
																  TimeVar=&trait,
																  CenVar=,
																  Predict=meanpred,
																  Freq=&cv_alossweight);

												* APPEND HARREL_C TO CANDIDATE_HAC DATA SET;
												data candidate_hac;	
													set candidate_hac;
													keep method m harrell;
													method = "mdlname";
													m = &m;
													call symput("harrell", harrell);
												run;

												%if %sysfunc(exist(model_hac)) = 1 %then %do;
													%NObsVars(model_hac);
													%let _aObs_ = &nObs; 
													data model_hac;
														set 
														%if &_aObs_ %then %do;
															model_hac
														%end;	
														candidate_hac;
													run;
												%end;
												%else %do;
													data model_hac;
														set candidate_hac;
													run;
												%end;
												*/
											%end;
										%end;
									%end;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;

			* FIND THE BEST MODEL IN DATA SET ASE;
			%if %sysfunc(exist(model_ase)) = 1 %then %do;
				%NObsVars(model_ase);
				%if &nObs and &nVars %then %do;
					proc sort data=model_ase;
						by descending ase;
					run; 

					data best;
						set model_ase;
						if _n_ = 1 then do;
							call symput("best_model", trim(left(method)));
							call symput("bestm", trim(left(put(m, best12.))));
							output;
						end;
					run;
					%put bestm=&bestm best_model=&best_model;
				%end;
			%end;

			* FOR BEST MODEL, COMPUTE METRICS ON OTHER HALF OF HOLDOUT SETS;
			%if %sysfunc(exist(candidate&bestm)) = 1 %then %do;
				%NObsVars(candidate&bestm);
				%let _bestObs_ = &nObs;
				%if &_bestObs_ %then %do; 

					data p;
						set candidate&bestm;
						%if &cv_alossprop ne 0.5 %then %do;
							_resid = &trait - meanpred;
							if missing(_resid) then &cv_alossweight = .;
							else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
							else &cv_alossweight = 2 * (1 - &cv_alossprop);
						%end;
						* USE OBSERVATIONS NOT USED ABOVE;
						if mod(_n_, 2) = 0 and meanpred ne . then output;
					run;

					%if %sysfunc(exist(p)) = 1 %then %do;
						%NObsVars(p);
						%let _pObs_ = &nObs;
						%if &_pObs_ %then %do;
							* COMPUTE MODEL ASE;
							%compute_ase_interval(data=p,
												  output=best_ase,
												  y=&trait,
												  predict=meanpred,
												  mdlname=&mdlname,
												  freq=&freq,
												  aweight=&cv_alossweight
												  );
					
							data best_ase;
								length iter 8;
								set best_ase;
								keep iter method m step ase rmse one_minus_ase one_minus_rmse;
								iter = &iter;
								method = "&best_model";
								m = &bestm;
								step = &s;
								rmse = sqrt(ase);
								one_minus_ase = 1 - ase;
								one_minus_rmse = 1 - rmse;
								call symputx("best_ase", ase);
							run;
							%put hold_out_best_ase=&best_ase;

							* UPDATE ENSEMBLE_STEP BY APPENDING MODEL NAME AND MEDIAN_ASE; 
			  				%if %sysfunc(exist(&cvoutlib..ensemble_steps)) = 1 %then %do;
								%NObsVars(&cvoutlib..ensemble_steps);
								%let _estpObs_ = &nObs;
								data &cvoutlib..ensemble_steps;  
									set 
									%if &_estpObs_ %then %do;
										&cvoutlib..ensemble_steps
									%end; 
										best_ase;
								run; 
							%end;
							%else %do;
								data &cvoutlib..ensemble_steps;  
									set best_ase;
								run; 
							%end;
							/*
							* COMPUTE MODEL HARREL_C;
							%compute_harrellc(data=p,
											  output=best_hac,
											  TimeVar=&trait,
											  CenVar=,
											  Predict=meanpred,
											  Freq=&cv_alossweight);
							data best_hac;
								length iter 8;
								set best_hac;
								keep iter method m step harrell;
								iter = &iter;
								method = "&best_model";
								m = &bestm;
								step = &s;
								call symputx("best_harrell", harrell);
							run;

							* UPDATE ENSEMBLE_STEP BY APPENDING MODEL NAME AND MEDIAN_HARRELC; 
			  				%if %sysfunc(exist(&cvoutlib..ensemble_steps)) = 1 %then %do;
								%NObsVars(&cvoutlib..ensemble_steps);
								%let _estpObs_ = &nObs;
								data &cvoutlib..ensemble_steps;  
									set 
									%if &_estpObs_ %then %do;
										&cvoutlib..ensemble_steps
									%end; 
										best_hac;
								run; 
							%end;
							%else %do;
								data &cvoutlib..ensemble_steps;  
									set best_hac;
								run; 
							%end;
							*/
							* UPDATE ENSEMBLE_ACCUM WITH MODEL BEST_M;  
						  	%if %sysfunc(exist(&best_model.i)) = 1 %then %do;
								%NObsVars(&best_model.i);
								%if &nObs and &nVars %then %do;
									data &cvoutlib..ensemble_accum;                                                                                                           
										merge &cvoutlib..ensemble_accum &best_model.i;                                                                                        
										keep iter _obs sum:;                                                                                              
										sum_pred  = sum_pred + Predicted_Value;
									run;     
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
%end;

data &cvoutlib..ensemble_steps;  
	set &cvoutlib..ensemble_steps;
	if iter = . then delete;
run; 

%ensemble_cv_exit:;
%mend;

************************************************************************
**  Copyright (c) 2006 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:     GENERATE_CV_VARS
**  SUPPORT:  saspyt, Pei-Yi Tan
**  DATE:     05/15/06
**  PURPOSE:  Generate cross validation variables
**  HISTORY:  action                                            date who
**            Created.	                                    05/15/06 pyt 
** 
**  MACRO PARAMETERS:
**  DATAPATH:      Library path of training data set 
**  OUTPUTPATH:    Library path of output data sets 
**  DSNAME:        Training data set name
**  TRAIT:         Trait variable name
**  TRAIT_TYPE:    Trait variable type: 
**                 qualitative(categorical), 
**                 quantitative(interval)
**  KFOLD:         K-fold cross validation 
**  GROUPNAME:     Grouping variable
**  CVNAME:        CV variable name
**  CV_SEED:       Cross validation initial random number seed 
**  CV_VARS_OUTDS: Output data set name contains cv variables
**  ITERATION:	   Iteration index
**
**  NOTES:
**  pseudo macro creates pseudo variable with values 1, 2, ..., kfold.
************************************************************************;
%macro generate_cv_vars(datapath=,
				    	outputpath=,
					    dsname=,
						trait=,
						trait_type=,
						kfold=,
						groupname=,
						cvname=,
						cv_seed=,
						cv_vars_outds=,
						iteration=,
						nbin=20
);

%put Entering generate_cv_vars;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname datalib "&datapath";
	%let datalib = datalib;
%end;
%else %do; 
	%let datalib = work;
%end;

%if %length(&dsname) %then %do;
    %let __cvvarsds = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__cvvarsds)) = 1 %then %do;
	%NObsVars(&__cvvarsds);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__cvvarsds CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__cvvarsds DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let out = output;
%end;
%else %do;
    %let out = work;
%end;

* KEEP TRAIT WITH NON-MISSING VALUES;  
%let _tid_ = %sysfunc(open(&__cvvarsds));
%let _tObs_ = %sysfunc(attrn(&_tid_, NOBS));
%if &_tObs_ %then %do;
	%let traitnum=%sysfunc(varnum(&_tid_, &trait));
	%let traitype=%sysfunc(vartype(&_tid_, &traitnum));
%end;
%let _tid_ = %sysfunc(close(&_tid_));

* KEEP TRAIT WITH NON-MISSING VALUES AND APPLY SPECIFIED WHERE_CLAUSE;  
data temp(keep=&trait __id) ds;
	set &__cvvarsds;
	%if &traitype = C %then %do; 
		if &trait ne " " then output;
	%end;
	%else %do;
		if &trait ne . then output;
	%end;
run;
%NObsVars(work.temp);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
	%if %length(&wc) > 0 %then %do;
		%put ERROR: &__cvvarsds WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do;
		%put ERROR: &__cvvarsds DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
	%end;
	%goto exit;
%end;

* for grouping, summarize the data to the group level;
* assign splits at this level then merge back;
%if %length(&groupname) %then %do;
	proc sort data=ds out=ds0;
		by &groupname;
	run;
	/*
	data outlib.ds0;
		set ds0;
	run;
	*/
	proc means data=ds0 noprint;
		by &groupname;
		var &trait;
		output out=ds mean=&trait;
	run;
	%let trait_type = quantitative;
%end;

%if %upcase(&trait_type) = QUALITATIVE %then %do; 

	* RETRIEVE THE NUMBER OF CLASS OF TRAIT VARIABLE;
	proc sort data=temp nodup;
	    by __id;
	run;
	proc freq data=temp noprint;
	    table &trait/out=trgtfreq;    
	run;
	proc sort data=trgtfreq;
		by PERCENT;
	run;
	%let _fid_ = %sysfunc(open(trgtfreq));
	%let trait_level = %sysfunc(attrn(&_fid_, NOBS));
	%if &trait_level > 0 %then %do;
		%let traitnum=%sysfunc(varnum(&_fid_, &trait));
		%let traitype=%sysfunc(vartype(&_fid_, &traitnum));
		%do i=1 %to &trait_level;
			%let rc=%sysfunc(fetchobs(&_fid_,&i));
			%if &traitype = C %then %do;
				%let trait_value&i=%sysfunc(getvarc(&_fid_,&traitnum));
			%end;
			%else %do;
				%let trait_value&i=%sysfunc(getvarn(&_fid_,&traitnum));
			%end;
			%if &i=1 %then %do;
				%let event=&&trait_value&i;
			%end;
		%end;
	%end;
	%let _fid_ = %sysfunc(close(&_fid_));
	
    %do i=1 %to &trait_level;
        data trait&i;
            set temp;
            if &trait = "&&trait_value&i" then output;
        run;
		data _null_;
	        aa = &cv_seed + 607081;
	        seed = mod(aa, 2090069);
	        call symput("cv_seed", trim(left(put(seed,12.0))));
	    run;

		%pseudouniform(dsname=trait&i, 
                       output=pseudo&i,
                       seed=&cv_seed);
        data trait&i;
            merge trait&i pseudo&i;
            keep __id pseudo;
        run;
        proc sort data=trait&i; by pseudo; run;

    %end;

    data all;
        set 
        %do i=1 %to &trait_level;
            trait&i
        %end;
        ;
    run;

    data all; 
         set all;
         drop pseudo;
         &cvname = mod(_n_, &kfold) + 1;
    run;

    proc sort data=all out=&out..&cv_vars_outds.&iteration;
        by __id;
    run;

%end;
%else %do;
	
	* DIVIDE TRAIT INTO EQUALLY SPACED BINS;
	proc means data=ds noprint;
		var &trait;
		output out=mean min=minvar max=maxvar;
	run;
	data _null_;
		set mean;
		call symput("minvar", minvar);
		call symput("maxvar", maxvar);
		call symput("sum", _FREQ_);
	run;
	data bin;
		set ds;
		bin = floor(((&trait - &minvar)/(&maxvar - &minvar))*&nbin) + 1;
		if bin < 1 then bin = 1;
		else if bin >= &nbin then bin = &nbin;
	run;
	
	* CREATE PSEUDO VARIABLE WITH VALUES 1, 2, ...,KFOLD WITHIN EACH BIN;
	%do b=1 %to &nbin;
		data bin&b;
			set bin;
			if bin=&b then output;
		run;
		%NObsVars(bin&b);
		%if &nObs and &nVars %then %do;
			data _null_;
		        aa = &cv_seed + 607081;
		        seed = mod(aa, 2090069);
		        call symput("cv_seed", trim(left(put(seed,12.0))));
		    run;
		    %pseudo(dsname=bin&b, 
		            kfold=&kfold, 
		            output=pseudo&b,
					seed=&cv_seed);
					
			data bin&b;
				merge bin&b pseudo&b;
				%if %length(&groupname) %then %do;
					keep &groupname pseudo;
				%end;
				%else %do;
					keep __id pseudo;
				%end;
				rename pseudo=&cvname;
			run;
		%end;
	%end;
	 data all;
        set 
        %do b=1 %to &nbin;
			%if %sysfunc(exist(bin&b)) = 1 %then %do;
				%NObsVars(bin&b)
				%if &nObs and &nVars %then %do;
		            bin&b
				%end;
			%end;
        %end;
        ;
    run;
    %if %length(&groupname) %then %do;
    	proc sort data=all;
    		by &groupname;
    	run;
    	data all;
    		merge ds0 all;
    		by &groupname;
    		keep __id &cvname;
    	run;
    %end;
    proc sort data=all out=&out..&cv_vars_outds.&iteration;
        by __id;
    run;
    /*
    data outlib.all;
    	set all;
    run;
    */

%end;

%exit:;
%mend;

%macro innerstart;
       inner_start = ninner - inner + 1;
       inner_end = inner_start;
%mend  innerstart;

%macro ndperin;
        nodes_per_inner = floor(nodes_per_outer / ninner);
        remain_inner = nodes_per_outer - ninner * nodes_per_inner;
%mend ndperin;

%macro itpernd;
        iters_per_node = floor(niters / nodes_per_inner );
        remain_iter = niters - nodes_per_inner * iters_per_node;
%mend itpernd;

* CHECK FOR GRID TASK COMPLETION;
%macro get_results_when_complete(nnodes=);
   %do i=1 %to &nnodes;    
   	   %let gndone=&&gndone&i;
	   %put node=&i status=&gndone; 
       %if &gndone =0 %then %do;
          %put Remote submit complete;
	   %end;
	   %else %if &gndone = 1 %then %do;
	        %put Remote submit failed to execute;
	   %end;
	   %else %do;
	         %put Remote submit is still in progress;
	   %end;
	%end;
%mend;

%macro get_priors(priors_string=, traittype=);

%put priors_string = &priors_string;
* RETRIEVE PRIORS;
data pr;
   length word $32767;
   drop string count;
   string = "&priors_string";
   do until(word=' ');
      count+1;
      word = upcase(scan(string, count, "=,"));
	  if word ne '' then output;
   end;
run;

%NObsVars(pr);
%let nlevels=%sysevalf(&nObs/2, int);
%if &nlevels ne &nclasses %then %do;
	%let exiterror = 1;
	%put ERROR: Number of prevalences do not match the level of trait;
	%goto exit;
%end;

data _null_;
	set pr end=end;
	retain total 0;
	if mod(_n_, 2)=0 then do;
	   prior=input(word, best12.);
	   total = total + prior;
 	end;
	if end ne 0 then do;
	   call symput("total", trim(left(total)));
	end; 
run;

data adjust_pr;
	set pr;
	drop word trait;
	length &trait $512 priors 8;
	retain trait;
	if mod(_n_, 2) = 1 then do;
		trait = trim(left(word));
		put trait=;
	end;
	if mod(_n_, 2) = 0 then do;
	    &trait = trait;
		priors = input(word, best12.)/&total;
		output;
	end;
run;

%if &traittype = N %then %do;
	data adjust_pr;
		set adjust_pr;
		length trait_n 8;
		drop &trait;
		trait_n = 0 + &trait;
		rename trait_n = &trait;
	run;
%end;

%exit:;
%mend;
/* %get_priors(priors_string='1x 2 =0.2, 0 y 3 = 0.7, 1x 3=0.6'); */

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    GRIDSEQUENCE
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    08/11/08
**  PURPOSE: Generate grid computing distribution
**  HISTORY: action                                        date     who
**           Created.                                      08/11/08 pyt 
** 
**  MACRO PARAMETERS:
**  PROCESS: Type of predictive modeling process
**			 CVMC - Cross validation model comparison
**			 LCMC - Learning curve model comparison
**			 LC_STANDARD - Standard learning Curve 
**			 LEAVE_L_OUT - CVMC with leave 1, 2, or 3 out cv method
**			 LC_LEAVE_L_OUT - LCMC with leave_l_out cv method 
**  CV_HOLDOUT_ITERATION_START: Hold-out interation starting index
**  CV_HOLDOUT_ITERATION_END: Hold-out interation ending index
**  INNERLOOP_START: LC inner loop iteration starting index
**  INNERLOOP_END: LC inner loop iteration ending index
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  NNODES:	Number of grid nodes
**  GRIDDS:	Grid distribution assignment data set name
**
**  NOTES:
**
***********************************************************************;
%macro gridsequence(process=,
					cv_holdout_iteration_start=,
			 	    cv_holdout_iteration_end=,
					innerloop_start=,
					innerloop_end=,
			        outerloop_start=,
					outerloop_end=,
					leave_l_out=,
					nnodes=,
		            gridds=);

%if &process = CVMC %then %do;
	* REDUCE NUMBER OF NODES IF NECESSARY;
	%let niters = %eval(&cv_holdout_iteration_end - &cv_holdout_iteration_start + 1);
	%if &nnodes > &niters %then %do;
		%let nnodes = &niters;
	%end;

	data &gridds;
		retain nnodes niters 0;
		length node_name $32;
		keep grid_node 
			 iter_start iter_end 
			 run_status node_name;

		* COERCE ORDER OF OUTPUT VARIABLES;        
		grid_node = 0;        
		iter_start = 0;        
		iter_end = 0;
		run_status = .;
		node_name = "";

		nnodes = &nnodes;
		niters = &niters;
		
		if nnodes < niters then do;
			nper = floor(niters/nnodes);
			nremain = niters - nper * nnodes;
		end;
		else do;
			nper = 1;
			nremain = 0;
		end;

		iter_next = &cv_holdout_iteration_start;

		%do node=1 %to &nnodes;
			grid_node=&node;
			iter_start = iter_next;
			if grid_node <= nremain then
				iter_end = iter_start + nper;
			else 
				iter_end = iter_start + nper - 1;
			iter_next = iter_end + 1;
			output;
		%end;
	run;
%end;
%else %if &process = LC_STANDARD %then %do;

	* REDUCE NUMBER OF NODES IF NECESSARY;
	%let n_inner = %eval(&innerloop_end - &innerloop_start + 1);
	%let n_outer = %eval(&outerloop_end - &outerloop_start + 1);
	%let n_outer_inner = %eval(&n_outer * &n_inner);
	%if &nnodes > &n_outer_inner %then %do;
		%let nnodes = &n_outer_inner;
	%end;

	data &gridds;
		retain nnodes nouter ninner n_outer_inner 0;
		length node_name $32;
		keep grid_node 
			 outerloop_start outerloop_end 
			 inner_start inner_end
			 run_status node_name; 

		* COERCE ORDER OF OUTPUT VARIABLES;        
		grid_node = 0;        
		outer_start = 0; 
		outerloop_start = 0; 
		outer_end = 0;   
		outerloop_end = 0; 
		inner_start = 0;        
		inner_end = 0; 
		run_status = .;
	    node_name = ""; 
 
		nnodes = &nnodes;
		nouter = &n_outer;
		ninner = &n_inner;
		n_outer_inner = &n_outer_inner;

		if nnodes < nouter then do;
			nper = floor(nouter/nnodes);
			remainder = nouter - nper * nnodes;
			outer_next = 1;	 
		end;
		else if nouter <= nnodes < n_outer_inner then do;
			nodes_per_outer = floor(nnodes/nouter); * nper_outer < n_innerloop;
			remain_outer = nnodes - nouter * nodes_per_outer; 
			if remain_outer > 0 then
				nodes_per_outer = nodes_per_outer + 1; * nodes_per_outer might now = n_inner; 
			node_within_outer = 0;
			outer_next = 1;	 
			nodes_per_inner = floor(ninner/nodes_per_outer);
			remain_inner = ninner - nodes_per_outer * nodes_per_inner;
			inner_next = ninner;	
			put nodes_per_outer= remain_outer= node_within_outer=
				nodes_per_inner= remain_inner=;
		end;
		else if n_outer_inner <= nnodes	then do;
			nodes_per_outer = floor(nnodes/nouter);	         
			nodes_per_inner = floor(nodes_per_outer/ninner); 
			outer = 1;
			inner = ninner;									 
			node_within_outer = 0;
			node_within_inner = 0; 
			put nodes_per_outer= nodes_per_inner= node_within_outer= node_within_inner=;
		end;

		%do node=1 %to &nnodes;
			grid_node=&node;
			if nnodes < nouter then do;
				outer_start = outer_next;
				outer_end = outer_start + nper;
				node = &node;
				if node > remainder then 
					outer_end = outer_end - 1;
				outer_next = outer_end + 1;
				inner_start = ninner;
				inner_end = 1;
			end;
			else if nouter <= nnodes < n_outer_inner then do;
				node_within_outer = node_within_outer + 1;
				if node_within_outer > nodes_per_outer then do;
	               if outer_next = remain_outer then do;  
					   nodes_per_outer = nodes_per_outer - 1;
	                   nodes_per_inner = floor(ninner/nodes_per_outer);
	                   remain_inner = ninner - nodes_per_outer * nodes_per_inner;
				   end;
				   outer_next = outer_next + 1;
	               node_within_outer = 1;
	               inner_next = ninner;	
				end;
				outer_start = outer_next;
				outer_end = outer_next;
				if nodes_per_outer = ninner then do;
					inner_start = node_within_outer;
					inner_end = node_within_outer;
				end;
				else do;
					inner_start = inner_next;
					inner_end = inner_start - nodes_per_inner;
					if node_within_outer > remain_inner then 
						inner_end = inner_end + 1;
					inner_next = inner_end - 1;
				end;
			end;
			else if n_outer_inner <= nnodes	then do;
			   node_within_outer = node_within_outer + 1;
			   put grid_node= node_within_outer=;
			   if node_within_outer > nodes_per_outer then do;
			   		outer = outer + 1;
					node_within_outer = 0;
			   end;
			   outer_start = outer;
			   outer_end = outer;
				
			   node_within_inner = node_within_inner + 1;
			   put node_within_inner=;
			   if node_within_inner > nodes_per_inner then do;
			   	    node_within_inner = 1;
					inner = inner - 1;
					if inner <= 0 then do;
					  	inner = ninner;
					end;
			   end;
			   inner_start = inner;
	  		   inner_end = inner;

			end;

			outerloop_start = outer_start + &outerloop_start - 1;
			outerloop_end = outer_end + &outerloop_start - 1;
			output;
		%end;
		drop outer_start outer_end;
		rename outerloop_start = outer_start;
		rename outerloop_end = outer_end;
	run;
%end;
%else %if &process = LCMC %then %do;

	* REDUCE NUMBER OF NODES IF NECESSARY;
	%let niters = %eval(&cv_holdout_iteration_end - &cv_holdout_iteration_start + 1);
	%let n_inner = %eval(&innerloop_end - &innerloop_start + 1);
	%let n_outer = %eval(&outerloop_end - &outerloop_start + 1);
	%let n_outer_inner = %eval(&n_outer * &n_inner);
	%let n_outer_inner_iter = %eval(&n_outer_inner * &niters);
	%if &nnodes > &n_outer_inner_iter %then %do;
		%let nnodes = &n_outer_inner_iter;
	%end;
	
	data &gridds;
		retain nnodes nouter ninner niters n_outer_inner n_outer_inner_iter 0;
		length node_name $32;
	    keep grid_node 
			 outerloop_start outerloop_end 
	         inner_start inner_end 
	         iter_start iter_end
			 node_name run_status;

		* COERCE ORDER OF OUTPUT VARIABLES;        
		grid_node = 0;        
		outer_start = 0; 
		outerloop_start = 0;
		outer_end = 0;
		outerloop_end = 0;
		inner_start = 0;        
		inner_end = 0;        
		iter_start = 0;        
		iter_end = 0;
		run_status = .;
		node_name = "";

		nnodes = &nnodes;
	    nouter = &n_outer;
	    ninner = &n_inner;
	    niters = &niters;
		n_outer_inner = &n_outer_inner;
		n_outer_inner_iter = &n_outer_inner_iter;
		put nnodes= nouter= ninner= niters= n_outer_inner= n_outer_inner_iter=;
		
		if nnodes < nouter then do; 
			nper = floor(nouter/nnodes);
			remainder = nouter - nper * nnodes;
			outer_next = 1;	
		end;
		else if nouter <= nnodes < n_outer_inner then do;
			nodes_per_outer = floor(nnodes/nouter); * nper_outer < n_innerloop;
			remain_outer = nnodes - nouter * nodes_per_outer; 
			if remain_outer > 0 then
				nodes_per_outer = nodes_per_outer + 1; * nodes_per_outer might now = n_inner; 
			node_within_outer = 0;
			outer_next = 1;	
			nodes_per_inner = floor(ninner/nodes_per_outer);
			remain_inner = ninner - nodes_per_outer * nodes_per_inner;
			inner_next = ninner;
		end;
		else if n_outer_inner <= nnodes < n_outer_inner_iter then do;
			nodes_per_outer   = floor(nnodes/nouter);
	        remain_outer      = nnodes - nouter * nodes_per_outer;
	        if remain_outer > 0 then nodes_per_outer = nodes_per_outer + 1;
	        put nodes_per_outer = ;

	        %ndperin;
	        if remain_inner > 0 then nodes_per_inner = nodes_per_inner + 1;
	        put remain_inner = ;
	        put nodes_per_inner = ;

	        %itpernd;
	        if remain_iter > 0 then iters_per_node = iters_per_node + 1;
	        put iters_per_node  = ;

	        outer_start       = 1;
	        inner             = 1;
	        %innerstart;
	        iter_next         = 1;
	        iter_group        = 1;
	        node_within_outer = 0;
	        node_within_inner = nodes_per_inner + 1;
	        outer_end = outer_start;
		end;
		else if n_outer_inner_iter <= nnodes then do;
			nnodes = n_outer_inner_iter;	
			nodes_per_outer = floor(nnodes/nouter);
			nodes_per_inner = floor(nodes_per_outer/ninner);
			outer = 1;
			inner = ninner;
			iter = 0;
			iter_next = 1;
			node_within_outer = 0;
			node_within_inner = 0;
		end;

		%do node=1 %to &nnodes;
			grid_node=&node;
			if nnodes < nouter then do; 
				outer_start = outer_next;
				outer_end = outer_start + nper;
				node = &node;
				if node > remainder then 
					outer_end = outer_end - 1;
				outer_next = outer_end + 1;
				inner_start = ninner;
				inner_end = 1;
				iter_start = &cv_holdout_iteration_start;
				iter_end = &cv_holdout_iteration_end;
			end;
			else if nouter <= nnodes < n_outer_inner then do;
				node_within_outer = node_within_outer + 1;
				if node_within_outer > nodes_per_outer then do;
	               if outer_next = remain_outer then do;  
					   nodes_per_outer = nodes_per_outer - 1;
	                   nodes_per_inner = floor(ninner/nodes_per_outer);
	                   remain_inner = ninner - nodes_per_outer * nodes_per_inner;
				   end;
				   outer_next = outer_next + 1;
	               node_within_outer = 1;
	               inner_next = ninner;	
				end;
				outer_start = outer_next;
				outer_end = outer_next;
				if nodes_per_outer = ninner then do;
					inner_start = node_within_outer;
					inner_end = node_within_outer;
				end;
				else do;
					inner_start = inner_next;
					inner_end = inner_start - nodes_per_inner;
					if node_within_outer > remain_inner then 
						inner_end = inner_end + 1;
					inner_next = inner_end - 1;
				end;
				iter_start = &cv_holdout_iteration_start;
				iter_end = &cv_holdout_iteration_end;
			end;
			else if n_outer_inner <= nnodes < n_outer_inner_iter then do;
	            node_within_outer = node_within_outer + 1;
	            node_within_inner = node_within_inner - 1;

	            if node_within_outer > nodes_per_outer then do;
	               if outer_start = remain_outer then do;
	                  nodes_per_outer = nodes_per_outer - 1;
	                  %ndperin;
	                  put nodes_per_outer = remain_inner= nodes_per_inner= ;
	               end;
	               outer_start = outer_start + 1;
	               outer_end = outer_start;
			
				   node_within_outer = 1;

	               inner             = 1;
	               %innerstart;
	               if remain_inner > 0 then 
					  nodes_per_inner = nodes_per_inner + 1;
	               node_within_inner = nodes_per_inner;
	               put "new outer" nodes_per_inner=  remain_inner=;
	        
	               iter_next         = 1;
	               iter_group        = 1;
	               %itpernd;
	               if remain_iter > 0 then iters_per_node = iters_per_node + 1;
	               put remain_iter= iters_per_node=;
	            end;
	            else if node_within_inner = 0 then do;
	               if inner = remain_inner then do;
	                  nodes_per_inner = nodes_per_inner - 1;
	               end;
	               inner = inner + 1;
	               %innerstart; 
	               node_within_inner = nodes_per_inner;
	               iter_next = 1;
	               iter_group = 1;
	               %itpernd;
	               if remain_iter > 0 then 
					  iters_per_node = iters_per_node + 1;
	               put nodes_per_inner = remain_iter= iters_per_node=;
	            end;

	            iter_start = iter_next;
				iter_start = &cv_holdout_iteration_start + iter_start - 1;
				iter_end = iter_start + iters_per_node - 1; 

				iter_next = iter_next + iters_per_node;
	            if iter_group = remain_iter then 
				   iters_per_node = iters_per_node - 1;
	            iter_group = iter_group + 1;

			end;
			else if n_outer_inner_iter <= nnodes then do;
			   node_within_outer = node_within_outer + 1;
			   if node_within_outer > nodes_per_outer then do;
			   		outer = outer + 1;
					node_within_outer = 1;
			   end;
			   outer_start = outer;
			   outer_end = outer;
		
			   node_within_inner = node_within_inner + 1;
			   if node_within_inner > nodes_per_inner then do;
			   	    node_within_inner = 1;
					inner = inner - 1;
					if inner <= 0 then do;
					  	inner = ninner;
					end;
			   end;
			   inner_start = inner;
	  		   inner_end = inner;
		 	   
			   if iter_next > niters then do;
			   	  iter_next = 1;
				  iter = 0;
			   end;	
			   iter_next = iter_next + 1;
			   iter = iter + 1;
			   put iter;
			   iter_start = &cv_holdout_iteration_start + iter - 1;
			   iter_end = iter_start; 
			end;

			outerloop_start = outer_start + &outerloop_start - 1;
			outerloop_end = outer_end + &outerloop_start - 1;
			output;
		%end;
		drop outer_start outer_end;
		rename outerloop_start = outer_start;
		rename outerloop_end = outer_end;
	run;
%end;
%else %if &process = LEAVE_L_OUT %then %do;
	data &gridds;
		length node_name $32;
		keep grid_node 
			 i_start i_end 
 	   	     j_start j_end 
        	 k_start k_end
			 run_status node_name;

		* COERCE ORDER OF OUTPUT VARIABLES;        
	    grid_node=0;
	    i_start=0;
	    i_end=0;
	    j_start=0;
	    j_end=0;
	    k_start=0;
	    k_end=0;
		run_status=.;
		node_name="";

        %if &leave_l_out = 1 %then %do;
		    j_start = j_end = .;
		    k_start = k_end = .;
		    if &nnodes >= &nobs then do;
				do grid_node = 1 to &nobs;
		        	i_start = grid_node;
		           	i_end   = grid_node;
		           	output;
		       	end;
		    end;
		    else do;
		    	ipernode = floor(&nobs / &nnodes); 
		        remain = &nobs - &nnodes * ipernode;
				put ipernode= remain=;
		        if remain > 0 then ipernode = ipernode + 1;
		        i_next   = 1;
		        do grid_node = 1 to &nnodes;
		        	i_start = i_next;
		           	i_end   = i_start + ipernode - 1;
		           	i_next  = i_end + 1;
		           	if grid_node = remain then 
						ipernode = ipernode - 1;
		           	output;
		        end;
		   end;
	    %end;
	    %if &leave_l_out = 2 %then %do;

		   k_start = k_end = .;

		   n_to_fit = &nobs * (&nobs - 1 ) / 2 ;
		   if &nnodes >= n_to_fit then do;
              	i_start = 1;
                j_start = 2;
                do  grid_node = 1 to n_to_fit;
                    i_end = i_start;
                    j_end = j_start;
                    output;
                    if j_start = &nobs then do;
                       i_start = i_start + 1;
                       j_start = i_start + 1;
                    end;
                    else do;
                       j_start = j_start + 1;
                    end;
                end;
		    end;
		    else do;
		        pernode  = floor(n_to_fit / &nnodes); 
		        remain   = n_to_fit - &nnodes * pernode;
		        if remain > 0 then pernode = pernode + 1;
		        i_start = 1;
		        j_start = 2;

		        do grid_node = 1 to &nnodes;
		           m_to_go = pernode;
		           i_end   = i_start;
		           j_next  = j_start;
		           nj_this_i = &nobs - j_start + 1;

		           put grid_node= m_to_go= i_start= i_end= nj_this_i=;

		           do while(m_to_go > nj_this_i);
		               m_to_go   = m_to_go - nj_this_i;
		               i_end     = i_end + 1;
		               j_next    = i_end + 1;
		               nj_this_i = &nobs - j_next + 1;
		          	   put "while loop" m_to_go= j_next= i_end= nj_this_i = ;
		           end;
		           j_end = j_next + m_to_go - 1;
		           output;
		           put "did output" i_start= i_end= j_start= j_end=;

		           if grid_node = remain then pernode = pernode - 1;
		           i_start = i_end;
		           j_start = j_end + 1;
		           if j_start > &nobs then do;
		              i_start = i_start + 1;
		              j_start = i_start + 1;
		           end;
		        end;
		     end;
		%end;
		%else %if &leave_l_out = 3 %then %do;

		    n_to_fit = &nobs * (&nobs - 1 ) * (&nobs - 2) / 6 ;

		    if &nnodes >= n_to_fit then do;
	            i_start = 1;
	            j_start = 2;
	            k_start = 3;
	            do  grid_node = 1 to n_to_fit;
	                i_end = i_start;
	                j_end = j_start;
	                k_end = k_start;
	                output;
	                if k_start = &nobs then do;
	                   if j_start = &nobs - 1 then do;
	                      i_start = i_start + 1;
	                      j_start = i_start + 1;
	                   end;
	                   else do;
	                      j_start = j_start + 1;
	                   end;
	                   k_start = j_start + 1;
	                end;
	                else do;
	                   k_start = k_start + 1;
	                end;
	            end;
		    end;
		    else do;
		        pernode  = floor(n_to_fit / &nnodes);
		        remain   = n_to_fit - &nnodes * pernode;
		        if remain > 0 then pernode = pernode + 1;
		        i_start = 1;
		        j_start = 2;
		        k_start = 3;

		        put n_to_fit= pernode= remain= "nobs = &nobs nnodes= &nnodes";

		        do grid_node = 1 to &nnodes;
		           m_to_go = pernode;
		           i_end = i_start;
		           j_end = j_start;
		           j_next = j_start;
		           k_next = k_start;
		           nk_this_ij = &nobs - k_next + 1;
		           njk_this_i = nk_this_ij + ((&nobs - j_next - 1)*(&nobs-j_next))/2;
		           put grid_node= i_start= j_start= k_start= nk_this_ij= njk_this_i=;
		           if m_to_go <= nk_this_ij then do;
		               j_end = j_start;
		           end;
		           else if m_to_go <= njk_this_i then do;
		               put m_to_go= "<=njk";
		               do while(m_to_go > nk_this_ij);
		                   m_to_go    = m_to_go - nk_this_ij;
		                   j_end      = j_end + 1;
		                   k_next     = j_end + 1;
		                   nk_this_ij = &nobs - k_next  + 1;
		               end;
		           end;
		           else do;
		               put m_to_go= ">njk";
		               do while(m_to_go > njk_this_i);
		                  m_to_go = m_to_go - njk_this_i;
		                  i_end   = i_end + 1;
		                  j_end   = i_end + 1;
		                  k_next  = j_end + 1;
		                  njk_this_i = ((&nobs - j_end + 1)*(&nobs - j_end))/2;
		                  put m_to_go= i_end= j_end= k_next= njk_this_i=;
		               end;
		               nk_this_ij = &nobs - k_next + 1;
		               put m_to_go= nk_this_ij= k_next= ;
		               do while(m_to_go > nk_this_ij);
		                   m_to_go    = m_to_go - nk_this_ij;
		                   j_end      = j_end + 1;
		                   k_next     = j_end + 1;
		                   nk_this_ij = &nobs - k_next  + 1;
		                   put m_to_go= j_end= k_next= nk_this_ij=;
		               end;
		           end;
		           k_end = k_next + m_to_go - 1;

		           output;
		           put "did output" i_start= i_end= j_start= j_end= k_start= k_end=;

		           if grid_node = remain then pernode = pernode - 1;
		           i_start = i_end;
		           j_start = j_end;
		           k_start = k_end + 1;
		           if k_start > &nobs then do;
		              j_start = j_start + 1;
		              if j_start > &nobs - 1 then do;
		                 i_start = i_start + 1;
		                 j_start = i_start + 1;
		              end;
		              k_start = j_start + 1;
		           end;
				end;
		    end;
   		%end;
	run; 

%end;
%else %if &process = LC_LEAVE_L_OUT %then %do;
	data &gridds;
		length node_name $32;
		keep grid_node 
			 i_start i_end 
 	   	     j_start j_end 
        	 k_start k_end
			 inner_loop outer_loop
			 run_status node_name;

		* COERCE ORDER OF OUTPUT VARIABLES;        
	    grid_node=0;
	    i_start=0;
	    i_end=0;
	    j_start=0;
	    j_end=0;
	    k_start=0;
	    k_end=0;
		inner_loop=&innerloop_start;
		outer_loop=&outerloop_start;
		run_status=.;
		node_name="";

        %if &leave_l_out = 1 %then %do;
		    j_start = j_end = .;
		    k_start = k_end = .;
		    if &nnodes >= &nobs then do;
				do grid_node = 1 to &nobs;
		        	i_start = grid_node;
		           	i_end   = grid_node;
		           	output;
		       	end;
		    end;
		    else do;
		    	ipernode = floor(&nobs / &nnodes); 
		        remain = &nobs - &nnodes * ipernode;
				put ipernode= remain=;
		        if remain > 0 then ipernode = ipernode + 1;
		        i_next   = 1;
		        do grid_node = 1 to &nnodes;
		        	i_start = i_next;
		           	i_end   = i_start + ipernode - 1;
		           	i_next  = i_end + 1;
		           	if grid_node = remain then 
						ipernode = ipernode - 1;
		           	output;
		        end;
		   end;
	    %end;
	    %if &leave_l_out = 2 %then %do;

		   k_start = k_end = .;

		   n_to_fit = &nobs * (&nobs - 1 ) / 2 ;
		   if &nnodes >= n_to_fit then do;
              	i_start = 1;
                j_start = 2;
                do  grid_node = 1 to n_to_fit;
                    i_end = i_start;
                    j_end = j_start;
                    output;
                    if j_start = &nobs then do;
                       i_start = i_start + 1;
                       j_start = i_start + 1;
                    end;
                    else do;
                       j_start = j_start + 1;
                    end;
                end;
		    end;
		    else do;
		        pernode  = floor(n_to_fit / &nnodes); 
		        remain   = n_to_fit - &nnodes * pernode;
		        if remain > 0 then pernode = pernode + 1;
		        i_start = 1;
		        j_start = 2;

		        do grid_node = 1 to &nnodes;
		           m_to_go = pernode;
		           i_end   = i_start;
		           j_next  = j_start;
		           nj_this_i = &nobs - j_start + 1;

		           put grid_node= m_to_go= i_start= i_end= nj_this_i=;

		           do while(m_to_go > nj_this_i);
		               m_to_go   = m_to_go - nj_this_i;
		               i_end     = i_end + 1;
		               j_next    = i_end + 1;
		               nj_this_i = &nobs - j_next + 1;
		          	   put "while loop" m_to_go= j_next= i_end= nj_this_i = ;
		           end;
		           j_end = j_next + m_to_go - 1;
		           output;
		           put "did output" i_start= i_end= j_start= j_end=;

		           if grid_node = remain then pernode = pernode - 1;
		           i_start = i_end;
		           j_start = j_end + 1;
		           if j_start > &nobs then do;
		              i_start = i_start + 1;
		              j_start = i_start + 1;
		           end;
		        end;
		     end;
		%end;
		%else %if &leave_l_out = 3 %then %do;

		    n_to_fit = &nobs * (&nobs - 1 ) * (&nobs - 2) / 6 ;

		    if &nnodes >= n_to_fit then do;
	            i_start = 1;
	            j_start = 2;
	            k_start = 3;
	            do  grid_node = 1 to n_to_fit;
	                i_end = i_start;
	                j_end = j_start;
	                k_end = k_start;
	                output;
	                if k_start = &nobs then do;
	                   if j_start = &nobs - 1 then do;
	                      i_start = i_start + 1;
	                      j_start = i_start + 1;
	                   end;
	                   else do;
	                      j_start = j_start + 1;
	                   end;
	                   k_start = j_start + 1;
	                end;
	                else do;
	                   k_start = k_start + 1;
	                end;
	            end;
		    end;
		    else do;
		        pernode  = floor(n_to_fit / &nnodes);
		        remain   = n_to_fit - &nnodes * pernode;
		        if remain > 0 then pernode = pernode + 1;
		        i_start = 1;
		        j_start = 2;
		        k_start = 3;

		        put n_to_fit= pernode= remain= "nobs = &nobs nnodes= &nnodes";

		        do grid_node = 1 to &nnodes;
		           m_to_go = pernode;
		           i_end = i_start;
		           j_end = j_start;
		           j_next = j_start;
		           k_next = k_start;
		           nk_this_ij = &nobs - k_next + 1;
		           njk_this_i = nk_this_ij + ((&nobs - j_next - 1)*(&nobs-j_next))/2;
		           put grid_node= i_start= j_start= k_start= nk_this_ij= njk_this_i=;
		           if m_to_go <= nk_this_ij then do;
		               j_end = j_start;
		           end;
		           else if m_to_go <= njk_this_i then do;
		               put m_to_go= "<=njk";
		               do while(m_to_go > nk_this_ij);
		                   m_to_go    = m_to_go - nk_this_ij;
		                   j_end      = j_end + 1;
		                   k_next     = j_end + 1;
		                   nk_this_ij = &nobs - k_next  + 1;
		               end;
		           end;
		           else do;
		               put m_to_go= ">njk";
		               do while(m_to_go > njk_this_i);
		                  m_to_go = m_to_go - njk_this_i;
		                  i_end   = i_end + 1;
		                  j_end   = i_end + 1;
		                  k_next  = j_end + 1;
		                  njk_this_i = ((&nobs - j_end + 1)*(&nobs - j_end))/2;
		                  put m_to_go= i_end= j_end= k_next= njk_this_i=;
		               end;
		               nk_this_ij = &nobs - k_next + 1;
		               put m_to_go= nk_this_ij= k_next= ;
		               do while(m_to_go > nk_this_ij);
		                   m_to_go    = m_to_go - nk_this_ij;
		                   j_end      = j_end + 1;
		                   k_next     = j_end + 1;
		                   nk_this_ij = &nobs - k_next  + 1;
		                   put m_to_go= j_end= k_next= nk_this_ij=;
		               end;
		           end;
		           k_end = k_next + m_to_go - 1;

		           output;
		           put "did output" i_start= i_end= j_start= j_end= k_start= k_end=;

		           if grid_node = remain then pernode = pernode - 1;
		           i_start = i_end;
		           j_start = j_end;
		           k_start = k_end + 1;
		           if k_start > &nobs then do;
		              j_start = j_start + 1;
		              if j_start > &nobs - 1 then do;
		                 i_start = i_start + 1;
		                 j_start = i_start + 1;
		              end;
		              k_start = j_start + 1;
		           end;
				end;
		    end;
   		%end;
	run; 

%end;


%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_BIAS_PLOTS_DATA
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/13/07
**  PURPOSE: Generate learning curve bias estimates jmp plots data 
**  HISTORY: action                                        date     who
**           Created.                                      07/13/07 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:   Library path of output data sets 
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval) 
**  NCLASSES:	Number of qualitative trait variable category
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  BIAS_PLOTS_OUTPUT: Bias estimates jmp plots output data name
**
**  NOTES:
************************************************************************;
%macro lc_bias_plots_data(outputpath=,
						  trait_type=,
					      nclasses=,
						  n_innerloop=,
						  outerloop_start=,
						  outerloop_end=,
						  models_comparison_output=,
						  bias_plots_output=);

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%if %sysfunc(exist(&cvoutlib..&models_comparison_output)) = 1 %then %do;
	%NObsVars(&cvoutlib..&models_comparison_output);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&models_comparison_output CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&models_comparison_output DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(&cvoutlib.._metadat)) ne 1 %then %do;
	%let exiterror = 1;
	%put ERROR: REQUIRED &cvoutlib.._metadat DOES NOT EXIST.;
	%goto exit;
%end;

proc freq data=&cvoutlib.._metadat order=data noprint;
	tables METHOD*METHOD_I/out=fmethod(keep=METHOD METHOD_I);
	tables OUTER_LOOP/out=fouter(keep=OUTER_LOOP);
	tables INNER_LOOP/out=finner(keep=INNER_LOOP);
run;
proc sort data=fmethod;
	by METHOD_I;
run;
* RETRIEVE NUMBER OF MODELS, AND EACH MODEL NAMES;
%NObsVars(fmethod);
%let n_models = &nObs; 
%let _mid_ = %sysfunc(open(fmethod));
%let methodnum = %sysfunc(varnum(&_mid_, METHOD));
%do m=1 %to &n_models;
	%let rc = %sysfunc(fetchobs(&_mid_, &m));
	%let mdlname&m = %sysfunc(getvarc(&_mid_, &methodnum));
%end;
%let _mid_ = %sysfunc(close(&_mid_));

* RETRIEVE OUTERLOOP_START AND OUTERLOOP_END;
proc sort data=fouter;
	by OUTER_LOOP;
run;
%NObsVars(fouter);
%let n_outers = &nObs;
%let _oid_ = %sysfunc(open(fouter));
%let rc = %sysfunc(fetchobs(&_oid_, 1));
%let outerloop_start = %sysfunc(getvarn(&_oid_, 1));
%let rc = %sysfunc(fetchobs(&_oid_, &n_outers));
%let outerloop_end = %sysfunc(getvarn(&_oid_, 1)); 
%let _oid_ = %sysfunc(close(&_oid_));

* RETRIEVE N_INNERLOOP;
%NObsVars(finner);
%let n_innerloop = &nObs;

%do m=1 %to &n_models;
   %let mdllabel&m = %scan(&model_labels,&m,"|");
%end;

proc sort data=&cvoutlib..&models_comparison_output out=sorted;
	by train_size inner_loop;
run;
proc means data=sorted noprint;
	by train_size inner_loop;
	output out=fitmean;
run;
data &cvoutlib..&bias_plots_output;
	set fitmean;
	drop _TYPE_ _FREQ_ _STAT_ OUTER_LOOP;
	if _STAT_ = "MEAN" then output;
run;

%do inner_loop = 1 %to &n_innerloop;
	%do outer_loop = &outerloop_start %to &outerloop_end;
		data p;
			set &cvoutlib..&models_comparison_output;
				where INNER_LOOP = &inner_loop and 
					  OUTER_LOOP = &outer_loop;
				%do nm=1 %to &n_models;
					%let mdlname=&&mdlname&nm;
					%let mdllabel=&&mdllabel&nm;
					rename BIAS_ASE_m&nm=BIAS_ASE_m&nm._&outer_loop;
					rename BIAS_RMSE_m&nm=BIAS_RMSE_m&nm._&outer_loop;
					rename ABIAS_ASE_m&nm=ABIAS_ASE_m&nm._&outer_loop;
					rename ABIAS_RMSE_m&nm=ABIAS_RMSE_m&nm._&outer_loop;
					rename SBIAS_ASE_m&nm=SBIAS_ASE_m&nm._&outer_loop;
					rename SBIAS_RMSE_m&nm=SBIAS_RMSE_m&nm._&outer_loop;
					label BIAS_ASE_m&nm="BIAS ASE &mdllabel &outer_loop";
					label BIAS_RMSE_m&nm="BIAS RMSE &mdllabel &outer_loop";
					label ABIAS_ASE_m&nm="ABIAS ASE &mdllabel &outer_loop";
					label ABIAS_RMSE_m&nm="ABIAS RMSE &mdllabel &outer_loop";
					label SBIAS_ASE_m&nm="SBIAS ASE &mdllabel &outer_loop";
					label SBIAS_RMSE_m&nm="SBIAS RMSE &mdllabel &outer_loop";
					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						rename BIAS_ACC_m&nm=BIAS_ACC_m&nm._&outer_loop;
						rename ABIAS_ACC_m&nm=ABIAS_ACC_m&nm._&outer_loop;
						rename SBIAS_ACC_m&nm=SBIAS_ACC_m&nm._&outer_loop;
						label BIAS_ACC_m&nm="BIAS ACC &mdllabel &outer_loop";
						label ABIAS_ACC_m&nm="ABIAS ACC &mdllabel &outer_loop";
						label SBIAS_ACC_m&nm="SBIAS ACC &mdllabel &outer_loop";
			    		%if &nclasses = 2 %then %do;
							rename BIAS_AUC_m&nm=BIAS_AUC_m&nm._&outer_loop;
							rename ABIAS_AUC_m&nm=ABIAS_AUC_m&nm._&outer_loop;
							rename SBIAS_AUC_m&nm=SBIAS_AUC_m&nm._&outer_loop;
							label BIAS_AUC_m&nm="BIAS AUC &mdllabel &outer_loop";
							label ABIAS_AUC_m&nm="ABIAS AUC &mdllabel &outer_loop";
							label SBIAS_AUC_m&nm="SBIAS AUC &mdllabel &outer_loop";
						%end;
					%end;
					%else %do;
						rename BIAS_HAC_m&nm=BIAS_HAC_m&nm._&outer_loop;
						rename ABIAS_HAC_m&nm=ABIAS_HAC_m&nm._&outer_loop;
						rename SBIAS_HAC_m&nm=SBIAS_HAC_m&nm._&outer_loop;
						label BIAS_HAC_m&nm="BIAS HARRELLC &mdllabel &outer_loop";
						label ABIAS_HAC_m&nm="ABIAS HARRELLC &mdllabel &outer_loop";
						label SBIAS_HAC_m&nm="SBIAS HARRELLC &mdllabel &outer_loop";
						rename BIAS_CORR_m&nm=BIAS_CORR_m&nm._&outer_loop;
						rename ABIAS_CORR_m&nm=ABIAS_CORR_m&nm._&outer_loop;
						rename SBIAS_CORR_m&nm=SBIAS_CORR_m&nm._&outer_loop;
						label BIAS_CORR_m&nm="BIAS CORR &mdllabel &outer_loop";
						label ABIAS_CORR_m&nm="ABIAS CORR &mdllabel &outer_loop";
						label SBIAS_CORR_m&nm="SBIAS CORR &mdllabel &outer_loop";
					%end;
					keep BIAS_ASE_m&nm 
						 BIAS_RMSE_m&nm 
						 ABIAS_ASE_m&nm 
						 ABIAS_RMSE_m&nm 
						 SBIAS_ASE_m&nm 
						 SBIAS_RMSE_m&nm 
						 %if %upcase(&trait_type)=QUALITATIVE %then %do;
							 BIAS_ACC_m&nm 
							 ABIAS_ACC_m&nm 
							 SBIAS_ACC_m&nm 
			    			 %if &nclasses = 2 %then %do;
								 BIAS_AUC_m&nm
								 ABIAS_AUC_m&nm	
								 SBIAS_AUC_m&nm
							 %end;
						 %end;
						 %else %do;
							 BIAS_HAC_m&nm 
							 ABIAS_HAC_m&nm 
							 SBIAS_HAC_m&nm 
							 BIAS_CORR_m&nm 
							 ABIAS_CORR_m&nm 
							 SBIAS_CORR_m&nm 
						 %end;
					;
				%end;
				keep TRAIN_SIZE;
		run;
		%NObsVars(p);
		%if &nObs and &nVars %then %do;
			data &cvoutlib..&bias_plots_output;
				merge
					&cvoutlib..&bias_plots_output
					p;
				by train_size;
			run;
		%end;
	%end;
%end;

* CALCULATE MIN AND MAX OF MODEL FIT STATISTICS FOR EACH TRAIN_SIZE;
proc means data=sorted noprint; 
	by train_size;
	var 
	%do nm=1 %to &n_models;
		BIAS_ASE_m&nm
		BIAS_RMSE_m&nm
		ABIAS_ASE_m&nm
		ABIAS_RMSE_m&nm
		SBIAS_ASE_m&nm
		SBIAS_RMSE_m&nm
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			BIAS_ACC_m&nm
			ABIAS_ACC_m&nm
			SBIAS_ACC_m&nm
			%if &nclasses = 2 %then %do;
				BIAS_AUC_m&nm	
				ABIAS_AUC_m&nm
				SBIAS_AUC_m&nm
			%end;
		%end;
		%else %do;
			BIAS_HAC_m&nm
			ABIAS_HAC_m&nm
			SBIAS_HAC_m&nm
			BIAS_CORR_m&nm
			ABIAS_CORR_m&nm
			SBIAS_CORR_m&nm
		%end;
	%end;
	;
	output out=minmax;
run;
data min;
	set minmax;
	if _STAT_ = "MIN" then output;
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
			rename BIAS_ASE_m&nm=B_ASE_m&nm._MIN;
			rename BIAS_RMSE_m&nm=B_RMSE_m&nm._MIN;
			label BIAS_ASE_m&nm="BIAS ASE &mdllabel MIN";
			label BIAS_RMSE_m&nm="BIAS RMSE &mdllabel MIN";
			rename ABIAS_ASE_m&nm=AB_ASE_m&nm._MIN;
			rename ABIAS_RMSE_m&nm=AB_RMSE_m&nm._MIN;
			label ABIAS_ASE_m&nm="ABIAS ASE &mdllabel MIN";
			label ABIAS_RMSE_m&nm="ABIAS RMSE &mdllabel MIN";
			rename SBIAS_ASE_m&nm=SB_ASE_m&nm._MIN;
			rename SBIAS_RMSE_m&nm=SB_RMSE_m&nm._MIN;
			label SBIAS_ASE_m&nm="SBIAS ASE &mdllabel MIN";
			label SBIAS_RMSE_m&nm="SBIAS RMSE &mdllabel MIN";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				rename BIAS_ACC_m&nm=B_ACC_m&nm._MIN;
				label BIAS_ACC_m&nm="BIAS ACC &mdllabel MIN";
				rename ABIAS_ACC_m&nm=AB_ACC_m&nm._MIN;
				label ABIAS_ACC_m&nm="ABIAS ACC &mdllabel MIN";
				rename SBIAS_ACC_m&nm=SB_ACC_m&nm._MIN;
				label SBIAS_ACC_m&nm="SBIAS ACC &mdllabel MIN";
				%if &nclasses = 2 %then %do;
					rename BIAS_AUC_m&nm=B_AUC_m&nm._MIN;
					label BIAS_AUC_m&nm="BIAS AUC &mdllabel MIN";
					rename ABIAS_AUC_m&nm=AB_AUC_m&nm._MIN;
					label ABIAS_AUC_m&nm="ABIAS AUC &mdllabel MIN";
					rename SBIAS_AUC_m&nm=SB_AUC_m&nm._MIN;
					label SBIAS_AUC_m&nm="SBIAS AUC &mdllabel MIN";
				%end;
			%end;
			%else %do;
				rename BIAS_HAC_m&nm=B_HAC_m&nm._MIN;
				label BIAS_HAC_m&nm="BIAS HARRELLC &mdllabel MIN";
				rename ABIAS_HAC_m&nm=AB_HAC_m&nm._MIN;
				label ABIAS_HAC_m&nm="ABIAS HARRELLC &mdllabel MIN";
				rename SBIAS_HAC_m&nm=SB_HAC_m&nm._MIN;
				label SBIAS_HAC_m&nm="SBIAS HARRELLC &mdllabel MIN";
				rename BIAS_CORR_m&nm=B_CORR_m&nm._MIN;
				label BIAS_CORR_m&nm="BIAS CORR &mdllabel MIN";
				rename ABIAS_CORR_m&nm=AB_CORR_m&nm._MIN;
				label ABIAS_CORR_m&nm="ABIAS CORR &mdllabel MIN";
				rename SBIAS_CORR_m&nm=SB_CORR_m&nm._MIN;
				label SBIAS_CORR_m&nm="SBIAS CORR &mdllabel MIN";
			%end;
	%end;
	drop _TYPE_ _FREQ_ _STAT_;
run;
data max;
	set minmax;
	if _STAT_ = "MAX" then output;
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
			rename BIAS_ASE_m&nm=B_ASE_m&nm._MAX; 
			rename BIAS_RMSE_m&nm=B_RMSE_m&nm._MAX;
			label BIAS_ASE_m&nm="BIAS ASE &mdllabel MAX";
			label BIAS_RMSE_m&nm="BIAS RMSE &mdllabel MAX";
			rename ABIAS_ASE_m&nm=AB_ASE_m&nm._MAX;
			rename ABIAS_RMSE_m&nm=AB_RMSE_m&nm._MAX;
			label ABIAS_ASE_m&nm="ABIAS ASE &mdllabel MAX";
			label ABIAS_RMSE_m&nm="ABIAS RMSE &mdllabel MAX";
			rename SBIAS_ASE_m&nm=SB_ASE_m&nm._MAX;
			rename SBIAS_RMSE_m&nm=SB_RMSE_m&nm._MAX;
			label SBIAS_ASE_m&nm="SBIAS ASE &mdllabel MAX";
			label SBIAS_RMSE_m&nm="SBIAS RMSE &mdllabel MAX";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				rename BIAS_ACC_m&nm=B_ACC_m&nm._MAX;
				label BIAS_ACC_m&nm="BIAS ACC &mdllabel MAX";
				rename ABIAS_ACC_m&nm=AB_ACC_m&nm._MAX;
				label ABIAS_ACC_m&nm="ABIAS ACC &mdllabel MAX";
				rename SBIAS_ACC_m&nm=SB_ACC_m&nm._MAX;
				label SBIAS_ACC_m&nm="SBIAS ACC &mdllabel MAX";
				%if &nclasses = 2 %then %do;
					rename BIAS_AUC_m&nm=B_AUC_m&nm._MAX;
					label BIAS_AUC_m&nm="BIAS AUC &mdllabel MAX";
					rename ABIAS_AUC_m&nm=AB_AUC_m&nm._MAX;
					label ABIAS_AUC_m&nm="ABIAS AUC &mdllabel MAX";
					rename SBIAS_AUC_m&nm=SB_AUC_m&nm._MAX;
					label SBIAS_AUC_m&nm="SBIAS AUC &mdllabel MAX";
				%end;
			%end;
			%else %do;
				rename BIAS_HAC_m&nm=B_HAC_m&nm._MAX;
				label BIAS_HAC_m&nm="BIAS HARRELLC &mdllabel MAX";
				rename ABIAS_HAC_m&nm=AB_HAC_m&nm._MAX;
				label ABIAS_HAC_m&nm="ABIAS HARRELLC &mdllabel MAX";
				rename SBIAS_HAC_m&nm=SB_HAC_m&nm._MAX;
				label SBIAS_HAC_m&nm="SBIAS HARRELLC &mdllabel MAX";
				rename BIAS_CORR_m&nm=B_CORR_m&nm._MAX;
				label BIAS_CORR_m&nm="BIAS CORR &mdllabel MAX";
				rename ABIAS_CORR_m&nm=AB_CORR_m&nm._MAX;
				label ABIAS_CORR_m&nm="ABIAS CORR &mdllabel MAX";
				rename SBIAS_CORR_m&nm=SB_CORR_m&nm._MAX;
				label SBIAS_CORR_m&nm="SBIAS CORR &mdllabel MAX";
			%end;
	%end;
	drop _TYPE_ _FREQ_ _STAT_;
run;
data &cvoutlib..&bias_plots_output;
	merge
		&cvoutlib..&bias_plots_output
		min 
		max;
	by train_size;
run;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_CV_ITERATION_LOOP
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    08/11/07
**  PURPOSE: Learning curve hold-out iteration loop  
**  HISTORY: action                                       date     who
**           Created.                                     08/11/08 pyt 
**			 Added innerloop_option and testset_option	  11/03/09 pyt					  11/13/09 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:     Number of models
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  TRAIN_SIZE:   Training size 
**  OUTER_LOOP:	Outer loop index
**  INNER_LOOP:	Inner loop index
**  ITER_START:	Hold-out iteration starting index
**  ITER_END: Hold-out iteration ending index
**  KFOLD: K-fold cross validation 
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  LCR_PREFIX: Prefix of LC prediction results data set name
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**
**  NOTES:
** 
************************************************************************;
%macro lc_cv_iteration_loop(datapath=,
				            outputpath=,
						    dsname=,
							innerloop_option=,
							testset_option=,
						    model_names=,
						    n_models=,
						    trait=,
						    trait_type=,
						    cv_method=,
					        train_size=,
					        outer_loop=,	
				            inner_loop=,
 						    iter_start=,
				   		    iter_end=,
						    kfold=,
						    groupname=,
						    blockname=,
						    cv_vars_outds=,
						    cvname=,
						    cv_holdout_percent=,
							lcr_prefix=,
						    prevalences=
						    );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname datalib "&datapath";
	%let datalib = datalib;
%end;
%else %do;
    %let datalib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lclpname = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__lclpname)) = 1 %then %do;
	%NObsVars(&__lclpname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lclpname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lclpname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%do m=1 %to &n_models;
    %let mdlname&m = %scan(&model_names,&m," ");
%end;

* MAIN ITERATION LOOP;
%do iter = &iter_start %to &iter_end;

	%put;
	%put STARTING LC CV ITERATION &iter. of &iter_end;

	* GENERATE MACRO RANDSEED FOR THIS CROSS-VALIDATION ITERATION;
	* KFOLD BASIC uses seed to create cv_var= 1, ..., KFOLD      ;
	* RANDOM, STRATIFIED uses seed to draw random train sample   ; 
	* BLOCK, SPLIT, LEAVEOUT do not use a seed   				 ;

	* %seedget(i=&iter, seedds=&cvoutlib..seedseq, thread=1);

	* RUSS MODIFICATION;
	%seedget(
		i=%eval((&outer_loop-1) * &cv_holdout_iterations + &iter), 
		seedds=&cvoutlib..seedseq, 
		thread=1
	);

	%NObsVars(&__lclpname);
	%let _trainObs_ = &nObs;

	%if &cv_method=BLOCK %then %do;
		%put CV_METHOD: K-FOLD BLOCK.;
		%if %length(&blockname) > 0 and &blockname ne "" %then %do;
			%let block_var=&blockname;
		%end;
		%else %do;
			%let block_var=;
			data _null_;
				 call symput("blocksize", trim(left(put(&_trainObs_/&kfold, best12.))));
			run;
		%end;
	%end;
	%else %if &cv_method=SPLIT %then %do;
		%put CV_METHOD: K-FOLD SPLIT.;
	%end;
	%else %if &cv_method=KFOLD %then %do;
		%put CV_METHOD: K-FOLD RANDOM.;
		%if &kfold > 1 %then %do;
			%if %length(&cv_vars_outds) > 0 and &cv_vars_outds ne "" %then %do;
				 %let cv_outds=&cv_vars_outds;
			%end;
			%else %do;
				 %let cv_outds=&dsname._CV;
			%end;
			%if %length(&cvname) > 0 and &cvname ne "" %then %do;
				 %let cv_var=&cvname;
			%end;
			%else %do;
				 %let cv_var=_CV_;
			%end;
			* GENERATE KFOLD _CV_ VARIABLE;
			* SAVE THE CV_OUTDS DATA SET TO SASUSER ON EACH GRIDNODE;
			%let ntries = 0;
			%let isok = 0;
			%do %while(&isok = 0 and &ntries < 10);
				%let ntries = %eval(&ntries + 1);
				%generate_cv_vars(datapath=&datapath,
					     		  outputpath=,
							      dsname=&dsname,
								  trait=&trait,
								  trait_type=&trait_type,
								  kfold=&kfold,
								  groupname=&groupname,
								  cvname=&cv_var, 
								  cv_seed=&randseed,
								  cv_vars_outds=&cv_outds,
								  iteration=&iter); 
				* UP TO 10 TRIES UNTIL ALL KFOLD ARE CREATED;  
				proc freq data=&cv_outds.&iter noprint;
					tables &cv_var/out=cvfreq;
				run;
				%NObsVars(work.cvfreq);
				%if &nObs = &kfold %then %do;
					%let isok = 1;
				%end; 
				%else %do;
				 	%let randseed = %eval(&randseed+7);
				%end;
			%end;
		%end;
	%end;

	* MAIN KFOLD LOOP;
	%do k=1 %to &kfold;
		
		%put STARTING K-FOLD ITERATION &k OF &kfold.;

		data cvtrain;
			delete;
		run;
		data cvtest;
			delete;
		run;

		* FOR KFOLD=1 USE ALL OF THE DATA FOR BOTH TRAINING AND TEST;
		%if &cv_holdout_percent = 0 %then %do;
			%put CV_METHOD: USING ALL OF THE DATA FOR BOTH TRAINING AND TESTING, 
				 RESULTS ARE LIKELY OPTIMISTIC.;
			data cvtrain;
				set &__lclpname;
			run;
			data cvtest;
				set &__lclpname;
			run;
		%end;
		%else %if &cv_method=RANDOM or &cv_method=STRATIFIED %then %do;
			* 10 TRIES UNTIL CVTRAIN DOES NOT HAVE CONSTANT TRAIT, 
			  TEST SET HAS AT LEAST ONE OBSERVATION;
			%let ntrain = 0;
			%let ntest = 0;
			%let ntries = 0;
		    %let isconstant = 0;
			%let isok = 0;

			%if &cv_method=RANDOM %then %do;
			 	%put CV_METHOD: RANDOM SIMPLE.;
			%end;
			%else %if &cv_method=STRATIFIED	%then %do;
				%put CV_METHOD: RANDOM STRATIFIED.;
			%end;
			%do %while(&isok = 0 and &ntries < 10);
				%let ntries = %eval(&ntries + 1);
				%if &cv_method=RANDOM %then %do;
					data cvtrain cvtest;
						set &__lclpname;
						_I = &iter;
						_K = &k;
						_U = ranuni(&randseed);
						if (_U < &cv_holdout_percent/100) then output cvtest;
						else output cvtrain;
					run;
				%end; 
				%else %if &cv_method=STRATIFIED	%then %do;
					%stratified_sample(datapath=&datapath,
					  			     outputpath=,
								     dsname=&dsname, 
								     trait=&trait, 
								     trait_type=&trait_type, 
								     holdout_percent=&cv_holdout_percent,
								     groupname=&groupname,
								     train_dsname=cvtrain,
								     test_dsname=cvtest,
								     seed=&randseed,
									 iter=&iter,
								     k=&k);
				%end;
				%NObsVars(work.cvtrain);
				%let ntrain = &nObs;
				%if &ntrain > 0 %then %do;
				  %if &trait_type = qualitative %then %do;
						 proc freq data=cvtrain noprint;
							tables &trait/out=tfreq;
						 run;
						 %NObsVars(work.tfreq)
						 %if &nObs < 2 %then %do;
							%let isconstant = 1;
						 %end;
						 %else %do;
							%let isconstant = 0;
						 %end;
				   %end;
				   %else %do;
				   	%if &ntrain = 1 %then %do;
							%let isconstant = 1;
						%end;
						%else %do;
							%let isconstant = 0;
						%end;				   
				   %end;
				%end;
				%if %upcase(innerloop_option) = CVHOLDOUT %then %do;
					%NObsVars(work.cvtest);
					%let ntest = &nObs;
					%if ^&ntest or ^&ntrain or &isconstant = 1 %then %do;
					  	%let isok = 0;
					 	%let randseed = %eval(&randseed+7);
					%end;
					%else %do;
						%let isok = 1;
					%end;
				%end;
				%else %do;
					%if ^&ntrain or &isconstant = 1 %then %do;
					  	%let isok = 0;
					 	%let randseed = %eval(&randseed+7);
					%end;
					%else %do;
						%let isok = 1;
					%end;
				%end;
			%end;
			%if &ntries > 10 %then %do;
				%put OUTER LOOP &outer_loop; 
				%put INNER LOOP &inner_loop; 
				%put CV_ITERATION &iter; 
				%put KFOLD &k; 
				%put ERROR: UNABLE TO OBTAIN TRAINING SET WITH NONCONSTANT TRAIT.;
				%goto iter_end;
			%end;

			* CONFIRM CVTEST IS NOT EMPTY;
			%if %sysfunc(exist(cvtest)) = 1 %then %do;
				%NObsVars(cvtest);
				%if ^&nObs or ^&nVars %then %do;
					%put OUTER LOOP &outer_loop; 
					%put INNER LOOP &inner_loop; 
					%put CV_ITERATION &iter; 
					%put KFOLD &k; 
					%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
					%goto iter_end;
				%end; 
			%end;
			%else %do;
				%put OUTER LOOP &outer_loop; 
				%put INNER LOOP &inner_loop; 
				%put CV_ITERATION &iter; 
				%put KFOLD &k; 
				%put ERROR: CVTEST DOES NOT EXIST.;
				%goto iter_end;
			%end;

		%end;
		%else %if &cv_method=KFOLD or &cv_method=BLOCK or &cv_method=SPLIT %then %do;
			 %if &cv_method=KFOLD %then %do;
				 %put CV_METHOD: K-FOLD RANDOM.;
				 %if &kfold > 1 %then %do;
				 	 proc sort data=&__lclpname;
					 	by __id;
					 run;
					 proc sort data=&cv_outds.&iter;
					 	by __id;
					 run;
					 data ds;
						merge &__lclpname &cv_outds.&iter;
						by __id;
					 run;
					 data cvtrain cvtest;
						  set ds;
						  _I = &iter;
						  _K = &k;
						  if &cv_var = &k then output cvtest;
						  else output cvtrain;
					 run;
				 %end;
				 %else %do;
					 data cvtrain;
					 	set train;
					 run;
				 %end;
			 %end;
			 %else %if &cv_method=BLOCK %then %do;
				 %put CV_METHOD: K-FOLD BLOCK.;
				 %if %length(&block_var) > 0 and &block_var ne "" %then %do;
					data cvtrain cvtest;
						set &__lclpname;
						_I = &iter;
						_K = &k;
						if &block_var = &k then output cvtest;
						else output cvtrain;
					run;
				 %end;
				 %else %do;
					 data cvtrain cvtest;
						  set &__lclpname;
						  _I = &iter;
						  _K = &k;
						  if (&k-1)*&blocksize < _n_ <= &k*&blocksize then output cvtest;
						  else output cvtrain;
					 run;
				 %end;
			 %end;
			 %else %if &cv_method=SPLIT %then %do;
				 %put CV_METHOD: K-FOLD SPLIT.;
				 data cvtrain cvtest;
					  set &__lclpname;
					  _I = &iter;
					  _K = &k;
					  if mod(_n_, &kfold) = (&k-1) then output cvtest;
					  else output cvtrain;
				 run;
			 %end;
			 %if &trait_type = qualitative %then %do;
				%NObsVars(cvtrain);
				%if ^&nObs or ^&nVars %then %do;
					%put OUTER LOOP &outer_loop; 
					%put INNER LOOP &inner_loop; 
					%put CV_ITERATION &iter; 
					%put KFOLD &k; 
					%put ERROR: CVTRAING CONTAINS 0 OBSERVATIONS.;
					%goto iter_end;
				%end;
				proc freq data=cvtrain noprint;
					tables &trait/out=tfreq;
				run;
				%NObsVars(work.tfreq);
				%let _ftnObs_ = &nObs;
				%if &_ftnObs_ < 2 %then %do;
					%put OUTER LOOP &outer_loop; 
					%put INNER LOOP &inner_loop; 
					%put CV_ITERATION &iter; 
					%put KFOLD &k; 
					%put ERROR: UNABLE TO OBTAIN TRAINING SET WITH NONCONSTANT TRAIT.;
					%goto iter_end;
				%end;
			 %end;

			 %if &kfold > 1 %then %do;
				* CONFIRM CVTEST IS NOT EMPTY;
				%if %sysfunc(exist(cvtest)) = 1 %then %do;
					%NObsVars(cvtest);
					%if ^&nObs or ^&nVars %then %do;
						%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
						%goto iter_end;
					%end; 
				%end;
				%else %do;
					%put ERROR: CVTEST DOES NOT EXIST.;
					%goto iter_end;
				%end;
			%end;

		%end;

		%if (&CV_ListAll = Yes) %then %do;
			 data &cvoutlib..cvtrain_&iter._&k._i&inner_loop._o&outer_loop;
				  set cvtrain;
			 run;
			 data &cvoutlib..cvtest_&iter._&k._i&inner_loop._o&outer_loop;
				  set cvtest;
			 run;
		%end;

		* CREATE CROSS VALIDATION MODELS PREDICTIONS,
		  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
		  CV_<MODELNAME>_PREDS AND CV_<MODELNAME>_VARS, 
		  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
		%if %upcase(&innerloop_option) = CVTEST %then %do; 
			%if %upcase(&testset_option) = FIXTEST %then %do;
				%fitmodels(train_dsname=cvtrain,
						   test_dsname=&cvoutlib..Fixtest);
			%end;
			%else %if %upcase(&testset_option) = HOLDOUT %then %do;
				%fitmodels(train_dsname=cvtrain,
						   test_dsname=outer_holdout);
			%end;
		%end;
		%else %if %upcase(&innerloop_option) = CVHOLDOUT %then %do;
			%fitmodels(train_dsname=cvtrain,
					   test_dsname=cvtest);
		%end;

		%if &trait_type = qualitative %then %do;
			%put nclasses=&nclasses;
			%put event=&event;
		%end;

		* APPEND CV ITERATION MODELS PREDICTIONS AND VARS;
		%do nm=1 %to &n_models;
			%let FM_Error = &&FM_Error&nm;
			%put FM_ERROR&nm = &FM_Error;
			%let mdlname=&&mdlname&nm;
			%put mdlname=&mdlname;
			* ERROR CHECKING;
			%if &FM_Error %then %do;
				data &cvoutlib..cvtrain_m&nm._i&iter._k&k._i&inner_loop._o&outer_loop._error;
					set cvtrain;
				run;
				data &cvoutlib..cvtest_m&nm._i&iter._k&k._i&inner_loop._o&outer_loop._error;
					set cvtest;
				run;   	
				
			%end;
			%else %do; 
				%let mdlname=&&mdlname&nm;
				* APPEND PREDICTIONS;
				%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds)) = 1 %then %do;
					%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds);
					%if &nObs and &nVars %then %do;
						data cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							length METHOD $64 ITER 8 K 8 INNER_LOOP 8 OUTER_LOOP 8 TRAIN_SIZE 8;
							set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							ITER=&iter;
							K=&k; 
							RANDSEED=&randseed;
							METHOD = "&mdlname";
							INNER_LOOP=&inner_loop;
							OUTER_LOOP=&outer_loop;
							TRAIN_SIZE=&train_size;
						run;

						%if %sysfunc(exist(&cvoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop)) = 1 %then %do;
							%NObsVars(&cvoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop);
							%let _cvpcumObs_ = &nObs;
							data &cvoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop;
								set 
								%if &_cvpcumObs_ %then %do;
									&cvoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop
								%end;
									cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							run;
						%end;
						%else %do;
							data &cvoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop;
								set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							run;
						%end;
					%end;
				%end;
			%end; 
		%end; * END OF N_MODELS LOOP;
	%end; * END OF KFOLD LOOP; 
	%iter_end:;
%end; * END OF MAIN ITERATION;

%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_STANDARD_ESTIMATES
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/08/07
**  PURPOSE: Learning curve with fixed test data set 
**  HISTORY: action                                        date     who
**           Created.                                      06/08/07 pyt 
**			 Added testset_option						   11/02/09 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  TESTPATH:     Library path of test data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  TEST_DSNAME:  Test data set name specified by user
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:     Number of models
**  NMIN: Min size of available data (default = Nmax / 10)
**  NMAX: Max size of available data (All by default)
**  N_INNERLOOP:  Number of total learning curve inner loop iteration
**  N_OUTERLOOP:  Maximum  number of total LC outer loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  OUTER_METHOD: Method to prepare inner_train training data set 
**				  from outer loop iteration
**				  Methods are: BLOCK or RANDOM	
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Client shared grid folder 
**  SERVERGSHARE: Server shared grid folder
**
**  NOTES:
**  N_OUTERLOOP is only required for BLOCK partition OUTER_METHOD outerloop ethod
**
**  There are two types of fixed test data set: 
**	1. Generalized test: user specifies test set
**  2. Fixed test: test set is created by random or stratified sample
**
**  The same test data is used for all inner, outer loops, and cv 
**  iterations. User specifies nmax or percent_holdout to determine the 
**  size of Strain and Stest. 
**  
**  Learning curve with fixed test data set is not cross validation, 
**  therefore kfold=1 and cv_iteration=1.
************************************************************************;
%macro lc_standard_estimates(datapath=,
							outputpath=,
							dsname=,
							testset_option=,
							trait=,
							trait_type=,
							freq=,
							model_names=,
							n_models=,
							nmin=,
							nmax=,
							n_innerloop=,
							n_outerloop=,
							outerloop_start=,
							outerloop_end=,
							outer_method=,
							groupname=,
							models_comparison_output=,
							prevalences=proportional,
							ngridnodes=,
						   clientgshare=,
						   servergshare=
							);

%put START STANDARD LEARNING CURVE;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) %then %do;
    libname datalib "&datapath";
   %let datalib = datalib;
%end;
%else %do;
    %let datalib = work;
%end;
%if %length(&dsname) > 0 %then %do; 
    %let __lcftname = &datalib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto lc_exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
%end;

%NObsVars(&__lcftname);
%let _tnObs_ = &nObs;  
%let _stnObs_ = &nObs;

* CHECK FOR N_INNER_LOOP SIZE,
  BECAUSE INNERLOOP SIZES ARE EQUALLY SPACED IN FLOOR(log(n)), 
  THIS WILL MAP SEVERAL INTEGERS TO THE SAME INTEGER.   
  MAX_INNERLOOP IS THE MAXIMUM NUMBER OF DISTINCT INTEGERS AFTER THE FLOOR(log(n)) TRANFORMATION. 
  IF NMIN EXISTS, N_INNERLOOP <=FLOOR((&_tnObs_-&nmin)/2.7182818281)
  ELSE N_INNERLOOP <= FLOOR((0.9*&_tnObs_)/2.7182818281);
%if &nmin > 0 %then %do;
	%let max_innerloop = %sysevalf((&_tnObs_-&nmin)/2.7182818281, int);
%end;
%else %do; 
	%let max_innerloop = %sysevalf((0.9*&_tnObs_)/2.7182818281, int);
%end;

%if &n_innerloop > &max_innerloop %then %do;
	%put WARNING: NUMBER OF TRAINING SIZES FOR EACH LEARNING CURVE > &max_innerloop;
	%put RESETTING n_innerloop TO &max_innerloop;
	%let n_innerloop = &max_innerloop;
%end;

%if ^%length(&nmax) %then %do;
    %if %upcase(&testset_option) = HOLDOUT %then %do;
		%let nmax = %sysevalf(&_tnObs_ * 0.9, int);
	%end;
	%else %do;
		%let nmax = &_tnObs_;
	%end;
%end;

data _null_;
	%if &nmax > 0 %then %do;
        %if &nmax > &_tnObs_ %then %do;
           %put ERROR: nmax > NUMBER OF OBSERVATIONS IN THE TRAINING DATA SET.; 
           %let exiterror = 1;
		   %goto lc_exit;
		%end;
		%str(all_train_size=&nmax;);
	%end;
	%else %do;
		%put ERROR: nmax MUST BE GREATER THAN 0.; 
		%let exiterror = 1;
		%goto lc_exit;
	%end;
    %if &outer_method = BLOCK %then %do;
		%if ^%length(&n_outerloop) or &n_outerloop < 1 %then %do;
			%put n_outerloop IS NOT SPECIFIED OR IS < 1 FOR BLOCK OUTER_METHOD.;
			%let exiterror = 1;
			%goto lc_exit;
		%end;
		block_size = floor(all_train_size/&n_outerloop);
        max_inner_size = block_size;
    %end;
    %else %do;
		block_size = &_tnObs_;
        max_inner_size = &nmax;
    %end;
	%if &nmin > 0 %then %do;
		%if &nmin < 8 %then %do;
           %put ERROR: MINIMUM SIZE OF TRAINING SIZE MUST BE >= 8.; 
           %let exiterror = 1;
		   %goto lc_exit;
		%end;
		%else %do;				 
			%str(min_inner_size =&nmin;);
		%end;
	%end;
	%else %do;
		n = ceil(max_inner_size/10);
		min_inner_size=max(n, 8);
	%end;
    %if &n_innerloop = 1 %then %do;
        %str(delta = 0;);
	%end;
    %else %do;
        %str(delta = (log(max_inner_size) - log(min_inner_size))/(&n_innerloop-1););
	%end;
    call symputx("delta", delta);
	call symputx("min_inner_size", put(min_inner_size, 12.0));
	call symputx("max_inner_size", put(max_inner_size, 12.0));
    call symputx("block_size", put(block_size, 12.0));
    call symputx("all_train_size", put(all_train_size, 12.0));
run;

%put delta = &delta;
%put block_size=&block_size ONLY USED WHEN OUTER_METHOD=BLOCK;
%put all_train_size=&all_train_size;
%put _tnObs_ = &_tnObs_;
%put min_inner_size = &min_inner_size;
%put max_inner_size = &max_inner_size;

%if &ngridnodes > 0 %then %do;

	%gridsequence(process=LC_STANDARD,
				  cv_holdout_iteration_start=,
			 	  cv_holdout_iteration_end=,
				  innerloop_start=1,
				  innerloop_end=&n_innerloop,
				  outerloop_start=&outerloop_start,
				  outerloop_end=&outerloop_end,
				  leave_l_out=,
				  nnodes=&ngridnodes,
		          gridds=&cvoutlib..ftgridseq);

	%NObsVars(&cvoutlib..ftgridseq);
	%let ngridnodes = &nObs;
	%put ngridnodes = &ngridnodes;
	%do node=1 %to &nnodes;
		%global node_name&node;
	%end;
	%global rungridseq;
	%let rungridseq=&ngridnodes;

	%UpdateFitModels(nnodes=&ngridnodes,outputpath=&outputpath);

	%do %while (&rungridseq > 0);

		%let max_tries = 100;
		%let waitlist = ;
		%let ng = 0;

		* CHECK GRID NODES RUN_STATUS IN cvgridseq.sas7bdat;
		* FOR ANY RUN_STATUS ne 0, RSUBMIT THE CODE TO GENERATE INTER-MEDIA RESULTS;
		%do gridseq=1 %to &ngridnodes;
			%let _gseq_ = %sysfunc(open(&cvoutlib..ftgridseq));
			%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
			%let status_num = %sysfunc(varnum(&_gseq_, run_status));
			%let gndone = %sysfunc(getvarn(&_gseq_, &status_num));
			%let _gseq_ = %sysfunc(close(&_gseq_));
			%if &gndone=0 %then %do;
	            %put REMOTE SUBMIT COMPLETE ON GNODE&gridseq;
		    %end;
 			%else %if &gndone = . or &gndone = 1 %then %do;
				* KEEP THE RERUN RECORD IN gridseq data set;
				%if &gndone = 1 %then %do;
		        	%put REMOTE SUBMIT FAILED TO EXECUTE ON GNODE&gridseq;
				%end;
				%let ng=%eval(&ng+1);
				%let node=&ng;
				%goto run_gridseq;
		    %end;
		    %else %if &gndone = 2 %then %do;
		        %put REMOTE SUBMIT IS STILL IN PROGRESS ON GNODE&gridseq;
			%end;

			%run_gridseq:
				* RETRIEVE OUTER_START, OUTER_END, INNER_START, INNER_END, 
				* ITER_START, ITER_END FROM ftgridseq.sas7bdat OF EACH NODE; 
				%let _gseq_ = %sysfunc(open(&cvoutlib..ftgridseq));
				%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
				%let outstart_num = %sysfunc(varnum(&_gseq_, outer_start));
				%let outer_start = %sysfunc(getvarn(&_gseq_, &outstart_num));
				%let outend_num	= %sysfunc(varnum(&_gseq_, outer_end));
				%let outer_end = %sysfunc(getvarn(&_gseq_, &outend_num));
				%let instart_num = %sysfunc(varnum(&_gseq_, inner_start));
				%let inner_start =  %sysfunc(getvarn(&_gseq_, &instart_num));
				%let inend_num = %sysfunc(varnum(&_gseq_, inner_end));
				%let inner_end =  %sysfunc(getvarn(&_gseq_, &inend_num));
				%let _gseq_ = %sysfunc(close(&_gseq_));

				data &cvoutlib..ftgridseq;
					set &cvoutlib..ftgridseq;
					if grid_node = &gridseq then do;
						run_status=2;
					end;
				run;

				%let waitlist=&waitlist gnode&node;
				%let tries = 1;
				%let finished = 0;
				%do %while(^&finished);
					%put SIGN ON ATTEMPT &tries OF &max_tries TO GRID NODE &node....;
					signon gnode&node;
					%let node_name = ;
					%let node_name&gridseq = %sysfunc(grdsvc_getname(gnode&node));
					%let node_name = &&node_name&gridseq;
					%if %length(&node_name) %then %do;
						%put SUCCESSFUL SIGN ON TO GNODE&node = &node_name;
						%put RUNNING OUTER_LOOP &OUTER_START TO &OUTER_END;
						%put INNER_LOOP &inner_start TO &inner_end ON GNODE&node = &node_name;
						%let finished = 1;
						* UPDATE NODE_NAME IN ftgridseq.sas7bdat;
						data &cvoutlib..ftgridseq;
							set &cvoutlib..ftgridseq;
							if grid_node = &gridseq then do;
								node_name="&node_name";
							end;
						run;	
					%end;
					%else %do;
						%let tries = %eval(&tries+1);
						%if &tries > &max_tries %then %do;
							%put ERROR: Grid sign-on to GNODE&gridseq failed after &max_tries tries.;
							%let exiterror = 1;
							%let finished = 1;
							signoff _all_;
							%goto cv_exit;
						%end;
					%end;
				%end;

				%if %sysfunc(sysprod(prodnum007)) ne 0 %then %do; 
					*S1313431: SAS/CONNECT be licensed*;
					%macro ConnectVars03;
						* SYSLPUT CREATES A NEW MACRO VARIABLE OT MODIFIES THE VALUE OF  
						  AN EXISTING MACRO VARIABLE ON A REMOTE HOST OR SERVER;
					  	%syslput GSHARE=&servergshare;
					  	%syslput LS_HOME=&LS_HOME;
					  	%syslput GHOME=&servergshare.gnode&node.%str(/);
					  	%syslput USER_HOME=&servergshare.gnode&node.%str(/);
						%syslput ProcessName=LCMC_FIXTEST&gridseq;
						%syslput inlib=inlib;
						%syslput cvoutlib=cvoutlib;
						%syslput datapath=&datapath;
						%syslput outpath=&servergshare;
						%syslput outputpath=&servergshare;
						%syslput dsname=&dsname;
						%syslput testset_option=&testset_option;
						%syslput model_names=&model_names;
						%syslput n_models=&n_models;
						%do i=1 %to &n_models;
							%syslput mdlname&i = &&mdlname&i;
							%syslput FM_Error&i = 0;
						%end;
						%syslput mdlname = &mdlname1;
						%syslput trait=&trait;
						%syslput trait_type=&trait_type;
						%syslput n_innerloop=&n_innerloop;
						%syslput inner_start=&inner_start;
						%syslput inner_end=&inner_end;
						%syslput n_outerloop=&n_outerloop;
						%syslput outer_start=&outer_start;
						%syslput outer_end=&outer_end;
						%syslput outer_method=&outer_method;
						%syslput prevalences=&prevalences;
						%syslput randseed=;
						%syslput delta=&delta;
						%syslput min_inner_size=&min_inner_size;
						%syslput max_inner_size=&max_inner_size;
						%syslput block_size=&block_size;
						%syslput node=&node;
						%syslput node_name=&node_name;
					  	%syslput node=&node;
						%syslput gridseq=&gridseq;
						%syslput gndone&gridseq = 2;
					%mend;
				%end;
				%else %do;
					%macro ConnectVars03;
						%put EMPTY.;	
					%mend;				
				%end;
				%ConnectVars03;

				* RSUBMIT BLOCK OF WORK TO BE SUBMITTED TO GRID NODE;
				rsubmit cwait=no persist=no cmacvar=gndone&gridseq;
				
					%macro ITERATION_LOOP;

					%put rsubmit for &ProcessName;
					%put gshare = &gshare;
			  		%put ghome = &GHOME;
			  		%put node number = &node;
			  		%put node name = &node_name;
					
					* MACRO DEFINED IN GRID SHARE FOLDER;
					filename gmacro "&GHOME.MacroLib";  
			  		%if ^%sysfunc(fexist(gmacro)) %then %do;
					    %put ERROR: Unable to find grid share folder &GHOME.MacroLib. ;
	  					 %put Make sure the server version of your grid share path is correct using ;
						 %put File > Configure Life Sciences Settings.;
			  		    %let exiterror = 1;
			  		    %goto iter_exit;
			  		%end;
					%inc gmacro(
						UtilityMacros.sas,
						CrossValidationModelComparisonMacros.sas,
						FitModels.sas,
						BinaryResponseEffectSelectionMacros.sas,
						SeedMacros.sas
					);
					filename gmacro clear;

					%put outer_start = &outer_start;
					%put outer_end = &outer_end;
					%put inner_start = &inner_start;
					%put inner_end = &inner_end;

					* LOOP;
					%lc_standard_iteration_loop(datapath=&datapath,
											    outputpath=&outputpath,
											    dsname=&dsname,
											    testset_option=&testset_option,
											    model_names=&model_names,
											    n_models=&n_models,
											    trait=&trait,
											    trait_type=&trait_type,
											    n_outerloop=&n_outerloop,
											    outer_start=&outer_start,
											    outer_end=&outer_end,
											    n_innerloop=&n_innerloop,
											    inner_start=&inner_start,
											    inner_end=&inner_end,
											    outer_method=&outer_method,
											    groupname=&groupname,
											    prevalences=&prevalences,
											    delta=&delta,
											    min_inner_size=&min_inner_size,
											    max_inner_size=&max_inner_size,
											    block_size=&block_size
											   );

					* SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
					%if %upcase(&trait_type) = QUALITATIVE %then %do;	
						%do c=1 %to &nclasses;
							%let class=&&class&c;
							%let p_prob=&&p_prob&c;
							data _namedat;
								length CLASS P_PROB $32 ;
							 	CLASS="&class";
								P_PROB="&p_prob";
								EVENT="&event";
							run;
							%if &c=1 %then %do;
								data &cvoutlib.._namedat&gridseq;
									set _namedat;
								run;
							%end;
							%else %do;
								data &cvoutlib.._namedat&gridseq;
									set &cvoutlib.._namedat&gridseq
										_namedat;
								run;
							%end;
						%end;
					%end;

					%iter_exit:;
					%mend;

					%ITERATION_LOOP;

				endrsubmit;
			;

		%end;

		* WAIT FOR ALL RSUBMITS TO FINISH;
		%put waitlist = &waitlist;
		waitfor _ALL_ &waitlist;

		* STOP SAS RUNNING ON THE GRID NODES;                                 
		* signoff _all_;

		* UPDATE GRID NODES RUN_STATUS IN ftgridseq.sas7bdat;
		data &cvoutlib..ftgridseq;
			set &cvoutlib..ftgridseq;
			%do node=1 %to &ngridnodes;
				%let gndone=&&gndone&node;
				if grid_node = &node and run_status ne 0 then do;
					run_status=&gndone;
				end;
			%end;
		run;

		data rungrid;
			set &cvoutlib..ftgridseq;
			if run_status ne 0 then output;
		run;

		%NObsVars(rungrid);
		%let rungridseq=&nObs;

	%end;

%end;
%else %do;
	
	%let outer_start = &outerloop_start;
	%let outer_end = &outerloop_end;
	%let inner_start = &n_innerloop;
	%let inner_end = 1;

	%lc_standard_iteration_loop(datapath=&datapath,
							   outputpath=&outputpath,
							   dsname=&dsname,
							   testset_option=&testset_option,
							   model_names=&model_names,
							   n_models=&n_models,
							   trait=&trait,
							   trait_type=&trait_type,
							   n_outerloop=&n_outerloop,
							   outer_start=&outer_start,
							   outer_end=&outer_end,
							   n_innerloop=&n_innerloop,
							   inner_start=&inner_start,
							   inner_end=&inner_end,
							   outer_method=&outer_method,
							   groupname=&groupname,
							   prevalences=&prevalences,
							   delta=&delta,
							   min_inner_size=&min_inner_size,
							   max_inner_size=&max_inner_size,
							   block_size=&block_size
							   );

	*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%do c=1 %to &nclasses;
			%let class=&&class&c;
			%let p_prob=&&p_prob&c;
			data _namedat;
				length CLASS P_PROB $32 ;
			 	CLASS="&class";
				P_PROB="&p_prob";
				EVENT="&event";
			run;
			%if &c=1 %then %do;
				data &cvoutlib.._namedat;
					set _namedat;
				run;
			%end;
			%else %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat
						_namedat;
				run;
			%end;
		%end;
	%end;

%end;

%lc_exit:
;

%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_STANDARD_ITERATION_LOOP
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    08/11/08
**  PURPOSE: Learning curve with fixed test data set iterations 
**  HISTORY: action                                        date     who
**           Created.                                      08/11/08 pyt 
** 			 Added testset_option						   11/02/09	pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:     Number of models
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  N_OUTERLOOP:  Number of learning curve outer loop iteration
**  OUTER_START:  Outer loop starting index
**  OUTER_END:    Outer loop ending index
**  N_INNERLOOP:  Number of learning curve inner loop iteration
**  INNER_START:  Inner loop starting index
**  INNER_END:    Inner loop ending index
**  OUTER_METHOD: Method to prepare inner_train training data set 
**				  from outer loop iteration
**				  Methods are: BLOCK or RANDOM	
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  DELTA: Parameter for determining training size within inner loop
**  MIN_INNER_SIZE:	Minimum training size for inner loop 
**  MAX_INNER_SIZE:	Maximum training size for inner loop
**  BLOCK_SIZE: Bloc size for BLOCK outer_method 
**
**  NOTES:
**
************************************************************************;
%macro lc_standard_iteration_loop(datapath=,
								  outputpath=,
								  dsname=,
								  testset_option=,
								  model_names=,
								  n_models=,
								  trait=,
								  trait_type=,
								  n_outerloop=,
								  outer_start=,
								  outer_end=,
								  n_innerloop=,
								  inner_start=,
								  inner_end=,
								  outer_method=,
								  prevalences=,
								  delta=,
								  min_inner_size=,
								  max_inner_size=,
								  groupname=,
								  block_size=
								  );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lcftlpname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto lc_exit;
%end;

%if %sysfunc(exist(&__lcftlpname)) = 1 %then %do;
	%NObsVars(&__lcftlpname);
	%let _tnObs_ = &nObs;
	%let _stnObs0_ = &nObs;
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lcftlpname CONTAINS 0 OBSERVATIONS.;
		%goto lc_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lcftlpname DOES NOT EXIST.;
	%goto lc_exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;

%do outer_loop = &outer_start %to &outer_end;
	%put;
	%put Computing outerloop &outer_loop of &outer_end;
	%put;

	* INITIALIZE TRAINING SIZE;
	%let _stnObs_ = &_stnObs0_;
	
	* GENERATE MACRO RANDSEED FOR OUTER LOOP THREAD=3, INDEX=&outer_loop;
	%seedget(i=&outer_loop, seedds=&cvoutlib..seedseq, thread=3);

	* PREPARE INNER_TRAIN: TRAINING DATA AVAILABLE IN INNER LOOP;
	%if &n_outerloop = 1 and %upcase(&testset_option) = FIXTEST %then %do;
		data inner_train;
			set &cvoutlib..Strain;
		run;
	%end;
	%else %do;
		%if &outer_method = BLOCK %then %do;
		    %let FirstObs = %eval(&block_size * (&outer_loop - 1) + 1);
            %let LastObs  = %eval(&FirstObs + &block_size - 1);
            %put FirstObs = &FirstObs;
            %put LastObs = &LastObs;
            data inner_train outer_holdout;
                set &cvoutlib..Strain;
				if _n_ >= &FirstObs and _n_ <= &LastObs then output inner_train;
				else output outer_holdout;
            run;
			%NObsVars(inner_train);
			%if ^&nObs or ^&nVars %then %do;
				%let exiterror = 1;
	            %put ERROR: OUTER_LOOP SAMPLE TRAINING DATA CONTAINS 0 OBSERVATIONS.;
	            %goto lc_exit;
			%end;		
			%if %upcase(&testset_option) = HOLDOUT %then %do;
				%NObsVars(outer_holdout);
				%if ^&nObs or ^&nVars %then %do;
					%let exiterror = 1;
		            %put ERROR: OUTER_LOOP SAMPLE HOLDOUT DATA CONTAINS 0 OBSERVATIONS.;
		            %goto lc_exit;
				%end;
			%end;
			%if &trait_type = qualitative %then %do;
				proc freq data=inner_train noprint;
					tables &trait/out=tfreq;
				run;
				%NObsVars(work.tfreq);
				%if &nObs < 2 %then %do;
					%put OUTER_LOOP &outer_loop INNER_TRAIN HAS CONSTANT TRAIT;
					%goto outer_end;
				%end;
			%end;
		%end; 
		%else %if &outer_method = RANDOM %then %do;
			/*
			%if &max_inner_size = &_stnObs_ %then %do;
				data inner_train;
					set &cvoutlib..Strain;
				run;
			%end;
			%else %do;	
			*/
				/*
				%if &trait_type = quantitative %then %do;
				    * GET SMALLER INNER_TRAIN BY SUBSAMPLING CURRENT ONE.;
	                %random_sample(datapath=&outputpath,
							       dsname=Strain,
	                               sample_size=&max_inner_size,
								   seed=&randseed);
	                data inner_train outer_holdout;
	                    merge &cvoutlib..Strain pseudo;
	                    if pseudo eq 0 then output inner_train;
						else output outer_holdout;
	                    drop pseudo;
	                run;
	            %end;
	            %else %do;
	            */
	                %stratified_sample(datapath=&outputpath,
									   outputpath=,
									   dsname=Strain, 
	                           trait=&trait,
									   trait_type=&trait_type,
									   holdout_percent=((&_stnObs_-&max_inner_size)/&_stnObs_)*100,
									   groupname=&groupname,
									   train_dsname=inner_train,
									   test_dsname=outer_holdout,
									   seed=&randseed);
					/*
	            %end; 
	            */
	      /*
			%end;
			*/
		%end; 
		%else %do;
			%let exiterror = 1;
         	%put ERROR: OUTER_METHOD = &outer_method IS UNKNOWN OR REQUIRES N_OUTERLOOP = 1.;
         	%goto lc_exit;
		%end; 
	%end; * N_OUTERLOOP > 1;

	%do inner_loop=&inner_start %to &inner_end %by -1;
		%put;
		%put For outer loop &outer_loop of &outer_end, computing inner loop &inner_loop of &n_innerloop by -1;
		%put;

		*GENERATE MACRO RANDSEED FOR INNER LOOP THREAD=2, INDEX=&inner_loop;
		%seedget(i=%eval((&outer_loop-1) * &n_innerloop + &n_innerloop - &inner_loop + 1), 
				 seedds=&cvoutlib..seedseq,
			     thread=2);

		%if %sysfunc(exist(work.inner_train)) = 1 %then %do;
			%NObsVars(work.inner_train);
			%let _stnObs_ = &nObs;
			%if ^&_stnObs_ %then %do;
				%let exiterror = 1;
	            %put ERROR: inner_train SAMPLE TRAINING DATA CONTAINS 0 OBSERVATIONS.;
	            %goto lc_exit;
			%end;
		%end;
		%else %do;
			%let exiterror = 1;
			%put ERROR: inner_train SAMPLE TRAINING DATA DOES NOT EXIST.;
			%goto lc_exit;
		%end;

		data _null_;
            train_size = &min_inner_size * exp((&inner_loop-1)*&delta); 
            call symput("train_size", put(train_size, 12.0));
        run;

	    * GET SMALLER INNER_TRAIN BY SUBSAMPLING CURRENT ONE;
        %if &trait_type = quantitative %then %do;
            %random_sample(datapath=,
						   dsname=inner_train,
                           sample_size=&train_size,
						   seed=&randseed);
            data inner_train;
                merge inner_train pseudo;
                if pseudo eq 0 then output inner_train;
                drop pseudo;
            run;
        %end;
        %else %do;
            %stratified_sample(datapath=,
							   outputpath=,
							   dsname=inner_train, 
							   trait=&trait,
							   trait_type=&trait_type,
							   holdout_percent=((&_stnObs_-&train_size)/&_stnObs_)*100,
							   groupname=&groupname,
							   train_dsname=inner_train,
							   test_dsname=,
							   seed=&randseed);
        %end;            
		
		%NObsVars(work.inner_train);
		%let _stnObs_ =  &nObs;
     %put NEW _stnObs_ = &_stnObs_;

		* CREATE LEARNING CURVE MODELS PREDICTIONS,
		  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
		  CV_<MODELNAME>_PREDS AND CV_<MODELNAME>_VARS, 
		  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
		%if &testset_option = FIXTEST %then %do;
			%fitmodels(train_dsname=inner_train,
			           test_dsname=&cvoutlib..Fixtest);
		%end;
		%else %do;
			%fitmodels(train_dsname=inner_train,
			           test_dsname=outer_holdout);
		%end; 

		%do nm=1 %to &n_models;
			%let FM_Error=&&FM_Error&nm;
			%put FM_Error for model &nm = &FM_Error;
		   	* ERROR CHECKING;
			%if ^&FM_Error %then %do;
				%let mdlname=&&mdlname&nm;
				%let preds_exist = %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds));
				%put mdlname = &mdlname;
				%put preds_exist = &preds_exist;
				%if &preds_exist = 1 %then %do;
					%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds);
					%put nObs = &nObs;
					%put nVars = &nVars;
					%if &nObs and &nVars %then %do;
					    * APPEND MODEL PREDICTIONS;
						 data lt_m&nm._preds;
							length METHOD $64 INNER_LOOP OUTER_LOOP TRAIN_SIZE 8;
							set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
							METHOD = "&mdlname";
							INNER_LOOP=&inner_loop;
							OUTER_LOOP=&outer_loop;
							TRAIN_SIZE=&train_size;
						run;

						%if %sysfunc(exist(&cvoutlib..lt_m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 %then %do;
							%NObsVars(&cvoutlib..lt_m&nm._pcum_i&inner_loop._o&outer_loop);
							%let _lcpcumObs_ = &nObs;
							data &cvoutlib..lt_m&nm._pcum_i&inner_loop._o&outer_loop;
								set 
								%if &_lcpcumObs_ %then %do;
									&cvoutlib..lt_m&nm._pcum_i&inner_loop._o&outer_loop
								%end;
									lt_m&nm._preds;
							run;
						%end;
						%else %do;
							data &cvoutlib..lt_m&nm._pcum_i&inner_loop._o&outer_loop;
								set lt_m&nm._preds;
							run;
						%end;
					%end;
				%end;
				%else %do;
					%put Did not find cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
				%end;
			%end;
		%end; 
	%end; * END OF INNER_LOOP;
	%outer_end:;
%end; * END OF OUTER_LOOP;

%mend;

***********************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME: LC_ITERATION_LOOP   
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    08/11/07
**  PURPOSE: Learning curve iteration loop  
**  HISTORY: action                                       date     who
**           Created.                                     08/11/08 pyt 
**			 Added innerloop_option and testset_option	  11/03/09 pyt				 
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:     Number of models
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  N_OUTERLOOP:  Number of learning curve outer loop iteration
**  OUTER_START:  Outer loop starting index
**  OUTER_END: 	  Outer loop ending index
**  N_INNERLOOP:  Number of learning curve inner loop iteration
**  INNER_START:  Inner loop starting index
**  INNER_END: 	  Inner loop ending index
**  OUTER_METHOD: Method to prepare inner_train training data set 
**				  from outer loop iteration
**				  Methods are: BLOCK or RANDOM	
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  ITER_START:	Hold-out iteration starting index
**  ITER_END: Hold-out iteration ending index
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  KFOLD: K-fold cross validation 
**  LEAVE_L_OUT: Leave l observations out for holdout test set
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  DELTA: Parameter for determining training size within inner loop
**  MIN_INNER_SIZE:	Minimum training size for inner loop 
**  MAX_INNER_SIZE:	Maximum training size for inner loop
**  BLOCK_SIZE: Bloc size for BLOCK outer_method 
**  LCR_PREFIX: Prefix of LC prediction results data set name
**
**  NOTES:
**  Invokes lc_cv_iteration_loop or leave_l_out
************************************************************************;
%macro lc_iteration_loop(datapath=,
					     outputpath=,
					     dsname=,
						 innerloop_option=,
						 testset_option=,
						 model_names=,
					     n_models=,
					     trait=,
					     trait_type=,
						 n_outerloop=,
					     outer_start=,
					     outer_end=,
						 n_innerloop=,
					     inner_start=,
					     inner_end=,
					     outer_method=,
					     cv_method=,
					     cv_holdout_percent=,
					     iter_start=,
					     iter_end=,
					     cv_vars_outds=,
					     cvname=,
					     groupname=,
					     blockname=,
					     kfold=,
						 leave_l_out=,
					     prevalences=,
					     delta=,
					     min_inner_size=,
					     max_inner_size=,
					     block_size=,
						 lcr_prefix=
					     );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%put Entering lc_iteration_loop;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;
%if %length(&dsname) %then %do;
    %let __lcitlpname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto lc_exit;
%end;
%if %sysfunc(exist(&__lcitlpname)) = 1 %then %do;
	%NObsVars(&__lcitlpname);
	%let _tnObs_ = &nObs;
	%let _stnObs_ = &nObs;
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lcitlpname CONTAINS 0 OBSERVATIONS.;
		%goto lc_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lcitlpname DOES NOT EXIST.;
	%goto lc_exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;

%do outer_loop = &outer_start %to &outer_end;
	%put;
	%put Computing outer loop &outer_loop of &outer_end;
	%put;

	* GENERATE MACRO RANDSEED FOR THREAD=3, INDEX=&outer_loop;
	%seedget(i=&outer_loop, seedds=&cvoutlib..seedseq, thread=3);

	* PREPARE INNER_TRAIN: TRAINING DATA AVAILABLE IN INNER LOOP;
	/*
	%if &n_outerloop = 1 %then %do;
		data inner_train;
			set &cvoutlib..Strain;
		run;
	%end;
	%else %do;
	*/
		%if &outer_method = BLOCK %then %do;
		    %let FirstObs = %eval(&block_size * (&outer_loop - 1) + 1);
            %let LastObs  = %eval(&FirstObs + &block_size - 1);
            %put FirstObs = &FirstObs;
            %put LastObs = &LastObs;
            data inner_train outer_holdout;
                set &cvoutlib..Strain;
				if _n_ >= &FirstObs and _n_ <= &LastObs then output inner_train;
				else output outer_holdout;
            run;
			%NObsVars(inner_train);
			%if ^&nObs or ^&nVars %then %do;
				%let exiterror = 1;
	            %put ERROR: OUTER_LOOP SAMPLE TRAINING DATA CONTAINS 0 OBSERVATIONS.;
	            %goto lc_exit;
			%end;			
			%if %upcase(&testset_option) = HOLDOUT %then %do;
				%NObsVars(outer_holdout);
				%if ^&nObs or ^&nVars %then %do;
					%let exiterror = 1;
		            %put ERROR: OUTER_LOOP SAMPLE HOLDOUT DATA CONTAINS 0 OBSERVATIONS.;
		            %goto lc_exit;
				%end;
			%end;
			%if &trait_type = qualitative %then %do;
				proc freq data=inner_train noprint;
					tables &trait/out=tfreq;
				run;
				%NObsVars(work.tfreq);
				%if &nObs < 2 %then %do;
					%put OUTER_LOOP &outer_loop INNER_TRAIN HAS CONSTANT TRAIT;
					%goto outer_end;
				%end;
			%end;
		%end; 
		%else %if &outer_method = RANDOM %then %do;
			* GET SMALLER INNER_TRAIN BY SUBSAMPLING CURRENT ONE.;
            * SOMETIMES INNER_TRAIN EQUALS ENTIRE DATA SET. ;
         /*
         %if &max_inner_size = &_tnObs_ %then %do; 
		      data inner_train;  
					set &cvoutlib..Strain; 
				run;
         %end;
			%else %do;
			*/
					/*
	            %if &trait_type = quantitative %then %do;
	                %random_sample(datapath=&outputpath,
							       dsname=Strain,
	                               sample_size=&max_inner_size,
								   seed=&randseed);

	                data inner_train outer_holdout;
	                    merge &cvoutlib..Strain pseudo;
	                    if pseudo eq 0 then output inner_train;
						else output outer_holdout;
	                    drop pseudo;
	                run;
	            %end;
	            %else %do;
	            */
	                %stratified_sample(datapath=&outputpath,
									   outputpath=,
									   dsname=Strain, 
									   trait=&trait,
									   trait_type=&trait_type,
									   holdout_percent=((&_tnObs_-&max_inner_size)/&_tnObs_)*100,
									   groupname=&groupname,
									   train_dsname=inner_train,
									   test_dsname=outer_holdout,
									   seed=&randseed);
					/*
	            %end; 
	            */
	      /*
			%end;
			*/
		%end; 
		%else %do;
			%let exiterror = 1;
            %put ERROR: OUTER_METHOD = &outer_method IS UNKNOWN OR REQUIRES N_OUTERLOOP = 1.;
            %goto lc_exit;
		%end; 
	/*
	%end; * N_OUTERLOOP > 1;
	*/

	%do inner_loop=&inner_start %to &inner_end %by -1;
		%put;
		%put For outer loop &outer_loop of &outer_end, computing inner loop &inner_loop of &n_innerloop by -1;
		%put;

		*GENERATE MACRO RANDSEED FOR THREAD=2, INDEX=&inner_loop;
		%seedget(i=%eval((&outer_loop-1) * &n_innerloop + &n_innerloop - &inner_loop + 1), 
				 seedds=&cvoutlib..seedseq,
				 thread=2);
		
		%if %sysfunc(exist(work.inner_train)) = 1 %then %do;
			%NObsVars(work.inner_train);
			%let _stnObs_ = &nObs;
			%if ^&_stnObs_ %then %do;
				%let exiterror = 1;
	            %put ERROR: SAMPLE TRAINING DATA CONTAINS 0 OBSERVATIONS.;
	            %goto lc_exit;
			%end;
		%end;
		%else %do;
			%let exiterror = 1;
            %put ERROR: inner_train SAMPLE TRAINING DATA DOES NOT EXIST.;
            %goto lc_exit;
		%end;
	
		data _null_;
			set &cvoutlib.._metadat;
			if inner_loop=&inner_loop then do;
            	call symputx("train_size", put(train_size, 12.0));
			end;
		run;

	    * GET SMALLER INNER_TRAIN BY SUBSAMPLING THE CURRENT ONE;
        %if &trait_type = quantitative %then %do;
            %random_sample(datapath=,
					       dsname=inner_train,
                           sample_size=&train_size,
						   seed=&randseed);
            data inner_train;
                merge inner_train pseudo;
                if pseudo eq 0 then output inner_train;
                drop pseudo;
            run;
        %end;
        %else %do;
            %stratified_sample(datapath=,
							   outputpath=,
							   dsname=inner_train, 
							   trait=&trait,
							   trait_type=&trait_type,
							   holdout_percent=((&_stnObs_-&train_size)/&_stnObs_)*100,
							   groupname=&groupname,
							   train_dsname=inner_train,
							   test_dsname=,
								seed=&randseed);
        %end;            

		%NObsVars(work.inner_train);
		%let _stnObs_ =  &nObs;
        %put NEW _stnObs_ = &_stnObs_;

		%if &cv_method=LEAVEOUT %then %do;
			%lc_leave_l_out(datapath=,
				            outputpath=&outputpath,
					        dsname=inner_train,
							innerloop_option=&innerloop_option,
							testset_option=&testset_option,
							model_names=&model_names,
							n_models=&n_models,
							trait=&trait,
							trait_type=&trait_type,
							leave_l_out=&leave_l_out,
							train_size=&train_size,
							outer_loop=&outer_loop,
						    inner_loop=&inner_loop,
							llo_prefix=&lcr_prefix,
							prevalences=&prevalences
							);
		%end;
		%else %do;
			* CHECK FOR THE SIZE OF KFOLD, IF KFOLD > NOBS, REDUCE IT TO NOBS;
			%if &kfold > &_stnObs_ %then %do;
			    %let kfold = &_stnObs_;
			%end;
			
			%put inner_loop=&inner_loop;
			%put outer_loop=&outer_loop;
			%put outputpath=&outputpath;
			%put model_names=&model_names;
			%put n_models=&n_models;
			%put trait=&trait;
			%put trait_type=&trait_type;
			%put cv_method=&cv_method;
			%put train_size=&train_size;
			%put outer_loop=&outer_loop;
			%put inner_loop=&inner_loop;
			%put iter_start=&iter_start;
			%put iter_end=&iter_end;
			%put kfold=&kfold;
			%put cv_vars_outds=&cv_vars_outds;
			%put cvname=&cvname;
			%put cv_holdout_percent=&cv_holdout_percent;

			%lc_cv_iteration_loop(datapath=,
								 outputpath=&outputpath,
								 dsname=inner_train,
								 innerloop_option=&innerloop_option,
								 testset_option=&testset_option,
								 model_names=&model_names,
								 n_models=&n_models,
								 trait=&trait,
								 trait_type=&trait_type,
								 cv_method=&cv_method,
								 train_size=&train_size,
								 outer_loop=&outer_loop,
								 inner_loop=&inner_loop,
								 iter_start=&iter_start,
								 iter_end=&iter_end,
								 kfold=&kfold,
								 groupname=&groupname,
								 blockname=&blockname,
								 cv_vars_outds=&cv_vars_outds,
								 cvname=&cvname,
								 cv_holdout_percent=&cv_holdout_percent,
								 lcr_prefix=&lcr_prefix,
								 prevalences=&prevalences
								 );

		%end;
	%end; * END OF INNER_LOOP;
	%outer_end:;
%end; * END OF OUTER_LOOP;

%lc_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_PLOTS_DATA
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/13/07
**  PURPOSE: Generate learning curve estimates jmp plots data 
**  HISTORY: action                                        date     who
**           Created.                                      07/13/07 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:   Library path of output data sets 
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  NCLASSES:	Number of qualitative trait variable category
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  LC_PLOTS_OUTPUT: Learning curve estimates jmp plots output data name
**
**  NOTES:
************************************************************************;
%macro lc_plots_data(outputpath=,
					 trait_type=,
					 nclasses=,
					 models_comparison_output=,
					 lc_plots_output=);

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

%if %sysfunc(exist(&cvoutlib..&models_comparison_output)) = 1 %then %do;
	%NObsVars(&cvoutlib..&models_comparison_output);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &cvoutlib..&models_comparison_output CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&models_comparison_output DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(&cvoutlib.._metadat)) ne 1 %then %do;
	%let exiterror = 1;
	%put ERROR: REQUIRED &cvoutlib.._metadat DOES NOT EXIST.;
	%goto exit;
%end;

proc freq data=&cvoutlib.._metadat order=data noprint;
	tables METHOD*METHOD_I/out=fmethod(keep=METHOD METHOD_I);
	tables OUTER_LOOP/out=fouter(keep=OUTER_LOOP);
	tables INNER_LOOP/out=finner(keep=INNER_LOOP);
run;
proc sort data=fmethod;
	by METHOD_I;
run;
* RETRIEVE NUMBER OF MODELS, AND EACH MODEL NAMES;
%NObsVars(fmethod);
%let n_models = &nObs; 
%put n_models = &n_models;
%let _mid_ = %sysfunc(open(fmethod));
%let methodnum = %sysfunc(varnum(&_mid_, METHOD));
%do m=1 %to &n_models;
	%let rc = %sysfunc(fetchobs(&_mid_, &m));
	%let mdlname&m = %sysfunc(getvarc(&_mid_, &methodnum));
%end;
%let _mid_ = %sysfunc(close(&_mid_));

* RETRIEVE OUTERLOOP_START AND OUTERLOOP_END;
proc sort data=fouter;
	by OUTER_LOOP;
run;
%NObsVars(fouter);
%let n_outers = &nObs;
%let _oid_ = %sysfunc(open(fouter));
%let rc = %sysfunc(fetchobs(&_oid_, 1));
%let outerloop_start = %sysfunc(getvarn(&_oid_, 1));
%let rc = %sysfunc(fetchobs(&_oid_, &n_outers));
%let outerloop_end = %sysfunc(getvarn(&_oid_, 1)); 
%let _oid_ = %sysfunc(close(&_oid_));

* RETRIEVE N_INNERLOOP;
%NObsVars(finner);
%let n_innerloop = &nObs;

%do m=1 %to &n_models;
   %let mdllabel&m = %scan(&model_labels,&m,"|");
%end;

proc sort data=&cvoutlib..&models_comparison_output out=sorted;
	by train_size inner_loop;
run;

proc means data=sorted noprint;
	by train_size inner_loop;
	output out=fitmean;
run;
data &cvoutlib..&lc_plots_output;
	set fitmean;
	drop _TYPE_ _FREQ_ _STAT_ OUTER_LOOP;
	if _STAT_ = "MEAN" then output;
run;

%do inner_loop = 1 %to &n_innerloop;
	%do outer_loop = &outerloop_start %to &outerloop_end;
		data p;
			set &cvoutlib..&models_comparison_output;
				where INNER_LOOP = &inner_loop and 
					  OUTER_LOOP = &outer_loop;
				%do nm=1 %to &n_models;
					%let mdlname=&&mdlname&nm;
					%let mdllabel=&&mdllabel&nm;
					rename ASE_m&nm=ASE_m&nm._&outer_loop;
					rename RMSE_m&nm=RMSE_m&nm._&outer_loop;
					label ASE_m&nm="ASE &mdllabel &outer_loop";
					label RMSE_m&nm="RMSE &mdllabel &outer_loop";
					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						rename ACC_m&nm=ACC_m&nm._&outer_loop;
						label ACC_m&nm="ACC &mdllabel &outer_loop";
			    		%if &nclasses = 2 %then %do;
							rename AUC_m&nm=AUC_m&nm._&outer_loop;
							label AUC_m&nm="AUC &mdllabel &outer_loop";
						%end;
					%end;
					%else %do;
						rename HAC_m&nm=HAC_m&nm._&outer_loop;
						label HAC_m&nm="HARRELLC &mdllabel &outer_loop";
						rename CORR_m&nm=CORR_m&nm._&outer_loop;
						label CORR_m&nm="CORR &mdllabel &outer_loop";
					%end;
					keep ASE_m&nm 
					     RMSE_m&nm 
						 %if %upcase(&trait_type)=QUALITATIVE %then %do;
					     	ACC_m&nm 
	 			    		%if &nclasses = 2 %then %do;
							     AUC_m&nm
							%end;
						%end;
						%else %do;
							HAC_m&nm
							CORR_m&nm
						%end;
						;
				%end;
				keep TRAIN_SIZE;
		run;
		data &cvoutlib..&lc_plots_output;
			merge
				&cvoutlib..&lc_plots_output
				p;
			by train_size;
		run;
	%end;
%end;

* CALCULATE MIN AND MAX OF MODEL FIT STATISTICS FOR EACH TRAIN_SIZE;
proc means data=sorted noprint; 
	by train_size;
	var 
	%do nm=1 %to &n_models;
		ASE_m&nm
		RMSE_m&nm
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			ACC_m&nm
			%if &nclasses = 2 %then %do;
				AUC_m&nm	
			%end;
		%end;
		%else %do;
			HAC_m&nm
			CORR_m&nm
		%end;
	%end;
	;
	output out=minmax;
run;
data min;
	set minmax;
	if _STAT_ = "MIN" then output;
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
		%let mdllabel=&&mdllabel&nm;
			rename ASE_m&nm=ASE_m&nm._MIN;
			rename RMSE_m&nm=RMSE_m&nm._MIN;
			label ASE_m&nm="ASE &mdllabel MIN";
			label RMSE_m&nm="RMSE &mdllabel MIN";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				rename ACC_m&nm=ACC_m&nm._MIN;
				label ACC_m&nm="ACC &mdllabel MIN";
				%if &nclasses = 2 %then %do;
					rename AUC_m&nm=AUC_m&nm._MIN;
					label AUC_m&nm="AUC &mdllabel MIN";
				%end;
			%end;
			%else %do;
				rename HAC_m&nm=HAC_m&nm._MIN;
				label HAC_m&nm="HARRELLC &mdllabel MIN";
				rename CORR_m&nm=CORR_m&nm._MIN;
				label CORR_m&nm="CORR &mdllabel MIN";
			%end;
	%end;
	drop _TYPE_ _FREQ_ _STAT_;
run;
data max;
	set minmax;
	if _STAT_ = "MAX" then output;
	%do nm=1 %to &n_models;
		%let mdlname=&&mdlname&nm;
		%let mdllabel=&&mdllabel&nm;
			rename ASE_m&nm=ASE_m&nm._MAX;
			rename RMSE_m&nm=RMSE_m&nm._MAX;
			label ASE_m&nm="ASE &mdllabel MAX";
			label RMSE_m&nm="RMSE &mdllabel MAX";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				rename ACC_m&nm=ACC_m&nm._MAX;
				label ACC_m&nm="ACC &mdllabel MAX";
				%if &nclasses = 2 %then %do;
					rename AUC_m&nm=AUC_m&nm._MAX;
					label AUC_m&nm="AUC &mdllabel MAX";
				%end;
			%end;
			%else %do;
				rename HAC_m&nm=HAC_m&nm._MAX;
				label HAC_m&nm="HARRELLC &mdllabel MAX";
				rename CORR_m&nm=CORR_m&nm._MAX;
				label CORR_m&nm="CORR &mdllabel MAX";
			%end;
	%end;
	drop _TYPE_ _FREQ_ _STAT_;
run;
data &cvoutlib..&lc_plots_output;
	merge
		&cvoutlib..&lc_plots_output
		min 
		max;
	by train_size;
run;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LEARNING_CURVE
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    07/08/07
**  PURPOSE: Learning curve with fixed test data set 
**  HISTORY: action                                        date     who
**           Created.                                      07/08/07 pyt 
** 			 Added INNERLOOP_OPTION, TESTSET_OPTION		   10/29/09 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  TESTPATH:     Library path of test data set 
**  OUTPUTPATH:   Library path of output data sets 
**  PLOT_OUTPUTPATH: 
**  DSNAME:       Training data set name
**  TEST_DSNAME:  Test data set name specified by user
**  WHERE_CLAUSE: Where clause
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  NMIN: 		  Parameter used to determine the minumin size of training set 
**  NMAX:         Parameter used to determine the maximum size of training set 
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  N_OUTERLOOP:  Maximum  number of total LC outer loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  OUTER_METHOD: Method to prepare inner_train training data set 
**				  from outer loop iteration
**				  Methods are: BLOCK or RANDOM	
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_PERCENT: Percentange of observations in hold out
**  CV_HOLDOUT_ITERATIONS: Number of total random hold-out iterations
**  CV_SEED: Random number seed
**  CV_VARS_OUTDS: Output data set name contains CV variables
**                 &DSNAME._CV is default, or user defined name
**  CVNAME: K-fold random partition variable name
**          _CV_ is default, or user precreated CV var
**  KFOLD: K-fold cross validation 
**  BLOCKNAME: K-fold block partition variable name 
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  CVLC_ESTIMATES_OUTPUT: Cross validation learning curve fit estimates 
**  CVLC_PLOTS_OUTPUT: Cross validation learning curve plots data 
**  LC_FIXTEST_ESTIMATES_OUTPUT: Learning curve fix test fits estimates
**  LC_FIXTEST_PLOTS_OUTPUT: Learning curve fix test plots data 
**  BIAS_ESTIMATES_OUTPUT: Bias estimates output data 
**  BIAS_PLOTS_OUTPUT: Bias plots data
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Client grid share folder 
**  SERVERGSHARE: Server grid share folder
**
**  NOTES:
**  Currently outer_method is hard-coded for RANDOM outer_method 
**  N_OUTERLOOP is only required for BLOCK outer_method	if supported later
**
**  Define random seeds sequence:
**     AN OUTER LOOP OF &n_outerloop ITERATIONS. USES THREAD=3;
**     AN INNER LOOP OF &n_innerloop ITERATIONS. USES THREAD=2;
**     A CV HOLDOUT LOOP of &cv_holdout_iteration ITERATIONS. USES THREAD=1; 
**
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Fixed Test Set 
**
**  TO DO: Within each kfold, need to compare the difference between with and without 
**  applying random seed to the cvtest
**  
**  CVBOTH Bias estimates:
**  	difference of CVHOLDOUT - CVTEST  
************************************************************************;
%macro learning_curve(datapath=,
					  testpath=,
					  outputpath=,
					  plot_outputpath=,
					  dsname=,
					  test_dsname=,
					  where_clause=,
					  trait=,
					  trait_type=,
					  freq=,
					  model_names=,
					  model_labels=,
					  model_label_list=,
					  nmin=,
					  nmax=,
					  innerloop_option=,
					  testset_option=,
					  n_innerloop=,
					  n_outerloop=,
					  outerloop_start=,
					  outerloop_end=,
					  outer_method=,
					  cv_method=,
					  cv_holdout_percent=,
					  cv_holdout_iterations=,
					  cv_seed=,
					  cv_vars_outds=,
					  cvname=,
					  kfold=,
					  groupname=,
					  blockname=,
					  leave_l_out=,
					  cvlc_estimates_output=,
					  cvlc_plots_output=,
					  lc_fixtest_estimates_output=,
					  lc_fixtest_plots_output=,
					  bias_estimates_output=,
					  bias_plots_output=,
					  prevalences=proportional,
					  ngridnodes=,
					  clientgshare=,
					  servergshare=
					  );

* JMPG 5.0 DOES NOT SUPORT SURVIVAL MODELS IN LCMC;  
%if %upcase(&trait_type)=SURVIVAL %then %do; 
	%let exiterror = 1;
	%put ERROR: Proportional Hazards Regression (previous called SURVIVAL PREDICTIVE) MODELING METHOD IS NOT SUPPORTED IN THIS RELEASE.;
	%goto lc_exit;
%end;

* ASSIGN GLOBALS;
%global LS_HOME LCMC BPV;
%let LCMC = 1;

%put innerloop_option=&innerloop_option;
%put testset_option=&testset_option;

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&clientgshare) %then %do;
	%let cv_datapath = &datapath;
	%let datapath = &clientgshare;
	%if %length(&testpath) %then %do;
		%let cv_testpath = &testpath;
		%let testpath = &clientgshare;
	%end;
%end;
%if %length(&datapath) and "&datapath" ne "" %then %do;
   libname inlib "&datapath";
   %let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) > 0 %then %do; 
    %let __lcdsname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto lc_exit;
%end;

%if %sysfunc(exist(&__lcdsname)) = 1 %then %do;
	%NObsVars(&__lcdsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lcdsname CONTAINS 0 OBSERVATIONS.;
		%goto lc_exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lcdsname DOES NOT EXIST.;
	%goto lc_exit;
%end;

* KEEP TRAIT WITH NON-MISSING VALUES;  
%let _tid_ = %sysfunc(open(&__lcdsname));
%let _tObs_ = %sysfunc(attrn(&_tid_, NOBS));
%if &_tObs_ %then %do;
	%let traitnum=%sysfunc(varnum(&_tid_, &trait));
	%let traitype=%sysfunc(vartype(&_tid_, &traitnum));
%end;
%let _tid_ = %sysfunc(close(&_tid_));

* KEEP TRAIT WITH NON-MISSING VALUES AND APPLY SPECIFIED WHERE_CLAUSE;  
%let wc=%superq(where_clause); 
data Strain;
	set &__lcdsname;
	__id = _n_;
	%if &traitype = C %then %do; 
		if &trait ne " " 
		%if %length(&wc) > 0 %then %do;
			and %unquote(&wc)
		%end;
		then output;
	%end;
	%else %do;
		if &trait ne . 
		%if %length(&wc) > 0 %then %do;
			and %unquote(&wc)
		%end;
		then output;
	%end;
run;

%NObsVars(Strain);
%let _tnObs_ = &nObs;
%let _stnObs_ = &nObs;
%if ^&_tnObs_ %then %do;
	%let exiterror = 1;
    %if %length(&wc) > 0 %then %do;
		%put ERROR: &__lcdsname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do; 
		%put ERROR: &__lcdsname DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.; 
	%end;
	%goto lc_exit;
%end;

* RETRIEVE THE NUMBER OF CLASS OF TRAIT VARIABLE;
%if %upcase(&trait_type) = QUALITATIVE %then %do; 
    proc freq data=Strain noprint;
        tables &trait/out=ft;
    run;

	* RETRIEVE TRAIT LEVELS;
	%let _fid_ = %sysfunc(open(ft));
	%let _nclasses_ = %sysfunc(attrn(&_fid_, NOBS));
	%let _fid_ = %sysfunc(close(&_fid_));

    %do c=1 %to &_nclasses_;
        %global actualp&c;
    %end;

    %global mann auc;

%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&clientgshare) %then %do;
	%let cv_outputpath = &outputpath;
	%let outputpath = &clientgshare;
%end;

%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* DETERMINE WHETHER TEST PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&testpath) > 0 %then %do;
    libname testlib "&testpath";
   %let testlib = testlib;
%end;
%else %do;
    %let testlib = work;
%end;
* DETERMINE WHETHER TEST SET IS SPECIFIED.; 
%if %length(&test_dsname) > 0 and %upcase(&testset_option) = FIXTEST %then %do;
	%let testname = &testlib..&test_dsname;
	%if %sysfunc(exist(&testname)) = 1 %then %do;
		%NObsVars(&testname);
		%if &nObs and &nVars %then %do;
	   		%let wc=%superq(where_clause); 
			data Fixtest;
				set 
			    %if %length(&wc) > 0 %then %do;
			        &testname(where=(%unquote(&wc)))
			    %end;
			    %else %do;
			        &testname
			    %end;
			    ;
			run;
			%NObsVars(work.Fixtest);
			%let _ftnObs_ = &nObs;
			%if ^&_ftnObs_ %then %do;
			    %let exiterror = 1;
			    %if %length(&wc) > 0 %then %do;
					%put ERROR: &testname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
				%end;
				%else %do;
			    	%put ERROR: &testname FIXED TEST DATA DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
				%end;
			    %goto lc_exit;
			%end;

			* COPY FIXTEST TO OUTPUT LIBRARY FOR GRID NODES TO ACCESS;
			data &cvoutlib..Fixtest;
				set Fixtest;
			run;

		%end;
		%else %do;
			%let exiterror = 1;
			%put ERROR: &testname CONTAINS 0 OBSERVATIONS.;
			%goto lc_exit;
		%end;
	%end;
%end;

* RETRIEVE NUMBER OF MODELS;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);

* LC OUTER LOOP ITERATION PARAMETERS;
%if %length(&outerloop_start) = 0 %then %do;
	%let outerloop_start = 1;
%end;
%if %length(&outerloop_end) = 0 %then %do;
	%let exiterror = 1;
	%put NUMBER OF RANDOM ITERATIONS IS UNKNOWN.;
	%goto lc_exit;
%end;

 * DEFINE SEEDS FOR                                     ;
 *     AN OUTER LOOP OF &n_outerloop ITERATIONS. USES THREAD=3;
 *     AN INNER LOOP OF &n_innerloop ITERATIONS. USES THREAD=2;
 *     A CV HOLDOUT LOOP of &cv_holdout_iteration ITERATIONS. USES THREAD=1; 
%global randseed;
data _null_;
	n = max(&outerloop_end * &n_innerloop, &outerloop_end * &cv_holdout_iterations);
	call symput("nseed", n);
run;
%seedsequence(n=&nseed, 
		      seedds=&cvoutlib..seedseq,
			  nthreads=3, 
			  seqseed=&cv_seed);


* COPY STRAIN TO OUTPUT LIBRARY FOR GRID NODES TO ACCESS;
data &cvoutlib..Strain;
	set Strain;
run;

%put innerloop_option = &innerloop_option;
%put testset_option = &testset_option;

* START LEARNING CURVE MODEL COMPARISON;
%if %upcase(&innerloop_option) = STANDARD %then %do;

	%lc_create_metadat;
	%if &exiterror %then %goto lc_exit;
	
	* MODELS FIT ON FULL DATA(WITHOUT TEST SET), EVALUATE ON TEST SET;
	* TEST SET CAN BE EITHER OF THE FOLLOWING:
	*	 SUBSET (IN THE OUTER LOOP) OF INPUT DATA SET, OR
	*    EXTERNAL VALIDATION DATA SET;
	%lc_standard_estimates(datapath=&outputpath,
						  outputpath=&outputpath,
						  dsname=Strain,
						  testset_option=&testset_option,
						  trait=&trait,
						  trait_type=&trait_type,
						  freq=&freq,
						  model_names=&model_names,
						  n_models=&n_models,
						  nmin=&nmin,
						  nmax=&nmax,
						  n_innerloop=&n_innerloop,
						  n_outerloop=&n_outerloop,
						  outerloop_start=&outerloop_start,
						  outerloop_end=&outerloop_end,
						  outer_method=&outer_method,
						  groupname=&groupname,
						  models_comparison_output=&lc_fixtest_estimates_output,
						  prevalences=&prevalences,
						  ngridnodes=&ngridnodes,
						  clientgshare=&clientgshare,
						  servergshare=&servergshare
						  );

	proc datasets library=&cvoutlib nolist;
		delete Strain;
	quit;

	%if %length(&test_dsname) > 0 and %upcase(&testset_option) = FIXTEST %then %do;
		proc datasets library=&cvoutlib nolist;
			delete Fixtest;
		quit;
	%end;

	%if %length(&clientgshare) %then %do;
		%let datapath = &cv_datapath;
		%let outputpath = &cv_outputpath;
	%end;

	%global nclasses event;

	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		
		%if &ngridnodes > 0 %then %do;
			data &cvoutlib.._namedat;
				set &cvoutlib.._namedat1;
			run;
			proc datasets library=&cvoutlib nolist;
				delete
				%do node=1 %to &ngridnodes;
					_namedat&node
				%end;
				;
			quit;
		%end;

		%classes_event(namedat=&cvoutlib.._namedat);
		
	%end;
	%else %do;
		%let nclasses=;
		%let event=;
	%end;

	%put Starting merge_lc_results;

	* EXECUTE THE CODE BELOW AFTER ALL LEARNING CURVE OUTER AND INNER LOOPS ARE RUN;
	* COMPUTE LEARNING CURVE MODELS FIT STATISTICS: RMSE, ACC, AUC;
	%merge_lc_results(outputpath=&outputpath,
					  lcr_prefix=lt,
					  trait=&trait,
					  trait_type=&trait_type,
					  nclasses=&nclasses,
					  event=&event,
					  freq=&freq,
					  model_names=&model_names,
					  model_labels=&model_labels,
					  n_models=&n_models,
					  n_innerloop=&n_innerloop,
					  outerloop_start=&outerloop_start,
  					  outerloop_end=&outerloop_end,	
					  nmax=&nmax,
					  cv_method=&cv_method,
					  cv_holdout_iterations=0,
					  models_comparison_output=&lc_fixtest_estimates_output,
					  prevalences=&prevalences,
					  gshare=&clientgshare
					  );

	%lc_plots_data(outputpath=&outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&lc_fixtest_estimates_output,
				   lc_plots_output=&lc_fixtest_plots_output);

%end;
%else %if %upcase(&innerloop_option) = CVTEST or 
		  %upcase(&innerloop_option) = CVHOLDOUT %then %do;
	
		* CROSS VALIDATION ON SUBSETS, EVALUATE ON TEST SET
		* TEST SET CAN BE EITHER OF THE FOLLOWING:
		*	 SUBSET (IN THE OUTER LOOP) OF INPUT DATA SET, OR
		*    EXTERNAL VALIDATION DATA SET;
		%cv_learning_curve_estimates(datapath=&outputpath,
									 outputpath=&outputpath,
									 dsname=Strain,
									 innerloop_option=&innerloop_option,
									 testset_option=&testset_option,
									 trait=&trait,
									 trait_type=&trait_type,
									 freq=&freq,
									 model_names=&model_names,
									 n_models=&n_models,
									 nmin=&nmin,
									 nmax=&nmax,
									 n_innerloop=&n_innerloop,
									 n_outerloop=&n_outerloop,
									 outerloop_start=&outerloop_start,
									 outerloop_end=&outerloop_end,
									 outer_method=&outer_method,
									 cv_method=&cv_method,
									 cv_holdout_percent=&cv_holdout_percent,
									 cv_holdout_iterations=&cv_holdout_iterations,
									 cv_seed=&cv_seed,
									 cv_vars_outds=&cv_vars_outds,
									 cvname=&cvname,
									 kfold=&kfold,
									 groupname=&groupname,
									 blockname=&blockname,
									 leave_l_out=&leave_l_out,
						          lcr_prefix=cv,
									 models_comparison_output=&cvlc_estimates_output,
									 prevalences=&prevalences,
									 ngridnodes=&ngridnodes,
									 clientgshare=&clientgshare,
									 servergshare=&servergshare
									 );

		proc datasets library=&cvoutlib nolist;
			delete Strain;
		quit;

		%if %length(&test_dsname) > 0 and %upcase(&testset_option) = FIXTEST %then %do;
			proc datasets library=&cvoutlib nolist;
				delete Fixtest;
			quit;
		%end;

		%if %length(&clientgshare) %then %do;
			%let datapath = &cv_datapath;
			%let outputpath = &cv_outputpath;
		%end;

		%global nclasses event;
		%if %upcase(&trait_type) = QUALITATIVE %then %do;	
			
			%if &ngridnodes > 0 %then %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat1;
				run;
				proc datasets library=&cvoutlib nolist;
					delete
					%do node=1 %to &ngridnodes;
						_namedat&node
					%end;
					;
				quit;
			%end;

			%classes_event(namedat=&cvoutlib.._namedat);
			
		%end;
		%else %do;
			%let nclasses=;
			%let event=;
		%end;

		%put Starting merge_lc_results;

		* EXECUTE CODE BELOW AFTER ALL LEARNING CURVE OUTER AND INNER LOOPS ARE RUN;
		* COMPUTE LEARNING CURVE MODELS FIT STATISTICS: RMSE, ACC, AUC;
		%merge_lc_results(outputpath=&outputpath,
						  lcr_prefix=cv,
						  trait=&trait,
						  trait_type=&trait_type,
						  nclasses=&nclasses,
						  event=&event,
						  freq=&freq,
						  model_names=&model_names,
					  	  model_labels=&model_labels,
						  n_models=&n_models,
						  n_innerloop=&n_innerloop,
						  outerloop_start=&outerloop_start,
  						  outerloop_end=&outerloop_end,	
						  nmax=&nmax,
					      cv_method=&cv_method,
						  cv_holdout_iterations=&cv_holdout_iterations,
						  models_comparison_output=&cvlc_estimates_output,
						  prevalences=&prevalences,
						  gshare=&clientgshare
						  );

		%lc_plots_data(outputpath=&outputpath,
					   trait_type=&trait_type,
					   nclasses=&nclasses,	
					   models_comparison_output=&cvlc_estimates_output,
					   lc_plots_output=&cvlc_plots_output);


%end;
%else %if %upcase(&innerloop_option) = CVBOTH %then %do;
	
	%cv_learning_curve_estimates(datapath=&outputpath,
								 outputpath=&outputpath,
								 dsname=Strain,
								 innerloop_option=CVTEST,
								 testset_option=&testset_option,
								 trait=&trait,
								 trait_type=&trait_type,
								 freq=&freq,
								 model_names=&model_names,
								 n_models=&n_models,
								 nmin=&nmin,
								 nmax=&nmax,
								 n_innerloop=&n_innerloop,
								 n_outerloop=&n_outerloop,
								 outerloop_start=&outerloop_start,
								 outerloop_end=&outerloop_end,
								 outer_method=&outer_method,
								 cv_method=&cv_method,
								 cv_holdout_percent=&cv_holdout_percent,
								 cv_holdout_iterations=&cv_holdout_iterations,
								 cv_seed=&cv_seed,
								 cv_vars_outds=&cv_vars_outds,
								 cvname=&cvname,
								 kfold=&kfold,
								 leave_l_out=&leave_l_out,
					  			 lcr_prefix=lt,
							  	 models_comparison_output=&lc_fixtest_estimates_output,
								 prevalences=&prevalences,
								 ngridnodes=&ngridnodes,
								 clientgshare=&clientgshare,
								 servergshare=&servergshare
								 );

	%cv_learning_curve_estimates(datapath=&outputpath,
								 outputpath=&outputpath,
								 dsname=Strain,
								 innerloop_option=CVHOLDOUT,
								 testset_option=&testset_option,
								 trait=&trait,
								 trait_type=&trait_type,
								 freq=&freq,
								 model_names=&model_names,
								 n_models=&n_models,
								 nmin=&nmin,
								 nmax=&nmax,
								 n_innerloop=&n_innerloop,
								 n_outerloop=&n_outerloop,
								 outerloop_start=&outerloop_start,
								 outerloop_end=&outerloop_end,
								 outer_method=&outer_method,
								 cv_method=&cv_method,
								 cv_holdout_percent=&cv_holdout_percent,
								 cv_holdout_iterations=&cv_holdout_iterations,
								 cv_seed=&cv_seed,
								 cv_vars_outds=&cv_vars_outds,
								 cvname=&cvname,
								 kfold=&kfold,
								 leave_l_out=&leave_l_out,
								 lcr_prefix=cv,
								 models_comparison_output=&cvlc_estimates_output,
								 prevalences=&prevalences,
								 ngridnodes=&ngridnodes,
								 clientgshare=&clientgshare,
								 servergshare=&servergshare
								 );
 	
	proc datasets library=&cvoutlib nolist;
		delete Strain;
	quit;

	proc datasets library=&cvoutlib nolist;
		delete Fixtest;
	quit;

	%if %length(&clientgshare) %then %do;
		%let datapath = &cv_datapath;
		%let outputpath = &cv_outputpath;
	%end;

	%global nclasses event;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		
		%if &ngridnodes > 0 %then %do;
			data &cvoutlib.._namedat;
				set &cvoutlib.._namedat1;
			run;
			proc datasets library=&cvoutlib nolist;
				delete
				%do node=1 %to &ngridnodes;
					_namedat&node
				%end;
				;
			quit;
		%end;

		%classes_event(namedat=&cvoutlib.._namedat);

	%end;
	%else %do;
		%let nclasses=;
		%let event=;
	%end;

	* EXECUTE THE CODE BELOW AFTER ALL LEARNING CURVE OUTER AND INNER LOOPS ARE RUN;
	* COMPUTE LEARNING CURVE MODELS FIT STATISTICS: RMSE, ACC, AUC;
	%merge_lc_results(outputpath=&outputpath,
					  lcr_prefix=lt,
					  trait=&trait,
					  trait_type=&trait_type,
					  nclasses=&nclasses,
					  event=&event,
					  freq=&freq,
					  model_names=&model_names,
					  model_labels=&model_labels,
					  n_models=&n_models,
					  n_innerloop=&n_innerloop,
					  outerloop_start=&outerloop_start,
  					  outerloop_end=&outerloop_end,	
					  nmax=&nmax,
					  cv_method=&cv_method,
					  cv_holdout_iterations=&cv_holdout_iterations,
					  models_comparison_output=&lc_fixtest_estimates_output,
					  prevalences=&prevalences,
					  gshare=&clientgshare
					  );

	%lc_plots_data(outputpath=&outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&lc_fixtest_estimates_output,
				   lc_plots_output=&lc_fixtest_plots_output);

	* EXECUTE CODE BELOW AFTER ALL LEARNING CURVE OUTER AND INNER LOOPS ARE RUN;
	* COMPUTE LEARNING CURVE MODELS FIT STATISTICS: RMSE, ACC, AUC;
	%merge_lc_results(outputpath=&outputpath,
					  lcr_prefix=cv,
					  trait=&trait,
					  trait_type=&trait_type,
					  nclasses=&nclasses,
					  event=&event,
					  freq=&freq,
					  model_names=&model_names,
					  model_labels=&model_labels,
					  n_models=&n_models,
					  n_innerloop=&n_innerloop,
					  outerloop_start=&outerloop_start,
  					  outerloop_end=&outerloop_end,	
					  nmax=&nmax,
					  cv_method=&cv_method,
					  cv_holdout_iterations=&cv_holdout_iterations,
					  models_comparison_output=&cvlc_estimates_output,
					  prevalences=&prevalences,
					  gshare=&clientgshare
					  );

	%lc_plots_data(outputpath=&outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&cvlc_estimates_output,
				   lc_plots_output=&cvlc_plots_output);

	%bias_estimates(outputpath=&outputpath,
					model_names=&model_names,
					model_labels=&model_labels,
					n_models=&n_models,
					cvlc_estimates_output=&cvlc_estimates_output,
					lc_fixtest_estimates_output=&lc_fixtest_estimates_output,
					bias_estimates_output=&bias_estimates_output);

	%lc_bias_plots_data(outputpath=&outputpath,
						trait_type=&trait_type,
				   	    nclasses=&nclasses,
				   	    n_innerloop=&n_innerloop,
					    outerloop_start=&outerloop_start,
  					    outerloop_end=&outerloop_end,
					    models_comparison_output=&bias_estimates_output,
						bias_plots_output=&bias_plots_output);
	
%end;

%if &exiterror %then %do;
	%goto lc_exit;
%end;

* MAKE SURE THE OLD RESULTS ARE NOT PICKED UP BY THE PLOTTING CODE;
%if %upcase(&innerloop_option) ne CVBOTH %then %do;
	%let bias_estimates_output = does_not_exist;
	%let bias_plots_output = does_not_exist;
	%if %upcase(&innerloop_option) = CVTEST or 
		%upcase(&innerloop_option) = CVHOLDOUT %then %do;
		%let lc_fixtest_estimates_output = does_not_exist;
		%let lc_fixtest_plots_output = does_not_exist;
	%end;
	%if %upcase(&innerloop_option) = STANDARD %then %do;
		%let cvlc_estimates_output = does_not_exist;
		%let cvlc_plots_output = does_not_exist;
	%end; 
%end;

* 
* JSL PLOTTING CODE, LAUNCHED FROM ExitAndRetrieveLogLstFromJMP IN UtilityMacros.sas;
%if ^%length(&plot_outputpath) %then %let plot_outputpath = &outputpath;
filename jslfile "&outputpath.&lcPrefix._lcmc.jsl";
data _null_;
    file jslfile encoding="utf-8";
    put "//!";
    put "Names Default to Here(1);";
    put "// parameters for learning curve model comparison plots;";
    put "outputpath=""&plot_outputpath"";";
	put "cvlc_estimates_output=""&cvlc_estimates_output"";";
	put "cvlc_plots_output=""&cvlc_plots_output"";";
	put "lc_fixtest_estimates_output=""&lc_fixtest_estimates_output"";";
	put "lc_fixtest_plots_output=""&lc_fixtest_plots_output"";";
	put "bias_estimates_output=""&bias_estimates_output"";";
	put "bias_plots_output=""&bias_plots_output"";";
    put "cv_method = ""&cv_method"";";
	%if &cv_holdout_iterations > 0 %then %do;
    	put "cv_holdout_iterations = &cv_holdout_iterations;";
	%end;
	%else %do;
    	put "cv_holdout_iterations = .;";
	%end;
	%if &kfold > 0 %then %do;
    	put "kfold = &kfold;";
	%end;
	%else %do;
    	put "kfold = .;";
	%end;
	%if &leave_l_out > 0 %then %do;
    	put "leave_l_out = &leave_l_out;";
	%end;
	%else %do;
		put "leave_l_out = .;";
	%end;
    put "cvname = ""_CV_"";";
    put "model_names = ""&model_names"";";
    put "model_labels = ""&model_labels"";";
    put "trait = ""&trait"";";
    put "trait_type = ""&trait_type"";";
	%if %upcase(&trait_type)=QUALITATIVE %then %do;
		put "nclasses = &nclasses;";
		put "event = ""&event"";";
	%end;
	%else %do;
		put "nclasses = .;";
		put "event = "" "";";
	%end;
	put "freq = ""&freq"";";
	put "innerloop_option = ""&innerloop_option"";";
	put "testset_option = ""&testset_option"";";
	%if &outerloop_start > 0 %then %do;
		put "outerloop_start = &outerloop_start;";
	%end;
	%else %do;
		put "outerloop_start = .;";
	%end;
	%if &outerloop_end > 0 %then %do;
		put "outerloop_end = &outerloop_end;";
	%end;
	%else %do;
		put "outerloop_end = .;";
	%end;
	%if &n_innerloop > 0 %then %do;
		put "n_innerloop = &n_innerloop;";
	%end;
	%else %do;
		put "n_innerloop = .;";
	%end;
	put "prevalences = ""&prevalences"";";
	put "if(Host Is(Windows),";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
    put "include(""$LS_HOME\JSLFiles\Learning Curve Model Comparison.jsl"");";
run;

%lc_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LEAVE_L_OUT
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/022/07
**  PURPOSE: Leave l out cross validation method  
**  HISTORY: action                                       date     who
**           Created.                                     06/11/07 pyt 
**			 Grid computing 						      09/08/08 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Fixed Test Set 
**  MODEL_NAMES:  List of model names
**  N_MODELS: Number of modeling methods
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  OUTER_LOOP: Learning curve outer loop index
**  INNER_LOOP: Learning curve inner loop index 
**  LLO_PREFIX: Prefix of llo prediction results data set name
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Client shared grid folder
**  SERVERGSHARE: Server shared grid folder
**
**  NOTES:
**  Leave_l_out supports 1, 2, up to 3 observations out for holdout 
**
************************************************************************;
%macro lc_leave_l_out(datapath=,
					  outputpath=,
					  dsname=,
					  innerloop_option=,
					  testset_option=,
					  model_names=,
					  n_models=,
					  trait=,
					  trait_type=,
					  leave_l_out=,
					  train_size=,
					  outer_loop=,
					  inner_loop=,
					  llo_prefix=cv,
					  prevalences=proportional,
					  ngridnodes=,
	  				  clientgshare=,
	  				  servergshare=
	  				  );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lloname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__lloname)) = 1 %then %do;
	%NObsVars(&__lloname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lloname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lloname DOES NOT EXIST.;
	%goto exit;
%end;

%NObsVars(&__lloname);
%let __nobs = &nObs;
%put __nobs = &__nobs;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%do m=1 %to &n_models;
    %let mdlname&m = %scan(&model_names,&m," ");
%end;

* TODO CLEAN TEMPORARY RESULTS FROM THE PREVIOUS RUNS;

%if &ngridnodes > 0 %then %do;

	%gridsequence(process=LC_LEAVE_L_OUT,
				  cv_holdout_iteration_start=,
			 	  cv_holdout_iteration_end=,
			      innerloop_start=&inner_loop,
				  innerloop_end=&inner_loop,
			      outerloop_start=&outer_loop,
				  outerloop_end=&outer_loop,
				  leave_l_out=&leave_l_out,
				  nnodes=&ngridnodes,
		          gridds=&cvoutlib..lclogridseq_i&inner_loop._o&outerloop);
	
	%NObsVars(&cvoutlib..lclogridseq_i&inner_loop._o&outerloop);
	%let nnodes = &nObs;
	%let ngridnodes = &nObs;
	%do node=1 %to &nnodes;
		%global node_name&node;
	%end;
	%global rungridseq;
	%let rungridseq=&ngridnodes;

	%UpdateFitModels(nnodes=&ngridnodes, outputpath=&outputpath);

	%do %while (&rungridseq > 0);

		%let max_tries = 100;
		%let waitlist = ;
		%let ng = 0;

		* CHECK GRID NODES RUN_STATUS IN logridseq.sas7bdat;
		* FOR ANY RUN_STATUS ne 0, RSUBMIT THE CODE TO GENERATE INTER-MEDIA RESULTS;
		%do gridseq=1 %to &ngridnodes;
			%let _gseq_ = %sysfunc(open(&cvoutlib..lclogridseq_i&inner_loop._o&outer_loop));
			%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
			%let status_num = %sysfunc(varnum(&_gseq_, run_status));
			%let gndone = %sysfunc(getvarn(&_gseq_, &status_num));	
			%let _gseq_ = %sysfunc(close(&_gseq_));
			%if &gndone=0 %then %do;
	            %put REMOTE SUBMIT COMPLETE ON GNODE&gridseq;
		    %end;
 			%else %if &gndone = . or &gndone = 1 %then %do;
				* KEEP THE RERUN RECORD IN gridseq data set;
				%if &gndone = 1 %then %do;
		        	%put REMOTE SUBMIT FAILED TO EXECUTE ON GNODE&gridseq;
				%end;
				%let ng=%eval(&ng+1);
				%let node=&ng;
				%goto run_gridseq;
		    %end;
		    %else %if &gndone = 2 %then %do;
		        %put REMOTE SUBMIT IS STILL IN PROGRESS ON GNODE&gridseq;
			%end;

			%run_gridseq:
				* RETRIEVE LEAVE_L_OUT ITERATIONS FROM logridseq.sas7bdat OF EACH NODE;  
				%let _gseq_ = %sysfunc(open(&cvoutlib..logridseq_i&inner_loop._o&outer_loop));
				%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
				%let istart_num = %sysfunc(varnum(&_gseq_, i_start));                                                                                
				%let i_start = %sysfunc(getvarn(&_gseq_, &istart_num));                                                                             
				%let iend_num = %sysfunc(varnum(&_gseq_, i_end));                                                                                    
				%let i_end = %sysfunc(getvarn(&_gseq_, &iend_num));  
				%if &leave_l_out > 1 %then %do;
				   %let jstart_num = %sysfunc(varnum(&_gseq_, j_start));                                                                                
				   %let j_start =  %sysfunc(getvarn(&_gseq_, &jstart_num));                                                                             
				   %let jend_num = %sysfunc(varnum(&_gseq_, j_end));                                                                                    
				   %let j_end =  %sysfunc(getvarn(&_gseq_, &jend_num));                                                                                 
				%end;
				%else %do;
			 	   %let j_start = 0;
			  	   %let j_end = 0;
			  	%end;
			  	%if &leave_l_out > 2 %then %do;
				   %let kstart_num = %sysfunc(varnum(&_gseq_, k_start));                                                                                
				   %let k_start =  %sysfunc(getvarn(&_gseq_, &kstart_num));                                                                             
				   %let kend_num = %sysfunc(varnum(&_gseq_, k_end));                                                                                    
				   %let k_end =  %sysfunc(getvarn(&_gseq_, &kend_num));     
				%end;
			   	%else %do;
			  	   %let k_start = 0;
			  	   %let k_end = 0;
			  	%end;
			    %let _gseq_ = %sysfunc(close(&_gseq_));          

				data &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
					set &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
					if grid_node = &gridseq then do;
						run_status=2;
					end;
				run;

				%let waitlist=&waitlist gnode&node;
				%let tries = 1;
				%let finished = 0;
				%do %while(^&finished);
					%put Sign on attempt &tries of &max_tries to grid node &node....;
					signon gnode&node;
					%let node_name = ;
					%let node_name&gridseq = %sysfunc(grdsvc_getname(gnode&node));
					%let node_name = &&node_name&gridseq;
					%if %length(&node_name) %then %do;
						%put SUCCESSFUL SIGN ON TO GNODE&node = &node_name;
						%if &leave_l_out = 1 %then %do;
							%put RUNNING I_START &i_start I_END &i_end ON GNODE&node = &node_name;
						%end;
						%if &leave_l_out = 2 %then %do;
							%put RUNNING I_START &i_start I_END &i_end, J_START &j_start J_END &j_end ON GNODE&node = &node_name;
						%end;
						%if &leave_l_out = 3 %then %do;
							%put RUNNING I_START &i_start I_END &i_end, J_START &j_start J_END &j_end,
							     K_START &k_start K_END &k_end ON GNODE&node = &node_name;
						%end;
						%let finished = 1;
						* UPDATE NODE_NAME IN logridseq.sas7bdat;
						data &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
							set &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
							if grid_node = &gridseq then do;
								node_name="&node_name";
							end;
						run;		
					%end;
					%else %do;
						%let tries = %eval(&tries+1);
						%if &tries > &max_tries %then %do;
							%put ERROR: GRID SIGN ON TO GNODE&gridseq FAILED AFTER &max_tries TRIES.;
							%let exiterror = 1;
							%let finished = 1;
							signoff _all_;
							%goto exit;
						%end;
					%end;
				%end;

				%if %sysfunc(sysprod(prodnum007)) ne 0 %then %do; 
					*S1313431: SAS/CONNECT be licensed*;
					%macro ConnectVars04;
						* SYSLPUT CREATES A NEW MACRO VARIABLE OR MODIFIES THE VALUE OF 
						  AN EXISTING MACRO VARIABLE ON A REMOTE HOST OR SERVER;
					  	%syslput GSHARE=&servergshare;
					  	%syslput LS_HOME=&LS_HOME;
					  	%syslput GHOME=&servergshare.gnode&gridseq.%str(/);
					  	%syslput USER_HOME=&servergshare.gnode&gridseq.%str(/);
					 	%syslput ProcessName=CVMC_LLO&gridseq;  
						%syslput datalib=datalib;
						%syslput cvoutlib=cvoutlib;
						%syslput datapath=&datapath;
						%syslput outpath=&servergshare;
						%syslput outputpath=&servergshare;
						%syslput dsname=&dsname;
						%syslput innerloop_option=&innerloop_option;
						%syslput testset_option=&testset_option;
						%syslput model_names=&model_names;
						%syslput n_models=&n_models;
						%do i=1 %to &n_models;
							%syslput mdlname&i = &&mdlname&i;
							%syslput FM_Error&i = 0;
						%end;
						%syslput trait=&trait;
						%syslput trait_type=&trait_type;
						%syslput __nobs=&__nobs;
						%syslput leave_l_out=&leave_l_out;
						%syslput train_size=&train_size;
						%syslput outer_loop=&outer_loop;
						%syslput inner_loop=&inner_loop;
						%syslput i_start=&i_start;
						%syslput i_end=&i_end;
						%syslput __i=;
						%syslput __j=;
					  	%syslput j_start=&j_start;
					  	%syslput j_end=&j_end;	
					  	%syslput __k=;
					  	%syslput j_start=&j_start;
					  	%syslput j_end=&j_end;	
					  	%syslput k_start=&k_start;
					  	%syslput k_end=&k_end;
						%syslput llo_prefix=&llo_prefix;
						%syslput prevalences=&prevalences;
						%syslput node_name=&node_name;
					  	%syslput node=&node;
						%syslput gridseq=&gridseq;
						%syslput gndone&gridseq = 2;
					%mend;
				%end;
				%else %do;
					%macro ConnectVars04;
						%put EMPTY.;	
					%mend;				
				%end;
				%ConnectVars04;
				

				* RSUBMIT BLOCK OF WORK TO BE SUBMITTED TO GRID NODE;
				rsubmit cwait=no persist=no cmacvar=gndone&gridseq;

					%macro ITERATION_LOOP;
					
					%put rsubmit for &ProcessName;
					%put gshare = &gshare;
			  		%put ghome = &GHOME;
			  		%put node number = &node;
					%put node name = &node_name;
					
					* MACROS DEFINED IN GRID SHARE FOLDER;
					filename gmacro "&GHOME.MacroLib";  
			  		%if ^%sysfunc(fexist(gmacro)) %then %do;
			  		    %put ERROR: Unable to find grid share folder &GHOME.MacroLib. ;
						 %put Make sure the server version of your grid share path is correct using ;
						 %put File > Configure Life Sciences Settings.;
			  		    %let exiterror = 1;
			  		    %goto iter_exit;
			  		%end;

					%inc gmacro(
						UtilityMacros.sas,
						CrossValidationModelComparisonMacros.sas,
						FitModels.sas,
						BinaryResponseEffectSelectionMacros.sas,
						SeedMacros.sas
					);
					filename gmacro clear;
					
					%put i_start=&i_start;
					%put i_end=&i_end;
					%put j_start=&j_start;
					%put j_end=&j_end;
					%put k_start=&k_start;
					%put k_end=&k_end;

					%lc_leave_l_out_iteration_loop(datapath=&datapath,
												   outputpath=&outputpath,
												   dsname=&dsname,
												   innerloop_option=&innerloop_option,
												   testset_option=&testset_option,
												   model_names=&model_names,
												   n_models=&n_models,
												   trait=&trait,
												   trait_type=&trait_type,
												   __nobs=&__nobs,
												   leave_l_out=&leave_l_out,
												   train_size=&train_size,
												   outer_loop=&outer_loop,
												   inner_loop=&inner_loop,
												   i_start=&i_start,
												   i_end=&i_end,
												   j_start=&j_start,
												   j_end=&j_end,
												   k_start=&k_start,
												   k_end=&k_end,
												   llo_prefix=&llo_prefix,
												   prevalences=&prevalences
												   );
					
					*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
					%if %upcase(&trait_type) = QUALITATIVE %then %do;	
						%do c=1 %to &nclasses;
							%let class=&&class&c;
							%let p_prob=&&p_prob&c;
							data _namedat;
								length CLASS P_PROB $32 ;
							 	CLASS="&class";
								P_PROB="&p_prob";
								EVENT="&event";
							run;
							%if &c=1 %then %do;
								data &cvoutlib.._namedat&gridseq;
									set _namedat;
								run;
							%end;
							%else %do;
								data &cvoutlib.._namedat&gridseq;
									set &cvoutlib.._namedat&gridseq
										_namedat;
								run;
							%end;
						%end;
					%end;

					%iter_exit:;
					%mend;

					%ITERATION_LOOP;

			  	endrsubmit;
			;

		%end;

		* WAIT FOR ALL RSUBMITS TO FINISH;
		%put waitlist = &waitlist;
		waitfor _ALL_ &waitlist;

		* STOP SAS RUNNING ON THE GRID NODES;                                 
		* signoff _all_;

		* UPDATE GRID NODES RUN_STATUS IN logridseq.sas7bdat;
		data &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
			set &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
			%do node=1 %to &ngridnodes;
				%let gndone=&&gndone&node;
				if grid_node = &node and run_status ne 0 then do;
					run_status=&gndone;
				end;
			%end;
		run;

		data rungrid;
			set &cvoutlib..logridseq_i&inner_loop._o&outer_loop;
			if run_status ne 0 then output;
		run;

		%NObsVars(rungrid);
		%let rungridseq=&nObs;

	%end;	

%end;
%else %do;
	
	%put CV_METHOD: LEAVE &leave_l_out OUT ;

	%if &leave_l_out=3 %then %do;
		%do __i=1 %to &__nobs-2;
		  %do __j=&__i+1%to &__nobs-1;
			%do __k=&__j+1 %to &__nobs;
				%lc_leave_l_out_estimates(datapath=&datapath,
										  outputpath=&outputpath,
										  dsname=&dsname,
										  innerloop_option=&innerloop_option,
										  testset_option=&testset_option,
										  model_names=&model_names,
										  n_models=&n_models,
										  trait=&trait,
										  trait_type=&trait_type,
										  leave_l_out=&leave_l_out,
										  train_size=&train_size,
										  outer_loop=&outer_loop,
										  inner_loop=&inner_loop,
										  __i=&__i,
										  __j=&__j,
										  __k=&__k,
										  llo_prefix=&llo_prefix,
										  prevalences=&prevalences);
			%end; * END OF K LOOP;
		  %end; * END OF J LOOP;
		%end; * END OF I LOOP;
	%end;
	%else %if &leave_l_out=2 %then %do;
		%do __i=1 %to &__nobs-1;
			%do __j=&__i+1%to &__nobs;
				%lc_leave_l_out_estimates(datapath=&datapath,
										  outputpath=&outputpath,
										  dsname=&dsname,
										  innerloop_option=&innerloop_option,
										  testset_option=&testset_option,
										  model_names=&model_names,
										  n_models=&n_models,
										  trait=&trait,
										  trait_type=&trait_type,
										  leave_l_out=&leave_l_out,
										  train_size=&train_size,
										  outer_loop=&outer_loop,
										  inner_loop=&inner_loop,
										  __i=&__i,
										  __j=&__j,
										  __k=,
										  llo_prefix=&llo_prefix,
										  prevalences=&prevalences);
			%end; * END OF J LOOP;
		%end; * END OF I LOOP;
	%end;
	%else %if &leave_l_out=1 %then %do;
		%do __i=1 %to &__nobs;
			%lc_leave_l_out_estimates(datapath=&datapath,
									  outputpath=&outputpath,
									  dsname=&dsname,
									  innerloop_option=&innerloop_option,
									  testset_option=&testset_option,
									  model_names=&model_names,
									  n_models=&n_models,
									  trait=&trait,
									  trait_type=&trait_type,
									  leave_l_out=&leave_l_out,
									  train_size=&train_size,
									  outer_loop=&outer_loop,
									  inner_loop=&inner_loop,
									  __i=&__i,
									  __j=,
									  __k=,
									  llo_prefix=&llo_prefix,
									  prevalences=&prevalences);
		%end; * END OF I LOOP;
	%end;

	*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%do c=1 %to &nclasses;
			%let class=&&class&c;
			%let p_prob=&&p_prob&c;
			data _namedat;
				length CLASS P_PROB $32 ;
			 	CLASS="&class";
				P_PROB="&p_prob";
				EVENT="&event";
			run;
			%if &c=1 %then %do;
				data &cvoutlib.._namedat;
					set _namedat;
				run;
			%end;
			%else %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat
						_namedat;
				run;
			%end;
		%end;
	%end;

%end;
 
%exit:
;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LEAVE_L_OUT
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/022/07
**  PURPOSE: Leave l out cross validation method  
**  HISTORY: action                                       date     who
**           Created.                                     06/11/07 pyt 
**			 Grid computing 						      09/08/08 pyt
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  MODEL_NAMES:  List of model names
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  LLO_PREFIX: Prefix of llo prediction results data set name
** 	PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**  CV_CUSTOMCOSTS: Custom cost function
**  CV_ALOSSPROP: Asymmetric loss evaluation proportion
**  NGRIDNODES: Number of nodes	available on the grid
**  CLIENTGSHARE: Client shared grid folder
**  SERVERGSHARE: Server shared grid folder
**
**  NOTES:
**  Leave_l_out supports 1, 2, up to 3 observations out for holdout 
**
************************************************************************;
%macro leave_l_out(datapath=,
				   outputpath=,
				   dsname=,
				   model_names=,
				   trait=,
				   trait_type=,
				   leave_l_out=,
				   llo_prefix=cv,
				   prevalences=proportional,
				   cv_custompriors=,
				   cv_customcosts=,
				   cv_alossprop=,
				   ngridnodes=,
  				   clientgshare=,
  				   servergshare=
  				   );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lloname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__lloname)) = 1 %then %do;
	%NObsVars(&__lloname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lloname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lloname DOES NOT EXIST.;
	%goto exit;
%end;

%NObsVars(&__lloname);
%let __nobs = &nObs;
%put __nobs = &__nobs;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%let i=1;
%do %while(%length(%scan(&model_names,&i," ")));
    %let i = %eval(&i+1);
%end;
%let n_models = %eval(&i-1);
%do i=1 %to &n_models;
    %global mdlname&i;
%end;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
%end;

* TODO CLEAN TEMPORARY RESULTS FROM THE PREVIOUS RUNS;

%if &ngridnodes > 0 %then %do;

	%gridsequence(process=LEAVE_L_OUT,
				  cv_holdout_iteration_start=,
			 	  cv_holdout_iteration_end=,
			      innerloop_start=,
				  innerloop_end=,
			      outerloop_start=,
				  outerloop_end=,
				  leave_l_out=&leave_l_out,
				  nnodes=&ngridnodes,
		          gridds=&cvoutlib..logridseq);
	
	%NObsVars(&cvoutlib..logridseq);
	%let nnodes = &nObs;
	%let ngridnodes = &nObs;
	%do node=1 %to &nnodes;
		%global node_name&node;
	%end;
	%global rungridseq;
	%let rungridseq=&ngridnodes;

	%UpdateFitModels(nnodes=&ngridnodes, outputpath=&outputpath);

	%do %while (&rungridseq > 0);

		%let max_tries = 100;
		%let waitlist = ;
		%let ng = 0;

		* CHECK GRID NODES RUN_STATUS IN logridseq.sas7bdat;
		* FOR ANY RUN_STATUS ne 0, RSUBMIT THE CODE TO GENERATE INTER-MEDIA RESULTS;
		%do gridseq=1 %to &ngridnodes;
			%let _gseq_ = %sysfunc(open(&cvoutlib..logridseq));
			%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
			%let status_num = %sysfunc(varnum(&_gseq_, run_status));
			%let gndone = %sysfunc(getvarn(&_gseq_, &status_num));	
			%let _gseq_ = %sysfunc(close(&_gseq_));
			%if &gndone=0 %then %do;
	            %put REMOTE SUBMIT COMPLETE ON GNODE&gridseq;
		    %end;
 			%else %if &gndone = . or &gndone = 1 %then %do;
				* KEEP THE RERUN RECORD IN gridseq data set;
				%if &gndone = 1 %then %do;
		        	%put REMOTE SUBMIT FAILED TO EXECUTE ON GNODE&gridseq;
				%end;
				%let ng=%eval(&ng+1);
				%let node=&ng;
				%goto run_gridseq;
		    %end;
		    %else %if &gndone = 2 %then %do;
		        %put REMOTE SUBMIT IS STILL IN PROGRESS ON GNODE&gridseq;
			%end;

			%run_gridseq:
				* RETRIEVE LEAVE_L_OUT ITERATIONS FROM logridseq.sas7bdat OF EACH NODE;  
				%let _gseq_ = %sysfunc(open(&cvoutlib..logridseq));
				%let rc = %sysfunc(fetchobs(&_gseq_, &gridseq));
				%let istart_num = %sysfunc(varnum(&_gseq_, i_start));                                                                                
				%let i_start = %sysfunc(getvarn(&_gseq_, &istart_num));                                                                             
				%let iend_num = %sysfunc(varnum(&_gseq_, i_end));                                                                                    
				%let i_end = %sysfunc(getvarn(&_gseq_, &iend_num));  
				%if &leave_l_out > 1 %then %do;
				   %let jstart_num = %sysfunc(varnum(&_gseq_, j_start));                                                                                
				   %let j_start =  %sysfunc(getvarn(&_gseq_, &jstart_num));                                                                             
				   %let jend_num = %sysfunc(varnum(&_gseq_, j_end));                                                                                    
				   %let j_end =  %sysfunc(getvarn(&_gseq_, &jend_num));                                                                                 
				%end;
				%else %do;
			 	   %let j_start = 0;
			  	   %let j_end = 0;
			  	%end;
			  	%if &leave_l_out > 2 %then %do;
				   %let kstart_num = %sysfunc(varnum(&_gseq_, k_start));                                                                                
				   %let k_start =  %sysfunc(getvarn(&_gseq_, &kstart_num));                                                                             
				   %let kend_num = %sysfunc(varnum(&_gseq_, k_end));                                                                                    
				   %let k_end =  %sysfunc(getvarn(&_gseq_, &kend_num));     
				%end;
			   	%else %do;
			  	   %let k_start = 0;
			  	   %let k_end = 0;
			  	%end;
			    %let _gseq_ = %sysfunc(close(&_gseq_));          

				data &cvoutlib..logridseq;
					set &cvoutlib..logridseq;
					if grid_node = &gridseq then do;
						run_status=2;
					end;
				run;

				%let waitlist=&waitlist gnode&node;
				%let tries = 1;
				%let finished = 0;
				%do %while(^&finished);
					%put Sign on attempt &tries of &max_tries to grid node &node....;
					signon gnode&node;
					%let node_name = ;
					%let node_name&gridseq = %sysfunc(grdsvc_getname(gnode&node));
					%let node_name = &&node_name&gridseq;
					%if %length(&node_name) %then %do;
						%put SUCCESSFUL SIGN ON TO GNODE&node = &node_name;
						%if &leave_l_out = 1 %then %do;
							%put RUNNING I_START &i_start I_END &i_end ON GNODE&node = &node_name;
						%end;
						%if &leave_l_out = 2 %then %do;
							%put RUNNING I_START &i_start I_END &i_end, J_START &j_start J_END &j_end;
							%put ON GNODE&node = &node_name;
						%end;
						%if &leave_l_out = 3 %then %do;
							%put RUNNING I_START &i_start I_END &i_end, J_START &j_start J_END &j_end,
							     K_START &k_start K_END &k_end ON GNODE&node = &node_name;
						%end;
						%let finished = 1;
						* UPDATE NODE_NAME IN logridseq.sas7bdat;
						data &cvoutlib..logridseq;
							set &cvoutlib..logridseq;
							if grid_node = &gridseq then do;
								node_name="&node_name";
							end;
						run;		
					%end;
					%else %do;
						%let tries = %eval(&tries+1);
						%if &tries > &max_tries %then %do;
							%put ERROR: GRID SIGN ON TO GNODE&gridseq FAILED AFTER &max_tries TRIES.;
							%let exiterror = 1;
							%let finished = 1;
							signoff _all_;
							%goto exit;
						%end;
					%end;
				%end;

				%if %sysfunc(sysprod(prodnum007)) ne 0 %then %do; 
					*S1313431: SAS/CONNECT be licensed*;
					%macro ConnectVars05;
						* SYSLPUT CREATES A NEW MACRO VARIABLE OR MODIFIES THE VALUE OF 
						  AN EXISTING MACRO VARIABLE ON A REMOTE HOST OR SERVER;
					  	%syslput GSHARE=&servergshare;
					  	%syslput LS_HOME=&LS_HOME;
					  	%syslput GHOME=&servergshare.gnode&gridseq.%str(/);
					  	%syslput USER_HOME=&servergshare.gnode&gridseq.%str(/);
					 	%syslput ProcessName=CVMC_LLO&gridseq;  
						%syslput datalib=datalib;
						%syslput cvoutlib=cvoutlib;
						%syslput datapath=&datapath;
						%syslput outpath=&servergshare;
						%syslput outputpath=&servergshare;
						%syslput dsname=&dsname;
						%syslput model_names=&model_names;
						%syslput n_models=&n_models;
						%do i=1 %to &n_models;
							%syslput mdlname&i = &&mdlname&i;
							%syslput FM_Error&i = 0;
						%end;
						%syslput trait=&trait;
						%syslput trait_type=&trait_type;
						%syslput __nobs=&__nobs;
						%syslput leave_l_out=&leave_l_out;
						%syslput i_start=&i_start;
						%syslput i_end=&i_end;
						%syslput __i=;
						%syslput __j=;
					  	%syslput j_start=&j_start;
					  	%syslput j_end=&j_end;	
					  	%syslput __k=;
					  	%syslput j_start=&j_start;
					  	%syslput j_end=&j_end;	
					  	%syslput k_start=&k_start;
					  	%syslput k_end=&k_end;
						%syslput llo_prefix=&llo_prefix;
						%syslput prevalences=&prevalences;
						%syslput cv_custompriors=%nrbquote(&cv_custompriors);
						%syslput cv_customcosts=%nrbquote(&cv_customcosts);
						%syslput cv_alossprop=&cv_alossprop;
						%syslput node_name=&node_name;
					  	%syslput node=&node;
						%syslput gridseq=&gridseq;
						%syslput gndone&gridseq = 2;
					%mend;
				%end;
				%else %do;
					%macro ConnectVars05;
						%put EMPTY.;	
					%mend;
				%end;
				%ConnectVars05;

				* RSUBMIT BLOCK OF WORK TO BE SUBMITTED TO GRID NODE;
				rsubmit cwait=no persist=no cmacvar=gndone&gridseq;

					%macro ITERATION_LOOP;
					
					%put rsubmit for &ProcessName;
					%put gshare = &gshare;
			  		%put ghome = &GHOME;
			  		%put node number = &node;
					%put node name = &node_name;
					
					* MACROS DEFINED IN GRID SHARE FOLDER;
					filename gmacro "&GHOME.MacroLib";  
			  		%if ^%sysfunc(fexist(gmacro)) %then %do;
			  		    %put ERROR: Unable to find grid share folder &GHOME.MacroLib. ;
						 %put Make sure the server version of your grid share path is correct using ;
						 %put File > Configure Life Sciences Settings.;
			  		    %let exiterror = 1;
			  		    %goto iter_exit;
			  		%end;

					%inc gmacro(
						UtilityMacros.sas,
						CrossValidationModelComparisonMacros.sas,
						FitModels.sas,
						BinaryResponseEffectSelectionMacros.sas,
						SeedMacros.sas
					);
					filename gmacro clear;
					
					%put i_start=&i_start;
					%put i_end=&i_end;
					%put j_start=&j_start;
					%put j_end=&j_end;
					%put k_start=&k_start;
					%put k_end=&k_end;

					%leave_l_out_iteration_loop(datapath=&datapath,
											   outputpath=&outputpath,
											   dsname=&dsname,
											   model_names=&model_names,
											   n_models=&n_models,
											   trait=&trait,
											   trait_type=&trait_type,
											   __nobs=&__nobs,
											   leave_l_out=&leave_l_out,
											   i_start=&i_start,
											   i_end=&i_end,
											   j_start=&j_start,
											   j_end=&j_end,
											   k_start=&k_start,
											   k_end=&k_end,
											   llo_prefix=&llo_prefix,
											   prevalences=&prevalences
											   );
					
					*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
					%if %upcase(&trait_type) = QUALITATIVE %then %do;	
						%do c=1 %to &nclasses;
							%let class=&&class&c;
							%let p_prob=&&p_prob&c;
							data _namedat;
								length CLASS P_PROB $32 ;
							 	CLASS="&class";
								P_PROB="&p_prob";
								EVENT="&event";
							run;
							%if &c=1 %then %do;
								data &cvoutlib.._namedat&gridseq;
									set _namedat;
								run;
							%end;
							%else %do;
								data &cvoutlib.._namedat&gridseq;
									set &cvoutlib.._namedat&gridseq
										_namedat;
								run;
							%end;
						%end;
					%end;

					%iter_exit:;
					%mend;

					%ITERATION_LOOP;

			  	endrsubmit;
			;

		%end;

		* WAIT FOR ALL RSUBMITS TO FINISH;
		%put waitlist = &waitlist;
		waitfor _ALL_ &waitlist;

		* STOP SAS RUNNING ON THE GRID NODES;                                 
		* signoff _all_;

		* UPDATE GRID NODES RUN_STATUS IN logridseq.sas7bdat;
		data &cvoutlib..logridseq;
			set &cvoutlib..logridseq;
			%do node=1 %to &ngridnodes;
				%let gndone=&&gndone&node;
				if grid_node = &node and run_status ne 0 then do;
					run_status=&gndone;
				end;
			%end;
		run;

		data rungrid;
			set &cvoutlib..logridseq;
			if run_status ne 0 then output;
		run;

		%NObsVars(rungrid);
		%let rungridseq=&nObs;

	%end;	

%end;
%else %do;
	
	%put CV_METHOD: LEAVE &leave_l_out OUT ;

	%if &leave_l_out=3 %then %do;
		%do __i=1 %to &__nobs-2;
		  %do __j=&__i+1%to &__nobs-1;
			%do __k=&__j+1 %to &__nobs;
				%leave_l_out_estimates(datapath=&datapath,
									  outputpath=&outputpath,
									  dsname=&dsname,
									  model_names=&model_names,
									  n_models=&n_models,
									  trait=&trait,
									  trait_type=&trait_type,
									  leave_l_out=&leave_l_out,
									  __i=&__i,
									  __j=&__j,
									  __k=&__k,
									  llo_prefix=&llo_prefix,
									  prevalences=&prevalences,
									  cv_custompriors=%nrbquote(&cv_custompriors),
									  cv_customcosts=%nrbquote(&cv_customcosts),
									  cv_alossprop=&cv_alossprop
									  );
			%end; * END OF K LOOP;
		  %end; * END OF J LOOP;
		%end; * END OF I LOOP;
	%end;
	%else %if &leave_l_out=2 %then %do;
		%do __i=1 %to &__nobs-1;
			%do __j=&__i+1%to &__nobs;
				%leave_l_out_estimates(datapath=&datapath,
									  outputpath=&outputpath,
									  dsname=&dsname,
									  model_names=&model_names,
									  n_models=&n_models,
									  trait=&trait,
									  trait_type=&trait_type,
									  leave_l_out=&leave_l_out,
									  __i=&__i,
									  __j=&__j,
									  __k=,
									  llo_prefix=&llo_prefix,
									  prevalences=&prevalences,
									  cv_custompriors=%nrbquote(&cv_custompriors),
									  cv_customcosts=%nrbquote(&cv_customcosts),
									  cv_alossprop=&cv_alossprop
									  );
			%end; * END OF J LOOP;
		%end; * END OF I LOOP;
	%end;
	%else %if &leave_l_out=1 %then %do;
		%do __i=1 %to &__nobs;
			%leave_l_out_estimates(datapath=&datapath,
								  outputpath=&outputpath,
								  dsname=&dsname,
								  model_names=&model_names,
								  n_models=&n_models,
								  trait=&trait,
								  trait_type=&trait_type,
								  leave_l_out=&leave_l_out,
								  __i=&__i,
								  __j=,
								  __k=,
								  llo_prefix=&llo_prefix,
								  prevalences=&prevalences,
								  cv_custompriors=%nrbquote(&cv_custompriors),
								  cv_customcosts=%nrbquote(&cv_customcosts),
								  cv_alossprop=&cv_alossprop
								  );		
		%end; * END OF I LOOP;
	%end;

	*SAVE CLASSx P_PROBx GLOBAL MACRO VARIABLE VALUES IN _NAMEDAT DATA SET FOR REFERENCES;
	%if %upcase(&trait_type) = QUALITATIVE %then %do;	
		%do c=1 %to &nclasses;
			%let class=&&class&c;
			%let p_prob=&&p_prob&c;
			data _namedat;
				length CLASS P_PROB $32 ;
			 	CLASS="&class";
				P_PROB="&p_prob";
				EVENT="&event";
			run;
			%if &c=1 %then %do;
				data &cvoutlib.._namedat;
					set _namedat;
				run;
			%end;
			%else %do;
				data &cvoutlib.._namedat;
					set &cvoutlib.._namedat
						_namedat;
				run;
			%end;
		%end;
	%end;

%end;
 
%exit:
;
%mend;

%macro lc_leave_l_out_iteration_loop(datapath=,
									 outputpath=,
									 dsname=,
									 innerloop_option=,
									 testset_option=,
									 model_names=,
									 n_models=,
									 trait=,
									 trait_type=,
									 __nobs=,
									 leave_l_out=,
									 train_size=,
									 outer_loop=,
									 inner_loop=,
									 i_start=,
									 i_end=,
									 j_start=,
									 j_end=,
									 k_start=,
									 k_end=,
									 llo_prefix=cv,
									 prevalences=);

%if &leave_l_out=3 %then %do;
	%let __i = &i_start;
   	%let __j = &j_start;
   	%let __k = &k_start;
	%do %while(1);
		%lc_leave_l_out_estimates(datapath=&datapath,
								  outputpath=&outputpath,
								  dsname=&dsname,
								  innerloop_option=&innerloop_option,
								  testset_option=&testset_option,
								  model_names=&model_names,
								  n_models=&n_models,
								  trait=&trait,
								  trait_type=&trait_type,
								  leave_l_out=&leave_l_out,
								  train_size=&train_size,
								  outer_loop=&outer_loop,
								  inner_loop=&inner_loop,
								  __i=&__i,
								  __j=&__j,
								  __k=&__k,
								  llo_prefix=&llo_prefix,
								  prevalences=&prevalences);
		%put i=&__i j=&__j k=&__k;
		%if (&__i = &i_end) and (&__j = &j_end) and (&__k = &k_end) %then %do;
			%goto exit_l3;
		%end;
    	%let __k = %sysevalf(&__k + 1, int);
	    %if &__k > &__nobs %then %do;
	       %let __j = %sysevalf(&__j + 1, int);  
	       %if &__j > &__nobs -1 %then %do;
	          %let __i = %sysevalf(&__i + 1, int);
	          %let __j = %sysevalf(&__i + 1, int);
	       %end;
	       %let __k = %sysevalf(&__j + 1, int);;
	    %end;
 	%end;
	%exit_l3:;
%end;
%else %if &leave_l_out=2 %then %do;
	%let __i = &i_start;
	%let __j = &j_start;
	%do %while(1);
		%lc_leave_l_out_estimates(datapath=&datapath,
								  outputpath=&outputpath,
								  dsname=&dsname,
								  innerloop_option=&innerloop_option,
								  testset_option=&testset_option,
								  model_names=&model_names,
								  n_models=&n_models,
								  trait=&trait,
								  trait_type=&trait_type,
								  leave_l_out=&leave_l_out,
								  train_size=&train_size,
								  outer_loop=&outer_loop,
								  inner_loop=&inner_loop,
								  __i=&__i,
								  __j=&__j,
								  __k=,
								  llo_prefix=&llo_prefix,
								  prevalences=&prevalences);
		%put i=&__i j=&__j;  
		%if &__i = &i_end and &__j = &j_end %then %do;
			%goto exit_l2;
		%end;
		%let __j = %sysevalf(&__j + 1, int);  
	    %if &__j > &__nobs %then %do;
	         %let __i = %sysevalf(&__i + 1, int);
	         %let __j = %sysevalf(&__i + 1, int);
	    %end;
	%end;
	%exit_l2:;
%end;
%else %if &leave_l_out=1 %then %do;
	%do __i=&i_start %to &i_end;
		%lc_leave_l_out_estimates(datapath=&datapath,
								  outputpath=&outputpath,
								  dsname=&dsname,
								  innerloop_option=&innerloop_option,
								  testset_option=&testset_option,
								  model_names=&model_names,
								  n_models=&n_models,
								  trait=&trait,
								  trait_type=&trait_type,
								  leave_l_out=&leave_l_out,
								  train_size=&train_size,
								  outer_loop=&outer_loop,
								  inner_loop=&inner_loop,
								  __i=&__i,
								  __j=,
								  __k=,
								  llo_prefix=&llo_prefix,
								  prevalences=&prevalences);
	%end; * END OF I LOOP;
%end;

%mend;

%macro leave_l_out_iteration_loop(datapath=,
								  outputpath=,
								  dsname=,
								  model_names=,
								  n_models=,
								  trait=,
								  trait_type=,
								  __nobs=,
								  leave_l_out=,
								  i_start=,
								  i_end=,
								  j_start=,
								  j_end=,
								  k_start=,
								  k_end=,
								  llo_prefix=cv,
								  prevalences=);

%if &leave_l_out=3 %then %do;
	%let __i = &i_start;
   	%let __j = &j_start;
   	%let __k = &k_start;
	%do %while(1);
		%leave_l_out_estimates(datapath=&datapath,
							   outputpath=&outputpath,
							   dsname=&dsname,
							   model_names=&model_names,
							   n_models=&n_models,
							   trait=&trait,
							   trait_type=&trait_type,
							   leave_l_out=&leave_l_out,
							   __i=&__i,
							   __j=&__j,
							   __k=&__k,
							   llo_prefix=&llo_prefix,
							   prevalences=&prevalences);

		%put in leave_l_out_iteration_loop;
		%put i=&__i j=&__j k=&__k;
		%if (&__i = &i_end) and (&__j = &j_end) and (&__k = &k_end) %then %do;
			%goto exit_l3;
		%end;
    	%let __k = %sysevalf(&__k + 1, int);
	    %if &__k > &__nobs %then %do;
	       %let __j = %sysevalf(&__j + 1, int);  
	       %if &__j > &__nobs -1 %then %do;
	          %let __i = %sysevalf(&__i + 1, int);
	          %let __j = %sysevalf(&__i + 1, int);
	       %end;
	       %let __k = %sysevalf(&__j + 1, int);;
	    %end;
 	%end;
	%exit_l3:;
%end;
%else %if &leave_l_out=2 %then %do;
	%let __i = &i_start;
	%let __j = &j_start;
	%do %while(1);
		%leave_l_out_estimates(datapath=&datapath,
							   outputpath=&outputpath,
							   dsname=&dsname,
							   model_names=&model_names,
							   n_models=&n_models,
							   trait=&trait,
							   trait_type=&trait_type,
							   leave_l_out=&leave_l_out,
							   __i=&__i,
							   __j=&__j,
							   __k=,
							   llo_prefix=&llo_prefix,
							   prevalences=&prevalences);
		%put i=&__i j=&__j;  
		%if &__i = &i_end and &__j = &j_end %then %do;
			%goto exit_l2;
		%end;
		%let __j = %sysevalf(&__j + 1, int);  
	    %if &__j > &__nobs %then %do;
	         %let __i = %sysevalf(&__i + 1, int);
	         %let __j = %sysevalf(&__i + 1, int);
	    %end;
	%end;
	%exit_l2:;
%end;
%else %if &leave_l_out=1 %then %do;
	%do __i=&i_start %to &i_end;
		%leave_l_out_estimates(datapath=&datapath,
							   outputpath=&outputpath,
							   dsname=&dsname,
							   model_names=&model_names,
							   n_models=&n_models,
							   trait=&trait,
							   trait_type=&trait_type,
							   leave_l_out=&leave_l_out,
							   __i=&__i,
							   __j=,
							   __k=,
							   llo_prefix=&llo_prefix,
							   prevalences=&prevalences);
	%end; * END OF I LOOP;
%end;

%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LC_LEAVE_L_OUT_ESTIMATES
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    11/30/09
**  PURPOSE: Fit leave l out cross validation models  
**           Append models predictions and selected predictors
**  HISTORY: action                                       date     who
**           Created.                                     11/30/09 pyt 
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  INNERLOOP_OPTION: Learning curve inner loop algorithm on 
**		  	          how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**		
**  TESTSET_OPTION: Learning curve test set choice for evaluating model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  MODEL_NAMES: Model names
**  N_MODELS: Number of modeling methods
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**	OUTER_LOOP: Learning curve outer loop index
**  INNER_LOOP: Learning curve inner loop index
**  LLO_PREFIX: Prefix of llo prediction results data set name
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**
**  NOTES:
**  Leave_l_out supports 1, 2, up to 3 observations out for holdout 
************************************************************************;
%macro lc_leave_l_out_estimates(datapath=,
							    outputpath=,
								dsname=,
								innerloop_option=,
								testset_option=,
								trait=,
								trait_type=,
								model_names=,
								n_models=,
								leave_l_out=,
								train_size=,
								outer_loop=,
								inner_loop=,
								__i=,
								__j=,
								__k=,
								llo_prefix=cv,
								prevalences=proportional);	

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lloname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__lloname)) = 1 %then %do;
	%NObsVars(&__lloname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lloname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lloname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
%end;

* SUBSET CVTRAIN, CVTEST;
%if &leave_l_out = 1 %then %do;
	data cvtrain cvtest;
		set &__lloname;	
		if (_n_ ne &__i) then output cvtrain; 
	 	else output cvtest;
	run;  
%end;
%else %if &leave_l_out = 2 %then %do;
	data cvtrain cvtest;
		set &__lloname;
		if (_n_ ne &__i) and (_n_ ne &__j) then output cvtrain; 
	    else output cvtest;
	run;
%end;
%else %if &leave_l_out = 3 %then %do;
	%put __i=&__i;
	%put __j=&__j;
	%put __k=&__k;
	data cvtrain cvtest;
		set &__lloname;
		if ((_n_ ne &__i) and (_n_ ne &__j)) and (_n_ ne &__k) then output cvtrain;
	    else output cvtest;
	run;
%end;

* CONFIRM CVTEST IS NOT EMPTY;
%if %sysfunc(exist(cvtest)) = 1 %then %do;
	%NObsVars(cvtest);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
		%goto iter_end;
	%end; 
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: CVTEST DOES NOT EXIST.;
	%goto iter_end;
%end;

%if (&CV_ListAll = Yes) %then %do;
	%if &leave_l_out=3 %then %do;
	    data &cvoutlib..cvtrain_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
	        set cvtest;
	    run;
	%end;
	%else %if &leave_l_out=2 %then %do;
	    data &cvoutlib..cvtrain_i&inner_loop._o&outer_loop._&__i._&__j;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_i&inner_loop._o&outer_loop._&__i._&__j;
	        set cvtest;
	    run;
	%end;
	%else %if &leave_l_out=1 %then %do;
	    data &cvoutlib..cvtrain_i&inner_loop._o&outer_loop._&__i;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_i&inner_loop._o&outer_loop._&__i;
	        set cvtest;
	    run;
	%end;
%end;

* CREATE LEAVE-L-OUT MODELS PREDICTIONS,
  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
  CV_<MODELNAME>_PREDS AND CV_<MODELNAME>_VARS, 
  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
%if %upcase(&innerloop_option) = CVTEST %then %do; 
	%if %upcase(&testset_option) = FIXTEST %then %do;
		%fitmodels(train_dsname=cvtrain,
				   test_dsname=&cvoutlib..Fixtest);
	%end;
	%else %if %upcase(&testset_option) = HOLDOUT %then %do;
		%fitmodels(train_dsname=cvtrain,
				   test_dsname=outer_holdout);
	%end;
%end;
%else %if %upcase(&innerloop_option) = CVHOLDOUT %then %do;
	%fitmodels(train_dsname=cvtrain,
			   test_dsname=cvtest);
%end;

%if &trait_type = qualitative %then %do;
	%put nclasses=&nclasses;
	%put event=&event;
%end;

* APPEND LEAVE-L-OUT MODELS PREDICTIONS AND VARS;
%put llo_prefix=&llo_prefix;

%do nm=1 %to &n_models;
	%let FM_Error=&&FM_Error&nm;
	%put FM_ERROR&nm = &FM_Error;
	* ERROR CHECKING;
	%if &FM_Error %then %do;
		%if &leave_l_out=3 %then %do;
		    data &cvoutlib..cvtrain_m&nm._i&inner_loop._o&outer_loop._&__i._&__j._&__k;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._i&inner_loop._o&outer_loop._&__i._&__j._&__k;
		        set cvtest;
		    run;
		%end;
		%else %if &leave_l_out=2 %then %do;
		    data &cvoutlib..cvtrain_m&nm._i&inner_loop._o&outer_loop._&__i._&__j;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._i&inner_loop._o&outer_loop._&__i._&__j;
		        set cvtest;
		    run;
		%end;
		%else %if &leave_l_out=1 %then %do;
		    data &cvoutlib..cvtrain_m&nm._i&inner_loop._o&outer_loop._&__i;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._i&inner_loop._o&outer_loop._&__i;
		        set cvtest;
		    run;
		%end;
	%end;
	%else %do;
    	%let mdlname=&&mdlname&nm;

		%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds)) = 1 %then %do;
			%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds);
			%if &nObs and &nVars %then %do;
				%if &leave_l_out=3 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
						INNER_LOOP=&inner_loop;
						OUTER_LOOP=&outer_loop;
						TRAIN_SIZE=&train_size;
					run;
				%end;
				%else %if &leave_l_out=2 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
						INNER_LOOP=&inner_loop;
						OUTER_LOOP=&outer_loop;
						TRAIN_SIZE=&train_size;
					run;
				%end;
				%else %if &leave_l_out=1 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
						INNER_LOOP=&inner_loop;
						OUTER_LOOP=&outer_loop;
						TRAIN_SIZE=&train_size;
					run;
				%end;
			%end;
		%end;
		
	%end;

%end; * END OF N_MODELS LOOP;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LEAVE_L_OUT_ESTIMATES
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/22/07
**  PURPOSE: Fit leave l out cross validation models  
**           Append models predictions and selected predictors
**  HISTORY: action                                       date     who
**           Created.                                     06/22/07 pyt 
**
**  MACRO PARAMETERS:
**  DATAPATH:     Library path of training data set 
**  OUTPUTPATH:   Library path of output data sets 
**  DSNAME:       Training data set name
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  MODEL_NAMES: Model names
**  N_MODELS: Number of modeling methods
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**  LLO_PREFIX: Prefix of llo prediction results data set name
** 	PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)  
**  CV_CUSTOMCOSTS: Custom cost functions
**  CV_ALOSSPROP: Asymmetric loss evaluation proportion
**
**  NOTES:
**  Leave_l_out supports 1, 2, up to 3 observations out for holdout 
************************************************************************;
%macro leave_l_out_estimates(datapath=,
						    outputpath=,
							dsname=,
							trait=,
							trait_type=,
							model_names=,
							n_models=,
							leave_l_out=,
							__i=,
							__j=,
							__k=,
							llo_prefix=cv,
							prevalences=proportional,
						    cv_custompriors=,
							cv_customcosts=,
							cv_alossprop=
							);	

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __lloname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__lloname)) = 1 %then %do;
	%NObsVars(&__lloname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__lloname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__lloname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
%end;

* SUBSET CVTRAIN, CVTEST;
%if &leave_l_out = 1 %then %do;
	data cvtrain cvtest;
		set &__lloname;	
		if (_n_ ne &__i) then output cvtrain; 
	 	else output cvtest;
	run;  
%end;
%else %if &leave_l_out = 2 %then %do;
	data cvtrain cvtest;
		set &__lloname;
		if (_n_ ne &__i) and (_n_ ne &__j) then output cvtrain; 
	    else output cvtest;
	run;
%end;
%else %if &leave_l_out = 3 %then %do;
	%put __i=&__i;
	%put __j=&__j;
	%put __k=&__k;
	data cvtrain cvtest;
		set &__lloname;
		if ((_n_ ne &__i) and (_n_ ne &__j)) and (_n_ ne &__k) then output cvtrain;
	    else output cvtest;
	run;
%end;

* CONFIRM CVTEST IS NOT EMPTY;
%if %sysfunc(exist(cvtest)) = 1 %then %do;
	%NObsVars(cvtest);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: CVTEST CONTAINS 0 OBSERVATIONS.;
		%goto iter_end;
	%end; 
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: CVTEST DOES NOT EXIST.;
	%goto iter_end;
%end;

* CREATE BASELINE SCORED DATA SET;
* COMPUTE FIT STATISTICS: ASE, AUC FROM EACH CV RUN;
%if %upcase(&trait_type)=QUALITATIVE %then %do;
	 %basescor(train_dsname=cvtrain,
			   test_dsname=cvtest,
			   trait=&trait,
			   prevalences=&prevalences,
			   cv_custompriors=%nrbquote(&cv_custompriors));
%end;
%else %do;
	 %basescorint(train_dsname=cvtrain,
		          test_dsname=cvtest,
	    	      trait=&trait);
%end;

%if %sysfunc(exist(baseline_preds)) = 1 %then %do;
	%NObsVars(baseline_preds);
	%if &nObs and &nVars %then %do;
		%if &leave_l_out=3 %then %do;
			data &cvoutlib..&llo_prefix._baseline_preds_&__i._&__j._&__k;
				length METHOD $64;
				set baseline_preds;
				METHOD = "BASELINE";
			run;
		%end;
		%else %if &leave_l_out=2 %then %do;
			data &cvoutlib..&llo_prefix._baseline_preds_&__i._&__j;
				length METHOD $64;
				set baseline_preds;
				METHOD = "BASELINE";
			run;
		%end;
		%else %if &leave_l_out=1 %then %do;
			data &cvoutlib..&llo_prefix._baseline_preds_&__i;
				length METHOD $64;
				set baseline_preds;
				METHOD = "BASELINE";
			run;
		%end;
	%end;
%end;

%if (&CV_ListAll = Yes) %then %do;
	%if &leave_l_out=3 %then %do;
	    data &cvoutlib..cvtrain_&iter._&__i._&__j._&__k;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_&iter._&__i._&__j._&__k;
	        set cvtest;
	    run;
	%end;
	%else %if &leave_l_out=2 %then %do;
	    data &cvoutlib..cvtrain_&iter._&__i._&__j;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_&iter._&__i._&__j;
	        set cvtest;
	    run;
	%end;
	%else %if &leave_l_out=1 %then %do;
	    data &cvoutlib..cvtrain_&iter._&__i;
	        set cvtrain;
	    run;
	    data &cvoutlib..cvtest_&iter._&__i;
	        set cvtest;
	    run;
	%end;
%end;

* CREATE LEAVE-L-OUT MODELS PREDICTIONS,
  TWO OUTPUT DATA SETS WILL BE CREATED BY EACH MODEL: 
  CV_<MODELNAME>_PREDS AND CV_<MODELNAME>_VARS, 
  THE LATTER WILL CONTAIN THE PREDICTOR VARIABLES SELECTED;
%fitmodels(train_dsname=cvtrain,
		   test_dsname=cvtest);

%if &trait_type = qualitative %then %do;
	%put nclasses=&nclasses;
	%put event=&event;
%end;

* APPEND LEAVE-L-OUT MODELS PREDICTIONS AND VARS;
%put llo_prefix=&llo_prefix;

%do nm=1 %to &n_models;
	%let FM_Error=&&FM_Error&nm;
	%put FM_ERROR&nm = &FM_Error;
	* ERROR CHECKING;
	%if &FM_Error %then %do;
		%if &leave_l_out=3 %then %do;
		    data &cvoutlib..cvtrain_m&nm._&__i._&__j._&__k;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._&__i._&__j._&__k;
		        set cvtest;
		    run;
		%end;
		%else %if &leave_l_out=2 %then %do;
		    data &cvoutlib..cvtrain_m&nm._&__i._&__j;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._&__i._&__j;
		        set cvtest;
		    run;
		%end;
		%else %if &leave_l_out=1 %then %do;
		    data &cvoutlib..cvtrain_m&nm._&__i;
		        set cvtrain;
		    run;
		    data &cvoutlib..cvtest_m&nm._&__i;
		        set cvtest;
		    run;
		%end;
	%end;
	%else %do;
    	%let mdlname=&&mdlname&nm;

		%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds)) = 1 %then %do;
			%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds);
			%if &nObs and &nVars %then %do;
				%if &leave_l_out=3 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_&__i._&__j._&__k;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
					run;
				%end;
				%else %if &leave_l_out=2 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_&__i._&__j;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
					run;
				%end;
				%else %if &leave_l_out=1 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._preds_&__i;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_preds;
					    METHOD = "&mdlname";
					run;
				%end;
			%end;
		%end;
		
		* APPEND PREDICTOR VARS SELECTED DATA SETS;
		%if %sysfunc(exist(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars)) = 1 %then %do;
			%NObsVars(cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars);
			%if &nObs and &nVars %then %do;
				%if &leave_l_out=3 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._vars_&__i._&__j._&__k;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
				        METHOD = "&mdlname";
				    run;
				%end;
				%else %if &leave_l_out=2 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._vars_&__i._&__j;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
				        METHOD = "&mdlname";
				    run;
				%end;
				%else %if &leave_l_out=1 %then %do;
					data &cvoutlib..&llo_prefix._m&nm._vars_&__i;
					    length METHOD $64;
						set cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_vars;
				        METHOD = "&mdlname";
				    run;
				%end;
			%end;
		%end;

	%end;

%end; * END OF N_MODELS LOOP;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LO_FITSTATS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/022/07
**  PURPOSE: Macro statement to insert in the if statement for 
**           appending leave_l_out output
**  HISTORY: action                                        date     who
**           Created.                                      06/22/07 pyt 
**
**  MACRO PARAMETERS:
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  EVENT:        Trait event value
**  FREQ:         Frequency variable name (optional)
**  LEAVE_L_OUT:  Leave l observations out for holdout test set
**  WHERE_CLAUSE: Where clause used to subset cumulated models  
**                predictions for each leave_l_out iteration
**  PREVALENCE:   Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCE = CUSTOM)
**
**  NOTES:
**  Apply models predictions for each leave_l_out iteration to compute 
**  model ase, acc, and auc
***********************************************************************;
%macro lo_fitstats(trait=,
				   trait_type=,
				   event=,
				   freq=,
				   n_models=,
				   leave_l_out=,
				   where_clause=,
				   prevalences=proportional,
				   cv_custompriors=
				   );

%if &cv_alossprop ne 0.5 %then %do;
	%let cv_alossweight = Asymmetric;
%end;
%else %do;
	%let cv_alossweight = ;
%end;
		
%do nm=1 %to &n_models;
    %let mdlname=&&mdlname&nm;
	%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
		%NObsVars(&cvoutlib..cv_m&nm._pcum);
		%if &nObs and &nVars %then %do;
			* COMPUTE MODEL ASE;
		    %if %upcase(&trait_type)=QUALITATIVE %then %do;
				  %compute_ase_nominal(data=&cvoutlib..cv_m&nm._pcum,
											  output=cv_m&nm._ase,
											  actual=actualp,
											  nclasses=&nclasses,
											  mdlname=&mdlname,
											  freq=&freq,
											  prevalences=&prevalences,
									 		  cv_custompriors=%nrbquote(&cv_custompriors),
											  trait=&trait
											  );
				  %compute_acc_nominal(data=&cvoutlib..cv_m&nm._pcum,
											  output=cv_m&nm._acc,
											  trait=&trait,
											  correct=correct,
											  mdlname=&mdlname,
											  freq=&freq,
											  prevalences=&prevalences,
									 		  cv_custompriors=%nrbquote(&cv_custompriors)
											  );
				%NObsVars(cv_m&nm._acc);
				%if &nObs and &nVars %then %do;
			        data cv_m&nm._acc;
			            length METHOD $64;
			            set cv_m&nm._acc;
			            METHOD="&mdlname";
			        run; 
				%end;

					%if %length(&cv_customcosts) %then %do;
						%compute_aec_nominal(
							 data=&cvoutlib..cv_m&nm._pcum,
							 output=cv_m&nm._aec,
							 trait=&trait,
							 expcost=expected_cost,
							 mdlname=&mdlname,
							 freq=&freq
						);
						%let aec = cv_m&nm._aec;
						%NObsVars(&aec);
						%if &nObs and &nVars %then %do;
							  data &aec;
									length METHOD $64;
									set &aec;
									METHOD="&mdlname";
							  run; 
						%end;
					%end;
					%else %do;
						%let aec = ;
					%end;


				%if &nclasses = 2 %then %do;
			        %locate_pevent_isevent(data=&cvoutlib..cv_m&nm._pcum,
			                               trait=&trait,
			                               event=&event,
			                               actual=actualp);

			        * COMPUTE MODEL MANN;
			        %compute_mann(data=&cvoutlib..cv_m&nm._pcum,
			                      pevent=_pevent,
			                      isevent=_isevent);
			        %put mann=&mann;

			        data cv_m&nm._mann;
			            length METHOD $64;
			            METHOD = "&mdlname";    
			            AUC=&mann;
			        run;
			    %end;

		    %end;
		    %else %if %upcase(&trait_type)=QUANTITATIVE %then %do;

				%if &cv_alossprop ne 0.5 %then %do;
					data &cvoutlib..cv_m&nm._pcum;
						set &cvoutlib..cv_m&nm._pcum;
						_resid = &trait - Predicted_Value;
						if missing(_resid) then &cv_alossweight = .;
						else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
						else &cv_alossweight = 2 * (1 - &cv_alossprop);
					run;
				%end;

		       %compute_ase_interval(data=&cvoutlib..cv_m&nm._pcum,
		                             output=cv_m&nm._ase,
		                             y=&trait,
		                             predict=Predicted_Value,
		                             mdlname=&mdlname,
		                             freq=&freq,
		                             aweight=&cv_alossweight);
		       %compute_corr_interval(data=&cvoutlib..cv_m&nm._pcum,
		                             output=cv_m&nm._corr,
		                             y=&trait,
		                             predict=Predicted_Value,
		                             mdlname=&mdlname,
		                             freq=&freq,
		                             weight=&cv_alossweight);   
		                             
				 %compute_harrellc(data=&cvoutlib..cv_m&nm._pcum,
								  output=cv_m&nm._hac,
								  TimeVar=&trait,
								  CenVar=,
								  Predict=Predicted_Value,
								  Freq=&cv_alossweight);
			    data cv_m&nm._hac;
			       length METHOD $64;
			       set cv_m&nm._hac;
			       METHOD="&mdlname";
		    	 run;

			%end;

		    data cv_m&nm._ase;
		       length METHOD $64;
		        set cv_m&nm._ase;
		       METHOD="&mdlname";
	    	 run;
		%end;
	%end;
%end; * END OF N_MODELS LOOP;

%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LOCATE_PEVENT_ISEVENT
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/022/07
**  PURPOSE: Locate pevent corresponds to the actual event
**  HISTORY: action                                        date     who
**           Created.                                      06/22/07 pyt 
**
**  MACRO PARAMETERS:
**  LEAVE_L_OUT: Leave l observations out for holdout test set
**
**  NOTES:
***********************************************************************;
%macro locate_pevent_isevent(data=, 
							 trait=, 
							 event=, 
							 etype=C,
							 actual=);

%if %sysfunc(exist(&data)) = 1 %then %do;
	%NObsVars(&data);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &data contains 0 observations or 0 variables.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &data does not exist.;
	%goto exit;
%end;

data &data;
	label _isevent="Is_Event" _pevent="P_Event";
	set &data;
	retain ip 0;
	array _p p_:;
	if missing(&trait) then do;
		_isevent = .;
		_pevent = .;
	end;
	else do;
	  if ip eq 0 then do;
			if &trait eq %if &etype=C %then %do; "&event" %end; %else %do; &event %end;
				then do;  * actual corresponds to event;
				 if abs(&actual - _p(1)) < 1e-6  then ip = 1;
				 else ip = 2;
			end;
			else do;  * actual corresponds to non-event;
				 if abs(&actual - _p(1)) < 1e-6 then ip = 2;
				 else ip = 1;
			end;
	  end;

	  if &trait eq %if &etype=C %then %do; "&event" %end; %else %do; &event %end; 
		then _isevent = 1;
	  else _isevent = 0;

	  _pevent = _p(ip);
	end;
	drop ip;
run;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    LOOP_PREDS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    12/01/07
**  PURPOSE: Loop over predictors to drop one at a time from fit model
**  HISTORY: action                                        date     who
**           Created.                                      12/01/07 pyt 
** 
**  MACRO PARAMETERS:
**  EFFECTS_DSNAME: Models effects frequencies data set name
**  TRAIN_DSNAME:	Training data set name
**  TEST_DSNAME: 	Test data set set name
**  MODEL_NAME:  	Model name
**  TRAIT_TYPE:
**
**  NOTES:
**  Limit to 990 variables 
************************************************************************;
%macro loop_preds(effects_dsname=,
				  train_dsname=,
				  test_dsname=,
				  model_name=&mdlname);

%if %sysfunc(exist(&effects_dsname)) = 1 %then %do;
	%NObsVars(&effects_dsname);
	%let _nvars_ = &nObs;
	%if ^&nObs or ^&nVars %then %do;
		%put ERROR: &effects_dsname CONTAINS 0 OBSERVATIONS.;
		%put ERROR: MODEL &mdlname CONTAINS 0 VARIABLES.;
		%let exiterror = 1;
	    %goto cv_exit;
	%end;
	%else %if &nObs > 990 %then %do;
		%put ERROR: MODEL &mdlname CONTAINS &_nvars_ VARIABLES.;
		%put ERROR: TOO MANY FOR VARIABLE IMPORTANCE TO HANDLE.;
		%let exiterror = 1;
		%goto cv_exit;
	%end;
%end;
%else %do;
	%put ERROR: &effects_dsname DOES NOT EXIST.;
	%let exiterror = 1;
    %goto cv_exit;
%end;

* LOOP OVER PREDICTORS, DROP ONE AT A TIME;
%do v=1 %to &_nvars_;
	data _null_;
		set &effects_dsname;
		if _n_ = &v then do;
			call symput("DropPredVars", trim(left(NAME)));
			call symput("DROP_VAR", trim(left(NAME)));
			call symput("DROP_LABEL", trim(left(LABEL)));
		end;
	run;

	%fitmodels(train_dsname=&train_dsname,
			   test_dsname=&test_dsname);

	* ERROR CHECKING;
	%if ^&FM_Error1 %then %do; 
		%if %sysfunc(exist(cv_&model_name._preds)) = 1 %then %do;
			%NObsVars(cv_&model_name._preds);
			%if &nObs and &nVars %then %do;
				data vipreds; 
					length DROPVAR DROPLABEL $32;
					set cv_&model_name._preds;
					DROPVAR="&DROP_VAR";
					DROPLABEL="&DROP_LABEL";
				run;

				%if %sysfunc(exist(cv_vipreds)) = 1 %then %do;
					%NObsVars(cv_vipreds);
					%let _cipredObs_ = &nObs;
					data cv_vipreds;
						set 
						%if &_cipredObs_ %then %do;
							cv_vipreds
						%end;
							vipreds;
					run;
				%end;
				%else %do;
					data cv_vipreds;
						set vipreds;
					run;
				%end;
			%end;
		%end;

		%if %sysfunc(exist(cv_&model_name._vars)) = 1 %then %do;
			%NObsVars(cv_&model_name._vars);
			%if &nObs and &nVars %then %do;
				data vivars; 
					length DROPVAR DROPLABEL $32;
					set cv_&model_name._vars;
					DROPVAR="&DROP_VAR";
					DROPLABEL="&DROP_LABEL";
				run;

				%if %sysfunc(exist(cv_vivars)) = 1 %then %do;
					%NObsVars(cv_vivars);
					%let _vivarsObs_ = &nObs;
					data cv_vivars;
						set 
						%if &_vivarsObs_ %then %do;
							cv_vivars
						%end;
							vivars;
					run;
				%end;
				%else %do;
					data cv_vivars;
						set vivars;
					run;
				%end;
			%end;
		%end;
	%end; 
%end;  * END OF NVARS LOOP;

%cv_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2009 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_CV_MULTI_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    04/10/09
**  PURPOSE: Merge multiple cross validation results
**  HISTORY: action                                        date     who
**           Created.                                      04/10/09 pyt 
** 
**  MACRO PARAMETERS:
**  DATAPATH:  Library path of training data set 
**  OUTPUTPATHS: Library paths of output data sets 
**  MERGE_OUTPUTPATH: Library path of final merged ouput 
**  DSNAME: Data set name to be sampled
**  WHERE_CLAUSE: Where clause
**  TRAIT: Trait variable name
**  TRAIT_TYPE: Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  CENVAR:		Censor variable (only applicable for survival data)
**  CENVAL:		Censor value indicates if the record is censored 
**  NCLASSES:	Number of qualitative trait variable category
**  EVENT: 		Trait event level
**  RDIAGRAM_OUTPUT: Reliability diagram output data set names 
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  KFOLD: K-fold cross validation 
**  LEAVE_L_OUT: Leave l out cross validation method 
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  MODELS_EFFECTS_OUTPUT: Models effects frequencies output 
**  VARIMPORTANCE: Flag to indicate performing variable importance  
**	VARIMPORTANCE_OUTPUT: Variable importance results output
** 	PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**
**  NOTES:
**  All macro parameters except outputpaths and merge_outputpath 
**  are identical to the settings saved in cvmc_plot.jsl of all 
**  output folders 
**  
**  Example:
**  %merge_cv_multi_results(
**		  datapath=C:\...\datapath\,                                                                                                          
**        outputpaths=C:\...\outputpath1\ |                                                                         
**                    C:\...\outputpath2\ |
**					  ...
**					  C:\...\outputpathx\,                                                                          
**        merge_outputpath=C:\...\merge_results_outputpath\,           
**        dsname=train,                                                                                                                     
**        where_clause=,                                                                                                                         
**        trait=trait,                                                                                                                             
**        trait_type=qualitative,    
**    	  cenvar=,
**		  cenval=,
**        nclasses=2,                                                                                                                            
**        event=1,                                                                                                                               
**        cv_method=KFOLD,                                                                                                                       
**        kfold=2,                                                                                                                               
**        leave_l_out=2235,                                                                                                                      
**        models_comparison_output=train_cvr,                                                                                               
**        models_effects_output=train_cve,                                                                                                  
**        varimportance=,                                                                                                                        
**        varimportance_output=,
**		  prevalences=proportional 
**        );
**
************************************************************************;
%macro merge_cv_multi_results(datapath=,
							  outputpaths=,	
							  merge_outputpath=,
							  dsname=,
							  where_clause=,
							  trait=,
							  trait_type=,
							  cenvar=,
							  cenval=,
							  nclasses=,
							  event=,
							  cv_method=,    
							  kfold=,   
							  leave_l_out=, 
							  models_comparison_output=,
							  models_effects_output=,
							  varimportance=,
							  varimportance_output=,
							  prevalences=proportional 
							  );

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED IF SO ALLOCATE IT;
%if %length(&merge_outputpath) > 0 and "&merge_outputpath" ne "" %then %do;
	libname output "&merge_outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;
%put cvoutlib = &cvoutlib;

* RETRIEVE INDIVIDUAL OUTPUTPATHS;
%let i=1;
%do %while(%length(%scan(&outputpaths, &i, "|")));
	%let i = %eval(&i+1);
%end;
%let n_outputpaths = %eval(&i-1);
%do i=1 %to &n_outputpaths;
	%let path = %scan(&outputpaths, &i, "|");	
	libname outlib&i "&path";
%end;


* RETRIEVE MODEL NAMES AND LABELS;
data &cvoutlib..&prefix._cvr;                                                                                                                               
	set
	%do i=1 %to &n_outputpaths;
		outlib&i..&prefix._cvr
	%end;
	;
run;  

proc freq data=&cvoutlib..&prefix._cvr noprint;
	tables METHOD*MODEL/out=fmethod(keep=METHOD MODEL);
run;

%let _fmid_ = %sysfunc(open(work.fmethod));
%let n_models = %sysfunc(attrn(&_fmid_, NOBS));
%put n_models = &n_models;
%let mthnum = %sysfunc(varnum(&_fmid_, METHOD));
%let mdlnum = %sysfunc(varnum(&_fmid_, MODEL));
%do nm=1 %to &n_models;
	%let rc = %sysfunc(fetchobs(&_fmid_, &nm));
	%let mdlname&nm = %sysfunc(getvarc(&_fmid_, &mthnum));
	%let mdlname = &&mdlname&nm;
	%if &nm=1 %then %do;
		%let model_names =&mdlname;
	%end;
	%else %do;
		%let model_names =&model_names &mdlname;
	%end; 
	%global mdllabel&nm;
	%let mdllabel&nm = %sysfunc(getvarc(&_fmid_, &mdlnum));
	%let mdllabel = &&mdllabel&nm;
	%if &nm=1 %then %do;
		%let model_labels =&mdllabel;
	%end;
	%else %do;
		%let model_labels =&model_labels|&mdllabel;
	%end; 
%end;
%let _fmid_ = %sysfunc(close(&_fmid_));
%put model_names = &model_names;
%put model_labels = &model_labels;

data lab;
	length Model $ 64 Method $ 32;
	%do i=1 %to &n_models;
		Model = "&&mdllabel&i";
		Method = "&&mdlname&i";
		output;
	%end;
run;

* FIRST, CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..&models_comparison_output;   
	delete;
run;
data &cvoutlib..&models_effects_output;   
	delete;
run;
 
* ACCUMULATE RESULTS;
data &cvoutlib..&prefix._cvr_all;                                                                                                                               
	set
	%do i=1 %to &n_outputpaths;
		outlib&i..&prefix._cvr_all
	%end;
	;
run;  

proc sort data=&cvoutlib..&prefix._cvr_all;  
	by METHOD ITER;
run;

data &cvoutlib..&models_comparison_output;                                                                                                                               
	set 
	%do i=1 %to &n_outputpaths;
		outlib&i..&models_comparison_output
	%end;
	;
run;     

proc sort data=&cvoutlib..&models_comparison_output;     
	by METHOD ITER;
run;
                                                                                                                                  
data &cvoutlib..&prefix._cve_all;  
	length NAME $32;
	set
	%do i=1 %to &n_outputpaths;
		outlib&i..&prefix._cve_all
	%end;
	;
run;  
%NObsVars(&cvoutlib..&prefix._cve_all);
%if &nObs and &nVars %then %do;
	proc sort data=&cvoutlib..&prefix._cve_all;  
		by METHOD ITER;
	run;
	%compute_effects(outputpath=&merge_outputpath,
					 cv_method=&cv_method,
					 leave_l_out=&leave_l_out,
					 models_comparison_output=&models_comparison_output,
					 models_effects_output=&models_effects_output);
%end;
%else %do;
	%put NOTE: NO PREDICTOR VARIABLES ARE SELECTED FROM ALL MODELS.;
%end;

 
* DELETE INTERMEDIATETE DATA SETS;
proc datasets library=&cvoutlib nolist;
	delete
	%do nm=1 %to &n_models;
		cv_m&nm._vcum
	%end;
	;
quit;

* RETRIEVE BASELINE MEASUREMENTS;
%if %upcase(&cv_method) ne LEAVEOUT and %upcase(&trait_type) ne SURVIVAL %then %do;
	* COPY BASELINE MEASUREMENTS; 
	data &cvoutlib..baseline_pctl;
		set outlib1.baseline_pctl;
	run;
	%let _bpctid_ = %sysfunc(open(&cvoutlib..baseline_pctl));
	%let rc= %sysfunc(fetchobs(&_bpctid_, 1));
	%let rmsemnum = %sysfunc(varnum(&_bpctid_, RMSE_median));
	%let baseline_rmse_median = %sysfunc(getvarn(&_bpctid_, &rmsemnum));
	%let rmseunum = %sysfunc(varnum(&_bpctid_, RMSE_UWHISKER));
	%let baseline_rmse_uwhisker = %sysfunc(getvarn(&_bpctid_, &rmseunum));
	%let rmselnum = %sysfunc(varnum(&_bpctid_, RMSE_LWHISKER));
	%let baseline_rmse_lwhisker = %sysfunc(getvarn(&_bpctid_, &rmselnum));
	%if %upcase(&trait_type)=QUALITATIVE %then %do;
		%if &nclasses = 2 %then %do;
			%let aucmnum = %sysfunc(varnum(&_bpctid_, AUC_median));
			%let baseline_auc_median = %sysfunc(getvarn(&_bpctid_, &aucmnum));
			%let aucunum = %sysfunc(varnum(&_bpctid_, AUC_UWHISKER));
			%let baseline_auc_uwhisker = %sysfunc(getvarn(&_bpctid_, &aucunum));
			%let auclnum = %sysfunc(varnum(&_bpctid_, AUC_LWHISKER));
			%let baseline_auc_lwhisker = %sysfunc(getvarn(&_bpctid_, &auclnum));
		%end;
		%let accmnum = %sysfunc(varnum(&_bpctid_, Accuracy_median));
		%let baseline_accuracy_median = %sysfunc(getvarn(&_bpctid_, &accmnum));
		%let accunum = %sysfunc(varnum(&_bpctid_, Accuracy_UWHISKER));
		%let baseline_accuracy_uwhisker = %sysfunc(getvarn(&_bpctid_, &accunum));
		%let acclnum = %sysfunc(varnum(&_bpctid_, Accuracy_LWHISKER));
		%let baseline_accuracy_lwhisker = %sysfunc(getvarn(&_bpctid_, &acclnum));
		%if %length(&cv_customcosts) %then %do;
			%let aecmnum = %sysfunc(varnum(&_bpctid_, aec_median));
			%let baseline_aec_median = %sysfunc(getvarn(&_bpctid_, &aecmnum));
			%let aecunum = %sysfunc(varnum(&_bpctid_, aec_UWHISKER));
			%let baseline_aec_uwhisker = %sysfunc(getvarn(&_bpctid_, &aecunum));
			%let aeclnum = %sysfunc(varnum(&_bpctid_, aec_LWHISKER));
			%let baseline_aec_lwhisker = %sysfunc(getvarn(&_bpctid_, &aeclnum));
		%end;
	%end;
	%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
		%let hacnum = %sysfunc(varnum(&_bpctid_, HARRELL_median));
		%let baseline_harrell_median = %sysfunc(getvarn(&_bpctid_, &hacnum));
		%let hacunum = %sysfunc(varnum(&_bpctid_, HARRELL_UWHISKER));
		%let baseline_harrell_uwhisker = %sysfunc(getvarn(&_bpctid_, &hacunum));
		%let haclnum = %sysfunc(varnum(&_bpctid_, HARRELL_LWHISKER));
		%let baseline_harrell_lwhisker = %sysfunc(getvarn(&_bpctid_, &haclnum));
		%let corrnum = %sysfunc(varnum(&_bpctid_, CORR_median));
		%let baseline_corr_median = %sysfunc(getvarn(&_bpctid_, &corrnum));
		%let corrunum = %sysfunc(varnum(&_bpctid_, CORR_UWHISKER));
		%let baseline_corr_uwhisker = %sysfunc(getvarn(&_bpctid_, &corrunum));
		%let corrlnum = %sysfunc(varnum(&_bpctid_, CORR_LWHISKER));
		%let baseline_corr_lwhisker = %sysfunc(getvarn(&_bpctid_, &corrlnum));
	%end;
	%let _bpctid_ = %sysfunc(close(&_bpctid_));
%end;

* RETRIEVE TRAIT LEVELS;
%if %upcase(&trait_type) = QUALITATIVE %then %do;
	* COPY _NAMEDAT;
	data &cvoutlib.._namedat;
		set outlib1._namedat;
	run;
	%let _cid_ = %sysfunc(open(&cvoutlib.._namedat));
	%let nclasses = %sysfunc(attrn(&_cid_, NOBS));
	%let classnum = %sysfunc(varnum(&_cid_, CLASS));
	%do c=1 %to &nclasses;
		%let rc = %sysfunc(fetchobs(&_cid_, &c));
		%let class&c = %sysfunc(getvarc(&_cid_, &classnum));
	%end;
	%let _cid_ = %sysfunc(close(&_cid_));
%end;

* RETRIEVE NUMBER OF ITERATIONS, ITERATION START, AND ITERATION END;
proc freq data=&cvoutlib..&models_comparison_output	noprint;
	tables ITER/out=iter(keep=ITER);
run;
%let _itid_ = %sysfunc(open(iter));
%let cv_holdout_iterations = %sysfunc(attrn(&_itid_, NOBS));
%let iternum = %sysfunc(varnum(&_itid_, ITER));
%let rc = %sysfunc(fetchobs(&_itid_, 1));
%let cv_holdout_iteration_start = %sysfunc(getvarn(&_itid_, &iternum));
%let rc = %sysfunc(fetchobs(&_itid_, &cv_holdout_iterations));
%let cv_holdout_iteration_end = %sysfunc(getvarn(&_itid_, &iternum)); 
%let _itid_ = %sysfunc(close(&_itid_));

* GENERATE RELIABILITY DIAGRAM RESULTS TABLE;
%if %upcase(&trait_type)=SURVIVAL %then %do;
	%reliability_diagram(cvoutlib=&cvoutlib, 
			  			 trait=&trait,
			  		     trait_type=&trait_type,
						 cenvar=&cenvar,
						 cenval=&cenval,
			  			 models_comparison_output=&models_comparison_output);

	%compute_mean_survfunc(cvoutlib=&cvoutlib, 
			  			   trait=&trait,
						   cenvar=&cenvar,
						   cenval=&cenval,
			  			   models_comparison_output=&models_comparison_output);
%end;
%else %do;
	%reliability_diagram(cvoutlib=&cvoutlib, 
			  			 trait=&trait,
			  		     trait_type=&trait_type,
			  			 models_comparison_output=&models_comparison_output);
%end;

* OUTPUT TO JSL FILE;
filename jslfile "&merge_outputpath.&mcPrefix._cvmc.jsl";
data _null_;
	file jslfile encoding="utf-8";
	put "//!";
	put "Names Default to Here(1);";
	put "// parameters for cross-validation model comparison plots;";
	put "datapath=""&datapath"";";
	put "outputpath=""&merge_outputpath"";";
	put "dsname=""&dsname"";";
	put "where_clause=""&where_clause"";";
	put "models_comparison_output=""&models_comparison_output"";";
	put "models_effects_output = ""&models_effects_output"";";
	put "cv_method = ""&cv_method"";";
	put "cv_holdout_iterations = &cv_holdout_iterations;";
	put "cv_holdout_iteration_start = &cv_holdout_iteration_start;";
	put "cv_holdout_iteration_end = &cv_holdout_iteration_end;";
	put "cv_prevalences = ""&prevalences"";";
	put "cv_custompriors = """";";
	put "cv_customcosts = """";";
	put "kfold = &kfold;";
	put "leave_l_out = &leave_l_out;";
	put "cvname = ""_CV_"";";
	put "model_names = ""&model_names"";";
	put "model_labels = ""&model_labels"";";
	put "model_label_list = words(model_labels,""|"");";
	put "trait = ""&trait"";";
	put "trait_type = ""&trait_type"";";
	%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;
		%if &cv_method ne LEAVEOUT %then %do;
			put "baseline_rmse_median = &baseline_rmse_median;";
			put "baseline_rmse_uwhisker = &baseline_rmse_uwhisker;";
			put "baseline_rmse_lwhisker = &baseline_rmse_lwhisker;";
			%if %upcase(&trait_type)=QUALITATIVE %then %do;
				%if &nclasses = 2 %then %do;
					put "baseline_auc_median = &baseline_auc_median;";
					put "baseline_auc_uwhisker = &baseline_auc_uwhisker;";
					put "baseline_auc_lwhisker = &baseline_auc_lwhisker;";
				%end;
				put "baseline_accuracy_median = &baseline_accuracy_median;";
				put "baseline_accuracy_uwhisker = &baseline_accuracy_uwhisker;";
				put "baseline_accuracy_lwhisker = &baseline_accuracy_lwhisker;";
				%if %length(&cv_customcosts) %then %do;
					put "baseline_aec_median = &baseline_aec_median;";
					put "baseline_aec_uwhisker = &baseline_aec_uwhisker;";
					put "baseline_aec_lwhisker = &baseline_aec_lwhisker;";
				%end;
			%end;
			%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
				put "baseline_harrell_median = &baseline_harrell_median;";
				put "baseline_harrell_uwhisker = &baseline_harrell_uwhisker;";
				put "baseline_harrell_lwhisker = &baseline_harrell_lwhisker;";
				put "baseline_corr_median = &baseline_corr_median;";
				put "baseline_corr_uwhisker = &baseline_corr_uwhisker;";
				put "baseline_corr_lwhisker = &baseline_corr_lwhisker;";
			%end;
		%end;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
	    	put "nclasses = &nclasses;";
			put "event = ""&event"";";
			%do c=1 %to &nclasses;
				%if &c=1 %then %do;
					%let trait_classes = &&class&c;
				%end;
				%else %do;
					%let trait_classes = &trait_classes,&&class&c;
				%end;
			%end;
			put "trait_classes = ""&&trait_classes"";";
			put "_namedat = ""_namedat"";";
		%end;
		%else %do;
			put "nclasses = .;";
			put "event = "" "";";
			put "trait_classes= "" "";";
			put "_namedat = "" "";";
		%end;
		put "cenvar = "" "";";
		put "cenval = "" "";"; 
	%end;
	%else %do;
		put "nclasses = .;";
		put "event = "" "";";
		put "trait_classes= "" "";";
		put "_namedat = "" "";";
		put "cenvar = ""&cenvar"";";
		put "cenval = ""&cenval"";";
	%end;
	put "prevalences = ""&prevalences"";";
	put "rdiagram_output = ""&prefix._rdiagram"";";
	put "varimportance = ""&varimportance"";";
	put "varimportance_output=""&varimportance_output"";";
	put "if(Host Is(Windows),";
	put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
	put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
	put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
	put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
	%if &varimportance = YES %then %do;
		put "include(jslpath||""Variable Importance Fitsdiff.jsl"");"; 
	%end;
	%else %do;
		put "include(jslpath||""Cross Validation Model Comparison.jsl"");";
	%end;
run;

%exit:;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_CV_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/15/07
**  PURPOSE: Merge cross validation models ase, acc, auc results
**  HISTORY: action                                        date     who
**           Created.                                      06/15/07 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH: Library path of output data sets 
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS:   Number of modeling mthods
**  TRAIT: Trait variable name
**  TRAIT_TYPE: Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  NCLASSES:	Number of qualitative trait variable category
**  EVENT: 		Trait event level
**  FREQ:		Frequency variable
**  CENVAR:		Censor varialbe (only applicable for survival data)
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_ITERATION_START:	Random hold-out iteration starting index
**  CV_HOLDOUT_ITERATION_END: Random hold-out iteration ending index
**  KFOLD: K-fold cross validation 
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  MODELS_EFFECTS_OUTPUT: Models effects frequencies output 
**  VARIMPORTANCE: Flag to indicate performing variable importance  
**	 VARIMPORTANCE_OUTPUT: Variable importance results output
**  PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCES = CUSTOM)
**  GSHARE: Grid share folder
**
**  NOTES:
**  Execute this macro outside of cv_iterations after all iterations are run
**  1. Compute effects frequencies
**  2. Compute overall cv iterations models fir stats: ase, acc, auc
** 	   Merge all models overall ase, acc, auc
**  3. Compute each cv iteration models ase, acc, auc
**     Merge all cv iterations models acc, auc, ase and overall baseline ase
************************************************************************;
%macro merge_cv_results(outputpath=,
					   model_names=, 
					   model_labels=,
						n_models=,
						trait=,
						trait_type=,
						nclasses=,
						event=,
						cenvar=,
						freq=,
						cv_method=,
						cv_holdout_iteration_start=, 
			   	   cv_holdout_iteration_end=, 
						kfold=,
						models_comparison_output=,
						models_effects_output=,
						varimportance=,
						varimportance_output=,
					   prevalences=proportional,
						cv_custompriors=,
					   gshare=
					   );

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;
%if %length(&gshare) %then %do;
	libname gout "&gshare";
	%let itoutlib = gout;
%end;
%else %do;
	%let itoutlib = &cvoutlib;
%end;
%put cvoutlib = &cvoutlib;
%put itoutlib = &itoutlib;
options notes;
%if &cv_alossprop ne 0.5 %then %do;
	%let cv_alossweight = Asymmetric;
%end;
%else %do;
	%let cv_alossweight = ;
%end;

* ACCUMULATE RESULTS;
* FIRST, CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..cv_baseline_preds;
	delete;
run;
%do nm=1 %to &n_models;
	data &cvoutlib..cv_m&nm._pcum;
		delete;
	run;
	data &cvoutlib..cv_m&nm._vcum;
		delete;
	run;
	%if %upcase(&trait_type) = SURVIVAL %then %do; 
		data &cvoutlib..cv_m&nm._s1cum;
			delete;
		run;	
		data &cvoutlib..cv_m&nm._s2cum;
			delete;
		run;
		data &cvoutlib..cv_m&nm._s3cum;
			delete;
		run;
	%end;
%end;
%if %upcase(&varimportance) = YES %then %do;
	data &cvoutlib..cv_vipreds_pcum;
		delete;
	run;
	data &cvoutlib..cv_vivars_pcum;
		delete;
	run;
	data &cvoutlib..importance;
		delete;
	run;
	data &cvoutlib..impfit;
		delete;
	run;
%end;

* RETRIEVE MODEL NAMES AND LABELS;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
    %let mdllabel&i = %scan(&model_labels,&i,"|");
%end;

%if %length(&gshare) %then %do;
	%put outputpath = &outputpath;
	%put gshare = &gshare;
	%if %qupcase(&outputpath) ne %qupcase(&gshare) %then %do;
		* COPY _NAMEDAT FROM itoutlib TO cvoutlib;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			data &cvoutlib.._namedat;
				set &itoutlib.._namedat;
			run;
		%end;
	%end;
%end;

%let cv_pcum_exist=0;
%let cv_vcum_exist=0;
%if %upcase(&trait_type) = SURVIVAL %then %do; 
	%let cv_s1cum_exist=0;
	%let cv_s2cum_exist=0;
	%let cv_s3cum_exist=0;
%end;
%*put cv_holdout_iteration_start= &cv_holdout_iteration_start;
%*put cv_holdout_iteration_end= &cv_holdout_iteration_end; 
%*put n_models= &n_models;
%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
	%do nm=1 %to &n_models;
		%if %sysfunc(exist(&itoutlib..cv_m&nm._pcum&iter)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_m&nm._pcum&iter);
			%*put nObs= &nObs;
			%*put nVars= &nVars;
			%if &nObs and &nVars %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_m&nm._pcum);
					%let _cvpcumObs_ = &nObs;
					data &cvoutlib..cv_m&nm._pcum;
						set 
						%if &_cvpcumObs_ %then %do;
							&cvoutlib..cv_m&nm._pcum
						%end;
							&itoutlib..cv_m&nm._pcum&iter;
					run;
				%end;
				%else %do;
					data &cvoutlib..cv_m&nm._pcum;
						set &itoutlib..cv_m&nm._pcum&iter;
					run;
				%end;
				%let cv_pcum_exist=1;
			%end;
		%end;
		%if %sysfunc(exist(&itoutlib..cv_m&nm._vcum&iter)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_m&nm._vcum&iter);
			%if &nObs and &nVars %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_m&nm._vcum);
					%let _cvvcumObs_ = &nObs;
					data &cvoutlib..cv_m&nm._vcum;
						set 
						%if &_cvvcumObs_ %then %do;
							&cvoutlib..cv_m&nm._vcum
						%end;
							&itoutlib..cv_m&nm._vcum&iter;
					run;
				%end;
				%else %do;
					data &cvoutlib..cv_m&nm._vcum;
						set &itoutlib..cv_m&nm._vcum&iter;
					run;
				%end;
				%let cv_vcum_exist=1;
			%end;
		%end;
		%if %upcase(&trait_type) = SURVIVAL %then %do; 
			%if %sysfunc(exist(&itoutlib..cv_m&nm._s1cum&iter)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_m&nm._s1cum&iter);
				%if &nObs and &nVars %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_m&nm._s1cum)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_m&nm._s1cum);
						%let _cvs1cumObs_ = &nObs;
						data &cvoutlib..cv_m&nm._s1cum;
							set 
							%if &_cvs1cumObs_ %then %do;
								&cvoutlib..cv_m&nm._s1cum
							%end;
								&itoutlib..cv_m&nm._s1cum&iter;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_m&nm._s1cum;
							set &itoutlib..cv_m&nm._s1cum&iter;
						run;
					%end;
					%let cv_s1cum_exist=1;
				%end;
			%end;
			%if %sysfunc(exist(&itoutlib..cv_m&nm._s2cum&iter)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_m&nm._s2cum&iter);
				%if &nObs and &nVars %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_m&nm._s2cum)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_m&nm._s2cum);
						%let _cvs2cumObs_ = &nObs;
						data &cvoutlib..cv_m&nm._s2cum;
							set 
							%if &_cvs2cumObs_ %then %do;
								&cvoutlib..cv_m&nm._s2cum
							%end;
								&itoutlib..cv_m&nm._s2cum&iter;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_m&nm._s2cum;
							set &itoutlib..cv_m&nm._s2cum&iter;
						run;
					%end;
					%let cv_s2cum_exist=1;
				%end;
			%end;
			%if %sysfunc(exist(&itoutlib..cv_m&nm._s3cum&iter)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_m&nm._s3cum&iter);
				%if &nObs and &nVars %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_m&nm._s3cum)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_m&nm._s3cum);
						%let _cvs3cumObs_ = &nObs;
						data &cvoutlib..cv_m&nm._s3cum;
							set 
							%if &_cvs3cumObs_ %then %do;
								&cvoutlib..cv_m&nm._s3cum
							%end;
								&itoutlib..cv_m&nm._s3cum&iter;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_m&nm._s3cum;
							set &itoutlib..cv_m&nm._s3cum&iter;
						run;
					%end;
					%let cv_s3cum_exist=1;
				%end;
			%end;
		%end;
	%end;
	%if %upcase(&varimportance) = YES %then %do;
		%if %sysfunc(exist(&itoutlib..cv_vipreds_pcum&iter)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_vipreds_pcum&iter);
			%if &nObs and &nVars %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_vipreds_pcum)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_vipreds_pcum);
					%let _cvipcumObs_ = &nObs;
					data &cvoutlib..cv_vipreds_pcum;
						set 
						%if &_cvipcumObs_ %then %do;
							&cvoutlib..cv_vipreds_pcum
						%end;
							&itoutlib..cv_vipreds_pcum&iter;
					run;
				%end;
				%else %do;
					data &cvoutlib..cv_vipreds_pcum;
						set &itoutlib..cv_vipreds_pcum&iter;
					run;
				%end;
			%end;
		%end;
		%if %sysfunc(exist(&itoutlib..cv_vivars_pcum&iter)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_vivars_pcum&iter);
			%if &nObs and &nVars %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_vivars_pcum)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_vivars_pcum);
					%let _cvipcumObs_ = &nObs;
					data &cvoutlib..cv_vivars_pcum;
						set 
						%if &_cvipcumObs_ %then %do;
							&cvoutlib..cv_vivars_pcum
						%end;
							&itoutlib..cv_vivars_pcum&iter;
					run;
				%end;
				%else %do;
					data &cvoutlib..cv_vivars_pcum;
						set &itoutlib..cv_vivars_pcum&iter;
					run;
				%end;
			%end;
		%end;
	%end;
%end;

* SAVE OVERALL MODELS PREDICTIONS;
* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..&prefix._cvr_all;
	delete;
run;
%if &cv_pcum_exist=0 %then %do;
	%let exiterror = 1;
	%put ERROR: FAIL TO GENERATE ALL MODELS PREDICTIONS.;
	%goto cv_exit;
%end;
data &cvoutlib..&prefix._cvr_all;
    set 
    %do nm=1 %to &n_models;
		%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
			%NObsVars(&cvoutlib..cv_m&nm._pcum)
			%if &nObs and &nVars %then %do;
		        &cvoutlib..cv_m&nm._pcum
			%end;
		%end;
    %end;
    ;
run;
%if (&CV_ListAll = Yes) %then %do;
    %do nm=1 %to &n_models;
		%if %sysfunc(exist(&cvoutlib..cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_pcum)) = 1 %then %do;
			%NObsVars(&cvoutlib..cv_%substr(&mdlname,1,%sysfunc(min(%length(&mdlname),23)))_pcum);
			%if &nObs and &nVars %then %do;
		        proc contents data=&cvoutlib..cv_m&nm._pcum;
		        run;
			%end;
		%end;
    %end;
%end;
%if %upcase(&trait_type) = SURVIVAL %then %do; 
	* SAVE OVERALL MODELS SURVIVAL CURVES;
	* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
	data &cvoutlib..&prefix._spm1all;
		delete;
	run;
	data &cvoutlib..&prefix._spm2all;
		delete;
	run;
	data &cvoutlib..&prefix._spm3all;
		delete;
	run;
	%*put cv_s1cum_exist= &cv_s1cum_exist;
	%*put cv_s2cum_exist= &cv_s2cum_exist;
	%*put cv_s3cum_exist= &cv_s3cum_exist;
	%if &cv_s1cum_exist=0 and &cv_s2cum_exist=0 and &cv_s3cum_exist=0 %then %do;
		%let exiterror = 1;
		%*put ERROR: FAIL TO GENERATE ALL MODELS SURVIVAL CURVES.;
		%put ERROR: Cross validation cannot be run when Proportional Hazards Regression is specified among another other models in the Models tab.;		
		%goto cv_exit;
	%end;
	data &cvoutlib..&prefix._spm1all;
	    set 
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(&cvoutlib..cv_m&nm._s1cum)) = 1 %then %do;
				%NObsVars(&cvoutlib..cv_m&nm._s1cum)
				%if &nObs and &nVars %then %do;
			        &cvoutlib..cv_m&nm._s1cum
				%end;
			%end;
	    %end;
	    ;
		drop RANDSEED;
	run;
	data &cvoutlib..&prefix._spm2all;
	    set 
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(&cvoutlib..cv_m&nm._s2cum)) = 1 %then %do;
				%NObsVars(&cvoutlib..cv_m&nm._s2cum)
				%if &nObs and &nVars %then %do;
			        &cvoutlib..cv_m&nm._s2cum
				%end;
			%end;
	    %end;
	    ;
		drop RANDSEED;
	run;
	data &cvoutlib..&prefix._spm3all;
	    set 
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(&cvoutlib..cv_m&nm._s3cum)) = 1 %then %do;
				%NObsVars(&cvoutlib..cv_m&nm._s3cum)
				%if &nObs and &nVars %then %do;
			        &cvoutlib..cv_m&nm._s3cum
				%end;
			%end;
	    %end;
	    ;
		drop RANDSEED;
	run;
%end;

* COPY _NAMEDAT FROM itoutlib to cvoutlib;
%if %upcase(&trait_type)=QUALITATIVE and %length(&gshare) %then %do;
	data &cvoutlib.._namedat;
		set &itoutlib.._namedat;
	run;
%end;

* COMPUTE EACH CV ITERATION MODELS ASE, ACC, AUC, HARRELL, CORR;
* FIRST, CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
%do nm=1 %to &n_models;
	data cv_m&nm._acc;
		delete;
	run;
	data cv_m&nm._mann;
		delete;
	run;
	data cv_m&nm._ase;
		delete;
	run;
	data cv_m&nm._hac;
		delete;
	run;
	data cv_m&nm._corr;
		delete;
	run;
%end;
data cv_baseline;
	delete;
run;
data cv_baseline_acc;
	delete;
run;
data cv_baseline_aec;
	delete;
run;
data cv_baseline_ase;
	delete;
run;
data cv_baseline_mann;
	delete;
run;
data cv_baseline_hac;
	delete;
run;
data cv_baseline_corr;
	delete;
run;
%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
	
	%if %sysfunc(exist(&itoutlib..cv_baseline_preds&iter)) = 1 %then %do;
		%NObsVars(&itoutlib..cv_baseline_preds&iter);
		%if &nObs and &nVars %then %do; 
			%let predObs = &nObs;
			* COMPUTE OVERALL CV ITERATIONS MODELS FIT STATISTICS: ASE, ACC, AEC, AUC;
			%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do; 
				%if %upcase(&trait_type)=QUALITATIVE %then %do;
					
				     * COMPUTE OVERALL BASELINE ASE;
				    %compute_ase_nominal(data=&itoutlib..cv_baseline_preds&iter,
				                         output=baseline_ase,
				                         actual=actualp,
				                         nclasses=&nclasses,
				                         mdlname=BASELINE,
				                         freq=&freq,
				                         prevalences=&prevalences,
										 cv_custompriors=%nrbquote(&cv_custompriors),
									     trait=&trait
				                         );
					%put BEFORE BASELINE COMPUTE_ACC_NOMINAL;
				    %compute_acc_nominal(data=&itoutlib..cv_baseline_preds&iter,
				                         output=baseline_acc,
				                         trait=&trait,
				                         correct=correct,
				                         mdlname=BASELINE,
				                         freq=&freq,
				                         prevalences=&prevalences,
										 cv_custompriors=%nrbquote(&cv_custompriors)
				                         );
					%put AFTER BASELINE COMPUTE_ACC_NOMINAL;

					%if %sysfunc(exist(baseline_acc)) = 1 %then %do;
						%NObsVars(baseline_acc);
						%let _baccObs_ = &nObs;
						%if &_baccObs_ %then %do;
							data baseline_acc;
								length METHOD $64 ITER 8;
								set baseline_acc;
								array nb NOBS_:;
								NOBS=0;
								METHOD="BASELINE";
								ITER=&iter;
								do n=1 to &nclasses;
									NOBS=NOBS+nb(n);
								end;
								drop n;
							run;

							* APPEND BASELINE ACC OVER EACH CV ITERATION;
							%if %sysfunc(exist(cv_baseline_acc)) = 1 %then %do;
								%NObsVars(cv_baseline_acc);
								%let _cvbaccObs_ = &nObs;
								data cv_baseline_acc;
									set 
									%if &_cvbaccObs_ %then %do;
										cv_baseline_acc
									%end;
									baseline_acc;
								run;
							%end;
							%else %do;
								data cv_baseline_acc;
									set baseline_acc;
								run;
							%end;
						%end; 
					%end;

					%put cv_customcosts=&cv_customcosts length=%length(&cv_customcosts);
					%if %length(&cv_customcosts) %then %do;
										
						data outlib.bpreds&iter;
							set &itoutlib..cv_baseline_preds&iter;
						run;
					
						%compute_aec_nominal(
							 data=&itoutlib..cv_baseline_preds&iter,
							 output=baseline_aec,
							 trait=&trait,
							 expcost=expected_cost,
							 mdlname=BASELINE,
							 freq=&freq
						);
						%if %sysfunc(exist(baseline_aec)) = 1 %then %do;
							%NObsVars(baseline_aec);
							%let _baecObs_ = &nObs;
							%if &_baecObs_ %then %do;
								data baseline_aec;
									length METHOD $64 ITER 8;
									set baseline_aec;
									array nb NOBS_:;
									NOBS=0;
									METHOD="BASELINE";
									ITER=&iter;
									do n=1 to &nclasses;
										NOBS=NOBS+nb(n);
									end;
									drop n;
								run;

								* APPEND BASELINE AEC OVER EACH CV ITERATION;
								%if %sysfunc(exist(cv_baseline_aec)) = 1 %then %do;
									%NObsVars(cv_baseline_aec);
									%let _cvbaecObs_ = &nObs;
									data cv_baseline_aec;
										set 
										%if &_cvbaecObs_ %then %do;
											cv_baseline_aec
										%end;
										baseline_aec;
									run;
								%end;
								%else %do;
									data cv_baseline_aec;
										set baseline_aec;
									run;
								%end;
							%end; 
						%end;
						%let cv_baseline_aec = cv_baseline_aec;
					%end;
					%else %do;
						%let cv_baseline_aec = ;
					%end;

				    %if &nclasses = 2 %then %do;
						
				        %locate_pevent_isevent(data=&itoutlib..cv_baseline_preds&iter,
											   trait=&trait,
											   event=&event,
											   actual=actualp);

				        * COMPUTE OVERALL BASELINE MANN-WHITNEY STATISTIC;
				        %compute_mann(data=&itoutlib..cv_baseline_preds&iter,
				                      pevent=_pevent,
				                      isevent=_isevent);
				        %put mann=&mann;
				        data baseline_mann;
				            length METHOD $64 ITER 8;
				            METHOD="BASELINE";
							AUC=&mann;
							ITER=&iter;
							NOBS=&predObs;
				        run;

						* APPEND BASELINE MANN-WHITNEY AUC OVER EACH CV ITERATIONS;
						%NObsVars(baseline_mann);
						%let _bmannObs_ = &nObs;
						%if &_bmannObs_ %then %do;
							
							%if %sysfunc(exist(cv_baseline_mann)) = 1 %then %do;
								%NObsVars(cv_baseline_mann);
								%let _cvbmannObs_ = &nObs;
								data cv_baseline_mann;
									set 
									%if &_cvbmannObs_ %then %do;
										cv_baseline_mann
									%end;
									baseline_mann;
								run;
							%end;
							%else %do;
								data cv_baseline_mann;
									set baseline_mann;
								run;
							%end;
						%end;
				    %end;
				%end;
				%else %if %upcase(&trait_type)=QUANTITATIVE %then %do; * TRAIT_TYPE = QUANTITATIVE;

					%if &cv_alossprop ne 0.5 %then %do;
						data &itoutlib..cv_baseline_preds&iter;
							set &itoutlib..cv_baseline_preds&iter;
							_resid = &trait - Predicted_Value;
							if missing(_resid) then &cv_alossweight = .;
							else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
							else &cv_alossweight = 2 * (1 - &cv_alossprop);
						run;
					%end;

				   * COMPUTE OVERALL BASELINE ASE;
				   %compute_ase_interval(data=&itoutlib..cv_baseline_preds&iter,
				                         output=baseline_ase,
				                         y=&trait,
				                         predict=Predicted_Value,
				                         mdlname=BASELINE,
				                         freq=&freq,
				                         aweight=&cv_alossweight);

					%compute_harrellc(data=&itoutlib..cv_baseline_preds&iter,
									  output=baseline_hac,
									  TimeVar=&trait,
									  CenVar=,
									  Predict=Predicted_Value,
									  Freq=&cv_alossweight);

				   %compute_corr_interval(data=&itoutlib..cv_baseline_preds&iter,
				                         output=baseline_corr,
				                         y=&trait,
				                         predict=Predicted_Value,
				                         mdlname=BASELINE,
				                         freq=&freq,
				                         weight=&cv_alossweight);
				                         
					* APPEND BASELINE HARRELL OVER EACH CV ITERATIONS;
					%NObsVars(baseline_hac);
					%let _bhacObs_ = &nObs;
					%if &_bhacObs_ %then %do;

						data baseline_hac;
							length METHOD $64 ITER 8;
							set baseline_hac;
							ITER=&iter;
							METHOD="BASELINE";
						run;
						
						%if %sysfunc(exist(cv_baseline_hac)) = 1 %then %do;
							%NObsVars(cv_baseline_hac);
							%let _cvbhacObs_ = &nObs;
							data cv_baseline_hac;
								set 
								%if &_cvbhacObs_ %then %do;
									cv_baseline_hac
								%end;
								baseline_hac;
							run;
						%end;
						%else %do;
							data cv_baseline_hac;
								set baseline_hac;
							run;
						%end;
					%end;

				%end;

				%if %sysfunc(exist(baseline_ase)) = 1 %then %do;
					%NObsVars(baseline_ase);
					%let _baseObs_ = &nObs;
					%if &_baseObs_ %then %do;
						data baseline_ase;
							length METHOD $64 ITER 8;
							set baseline_ase;
							ITER=&iter;
							METHOD="BASELINE";
						run;

						* APPEND BASELINE ASE OVER CV ITERATIONS;
						%if %sysfunc(exist(cv_baseline_ase)) = 1 %then %do;
							%NObsVars(cv_baseline_ase);
							%let _cvbaseObs_ = &nObs;
							data cv_baseline_ase;
								set 
								%if &_cvbaseObs_ %then %do;
									cv_baseline_ase
								%end;
								baseline_ase;
							run;
						%end;
						%else %do;
							data cv_baseline_ase;
								set baseline_ase;
							run;
						%end;
					%end;
				%end;

				%if %sysfunc(exist(baseline_corr)) = 1 %then %do;
					%NObsVars(baseline_corr);
					%let _baseObs_ = &nObs;
					%if &_baseObs_ %then %do;
						data baseline_corr;
							length METHOD $64 ITER 8;
							set baseline_corr;
							ITER=&iter;
							METHOD="BASELINE";
						run;

						* APPEND BASELINE CORR OVER CV ITERATIONS;
						%if %sysfunc(exist(cv_baseline_corr)) = 1 %then %do;
							%NObsVars(cv_baseline_corr);
							%let _cvbaseObs_ = &nObs;
							data cv_baseline_corr;
								set 
								%if &_cvbaseObs_ %then %do;
									cv_baseline_corr
								%end;
								baseline_corr;
							run;
						%end;
						%else %do;
							data cv_baseline_corr;
								set baseline_corr;
							run;
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;

	%do nm=1 %to &n_models;
        %let mdlname=&&mdlname&nm;
		%if %sysfunc(exist(&itoutlib..cv_m&nm._pcum&iter)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_m&nm._pcum&iter);
			%let _pObs_ = &nObs;
			%if &_pObs_ %then %do;

				%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;
					* COMPUTE MODEL ASE;
				    %if %upcase(&trait_type)=QUALITATIVE %then %do;
				        %compute_ase_nominal(data=&itoutlib..cv_m&nm._pcum&iter,
				                             output=m&nm._ase,
				                             actual=actualp,
				                             nclasses=&nclasses,
				                             mdlname=&mdlname,
				                             freq=&freq,
				                             prevalences=&prevalences,
											        cv_custompriors=%nrbquote(&cv_custompriors),
								                 trait=&trait
				                             );

				        %compute_acc_nominal(data=&itoutlib..cv_m&nm._pcum&iter,
				                             output=m&nm._acc,
				                             trait=&trait,
				                             correct=correct,
				                             mdlname=&mdlname,
				                             freq=&freq,
				                             prevalences=&prevalences,
											        cv_custompriors=%nrbquote(&cv_custompriors)
				                             );

						%NObsVars(m&nm._acc);
						%let _accObs_ = &nObs;
						%if &_accObs_ %then %do; 
					        data m&nm._acc;
					            length METHOD $64 ITER 8;
					            set m&nm._acc;
					            ITER=&iter;
					            METHOD="&mdlname";
					        run; 

							* APPEND CV ITERATIONS EACH MODEL ACC;
							%if %sysfunc(exist(cv_m&nm._acc)) = 1 %then %do;
								%NObsVars(cv_m&nm._acc);
								%let _cvaccObs_ = &nObs;
								data cv_m&nm._acc;
									set 
									%if &_cvaccObs_ %then %do;
						            	cv_m&nm._acc
									%end;
							            m&nm._acc;
							    run;
							%end;
							%else %do;
								data cv_m&nm._acc;
									set m&nm._acc;
							    run;
							%end;
						%end;

						%if %length(&cv_customcosts) %then %do;
							%compute_aec_nominal(
								 data=&itoutlib..cv_m&nm._pcum&iter,
								 output=m&nm._aec,
								 trait=&trait,
								 expcost=expected_cost,
								 mdlname=&mdlname,
								 freq=&freq
							);
							%let m&nm._aec = m&nm._aec;
							%NObsVars(m&nm._aec);
							%let _aecObs_ = &nObs;
							%if &_aecObs_ %then %do; 
								  data m&nm._aec;
										length METHOD $64 ITER 8;
										set m&nm._aec;
										ITER=&iter;
										METHOD="&mdlname";
								  run; 

								* APPEND CV ITERATIONS EACH MODEL AEC;
								%if %sysfunc(exist(cv_m&nm._aec)) = 1 %then %do;
									%NObsVars(cv_m&nm._aec);
									%let _cvaecObs_ = &nObs;
									data cv_m&nm._aec;
										set 
										%if &_cvaecObs_ %then %do;
												cv_m&nm._aec
										%end;
												m&nm._aec;
									 run;
								%end;
								%else %do;
									data cv_m&nm._aec;
										set m&nm._aec;
									 run;
								%end;
							%end;
						%end;
						%else %do;
							%let m&nm._aec = ;
						%end;

						%if &nclasses = 2 %then %do;
					        %locate_pevent_isevent(data=&itoutlib..cv_m&nm._pcum&iter,
					                               trait=&trait,
					                               event=&event,
					                               actual=actualp);

					        * COMPUTE MODEL MANN;
					        %compute_mann(data=&itoutlib..cv_m&nm._pcum&iter,
					                      pevent=_pevent,
					                      isevent=_isevent);
					        %put mann=&mann;
					        data m&nm._mann;
					            length METHOD $64 ITER 8;
					            METHOD = "&mdlname";    
					            AUC=&mann;
					            ITER=&iter;
					        run;

							* APPEND CV ITERATIONS EACH MODEL MANN-WHITNEY AUC;
							%NObsVars(m&nm._mann);
							%let _mannObs_ = &nObs;
							%if &_mannObs_ %then %do;

								%if %sysfunc(exist(cv_m&nm._mann)) = 1 %then %do;
									%NObsVars(cv_m&nm._mann);
									%let _cvmannObs_ = &nObs;
						            data cv_m&nm._mann;
						                set 
										%if &_cvmannObs_ %then %do;
							            	cv_m&nm._mann
										%end;
						                    m&nm._mann;
						            run;
								%end;
								%else %do;
						            data cv_m&nm._mann;
						                set m&nm._mann;
						            run;
								%end;
							%end;
					    %end;
				    %end;
				    %else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
				    
				    	data piter;
				    		set &itoutlib..cv_m&nm._pcum&iter;
				    	run;

						%if &cv_alossprop ne 0.5 %then %do;
							data piter;
								set piter;
								_resid = &trait - Predicted_Value;
								if missing(_resid) then &cv_alossweight = .;
								else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
								else &cv_alossweight = 2 * (1 - &cv_alossprop);
							run;
						%end;

						 %compute_harrellc(data=piter,
				                         output=m&nm._hac,
										       TimeVar=&trait,
										       CenVar=&CensorVar,
										       Predict=Predicted_Value,
										       Freq=&cv_alossweight);
										       
						 * if censoring, compute ase and corr only on noncensored observations;
						 %if %length(&CensorVar) %then %do;
						 	data piter;
						 		set piter;
						 		if &CensorVar in(%unquote(&CensorValues)) then delete; 
						 	run;
						 %end;

				       %compute_ase_interval(data=piter,
				                             output=m&nm._ase,
				                             y=&trait,
				                             predict=Predicted_Value,
				                             mdlname=m&nm,
				                             freq=&freq,
				                             aweight=&cv_alossweight);


				       %compute_corr_interval(data=piter,
				                             output=m&nm._corr,
				                             y=&trait,
				                             predict=Predicted_Value,
				                             mdlname=m&nm,
				                             freq=&freq,
				                             weight=&cv_alossweight);

						* APPEND CV ITERATIONS EACH MODEL HARRELL;
						%NObsVars(m&nm._hac);
						%let _hacObs_ = &nObs;
						%if &_hacObs_ %then %do;

							data m&nm._hac; 
								length METHOD $64 ITER 8;
								set m&nm._hac;
								ITER=&iter;
								METHOD="&mdlname";
						    run;

							%if %sysfunc(exist(cv_m&nm._hac)) = 1 %then %do;
								%NObsVars(cv_m&nm._hac);
								%let _cvhacObs_ = &nObs;
					            data cv_m&nm._hac;
					                set 
									%if &_cvhacObs_ %then %do;
						            	cv_m&nm._hac
									%end;
					                    m&nm._hac;
					            run;
							%end;
							%else %do;
					            data cv_m&nm._hac;
					                set m&nm._hac;
					            run;
							%end;
						%end;

				    %end;

					%if %sysfunc(exist(m&nm._ase)) = 1 %then %do;
						%NObsVars(m&nm._ase);
						%let _aseObs_ = &nObs;
						%if &_aseObs_ %then %do;
						    data m&nm._ase; 
								length METHOD $64 ITER 8;
								set m&nm._ase;
								ITER=&iter;
								METHOD="&mdlname";
						    run;

							%if %sysfunc(exist(cv_m&nm._ase)) = 1 %then %do;
								* APPEND CV ITERATIONS EACH MODEL ASE;
								%NObsVars(cv_m&nm._ase);
								%let _cvaseObs_ = &nObs;
							    data cv_m&nm._ase;
							        set
									%if &_cvaseObs_ %then %do;
						            	cv_m&nm._ase
						        	%end;
							            m&nm._ase;
							    run;
							%end;
							%else %do;
							    data cv_m&nm._ase;
							        set m&nm._ase;
							    run;
							%end;
						%end;
					%end;

					%if %sysfunc(exist(m&nm._corr)) = 1 %then %do;
						%NObsVars(m&nm._corr);
						%let _corrObs_ = &nObs;
						%if &_corrObs_ %then %do;
						    data m&nm._corr; 
								length METHOD $64 ITER 8;
								set m&nm._corr;
								ITER=&iter;
								METHOD="&mdlname";
						    run;

							%if %sysfunc(exist(cv_m&nm._corr)) = 1 %then %do;
								* APPEND CV ITERATIONS EACH MODEL CORR;
								%NObsVars(cv_m&nm._corr);
								%let _cvaseObs_ = &nObs;
							    data cv_m&nm._corr;
							        set
									%if &_cvaseObs_ %then %do;
						            	cv_m&nm._corr
						        	%end;
							            m&nm._corr;
							    run;
							%end;
							%else %do;
							    data cv_m&nm._corr;
							        set m&nm._corr;
							    run;
							%end;
						%end;
					%end;
					
					
				%end;
				%else %do;
					* COMPUTE SURVIVAL AUC;
					data surf;
						set &itoutlib..cv_m&nm._pcum&iter;
						if _train_ = 0 then output; /* KEEP TEST SET PREDICTIONS */
					run;
					%NObsVars(surf);
					%let nsurf = &nObs;
					%if &nsurf %then %do;
						%compute_survival_auc(data=surf,
											  output=m&nm._auc&iter,
											  TimeVar=&trait,
											  Cenvar=&CVarH);
						
						%NObsVars(m&nm._auc&iter);
						%if &nObs and &nVars %then %do;
							data m&nm._auc&iter;
								length METHOD $64 ITER 8;
					            set m&nm._auc&iter;
					            ITER=&iter;
					            METHOD="&mdlname";
					        run; 

							* APPEND CV ITERATIONS EACH MODEL HARRELL CONCORDANCE INDEX;
							%if %sysfunc(exist(cv_m&nm._auc)) = 1 %then %do;
								%NObsVars(cv_m&nm._auc);
								%let _cvaucObs_ = &nObs;
								data cv_m&nm._auc;
									set 
									%if &_cvaucObs_ %then %do;
							           	cv_m&nm._auc
									%end;
								        m&nm._auc&iter;
								run;
							%end;
							%else %do;
								data cv_m&nm._auc;
									set m&nm._auc&iter;
							    run;
							%end;

							/*							
							libname PP "P:\Prostate Cancer\CVMC\SPMTest";
							data pp.m&nm._auc&iter; set m&nm._auc&iter; run;
							%put ERROR: 4.********** data = m&nm._auc&iter;	
							%put ERROR: %sysfunc(pathname(&cvoutlib));
							%put ERROR: %sysfunc(pathname(&itoutlib));
							%put ERROR: &RefTime, &cv_RefTime_m1_r1 &cv_RefTime_m1_r2, nRefTime=&&cv_nRefTime_m&Workflowrp;
							*/

							%compute_harrellc(data=m&nm._auc&iter,
											  output=m&nm._hc&iter,
											  TimeVar=&trait,
											  CenVar=&CVarH,
											  Predict=_AUC_);
											  
							%if %length(&&cv_MinRefTime_m&nm) & %length(&&cv_MaxRefTime_m&nm) %then %do;
								%compute_Time_Dep_AUC(Data=&itoutlib..time_dep_auc_m&nm._i&iter,
											  Output=iAUC_m&nm._i&iter,
											  TimeVar=Org_&trait,
											  CenVar=&CensorVar,
											  CenVal=&CensorValues,
											  Model=&mdlname,
											  ModleNo=&nm);

								%if &nm=1 & &iter = 1 %then %do;
									data &cvoutlib..cv_iAUC;
										set iAUC_m&nm._i&iter;
									run;
								%end;
								%else %do;
									data &cvoutlib..cv_iAUC;
										set &cvoutlib..cv_iAUC iAUC_m&nm._i&iter;
									run;
								%end;
							%end;

							%NObsVars(m&nm._hc&iter);
							%if &nObs and &nVars %then %do;
								data m&nm._hc&iter;
									length METHOD $64 ITER 8;
						            set m&nm._hc&iter;
						            ITER=&iter;
						            METHOD="&mdlname";
						        run; 

								* APPEND CV ITERATIONS EACH MODEL HARRELL CONCORDANCE INDEX;
								%if %sysfunc(exist(cv_m&nm._hc)) = 1 %then %do;
									%NObsVars(cv_m&nm._hc);
									%let _cvhcObs_ = &nObs;
									data cv_m&nm._hc;
										set 
										%if &_cvhcObs_ %then %do;
								           	cv_m&nm._hc
										%end;
									        m&nm._hc&iter;
									run;
								%end;
								%else %do;
									data cv_m&nm._hc;
										set m&nm._hc&iter;
								    run;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end; 
		%end;
    %end; * END OF N_MODELS LOOP;
%end; * END OF CV_HOLDOUT_ITERATION LOOP;

* APPEND OVERALL BASELINE ASE WITH EACH CV ITERATION MODELS ASE;
* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..&models_comparison_output;
	delete;
run;

%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;
	data cv_baseline;
		set cv_baseline_ase;
		RMSE = sqrt(ASE);
	run;
	data &cvoutlib..&models_comparison_output;
	    set
	    /* BASELINE_ASE	*/
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(cv_m&nm._ase)) = 1 %then %do;
				%NObsVars(cv_m&nm._ase)
				%if &nObs and &nVars %then %do;
	           		cv_m&nm._ase
				%end;
			%end;
	    %end;
	    ;
	    RMSE = sqrt(ASE);
	run;
%end;
%else %do;
	data cv_baseline;
		_AUC_ = 0.5;
	run;
	data &cvoutlib..&models_comparison_output;
	    set
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(cv_m&nm._hc)) = 1 %then %do;
				%NObsVars(cv_m&nm._hc)
				%if &nObs and &nVars %then %do;
	           		cv_m&nm._hc
				%end;
			%end;
	    %end;
	    ;
	run;

	data &cvoutlib..&prefix._auc_all;
		 set
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(cv_m&nm._auc)) = 1 %then %do;
				%NObsVars(cv_m&nm._auc)
				%if &nObs and &nVars %then %do;
	           		cv_m&nm._auc
				%end;
			%end;
	    %end;
	    ;
	run;
%end;

%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;

	%if %upcase(&trait_type)=QUALITATIVE %then %do;

		data cv_baseline;
			merge cv_baseline cv_baseline_acc;
		run;
		
		* APPEND OVERALL BASELINE ACC WITH ALL CV ITERATIONS MODELS ACC;
		* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
		data &models_comparison_output._ACC;
			delete;
		run;
	    data &models_comparison_output._ACC;
	        set
	        /* BASELINE_ACC	*/
	        %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._acc)) = 1 %then %do;
					%NObsVars(cv_m&nm._acc)
					%if &nObs and &nVars %then %do;
					    cv_m&nm._acc
					%end;
				%end;
	        %end;
	        ;
	    run;

		* MERGE ALL CV ITERATIONS MODELS ACC, ASE, AND OVERALL BASELINE ASE;
		%if %sysfunc(exist(&models_comparison_output._ACC)) = 1 %then %do;
			%NObsVars(&models_comparison_output._ACC);
			%if &nObs and &nVars %then %do;
				data &cvoutlib..&models_comparison_output;
			        merge &cvoutlib..&models_comparison_output
			              &models_comparison_output._ACC;
			    run;
			%end;
		%end;
		
		%if %length(&cv_customcosts) %then %do;

			data cv_baseline;
				merge cv_baseline cv_baseline_aec;
			run;
			
			* APPEND OVERALL BASELINE AEC WITH ALL CV ITERATIONS MODELS AEC;
			* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
			data &models_comparison_output._aec;
				delete;
			run;
			 data &models_comparison_output._aec;
				  set
				  /* BASELINE_aec	*/
				  %do nm=1 %to &n_models;
					%if %sysfunc(exist(cv_m&nm._aec)) = 1 %then %do;
						%NObsVars(cv_m&nm._aec)
						%if &nObs and &nVars %then %do;
							 cv_m&nm._aec
						%end;
					%end;
				  %end;
				  ;
			 run;

			* MERGE ALL CV ITERATIONS MODELS aec, ASE, AND OVERALL BASELINE ASE;
			%if %sysfunc(exist(&models_comparison_output._aec)) = 1 %then %do;
				%NObsVars(&models_comparison_output._aec);
				%if &nObs and &nVars %then %do;
					data &cvoutlib..&models_comparison_output;
						  merge &cvoutlib..&models_comparison_output
								  &models_comparison_output._aec;
					 run;
				%end;
			%end;
	
		%end;

	    %if &nclasses = 2 %then %do;

			data cv_baseline;
				merge cv_baseline cv_baseline_mann;
			run;

			* APPEND OVERALL BASELINE MANN-WHITNEY AUC WITH ALL CV ITERATIONS MODELS AUC;
			* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
			data &models_comparison_output._AUC;
				delete;
			run;
		    data &models_comparison_output._AUC;
		        set
	            /* BASELINE_MANN */
	            %do nm=1 %to &n_models;
					%if %sysfunc(exist(cv_m&nm._mann)) = 1 %then %do;
						%NObsVars(cv_m&nm._mann)
						%if &nObs and &nVars %then %do;
		                	cv_m&nm._mann
						%end;
					%end;
	            %end;
	            ;
		    run;
			* MERGE ALL CV ITERATIONS MODELS AUC, ACC, ASE, AND OVERALL BASELINE ASE;
			%if %sysfunc(exist(&models_comparison_output._AUC)) = 1 %then %do;
				%NObsVars(&models_comparison_output._AUC);
				%if &nObs and &nVars %then %do;
				    data &cvoutlib..&models_comparison_output;
				        merge &cvoutlib..&models_comparison_output
				              &models_comparison_output._AUC;
				    run;
				%end;
			%end;
		%end;
	%end;
	%else %do;
		
		data cv_baseline;
			merge cv_baseline cv_baseline_hac cv_baseline_corr;
		run;
		
		* APPEND OVERALL BASELINE HARRELL WITH ALL CV ITERATIONS MODELS HARRELL;
		* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
		data &models_comparison_output._HAC;
			delete;
		run;
	   data &models_comparison_output._HAC;
	      set
	      /* BASELINE_HARRELL */
	      %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._hac)) = 1 %then %do;
					%NObsVars(cv_m&nm._hac)
					%if &nObs and &nVars %then %do;
					    cv_m&nm._hac
					%end;
				%end;
	      %end;
	      ;
	   run;

		* APPEND OVERALL BASELINE CORR WITH ALL CV ITERATIONS MODELS HARRELL;
		* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
		data &models_comparison_output._CORR;
			delete;
		run;
	   data &models_comparison_output._CORR;
	      set
	      /* BASELINE_CORR */
	      %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._hac)) = 1 %then %do;
					%NObsVars(cv_m&nm._hac)
					%if &nObs and &nVars %then %do;
					    cv_m&nm._corr
					%end;
				%end;
	      %end;
	      ;
	   run;

		* MERGE ALL CV ITERATIONS MODELS HARRELL;
		%if %sysfunc(exist(&models_comparison_output._HAC)) = 1 %then %do;
			%NObsVars(&models_comparison_output._HAC);
			%if &nObs and &nVars %then %do;
				data &cvoutlib..&models_comparison_output;
			        merge &cvoutlib..&models_comparison_output
			              &models_comparison_output._HAC;
			    run;
			%end;
		%end;

		* MERGE ALL CV ITERATIONS MODELS CORR;
		%if %sysfunc(exist(&models_comparison_output._CORR)) = 1 %then %do;
			%NObsVars(&models_comparison_output._CORR);
			%if &nObs and &nVars %then %do;
				data &cvoutlib..&models_comparison_output;
			        merge &cvoutlib..&models_comparison_output
			              &models_comparison_output._CORR;
			    run;
			%end;
		%end;

	%end;

	* COMPUTE BASELINE ESTIMATES MEDIAN, UPPER AND LOWER WHISKERS OVER CV ITERATIONS;
	%if %upcase(&trait_type)=QUALITATIVE %then %do;
		%if %length(&cv_customcosts) %then %do;
			%let aec = AEC;
		%end;
		%else %do;
			%let aec = ;
		%end;
		%if &nclasses = 2 %then %do;
			%compute_whiskers(data=cv_baseline,
						     output=&cvoutlib..baseline_pctl,
							  method=BASELINE,
							  freq=NOBS,
							  varlist=RMSE Accuracy &aec AUC);
		%end;
		%else %do;
			%compute_whiskers(data=cv_baseline,
						      output=&cvoutlib..baseline_pctl,
							  method=BASELINE,
							  freq=NOBS,
							  varlist=RMSE Accuracy &aec);
		%end;
	%end;
	%else %do;
			%compute_whiskers(data=cv_baseline,
						     output=&cvoutlib..baseline_pctl,
							  method=BASELINE,
							  freq=NOBS,
							  varlist=RMSE HARRELL CORR);
	%end;

	%if %upcase(&trait_type)=QUALITATIVE %then %do;

		* CREATE FREQUENCY VARIABLE AS NOBS MIGHT VARY OVER EACH CV ITERATION;
		%if %sysfunc(exist(&cvoutlib..&models_comparison_output)) = 1 %then %do;
			%NObsVars(&cvoutlib..&models_comparison_output);
			%if &nObs and &nVars %then %do;
				data _null_;
					set &cvoutlib..baseline_pctl;
					call symput("_TOTALNOBS", trim(left(NOBS)));
				run;
				%let NITERS = %eval(&cv_holdout_iteration_end - &cv_holdout_iteration_start + 1);
				data &cvoutlib..&models_comparison_output;
					set &cvoutlib..&models_comparison_output;
					* _FREQ = NOBS * &NITERS / &_TOTALNOBS;
					drop NOBSALL;
				run;
			%end;
		%end;
	%end;

%end;

* merge in model labels;
data lab;
	length Model $ 64 Method $ 32;
	%do i=1 %to &n_models;
		Model = "&&mdllabel&i";
		Method = "&&mdlname&i";
		output;
	%end;
run;
	
data &cvoutlib..&models_comparison_output;
	merge lab &cvoutlib..&models_comparison_output;
	by Method;
run;

%if %upcase(&varimportance) = YES %then %do;
	%if %upcase(&trait_type)=QUALITATIVE or %upcase(&trait_type)=QUANTITATIVE %then %do;

		%let impfit_niters = 0;
		data &cvoutlib..impfit;
			merge
			%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
				%do k=1 %to &kfold;
					%if %sysfunc(exist(&itoutlib..impfit&iter&k)) = 1 %then %do;
						%NObsVars(&itoutlib..impfit&iter&k)
						%if &nObs and &nVars %then %do;
							&itoutlib..impfit&iter&k
							%let impfit_niters = %sysevalf(&impfit_niters + 1, int); 
						%end;
					%end;
				%end;
			%end; 
			;
			%if &impfit_niters > 0 %then %do;
				by _INPUT1_;
			%end;
		run;

		%if %sysfunc(exist(&cvoutlib..impfit)) = 1 %then %do;
			%NObsVars(&cvoutlib..impfit);
			%if &nObs and &nVars %then %do;
				data &cvoutlib..impfit;
					set &cvoutlib..impfit;
					_TASE_MEAN = mean( 
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
						%do k=1 %to &kfold;
							%if &iter ne &cv_holdout_iteration_end or &k ne &kfold %then %do;
								_TASE_&iter&k, 
							%end;
							%else %do;
								_TASE_&iter&k
							%end;
						%end;
					%end;
					)
					;
					_TASE_MEDIAN = median(
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
						%do k=1 %to &kfold;
							%if &iter ne &cv_holdout_iteration_end or &k ne &kfold %then %do;
								_TASE_&iter&k, 
							%end;
							%else %do;
								_TASE_&iter&k
							%end;
						%end;
					%end;
					)
					;
				run;
				proc sort data=&cvoutlib..impfit;
					by descending _TASE_MEDIAN;
				run;
			%end;
		%end;

		%if &n_models ne 1 %then %do;
			%put ERROR: MORE THAN ONE MODEL IS SPECIFIED.;
			%let exiterror = 1;
			%goto cv_exit;
		%end;

		%if %sysfunc(exist(&cvoutlib..&models_comparison_output)) = 1 %then %do;
			%NObsVars(&cvoutlib..&models_comparison_output);
			%if ^&nObs or ^&nVars %then %do;
				%put ERROR: &cvoutlib..&models_comparison_output CONTAINS 0 OBSERVATIONS.;
				%let exiterror = 1;
				%goto cv_exit;	
			%end;
		%end;
		%else %do;
			%put ERROR: &cvoutlib..&models_comparison_output DOES NOT EXIST.;
			%let exiterror = 1;
			%goto cv_exit;	
		%end;

		* SAVE OVERALL CV RUNS %LOOP_PREDS PREDICTIONS;
		* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
		data &cvoutlib..&prefix._vir_all;
			delete;
		run;
		data vi_auc;
			delete;
		run;
		data vi_acc;
			delete;
		run;
		data vi_aec;
			delete;
		run;
		data vi_ase;
			delete;
		run;
		data vi_hac;
			delete;
		run;
		data vi_corr;
			delete;
		run;
		data &varimportance_output._ased;
			delete;
		run;
		data &varimportance_output._accd;
			delete;
		run;
		data &varimportance_output._aecd;
			delete;
		run;
		data &varimportance_output._aucd;
			delete;
		run;
		data &varimportance_output._hacd;
			delete;
		run;
		data &varimportance_output._corrd;
			delete;
		run;
		data &cvoutlib..&varimportance_output._fitd;
			delete;
		run;

		%if %sysfunc(exist(&cvoutlib..cv_vipreds_pcum)) = 1 %then %do;
			%NObsVars(&cvoutlib..cv_vipreds_pcum);
			%let _pcumObs_ = &nObs;
			%if &_pcumObs_ %then %do;
				data &cvoutlib..&prefix._vir_all;
				    set &cvoutlib..cv_vipreds_pcum;
				run; 

				%let mdlname=&mdlname1;

				* CALCULATE VARIABLE IMPORTANCE;
				* USE ASE_V - ASE_full AS MEASURE OF IMPORTANCE OF VARIABLE;

				* COMPUTE EACH CV ITERATION MODELS ASE, ACC, AUC;
				%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
					data _null_;
						set &cvoutlib..&models_comparison_output;
						if METHOD = "&mdlname" and ITER = &iter then
							call symput("ASE_FULL", trim(left(put(ASE, best12.))));
					run;

					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						data _null_;
							set &cvoutlib..&models_comparison_output;
							if METHOD = "&mdlname" and ITER = &iter then
								call symput("ACC_FULL", trim(left(put(Accuracy, best12.))));
						run;
						%if %length(&cv_customcosts) %then %do;
							data _null_;
								set &cvoutlib..&models_comparison_output;
								if METHOD = "&mdlname" and ITER = &iter then
									call symput("AEC_FULL", trim(left(put(AEC, best12.))));
							run;
						%end;
						%if &nclasses = 2 %then %do;
							data _null_;
								set &cvoutlib..&models_comparison_output;
								if METHOD = "&mdlname" and ITER = &iter then
									call symput("AUC_FULL", trim(left(put(AUC, best12.))));
							run;
						%end;
					%end;
					%else %do;
						data _null_;
							set &cvoutlib..&models_comparison_output;
							if METHOD = "&mdlname" and ITER = &iter then
								call symput("HAC_FULL", trim(left(put(HARRELL, best12.))));
						run;
						data _null_;
							set &cvoutlib..&models_comparison_output;
							if METHOD = "&mdlname" and ITER = &iter then
								call symput("CORR_FULL", trim(left(put(CORR, best12.))));
						run;
					%end;

					%NObsVars(&itoutlib..cv_vipreds_pcum&iter);
					%let _iteriObs_ = &nObs;
					%if &_iteriObs_ %then %do;
						proc freq data=&itoutlib..cv_vipreds_pcum&iter noprint;                                                                                                         
							tables dropvar*droplabel/out=itervi;                                                                                                        
						run; 
						
						%NObsVars(itervi);
						%let _nvars_ = &nObs;
						%if &_nvars_ > 0 %then %do;
							%do v=1 %to &_nvars_;
								data _NULL_;
									set itervi;
									if _n_ = &v then do;
										call symput("DROP_VAR", trim(left(DROPVAR)));
										call symput("DROP_LABEL", trim(left(DROPLABEL)));
									end;
								run;
								data vipreds; 
									set &itoutlib..cv_vipreds_pcum&iter;
									where DROPVAR="&DROP_VAR";
								run;
								
								%NObsVars(vipreds);
								%let _vipredsObs_ = &nObs;
								%if &_vipredsObs_ %then %do;

									* COMPUTE MODEL ASE;
									%if %upcase(&trait_type)=QUALITATIVE %then %do;
								        %compute_ase_nominal(data=vipreds,
								                             output=ase_v,
								                             actual=actualp,
								                             nclasses=&nclasses,
								                             mdlname=&mdlname,
								                             freq=&freq,
						                             		  prevalences=&prevalences,
															        cv_custompriors=%nrbquote(&cv_custompriors),
										                       trait=&trait
						                             		  );
								        %compute_acc_nominal(data=vipreds,
								                             output=acc_v,
								                             trait=&trait,
								                             correct=correct,
								                             mdlname=&mdlname,
								                             freq=&freq,
						                             		  prevalences=&prevalences,
															        cv_custompriors=%nrbquote(&cv_custompriors)
						                                   );
						            %if %length(&cv_customcosts) %then %do;
								        %compute_aec_nominal(data=vipreds,
								                             output=aec_v,
								                             trait=&trait,
								                             expcost=expected_cost,
								                             mdlname=&mdlname,
								                             freq=&freq,
						                                     );
						            %end;
										%if &nclasses = 2 %then %do;
											 %locate_pevent_isevent(data=vipreds,
							                               			trait=&trait,
							                               			event=&event,
							                               			actual=actualp);
									        * COMPUTE MODEL MANN;
									        %compute_mann(data=vipreds,
									                      pevent=_pevent,
									                      isevent=_isevent);
											data auc_v;
												AUC=&mann;
											run;
										%end;
									%end;
								   %else %if %upcase(&trait_type)=QUANTITATIVE %then %do;

										%if &cv_alossprop ne 0.5 %then %do;
											data vipreds;
												set vipreds;
												_resid = &trait - Predicted_Value;
												if missing(_resid) then &cv_alossweight = .;
												else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
												else &cv_alossweight = 2 * (1 - &cv_alossprop);
											run;
										%end;

								      %compute_ase_interval(data=vipreds,
								                            output=ase_v,
								                            y=&trait,
								                            predict=Predicted_Value,
								                            mdlname=&mdlname,
								                            freq=&freq,
								                            aweight=&cv_alossweight);
								     
%* put ERROR: 5.********** data = vipreds;                       
										%compute_harrellc(data=vipreds,
														  output=hac_v,
														  TimeVar=&trait,
														  CenVar=,
														  Predict=Predicted_Value,
														  Freq=&cv_alossweight);

								      %compute_corr_interval(data=vipreds,
								                             output=ase_v,
								                             y=&trait,
								                             predict=Predicted_Value,
								                             mdlname=&mdlname,
								                             freq=&freq,
								                             weight=&cv_alossweight);
								   %end;
								
									%if %sysfunc(exist(ase_v)) = 1 %then %do;
										%NObsVars(ase_v);
										%let _asevObs_ = &nObs;
										%if &_asevObs_ %then %do;
											data ase_v;
												length INPUT $32;
												set ase_v;
												%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
													INPUT = "&DROP_LABEL";
												%end;
												%else %do;
													INPUT = "&DROP_VAR";
												%end;
												ASE_DIFF&iter=ASE - &ASE_FULL;
												DROP MAE ASE ITER K NOBS:;
											run;

										    %if %sysfunc(exist(vi_ase)) = 1 %then %do;
												%NObsVars(vi_ase);
												%let _viaseObs_ = &nObs;
												%if &_viaseObs_ %then %do;
													proc sort data=vi_ase;
														by INPUT;
													run;
												%end;
												data vi_ase;
													merge 
													%if &_viaseObs_ %then %do;
														vi_ase
													%end;
														ase_v;
													by INPUT;
												run;
											%end;
											%else %do;
												data vi_ase;
													set ase_v;
												run;
											%end;
										%end;
									%end;

							    	%if %upcase(&trait_type)=QUALITATIVE %then %do;
									   %if %sysfunc(exist(acc_v)) = 1 %then %do;
											%NObsVars(acc_v);
											%let _accvObs_ = &nObs;
											%if &_accvObs_ %then %do;
												data acc_v;
													length INPUT $32;
													set acc_v;
													%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
														INPUT = "&DROP_LABEL";
													%end;
													%else %do;
														INPUT = "&DROP_VAR";
													%end;
													ACC_DIFF&iter=&ACC_FULL - Accuracy;
													DROP Accuracy ITER K NOBS:;
												run;

									   			%if %sysfunc(exist(vi_acc)) = 1 %then %do;
													%NObsVars(vi_acc);
													%let _viaccObs_ = &nObs;
													%if &_viaccObs_ %then %do;
														proc sort data=vi_acc;
															by INPUT;
														run;
													%end;
													data vi_acc;
														merge 
														%if &_viaccObs_ %then %do;
															vi_acc
														%end;
															acc_v;
														by INPUT;
													run;
												%end;
												%else %do;
													data vi_acc;
														set acc_v;
													run;
												%end;
											%end;
										%end;
										%if %length(&cv_customcosts) %then %do;
											%if %sysfunc(exist(aec_v)) = 1 %then %do;
												%NObsVars(aec_v);
												%let _aecvObs_ = &nObs;
												%if &_aecvObs_ %then %do;
													data aec_v;
														length INPUT $32;
														set aec_v;
														%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
															INPUT = "&DROP_LABEL";
														%end;
														%else %do;
															INPUT = "&DROP_VAR";
														%end;
														aec_DIFF&iter=&aec_FULL - aec;
														DROP aec ITER K NOBS:;
													run;

														%if %sysfunc(exist(vi_aec)) = 1 %then %do;
														%NObsVars(vi_aec);
														%let _viaecObs_ = &nObs;
														%if &_viaecObs_ %then %do;
															proc sort data=vi_aec;
																by INPUT;
															run;
														%end;
														data vi_aec;
															merge 
															%if &_viaecObs_ %then %do;
																vi_aec
															%end;
																aec_v;
															by INPUT;
														run;
													%end;
													%else %do;
														data vi_aec;
															set aec_v;
														run;
													%end;
												%end;
											%end;
										%end;
										%if &nclasses = 2 %then %do;
											%if %sysfunc(exist(auc_v)) = 1 %then %do;
												%NObsVars(auc_v);
												%let _aucvObs_ = &nObs;
												%if &_aucvObs_ %then %do;
													data auc_v;
														length INPUT $32;
														set auc_v;
														%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
															INPUT = "&DROP_LABEL";
														%end;
														%else %do;
															INPUT = "&DROP_VAR";
														%end;
														AUC_DIFF&iter=&AUC_FULL - AUC;
														DROP AUC;
													run;

													%if %sysfunc(exist(vi_auc)) = 1 %then %do;
														%NObsVars(vi_auc);
														%let _viaucObs_ = &nObs;
														%if &_viaucObs_ %then %do;
															proc sort data=vi_auc;
																by INPUT;
															run;
														%end;
														data vi_auc;
															merge 
															%if &_viaucObs_ %then %do;
																vi_auc
															%end;
																auc_v;
															by INPUT;
														run;
													%end;
													%else %do;
														data vi_auc;
															set auc_v;
														run;	
													%end;
												%end;	
											%end;
										%end;
									%end;
							    	%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
										%if %sysfunc(exist(hac_v)) = 1 %then %do;
											%NObsVars(hac_v);
											%let _hacvObs_ = &nObs;
											%if &_hacvObs_ %then %do;
												data hac_v;
													length INPUT $32;
													set hac_v;
													%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
														INPUT = "&DROP_LABEL";
													%end;
													%else %do;
														INPUT = "&DROP_VAR";
													%end;
													HAC_DIFF&iter=&HAC_FULL - HARRELL;
													DROP HARRELL;
												run;

												%if %sysfunc(exist(vi_hac)) = 1 %then %do;
													%NObsVars(vi_hac);
													%let _vihacObs_ = &nObs;
													%if &_vihacObs_ %then %do;
														proc sort data=vi_hac;
															by INPUT;
														run;
													%end;
													data vi_hac;
														merge 
														%if &_vihacObs_ %then %do;
															vi_hac
														%end;
															hac_v;
														by INPUT;
													run;
												%end;
												%else %do;
													data vi_hac;
														set hac_v;
													run;	
												%end;
											%end;	
										%end;
										%if %sysfunc(exist(corr_v)) = 1 %then %do;
											%NObsVars(corr_v);
											%let _corrvObs_ = &nObs;
											%if &_corrvObs_ %then %do;
												data corr_v;
													length INPUT $32;
													set corr_v;
													%if %length(&DROP_LABEL) > 0 and &DROP_LABEL ne " " %then %do;
														INPUT = "&DROP_LABEL";
													%end;
													%else %do;
														INPUT = "&DROP_VAR";
													%end;
													CORR_DIFF&iter=&CORR_FULL - CORR;
													DROP CORR;
												run;

												%if %sysfunc(exist(vi_corr)) = 1 %then %do;
													%NObsVars(vi_corr);
													%let _vicorrObs_ = &nObs;
													%if &_vicorrObs_ %then %do;
														proc sort data=vi_corr;
															by INPUT;
														run;
													%end;
													data vi_corr;
														merge 
														%if &_vicorrObs_ %then %do;
															vi_corr
														%end;
															corr_v;
														by INPUT;
													run;
												%end;
												%else %do;
													data vi_corr;
														set corr_v;
													run;	
												%end;
											%end;	
										%end;
									%end;	
								%end;
							%end; * END OF VARIABLES LOOP; 
						%end; * _NVARS_ > 0;
					%end; 
					
					%if %sysfunc(exist(vi_ase)) = 1 %then %do;
						%NObsVars(vi_ase);
						%let _viaseObs_ = &nObs;
						%if &_viaseObs_ > 0 %then %do;
							%if %sysfunc(exist(&varimportance_output._ased)) = 1 %then %do;
								%NObsVars(&varimportance_output._ased);
								%let _viasedObs_ = &nObs;
								data &varimportance_output._ased;
									merge
									%if &_viasedObs_ > 0 %then %do;
										&varimportance_output._ased
									%end;
										vi_ase;
								run;
							%end;
							%else %do;
								data &varimportance_output._ased;
									set vi_ase;
								run;
							%end;
						%end;
					%end;
				
					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						%if %sysfunc(exist(vi_acc)) = 1 %then %do;
							%NObsVars(vi_acc);
							%let _viaccObs_ = &nObs;
							%if &_viaccObs_ > 0 %then %do;
								%if %sysfunc(exist(&varimportance_output._accd)) = 1 %then %do;
									%NObsVars(&varimportance_output._accd);
									%let _viaccdObs_ = &nObs; 
									data &varimportance_output._accd;
										merge
										%if &_viaccdObs_ > 0 %then %do;
											&varimportance_output._accd
										%end;
											vi_acc;
									run;
								%end;
								%else %do;
									data &varimportance_output._accd;
										set vi_acc;
									run;
								%end;
							%end;
						%end;

						%if %length(&cv_customcosts) %then %do;
							%if %sysfunc(exist(vi_aec)) = 1 %then %do;
								%NObsVars(vi_aec);
								%let _viaecObs_ = &nObs;
								%if &_viaecObs_ > 0 %then %do;
									%if %sysfunc(exist(&varimportance_output._aecd)) = 1 %then %do;
										%NObsVars(&varimportance_output._aecd);
										%let _viaecdObs_ = &nObs; 
										data &varimportance_output._aecd;
											merge
											%if &_viaecdObs_ > 0 %then %do;
												&varimportance_output._aecd
											%end;
												vi_aec;
										run;
									%end;
									%else %do;
										data &varimportance_output._aecd;
											set vi_aec;
										run;
									%end;
								%end;
							%end;
						%end;

						%if &nclasses = 2 %then %do;
							%if %sysfunc(exist(vi_auc)) = 1 %then %do;
								%NObsVars(vi_auc);
								%let _viaucObs_	= &nObs;
								%if &_viaucObs_ > 0 %then %do;
									%if %sysfunc(exist(&varimportance_output._aucd)) = 1 %then %do;
										%NObsVars(&varimportance_output._aucd);
										%let _viaucdObs_ = &nObs;
										data &varimportance_output._aucd;
											merge
											%if &_viaucdObs_ > 0 %then %do;
												&varimportance_output._aucd
											%end;
												vi_auc;
										run;
									%end;	
									%else %do;
										data &varimportance_output._aucd;
											set	vi_auc;
										run;
									%end;
								%end;
							%end;
						%end;
					%end;
					%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
						%if %sysfunc(exist(vi_hac)) = 1 %then %do;
							%NObsVars(vi_hac);
							%let _vihacObs_ = &nObs;
							%if &_vihacObs_ > 0 %then %do;
								%if %sysfunc(exist(&varimportance_output._hacd)) = 1 %then %do;
									%NObsVars(&varimportance_output._hacd);
									%let _vihacdObs_ = &nObs; 
									data &varimportance_output._hacd;
										merge
										%if &_vihacdObs_ > 0 %then %do;
											&varimportance_output._hacd
										%end;
											vi_hac;
									run;
								%end;
								%else %do;
									data &varimportance_output._hacd;
										set vi_hac;
									run;
								%end;
							%end;
						%end;
						%if %sysfunc(exist(vi_corr)) = 1 %then %do;
							%NObsVars(vi_corr);
							%let _vicorrObs_ = &nObs;
							%if &_vicorrObs_ > 0 %then %do;
								%if %sysfunc(exist(&varimportance_output._corrd)) = 1 %then %do;
									%NObsVars(&varimportance_output._corrd);
									%let _vicorrdObs_ = &nObs; 
									data &varimportance_output._corrd;
										merge
										%if &_vicorrdObs_ > 0 %then %do;
											&varimportance_output._corrd
										%end;
											vi_corr;
									run;
								%end;
								%else %do;
									data &varimportance_output._corrd;
										set vi_corr;
									run;
								%end;
							%end;
						%end;
					%end;
				%end; * END OF ITERATIONS;
			%end;
		%end;

		* CALCULATE AVERGE OF ASE_DIFF, ACC_DIFF, and AUC_DIFF;
		%if %sysfunc(exist(&varimportance_output._ased)) = 1 %then %do;
			%NObsVars(&varimportance_output._ased);
			%if &nObs and &nVars %then %do;
				data &cvoutlib..&varimportance_output._fitd;
					set &varimportance_output._ased;
					ASE_DIFF_MEAN=mean(
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
						%if &iter ne &cv_holdout_iteration_end %then %do;
							ASE_DIFF&iter,
						%end;
						%else %do;
							ASE_DIFF&iter
						%end;
					%end;
					);
					ASE_DIFF_MEDIAN=median(
					%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
						%if &iter ne &cv_holdout_iteration_end %then %do;
							ASE_DIFF&iter,
						%end;
						%else %do;
							ASE_DIFF&iter
						%end;
					%end;
					);
				run;
				proc sort data=&cvoutlib..&varimportance_output._fitd;
					by INPUT;
				run;
			%end;
		%end;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			%if %sysfunc(exist(&varimportance_output._accd)) = 1 %then %do;
				%NObsVars(&varimportance_output._accd);
				%if &nObs and &nVars %then %do;
					data &varimportance_output._accd;
						set &varimportance_output._accd;
						ACC_DIFF_MEAN=mean(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								ACC_DIFF&iter,
							%end;
							%else %do;
								ACC_DIFF&iter
							%end;
						%end;
						);
						ACC_DIFF_MEDIAN=median(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								ACC_DIFF&iter,
							%end;
							%else %do;
								ACC_DIFF&iter
							%end;
						%end;
						);
					run;
					proc sort data=&varimportance_output._accd;
						by INPUT;
					run;
					data &cvoutlib..&varimportance_output._fitd;
						merge &cvoutlib..&varimportance_output._fitd
							  &varimportance_output._accd;
						by INPUT;
					run;
				%end;
			%end;

			%if %length(&cv_customcosts) %then %do;
				%if %sysfunc(exist(&varimportance_output._aecd)) = 1 %then %do;
					%NObsVars(&varimportance_output._aecd);
					%if &nObs and &nVars %then %do;
						data &varimportance_output._aecd;
							set &varimportance_output._aecd;
							aec_DIFF_MEAN=mean(
							%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
								%if &iter ne &cv_holdout_iteration_end %then %do;
									aec_DIFF&iter,
								%end;
								%else %do;
									aec_DIFF&iter
								%end;
							%end;
							);
							aec_DIFF_MEDIAN=median(
							%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
								%if &iter ne &cv_holdout_iteration_end %then %do;
									aec_DIFF&iter,
								%end;
								%else %do;
									aec_DIFF&iter
								%end;
							%end;
							);
						run;
						proc sort data=&varimportance_output._aecd;
							by INPUT;
						run;
						data &cvoutlib..&varimportance_output._fitd;
							merge &cvoutlib..&varimportance_output._fitd
								  &varimportance_output._aecd;
							by INPUT;
						run;
					%end;
				%end;
			%end;

			%if &nclasses = 2 %then %do;
				%if %sysfunc(exist(&varimportance_output._aucd)) = 1 %then %do;
					%NObsVars(&varimportance_output._aucd);
					%if &nObs and &nVars %then %do;
						data &varimportance_output._aucd;
							set &varimportance_output._aucd;
							AUC_DIFF_MEAN=mean(
							%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
								%if &iter ne &cv_holdout_iteration_end %then %do;
									AUC_DIFF&iter,
								%end;
								%else %do;
									AUC_DIFF&iter
								%end;
							%end;
							);
							AUC_DIFF_MEDIAN=median(
							%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
								%if &iter ne &cv_holdout_iteration_end %then %do;
									AUC_DIFF&iter,
								%end;
								%else %do;
									AUC_DIFF&iter
								%end;
							%end;
							);
						run;	
						proc sort data=&varimportance_output._aucd;
							by INPUT;
						run;
						data &cvoutlib..&varimportance_output._fitd;
							merge &cvoutlib..&varimportance_output._fitd
								  &varimportance_output._aucd;
							by INPUT;
						run;
					%end;
				%end;
			%end;
		%end;
		%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
			%if %sysfunc(exist(&varimportance_output._hacd)) = 1 %then %do;
				%NObsVars(&varimportance_output._hacd);
				%if &nObs and &nVars %then %do;
					data &varimportance_output._hacd;
						set &varimportance_output._hacd;
						HAC_DIFF_MEAN=mean(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								HAC_DIFF&iter,
							%end;
							%else %do;
								HAC_DIFF&iter
							%end;
						%end;
						);
						HAC_DIFF_MEDIAN=median(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								HAC_DIFF&iter,
							%end;
							%else %do;
								HAC_DIFF&iter
							%end;
						%end;
						);
					run;
					proc sort data=&varimportance_output._hacd;
						by INPUT;
					run;
					data &cvoutlib..&varimportance_output._fitd;
						merge &cvoutlib..&varimportance_output._fitd
							  &varimportance_output._hacd;
						by INPUT;
					run;
				%end;
			%end;
			%if %sysfunc(exist(&varimportance_output._corrd)) = 1 %then %do;
				%NObsVars(&varimportance_output._corrd);
				%if &nObs and &nVars %then %do;
					data &varimportance_output._corrd;
						set &varimportance_output._corrd;
						CORR_DIFF_MEAN=mean(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								CORR_DIFF&iter,
							%end;
							%else %do;
								CORR_DIFF&iter
							%end;
						%end;
						);
						CORR_DIFF_MEDIAN=median(
						%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
							%if &iter ne &cv_holdout_iteration_end %then %do;
								CORR_DIFF&iter,
							%end;
							%else %do;
								CORR_DIFF&iter
							%end;
						%end;
						);
					run;
					proc sort data=&varimportance_output._corrd;
						by INPUT;
					run;
					data &cvoutlib..&varimportance_output._fitd;
						merge &cvoutlib..&varimportance_output._fitd
							  &varimportance_output._corrd;
						by INPUT;
					run;
				%end;
			%end;
		%end;
		
		%if %sysfunc(exist(&cvoutlib..&varimportance_output._fitd)) = 1 %then %do;
			%NObsVars(&cvoutlib..&varimportance_output._fitd);
			%if &nObs and &nVars %then %do;
				proc sort data=&cvoutlib..&varimportance_output._fitd;
					by descending ASE_DIFF_MEDIAN;
				run;
			%end;
		%end;
	%end;
%end; 
%else %do; 
	* SAVE OVERAL MODELS EFFECTS, AND COMPUTE EFFECTS FREQUENCIES;
	* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
	data &cvoutlib..&prefix._cve_all;
		delete;
	run;
	data &cvoutlib..&models_effects_output;
		delete;
	run;

	* SAVE OVERALL MODELS EFFECTS;
	%if &cv_vcum_exist=0 %then %do;
		%put NOTE: NO PREDICTOR VARIABLES ARE SELECTED FROM ALL MODELS.;
	%end;
	%else %do;
		data &cvoutlib..&prefix._cve_all;
		    length METHOD $64 NAME $32;
		    set 
		    %do nm=1 %to &n_models;
				%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_m&nm._vcum)
					%if &nObs and &nVars %then %do;
				        &cvoutlib..cv_m&nm._vcum
					%end;
				%end;
		    %end;
		    ;
		run;
		%compute_effects(outputpath=&outputpath,
						 cv_method=&cv_method,
						 leave_l_out=&leave_l_out,
						 models_comparison_output=&models_comparison_output,
						 models_effects_output=&models_effects_output);
	%end;
%end;  

* FINAL CLEAR INTERMEDIATE OUTPUT DATA SETS;
proc datasets library=&cvoutlib nolist;
	delete
	%do nm=1 %to &n_models;
		cv_m&nm._pcum
		cv_m&nm._vcum
		%if %upcase(&trait_type) = SURVIVAL %then %do; 
			cv_m&nm._s1cum
			cv_m&nm._s2cum
			cv_m&nm._s3cum
		%end;
	%end;
	%if %upcase(&varimportance) = YES %then %do;
		cv_vipreds_pcum
		cv_vivars_pcum
	%end;
	;
quit;

proc datasets library=&itoutlib nolist;
	delete
	%do iter=&cv_holdout_iteration_start %to &cv_holdout_iteration_end;
		cv_baseline_preds&iter 
		%do nm=1 %to &n_models;
			cv_m&nm._pcum&iter
			cv_m&nm._vcum&iter
			%if %upcase(&trait_type) = SURVIVAL %then %do; 
				cv_m&nm._s1cum&iter
				cv_m&nm._s2cum&iter
				cv_m&nm._s3cum&iter
			%end;
		%end;
		%if %upcase(&varimportance) = YES %then %do;
			cv_vipreds_pcum&iter
			cv_vivars_pcum&iter
			%do k=1 %to &kfold;
				importance&iter&k
				impfit&iter&k
			%end;
		%end;
	%end;
	;
quit;

%cv_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2009 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_LC_MULTI_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    04/10/09
**  PURPOSE: Merge multiple cross validation results
**  HISTORY: action                                        date     who
**           Created.                                      04/10/09 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATHS: Library paths of output data sets 
**  MERGE_OUTPUTPATH: Library path of final merged ouput 
**  INNERLOOP_OPTION: Inner loop algorithm on how model is fitted and evaluated
**		Options are: 
**  	- STANDARD: Models fit on full data (without test set), evaluate on test set. 
**  	- CVTEST: Cross validation on subsets, evaluate on test set
**  	- CVHOLDOUT: Cross validation on subsets, evaluate on corresponding hold-out sets
**  	- CVBOTH: Cross validation on subsets, evaluate on both test set and hold-out sets
**  TESTSET_OPTION: Test set choice for evaluating the model
**		Options are: 
**  	- HOLDOUT: Subset (in Outer Loop) of Input Data Set 
**  	- FIXTEST: External Validation Data Set 
**  TRAIT: Trait variable name
**  TRAIT_TYPE: Trait variable type: 
**              qualitative(categorical), 
**              quantitative(interval)
**  NCLASSES:	Number of qualitative trait variable category
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_ITERATIONS: Number of total random hold-out iterations
**  KFOLD: K-fold cross validation 
**  LEAVE_L_OUT: Leave l out cross validation method 
**  CVLC_ESTIMATES_OUTPUT: Cross validation learning curve fit estimates 
**  CVLC_PLOTS_OUTPUT: Cross validation learning curve plots data 
**  LC_FIXTEST_ESTIMATES_OUTPUT: Learning curve fix test fits estimates
**  LC_FIXTEST_PLOTS_OUTPUT: Learning curve fix test plots data 
**  BIAS_ESTIMATES_OUTPUT: Bias estimates output data 
**  BIAS_PLOTS_OUTPUT: Bias plots data
**
**  NOTES:
**  All macro parameters except outputpaths and merge_outputpath 
**  are identical to the settings saved in cvlc_plot.jsl of all 
**  output folders 
**
**  Example:
**  %merge_lc_multi_results(
**        outputpaths="C:\...\outputpath1\ |                                                                        
**                     C:\...\outputpath2\ |
**					   ...
**					   C:\...\outputpathx\",    
**        merge_outputpath=C:\...\merge_results_outputpath\,                                                                           
**		  innerloop_option=cvholdout,
**		  testset_option=, 
**        trait=trait,                                                                                                
**        trait_type=qualitative,                                                                                   
**        nclasses=2, 
**        n_innerloop=2,                                                                                            
**        cv_method=RANDOM,                                                         
**        cv_holdout_iterations=2,                                                                                  
**        kfold=2,                                                                                                  
**        leave_l_out=894,                                                                                          
**        cvlc_estimates_output=train_cvlc,                                                                    
**        cvlc_plots_output=train_cvlc_plt,                                                                    
**        lc_fixtest_estimates_output=train_fixt,                                                                             
**        lc_fixtest_plots_output=train_fixt_plt,                                                                                 
**        bias_estimates_output=train_bias,                                                                                   
**        bias_plots_output=train_bias_plt
**		  );    
**
************************************************************************;
%macro merge_lc_multi_results(outputpaths=,
							  merge_outputpath=,
							  innerloop_option=,
							  testset_option=,
							  trait=,
							  trait_type=,
							  nclasses=,
							  n_innerloop=,
  							  cv_method=,
							  cv_holdout_iterations=,
							  kfold=,
							  leave_l_out=,
		 					  cvlc_estimates_output=,
							  cvlc_plots_output=,
							  lc_fixtest_estimates_output=,
							  lc_fixtest_plots_output=,
							  bias_estimates_output=,
							  bias_plots_output=
							  );

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%put merge_outputpath=&merge_outputpath;
%if %length(&merge_outputpath) > 0 and "&merge_outputpath" ne "" %then %do;
    libname output "&merge_outputpath";
    %let outlib = output;
%end;
%else %do;
    %let outlib = work;
%end;

* RETRIEVE INDIVIDUAL OUTPUTPATHS;
%let i=1;
%do %while(%length(%scan(&outputpaths, &i, "|")));
	%let i = %eval(&i+1);
%end;
%let n_outputpaths = %eval(&i-1);
%put n_outputpaths = &n_outputpaths;
%do i=1 %to &n_outputpaths;
	%let path = %scan(&outputpaths, &i, "|");	
	%put path=&path;
	libname outlib&i "&path";
%end;

* MERGE _METADAT FROM OUTPUTPATHS;
data _meta;
	set 
	%do i = 1 %to &n_outputpaths;
		outlib&i.._metadat
	%end;
	;
run;

proc freq data=_meta noprint;
	tables outer_loop/out=fo(keep=OUTER_LOOP);
	tables method/out=fm(keep=METHOD);
	tables outer_loop*method/out=fom(keep=OUTER_LOOP METHOD);
	tables outer_loop*inner_loop/out=outer_inner(keep=OUTER_LOOP INNER_LOOP);
run;

* CHECK FOR COMPLETE LCMC RUNS FROM MULTIPLE OUTPUT PATHS;
%NObsVars(fo);
%let n_mouters = &nObs;
%NObsVars(fm);
%let n_mmodels = &nObs;
%NObsVars(fom);
%let n_momm = &nObs;
%if &n_momm ne %sysevalf( &n_mouters * &n_mmodels, int) %then %do;
	%let exiterror = 1;
	%put ERROR: SOME OUTER_LOOP OR MODEL RUNS ARE MISSING IN THE COMBINED RESULTS.;
	%goto lc_exit;
%end;

* RETRIEVE MODEL NAMES;
%let _mid_ = %sysfunc(open(fm));
%let mthnum = %sysfunc(varnum(&_mid_, METHOD));
%put mthnum = &mthnum;
%do m=1 %to &n_mmodels;
	%let rc = %sysfunc(fetchobs(&_mid_, &m));
	%let method = %sysfunc(getvarc(&_mid_, &mthnum));
	%if &m = 1 %then %do;
		%let model_names=&method;
	%end;
	%else %do;
		%let model_names=&model_names &method;
	%end;
%end;
%let _mid_ = %sysfunc(close(&_mid_));

* RETRIEVE OUTERLOOP_START OUTERLOOP_END;
%let _oid_ = %sysfunc(open(fo));
%let rc = %sysfunc(fetchobs(&_oid_, 1));
%let outerloop_start = %sysfunc(getvarn(&_oid_, 1));
%let rc = %sysfunc(fetchobs(&_oid_, &n_mouters));
%let outerloop_end = %sysfunc(getvarn(&_oid_, 1));
%let n_outerloop = &outerloop_end;
%let _oid_ = %sysfunc(close(&_oid_));

* RE-NUMBER METHOD_I FOR COMBINING RESULTS;
data fm;
	set fm;
	CMETHOD_I = _n_;
run;

* CREATE COMBINED _METADAT;
proc sort data=_meta;
	by METHOD;
run;
data &outlib.._metadat;
	merge _meta fm;
	by METHOD;
	drop METHOD_I;
	rename CMETHOD_I = METHOD_I;
run;

%do i = 1 %to &n_outputpaths;
	
	%let path = %scan(&outputpaths, &i, " ");	

	proc freq data=outlib&i.._metadat noprint;
		tables METHOD*METHOD_I/out=fm&i;
	run;
	proc sort data=fm&i;
		by METHOD;
	run;
	data fm&i;
		merge fm&i fm;
		by METHOD;
		keep METHOD METHOD_I CMETHOD_I;
		if METHOD_I = . then delete;
	run;
	proc sort data=fm&i;
		by METHOD_I;
	run;

	%NObsVars(fm&i);
	%let n_models = &nObs;

	%let _fmid_ = %sysfunc(open(work.fm&i));
	%let cmthnum = %sysfunc(varnum(&_fmid_, CMETHOD_I));
	%do nm=1 %to &n_models;
		%let rc = %sysfunc(fetchobs(&_fmid_, &nm));
		%let cmth&nm = %sysfunc(getvarn(&_fmid_, &cmthnum));
	%end;
	%let _fmid_ = %sysfunc(close(&_fmid_));
	
	%if %upcase(&innerloop_option) = STANDARD or
		%upcase(&innerloop_option) = CVBOTH %then %do;

		data &lc_fixtest_estimates_output.&i;
			set outlib&i..&lc_fixtest_estimates_output;
			by outer_loop inner_loop;
			%do nm = 1 %to &n_models;
				%let cmth = &&cmth&nm;
				rename ASE_m&nm = ASE_m&cmth;
				rename RMSE_m&nm = RMSE_m&cmth;
				%if %upcase(&trait_type)=QUALITATIVE %then %do;
					rename ACC_m&nm=ACC_m&cmth;
					%if &nclasses = 2 %then %do;
						rename AUC_m&nm=AUC_m&cmth;
					%end;
				%end;
			%end;
		run;


		%if %upcase(&innerloop_option) = CVBOTH %then %do;
			data &cvlc_estimates_output.&i;
				set outlib&i..&cvlc_estimates_output;
				by outer_loop inner_loop;
				%do nm = 1 %to &n_models;
					%let cmth = &&cmth&nm;
					rename ASE_m&nm = ASE_m&cmth;
					rename RMSE_m&nm = RMSE_m&cmth;
					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						rename ACC_m&nm=ACC_m&cmth;
						%if &nclasses = 2 %then %do;
							rename AUC_m&nm=AUC_m&cmth;
						%end;
					%end;
				%end;
			run;

			data &bias_estimates_output.&i;
				set outlib&i..&bias_estimates_output;
				by outer_loop inner_loop;
				%do nm = 1 %to &n_models;
					%let cmth = &&cmth&nm;
					rename BIAS_ASE_m&nm = BIAS_ASE_m&cmth;
					rename BIAS_RMSE_m&nm = BIAS_RMSE_m&cmth;
					rename ABIAS_ASE_m&nm = ABIAS_ASE_m&cmth;
					rename ABIAS_RMSE_m&nm = ABIAS_RMSE_m&cmth;
					rename SBIAS_ASE_m&nm = SBIAS_ASE_m&cmth;
					rename SBIAS_RMSE_m&nm = SBIAS_RMSE_m&cmth;
					%if %upcase(&trait_type)=QUALITATIVE %then %do;
						rename BIAS_ACC_m&nm=BIAS_ACC_m&cmth;
						rename ABIAS_ACC_m&nm=ABIAS_ACC_m&cmth;
						rename SBIAS_ACC_m&nm=SBIAS_ACC_m&cmth;
						%if &nclasses = 2 %then %do;
							rename BIAS_AUC_m&nm=BIAS_AUC_m&cmth;
							rename ABIAS_AUC_m&nm=ABIAS_AUC_m&cmth;
							rename SBIAS_AUC_m&nm=SBIAS_AUC_m&cmth;
						%end;
					%end;
				%end;
			run;	
		%end;
	%end;
	%else %if %upcase(&innerloop_option) = CVTEST or 
	  	      %upcase(&innerloop_option) = CVHOLDOUT %then %do;
		data &cvlc_estimates_output.&i;
			set outlib&i..&cvlc_estimates_output;
			by outer_loop inner_loop;
			%do nm = 1 %to &n_models;
				%let cmth = &&cmth&nm;
				rename ASE_m&nm = ASE_m&cmth;
				rename RMSE_m&nm = RMSE_m&cmth;
				%if %upcase(&trait_type)=QUALITATIVE %then %do;
					rename ACC_m&nm=ACC_m&cmth;
					%if &nclasses = 2 %then %do;
						rename AUC_m&nm=AUC_m&cmth;
					%end;
				%end;
			%end;
		run;
	%end;
%end;

%if %upcase(&innerloop_option) = STANDARD or
	%upcase(&innerloop_option) = CVBOTH %then %do;
	data &outlib..&lc_fixtest_estimates_output;
		merge outer_inner 
		%do i = 1 %to &n_outputpaths;
			&lc_fixtest_estimates_output.&i
		%end;
		;
		by outer_loop inner_loop;
	run;
	%if %upcase(&innerloop_option) = CVBOTH %then %do;
		data &outlib..&cvlc_estimates_output;
			merge outer_inner 
			%do i = 1 %to &n_outputpaths;
				&cvlc_estimates_output.&i
			%end;
			;
			by outer_loop inner_loop;
		run;
		data &outlib..&bias_estimates_output;
			merge outer_inner 
			%do i = 1 %to &n_outputpaths;
				&bias_estimates_output.&i
			%end;
			;
			by outer_loop inner_loop;
		run;
	%end;
%end;
%else %if %upcase(&innerloop_option) = CVTEST or 
  	      %upcase(&innerloop_option) = CVHOLDOUT %then %do;
	data &outlib..&cvlc_estimates_output;
		merge outer_inner 
		%do i = 1 %to &n_outputpaths;
			&cvlc_estimates_output.&i
		%end;
		;
		by outer_loop inner_loop;
	run;
%end;

%if %upcase(&innerloop_option) = STANDARD %then %do;

	%lc_plots_data(outputpath=&merge_outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&lc_fixtest_estimates_output,
				   lc_plots_output=&lc_fixtest_plots_output);
%end;
%else %if %upcase(&innerloop_option) = CVTEST or 
		  %upcase(&innerloop_option) = CVHOLDOUT %then %do;

	%lc_plots_data(outputpath=&merge_outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,	
				   models_comparison_output=&cvlc_estimates_output,
				   lc_plots_output=&cvlc_plots_output);

%end;
%else %if %upcase(&innerloop_option) = CVBOTH %then %do;

	%lc_plots_data(outputpath=&merge_outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&lc_fixtest_estimates_output,
				   lc_plots_output=&lc_fixtest_plots_output);

	%lc_plots_data(outputpath=&merge_outputpath,
				   trait_type=&trait_type,
				   nclasses=&nclasses,
				   models_comparison_output=&cvlc_estimates_output,
				   lc_plots_output=&cvlc_plots_output);

	%lc_bias_plots_data(outputpath=&merge_outputpath,
						trait_type=&trait_type,
				   	    nclasses=&nclasses,
				   	    n_innerloop=&n_innerloop,
					    outerloop_start=&outerloop_start,
  					    outerloop_end=&outerloop_end,
					    models_comparison_output=&bias_estimates_output,
						bias_plots_output=&bias_plots_output);

%end;

%if %upcase(&trait_type) = QUALITATIVE %then %do;	
	* COPY _NAMEDAT;
	data &outlib.._namedat;
		set outlib1._namedat;
	run;
%end;

* JSL PLOTTING CODE, LAUNCHED FROM ExitAndRetrieveLogLstFromJMP IN UtilityMacros.sas;
%if %upcase(&innerloop_option) ne CVBOTH %then %do;
	%let bias_estimates_output = does_not_exist;
	%let bias_plots_output = does_not_exist;
	%if %upcase(&innerloop_option) = CVTEST or 
		%upcase(&innerloop_option) = CVHOLDOUT %then %do;
		%let lc_fixtest_estimates_output = does_not_exist;
		%let lc_fixtest_plots_output = does_not_exist;
	%end;
	%if %upcase(&innerloop_option) = STANDARD %then %do;
		%let cvlc_estimates_output = does_not_exist;
		%let cvlc_plots_output = does_not_exist;
	%end; 
%end;

filename jslfile "&merge_outputpath.cvlc_plot.jsl";
data _null_;
    file jslfile encoding="utf-8";
    put "//!";
    put "Names Default to Here(1);";
    put "// parameters for learning curve model comparison plots;";
    put "outputpath=""&merge_outputpath"";";
	put "cvlc_estimates_output=""&cvlc_estimates_output"";";
	put "cvlc_plots_output=""&cvlc_plots_output"";";
	put "lc_fixtest_estimates_output=""&lc_fixtest_estimates_output"";";
	put "lc_fixtest_plots_output=""&lc_fixtest_plots_output"";";
	put "bias_estimates_output=""&bias_estimates_output"";";
	put "bias_plots_output=""&bias_plots_output"";";
    put "cv_method = ""&cv_method"";";
	%if &cv_holdout_iterations > 0 %then %do;
    	put "cv_holdout_iterations = &cv_holdout_iterations;";
	%end;
	%else %do;
    	put "cv_holdout_iterations = .;";
	%end;
	%if &kfold > 0 %then %do;
    	put "kfold = &kfold;";
	%end;
	%else %do;
    	put "kfold = .;";
	%end;
	%if &leave_l_out > 0 %then %do;
    	put "leave_l_out = &leave_l_out;";
	%end;
	%else %do;
		put "leave_l_out = .;";
	%end;
    put "cvname = ""_CV_"";";
    put "model_names = ""&model_names"";";
    put "model_labels = ""&model_labels"";";
    put "trait = ""&trait"";";
    put "trait_type = ""&trait_type"";";
	%if &nclasses > 0 %then %do;
		put "nclasses = &nclasses;";
	%end;
	%else %do;
		put "nclasses = .;";
	%end;
	put "innerloop_option = ""&innerloop_option"";";
    put "testset_option = ""&testset_option"";";
	%if &n_outerloop > 0 %then %do;
		put "n_outerloop = &n_outerloop;";
	%end;
	%else %do;
		put "n_outerloop = .;";
	%end;
	%if outerloop_start > 0 %then %do;
		put "outerloop_start = &outerloop_start;";
	%end;
	%else %do;
		put "outerloop_start = .;";
	%end;
	%if &outerloop_end > 0 %then %do;
		put "outerloop_end = &outerloop_end;";
	%end;
	%else %do;
		put "outerloop_end = .;";
	%end;
	%if &n_innerloop > 0 %then %do;
		put "n_innerloop = &n_innerloop;";
	%end;
	%else %do;
		put "n_innerloop = .;";
	%end;
	put "if(Host Is(Windows),";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""JSLFiles\"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, windows)||""MacroLib\"",";
    put "jslpath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""JSLFiles/"";";
    put "macropath=ConvertFilePath(""$LS_HOME"", absolute, POSIX)||""MacroLib/"");";
    put "include(""$LS_HOME\JSLFiles\Learning Curve Model Comparison.jsl"");";
run;

%lc_exit:
;
%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_LC_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/15/07
**  PURPOSE: Merge learning curve models ase, acc, auc results
**  HISTORY: action                                        date     who
**           Created.                                      06/15/07 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH: Library path of output data sets 
**  LCR_PREFIX:   Prefix of LC prediction results data set name
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  NCLASSES:	  Number of qualitative trait variable category
**  EVENT: 	 	  Trait event level
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS: 	  Number of models
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  NMAX: Parameter used to determine the maximum size of training set 
**  CV_METHOD: Cross validation method
**             Various methods are: 
**             RANDOM     (RANDOM SIMPLE)        
**             STRATIFIED (RANDOM STRATIFIED)
**             BLOCK      (K-FOLD BLOCK)
**             SPLIT      (K-FOLD SPLIT)
**             KFOLD      (K-FOLD RANDOM)
**             LEAVEOUT	  (LEAVE L OUT)
**  CV_HOLDOUT_ITERATIONS: Number of total random hold-out iterations
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**  GSHARE:  Shared grid folder
**
**  NOTES:
**  Compute learning curve models fit statistics: ase, acc, auc
************************************************************************;
%macro merge_lc_results(outputpath=,
						lcr_prefix=,
						trait=,
						trait_type=,
						nclasses=,
						event=,
						freq=,
						model_names=,
					   model_labels=,
						n_models=,
						n_innerloop=,
						outerloop_start=,
						outerloop_end=,
						nmax=,
						cv_method=,
						cv_holdout_iterations=,
						models_comparison_output=,
					    prevalences=proportional,
					    gshare=
					    );

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;
%if %length(&gshare) %then %do;
	libname gout "&gshare";
	%let itoutlib = gout;
%end;
%else %do;
	%let itoutlib = &cvoutlib;
%end;
%put cvoutlib = &cvoutlib;
%put itoutlib = &itoutlib;

* CLEAR MODEL_COMPARISON_OUTPUT TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..&models_comparison_output;
	delete;
run;

* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
    %let mdllabel&i = %scan(&model_labels,&i,"|");
%end;
%if %length(&gshare) %then %do;
	%if %upcase(&outputpath) ne %upcase(&gshare) %then %do;
		* COPY _NAMEDAT FROM itoutlib TO cvoutlib;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;
			data &cvoutlib.._namedat;
				set &itoutlib.._namedat;
			run;
		%end;
	
		* COPY _METADAT FROM itoutlib TO cvoutlib;
		data &cvoutlib.._metadat;
			set &itoutlib.._metadat;
		run;
	%end;
%end;

* options nonotes;
%put cv_method = &cv_method;
%put cv_holdout_iterations = &cv_holdout_iterations;
%put outerloop_start = &outerloop_start;
%put outerloop_end = &outerloop_end;
%put n_innerlop = &n_innerloop;
%put gshare = &gshare;
%put outputpath = &outputpath;
	
%let &lcr_prefix._pcum_exist=0;
* COPY PREDS RESULTS DATA SETS FROM itoutlib TO cvoutlib; 
%if %upcase(&cv_method) = LEAVEOUT %then %do;
	proc freq data=&itoutlib.._metadat noprint;
		tables inner_loop*train_size/out=trainsize;
	run;
	%do outer_loop=&outerloop_start %to &outerloop_end;
		%do inner_loop=1 %to &n_innerloop;
			data _null_;
				set trainsize;
				if inner_loop = &inner_loop then do;
					call symput('train_size', put(train_size, 12.0));
				end;
			run;
			%if &leave_l_out = 3 %then %do;
				%do nm=1 %to &n_models;
					%do __i=1 %to &train_size-2;
						%do __j=&__i+1%to &&train_size-1;
							%do __k=&__j+1 %to &&train_size;	
								%if %sysfunc(exist(
									&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k)) = 1 
									%then %do;
									%NObsVars(&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k);
									%if &nObs and &nVars %then %do;
										%let &lcr_prefix._pcum_exist=1;
										%if %sysfunc(exist(
											&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 
											%then %do;
											%NObsVars(&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
											%let _cvpcumObs_ = &nObs;
											data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
												set 
												%if &_cvpcumObs_ %then %do;
													&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop
												%end;
													&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
											run;
										%end;
										%else %do;
											data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
												set &itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
											run;
										%end; 	
										proc datasets library=&itoutlib nolist;
											delete &lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j._&__k;
										quit;
									%end;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
			%else %if &leave_l_out = 2 %then %do;
				%do nm=1 %to &n_models;
					%do __i=1 %to &train_size-1;
						%do __j=&__i+1%to &train_size;
							%if %sysfunc(exist(
								&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j)) = 1 
								%then %do;
								%NObsVars(&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j);
								%if &nObs and &nVars %then %do;
									%let &lcr_prefix._pcum_exist=1;
									%if %sysfunc(exist(
										&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 
										%then %do;
										%NObsVars(&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
										%let _cvpcumObs_ = &nObs;
										data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
											set 
											%if &_cvpcumObs_ %then %do;
												&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop
											%end;
												&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j;
										run;
									%end;
									%else %do;
										data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
											set &itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j;
										run;
									%end;
									proc datasets library=&itoutlib nolist;
										delete &lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i._&__j;
									quit;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
			%else %if &leave_l_out = 1 %then %do;
				%do __i=1 %to &train_size;
					%do nm=1 %to &n_models;
						%if %sysfunc(exist(&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i)) = 1 %then %do;
							%NObsVars(&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i);
							%if &nObs and &nVars %then %do;
								%let &lcr_prefix._pcum_exist=1;
								%if %sysfunc(exist(&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 %then %do;
									%NObsVars(&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
									%let _cvpcumObs_ = &nObs;
									data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
										set 
										%if &_cvpcumObs_ %then %do;
											&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop
										%end;
											&itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i;
									run;
								%end;
								%else %do;
									data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
										set &itoutlib..&lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i;
									run;
								%end;
								proc datasets library=&itoutlib nolist;
									delete &lcr_prefix._m&nm._preds_i&inner_loop._o&outer_loop._&__i;
								quit;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;	
	%end;
%end;
%else %do;
	%if &cv_holdout_iterations > 0 %then %do;
		%do outer_loop=&outerloop_start %to &outerloop_end;
			%do inner_loop=1 %to &n_innerloop;
				%do nm=1 %to &n_models;
					%do iter=1 %to &cv_holdout_iterations;	
						%if %sysfunc(exist(
							&itoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop)) = 1 %then %do;
							%NObsVars(&itoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop);
							%if &nObs and &nVars %then %do;
								%let &lcr_prefix._pcum_exist=1;
								%if %sysfunc(exist(
									&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 %then %do;
									%NObsVars(&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
									%let _cvpcumObs_ = &nObs;
									data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
										set 
										%if &_cvpcumObs_ %then %do;
											&cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop
										%end;
											&itoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop;
									run;
								%end;
								%else %do;
									data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
										set &itoutlib..&lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop;
									run;
								%end;
								proc datasets library=&itoutlib nolist;
									delete &lcr_prefix._m&nm._pcum&iter._i&inner_loop._o&outer_loop;
								quit;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
	%else %do;				
		%do outer_loop=&outerloop_start %to &outerloop_end;
			%do inner_loop=1 %to &n_innerloop;
				%do nm=1 %to &n_models;
					%if %sysfunc(exist(&itoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 %then %do;
						%NObsVars(&itoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
						%if &nObs and &nVars %then %do;
							%let &lcr_prefix._pcum_exist=1;
							%if %length(&gshare) %then %do;
								%if %upcase(&outputpath) ne %upcase(&gshare) %then %do;
									data &cvoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
										set &itoutlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
									run;
									proc datasets library=&itoutlib nolist;
										delete &lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
									quit;
								%end;
							%end;
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
%end;

%put done with looping;

%if &&lcr_prefix._pcum_exist=0 %then %do;
	%let exiterror = 1;
	%put ERROR: FAIL TO GENERATE ALL MODELS PREDICTIONS.;
	%goto lc_exit;
%end;

%put calling merge_lc_iterations;

* options nonotes;
%merge_lc_iterations_results(outlib=&cvoutlib,
							 lcr_prefix=&lcr_prefix,
							 trait=&trait,
							 trait_type=&trait_type,
							 nclasses=&nclasses,
							 event=&event,
							 freq=&freq,
							 n_models=&n_models,
							 cv_holdout_iterations=&cv_holdout_iterations,
							 n_innerloop=&n_innerloop,
							 outerloop_start=&outerloop_start,
							 outerloop_end=&outerloop_end,
						     models_comparison_output=&models_comparison_output,
					    	 prevalences=&prevalences
	 						 );
	
%lc_exit:;
%mend;

***********************************************************************
**  Copyright (c) 2009 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_LC_ITERATIONS_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/12/09
**  PURPOSE: Merge learning curve models ase, acc, auc results
**  HISTORY: action                                        date     who
**           Created.                                      06/12/09 pyt 
** 
**  MACRO PARAMETERS:
**  OUTLIB: Library reference of output data sets 
**  LCR_PREFIX:   Prefix of LC prediction results data set name
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  NCLASSES:	  Number of qualitative trait variable category
**  EVENT: 		  Trait event level
**  FREQ:         Frequency variable name (optional)
**  N_MODELS: 	  Number of models
**  CV_HOLDOUT_ITERATIONS: Number of total random hold-out iterations
**  N_INNERLOOP:  Number of total LC inner loop iterations
**  OUTERLOOP_START: LC outer loop iteration starting index
**  OUTERLOOP_END: LC outer loop iteration ending index
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  PREVALENCES: Prior probabilities (EQUAL or PROPORTIONAL)
**
**  NOTES:
**  Compute learning curve models fit statistics: ase, acc, auc
************************************************************************;
%macro merge_lc_iterations_results(outlib=,
								   lcr_prefix=,
								   trait=,
								   trait_type=,
								   nclasses=,
								   event=,
								   freq=,
								   n_models=,
								   cv_holdout_iterations=,
								   cv_alossprop=0.5,
								   n_innerloop=,
								   outerloop_start=,
								   outerloop_end=,
							       models_comparison_output=,
					    		   prevalences=proportional
					               );

%if %sysfunc(exist(&outlib.._metadat)) = 1 %then %do;
	%NObsVars(&outlib.._metadat);
	%if &nObs and &nVars %then %do;
		proc freq data=&outlib.._metadat noprint;
			tables outer_loop*inner_loop*train_size/out=freq;
			tables train_size*inner_loop/out=trnsize;
		run;
		* COMPUTE LEARNING CURVE EACH OUTER_LOOP, INNER_LOOP, MODEL FIT STATISTICS: RMSE, ACC, AUC, AND HARRELL_C;
		* FIRST, CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
		%do nm=1 %to &n_models;
			data lc_m&nm._acc;
				delete;
			run;
			data lc_m&nm._mann;
				delete;
			run;
			data lc_m&nm._ase;
				delete;
			run;
			data lc_m&nm._hac;
				delete;
			run;
			data lc_m&nm._corr;
				delete;
			run;
		%end;
		%put n_models = &n_models;
		%if ^%symexist(cv_custompriors) %then %let cv_custompriors = ;
		%if &cv_alossprop ne 0.5 %then %do;
			%let cv_alossweight = Asymmetric;
		%end;
		%else %do;
			%let cv_alossweight = ;
		%end;
		%let first = 1;
		%do outer_loop = &outerloop_start %to &outerloop_end;
			%do inner_loop = 1 %to &n_innerloop;
				data _null_;
					set trnsize;
					if inner_loop=&inner_loop then do;
						call symputx("train_size", put(train_size, 12.0));
					end;
				run;

				%do nm=1 %to &n_models;	
					%let mdlname=&&mdlname&nm;
					%let mdllabel=&&mdllabel&nm;

					%put outer_loop=&outer_loop;
					%put inner_loop=&inner_loop;
					%put train_size=&train_size;
					%put mdlname=&mdlname;
					%put mdllabel=&mdllabel;

					* USE SEPARATED MODEL RESULTS HERE BECAUSE P_ VARIABLES CAN CHANGE FROM MODEL TO MODEL;
					%if %sysfunc(exist(&outlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop)) = 1 %then %do;
						%NObsVars(&outlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop);
						%let _pcumObs_ = &nObs;
						%if &_pcumObs_ %then %do;
							data p;
								set &outlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
							run;
							
							%NObsVars(p);
							%let _pObs_ = &nObs;

							%if &_pObs_ %then %do;
								%if &first %then %do;
									data &outlib..lcr_all;
										set p;
									run;
									%let first = 0;
								%end;
								%else %do;
									data &outlib..lcr_all;
										set &outlib..lcr_all p;
									run;
								%end;

								* COMPUTE MODEL ASE;
							    %if %upcase(&trait_type)=QUALITATIVE %then %do;
							        %compute_ase_nominal(data=p,
							                             output=m&nm._ase,
							                             actual=actualp,
							                             nclasses=&nclasses,
							                             mdlname=&mdlname,
							                             freq=&freq,
						                                 prevalences=&prevalences,
														 cv_custompriors=%nrbquote(&cv_custompriors),
										                 trait=&trait
						                                );
							        %compute_acc_nominal(data=p,
							                             output=m&nm._acc,
							                             trait=&trait,
							                             correct=correct,
							                             mdlname=&mdlname,
							                             freq=&freq,
						                                 prevalences=&prevalences,
														 cv_custompriors=%nrbquote(&cv_custompriors)
 						                                 );
							    %end;
							    %else %if %upcase(&trait_type)=QUANTITATIVE %then %do;

										%if &cv_alossprop ne 0.5 %then %do;
											data p;
												set p;
												_resid = &trait - Predicted_Value;
												if missing(_resid) then &cv_alossweight = .;
												else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
												else &cv_alossweight = 2 * (1 - &cv_alossprop);
											run;
										%end;

							        %compute_ase_interval(data=p,
							                              output=m&nm._ase,
							                              y=&trait,
							                              predict=Predicted_Value,
							                              mdlname=&mdlname,
							                              freq=&freq,
							                              aweight=&cv_alossweight);
	
	%* put ERROR: 6.********** data = vipreds;
									  %compute_harrellc(data=p,
													        output=m&nm._hac,
													        TimeVar=&trait,
													        CenVar=,
													        Predict=Predicted_Value,
													        Freq=&cv_alossweight);
							        %compute_corr_interval(data=p,
							                              output=m&nm._corr,
							                              y=&trait,
							                              predict=Predicted_Value,
							                              mdlname=&mdlname,
							                              freq=&freq,
							                              weight=&cv_alossweight);
							    %end;

							    * APPEND MODEL ASE FROM EACH OUTER_LOOP BY INNER_LOOP;
								%if %sysfunc(exist(m&nm._ase)) = 1 %then %do;
									%NObsVars(m&nm._ase);
									%let _maseObs_ = &nObs;
									%if &_maseObs_ %then %do;
										data m&nm._ase;
											set m&nm._ase;
											TRAIN_SIZE=&train_size;
											OUTER_LOOP=&outer_loop;
											INNER_LOOP=&inner_loop;
											RMSE = sqrt(ASE);
											rename ASE=ASE_m&nm; 
											rename RMSE=RMSE_m&nm;
											label ASE="ASE &mdllabel";
											label RMSE="RMSE &mdllabel";
											drop method mae iter k nobs nobsall;
									    run;
										
										%if %sysfunc(exist(lc_m&nm._ase)) = 1 %then %do;
											%NObsVars(lc_m&nm._ase);
											%let _lcaseObs_ = &nObs;
										    data lc_m&nm._ase;
												set
												%if &_lcaseObs_ %then %do;
												       lc_m&nm._ase
												%end;
												    m&nm._ase;
										    run;
										%end;
										%else %do;
										    data lc_m&nm._ase;
												set m&nm._ase;
										    run;
										%end;
										proc sort data=lc_m&nm._ase;
											by outer_loop train_size;
										run;
									%end;
								%end;

							    %if %upcase(&trait_type)=QUALITATIVE %then %do;
									* APPEND MODEL ACC FROM EACH OUTER_LOOP BY INNER_LOOP;
									%if %sysfunc(exist(m&nm._acc)) = 1 %then %do;
										%NObsVars(m&nm._acc);
										%let _maccObs_ = &nObs;
										%if &_maccObs_ %then %do;
									        data m&nm._acc;
									            set m&nm._acc;
												TRAIN_SIZE=&train_size;
												OUTER_LOOP=&outer_loop;
												INNER_LOOP=&inner_loop;
												rename Accuracy=ACC_m&nm;
												label Accuracy="ACC &mdllabel";
												drop method NOBS_: Accuracy_:;	
									        run; 

											%if %sysfunc(exist(lc_m&nm._acc)) = 1 %then %do;
												%NObsVars(lc_m&nm._acc);
												%let _lcaccObs_ = &nObs;
												data lc_m&nm._acc;
													set 
													%if &_lcaccObs_ %then %do;
										            	lc_m&nm._acc
													%end;
											            m&nm._acc;
											    run;
											%end;
											%else %do;
												data lc_m&nm._acc;
													set m&nm._acc;
											    run;
											%end;
											proc sort data=lc_m&nm._acc;
												by outer_loop train_size;
											run;
								     	%end;
									%end;

								    %if &nclasses = 2 %then %do;
								        %locate_pevent_isevent(data=p,
								                               trait=&trait,
								                               event=&event,
								                               actual=actualp);
								                               
								        %put trait=&trait event=&event;

								        * COMPUTE MODEL MANN;
								        %compute_mann(data=p,
								                      pevent=_pevent,
								                      isevent=_isevent);

								        * APPEND MODEL MANN FROM EACH OUTER_LOOP BY INNER_LOOP;
								        data m&nm._mann;
											TRAIN_SIZE=&train_size;
											OUTER_LOOP=&outer_loop;
											INNER_LOOP=&inner_loop;
											AUC=&mann;
											rename AUC=AUC_m&nm;
											label AUC="AUC &mdllabel";
								        run;

										%if %sysfunc(exist(lc_m&nm._mann)) = 1 %then %do;
											%NObsVars(lc_m&nm._mann);
											%let _lcmannObs_ = &nObs;
								            data lc_m&nm._mann;
								                set 
												%if &_lcmannObs_ %then %do;
									            	lc_m&nm._mann
												%end;
								                    m&nm._mann;
								            run;
										%end;
										%else %do;
								            data lc_m&nm._mann;
								                set m&nm._mann;
								            run;
										%end;
										proc sort data=lc_m&nm._mann;
											by outer_loop train_size;
										run;
								    %end; 
							    %end;
							    %else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
									%if %sysfunc(exist(m&nm._hac)) = 1 %then %do;
										%NObsVars(m&nm._hac);
										%let _mhacObs_ = &nObs;
										%if &_mhacObs_ %then %do;
									        data m&nm._hac;
									            set m&nm._hac;
												TRAIN_SIZE=&train_size;
												OUTER_LOOP=&outer_loop;
												INNER_LOOP=&inner_loop;
												rename Harrell=HAC_m&nm;
												label Harrell="HARRELLC &mdllabel";
									        run; 

											%if %sysfunc(exist(lc_m&nm._hac)) = 1 %then %do;
												%NObsVars(lc_m&nm._hac);
												%let _lchacObs_ = &nObs;
												data lc_m&nm._hac;
													set 
													%if &_lchacObs_ %then %do;
										            	lc_m&nm._hac
													%end;
											            m&nm._hac;
											    run;
											%end;
											%else %do;
												data lc_m&nm._hac;
													set m&nm._hac;
											    run;
											%end;
											proc sort data=lc_m&nm._hac;
												by outer_loop train_size;
											run;
								     	%end;
									%end;
									%if %sysfunc(exist(m&nm._corr)) = 1 %then %do;
										%NObsVars(m&nm._corr);
										%let _mcorrObs_ = &nObs;
										%if &_mcorrObs_ %then %do;
									       data m&nm._corr;
									          set m&nm._corr;
											    TRAIN_SIZE=&train_size;
										       OUTER_LOOP=&outer_loop;
											    INNER_LOOP=&inner_loop;
											    rename CORR=CORR_m&nm;
											    label CORR="CORR &mdllabel";
									      run; 

											%if %sysfunc(exist(lc_m&nm._corr)) = 1 %then %do;
												%NObsVars(lc_m&nm._corr);
												%let _lccorrObs_ = &nObs;
												data lc_m&nm._corr;
													set 
													%if &_lccorrObs_ %then %do;
										            	lc_m&nm._corr
													%end;
											            m&nm._corr;
											    run;
											%end;
											%else %do;
												data lc_m&nm._corr;
													set m&nm._corr;
											    run;
											%end;
											proc sort data=lc_m&nm._corr;
												by outer_loop train_size;
											run;
								     	%end;
									%end;
								%end;
							%end;
						%end;
					%end;
					%else %do;
						%put Did not find &outlib..&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop;
					%end;
				%end; * END OF N_MODELS LOOP;

				* MERGE LEARNING CURVE ALL MODELS ASE FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
				data &outlib..&models_comparison_output;
					merge
					%do nm=1 %to &n_models;
						%if %sysfunc(exist(lc_m&nm._ase)) = 1 %then %do;
							%NObsVars(lc_m&nm._ase)
							%let _aseObs_ = &nObs;
							%if &_aseObs_ %then %do;
								lc_m&nm._ase
							%end;
						%end;
					%end;
					;
					by outer_loop train_size;
				run;

				%if %upcase(&trait_type)=QUALITATIVE %then %do;
					* MERGE LEARNING CURVE ALL MODELS ACC FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE ;
				    data &models_comparison_output._ACC;
				        merge
				        %do nm=1 %to &n_models;
							%if %sysfunc(exist(lc_m&nm._acc)) = 1 %then %do;
								%NObsVars(lc_m&nm._acc)
								%let _accObs_ = &nObs;
								%if &_accObs_ %then %do;
					            	lc_m&nm._acc
								%end;
							%end;
				        %end;
				        ;
						by outer_loop train_size;
				    run;

					* MERGE LEARNING CURVE ALL MODELS ASE AND ACC FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
					%if %sysfunc(exist(&models_comparison_output._ACC)) = 1 %then %do;
						%NObsVars(&models_comparison_output._ACC);
						%if &nObs and &nVars %then %do;
							data &outlib..&models_comparison_output;
								merge &outlib..&models_comparison_output
									  &models_comparison_output._ACC;
							run;
						%end;
					%end;

					%if &nclasses = 2 %then %do;
						* MERGE LEARNING CURVE ALL MODELS AUC FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
					    data &models_comparison_output._AUC;
					        merge
					        %do nm=1 %to &n_models;
								%if %sysfunc(exist(lc_m&nm._mann)) = 1 %then %do;
									%NObsVars(lc_m&nm._mann)
									%let _aucObs_ = &nObs;
									%if &_aucObs_ %then %do;
						            	lc_m&nm._mann
									%end;
								%end;
					        %end;
					        ;
							by outer_loop train_size;
					    run;

						* MERGE LEARNING CURVE ALL MODELS ASE, ACC, AND AUC FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
						%if %sysfunc(exist(&models_comparison_output._AUC)) = 1 %then %do;
							%NObsVars(&models_comparison_output._AUC);
							%if &nObs and &nVars %then %do;
								data &outlib..&models_comparison_output;
									merge &outlib..&models_comparison_output
										  &models_comparison_output._AUC;
								run;
							%end;
						%end;
					%end;
				%end;
				%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
					* MERGE LEARNING CURVE ALL MODELS HARRELL FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
				    data &models_comparison_output._HAC;
				        merge
				        %do nm=1 %to &n_models;
							%if %sysfunc(exist(lc_m&nm._hac)) = 1 %then %do;
								%NObsVars(lc_m&nm._hac)
								%let _hacObs_ = &nObs;
								%if &_hacObs_ %then %do;
					            	lc_m&nm._hac
								%end;
							%end;
				        %end;
				        ;
						by outer_loop train_size;
				    run;

					* MERGE LEARNING CURVE ALL MODELS CORR FROM EACH OUTER_LOOP BY INNER_LOOP/TRAIN_SIZE;
				    data &models_comparison_output._CORR;
				        merge
				        %do nm=1 %to &n_models;
							%if %sysfunc(exist(lc_m&nm._corr)) = 1 %then %do;
								%NObsVars(lc_m&nm._corr)
								%let _corrObs_ = &nObs;
								%if &_corrObs_ %then %do;
					            	lc_m&nm._corr
								%end;
							%end;
				        %end;
				        ;
						by outer_loop train_size;
				    run;

					* MERGE LEARNING CURVE ALL MODELS HARRELL TO MAIN OUTPUT;
					%if %sysfunc(exist(&models_comparison_output._HAC)) = 1 %then %do;
						%NObsVars(&models_comparison_output._HAC);
						%if &nObs and &nVars %then %do;
							data &outlib..&models_comparison_output;
								merge &outlib..&models_comparison_output
									  &models_comparison_output._HAC;
							run;
						%end;
					%end;

					* MERGE LEARNING CURVE ALL MODELS CORR TO MAIN OUTPUT;
					%if %sysfunc(exist(&models_comparison_output._CORR)) = 1 %then %do;
						%NObsVars(&models_comparison_output._CORR);
						%if &nObs and &nVars %then %do;
							data &outlib..&models_comparison_output;
								merge &outlib..&models_comparison_output
									  &models_comparison_output._CORR;
							run;
						%end;
					%end;
				%end;
			%end; * END OF INNER_LOOP;
		%end; * END OF OUTER_LOOP;

		* FINAL CLEAR INTERMEDIATE OUTPUT DATA SETS;
		proc datasets library=&outlib nolist;
			delete
			%do outer_loop = &outerloop_start %to &outerloop_end;
				%do inner_loop = 1 %to &n_innerloop;
					%do nm=1 %to &n_models;	
						&lcr_prefix._m&nm._pcum_i&inner_loop._o&outer_loop
					%end;
				%end;
			%end;
			;
		quit;

	%end;
%end;

%mend;

***********************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    MERGE_LLO_RESULTS
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    06/15/07
**  PURPOSE: Merge leave-l-out models ase, acc, auc results
**  HISTORY: action                                        date     who
**           Created.                                      06/22/07 pyt 
** 
**  MACRO PARAMETERS:
**  OUTPUTPATH:   Library path of output data sets 
**  __NOBS: 	  Number of observations in the training set
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**  NCLASSES:	  Number of qualitative trait variable category
**  EVENT:	      Trait event value
**  FREQ:         Frequency variable name (optional)
**  MODEL_NAMES:  List of model names
**                (Example: PLR PLR2 TREE TREEBOOST DISCRIMINANT)
**  N_MODELS: 	  Number of modeling methods
**  LEAVE_L_OUT:  Number of observations to leave out in cross validation	
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**  MODELS_EFFECTS_OUTPUT: Models effects frequencies output 
**  PREVALENCES: Prior probabilities (EQUAL, PROPORTIONAL, or CUSTOM)
**  CV_CUSTOMPRIORS: Custom prior probabilities (PREVALENCE = CUSTOM)
**  GSHARE: Shared grid folder
**
**  NOTES:
**  Compute learning curve models fit statistics: ase, acc, auc
************************************************************************;
%macro merge_llo_results(outputpath=,
						 __nobs=,
						 trait=,
						 trait_type=,
						 nclasses=,
						 event=,
						 freq=,
						 model_names=,    
						 model_labels=,    
						 n_models=,
						 leave_l_out=,
						 models_comparison_output=,
						 models_effects_output=,
					     prevalences=proportional,
						 cv_custompriors=,
						 gshare=
					     );

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;
%if %length(&gshare) %then %do;
    libname gout "&gshare";
    %let itoutlib = gout;
%end;
%else %do;
    %let itoutlib = &cvoutlib;
%end;
%put cvoutlib = &cvoutlib;
%put itoutlib = &itoutlib;

%if &cv_alossprop ne 0.5 %then %do;
	%let cv_alossweight = Asymmetric;
%end;
%else %do;
	%let cv_alossweight = ;
%end;
   
* RETRIEVE MODEL NAMES;
%do i=1 %to &n_models;
    %let mdlname&i = %scan(&model_names,&i," ");
    %let mdllabel&i = %scan(&model_labels,&i,"|");
%end;

* ACCUMULATE RESULTS;
* FIRST, CLEAR OUTPUT DATA SETS TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..cv_baseline_preds;
	delete;
run;
%do nm=1 %to &n_models;
	data &cvoutlib..cv_m&nm._pcum;
		delete;
	run;
	data &cvoutlib..cv_m&nm._vcum;
		delete;
	run;
%end;
%if %upcase(&varimportance) = YES %then %do;
	data &cvoutlib..cv_vipreds_pcum;
		delete;
	run;
	data &cvoutlib..cv_vivars_pcum;
		delete;
	run;
	data &cvoutlib..importance;
		delete;
	run;
	data &cvoutlib..impfit;
		delete;
	run;
%end;

%let cv_pcum_exist=0;
%let cv_vcum_exist=0;
%if &leave_l_out = 3 %then %do;
	%do __i=1 %to &__nobs-2;
		%do __j=&__i+1%to &__nobs-1;
			%do __k=&__j+1 %to &__nobs;	
				%if %sysfunc(exist(&itoutlib..cv_baseline_preds_&__i._&__j._&__k)) = 1 %then %do;
					%NObsVars(&itoutlib..cv_baseline_preds_&__i._&__j._&__k);
					%let _basepObs_ = &nObs;
					%if &_basepObs_ > 0 %then %do;
						%if %sysfunc(exist(&cvoutlib..cv_baseline_preds)) = 1 %then %do;
							%NObsVars(&cvoutlib..cv_baseline_preds);
							%let _cvbpObs_ = &nObs;
							data &cvoutlib..cv_baseline_preds;
								set 
								%if &_cvbpObs_ %then %do;
									&cvoutlib..cv_baseline_preds
								%end;
									&itoutlib..cv_baseline_preds_&__i._&__j._&__k;
							run;
						%end;
						%else %do;
							data &cvoutlib..cv_baseline_preds;
								set &itoutlib..cv_baseline_preds_&__i._&__j._&__k;
							run;
						%end;
					%end;
				%end;
				%do nm=1 %to &n_models;
					%if %sysfunc(exist(&itoutlib..cv_m&nm._preds_&__i._&__j._&__k)) = 1 %then %do;
						%NObsVars(&itoutlib..cv_m&nm._preds_&__i._&__j._&__k);
						%if &nObs and &nVars %then %do;
							%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
								%NObsVars(&cvoutlib..cv_m&nm._pcum);
								%let _cvpcumObs_ = &nObs;
								data &cvoutlib..cv_m&nm._pcum;
									set 
									%if &_cvpcumObs_ %then %do;
										&cvoutlib..cv_m&nm._pcum
									%end;
										&itoutlib..cv_m&nm._preds_&__i._&__j._&__k;
								run;
							%end;
							%else %do;
								data &cvoutlib..cv_m&nm._pcum;
									set &itoutlib..cv_m&nm._preds_&__i._&__j._&__k;
								run;
							%end;
							%let cv_pcum_exist=1;
						%end;
					%end;
					%if %sysfunc(exist(&itoutlib..cv_m&nm._vars_&__i._&__j._&__k)) = 1 %then %do;
						%NObsVars(&itoutlib..cv_m&nm._vars_&__i._&__j._&__k);
						%if &nObs and &nVars %then %do;
							%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
								%NObsVars(&cvoutlib..cv_m&nm._vcum);
								%let _cvvcumObs_ = &nObs;
								data &cvoutlib..cv_m&nm._vcum;
									set 
									%if &_cvvcumObs_ %then %do;
										&cvoutlib..cv_m&nm._vcum
									%end;
									&itoutlib..cv_m&nm._vars_&__i._&__j._&__k;
								run;
							%end;
							%else %do;
								data &cvoutlib..cv_m&nm._vcum;
									set &itoutlib..cv_m&nm._vars_&__i._&__j._&__k;
								run;
							%end;
							%let cv_vcum_exist=1;
						%end;
					%end;
				%end;
			%end;
		%end;
	%end;
%end;
%else %if &leave_l_out = 2 %then %do;
	%do __i=1 %to &__nobs-1;
		%do __j=&__i+1%to &__nobs;
			%if %sysfunc(exist(&itoutlib..cv_baseline_preds_&__i._&__j)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_baseline_preds_&__i._&__j);
				%let _basepObs_ = &nObs;
				%if &_basepObs_ > 0 %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_baseline_preds)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_baseline_preds);
						%let _cvbpObs_ = &nObs;
						data &cvoutlib..cv_baseline_preds;
							set 
							%if &_cvbpObs_ %then %do;
								&cvoutlib..cv_baseline_preds
							%end;
								&itoutlib..cv_baseline_preds_&__i._&__j;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_baseline_preds;
							set &itoutlib..cv_baseline_preds_&__i._&__j;
						run;
					%end;
				%end;
			%end;
			%do nm=1 %to &n_models;
				%if %sysfunc(exist(&itoutlib..cv_m&nm._preds_&__i._&__j)) = 1 %then %do;
					%NObsVars(&itoutlib..cv_m&nm._preds_&__i._&__j);
					%if &nObs and &nVars %then %do;
						%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
							%NObsVars(&cvoutlib..cv_m&nm._pcum);
							%let _cvpcumObs_ = &nObs;
							data &cvoutlib..cv_m&nm._pcum;
								set 
								%if &_cvpcumObs_ %then %do;
									&cvoutlib..cv_m&nm._pcum
								%end;
									&itoutlib..cv_m&nm._preds_&__i._&__j;
							run;
						%end;
						%else %do;
							data &cvoutlib..cv_m&nm._pcum;
								set &itoutlib..cv_m&nm._preds_&__i._&__j;
							run;
						%end;
						%let cv_pcum_exist=1;
					%end;
				%end;
				%if %sysfunc(exist(&itoutlib..cv_m&nm._vars_&__i._&__j)) = 1 %then %do;
					%NObsVars(&itoutlib..cv_m&nm._vars_&__i._&__j);
					%if &nObs and &nVars %then %do;
						%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
							%NObsVars(&cvoutlib..cv_m&nm._vcum);
							%let _cvvcumObs_ = &nObs;
							data &cvoutlib..cv_m&nm._vcum;
								set 
								%if &_cvvcumObs_ %then %do;
									&cvoutlib..cv_m&nm._vcum
								%end;
									&itoutlib..cv_m&nm._vars_&__i._&__j;
							run;
						%end;
						%else %do;
							data &cvoutlib..cv_m&nm._vcum;
								set &itoutlib..cv_m&nm._vars_&__i._&__j;
							run;
						%end;
						%let cv_vcum_exist=1;
					%end;
				%end;
			%end;
		%end;
	%end;
%end;
%else %if &leave_l_out = 1 %then %do;
	%do __i=1 %to &__nobs;
		%if %sysfunc(exist(&itoutlib..cv_baseline_preds_&__i)) = 1 %then %do;
			%NObsVars(&itoutlib..cv_baseline_preds_&__i);
			%let _basepObs_ = &nObs;
			%if &_basepObs_ > 0 %then %do;
				%if %sysfunc(exist(&cvoutlib..cv_baseline_preds)) = 1 %then %do;
					%NObsVars(&cvoutlib..cv_baseline_preds);
					%let _cvbpObs_ = &nObs;
					data &cvoutlib..cv_baseline_preds;
						set 
						%if &_cvbpObs_ %then %do;
							&cvoutlib..cv_baseline_preds
						%end;
							&itoutlib..cv_baseline_preds_&__i;
					run;
				%end;
				%else %do;
					data &cvoutlib..cv_baseline_preds;
						set &itoutlib..cv_baseline_preds_&__i;
					run;
				%end;
			%end;
		%end;
		%do nm=1 %to &n_models;
			%if %sysfunc(exist(&itoutlib..cv_m&nm._preds_&__i)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_m&nm._preds_&__i);
				%if &nObs and &nVars %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_m&nm._pcum);
						%let _cvpcumObs_ = &nObs;
						data &cvoutlib..cv_m&nm._pcum;
							set 
							%if &_cvpcumObs_ %then %do;
								&cvoutlib..cv_m&nm._pcum
							%end;
								&itoutlib..cv_m&nm._preds_&__i;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_m&nm._pcum;
							set &itoutlib..cv_m&nm._preds_&__i;
						run;
					%end;
					%let cv_pcum_exist=1;
				%end;
			%end;
			%if %sysfunc(exist(&itoutlib..cv_m&nm._vars_&__i)) = 1 %then %do;
				%NObsVars(&itoutlib..cv_m&nm._vars_&__i);
				%if &nObs and &nVars %then %do;
					%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
						%NObsVars(&cvoutlib..cv_m&nm._vcum);
						%let _cvvcumObs_ = &nObs;
						data &cvoutlib..cv_m&nm._vcum;
							set 
							%if &_cvvcumObs_ %then %do;
								&cvoutlib..cv_m&nm._vcum
							%end;
								&itoutlib..cv_m&nm._vars_&__i;
						run;
					%end;
					%else %do;
						data &cvoutlib..cv_m&nm._vcum;
							set &itoutlib..cv_m&nm._vars_&__i;
						run;
					%end;
					%let cv_vcum_exist=1;
				%end;
			%end;
		%end;
	%end;
%end;

* SAVE OVERALL LEAVE-L-OUT RUNS PREDICTIONS;
* FIRST, CLEAR OUTPUT DATA SET TO AVOID MERGE WITH PREVIOUS RUNS;
data &cvoutlib..&prefix._cvr_all;
	delete;
run;
%if &cv_pcum_exist=0 %then %do;
	%let exiterror = 1;
	%put ERROR: FAIL TO GENERATE ALL MODELS PREDICTIONS.;
	%goto cv_exit;
%end;
data &cvoutlib..&prefix._cvr_all;
    set 
    %do nm=1 %to &n_models;
		%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
			%NObsVars(&cvoutlib..cv_m&nm._pcum)
			%if &nObs and &nVars %then %do;
		        &cvoutlib..cv_m&nm._pcum
			%end;
		%end;
    %end;
    ;
run;
%if (&CV_ListAll = Yes) %then %do;
    %do nm=1 %to &n_models;
		%if %sysfunc(exist(&cvoutlib..cv_m&nm._pcum)) = 1 %then %do;
			%NObsVars(&cvoutlib..cv_m&nm._pcum);
			%if &nObs and &nVars %then %do;
		        proc contents data=&cvoutlib..cv_m&nm._pcum;
		        run;
			%end;
		%end;
    %end;
%end;

* COPY _NAMEDAT FROM itoutlib to cvoutlib;
%if %upcase(&trait_type)=QUALITATIVE and %length(&gshare) %then %do;
	data &cvoutlib.._namedat;
		set &itoutlib.._namedat;
	run;
%end;

* COMPUTE OVERALL BASELINE ASE, ACC, AEC, AUC;
%if %sysfunc(exist(&cvoutlib..cv_baseline_preds)) = 1 %then %do;
	%NObsVars(&cvoutlib..cv_baseline_preds);
	%if &nObs and &nVars %then %do;
		* COMPUTE LEAVE-L-OUT BASELINE MODELS FIT STATISTICS: ASE, ACC, AEC, AUC;
		%if %upcase(&trait_type)=QUALITATIVE %then %do;

		    * COMPUTE BASELINE ASE;
		    %compute_ase_nominal(data=&cvoutlib..cv_baseline_preds,
		                         output=BASELINE_ASE,
		                         actual=actualp,
		                         nclasses=&nclasses,
		                         mdlname=BASELINE,
		                         freq=&freq,
			                     prevalences=&prevalences,
								 cv_custompriors=%nrbquote(&cv_custompriors),
							     trait=&trait
			                     );
		    %compute_acc_nominal(data=&cvoutlib..cv_baseline_preds,
		                         output=BASELINE_ACC,
		                         trait=&trait,
		                         correct=correct,
		                         mdlname=BASELINE,
		                         freq=&freq,
			                     prevalences=&prevalences,
								 cv_custompriors=%nrbquote(&cv_custompriors)
			                     );

			%if %length(&cv_customcosts) %then %do;
				 %compute_aec_nominal(data=&cvoutlib..cv_baseline_preds,
											 output=BASELINE_AEC,
											 trait=&trait,
											 expcost=expected_cost,
											 mdlname=BASELINE,
											 freq=&freq,
											);
			%end;

		    %if &nclasses = 2 %then %do;
		        %locate_pevent_isevent(data=&cvoutlib..cv_baseline_preds,
				                       trait=&trait,
				                       event=&event,
				                       actual=actualp);

		        * COMPUTE BASELINE MANN-WHITNEY STATISTIC;
		        %compute_mann(data=&cvoutlib..cv_baseline_preds,
		                      pevent=_pevent,
		                      isevent=_isevent);
		        %put mann=&mann;
		        data BASELINE_MANN;
		            length METHOD $64;
		            AUC=&mann;
		            METHOD="BASELINE";
		        run;
		    %end;
		%end;
		%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;

			%if &cv_alossprop ne 0.5 %then %do;
				data &cvoutlib..cv_baseline_preds;
					set &cvoutlib..cv_baseline_preds;
					_resid = &trait - Predicted_Value;
					if missing(_resid) then &cv_alossweight = .;
					else if _resid >= 0 then &cv_alossweight = 2 * &cv_alossprop;
					else &cv_alossweight = 2 * (1 - &cv_alossprop);
				run;
			%end;

		    * COMPUTE OVERALL BASELINE ASE;
		    %compute_ase_interval(data=&cvoutlib..cv_baseline_preds,
		                          output=BASELINE_ASE,
		                          y=&trait,
		                          predict=Predicted_Value,
		                          mdlname=BASELINE,
		                          freq=&freq,
		                          aweight=&cv_alossweight);
		                          
	%* put ERROR: 7.********** data = &cvoutlib..cv_baseline_preds;
		    * COMPUTE OVERALL BASELINE HARRELL;
			 %compute_harrellc(data=&cvoutlib..cv_baseline_preds,
							       output=BASELINE_HAC,
							       TimeVar=&trait,
							       CenVar=,
							       Predict=Predicted_Value,
							       Freq=&cv_alossweight);
			 data baseline_hac;
			 	 length METHOD $64;
				 set baseline_hac;
				 METHOD="BASELINE";
			 run;

		    * COMPUTE OVERALL BASELINE CORR;
		    %compute_corr_interval(data=&cvoutlib..cv_baseline_preds,
		                           output=BASELINE_CORR,
		                           y=&trait,
		                           predict=Predicted_Value,
		                           mdlname=BASELINE,
		                           freq=&freq,
		                           weight=&cv_alossweight);

		%end;
	%end;
%end;

* options notes;
%put trait=&trait;
%put trait_type=&trait_type;
%put event=&event;
%put freq=&freq;
%put n_models=&n_models;
%put leave_l_out=&leave_l_out;
%put where_clause=;
%put prevalences=&prevalences;
%put cv_custompriors=%nrbquote(&cv_custompriors);

* COMPUTE LEAVE-L-OUT MODELS ASE, ACC, AUC;
%lo_fitstats(trait=&trait,
			 trait_type=&trait_type,
			 event=&event,
			 freq=&freq,
			 n_models=&n_models,
			 leave_l_out=&leave_l_out,
			 where_clause=,
			 prevalences=&prevalences,
			 cv_custompriors=%nrbquote(&cv_custompriors)
			 );

* APPEND BASELINE ASE WITH LEAVE_L_OUT MODELS ASE;
data &cvoutlib..&models_comparison_output;
    set
        BASELINE_ASE
        %do nm=1 %to &n_models;
			%if %sysfunc(exist(cv_m&nm._ase)) = 1 %then %do;
				%NObsVars(cv_m&nm._ase)
				%if &nObs and &nVars %then %do;
	            	cv_m&nm._ase
				%end;
			%end;
        %end;
        ;
    RMSE = sqrt(ASE);
run;

%if %upcase(&trait_type)=QUALITATIVE %then %do;
	* APPEND BASELINE ACC WITH LEAVE_L_OUT MODELS ACC;
    data &models_comparison_output._ACC;
        set
            BASELINE_ACC
            %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._acc)) = 1 %then %do;
					%NObsVars(cv_m&nm._acc)
					%if &nObs and &nVars %then %do;
	                	cv_m&nm._acc
					%end;
				%end;
            %end;
            ;
    run;

	* MERGE LEAVE_L_OUT MODELS ACC, ASE, AND BASELINE ASE;
	%if %sysfunc(exist(&models_comparison_output._ACC)) = 1 %then %do;
		%NObsVars(&models_comparison_output._ACC);
		%if &nObs and &nVars %then %do;
		    data &cvoutlib..&models_comparison_output;
		        merge &cvoutlib..&models_comparison_output
		              &models_comparison_output._ACC;
		    run;
		%end;
	%end;

	%if %length(&cv_customcosts) %then %do;
		* APPEND BASELINE AEC WITH LEAVE_L_OUT MODELS AEC;
		 data &models_comparison_output._aec;
			  set
					BASELINE_aec
					%do nm=1 %to &n_models;
					%if %sysfunc(exist(cv_m&nm._aec)) = 1 %then %do;
						%NObsVars(cv_m&nm._aec)
						%if &nObs and &nVars %then %do;
								cv_m&nm._aec
						%end;
					%end;
					%end;
					;
		 run;

		* MERGE LEAVE_L_OUT MODELS aec, ASE, AND BASELINE ASE;
		%if %sysfunc(exist(&models_comparison_output._aec)) = 1 %then %do;
			%NObsVars(&models_comparison_output._aec);
			%if &nObs and &nVars %then %do;
				 data &cvoutlib..&models_comparison_output;
					  merge &cvoutlib..&models_comparison_output
							  &models_comparison_output._aec;
				 run;
			%end;
		%end;
	%end;

    %if &nclasses = 2 %then %do;
		* APPEND BASELINE MANN-WHITNEY AUC WITH LEAVE_L_OUT MODELS AUC;
	    data &models_comparison_output._AUC;
	        set
	            BASELINE_MANN
	            %do nm=1 %to &n_models;
					%if %sysfunc(exist(cv_m&nm._mann)) = 1 %then %do;
						%NObsVars(cv_m&nm._mann)
						%if &nObs and &nVars %then %do;
			                cv_m&nm._mann
						%end;
					%end;
	            %end;
	            ;
	    run;
		* MERGE LEAVE_L_OUT MODELS AUC, ACC, ASE, AND BASELINE ASE;
		%if %sysfunc(exist(&models_comparison_output._AUC)) = 1 %then %do;
			%NObsVars(&models_comparison_output._AUC);
			%if &nObs and &nVars %then %do;
			    data &cvoutlib..&models_comparison_output;
			        merge &cvoutlib..&models_comparison_output
			              &models_comparison_output._AUC;
			    run;
			%end;
		%end;
	%end;
%end;
%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;

	* APPEND BASELINE HARRELL WITH LEAVE_L_OUT MODELS HARRELL;
   data &models_comparison_output._HAC;
       set
            BASELINE_HAC
            %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._hac)) = 1 %then %do;
					%NObsVars(cv_m&nm._hac)
					%if &nObs and &nVars %then %do;
		                cv_m&nm._hac
					%end;
				%end;
            %end;
            ;
   run;
	* MERGE LEAVE_L_OUT MODELS HARRELL, AUC, ACC, ASE, AND BASELINE ASE;
	%if %sysfunc(exist(&models_comparison_output._HAC)) = 1 %then %do;
		%NObsVars(&models_comparison_output._HAC);
		%if &nObs and &nVars %then %do;
		    data &cvoutlib..&models_comparison_output;
		        merge &cvoutlib..&models_comparison_output
		              &models_comparison_output._HAC;
		    run;
		%end;
	%end;

	* APPEND BASELINE HARRELL WITH LEAVE_L_OUT MODELS HARRELL;
   data &models_comparison_output._CORR;
       set
            BASELINE_CORR
            %do nm=1 %to &n_models;
				%if %sysfunc(exist(cv_m&nm._corr)) = 1 %then %do;
					%NObsVars(cv_m&nm._corr)
					%if &nObs and &nVars %then %do;
		                cv_m&nm._corr
					%end;
				%end;
            %end;
            ;
   run;
	* MERGE LEAVE_L_OUT MODELS HARRELL, AUC, ACC, ASE, AND BASELINE ASE;
	%if %sysfunc(exist(&models_comparison_output._CORR)) = 1 %then %do;
		%NObsVars(&models_comparison_output._CORR);
		%if &nObs and &nVars %then %do;
		    data &cvoutlib..&models_comparison_output;
		        merge &cvoutlib..&models_comparison_output
		              &models_comparison_output._CORR;
		    run;
		%end;
	%end;

%end;

* merge in model labels;
data lab;
	length Model $ 64 Method $ 32;
	%do i=1 %to &n_models;
		Model = "&&mdllabel&i";
		Method = "&&mdlname&i";
		output;
	%end;
run;
	
data &cvoutlib..&models_comparison_output;
	merge lab &cvoutlib..&models_comparison_output;
	by Method;
	if Method="BASELINE" then Model="Baseline";
run;

* COMPUTE EFFECTS FREQUENCIES;
%if %sysfunc(exist(&cvoutlib..&prefix._cve_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cve_all);
	%if &nObs or &nVars %then %do;
		proc datasets library=&cvoutlib nolist;
			delete &prefix._cve_all;
		quit; 
	%end;
%end;

%let cv_vcum_exist=0;
%do nm=1 %to &n_models;
	%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
		%NObsVars(&cvoutlib..cv_m&nm._vcum)
		%if &nObs and &nVars %then %do;
	        %let cv_vcum_exist=1;
		%end;
	%end;
%end;
%if &cv_vcum_exist=0 %then %do;
	%put NOTE: NO PREDICTOR VARIABLES ARE SELECTED FROM ALL MODELS.;
%end;
%else %do;
	data &cvoutlib..&prefix._cve_all;
	    length METHOD $64 NAME $32;
	    set 
	    %do nm=1 %to &n_models;
			%if %sysfunc(exist(&cvoutlib..cv_m&nm._vcum)) = 1 %then %do;
				%NObsVars(&cvoutlib..cv_m&nm._vcum)
				%if &nObs and &nVars %then %do;
		        	&cvoutlib..cv_m&nm._vcum
				%end;
			%end;
	    %end;
	    ;
	run;
	%if %sysfunc(exist(&cvoutlib..&prefix._cve_all)) = 1 %then %do;
		%NObsVars(&cvoutlib..&prefix._cve_all);
		%if ^&nObs or ^&nVars %then %do;
			%put ERROR: &cvoutlib..&prefix._cve_all CONTAINS 0 OBSERVATIONS.;
			%let exiterror = 1;
			%goto cv_exit;		
		%end;
	%end;
	%else %do;
		%put ERROR: &cvoutlib..&prefix._cve_all DOES NOT EXIST.;
		%let exiterror = 1;
		%goto cv_exit;	
	%end;
	%compute_effects(outputpath=&outputpath,
					 cv_method=&cv_method,
					 leave_l_out=&leave_l_out,
					 models_comparison_output=&models_comparison_output,
					 models_effects_output=&models_effects_output);
%end;

* FINAL CLEAR INTERMEDIATE OUTPUT DATA SETS;
proc datasets library=&cvoutlib nolist;
	delete
	lcgridseq
	cv_baseline_preds
	%do nm=1 %to &n_models;
		cv_m&nm._pcum
		cv_m&nm._vcum
	%end;
	%if %upcase(&varimportance) = YES %then %do;
		cv_vipreds_pcum
		cv_vivars_pcum
	%end;
quit;

proc datasets library=&itoutlib nolist;
  	delete
	%if &leave_l_out = 3 %then %do;
		%do __i=1 %to &__nobs-2;
			%do __j=&__i+1%to &__nobs-1;
				%do __k=&__j+1 %to &__nobs;	
					cv_baseline_preds_&__i._&__j._&__k
					%do nm=1 %to &n_models;
						cv_m&nm._preds_&__i._&__j._&__k
						cv_m&nm._vars_&__i._&__j._&__k
					%end;
				%end;
			%end;
		%end;
	%end;	
	%else %if &leave_l_out = 2 %then %do;
		%do __i=1 %to &__nobs-1;
			%do __j=&__i+1%to &__nobs;
				cv_baseline_preds_&__i._&__j
				%do nm=1 %to &n_models;
					cv_m&nm._preds_&__i._&__j
					cv_m&nm._vars_&__i._&__j
				%end;
			%end;
		%end;
	%end;
	%else %if &leave_l_out = 1 %then %do;
		%do __i=1 %to &__nobs;	
			cv_baseline_preds_&__i
			%do nm=1 %to &n_models;
				cv_m&nm._preds_&__i
				cv_m&nm._vars_&__i
			%end;
		%end;
	%end;
	;
quit;

%cv_exit:;
%mend;

%macro preselect_predictors(datapath=,
						    outputpath=,
							dsname=,
						    model_name=,
							pred_vars_dsname=
					        );

* DETERMINE WHETHER DATA PATH IS SPECIFIED, IF SO ALLOCATE IT,
  AND CHANGE THE TRAINING DATA SET MACRO VARIABLE ACCORDINGLY;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname inlib "&datapath";
	%let inlib = inlib;
%end;
%else %do;
    %let inlib = work;
%end;

%if %length(&dsname) %then %do;
    %let __ppdsname = &inlib..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__ppdsname)) = 1 %then %do;
	%NObsVars(&__ppdsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__ppdsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__ppdsname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
	libname output "&outputpath";
	%let cvoutlib = output;
%end;
%else %do;
	%let cvoutlib = work;
%end;

%fitmodels(train_dsname=&__ppdsname,
		   test_dsname=);

%if &FM_Error1 = 1 %then %do;
	%let exiterror = 1;
	%put ERROR: &model_name RUN FAILED.;
	%goto exit; 
%end;

* CHECK FOR cv_&model_name._vars EXIST;
* IF 0 OBSERVATION FOUND, EXIT; 
%if %sysfunc(exist(cv_&model_name._vars)) = 1 %then %do;
	%NObsVars(cv_&model_name._vars);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put NO IMPORTANT VARIABLES ARE FOUND.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: cv_&model_name._vars DOES NOT EXIST.;
	%goto exit;
%end;

data &cvoutlib..&pred_vars_dsname;
	set cv_&model_name._vars;
run;

%if %sysfunc(exist(&cvoutlib..metadata_variables)) = 1 %then %do;
	proc datasets library=&cvoutlib nolist;
		delete metadata_variables;
	quit; 
%end;

%metadata_variables(datapath=&datapath,
					outputpath=&outputpath,
					dsname=&dsname,
					model_name=&model_name,	 
					quantitative_vars=&PredictorVars,
					qualitative_vars=&ClassPredVars,
					ordinal_vars=,
					list_style_quantitative_vars=&ListContPredVars,
					list_style_qualitative_vars=&ListClassPredVars,
					list_style_ordinal_vars=);

data &cvoutlib..metadata_variables;
	set &cvoutlib..metadata_variables;
	label MODEL="&model_name";
run;

* PREPARE DropPredVars;
data cv_&model_name._vars;                                                                                                 
	set cv_&model_name._vars;                                                                                        
	UNAME=upcase(NAME);                                                                                                                
	keep UNAME IMPORTANCE;                                                                                                             
run;                                                                                                                                    
proc sort data=cv_&model_name._vars;                                                                                          
	by UNAME;                                                                                                                          
run;                                                                                                                                    
data metadata_variables;                                                                                                                
	set &cvoutlib..metadata_variables;                                                                                                     
	UNAME=upcase(NAME);                                                                                                                
run;                                                                                                                                    
proc sort data=metadata_variables;                                                                                                      
	by UNAME;                                                                                                                          
run;                                                                                                                                    
data preselected_vars;                                                                                                                                 
	merge metadata_variables                                                                                                           
	   cv_&model_name._vars;                                                                                                
	by UNAME;  
	if not missing(IMPORTANCE) then output; 
run;       

* MODIFY PredictorVars, ClassPredVars, ListContPredVars, ListClassPredVars 
* BASE ON THE INITIAL MODEL SELECTED PREDICTORS;
%if %sysfunc(exist(preselected_vars)) %then %do;
	%NObsVars(preselected_vars);
	%let _varsObs_ = &nObs;
	%if &_varsObs_ %then %do;
		%global NewPredictorVars;
		%global NewClassPredVars;
		%global NewListContPredVars;
		%global NewListClassPredVars;

		* SEPARATE QUANTITATIVE AND QUALITATIVE VARIABLES INTO DIFFERENT DATA SETS;
		data quantitative qualitative;
		 	keep NAME MODEL;
		 	set preselected_vars;
			if upcase(trim(left(MODEL))) = "QUANTITATIVE" or 
			   upcase(trim(left(MODEL))) = "ORDINAL" then 
				output quantitative;
			else if upcase(trim(left(MODEL))) = "QUALITATIVE" then
				output qualitative;
		run;
		%if %sysfunc(exist(work.quantitative)) %then %do;
			%NObsVars(work.quantitative);
			%let _qtObs_ = &nObs;
			%if &_qtObs_ %then %do;
				data _NULL_;
					length var_list $32767;
					set quantitative end=end;
					retain var_list "";
					var_list=left(trim(var_list))||""||trim(left(NAME));
					if end ne 0 then			
					 	call symput("NewPredictorVars", trim(left(var_list)));
				run;
			%end;
		%end;
		%if %sysfunc(exist(work.qualitative)) %then %do;
			%NObsVars(work.qualitative); 
			%let _qlObs_ = &nObs;
			%if &_qlObs_ %then %do;
				data _NULL_;
					length var_list $32767;
					set qualitative end=end;
					retain var_list "";
					var_list=left(trim(var_list))||""||trim(left(NAME));
					if end ne 0 then			
					 	call symput("NewClassPredVars", trim(left(var_list)));
				run;
			%end;
		%end;
		data _null_;
			call symput("NewListContPredVars", "  ");
			call symput("NewListClassPredVars", "  ");
		run;
	%end;
%end;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2006 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    PSEUDO
**  AUTHOR:  saspyt, Pei-Yi Tan
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    05/15/06
**  PURPOSE: Creates pseudo variables with values 1, 2, ...., kfold.
**  HISTORY: Action                                             date who
**           Created.                                       05/15/06 pyt 
** 
**  MACRO PARAMETERS:
**  DSNAME:   Original data set name
**  KFOLD:    K-fold cross validation 
**  OUTPUT:   Output data set name which contains pseudo variable
**	RANDSEED: Random seed number
**
**  NOTES:
************************************************************************;
%macro pseudo(dsname=, 
			  kfold=, 
			  output=, 
			  seed=);

%if %sysfunc(exist(&dsname)) = 1 %then %do;
	%NObsVars(&dsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &dsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &dsname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE NOBS IN THE DATA SET;
data _null_;
     set &dsname nobs=nobs;
     call symput("N", left(put(nobs, 12.0)));
run;

* CREATE PSEUDO VARIABLE WITH VALUES 1, 2, ..., KFOLD ;
data &output;
     keep pseudo;
     array holdout{&kfold} ;
     array nwant{&kfold} ;
     array foldtodo{&kfold};
     retain nremain &kfold;

     ntotal   = %eval(&N);
     nfolds   = %eval(&kfold);
     nperfold = floor(ntotal/nfolds);
     nextra   = ntotal - nperfold * nfolds;

     do i = 1 to nfolds;
        holdout{i}  = 0;
        foldtodo{i} = i;
        nwant{i}    = nperfold;
        if i <= nextra then nwant{i} = nwant{i} + 1;
     end;

     call streaminit(&seed);

     do i=1 to ntotal;
         a = rand("uniform") * nremain;
         b = floor(a);
         if b eq nremain then b = b - 1;
         b + 1;
         pseudo   = foldtodo{b};
         holdout{pseudo} + 1;

         output;

         if holdout{pseudo} >= nwant{pseudo} then do;
            if b < nremain then do;
               c  = nremain - 1;
               do j = b to c;
                  k = j + 1;
                  foldtodo{j} = foldtodo{k};
               end;
            end;
            nremain = nremain - 1;
         end;
     end;
run;

%exit:;
%mend;

* CREATE DATA SET CONTAINING UNIFORM RANDOM NUMBERS;
%macro pseudouniform(dsname=, output=, seed=);

data _null_;
     set &dsname nobs=nobs;
     call symput("N", left(put(nobs, 12.0)));
run;

data &output;
     keep pseudo;
     ntotal   = %eval(&N);

     call streaminit(&seed);

     do i=1 to ntotal;
         pseudo = rand("uniform");
         output;
     end;
run;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    RANDOM_SAMPLE
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Generate random sample  
**  HISTORY: action                                            date who
**           Created.                                      02/19/06 pyt 
**
**  MACRO PARAMETERS:
**  DATAPATH:     	 Library path of training data set 
**  DSNAME:      Data set name to be sampled
**  SAMPLE_SIZE: Sample size
**  SEED:        Random seed
**
**  NOTES: 
************************************************************************;
%macro random_sample(datapath=,
				     dsname=, 
					 sample_size=, 
					 seed=);

%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname in "&datapath";
	%let in = in;
%end;
%else %do; 
	%let in = work;
%end;

%if %length(&dsname) %then %do;
    %let __rsdsname = &in..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__rsdsname)) = 1 %then %do;
	%NObsVars(&__rsdsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__rsdsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__rsdsname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE NOBS IN THE DATA SET;
data _null_;
    set &__rsdsname nobs=nobs;
    call symput("N0", left(put(nobs, 12.0)));
run;

data pseudo;
     keep pseudo;
     retain output0 output1 0;
     N_0 = &sample_size;
     N_1 = &N0 - N_0;

     call streaminit(&seed);
    
     do i=1 to &N0;
        if output0 < N_0 and output1 < N_1 then do;
           p1 = (N_1 - output1) / (N_0 + N_1 - output0 - output1);
           pseudo = rand("bern", p1);
        end;
        else if output0 < N_0 then do;
           pseudo = 0;
        end; 
        else
           pseudo = 1;

        if pseudo eq 1 then output1 + 1;
        else output0 + 1;
        output;
     end;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2008 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    RELIABILITY_DIAGRAM
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    05/12/08
**  PURPOSE: Generate Reliability Diagram  
**  HISTORY: action                                            date who
**           Created.                                      05/12/08 pyt 
**
**  MACRO PARAMETERS:
**  cvoutlib:  	  Libref of output data sets 
**  TRAIT:        Trait variable name
**  TRAIT_TYPE:   Trait variable type: 
**                qualitative(categorical), 
**                quantitative(interval)
**				  survival (only applicable for survival data)
**	CENVAR:	  	  Censor variable of survival data	
**  CENVAL: 	  Censor value indicates if the record is censored
**  MODELS_COMPARISON_OUTPUT: Models comparison fit statistics output
**
**  NOTES: 
************************************************************************;
%macro reliability_diagram(cvoutlib=, 
						   trait=,
						   trait_type=,
						   cenvar=,
						   cenval=,
						   models_comparison_output=);

%let prefix=%substr(&MODELS_COMPARISON_OUTPUT,1,%eval(%length(&MODELS_COMPARISON_OUTPUT)-4));
%put prefix=&prefix;
%if %sysfunc(exist(&cvoutlib..&prefix._cvr_all)) = 1 %then %do;
	%NObsVars(&cvoutlib..&prefix._cvr_all);
	%if ^&nObs or ^&nVars %then %do;
		%put ERROR: &cvoutlib..&prefix._cvr_all CONTAINS 0 OBSERVATIONS.;
		%let exiterror = 1;
		%goto exit;		
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &cvoutlib..&prefix._cvr_all DOES NOT EXIST.;
	%goto exit;
%end;

%if %upcase(&trait_type)=QUALITATIVE %then %do;
 
	* RETRIEVE P_PROB and MAPPING CLASS FROM _NAMEDAT DATA SET;	
	%put cvoutlib=&cvoutlib;
	%let _nid_ = %sysfunc(open(&cvoutlib.._namedat));
	%put _nid_ = &_nid_;
	%let nclasses = %sysfunc(attrn(&_nid_, NOBS));
	%put nclasses = &nclasses;
	%let classnum = %sysfunc(varnum(&_nid_, CLASS));
	%let probnum = %sysfunc(varnum(&_nid_, P_PROB));
	%do c=1 %to &nclasses;
		%let rc = %sysfunc(fetchobs(&_nid_, &c));
		%let class&c = %sysfunc(getvarc(&_nid_, &classnum));
		%let p_prob&c = %sysfunc(getvarc(&_nid_, &probnum));
		%put class&c = &&class&c;
		%put p_prob&c = &&p_prob&c;
	%end;
	%let _nid_ = %sysfunc(close(&_nid_));

	data pvar;
		set &cvoutlib..&prefix._cvr_all;
		keep METHOD &trait P_: ;
	run;
	
	%let _tid_ = %sysfunc(open(pvar));
	%let traitnum=%sysfunc(varnum(&_tid_, &trait));
	%let traitype=%sysfunc(vartype(&_tid_, &traitnum));
   %let _tid_ = %sysfunc(close(&_tid_));

	data bin;
		set pvar;
		keep METHOD &trait P_: bin:;
		array bin{&nclasses}
		%do c=1 %to &nclasses;
			bin&c
		%end;
		;
		range=1/10;
		%do c=1 %to &nclasses;
			%let p_var = &&p_prob&c;
			bin&c=floor(((&p_var-0)/range)+1);
			if bin&c > 10 then bin&c = 10;
		%end;
	run;	

	%do c=1 %to &nclasses;
		%let p_var=&&p_prob&c;
		%let class=&&class&c;
		proc means data=bin noprint;
			class method bin&c;
			var &p_var;
			types () method*bin&c;
			output out=r&c(where=(method ne "")) mean(&p_var)=MEAN_pvar;
		run;
		data r&c;
			set r&c;
			drop _TYPE_ _FREQ_;
			rename bin&c = bin;
			label MEAN_pvar="Mean Predicted";
		run;

		proc sort data=bin out=sbin;
			by method bin&c;
		run;
		proc freq data=sbin noprint;
			by method bin&c;
			%if &traitype = C %then %do;
				tables &trait/out=frq&c(where=(&trait="&class"));
			%end;
			%else %do;
				tables &trait/out=frq&c(where=(&trait=&class));
			%end;
		run;
		data frq&c;
			set frq&c;
			F = PERCENT/100;
			rename bin&c = bin;
			label COUNT = "Count";
			label F = "Fraction";
			keep METHOD COUNT bin&c F;
		run;
		data rdiagram&c;
			merge r&c frq&c;
			by method bin;
			%if &traitype = C %then %do;
				length &trait $ 32;
				&trait="&class";
			%end;
			%else %do;
				&trait=&class;
			%end;		
		run;
		%if &c=1 %then %do;
			data &cvoutlib..&prefix._rdiagram;
				set rdiagram&c;
			run;
		%end;
		%else %do;
			data &cvoutlib..&prefix._rdiagram;
				set &cvoutlib..&prefix._rdiagram
					rdiagram&c;
			run;
		%end;
	%end;
%end;
%else %if %upcase(&trait_type)=QUANTITATIVE %then %do;
	proc means data=&cvoutlib..&prefix._cvr_all noprint;
		class method;
		var predicted_value;
		output out=mean max=maxp min=minp range=diffp;
	run;

	data mean;
		set mean;
		if method ne "" then output;
		drop _TYPE_ _FREQ_;
	run;

	data cvr_all;
		merge &cvoutlib..&prefix._cvr_all mean;
		by method;
	run;

	data bin;
		set cvr_all;
		range=diffp/10;
		bin=floor(((predicted_value-minp)/range)+1);
		if bin > 10 then bin = 10;
	run;

	proc means data=bin noprint;
		class method bin;
		var &trait predicted_value;
		types () method*bin;
		output out=rdiagram(where=(method ne ""))
			mean(&trait)=mean_&trait 
		    mean(predicted_value)=mean_pred;
	run;

	data &cvoutlib..&prefix._rdiagram;
		set rdiagram;
		label mean_&trait="Trait Mean";
	    label mean_pred="Predicted for Mean";
		drop _TYPE_;
	run;
%end;
%else %if %upcase(&trait_type)=SURVIVAL %then %do;

	proc means data=&cvoutlib..&prefix._auc_all noprint; 
		class method;
		var _AUC_;                                                                                                                              
		output out=mean max=maxp min=minp range=diffp;                                                                                          
	run;                                                                                                                                    
	          
	data mean;
		set mean;
		if method ne "" then output;
		drop _TYPE_ _FREQ_;
	run;

	data auc_all;
		merge &cvoutlib..&prefix._auc_all mean;
		by method;
	run;
 
	data bin;
		set auc_all;
		keep method bin &trait &CenVar _Obs _AUC_;
		range=diffp/10;
		bin=floor(((_AUC_-minp)/range)+1);
		if bin > 10 then bin = 10;
	run;

	* COMPUTE PREDICTED AUC FOR EACH BIN BY METHOD;
 	proc means data=bin noprint;                                                                                                        
        class method bin;                                                                                                           
        var _AUC_;                                                                                                                  
        types () method*bin;                                                                                                        
        output out=rdiagram(where=(method ne ""))                                                                                   
        mean(_AUC_)=MEAN_P_AUC;                                                                                             
 	run;   

	data rdiagram;
		set rdiagram;
		label MEAN_P_AUC = "Predicted AUC Mean";
	run;

	proc sort data=rdiagram(drop= _TYPE_ _FREQ_);
		by METHOD BIN;
	run;

	* COMPUTE ACTUAL AUC FOR EACH BIN BY EACH METHOD;
	* FIRST, COMPUTE KAPLAN-MEIER ESTIMATE FOR EACH BIN BY EACH METHOD;
	proc sort data=bin;
		by method bin;
	run;

	%if %length(%trim(%left(&CenVar))) %then %do;
 		*REPLACE CHARACTER CENSOR VARIABLE TO NUMERIC; 
		%let binid = %sysfunc(open(bin));
		%let cenum = %sysfunc(varnum(&binid, &cenvar));
		%let centype = %sysfunc(vartype(&binid, &cenum));
		%let binid = %sysfunc(close(&binid));
		%if &centype = C %then %do;                                                                                                            
			data bin;                                                                                                                       
				set bin;                                                                                                                  
				length &cenvar.n 8;                                                                                                         
				&cenvar.n=&cenvar;                                                                                                          
				drop &cenvar;                                                                                                               
				rename &cenvar.n=&cenvar;                                                                                                   
			run;                                                                                                                              
		%end;     
	%end;
	
	proc lifetest data=bin outsurv=surv noprint;
		by method bin;
		%if %length(%trim(%left(&CenVar))) %then %do;
			time &trait*&CenVar(&Cenval);
		%end;
		%else %do;
			time &trait;
		%end;
	run;

	%if %length(%trim(%left(&CenVar))) %then %do;
	* MERGE _Obs back in surv;
	data surv;
		set surv;
		rename _CENSOR_=&CenVar;
	run;
	%end;

	data surv;                                                                                                                                    
        set surv;                                                                                                                               
        retain _Obs _bin;                                                                                                                       
                                                                                                                                                
        drop _bin;                                                                                                                              
        if _n_ = 1 then do;                                                                                                                     
           _bin = bin;                                                                                                                          
           _Obs = 1;                                                                                                                            
                                                                                                                                                
        end;                                                                                                                                    
        else do;                                                                                                                                
            if _bin ne bin  then do;                                                                                                            
                _bin = bin;                                                                                                                     
               _Obs+1;                                                                                                                          
           end;                                                                                                                                 
        end;                                                                                                                                    
    run;    
                                                                                                                                        
	%compute_survival_auc(data=surv,
						  output=surv_auc,
						  TimeVar=&trait,
						  CenVar=&CenVar);
	
	* MERGE METHOD AND BIN BACK IN SURV_AUC;
	proc freq data=surv noprint;
		table method*bin*_Obs/out=sfq(drop=COUNT PERCENT);
	run;

	proc sort data=sfq;
		by descending _Obs;
	run;
	
	data surv_auc;
		merge surv_auc sfq;
		rename _AUC_ = MEAN_A_AUC;
		label _AUC_ = "Actual AUC Mean";
	run;
	
	* MERGE PREDICTED AUC MEAN WITH ACTUAL AUC MEAN;
	proc sort data=surv_auc;
		by METHOD BIN;
	run;

	data &cvoutlib..&prefix._rdiagram;
		merge rdiagram surv_auc;
		by METHOD BIN;
	run;
                                                                                                                            
%end;

* add model labels;
proc sort data=&cvoutlib..&prefix._rdiagram;
	by METHOD;
run;
data &cvoutlib..&prefix._rdiagram;
	merge lab &cvoutlib..&prefix._rdiagram;
	by METHOD;
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007 by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    STRATIFIED_SAMPLE
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Generate stratified random sample  
**  HISTORY:  action                                            date who
**            Created.                                      02/19/06 pyt 
**
**  MACRO PARAMETERS:
**  DATAPATH:     	Library path of training data set 
**  OUTPUTPATH:   	Library path of output data sets 
**  DSNAME:          Data set name to be sampled
**  TRAIT:           Trait response variable name
**  TRAIT_TYPE:      Trait variable type: qualitative or quantitative
**  TRAIN_DSNAME:    Sample training data set name
**  TEST_DSNAME:     Sample test data set name
**  HOLDOUT_PERCENT: Holdout percentage 
**  GROUPNAME        Grouping variable name
**  SEED:			   Random seed 
**  ITER:				Iteration
**  K:					K within iteration
**
**  NOTES: 
**  Use holdout_percent to calculate sample size.
**  Stratified sample only applies to categorical trait variable.
************************************************************************;
%macro stratified_sample(datapath=,
						 outputpath=,
					    dsname=, 
						 trait=, 
						 trait_type=, 
						 holdout_percent=, 
						 groupname=,
						 train_dsname=,
						 test_dsname=,
						 seed=,
						 iter=,
						 k=,
						 nbin=5);

%put entering stratified_sample with holdout_percent = &holdout_percent;
%if %length(&datapath) > 0 and "&datapath" ne "" %then %do;
	libname in "&datapath";
	%let in = in;
%end;
%else %do; 
	%let in = work;
%end;

%if %length(&dsname) %then %do;
    %let __ssdsname = &in..&dsname;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: dsname MACRO VARIABLE IS NOT SPECIFIED.;
	%goto exit;
%end;

%if %sysfunc(exist(&__ssdsname)) = 1 %then %do;
	%NObsVars(&__ssdsname);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &__ssdsname CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &__ssdsname DOES NOT EXIST.;
	%goto exit;
%end;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname out "&outputpath";
    %let out = out;
%end;
%else %do;
    %let out = work;
%end;

* KEEP TRAIT WITH NON-MISSING VALUES;  
%let _tid_ = %sysfunc(open(&__ssdsname));
%let _tObs_ = %sysfunc(attrn(&_tid_, NOBS));
%if &_tObs_ %then %do;
	%let traitnum=%sysfunc(varnum(&_tid_, &trait));
	%let traitype=%sysfunc(vartype(&_tid_, &traitnum));
%end;
%let _tid_ = %sysfunc(close(&_tid_));

* KEEP TRAIT WITH NON-MISSING VALUES AND APPLY SPECIFIED WHERE_CLAUSE;  
data temp(keep=&trait __id &groupname) ds;
	set &__ssdsname;
	__id = _n_;
	%if &traitype = C %then %do; 
		if &trait ne " " then output;
	%end;
	%else %do;
		if &trait ne . then output;
	%end;
run;
%NObsVars(work.temp);
%if ^&nObs or ^&nVars %then %do;
	%let exiterror = 1;
	%if %length(&wc) > 0 %then %do;
		%put ERROR: &__ssdsname WITH WHERE CLAUSE APPLIED CONTAINS 0 OBSERVATIONS.;
	%end;
	%else %do;
		%put ERROR: &__ssdsname DOES NOT EXIST, OR CONTAINS 0 OBSERVATIONS.;
	%end;
	%goto exit;
%end;

* DELETE OBSERVATIONS WITH DUPLICATE BY VALUES.;
proc sort data=temp nodup;
    by __id;
run;

* for grouping, compute means at the group level;
* assign splits at this level then merge back;
%if %length(&groupname) %then %do;
	proc sort data=temp out=temp0;
		by &groupname;
	run;
	/*
	data outlib.temp0;
		set temp0;
	run;
	*/
	proc means data=temp0 noprint;
		by &groupname;
		var &trait;
		output out=temp mean=&trait;
	run;
	%let trait_type = quantitative;
%end;


%if &trait_type = qualitative %then %do;
	proc freq data=temp noprint;
	    table &trait/out=trgtfreq;    
	run;
	proc sort data=trgtfreq;
		by PERCENT;
	run;
	data _null_;
	    set trgtfreq nobs=nobs end=end;
	    call symput("trait_value"||compress(_n_), left(trim(&trait)));
	    call symput("trait_holdout_count"||compress(_n_), trim(left(put(COUNT*&holdout_percent/100, 12.0))));
	    if end ne 0 then 
	        call symput("trait_level", nobs);
	run;

	%do i=1 %to &trait_level;
	    data trait&i;
	        set ds;
	        if &trait="&&trait_value&i" then output;
	    run;
		data _null_;
			aa = &seed + 607081;
	        seed = mod(aa, 2090069);
	        call symput("seed", trim(left(put(seed,12.0))));
	     run;
		%random_sample(datapath=,
					   dsname=trait&i, 
					   sample_size=&&trait_holdout_count&i, 
					   seed=&seed);
	    data keep&i out&i;
	        merge trait&i pseudo;
	        if pseudo eq 0 then output out&i;
	        else output keep&i;
	    run;         
	%end;

	%if %length(&train_dsname) > 0 and &train_dsname ne "" %then %do;
	    data &out..&train_dsname;
			set 
			%do i=1 %to &trait_level;
				keep&i
			%end;
			;
			%if %length(&iter) > 0 %then %do;
				%str(_I = &iter;);
			%end;
			%if %length(&k) > 0 %then %do;
				%str(_K = &k;);
			%end;
	    run;
	%end;
	%if %length(&test_dsname) > 0 and &test_dsname ne "" %then %do;
	    data &out..&test_dsname;
			set 
			%do i=1 %to &trait_level;
				out&i
			%end;
			;
			%if %length(&iter) > 0 %then %do; 
				%str(_I = &iter;);
			%end;
			%if %length(&k) > 0 %then %do;
				%str(_K = &k;);
			%end;
	    run;
	%end; 
%end;
%else %do;
	* create bins and reduce if necessary;
	proc means data=temp noprint;
		var &trait;
		output out=mean(drop=_TYPE_ _FREQ_)
 			   max=max min=min range=diff n=n;
	run;
	
	data _null_;
		set mean;
		call symputx("maxtrgt", max);
		call symputx("mintrgt", min);
		call symputx("range", trim(left(put(diff/&nbin, best12.)))); 
		call symputx("ntot",n);
		call symputx("nho",trim(left(put(n*&holdout_percent/100, 12.0))));
	run;
	
	%put mintrgt = &mintrgt;
	%put maxtrgt = &maxtrgt;
	%put range = &range;
	%put ntot = &ntot;
	%put nho = &nho;

	data bin;
		set temp;
		bin=floor(((&trait-&mintrgt)/&range)+1);
		if bin < 1 then bin = 1;
		else if bin > &nbin then bin = &nbin;
	run;	

	proc freq data=bin noprint;
		tables bin/out=binf;
	run;

	%NObsVars(binf);
	%let _nbins_ = &nObs;
	%put _nbins_ = &_nbins_;
	%if &_nbins_ < &nbin %then %do;
		data binf;
			set binf;
			new_bin = _n_;
		run;
		proc sort data=binf;
			by bin;
		run;
		proc sort data=bin;
			by bin;
		run;
		data bin;
			merge bin binf;
			by bin;
			drop bin;
			rename new_bin=bin;
		run;
		%let nbin = &_nbins_;
	%end;

	data binf;
		set binf;
		nho = max(1,round(COUNT*&holdout_percent/100));
	run;
	
	/*
	data outlib.binf_before;
		set binf;
	run;
	*/
	
	* adjust bin holdout counts so they add to the total desired holdout count;
	%let they_differ = 1;
	%let i = 1;
	%do %while(&they_differ);
		proc means data=binf noprint;
			var nho;
			output out=sumho sum=sum;
		run;
		
		data _null_;
			set sumho;
			call symputx("nho_sum",sum);
		run;
		
		%put nho_sum = &nho_sum;
		
		%if &nho_sum = &nho %then %do;
			%let they_differ = 0;
		%end;
		%else %if &nho_sum < &nho %then %do;
			proc sort data=binf;
				by nho descending count;
			run;
			data binf;
				set binf;
				if _n_ = 1 then do;
					nho = nho + 1;
				end;
			run;
		%end;
		%else %do;
			proc sort data=binf;
				by descending nho count;
			run;
			data binf;
				set binf;
				if _n_ = 1 then do;
					nho = nho - 1;
				end;
			run;
		%end;
		%let i = %eval(&i+1);
		%if &i > 10 %then %do;
			%put ERROR: Could not reconcile stratified bin counts after 10 iterations;
			%let exiterror = 1;
			%let they_differ = 0;
		%end;
	%end;
	
	/*
	data outlib.binf_after;
		set binf;
	run;
	*/
	
	proc sort data=binf;
		by bin;
	run;

	data binf;
		set binf;
		call symputx("bin_holdout_count"||compress(_n_),nho);
	run;
		
	%do i=1 %to &_nbins_;
	    data bin&i;
	        set bin;
	        if bin=&i then output;
	    run;
		data _null_;
			aa = &seed + 607081;
	        seed = mod(aa, 2090069);
	        call symput("seed", trim(left(put(seed,12.0))));
	     run;
		%random_sample(datapath=,
					   dsname=bin&i, 
					   sample_size=&&bin_holdout_count&i, 
					   seed=&seed);
		
		 data bin&i;
		 	merge bin&i pseudo;
		 run;
		 
		%if %length(&groupname) %then %do;
			data bin&i;
				merge temp0 bin&i(in=_in);
				by &groupname;
				if _in;
				drop _in;
			run;
		%end;

		proc sort data=bin&i;
			by __id;
		run;

		data bin&i;
			merge bin&i(in=_in) ds; 
			by __id;
			if _in;
			drop _in;
		run;
		
	    data keep&i out&i;
	        set bin&i;
	        if pseudo eq 0 then output out&i;
	        else output keep&i;
	    run;         
	%end;

	%if %length(&train_dsname) > 0 and &train_dsname ne "" %then %do;
	    data &out..&train_dsname;
	        set 
	        %do i=1 %to &_nbins_;
	            keep&i
	        %end;
	        ;
			%if %length(&iter) > 0 %then %do;
	           	%str(_I = &iter;);
			%end;
	        %if %length(&k) > 0 %then %do;
	           	%str(_K = &k;);
			%end;
	    run;
	%end;
	%if %length(&test_dsname) > 0 and &test_dsname ne "" %then %do;
	    data &out..&test_dsname;
	        set 
	        %do i=1 %to &_nbins_;
	            out&i
	        %end;
	        ;
			%if %length(&iter) > 0 %then %do;
	           	%str(_I = &iter;);
			%end;
	        %if %length(&k) > 0 %then %do;
	           	%str(_K = &k;);
			%end;
	    run;
	%end; 
%end;

%exit:;
%mend;

%macro Survival_Plot_Data(outputpath=,
					      _SPM2_=,
						  _SPM3_=,
						  _outDs_=,
						  EventTime=,
						  _idVar_=,
						  _SelectedMethods_=, 
						  _BinDs_=); 

	*_SPM2_*: Input data set from &SPM_OutData2;
	*_SPM3_*: Input data set from &SPM_OutData3;
	*_outDs_*: name for the output data set;
	*_idVar_*: variable to merge average survival data calculated from &_SPM2_ with experimental information extracted from &_SPM3_;
	*_SelectedMethod_*: Method selected for drawing survival plot;
	*_BinDs_*: Bin output data set;

* DETERMINE WHETHER OUTPUT PATH IS SPECIFIED, IF SO ALLOCATE IT;
%if %length(&outputpath) > 0 and "&outputpath" ne "" %then %do;
    libname output "&outputpath";
    %let cvoutlib = output;
%end;
%else %do;
    %let cvoutlib = work;
%end;

* RETRIEVE MODEL NAMES;
%let nm=1;
%do %while(%length(%scan(&_SelectedMethods_,&nm," ")));
    %let nm = %eval(&nm+1);
%end;
%let n_models = %eval(&nm-1);
%do nm=1 %to &n_models;

	%let mdlname&nm = %scan(&_SelectedMethods_,&nm," ");
	%let mdlname=&&mdlname&nm;

	data outdata2;
		set &cvoutlib..&_SPM2_;
		where METHOD="&mdlname";
	run;

	data outdata3;
		set &cvoutlib..&_SPM3_;
		where _DataType_ = "Test" and METHOD="&mdlname";
	run;

	proc contents data=outdata3 out=tmp noprint;
	run;
	
	%let col_id = 0;
	%let _name = ;

	data _null_;
		set tmp;
		if lowcase(Name) = "__id" then do;
			call symput('col_id',varnum);
			stop;
		end;
	run;

	data _null_;
		set tmp;
		if varnum < &col_id then do;
			_name = "col_"||trim(left(varnum));
			if NAME not in ("ITER","K") then call symput(_name,NAME);
			else call symput(_name," ");
		end;
	run;
	
	%if ^%length(&_idVar_) or &_idVar_ = "" or &_idVar_ = . %then %let _idVar_=_Obs;
	%put _idVar_ = &_idVar_;
	%put col_id = &col_id;
	%if %length(&_idVar_) > 0 and &_idVar_ ne "" %then %do;
		proc sort data=outdata3 /*(keep=%do i = 1 %to %eval(&col_id-1); &&col_&i %end; _DataType_ &_idVar_)*/
			out=outdata3_2 nodupkey;
			by &_idVar_;
		run;
	%end;

	%Surfunc_Average(_inDs=Outdata2,_outDs=Outdata2_avg,_tVar=&EventTime);

	%SurFunc_Tall_to_Wide(_inDs=Outdata2_avg,_outDs=&cvoutlib..&_outDs_.&nm,_idVar=&_idVar_,
				          _nBin=50,_DsToMerge=outdata3_2,
						  _time_vListPrefix=wide_vList,_time_lListPrefix=wide_lList);

	data &cvoutlib..&_BinDs_.&nm;
		set tmp3;
		keep _LABEL_;
		rename _LABEL_ = bin;
		label _LABEL_ = "Bin";
		_LABEL_ = trim(left(_LABEL_));
		if _LABEL_ = "" then delete;
	run;
%end;

* set JMP script to draw parallel plot for survival curves;

%mend Survival_Plot_Data;

************************************************************************
**  Copyright (c) 2007by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    TRANSCORINT
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Convert interval trait scored data created from 
**           PROC GENESELEC to cv wrapper form of scored data set.
**  HISTORY: action                                            date who
**           Created.	                                   02/19/07 pyt 
**
**  MACRO PARAMETERS:
**  INSCOR:  Input PROC GENESELECT scored data set name 
**  OUTSCOR: Output cv wrapper form of scored data set name
**  TRAIT:   Trait response variable name
**
**  NOTES: 
************************************************************************;
%macro transcorint( inscor=, 
			        outscor=, 
					trait=);

%if %sysfunc(exist(&inscor)) = 1 %then %do;
	%NObsVars(&inscor);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &inscor CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &inscor DOES NOT EXIST.;
	%goto exit;
%end;

data &outscor;
	set &inscor;
	/*
	keep &trait p_&trait;
	rename p_&trait = Predicted_Value;
	*/
run;

%exit:;
%mend;

************************************************************************
**  Copyright (c) 2007by SAS Institute Inc., Cary, NC USA 27513  
**  NAME:    TRANSCOR
**  SUPPORT: saspyt, Pei-Yi Tan
**  DATE:    02/19/07
**  PURPOSE: Convert categorical trait scored data created from 
**           PROC GENESELECT to cv wrapper form of scored data set.
**  HISTORY: action                                            date who
**           Created.                                      02/19/07 pyt 
**
**  MACRO PARAMETERS:
**  INSCOR:  Input PROC GENESELECT scored data set name 
**  OUTSCOR: Output cv wrapper form of scored data set name
**  TRAIT:   Trait response variable name
**
**  NOTES: 
**  Expect _NAMEDAT to exist if trait is categorical           
************************************************************************;
%macro transcor( inscor=, 
				 outscor=, 
				 trait=);

%if %sysfunc(exist(&inscor)) = 1 %then %do;
	%NObsVars(&inscor);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: &inscor CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: &inscor DOES NOT EXIST.;
	%goto exit;
%end;

%if %sysfunc(exist(_namedat)) = 1 %then %do;
	%NObsVars(_namedat);
	%if ^&nObs or ^&nVars %then %do;
		%let exiterror = 1;
		%put ERROR: _namedat CONTAINS 0 OBSERVATIONS.;
		%goto exit;
	%end;
%end;
%else %do;
	%let exiterror = 1;
	%put ERROR: _namedat DOES NOT EXIST.;
	%goto exit;
%end;

data a;
  set &inscor(obs=1);
  keep p_:;
run;
proc transpose data=a out=t;
run;
%global nclass;
data _null_;
     set t end=_last;
     if _last then call symput("nclass",trim(left(_n_)));
run;
%do c=1 %to &nclass;
   %global pprob&c;
%end;
data t;
  keep _name_ _i_;
  rename _name_ = p_name;
  set t;
  _i_ = _n_;
  call symput("pprob"||trim(left(_n_)),trim(left(_name_)));
run;

%let _d1_ = %sysfunc(open(&inscor));
%let _vn_ = %sysfunc(varnum(&_d1_,&trait));
%let _vt_ = %sysfunc(vartype(&_d1_,&_vn_));
%let _rc_ = %sysfunc(close(&_d1_));
 

proc sort data=_namedat out=_namesor; 
  by P_NAME;
run;
proc sort data=t; 
     by p_name;
run;

data b;
    merge _namesor t;
    by P_NAME;
    %if &_vt_ = C %then %do;
        &trait = trim(left(level));
    %end;
    %else %do;
        &trait = 0 + level;
    %end;
    keep &trait _i_;
run;

proc sort data=b; 
     by &trait;
run;

proc sort data=&inscor out=bylevel; 
     by &trait;
run;

proc freq data=bylevel noprint;
  tables &trait/out=of;
run;

data e;
  merge b of; 
  by &trait;
  keep &trait _i_;
  if not missing(count ) then do;
     if count > 0 then output;
  end;
run;

data c;
   merge bylevel e;
   by &trait;
run;

data &outscor;
  set c;
  array _p p_:;
    if not missing(_i_) then actualp = _p(_i_);
    else actualp = .;
  drop _i_;
run;

%exit:;
%mend;

* COPY SAS FILES NEEDED FOR GRID PROCESSING;
* USE SUBFOLDERS TO AVOID ACCESS COLLISIONS;
%macro UpdateFitModels(nnodes=,outputpath=);

	* Create SUBFOLDERS FROM JMP Life Sciences for each node;
	%do node=1 %to &nnodes;
		%let rc = %sysfunc(dcreate(gnode&node,&outputpath));
		/*
		%let rc = %sysfunc(dcreate(MacroLib,&outputpath.gnode&node));
		%let rc = %sysfunc(dcreate(ProcessLibrary,&outputpath.gnode&node));
		*/
	%end;
	
	/*
	* COPY MACROLIB FILES;
	filename inmref "&LS_HOME.MacroLib"; 
	%let outmrefs = ;
	*/
	%do node=1 %to &nnodes;	
		/*
		filename outm&node "&outputpath.gnode&node/MacroLib";
		%let outmrefs = &outmrefs outm&node;
		*/
		
		* customize FitModels.sas for each node;
		data _tmp_;
			infile "&outputpath.FitModels.sas" truncover;
			file "&outputpath.gnode&node/FitModels.sas";
			input instring $1-2000; 
			outstring = instring;
			first11 = substr(outstring,1,11);
			if first11 = '%let InData' then do;
				endpart = scan(outstring,-1,'\/');
				fname = substr(endpart,1,length(endpart)-2);
				outstring = cat('%let InData = ',"&servergshare",trim(left(fname)),trim(left(";")));
			end;
			else if first11 = '%let OutPat' then do;
				outstring = cat('%let OutPath = ',"&servergshare.gnode&node/;");
			end;
			else if first11 = '%let MacroP' then do;
				outstring = '%let MacroPath = &workpath./MacroLib/;';
			end;
			else if index(first11,'%include') and index(outstring,'.sas') then do;
				endpart = scan(outstring,-1,'\/');
				fname = substr(endpart,1,length(endpart)-2);
				outstring = cat('%','include "&workpath/ProcessLibrary/',fname,'";');			
			end;
			else if first11 = '%let LS_HOM' then do;
				outstring = '%let LS_HOME = &workpath;';
			end;
			else if first11 = '%let USER_H' then do;
				outstring = cat('%let USER_HOME = ',"&servergshare.gnode&node/;");
			end;
			else if first11 = '%let LSUSER' then do;
				outstring = cat('%let LSUSER_HOME = ',"&servergshare.gnode&node/;");
			end;
			put outstring;
		run;
	%end;
	
	/*
	libname o "&outputpath";
	data o._tmp_;
		set _tmp_;
	run;
	*/
	
	/*
	%CopyFile(inmref,&outmrefs,BinaryResponseEffectSelectionMacros.sas);
	%CopyFile(inmref,&outmrefs,ClassVars.sas);
	%CopyFile(inmref,&outmrefs,CrossValidationModelComparisonMacros.sas);
	%CopyFile(inmref,&outmrefs,GAMacros.sas);
	%CopyFile(inmref,&outmrefs,MixedModelMacros.sas);
	%CopyFile(inmref,&outmrefs,PredMacros.sas);
	%CopyFile(inmref,&outmrefs,SeedMacros.sas);
	%CopyFile(inmref,&outmrefs,UtilityMacros.sas);
	%CopyFile(inmref,&outmrefs,MultipleTestingAdjustment.sas);
	
	filename inmref clear; 
	%do node=1 %to &nnodes;	
		filename outm&node clear;
	%end;
	*/
	
	/*
	filename gref "&outputpath"; 
	%CopyFile(gref,&outmrefs,FitModels.sas);
	*/
	
	* COPY PROCESSLIBRARY FILES;
	/*
	filename inpref "&LS_HOME.ProcessLibrary"; 
	%let outprefs = ;
	%do node=1 %to &nnodes;	
		filename outp&node "&outputpath.gnode&node\ProcessLibrary";
		%let outprefs = &outprefs outp&node;
	%end;
	%CopyFile(inpref,&outprefs,DiscriminantAnalysis.sas);
	%CopyFile(inpref,&outprefs,DistanceScoring.sas);
	%CopyFile(inpref,&outprefs,GLMSelect.sas);
	%CopyFile(inpref,&outprefs,KNearestNeighbors.sas);
	%CopyFile(inpref,&outprefs,LogisticRegression.sas);
	%CopyFile(inpref,&outprefs,PartitionTrees.sas);
	%CopyFile(inpref,&outprefs,PartialLeastSquares.sas);
	%CopyFile(inpref,&outprefs,RadialBasisMachine.sas);	
	%CopyFile(inpref,&outprefs,RidgeRegression.sas);	
	%CopyFile(inpref,&outprefs,SurvivalPredictiveModeling.sas);	
	%CopyFile(inpref,&outprefs,GenomicBLUP.sas);
	%CopyFile(inpref,&outprefs,GenomicBayesianRegression.sas);

	filename inpref clear; 
	%do node=1 %to &nnodes;	
		filename outp&node clear;
	%end;
	*/
%mend;


* GET SAS FILES NEEDED FOR GRID PROCESSING;
* USE WORK DIRECTORY AND SUBFOLDERS TO MATCH JMP LIFE SCIENCES;
* THIS CODE IS DESIGNED TO RUN ON EACH GRID NODE WITHIN AN RSUBMIT BLOCK;
%macro GetGridFiles;

	%global workpath;
	data _null_;
		call symput('workpath',pathname("work"));
	run;
	%put workpath = &workpath;
	
	* CREATE SUBFOLDERS FROM JMP LIFE SCIENCES;
	%let rc = %sysfunc(dcreate(MacroLib,&workpath));
	%let rc = %sysfunc(dcreate(ProcessLibrary,&workpath));
		
	* GET MACROLIB FILES;
	proc upload infile="&GHOME.FitModels.sas" 
		outfile="&workpath./MacroLib/FitModels.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\BinaryResponseEffectSelectionMacros.sas" 
		outfile="&workpath./MacroLib/BinaryResponseEffectSelectionMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\ClassVars.sas" 
		outfile="&workpath./MacroLib/ClassVars.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\CrossValidationModelComparisonMacros.sas" 
		outfile="&workpath./MacroLib/CrossValidationModelComparisonMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\GAMacros.sas" 
		outfile="&workpath./MacroLib/GAMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\MixedModelMacros.sas" 
		outfile="&workpath./MacroLib/MixedModelMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\PredMacros.sas" 
		outfile="&workpath./MacroLib/PredMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\SeedMacros.sas" 
		outfile="&workpath./MacroLib/SeedMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\UtilityMacros.sas" 
		outfile="&workpath./MacroLib/UtilityMacros.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.MacroLib\MultipleTestingAdjustment.sas" 
		outfile="&workpath./MacroLib/MultipleTestingAdjustment.sas" connectstatus=no;
	run;
	
	* GET PROCESSLIBRARY FILES;
	proc upload infile="&LS_HOME.ProcessLibrary\DiscriminantAnalysis.sas" 
		outfile="&workpath./ProcessLibrary/DiscriminantAnalysis.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\DistanceScoring.sas" 
		outfile="&workpath./ProcessLibrary/DistanceScoring.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\GLMSelect.sas" 
		outfile="&workpath./ProcessLibrary/GLMSelect.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\KNearestNeighbors.sas" 
		outfile="&workpath./ProcessLibrary/KNearestNeighbors.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\LogisticRegression.sas" 
		outfile="&workpath./ProcessLibrary/LogisticRegression.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\PartitionTrees.sas" 
		outfile="&workpath./ProcessLibrary/PartitionTrees.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\PartialLeastSquares.sas" 
		outfile="&workpath./ProcessLibrary/PartialLeastSquares.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\RadialBasisMachine.sas" 
		outfile="&workpath./ProcessLibrary/RadialBasisMachine.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\RidgeRegression.sas" 
		outfile="&workpath./ProcessLibrary/RidgeRegression.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\SurvivalPredictiveModeling.sas" 
		outfile="&workpath./ProcessLibrary/SurvivalPredictiveModeling.sas" connectstatus=no;
	run;
	proc upload infile="&LS_HOME.ProcessLibrary\GenomicBLUP.sas" 
		outfile="&workpath./ProcessLibrary/GenomicBLUP.sas" connectstatus=no;
	run;	
	proc upload infile="&LS_HOME.ProcessLibrary\GenomicBayesianRegression.sas" 
		outfile="&workpath./ProcessLibrary/GenomicBayesianRegression.sas" connectstatus=no;
	run;	

%mend;
