/*
JMP Genomics Analytical Process

Name: Genomic Heritability
Author: Luciano Silva

Description:
Genomic Heritability calculates the proportion of 
genetic variance explained by SNPs, as described in 
Yang, J. et al. (Nature Genetics, 42: 565-569, 2010).
*/

/*
-----------------------------------------------------------
Copyright (C) 2017 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars SexVar IDVars ListMarkerVars ListTraitVars WhereStmt 
TraitVars QVars ListQVars QSumto1 TraitType ClassVars FixedEff   
RandEff RandOpts AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc 
AnnoAccession AnnoWhereStmt XLVarStmt DDFM UseLowerB 
AUnBiased OutAddMatrix OutRootAddMatrix VarAutoMarkers VarXMarkers VarByAuto
Listing OutputResid OutputPred OutputFitStat  
OutPath ServerOutPath OutName MacroPath GMethod MMethod EstMethod 
GH_OutData GH_Script;

*ProcessBody;

%macro GenomicHeritability;

	%let UseLowerB= No;
	%let OutputResid= No;
	%let OutputPred= No;
	%let OutputFitStat= Yes;

	%global ParmsStmt PedID ByVars CensorVar StrataVars MarkerFmt XLVars;
	%let ParmsStmt= ;
	%let PedID= ;
	%let ByVars= ;
	%let CensorVar= ;
	%let StrataVars= ;
	%let MarkerFmt= %str(Numeric Genotypes); /*begin: Needed to avoid error in the %GenInit()*/
	%let XLVars= ;
	
	%let OrigClassVars= &ClassVars;
	%let OrigRandEff= &RandEff;
	%let OrigParmsStmt= &ParmsStmt;

	ods noresults;
	ods html close;
	*ods listing close;
	*options nonotes;
	options label;

	%include "&MacroPath./GenMacros.sas" / nosource;
	
	%GenInit(GenomicHeritability,28);
	%if &exiterror=1 %then %goto exit;
	
	%let InDataPath= &TmpPath;
	%put InDataPath= &InDataPath;
	
	%global FullAPName;
	%let FullAPName=%str(Genomic Heritability);
	libname OutLib "&OutPath";	
	
	%if %sysevalf(%length(&OutName) >= 28) %then
		%let OutName= %substr(&OutName, 1, 28);
	%put OutName=&OutName;	
	
	%global GH_H2OutData GH_PEOutData GH_FSOutData GH_ADOutData GH_RAOutData;
	%PrepOutDS(GH_H2OutData,GH_H2OutDataName,&OutName._ghe);
	%PrepOutDS(GH_PEOutData,GH_PEOutDataName,&OutName._gpe);
	%PrepOutDS(GH_FSOutData,GH_FSOutDataName,&OutName._gfs);
	%if %upcase(&OutAddMatrix)= YES %then %do;
		%PrepOutDS(GH_ADOutData,GH_ADOutDataName,&OutName._gad);
	%end;
	%if %upcase(&OutRootAddMatrix)= YES %then %do;
		%PrepOutDS(GH_RAOutData,GH_RAOutDataName,&OutName._gra);
	%end;
	%PrepOutFile(GH_Script,GH_ScriptName,&ProcessName Script for &OutName..jsl);	
	
	%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
		%put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;	
	
	%if %length(&ListTraitVars &TraitVars)=0 %then %do;
		%put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
		%let exiterror=1;
		%goto exit;
	%end;
	
	%let TraitVars = &TraitVars &ListTraitVars;	

	/*
	%if %upcase(&FixCovParms)=YES and (%upcase(&DDFM)=SATTERTHWAITE or %upcase(&DDFM)=KENWARDROGER) %then %do;
		%put ERROR: You cannot select SATTERTHWAITE or KENWARDROGER for the Denominator Degrees of Freedom Method when the Fix Covariance Parameters checkbox is checked.;
		%let exiterror=1;
		%goto exit;
	%end;
	*/	
	
	* need to combine AlleleVars and ListMarkerVars into AlleleVars for %PrepMapandBy *;
	%let OrigAlleleVars = &AlleleVars;
	%let AlleleVars = &AlleleVars &ListMarkerVars;
	
	%global ReorderingVars;
	* set ReorderingVars to empty to avoid certain conflict when and AnnoLoc is empty and call %GenSubset within %PrepMapandBy;
	%let ReorderingVars = ; 
	%if %length(&AnnoLoc)>0 %then %let ReorderingVars = &AnnoGroup &AnnoLoc &AnnoLabel;
	 
	* AnnoWhereStmt statement applied in the %PrepMapandBy *;
	* If AnnoWhereStmt is not empty, then AlleleVars is updated in the %PrepMapandBy *;
	* ds is created within %PrepMapandBy *;
	%PrepMapandBy;
	run;
	%if &exiterror=1 %then %goto exit;	
	
	/*data OutLib.gh_ds0; set &ds; run;*/
	/*data OutLib.gh_count0; set count; run;*/
	/*data OutLib.gh_mapdata; set mapdata; run;*/
	
	%*let exiterror=1;
	%*goto exit;
	
	*begin: from markerproperties*;
	*nummark is defined within PrepMapandBy *;
	%let nm=&nummark;
	%let sexclause =;
	%if %length(&XLVars) %then %do;
		*numxmark is defined within PrepMapandBy *;
		%let numnxmark = %eval(&nm - &numxmark);
		/*S1151346: Luciano, making sure that SexVar is not empty and XLVars is being used*/
		%*if %length(&SexVar)=0 %then %let sexvar=sex;
		%if %length(&SexVar)=0 %then %do;
			%put ERROR: A Sex Variable must be specified in the General Tab when using the X-Link Markers Clause from the Annotation Tab.;
			%let exiterror = 1;
			%goto exit;
		%end;
		%let dsid=%sysfunc(open(&ds,i));
		%let sextype=%sysfunc(vartype(&dsid,%sysfunc(varnum(&dsid,&sexvar))));
		%let rc=%sysfunc(close(&dsid));
		%if %quote(&sextype) = %str(N) %then %do;
			%let sexclause = %nrbquote(&sexvar = 2);
		%end;
		%else %do;
			%let sexclause = %nrbquote(substr(&sexvar,1,1) in ("F","f"));
		%end;
	%end;
	%else %do;
		%let numnxmark=&nm;
		%let sexclause = 1;
	%end;
	*end: from markerproperties*;
	
	*begin: from qkmodelfitness.sas*;
	%if %length(&ListQVars) %then %do;
		data QVars;
			set &ds(keep=&ListQVars obs=1);
		run;

	   	%let dsid=%sysfunc(open(QVars));
	   	%let nvars= %sysfunc(attrn(&dsid,NVARS));
	   	%do i=1 %to &nvars;
			%let QVars=&QVars %sysfunc(varname(&dsid,&i));
		%end;
		%let rc = %sysfunc(close(&dsid));
	%end;	

	%if %upcase(&QSumto1)=YES and %length(&QVars)>0 %then %do;
		* Remove last Q var since sum to 1 *;
		%let Orig_QVars = &QVars;
		%let QVars = ;
		%let w=1;
		%do %while(%length(%scan(&Orig_QVars,%sysevalf(&w+1))));
			%let QVars = &QVars %scan(&Orig_QVars,&w);
			%let w=%sysevalf(&w+1);
		%end;
		%if %length(&QVars)=0 %then %do;
			%let QVars = &Orig_QVars;
			%put WARNING: There is only one Q Matrix variable selected for Q1 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored.;
		%end;
	%end;
	%else %if %upcase(&QSumto1)=YES and %length(&QVars)=0 %then %do;
		%put WARNING: Q Matrix variable were not selected for Q1 Model so the Columns of Q Matrix Sum to 1 checkbox is ignored.;	
	%end;
	%put QVars= &QVars;	
	
	%let FixedEff = &QVars &FixedEff; /*ListQVars has already been included into QVars*/
	
	* Table name from all but MIXED *;
	%global ParmEst EffName;
	%let ParmEst=ParameterEstimates;
	%let EffName=Effect;

	%let WhichProc=MIXED;
	%let TraitOpt= ;
	%let Stat=F; 
	%let Gtable=Tests3;
	%let TStmt=contrast;
	%let Ttable=Contrasts;
	%let usestrata=0;
	%let ModOption=%quote(/ htype=3 ddfm=&DDFM s); 
	%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption residual outp=outres);
	%let ClassOpt=;
	/*%let teststmt=%str('genotypes' Genotype 1);*/
	%let ProcOption= %str(Method=&MMethod);
	%let ParmEst = SolutionF;
	%let EffName = Effect;
	%if %length(&PedID) %then %let ClassVars = &ClassVars &PedID;

	* Need a separate check for OR when using GLIMMIX since not creating OR data set *;
	%let AnyOdds=0;
	%if %upcase(&TraitType)=COUNT %then %do;
		%let WhichProc=GLIMMIX; 
		%let ModOption=%quote(/ link=log dist=p htype=3 ddfm=&DDFM s);
		%if %upcase(&OutputResid)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
		%let ParmEst = ParameterEstimates;
	%end;
	%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
		%if %length(&RandEff)>0 or %length(&KVarsModel1)>0 or %length(&KVarsModel2)>0 %then %do;
			%let WhichProc=GLIMMIX;
			/*%let ProcOption=%str(IC= PQ Method=&GMethod);*/
			%let ProcOption=%str(Method=&GMethod);
			%if %upcase(&TraitType)=NOMINAL %then 
				%let TraitOpt=(Ref=FIRST);
			%else 
				%let TraitOpt=(Event=LAST);		
			%if %upcase(&EstMethod)=YES %then 
				%let ProcOption=%quote(&ProcOption EMPIRICAL);
			%let Stat=F;     
			%let Gtable=Tests3;	
			%let TStmt=estimate;				
			%let Ttable=Estimates;		
			%let ParmEst = ParameterEstimates;
			%if %upcase(&DDFM)=CONTAIN or %upcase(&DDFM)=BETWITHIN %then 
				%let DDFM=RESIDUAL;
			%if %upcase(&TraitType)=BINARY %then 
				%let ModOption=%quote(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
			%else %if %upcase(&TraitType)=NOMINAL %then %do;
				%*let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=betwithin s);
				%let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=&DDFM s);
				%let ClassVars = &ClassVars &TraitVars;
			%end;
			%else %if %upcase(&TraitType)=ORDINAL %then 
				%let ModOption=%quote(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
			%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid predicted=Pred);
			%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
			%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=Pred);
			/*%let teststmt=%str('genotypes' Genotype 1);*/		
			/*%if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %let lsmi=0;*/
		%end;
		%else %do;
			%let WhichProc=LOGISTIC;
			%let ProcOption=;
			%let TraitOpt=(Event=LAST);		
			%let Stat=ChiSq;
			%*let Gtable=Type3;
			%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
			%let TStmt=contrast;
			%let Ttable=ContrastTest;
			%let Otable=OddsRatiosWald;
			%let usestrata=1;
			%if %upcase(&TraitType)=NOMINAL %then 
				%let ModOption=%str(/ link=glogit);
			%else %if %length(StrataVars)>0 %then 
				%let ModOption=/ GCONV=1E-6; 
			%else %let ModOption =;
			%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid predicted=pred);
			%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid);
			%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=pred);
			%let ClassOpt = %str(/ cprefix=8);
			/*%let teststmt=%str('genotypes' Genotype 1);*/
			%let ParmEst = ParameterEstimates;
			%let EffName = Variable;
			/*%let lsmi=0;*/
		%end;
	%end; 
	%put WhichProc= &WhichProc;
	
	/*
	%if %upcase(&OutputLSM)=YES and &lsmi=0 %then %do;
		%put LS means cannot be calculated for this model.;
	%end;
	*/

	%if &usestrata=0 %then %let StrataVars=;

	/*S1089309
	%if %upcase(&UseLowerB)=YES %then %do;
	*/
	%if (%upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= GLIMMIX) %then %do;
		%if %upcase(&UseLowerB)=YES %then %do;
			%if %length(&ParmsStmt) %then %do;
				%if %index(&ParmsStmt,%str(/)) %then %do;
					%let ParmsStmt = %quote(&ParmsStmt lowerb=0);
				%end;
				%else %do;
					%let ParmsStmt = %quote(&ParmsStmt / lowerb=0);
				%end;
			%end;
			%else %let ParmsStmt = %str( / lowerb=0);
		%end;
	%end;
	%else %let ParmsStmt=;	
	*end: from qkmodelfitness.sas*;
		
	*begin: from snptraitassocmacro.sas*;
	*S1415561: check variable avaibilityt first for additional Random or fixed effects*;
	%let _dsid_=%sysfunc(open(&ds));	
	
	%let FixedEffVars = ;
	%let i = 1;
	%let FixedEffVarNotFound= ;
	%do %while(%length(%scan(&FixedEff,&i,'*| ')));
		%let FixedEffVarTmp= %scan(&FixedEff,&i,'*| ');
		%let FixedEffVars = &FixedEffVars &FixedEffVarTmp;
		%let i=%eval(&i+1);
		%if ^%sysfunc(varnum(&_dsid_,&FixedEffVarTmp)) %then %do;
			%let FixedEffVarNotFound= &FixedEffVarNotFound &FixedEffVarTmp;
		%end;
	%end;

	%let RandEffVars = ;
	%let i = 1;
	%let RandEffVarNotFound= ;
	%do %while(%length(%scan(&RandEff,&i,'*| ')));
		%let RandEffVarTmp= %scan(&RandEff,&i,'*| ');;
		%let RandEffVars = &RandEffVars &RandEffVarTmp;
		%let i=%eval(&i+1);
		%if ^%sysfunc(varnum(&_dsid_,&RandEffVarTmp)) %then %do;
			%let RandEffVarNotFound= &RandEffVarNotFound &RandEffVarTmp;
		%end;		
	%end;
	%let RandEffVarsOrig = &RandEffVars;
	
	%let rc = %sysfunc(close(&_dsid_));
	
	%put FixedEffVarNotFound= &FixedEffVarNotFound;
	%put RandEffVarNotFound= &RandEffVarNotFound;
	%if %length(&FixedEffVarNotFound &RandEffVarNotFound) %then %do;
		%put ERROR: These variables were not found in the input data set: &FixedEffVarNotFound &RandEffVarNotFound..;
		%let exiterror=1;
		%goto exit;
	%end; 

	%*put i=&i;
	%if &i>1 %then %do;
		/* If just one random effect variable, use alternative random statement */
		%if &i=2 and %length(&RandOpts)=0 and %length(&ParmsStmt)=0 and %index(&RandEffVars,%str(-))=0 %then %do;
			
			%if %length(&ByVars &RandEffVars) %then %do;
				proc sort data=&ds out=dssort;
					by &ByVars &RandEffVars;
				run;
			%end;
			%else %do;
				data dssort;
					set &ds;
				run;
			%end;
			
			%if %index(%upcase(&ClassVars),%upcase(&RandEffVars))>0 %then %do;
				%let NumRandEff = %trim(%substr(&RandEffVars,1,28)_num);
				
				%if %length(&ByVars &RandEffVars) %then %do;
					data numrandeff;
						set dssort;
						by &ByVars &RandEffVars;
						if first.&RandEffVars then &NumRandEff+1;
					run;
				%end;
				%else %do;
					data numrandeff;
						set dssort;
					run;
				%end;				

				%let ClassVars = %sysfunc(tranwrd(%upcase(&ClassVars),%upcase(&RandEffVars),%str()));
				%let RandEffVars = &NumRandEff;
				%let ds = numrandeff;
			%end;
			%else 
				%let ds=dssort;	
			/*
			%if &imputed=1 %then %do;
				* Need ds to be genotypes since not running %CreateStackedNumeric below *;
				proc datasets nolist;
					delete genotypes;
					change &ds=genotypes;
				quit;
				%let ds=genotypes;
			%end;
			*/
			%let RandEff = %str(intercept / subject=&RandEffVars);
		%end;
		%else %do;
			%let foundsub=0;
			%let RandEff = %str(&RandEff / &RandOpts);
			/* Need to see if there's a SUBJECT= variable that needs to be kept */
			%let subind=%index(&RandOpts,sub=)+4;
			%if &subind=4 %then %do;
				%let subind=%index(&RandOpts,subject=)+8;
				%if &subind>8 %then
					%let foundsub=1;
			%end;
			%else 
				%let foundsub=1;
			%if &foundsub=1 %then 
				%let RandEffVars = &RandEffVars %scan(%substr(&RandOpts,&subind),1);
		%end;
	%end;
	%put RandEff= &RandEff;
	
	/*data OutLib.gh_ds1; set &ds; run;*/
	
	%*goto exit;
	
	/*begin S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/
	%let ClassVarsTmp= &ClassVars;
	%if %symexist(TraitVars) %then %do;
		%if %length(&TraitVars) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&TraitVars), %str()));
		%end;
	%end;

	%if %symexist(PedID) %then %do;
		%if %length(&PedID) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&PedID), %str()));
		%end;
	%end;
	
	%put ClassVarsTmp= &ClassVarsTmp;	
	
	%let NeedConstant=0;
	%if %length(&ClassVarsTmp) = 0 %then %do;
		/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
		table for Trend test when no Class var is present*/
		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			%let NeedConstant=1;
		%end;
		%else 
			%let NeedConstant=0;
	%end;
	%else %if %length(&ClassVarsTmp) %then %do;

		/*Checking if QKClassVars have been included into any of the variables that goes into the model statement*/
		%let IntEffVars= ;
		%let FixRandEffVars= &FixedEffVars &RandEffVars &IntEffVars;
		%put FixRandEffVars= &FixRandEffVars;		

		%let j=1;
		%do %while(%length(%scan(&ClassVarsTmp, &j, %str( ))));
			%let j= %sysevalf(&j + 1);
			%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
		%end;
		%let ncin= %sysevalf(&j - 1);

		%macro CheckClassVars();
			%global ncout ClassVarsOut FoundAtLeastOne_flag;
			%let ncout= 0;
			%let ClassVarsOut= ;
			%let FoundAtLeastOne_flag= 0;

			%let FixRandEffVarsOrig= &FixRandEffVars;

			%if %length(&FixRandEffVars) = 0 %then %do;
				%let ClassVarsOut= &ClassVarsTmp;
				%let FoundAtLeastOne_flag= 0;
			%end;		
			%else %do;
				%let j=1;
				%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%do %while(%length(&currclass));
					%put currclass= &currclass;
					%let k=1;
					%let currvar= %scan(&FixRandEffVars, &k, %str( ));
					%if %length(&currvar) %then %do;
						%let CandidateClassVarOut= ;
						%let found_flag= 0;
						%do %while(%length(&currvar));
							%put &currclass &currvar;
							%if %upcase(&currclass) = %upcase(&currvar) %then %do;
								%*put YES;
								%let found_flag= 1;
								%let FoundAtLeastOne_flag= 1;
								%let FixRandEffVarsTmp=;
								%let l=1;
								%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%do %while(%length(&currvartmp));
									%if &l ^= &k %then %do;
										%let FixRandEffVarsTmp= &FixRandEffVarsTmp &currvartmp;
									%end;
									%let l= %sysevalf(&l + 1);
									%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%end;
								%let FixRandEffVars= &FixRandEffVarsTmp;
								%goto nextclassvar;
							%end;
							%else %do;
								%*put NO;
								%let CandidateClassVarOut= &currclass;
							%end;

							%let k= %sysevalf(&k + 1);
							%let currvar= %scan(&FixRandEffVars, &k, %str( ));
						%end;

						%if &found_flag = 0 %then %do;
							%let ClassVarsOut= &ClassVarsOut &CandidateClassVarOut;
						%end;
					%end;
					%else %do;
						%let ClassVarsOut= &ClassVarsOut &currclass;
					%end;

					%nextclassvar:;
					%let j= %sysevalf(&j + 1);
					%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%end;
				%let FixRandEffVars= &FixRandEffVarsOrig;
				%put ClassVarsOut= &ClassVarsOut;
				%put FixRandEffVars= &FixRandEffVars;
			%end;

			%if %length(&ClassVarsOut) > 0 %then %do;
				%let ncout=1;
				%do %while(%length(%scan(&ClassVarsOut, &ncout, %str( ))));
					%let ncout= %sysevalf(&ncout + 1);
				%end;
				%let ncout= %sysevalf(&ncout - 1);
			%end;
		%mend CheckClassVars;
		%CheckClassVars();

		%put ncout= &ncout;
		%if &ncout %then %do;
			%if &ncout = 1 %then %do;
				%let ErrString0= Class Variable &ClassVarsOut has neither been specified as fixed, random nor interaction effect. Please, review the Model Variables Tab.;
			%end;
			%else %if &ncout > 1 %then %do;
				%let ErrString0= Class Variables &ClassVarsOut have neither been specified as fixed, random nor interaction effects. Please, review the Model Variables Tab.;
			%end;

			%if %length(&ProcessName) > 22 %then %do;
				%let ProcessNameShort= %substr(&ProcessName, 1, 22);
			%end;
			%else %do;
				%let ProcessNameShort= &ProcessName;
			%end;

			%let ScErr0= &ProcessNameShort._ScErr;
			%*put ScriptErr= &ScErr0;
			%global &ScErr0;
			%PrepOutFile(&ScErr0,&ScErr0.Name,saserror.jsl);
			%*put ScriptErr= &&&ScErr0;
			%*put ScriptErrName= &&&ScErr0.Name;

			data _null_;
				file "&&&ScErr0" lrecl=32767 ;
				put '//!' ;
				put "Names Default to Here(1);" ;
				put "new Window(""ERROR running &FullAPName.""," ;
				put "vlb=VListBox(" ; 
				put "textBox(""The following SAS error ocurred in process &FullAPName:""),";
				put "errtxt = TextEditBox(""\!N""||""";
				put "&ErrString0";
				put """||""\!N"",WhiteBoxStyle(0)),";
				put "errtxt << Font Color(3)," ; 
				put "errtxt << Set Wrap(1000)," ;
				/*put "TextBox(""Please refer to the SAS log displayed in the JMP log window for further details.\!N\!N""),";*/
				put ")" ;
				put ");";
			run;
			%*PackageInsertFile(jsl code from &ProcessName, &OutPath, &&&ScErr0.Name, no, no, yes);

			%put ERROR: &ErrString0;
			%let exiterror=1;
			%goto exit;
		%end;

		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
			table for Trend test when no Class var is neither included as fixed nor random effect*/
			%let FixRandEffVars= &FixedEffVars &RandEffVars;
			%put FixRandEffVars0= &FixRandEffVars;
			%CheckClassVars();
			%put FoundAtLeastOne_flag= &FoundAtLeastOne_flag;
			%if &FoundAtLeastOne_flag = 0 %then %do;
				%let NeedConstant=1;
			%end;
			%else %do;
				%let NeedConstant=0;
			%end;
		%end;
	%end;
	%put NeedConstant= &NeedConstant;
	/*end S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/
	*end: from snptraitassocmacro.sas*;

	*begin: from ImputeMissingGenotypes.sas*;
	data _null_;
		* mapdata is created by %PrepMapandBy *;
		set mapdata;
		call symput('HapGp'||left(_n_),strip(&AnnoGroup));
	run;

	%*goto skip0;
	
	* create index for observations if there are no IDVars *;
	%let InputRowID= ;
	%let OutputRowID= ;
	%if %length(&WhereStmt) %then %do;
		%let InputRowID= _InputRowID_;
		data indatn;
			set &ds;
			&InputRowID= _n_;
		run;

		%*let OutputRowID= _OutputRowID_;
		data indatn;
			set indatn;
			where &WhereStmt;
			*&OutputRowID= _n_;
			keep &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &OutputRowID &IDVars &ByVars &AlleleVars &SexVar &IDVars;
		run;
		
		%let dsnew= indatn;
	%end;
	%else %do;
		%let dsnew= &ds;
	%end;
	
	/*data OutLib.gh_dsnew; set &dsnew; run;*/

	* GenoCol is created by %GenInit *;
	%if %length(&GenoCol) %then %do;
		%let maxavlen=0;
		%let dsid=%sysfunc(open(count));
		* nummark is created by %PrepMapandBy *;
		%do m=1 %to &nummark;
			%let templen=%sysfunc(varlen(&dsid,&m));
			%if &templen > &maxavlen %then %let maxavlen = &templen;
		%end;
		%let rc = %sysfunc(close(&dsid));

		data indat;
			set &dsnew;
			%*if %length(&WhereStmt) %then %do;
			*	where &WhereStmt;
			%*end;
			array av{&nummark} &AlleleVars &XLVars;
			array _a_{&nummark} %if %upcase(&MarkerFmt) ne %str(NUMERIC GENOTYPES) %then $&maxavlen;;
			do i=1 to &nummark;
				%if %upcase(&MarkerFmt) ne %str(NUMERIC GENOTYPES) %then %do;
					if missing(av{i}) then _a_{i}='';
					else _a_{i}=strip(av{i});
				%end;
				%else %do;
					if missing(av{i}) then _a_{i}=.;
					else _a_{i}=av{i};
				%end;    
			end;
			/*
			%if %upcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
				* format is created by %GenInit *;
				format _a_: numtochar.;
			%end;
			*/
			* Luciano, included InputRowID and IDVars*;
			keep &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars _a_1-_a_&nummark; 
		run;
		%*let nmv=&nummark;
	%end;
	%else %do;
		%let maxavlen=0;
		%let dsid=%sysfunc(open(count));
		* n2snps is created by %PrepMapandBy *;
		%do m=1 %to &n2snps;
			%let templen=%sysfunc(varlen(&dsid,&m));
			%if &templen > &maxavlen %then %let maxavlen = &templen;
		%end;
		%let rc = %sysfunc(close(&dsid));

		data indat;
			set &dsnew;
			%*if %length(&WhereStmt) %then %do;
			*	where &WhereStmt;
			%*end;
			array av{&n2snps} &AlleleVars &XLVars;
			array _a_{&n2snps} $&maxavlen;
			do i=1 to &n2snps;
				if missing(av{i}) then _a_{i}='';
				else _a_{i}=strip(av{i});
			end;
			* Luciano, included InputRowID and IDVars *;
			keep &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars _a_1-_a_&n2snps; 
		run;
		%*let nmv=&n2snps;
	%end;
	%let ds= indat;
	*end: from ImputeMissingGenotypes.sas*;
	
	/*data OutLib.gh_indat0; set indat; run;*/	
	
	%skip0:;

	%*let exiterror=1;
	%*goto exit;

	*begin: get trait variables*;
	data tr1;
		set &ds(obs= 1 keep= &TraitVars);
	run;

	%put TraitVars= &TraitVars;
	proc transpose data=tr1 out=at;
		var &TraitVars;
	run;

	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exit;
	%end;

	%NObsVars(at);
	%global NTrait;
	%let Ntrait= &nobs;
	%put NTrait= &NTrait;
	%if &NTrait=0 %then %do;
		%let exiterror=1;
		%put ERROR: No variables found matching the Trait Variables specified.;
		%goto exit;
	%end;
	%if &NTrait > 1 %then %let OutputSurvFunc = NO;

	%do i=1 %to &Ntrait; 
		%global _tv&i;
	%end; 

	proc sort data=at;
		by _name_;
	run;

	data _null_;
		set at;
		length is $16;
		is = "_tv" || strip(_n_);
		call symput(is,strip(_name_));
	run;
	*end: get trait variables*;
	
	**start: setting for fuilding additive and dominance relationship matrices. This is from GenomicBLUP.sas**;
	%let RandSVD= No;
	%let RandSVD_error= No;	
	%let IncludeDom= No;
	%let RandSVD_K= 0;
	%let RandSVD_Q= 0;
	%let RandSVD_seed= ;
	
	%put RandSVD= &RandSVD;
	%put RandSVD_error= &RandSVD_error;
	%global RandSVDFlag RandSVD_errorFlag;
	%if %upcase(&RandSVD) = NO %then %do; 
		%let RandSVDFlag= 0;
		%let RandSVD_errorFlag= 0;
		*Must set these variables to numeric values to avoid error in the IML code within the PredMacros.sas when not computing Randomized SVD*;
		%let RandSVD_K= 0; 
		%let RandSVD_Q= 0;
		%let RandSVD_seed= 0;
	%end;
	%else %do;
		%let RandSVDFlag= 1;
		%if %upcase(&RandSVD_error) = NO %then %do;
			%let RandSVD_errorFlag= 0;
		%end;
		%else %do;
			%let RandSVD_errorFlag= 1;		
		%end;
	%end;
	%put RandSVDFlag= &RandSVDFlag;
	%put RandSVD_errorFlag= &RandSVD_errorFlag;
	
	%let PrintFrag= 0;
	proc iml;
	
		start rsvd(_M, _O, _k, _q, _n, _seed) global(lambda, U);

			*1. Draw an n  k Gaussian random matrix.*;
			if IsEmpty(_O) then do;       
				_O = normal(j(_n, _k, _seed));
			end;
			%if &PrintFrag %then %do;
				print _O;
			%end;					
		
			*2. Form the m  k sample matrix Y = MO.*;
			Y = _M*_O;
			Free _O;
			*print Y;
		
			if _q then do;
				*2.1. subspace iterations Y= (MM`)^(q)MO.*;
				do i=1 to _q;
					CALL GSORTH(Q,R,lindep,Y);
					Z= _M`*Q;
					Free Q R Y;
		
					CALL GSORTH(Q,R,lindep,Z);
					Y= _M*Q;
					Free Q R Z;
				end;
			end;
			%if &PrintFrag %then %do;
				print Y;
			%end;
		
			*3. Form an m  k orthonormal matrix Q such that Y = QR.*;
			CALL GSORTH(Q,R,lindep,Y);
			%if &PrintFrag %then %do;
				print Q;
			%end;
			Free Y R;
		
			*4. Form the k  n matrix B = Q`M.*;
			B= Q`*_M;
			%if &PrintFrag %then %do;
				print B;
			%end;
			*create B_Mat from B;
			*append from B;
		
			*5. Compute the SVD of the small matrix B: B = UhatDiag(d)Vhat`*;
			ncb= ncol(B);
			nrb= nrow(B);
			if ncb > nrb then do;
				*either transpose B or select just non-zero columns from Uhat and lambda*;
				B= B`; 
				call svd(Vhat,lambda,Uhat,B);
			end;
			else do;
				call svd(Uhat,lambda,Vhat,B);
			end;
			Free B Vhat; 
			*create lambda_mat from lambda;
			*append from lambda;
			%if &PrintFrag %then %do;
				print Uhat lambda Vhat;
			%end;
		
			*6. Form the matrix U = QUhat*;
			U= Q*Uhat;
			free Q Uhat;
			%if &PrintFrag %then %do;
				print U;
			%end;
			*create U_Mat from U;
			*append from U;
		finish rsvd;
		store module=(rsvd);
	quit;
	**end: setting for fuilding additive and dominance relationship matrices. This is from GenomicBLUP.sas**;
	
	%if %length(%trim(&FixedEffVars &RandEffVarsOrig)) %then %do;
		/*S1366815: fixing error for different type of variables in the model*/
		data modvarsname;
			set &ds;
			keep &FixedEffVars &RandEffVarsOrig;
			where 0;
		run;
		
		%let _did_=%sysfunc(open(modvarsname,i));
		%let _nmv_=%sysfunc(attrn(&_did_, nvars));
		%let ModelVarsN= ;
		%let ModelVarsC= ;
		%let nnumvar= 0;
		%let nchavar= 0;
		%do i=1 %to &_nmv_;
			%let ModelVars= %sysfunc(varname(&_did_,&i));
			%let MVarType= %sysfunc(vartype(&_did_,&i));
			%if %upcase(&MVarType) = N %then %do;
				%let ModelVarsN= &ModelVarsN &ModelVars;
				%let nnumvar= %eval(&nnumvar + 1);
			%end;
			%else %do;
				%let ModelVarsC= &ModelVarsC &ModelVars;
				%let nchavar= %eval(&nchavar + 1);
			%end;
		%end;
		%put &ModelVarsN;
		%put &ModelVarsC;
	%end;
	
	* remove rows with missing predictors;
	data d_nomisspred;
		/*
		set &ts1
		%if %length(%trim(&LockContPredVars &LockClassPredVars &PredictorVars &ClassPredVars)) %then %do;
			array pv{*} &LockContPredVars &LockClassPredVars %if ^&upred %then %do; &PredictorVars %end; %else %do; %do j=1 %to &nix_blocks; &&PredictorVars&j %end; %end; &ClassPredVars;
			do _i_=1 to dim(pv);
				if missing(pv[_i_]) then do;
					delete;
					leave;
				end;
			end;
		%end;
		%if %length(&WeightVar) %then %do;
			if missing(&WeightVar) or &WeightVar = 0 then delete;
		%end;
		;
		*/
		/*
		set &ds;
		%if %length(%trim(&FixedEffVars &RandEffVarsOrig)) %then %do;
			*array pv{*} &FixedEffVars &RandEffVarsOrig _a_:;
			array pv{*} &FixedEffVars &RandEffVarsOrig;
			do _i_=1 to dim(pv);
				if missing(pv[_i_]) then do;
					delete;
					leave;
				end;
			end;
		%end;
		*/
		set &ds;
		%if %length(%trim(&FixedEffVars &RandEffVarsOrig)) %then %do;
			/*S1366815: fixing error for different type of variables in the model*/
			set &ds;
			%if %length(&ModelVarsN) %then %do;
				array nv{*} &ModelVarsN; 
				do _i_=1 to dim(nv);
					if missing(nv[_i_]) then do;
						delete;
						leave;
					end;
				end;
			%end;
			%if %length(&ModelVarsC) %then %do;
				array cv{*} &ModelVarsC; 
				do _i_=1 to dim(cv);
					if missing(cv[_i_]) then do;
						delete;
						leave;
					end;
				end;
			%end;
		%end;		
		/*
		%if %length(&WeightVar) %then %do;
			if missing(&WeightVar) or &WeightVar = 0 then delete;
		%end;
		*/
		;
		%if %length(%trim(&FixedEffVars &RandEffVarsOrig)) %then drop _i_;;
	run;
	
	/*data OutLib.d_nomisspred; set d_nomisspred; run;*/

	/*
	data d_nomisspred;
		set &ts1;
		if not missing(1) 
			%do i=1 %to &nixl; and not missing(%sysfunc(varname(&_dsixl_,&i))) %end; 
			%do i=1 %to &nipl; and not missing(%sysfunc(varname(&_dsipl_,&i))) %end; 
			%do i=1 %to &nix; and not missing(%sysfunc(varname(&_dsix_,&i))) %end; 
			%do i=1 %to &nip; and not missing(%sysfunc(varname(&_dsip_,&i))) %end; 
			%if %length(&WeightVar) %then %do;
				and not missing(&WeightVar) and &WeightVar > 0
			%end;
		;
	run;
	*/
	%let ts1 = d_nomisspred;
	%put ts1 = &ts1;
	
	/*
	%*put CVFlag = &CVFlag;
	%*put ts2 = &ts2;
	%*put TestSpec = &TestSpec;

	%if ^&CVFlag and (&TestSpec = Yes) and (&Kernel^=Dot) %then %do;
		data b2;
			set d d_test;
		run;
		%let ts2 = b2;
	%end;

	* using missing values for the dependent variable to create predictions;
	%if &CVFlag or (&TestSpec=Yes) or (&Kernel^=Dot) %then %do;
		data &ts1;
			set &ts1;
			_y0 = &DepVar;
			_y = &DepVar;
		run;
		data ts1o;
			set &ts1;
		run;
		data &ts2;
			set &ts2;
			_y0 = &DepVar;
			%if (&_dtype_=C) %then %do;
				_y = " ";
			%end;
			%else %do;
				_y = .;
			%end;
		run;
		data &ts1;
			set &ts1 &ts2;
			_obs = _n_;
			_one = 1;
			_name_ = "";
			drop _name_;
		run;
		%let UseDep = _y;
	%end;
	%else %do;
		%let UseDep = &DepVar;
		data &ts1;
			set &ts1;
			_obs = _n_;
			_one = 1;
			_name_ = "";
			drop _name_;
		run;
	%end;

	data d_nomiss;
		set &ts1;
		if not missing(&UseDep);
	run;

	%NObsVars(d_nomiss);
	%let nnmiss = &nobs;
	%put nnmiss = &nnmiss;

	%let uss1 = 1;
	%*let uss1 = &npv;
	%put uss1 = &uss1;
	%let ts1prev = &ts1;
	*/
	/*data OutLib.ts12; set &ts1; run;*/
	
	%if %upcase(&TraitType)= CONTINUOUS %then %do;
		proc means data=&ts1 noprint;
			**compute variance= (x - mu)^2/(n-1)**;
			var %do i=1 %to &Ntrait; &&_tv&i %end;;
			%if %length(&ByVars) %then %do;
				&ByVars;
			%end;
			output out=Sigma2traits var= %do i=1 %to &Ntrait; &&_tv&i %end;;
		run;
		/*data OutLib.gh_Sigma2Traits; set Sigma2Traits; run;*/
		
		proc transpose data=Sigma2traits out= Sigma2traitst(rename=(_name_= TraitName col1=Phenotypic_Variance));
			var %do i=1 %to &Ntrait; &&_tv&i %end;;
		run;
		
		/*data OutLib.gh_Sigma2Traitst; set Sigma2Traitst; run;*/
	%end;
	
	data LGroup;
		length &AnnoGroup $100;
		%do g=1 %to &NumGps;
			&AnnoGroup= "&&Groupn&g";
			_lgord0_= &g;
			output;
		%end;
	run;
	/*data OutLib.gh_LGroup; set LGroup; run;*/
	
	%*goto exit;

	%let numbChrToSkip= 0;
	%let listChrToSkip= ;
	%do g=1 %to &NumGps;
		*annotation group loop*;
		%put g=&g of &NumGps;
		%put Groupn= &&Groupn&g;
		%*goto loopend;

		data indat_sub;
			set /*indat*/ /*&ds*/ &ts1;
			*_IndivID_= _n_;
			%if &NumGps=1 %then %do;
				keep /*_IndivID_*/ &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars _a_:; 
			%end;
			%else %do;
				keep /*_IndivID_*/ &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars
				%do j=1 %to &nummark;
					%if %quote(&&HapGp&j)=%quote(&&Groupn&g) %then %do;
						%if %length(&GenoCol) %then %do;
							_a_&j
						%end;
						%else %do;
							%let j1=%eval(2*&j-1);
							%let j2=%eval(2*&j);
							_a_&j1 _a_&j2
						%end;
					%end;
				%end;
				;
			%end;
		run;	
		
		**start: Building additive and dominance relationship matrices. This is from GenomicBLUP.sas with modification on the additive matrix**; 
		%if %symexist(_dsimk_) %then %do;
			%let _rc_ = %sysfunc(close(&_dsimk_));
		%end;		
		
		%let ts1_sub= indat_sub;
		
		/*data OutLib.gh_ts1_sub&g; set &ts1_sub; run;*/
		
		data dsimk;
			set &ts1_sub(Keep= _a_:);
			where 0;
		run;
		%let _dsimk_ = %sysfunc(open(dsimk));
		%let nimk = %sysfunc(attrn(&_dsimk_,NVARS));
		
		proc means data=&ts1_sub noprint;
			var 
				%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
			;
			output out=mean_mrk 
				mean= 
				%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
			;
		run;
		
		/*data OutLib.gh_mean_mrk01; set mean_mrk; run;*/
		
		/*
		proc means data=&ts1_sub noprint;
			var 
				%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
			;
			output out=OutLib.gh_mean_mrk_tmp
				mean= 
			;
		run;
		*/
		
		proc transpose data=mean_mrk out=mean_mrk;
			var 
				%do i=1 %to &nimk; %sysfunc(varname(&_dsimk_,&i)) %end;
			;
		run;
		/*data OutLib.gh_mean_mrk02_&g; set mean_mrk; run;*/

		%let SwitchGeno= 0;
		data MAF_Data(where=(MinorAF > 0)); 
			set mean_mrk(rename=(col1=mean_mk)) end=_eof_;
			tmp=0;
			SwitchG= 0;
			MinorAF= mean_mk/2; /*minor allele frequency assuming diploid*/
			MajorAF= 1 - MinorAF; /*major allele frequency assuming diploid*/
			if MinorAF > MajorAF then do;
				tmp= MajorAF;
				MajorAF= MinorAF;
				MinorAF= tmp;
				SwitchG= 1;
				call symputx("SwitchGeno", 1);
			end;
			drop tmp;					
			if _n_ = 1 then _count_ = 0;
			if MinorAF = 0 then do;
				_count_= _count_ + 1;	
				call symputx("_md"||trim(left(_count_)),trim(left(_name_)));
				*delete;
			end;
			if _eof_ then call symputx("_nmd", _count_);
			retain _count_;
		run;		
		/*data OutLib.gh_MAF_Data&g; set MAF_Data; run;*/

		**S1368455: dropping markers with MAF of zero**;
		%put _nmd= &_nmd;
		%if &_nmd > 0 %then %do;
			%if &_nmd < &nimk %then %do;
				data &ts1_sub;	
					set &ts1_sub;
					drop 
					%do i=1 %to &_nmd;
						&&_md&i
					%end;
					;
				run;	
				%if &_nmd = 1 %then %do;
					%put WARNING: One marker was dropped from the annotation group &&Groupn&g after filtering for minor allele frequency of zero.;
				%end;
				%else %do;
					%put WARNING: A total of &_nmd markers were dropped from the annotation group &&Groupn&g after filtering for minor allele frequency of zero.;				
				%end;
			%end;
			%else %do;
				%let numbChrToSkip= %eval(&numbChrToSkip + 1);  
				%let listChrToSkip= &listChrToSkip &g;
				%put WARNING: All markers from the annotation group &&Groupn&g were dropped after filtering for minor allele frequency of zero.;
				%goto endchr1;
			%end;
		%end;
		
		/*data OutLib.gh_ts1_subd&g; set &ts1_sub; run;*/
		
		%*goto exit;

		%NObsVars(&ts1_sub); 
		%let nind= &nobs;
		
		%if &RandSVDFlag %then %do;
			%if %sysevalf(&RandSVD_K >= &nind) %then %do;
				%let RandSVDFlag= 0;
				%let RandSVD_errorFlag= 0;
			%end;
		%end;
		
		proc iml;
			do;
			**input marker data**;
			use &ts1_sub(keep=_a_:);
			read /*next 20*/ all var _num_ into M;
			close &ts1_sub;
			nind= nrow(M);
			nmrk= ncol(M);
			
			**input allele frequency data**;
			use MAF_Data;
			read /*next 10*/ all var {MinorAF MajorAF} into af;
			close MAF_Data;
			
			**Switch 0 and 2 codes if frequency of 2 is greater than frequency of 0. The code 2 is the reference minor allele**;
			%if &SwitchGeno %then %do;
				use MAF_Data;
				read /*next 10*/ all var {SwitchG} into sg;
				close MAF_Data;
				loc1= loc(sg = 1);
				*print loc1;
				T= M[,loc1];
				T[loc(T= 2)]= -9;
				T[loc(T= 0)]=  2;
				T[loc(T=-9)]=  0;
				M[,loc1]= T;
				*print M;
				free T;
			%end;			
			
			W= M - 2*shape(af[,1], nind, nmrk);
			*create OutLib.gh_W from W;
			*append from W;
			
			/*VanRaden (J Dairy Sci. 441423, 2008) and  Azevedo et al. (BMC Genetics 16:105, 2015) 
			define the the additive matrix defined as shown equation 1 below. This is the additive matrix 
			used in the Genomic BLUP*/
			*A= (W*W`)/(2*sum(af[,1]#af[,2])); /*Equation 1 - centered additive relationship matrix of Azevedo et al. and VanRaden method 1*/ 
			*W_STD= (W/sqrt(2*sum(af[,1]#af[,2])));  
			
			/* Yang et al. (AJHG 88:7682, 2011) defines the the additive 
			matrix defined as showns equation 2 and 3 below. Notice that
			there is a division by number of markers (N=nrow(M)). By doing so, 
			the variance component of matrix A in the mixed model will
			be the genetic variance explained by all SNPs entered in the computation
			of A. Otherwise, one would have to multiply the variance component of A 
			(without division by N) by N to restore the genetic variance explained
			by the SNPs. */	
			*sd= (1/(sqrt(2*af[,1]#af[,2])))`;
			*create OutLib.gh_sd&g from sd;
			*append from sd;
			
			W_STD= (W#(1/(sqrt(2*af[,1]#af[,2])))`); /*Equation 2 - centered additive relationship matrix from Yang et al. and VanRaden method 2*/
			*create OutLib.gh_W_STD&g from W_STD;
			*append from W_STD;
			
			free %if %upcase(&IncludeDom) ^= YES and %upcase(&AUnBiased) ^= YES %then M af; W;
			
			*Compute A matrix for each chromosome to estimate variance per chromosome.* 
			*Scaling each entry of the A the matrix by the number of non-missing markers*;
			missm= loc(W_STD =.);
			if ncol(missm) then do;
				W_STD0 = W_STD;
				W_STD0[ missm ] = 0;   /* replace missing values with 0 */
				non0= W_STD;
				non0[ loc(W_STD ^= .) ] = 1;   /* replace non missing values with 1 */
				non0[ missm ] = 0;   /* replace missing values with 0 */
				free W_STD;

				*create OutLib.gh_W_STD0&g from W_STD0;
				*append from W_STD0;
			
				*create OutLib.gh_non0&g from non0;
				*append from non0;			
				
				Nnon0= non0*non0`;
				free non0;
				WWt= (W_STD0*W_STD0`);
				free W_STD0;
			
				%if %upcase(&AUnBiased) = YES %then %do;
					*DiagWWt= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
					Z= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`));
					Z[missm]= 0;
					DiagWWt= Z*J(nmrk, 1, 1);
					*J(nmrk, 1, 1);  
					*create OutLib.gh_DiagWWt from DiagWWt;
					*append from DiagWWt;
					diagIdx = do(1, nind*nind, nind+1);
					WWt[diagIdx]= vecdiag(Nnon0) + DiagWWt;
					free DiagWWt diagIdx Z;
				%end;		
				
				*A = (WWt);
				*A = (WWt)#(1/Nnon0);
			end;
			else do;
				Nnon0= J(nind, nind, nmrk);
				WWt= (W_STD*W_STD`);
				free W_STD;
				
				%if %upcase(&AUnBiased) = YES %then %do;
					*b1= (1 + 2*af[,1]`)#M ;
					*create OutLib.gh_b1 from b1;
					*append from b1;					
					*b2= 2*((af[,1]##2)`);
					*create OutLib.gh_b2 from b2;
					*append from b2;		
					*b3= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`));
					*create OutLib.gh_b3 from b3;
					*append from b3;						
					DiagWWt= (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
					*DiagA= 1 + (M##2 - (1 + 2*af[,1]`)#M + 2*((af[,1]##2)`))*J(nmrk, 1, 1);  
					*create OutLib.gh_DiagA from DiagA;
					*append from DiagA;
					diagIdx = do(1, nind*nind, nind+1);
					WWt[diagIdx]= vecdiag(Nnon0) + DiagWWt;
					*A[diagIdx]= DiagA;
					free DiagWWt diagIdx;
				%end;	
				
				*A = (WWt);
				*A = (WWt)#(1/Nnon0);
			end;
			
			ZeroCount= loc(Nnon0 = 0);
			if ncol(ZeroCount) then do;
				call symputx("FlagZeroCount", 1);  
				goto TheEnd;
			end;
			else do;
				call symputx("FlagZeroCount", 0); 
			end;					
					
			A = (WWt)#(1/Nnon0);			
			
			
			%if %upcase(&IncludeDom) ^= YES and %upcase(&AUnBiased) = YES %then free M af;;
					
			*create OutLib.gh_Nnon0&g from Nnon0;
			*append from Nnon0;

			*create OutLib.gh_A&g from A;
			*append from A;	
			
			%if &NumGps > 1 %then %do;
				*Save each A matrix scaled by the total of automosomal markers to compute total heritability*;
				*WWt&g= (nmrk/&numnxmark)*A;
				*WWt&g= (W_STD0*W_STD0`);
				%if &g=1 %then %do;
					create WWt_Auto from WWt;
					append from WWt;
					
					create Nnon0_Auto from Nnon0;
					append from Nnon0;					
				%end;
				%else %do;
					use WWt_Auto;
					read all var _num_ into WWt_In;
					close WWt_Auto;
					WWt_In= WWt_In + WWt;
					create WWt_Auto from WWt_In;
					append from WWt_In;
					free WWt_In;
					
					use Nnon0_Auto;
					read all var _num_ into Nnon0_In;
					close Nnon0_Auto;
					Nnon0_In= Nnon0_In + Nnon0;
					create Nnon0_Auto from Nnon0_In;
					append from Nnon0_In;
					free Nnon0_In;					
				%end;
			%end;
			%else %if %upcase(&OutAddMatrix) = YES %then %do;
				labA= "ADD1":"ADD&nind";
				create A&g from A(|colname= labA|);
				append from A;
			%end;
			free WWt Nnon0;

			**input k, q and seed parameters for the randomized SVD**;
			RandSVDFlag= &RandSVDFlag;
			k= &RandSVD_K;
			q= &RandSVD_Q;
			seed= &RandSVD_seed;
			FrobFlag= &RandSVD_errorFlag;
			
			**compute square root of addtive matrix**;
			n= nrow(A);
			nlim= n/1.5;
			if RandSVDFlag & k <= nlim & k > 0 then do;
				O= {};
				load module=(rsvd);
				run rsvd(A, O, k, q, n, seed);
				if ^FrobFlag then do;
					free A;
				end;
			end;
			else do;
				call svd(U,lambda,V,A);
				if ^FrobFlag then do;
					free A V;
				end;
				else do;
					free V;
				end;
			end;
			
			if (RandSVDFlag & k <= nlim & k > 0) | ^RandSVDFlag then do;
				UL=U#sqrt(lambda)`;
				*UL=U*diag(sqrt(lambda));
			end;
			else if (RandSVDFlag & k > nlim) then do;
				UL=U[,1:k]#sqrt(lambda[1:k])`;
				*UL=U[,1:k]*diag(sqrt(lambda[1:k]));
			end;
			
			**compute additive matrix**;
			labR= "_&g.ra1":"_&g.ra&nind";
			
			*print lambda UL;
			free U lambda;
			create A_SVD_RT from UL (|colname=labR|);
			append from UL;
								
			if RandSVDFlag & FrobFlag then do;
				*Rebuild the input matrix from the square root matrix U = Udiag(d)U`*;
				A_re= UL*UL`;
				free UL;
				%if &PrintFrag %then %do;
					print A_re;
				%end;
				*create A_SVD from A_re;
				*append from A_re; 
				
				*Computing the Frobenius measure*;
				Frob= 100*norm(A - A_re, "Frobenius")/norm(A, "Frobenius");
				free A A_re;					
				Frob= k||q||Frob;
				FrobLab= {k q Frob_Percent};
				create Frob_A from Frob (|colname= FrobLab|);
				append from Frob;
			end;
			else do;
				free UL;
			end;
			
			/*
			**compute dominance matrix**;
			%if %upcase(&IncludeDom) = YES %then %do;
				labD= "_&g.DOM1":"_&g.DOM&nind";
				
				M= round(M);
				S= 2*(-(M=2)#(af[,2]##2)` + (M=1)#(af[,1]#af[,2])` - (M=0)#(af[,1]##2)`);
				free M;
				D= (S*S`)/(4*(sum((af[,1]#af[,2])##2)));
				free af S;
				
				*create OutLib.gh_D_Mat from D(|colname=labD|);
				*append from D;	
				
				**compute square root of dominance matrix**;
				n= nrow(D);
				nlim= n/1.5;
				if RandSVDFlag & k <= nlim & k > 0 then do;
					O= {};
					run rsvd(D, O, k, q, n, seed);
					if ^FrobFlag then do;
						free D;
					end;
				end;
				else do;
					call svd(U,lambda,V,D);
					if ^FrobFlag then do;
						free D V;
					end;
					else do;
						free V;
					end;
				end;
				
				if (RandSVDFlag & k <= nlim & k > 0) | (^RandSVDFlag) then do;
					UL=U#sqrt(lambda)`;
					*UL=U*diag(sqrt(lambda));
				end;
				else if (RandSVDFlag & k > nlim) then do;
					UL=U[,1:k]#sqrt(lambda[1:k])`;
					*UL=U[,1:k]*diag(sqrt(lambda[1:k]));
				end;
				*print U lambda;
				free U lambda;
				
				labR= "_&g.rd1":"_&g.rd&nind";
				create D_SVD_RT from UL (|colname=labR|);
				append from UL;
				
				if RandSVDFlag & FrobFlag then do;
					*Rebuild the input matrix from the square root matrix U = Udiag(d)U`*;
					D_re= UL*UL`;
					free UL;
					%if &PrintFrag %then %do;
						print D_re;
					%end;
					*create D_SVD from D_re;
					*append from D_re; 
					
					*Computing the Frobenius measure*;
					Frob= 100*norm(D - D_re, "Frobenius")/norm(D, "Frobenius");
					free D D_re;					
					Frob= k||q||Frob;
					FrobLab= {k q Frob_Percent};
					create Frob_D from Frob (|colname= FrobLab|);
					append from Frob;
				end;
				else do;
					free UL;
				end;						
			%end;
			*/

			TheEnd:
			end;
			
		quit; 
		%let _rc_ = %sysfunc(close(&_dsimk_));
		
		%if &syserr > 4 %then %do;
			%let exiterror=1;
			%goto exit;
		%end;
		
		%if &FlagZeroCount %then %do;
			%put ERROR: Due to excessive missing marker genotypes the additive relationship matrix could not be computed.;
			%let exiterror= 1;
			%goto exit;
		%end;

		/*data OutLib.gh_A_SVD_RT; set A_SVD_RT; run;*/
				
		%*goto exit;
				
		proc datasets nolist;
			*modify A_SVD_RT;
			delete A_SVD_RT&g;
			change A_SVD_RT=A_SVD_RT&g;
		quit;
		
		%let nlim= %sysevalf(&nind/1.5);
		%put nlim= &nlim;
		%if %upcase(&RandSVD) = YES and ^&RandSVDFlag %then %do;
			%let String1= The target rank (k= &RandSVD_K) of the low-rank SVD is not smaller than the number of observations (n= &nind) with non-missing values values for the dependent variable. Therefore, neither randomized nor truncated low-rank SVD was computed, instead, a full-rank SVD was computed.;
			%put WARNING: &String1; 					
		%end;
		%else %if &RandSVDFlag and %sysevalf(&RandSVD_K > &nlim) %then %do;
			%let String1= Truncated SVD was computed instead of randomized SVD because the target rank (k= &RandSVD_K) of the low-rank randomzed SVD  is not smaller than the number of observations (n= &nind) with non-missing values values for the dependent variable divided by 1.5 (i.e., k > n/1.5).;
			%put WARNING: &String1; 	
		%end;
		
		%if &RandSVDFlag %then %do;
			%let SVD_SETTING_TEXT= %nrbquote(k= )&RandSVD_K;
			%if %sysevalf(&RandSVD_K <= &nlim) %then %do;
				%let SVD_SETTING_TEXT= &SVD_SETTING_TEXT %nrbquote(and q= )&RandSVD_Q;
			%end;
		%end;

		%if &RandSVDFlag and &RandSVD_errorFlag %then %do;
			%let SVD_ERROR_TEXT= ;
			data _null_ /*OutLib.gh_Frob_A*/;
				set Frob_A;
				call symputx("RSVD_AError", trim(left(Frob_Percent)));
			run;
			%let SVD_ERROR_TEXT= Additive matrix= %sysfunc(round(&RSVD_AError,.11))%nrbquote(%); 
			%put SVD_ERROR_TEXT= &SVD_ERROR_TEXT;
		%end;				
		
		%if %upcase(&IncludeDom) = YES %then %do;
			/*data OutLib.gh_D_SVD_RT; set D_SVD_RT; run;*/
			%if &RandSVDFlag and &RandSVD_errorFlag %then %do;
				data _null_ /*OutLib.gh_Frob_D*/;
					set Frob_D;
					call symputx("RSVD_DError", trim(left(Frob_Percent)));
				run;
				%let SVD_ERROR_TEXT= &SVD_ERROR_TEXT.%nrbquote(, Dominance matrix=) %sysfunc(round(&RSVD_DError,.11))%nrbquote(%); 
				%put SVD_ERROR_TEXT= &SVD_ERROR_TEXT;
			%end;
		%end;

		*Reset variables*;
		%if %upcase(&RandSVD) = NO %then %do; 
			%let RandSVDFlag= 0;
			%let RandSVD_errorFlag= 0;
			*Must set these variables to numeric values to avoid error in the IML code within the PredMacros.sas when not computing Randomized SVD*;
			%let RandSVD_K= 0; 
			%let RandSVD_Q= 0;
			%let RandSVD_seed= 0;
		%end;
		%else %do;
			%let RandSVDFlag= 1;
			%if %upcase(&RandSVD_error) = NO %then %do;
				%let RandSVD_errorFlag= 0;
			%end;
			%else %do;
				%let RandSVD_errorFlag= 1;		
			%end;
		%end;
		%put RandSVDFlag= &RandSVDFlag;
		%put RandSVD_errorFlag= &RandSVD_errorFlag;	
		
		%endchr1:;
	%end;/* end chromosome loop */

	%if &numbChrToSkip = &NumGps %then %do;
		%put ERROR: All markers were dropped after filtering for minor allele frequency of zero.;
		%let exiterror= 1;
		%goto exit;
	%end;
	
	%*goto exit;
	
	**Create data set with additive matrix scaled for all autosomal markers to compute heritability**;
	**No need to do this when there is just one chromosome, because it will come out from the iml code above**;
	%if &NumGps > 1 %then %do;	
		%if &RandSVDFlag %then %do;
			%if %sysevalf(&RandSVD_K >= &nind) %then %do;
				%let RandSVDFlag= 0;
				%let RandSVD_errorFlag= 0;
			%end;
		%end;	
				
		/*data OutLib.gh_WWt_Auto; set WWt_Auto; run;*/	
		/*data OutLib.gh_Nnon0_Auto; set Nnon0_Auto; run;*/
	
		proc iml;
			*create A matrix with all autosomal markers*;

			use WWt_Auto;
			read all var _num_ into WWt;
			close WWt_Auto;
			
			use Nnon0_Auto;
			read all var _num_ into Nnon0;
			close Nnon0_Auto;
			
			A= (WWt)#(1/Nnon0);
			free WWt Nnon0;
			
			%if %upcase(&OutAddMatrix) = YES %then %do;
				labA= "ADD1":"ADD&nind";
				create A_Auto from A(|colname= labA|);
				append from A;
			%end;
			
			**input k, q and seed parameters for the randomized SVD**;
			RandSVDFlag= &RandSVDFlag;
			k= &RandSVD_K;
			q= &RandSVD_Q;
			seed= &RandSVD_seed;
			FrobFlag= &RandSVD_errorFlag;
			
			**compute square root of addtive matrix**;
			n= nrow(A);
			nlim= n/1.5;
			if RandSVDFlag & k <= nlim & k > 0 then do;
				load module=(rsvd);
				O= {};
				run rsvd(A, O, k, q, n, seed);
				if ^FrobFlag then do;
					free A;
				end;
			end;
			else do;
				call svd(U,lambda,V,A);
				if ^FrobFlag then do;
					free A V;
				end;
				else do;
					free V;
				end;
			end;
			
			if (RandSVDFlag & k <= nlim & k > 0) | ^RandSVDFlag then do;
				UL=U#sqrt(lambda)`;
				*UL=U*diag(sqrt(lambda));
			end;
			else if (RandSVDFlag & k > nlim) then do;
				UL=U[,1:k]#sqrt(lambda[1:k])`;
				*UL=U[,1:k]*diag(sqrt(lambda[1:k]));
			end;
			
			labR= "_rau1":"_rau&nind";
			*print lambda UL;
			free U lambda;
			create Auto_SVD_RT from UL (|colname=labR|);
			append from UL;			
			
		quit;
		
		/*data OutLib.gh_Auto_SVD_RT; set Auto_SVD_RT; run;*/
				
		proc datasets nolist;
			delete WWt_Auto Nnon0_Auto;
		quit;		
	%end;
	
	%if &NumGps > 1 %then %do;
		%if %upcase(&OutAddMatrix) = YES %then %do;
			data OutLib.&GH_ADOutDataName;
				retain _ColumnSuffix_;
				merge 
					%if (%length(&IDVars) or %length(&InputRowID)) %then &ts1(keep= &IDVars &InputRowID);
					A_Auto
				;
				_ColumnSuffix_= _n_;
			run;
		%end;
		
		%if %upcase(&OutRootAddMatrix) = YES %then %do;
			data OutLib.&GH_RAOutDataName;
				retain _ColumnSuffix_;
				merge 
					%if (%length(&IDVars) or %length(&InputRowID)) %then &ts1(keep= &IDVars &InputRowID);
					Auto_SVD_RT(rename=(%do i=1 %to &nind; _rau&i= ra&i %end;))
				;
				_ColumnSuffix_= _n_;
			run;
		%end;
	%end;
	%else %do;
		%if %upcase(&OutAddMatrix) = YES %then %do;
			data OutLib.&GH_ADOutDataName;
				retain _ColumnSuffix_;
				merge 
					%if (%length(&IDVars) or %length(&InputRowID)) %then &ts1(keep= &IDVars &InputRowID);
					A1
				;
				_ColumnSuffix_= _n_;
			run;
		%end;	
	
		%if %upcase(&OutRootAddMatrix) = YES %then %do;
			data OutLib.&GH_RAOutDataName;
				retain _ColumnSuffix_;
				merge 
					%if (%length(&IDVars) or %length(&InputRowID)) %then &ts1(keep= &IDVars &InputRowID);
					A_SVD_RT1(rename=(%do i=1 %to &nind; _1ra&i= ra&i %end;))
				;
				_ColumnSuffix_= _n_;
			run;
		%end;
	%end;
	
	%*let exiterror=1;
	%*goto exit;
	
	*preparing for model fit*;
	%let covp= 1;
	%let Genotype= ;
	%let IntEffBuild= ;
	%let IntTestsOnly= NO;
	%let geno= 1;
	%let lsm= 0;
	%let snpsnp= 0;
	%let survey= 0;
	%let OutputMajAllele= ;
	%let ClusterVars= ;
	%let WeightVar= ;
	%let OutputSurvFunc= No;
	%let p= 0;
	%let pest= 1;
	%let OutputRSquare= No;
	%let ParmsStmt= ;
	
	%*if %length(&KVars) %then %do;
		%let kmodel= 1;
	%*end;
	%*else %*do;
		%*let kmodel= 0;
	%*end;
	
	/*
	%if %Upcase(&Odds)=YES  %then %do; 
		data or_all;
			delete;
		run;
	%end;
	*/

	data ma_a;
		length Effect $32;
	  	delete;
	run;

	data pag_all;
		length &EffName Trait $32;
		delete;
	run;

	data cpg_all;
		length Trait $32;
		delete;
	run;

	data sfg_all;
		length Trait $32;
		delete;
	run;

	%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
		data outresg_all;
			length _Trait_ $32;
			delete;
		run;
	%end;

	%if &survey=1 %then %do;
		data rs_all;
			delete;
		run;
	%end;

	%if &lsm=1 %then %do;
		data outlsm_all;
			length Trait $32 Effect $17;
			delete;
		run;
	%end;	
	
	%*let tmpClassVars= &ClassVars;
	%*let tmpRandEff= &RandEff;
	%*let tmpds= &ds;
	
	%let Flag_EverNonConvergence = 0;
	%global exiterror_fit Flag_NonConvergence;
	
	%macro FitMixedModel;
		
		%let Flag_NonConvergence= 0;
		%let exiterror_fit= 0;

		%*goto exit_fit;	

		%do t=1 %to &NTrait;
			*begin: trait loop*;

			%let TraitVar  = &&_tv&t;

			%let ie=0;
			%let Interaction=;

			%if %upcase(&IntTestsOnly)=YES %then %do;
				%let ie=1;
				%let Interaction=%scan(&IntEffBuild,&ie,' ');
			%end;			

			%do %until(%length(%scan(&IntEffBuild,&ie,' '))=0);

				title "Trait &TraitVar";
				options nonotes;

				%if %upcase(&Listing)=NO %then %do;
					ods listing close;
				%end;	

				%if (&geno=1) %then %do;	

					data ma; /* kemicl - added to ensure data is not repeated if PROC does not produce estimates */
						delete;
					run;
					
					data pag %if %upcase(&OutputFitStat)=YES %then fsg;; /* sbrlsi - added to avoid error when the procedure does not converge */
						delete;
					run;					

					proc datasets nolist;
						delete outres;
					quit;
					
					%*let exiterror_fit= 1;
					%*goto exit_fit;

					*title2 "Genotype Test &Interaction";
					ods output &Gtable=ma
						%if &p=0 %then %do;
							%if &WhichProc ne SURVEYREG %then %do;
								nobs = mnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
							%end;
							&ParmEst=pag 
							%if (&kmodel=1 or %length(&RandEff)) and &covp=1 %then CovParms=cp; 
							%if (&kmodel=1 or %length(&RandEff)) %then ConvergenceStatus=convstatus/*(where=(status=1))*/;
							%if &lsm=1 and &ie=0 %then lsmeans=lsm diffs=lsd;
							%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 							
							/*begin: Luciano*/
							%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
								rsquare= grsq;
							/*end: Luciano*/
							%if %upcase(&OutputFitStat)=YES %then 
								FitStatistics= fsg;
						%end;
					;

					proc &WhichProc	data=/*&ds*/ indat_sub &ProcOption %if &survey=1 %then &ratetotal; ;
						class /*%sysfunc(translate(&Genotype,%str( ),%str(*)))*/ &ClassVars &ClassOpt;

						model &TraitVar &TraitOpt = &FixedEff &Interaction &ModOption;

						%if &kmodel=1 %then %do;
							%do h=1 %to &NumGps;
								%if %length(&&KVars&h) %then %do;
									random &&KVars&h / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
								%end;
							%end;
						%end;

						%if %length(&RandEff)>0 %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&ParmsStmt) %then %do;
							parms &ParmsStmt ;
						%end;

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						%if &lsm=1 %then %do;
							%if &snpsnp %then %do;
								lsmeans &genotype / diff=control('0' '0');							
							%end;
							%else %do;
								lsmeans &genotype / diff=control('0');
							%end;
						%end;

						%if &survey=1 %then %do;
							%if %length(&ClusterVars) %then %do;
								cluster &ClusterVars;
							%end;
							%if %length(&WeightVar) %then %do;
								weight &WeightVar;
							%end;
						%end;	
						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
							id _IndivID_;
						%end;
						/*by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;*/

						%if %length(&ByVars) %then %do;
							&ByVars;
						%end;

						%if %upcase(&OutputSurvFunc)=YES or ((%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %upcase(&TraitType)=SURVIVAL) %then %do;
							output out=outres
							%if %upcase(&OutputSurvFunc)=YES %then survival=SurvivalFunctionEst;
							%if %upcase(&OutputResid)=YES and %upcase(&TraitType)=SURVIVAL %then resdev=Resid;
							%if %upcase(&OutputPred)=YES and %upcase(&TraitType)=SURVIVAL %then predicted=pred;
						   ;
						%end;
					quit;
					
					options notes;

					%if %length(&FixedEff) %then %do;
						/* kemicl - added to catch error when all effects are not estimable */	
						%NObsVars(ma);
						%if ^&nobs %then %do;
							%*put ERROR:  PROC &WhichProc generated no estimates for the Genotype Test for trait &TraitVar..  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
							%put ERROR:  PROC &WhichProc generated no estimates for trait &TraitVar..  There may be excessive missing data. Please see the log for more details.;							
							%goto skip1;
						%end;
					%end;
					%else %do;
						%if (&syserr>4) %then %do;
							%let exiterror_fit=1;
							%goto exit_fit;
						%end;						
					%end;						
					
					%if (&kmodel=1 or %length(&RandEff)) %then %do;

						data ConvStatus1;
							set ConvStatus(where=(status=1));
						run;
						
						%NObsVars(ConvStatus1);	
						%if &NObs %then %do;												
							%let Flag_NonConvergence = 1;	
							%let Flag_EverNonConvergence = 1;
						%end;						
						
					%end;
						
					%if (&kmodel=1 or %length(&RandEff)) and %upcase(&OutputFitStat)=YES %then %do;

						%if &Flag_NonConvergence %then %do;															
							data fsg;
								length Descr $50 Value 8;
								Descr= '';
								Value= .;
							run;

							options label;
							data fsg;
								merge fsg ConvStatus1(keep=Reason);	
								label Reason= "Convergence Status";					
							run;
							*options nolabel;
						%end;
					%end;						
					
					%*let ts1_sub = &ts1prev;

					%*put Test = Genotype, Trait = &TraitVar, Effects = Genotype &Interaction, Permutation = &p;  /* kemicl - added to put details of run in log */                   

					%if &p=0 %then %do;

						%if %upcase(&TraitType)=BINARY %then %do;
							proc transpose data=rp out=rp(drop=_name_) prefix=NumTrait;
								%if %length(&ByVars) %then %do;
									by /*_order_*/ &ByVars;
								%end;
								var count;
								id %if %index(%upcase(&WhichProc),LOGISTIC) %then Outcome; %else &TraitVar;;
							run;

							%let datatomerge=rp;
						%end;
						%else %let datatomerge=;					

						%if &WhichProc ne SURVEYREG %then %do;
							data ma;
								merge mnobs(keep=/*_order_*/ &ByVars nobsused) &datatomerge ma;
								%if %length(&ByVars) %then %do;
									by /*_order_*/ &ByVars;
								%end;
								rename nobsused=SampleSize;
							run;
						%end;

						/*begin: including RSquare for logistic regression. Luciano*/
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then %do;
							data ma;
								merge ma grsq(keep= /*_order_*/ nValue1 nValue2 %if &OutputMajAllele %then MajorAllele; &ByVars);
								%if %length(&OutputMajAllele &ByVars) %then %do;
									by /*_order_*/ %if &OutputMajAllele %then MajorAllele; &ByVars;
								%end;
								rename nValue1= RSquare_Genotype nValue2=Rescaled_RSquare_Genotype;
							run;					
						%end;
						/*end: Luciano*/

						%if %upcase(&OutputFitStat)=YES %then %do;
							%*put WhichProc= &WhichProc;
							/*data OutLib.fsg; set fsg; run;*/
							/*data OutLib.ma; set ma; run;*/

							proc transpose data=fsg(
								%if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
									rename=(
										criterion= descr 
										interceptandcovariates= value
									)
								%end;
								%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
									rename=(
										criterion= descr  
										withcovariates= value
									)
								%end;
								)
								out=fsg(drop= _name_); 
								var value;
								id descr;
								%if %length(&OutputMajAllele &ByVars) %then %do;
									by /*_order_*/ %if &OutputMajAllele %then MajorAllele; &ByVars;
								%end;
							run;

							/*data OutLib.fsg2; set fsg; run;*/

							data ma;
								merge ma fsg;
								%if %length(&OutputMajAllele &ByVars) %then %do;
									by /*_order_*/ %if &OutputMajAllele %then MajorAllele; &ByVars;
								%end;
								rename
								%if %index(%upcase(&WhichProc),MIXED) %then %do;
									_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Genotype
									AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Genotype
									AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Genotype
									BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Genotype
								%end;
								%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
									%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
										or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
										%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
											_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Genotype
											Generalized_Chi_Square= Generalized_ChiSquare_Genotype
											Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Genotype
										%end;
										%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
											_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Genotype
											Generalized_Chi_Square= Generalized_ChiSquare_Genotype
											Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Genotype
										%end;
										%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
											_2_Log_Likelihood= _2_Log_Likelihood_Genotype
											AIC___smaller_is_better_= AIC_Smaller_is_Better_Genotype
											AICC__smaller_is_better_= AICC_Smaller_is_Better_Genotype
											BIC___smaller_is_better_= BIC_Smaller_is_Better_Genotype
											CAIC__smaller_is_better_= CAIC_smaller_is_better_Genotype
											HQIC__smaller_is_better_= HQIC_smaller_is_better_Genotype											
										%end;
									%end;
									%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 		
										_2_Log_Likelihood= _2_Log_Likelihood_Genotype
										AIC___smaller_is_better_= AIC_Smaller_is_Better_Genotype
										AICC__smaller_is_better_= AICC_Smaller_is_Better_Genotype
										BIC___smaller_is_better_= BIC_Smaller_is_Better_Genotype
										CAIC__smaller_is_better_= CAIC_smaller_is_better_Genotype
										HQIC__smaller_is_better_= HQIC_smaller_is_better_Genotype
										Pearson_Chi_Square= Pearson_ChiSquare_Genotype
										Pearson_Chi_Square___DF= Pearson_ChiSq_DF_Ratio_Genotype											
									%end;
									%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
										%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
											_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Genotype
										%end;
										%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
											_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Genotype
										%end;
										%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
											_2_Log_Likelihood= _2_Log_Likelihood_Genotype
											AIC___smaller_is_better_= AIC_Smaller_is_Better_Genotype
											AICC__smaller_is_better_= AICC_Smaller_is_Better_Genotype
											BIC___smaller_is_better_= BIC_Smaller_is_Better_Genotype
											CAIC__smaller_is_better_= CAIC_smaller_is_better_Genotype
											HQIC__smaller_is_better_= HQIC_smaller_is_better_Genotype
										%end;	
									%end;
									%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
										%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
											_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Genotype
											Generalized_Chi_Square= Generalized_ChiSquare_Genotype
											Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Genotype
										%end;
										%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
											_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Genotype
											Generalized_Chi_Square= Generalized_ChiSquare_Genotype
											Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Genotype											
										%end;
									%end;										
								%end;
								%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
									_2_LOG_L= _2_LOG_L_Genotype
									AIC= AIC_Genotype
									SBC= SBC_Genotype
								%end;
								%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
									AIC= AIC_Genotype
									SC= SC_Genotype
									_2_Log_L= _2_Log_L_Genotype
								%end;
								;
							run;
						%end;

						%if %length(&FixedEff) %then %do;
							data ma_a; 
								length Trait /*%if %length(&AnnoGroup) %then &AnnoGroup;*/ MarkerSet $32 ;
								set ma_a(in=orig) ma;
								%if &ie>0 %then %do;
									if orig=1 or index(Effect,"Genotype*")>0 or index(Effect,"*Genotype")>0; 
									/* To make sure interactions in same order for Genotype and Trend tests */
									if index(Effect,"Genotype*")>1 then do;
										Effect=cats("Genotype*",tranwrd(Effect,"Genotype*",""));
									end;
									else if index(Effect,"Genotype")>1 then do;
										Effect=cats("Genotype*",tranwrd(Effect,"*Genotype",""));
									end;
								%end;
								%else %do;
									where index(Effect,"&Genotype")>0; 
								%end;
								if not orig then do;
									Trait = "&TraitVar";
									/*%if %length(&AnnoGroup) %then &AnnoGroup= trim(left(&Groupn));;*/
									MarkerSet= trim(left(&MarkerSet));
								end;
								label Prob&Stat=' ';
							run;		
						%end;
						%else %do;
							data ma_a; 
								length Trait /*%if %length(&AnnoGroup) %then &AnnoGroup;*/ MarkerSet $32 ;
								set ma_a(in=orig) ma;
								if not orig then do;
									Trait = "&TraitVar";
									/*%if %length(&AnnoGroup) %then &AnnoGroup= trim(left(&Groupn));;*/
									MarkerSet= trim(left(&MarkerSet));
								end;
							run;								
						%end;

						%if &pest = 1 %then %do;
							data pag_all;	
								length Trait /*%if %length(&AnnoGroup) %then &AnnoGroup;*/ MarkerSet $32 ;
								%if %length(&IntEffBuild) %then %do;
									length Interaction $32;
								%end;
								set pag_all(in=orig) pag;
								if orig=0 then do;
									Trait="&TraitVar";
									/*%if %length(&AnnoGroup) %then &AnnoGroup= trim(left(&Groupn));;*/
									MarkerSet= trim(left(&MarkerSet));										
									%if %length(&IntEffBuild) %then %do;
										Interaction="&Interaction";
									%end;
								end;
							run;
						%end;

						%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and &ie=0 %then %do;

							/*data outlib.outres; set outres; run;*/

							%if %sysfunc(exist(outres)) %then %do;
								data outresg_all;
									set outresg_all(in=orig) outres;
									if orig=0 then do;
										_Trait_="&TraitVar";
									end;
								run;		
							%end;
						%end;

						%if &lsm=1 and &ie=0 %then %do;
							data outlsm_all;
								set outlsm_all(in=orig) lsm lsd(in=idiff);
								if orig=0 then do;
									diff=0;
									Trait="&TraitVar";
									if idiff then do;
										diff=1;
										%if &snpsnp %then %do;
											Effect = "Diff with (0,0)";										
										%end;
										%else %do;
											Effect = "Diff with 0";
										%end;
									end;

								end;
							run;		
						%end;				

						%if (&kmodel=1 or %length(&RandEff)) and &covp %then %do;
						
							/*data OutLib.cp; set cp; run;*/
							/*data OutLib.ConvStatus; set ConvStatus; run;*/
							
							data cp;
								merge cp ConvStatus(keep=Reason Status);
								rename Reason= Convergency_Message Status= Convergency_Status;
							run;
						
							data cpg_all;
								length Trait 
									/*%if %length(&AnnoGroup) %then &AnnoGroup;*/
									MarkerSet
									%if %length(&IntEffBuild) %then Interaction;
									$32;				
								set cpg_all(in=orig) cp;
								if orig=0 then do;
									Trait="&TraitVar";
									/*%if %length(&AnnoGroup) %then &AnnoGroup= trim(left(&Groupn));;*/
									MarkerSet= trim(left(&MarkerSet));										
									%if %length(&IntEffBuild) %then %do;
										Interaction="&Interaction";
									%end;
								end;
							run;
						%end;

						%if %upcase(&OutputSurvFunc)=YES %then %do;
							proc sort data=outres out=geno nodupkey;
								by /*_order_*/ &genotype;
							run;

							data geno;
								set geno;
								SurvivalFunctionEst=1;
								&TraitVar = 0;
							run;

							data survcurvesg;
								set outres geno;
								if &genotype ne .;
							run;

							data sfg_all;
								%if %length(&IntEffBuild) %then %do;
									length Interaction $32;
								%end;						
								set sfg_all(in=orig) survcurvesg;
								if orig=0 then do;
									Trait="&TraitVar";
									%if %length(&IntEffBuild) %then %do;
										Interaction="&Interaction";
									%end;
								end;
							run;								

						%end;							
					%end; /* end p=0 */
				%end;

				%skip2:;

				/*
				%if &ie=0 and &survey=1 %then %do;
					%if &raoscottf=1 or &raoscottchi=1 %then %do;

						title2 'Rao-Scott Test';

						ods output ChiSq = rscs(where=(Name1 ? "RS"));;
						proc surveyfreq data=genotypes &ratetotal;
							%if %length(&StrataVars)>0 %then %do;
								strata &StrataVars;
							%end;
							%if %length(&ClusterVars) %then %do;
								cluster &ClusterVars;
							%end;
							%if %length(&WeightVar) %then %do;
								weight &WeightVar;
							%end;			
							tables &TraitVar * Genotype / chisq noprint;
							by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;
						run;

						%if (&syserr>4) %then %do;
							%let exiterror_fit=1;
							%goto exit_fit;
						%end;					

						options notes;
						%if %sysfunc(exist(rscs))=0 %then %do;

							data rs;
								set genotypes(keep=_order_ %if &OutputMajAllele %then MajorAllele; &ByVars);
								by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;
								%if %length(&ByVars) %then %do;
									if first.&lastby;
								%end;
								%else %do;
									if first._order_;
								%end;
								retain Effect "Genotype";
								%if &raoscottchi=1 %then %do;
									retain ChiSq_RaoScott DF_RaoScott Prob_RaoScott_ChiSq .;
								%end;
								%if &raoscottf=1 %then %do;
									retain FValue_RaoScott NDF_RaoScott	DDF_RaoScott Prob_RaoScott_F .;
								%end;
							run;
						%end;
						%else %do;
							proc transpose data=rscs out=rs;
								var nValue1;
								by _order_ %if &OutputMajAllele %then MajorAllele; &ByVars;
								id Name1;
							run;		

							proc datasets nolist;
								delete rscs;
							quit;

							data rs;
								set rs;
								drop _name_ %if &raoscottf=0 %then _rsf_ ndf_rsf ddf_rsf p_rsf; 
												%if &raoscottchi=0 %then _rschi_ df_rschi P_rschi;;
								rename %if &raoscottchi=1 %then _rschi_=ChiSq_RaoScott df_rschi=DF_RaoScott P_rschi=Prob_RaoScott_ChiSq;
										 %if &raoscottf=1 %then _rsf_=FValue_RaoScott ndf_rsf=NDF_RaoScott ddf_rsf=DDF_RaoScott p_rsf=Prob_RaoScott_F;;
								Effect="Genotype";
							run;
						%end;

						data rs_all; 
							length Trait $32;
							set rs_all(in=orig) rs;
							if not orig then Trait = "&TraitVar";
							label %if &raoscottchi=1 %then Prob_RaoScott_ChiSq=' '; %if &raoscottf=1 %then Prob_RaoScott_F=' ';;
						 run;

					%end;
				%end;	
				*/

				options notes;

				title2;

				%let ie=%eval(&ie+1);   
				%let Interaction=%scan(&IntEffBuild,&ie,' ');

			%end; /* end loop over interactions */

			%skip1:;

			%endtrait:;
			* end loop over traits *;
		%end;  		
		%exit_fit:;
		
	%mend FitMixedModel;
	
	**Fitting an individual mixed model for each chromosome**;
	/*
	%do g=1 %to &NumGps;	
		%if ^%sysfunc(findw(&listChrToSkip, &g)) %then %do;
			*annotation group loop*;
			%put g=&g of &NumGps;
			%let Groupn= "&&Groupn&g";
			%put Groupn= &Groupn;
	
			data &ts1_sub;
				merge 
					&ts1(keep= &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars) 
					A_SVD_RT&g
					%if %upcase(&IncludeDom) = YES %then %do;
						D_SVD_RT&g
					%end;
				;
				_IndivID_ = _n_;
			run;	
	
			data OutLib.gh_ts1_sub; set &ts1_sub; run;
	
			%*goto endchr3;
	
			%FitMixedModel;
	
			%put exiterror_fit= &exiterror_fit;
			%if &exiterror_fit %then %do;
				%let exiterror=1;
				%goto exit;
			%end;
	
			%endchr3:;
		%end;
	* end chromosome loop *;
	%end;
	*/

	**Estimating genetic variance by chromosome**;
	data &ts1_sub;
		merge 
			&ts1(keep= &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars) 
			%do g=1 %to &NumGps; 
				%if ^%sysfunc(findw(&listChrToSkip, &g)) %then %do;
					A_SVD_RT&g
					%if %upcase(&IncludeDom) = YES %then %do;
						D_SVD_RT&g
					%end;
				%end;
			%end;
		;
		_IndivID_ = _n_;
	run;	

	/*data OutLib.gh_ts1_sub; set &ts1_sub; run;*/
	
	%if &NumGps > 1 %then %do;
		%let MarkerSet= "By_Chromosome";
	%end;
	%else %do;
		%let MarkerSet= "All_Chromosomes";
	%end;
	%do g=1 %to &NumGps; 
		%let KVars&g= ;
		%if ^%sysfunc(findw(&listChrToSkip, &g)) %then %do;
			%let KVars&g= _&g.ra:;
		%end;
	%end;

	%FitMixedModel;

	%put exiterror_fit= &exiterror_fit;
	%if &exiterror_fit %then %do;
		%let exiterror=1;
		%goto exit;
	%end;
	
	**Estimating genetic variance explained by all autosomal markers to compute heritability**;
	%if &NumGps > 1 %then %do;
		
		data &ts1_sub;
			merge 
				&ts1(keep= &FixedEffVars &RandEffVars &RandEffVarsOrig &TraitVars &InputRowID &IDVars &ByVars &SexVar &IDVars) 
				Auto_SVD_RT
				%if %upcase(&IncludeDom) = YES %then %do;
					D_Auto_SVD_RT
				%end;
			;
			_IndivID_ = _n_;
		run;

		/*data OutLib.gh_ts1_sub; set &ts1_sub; run;*/

		%let OrigNumGps= &NumGps;
		%let NumGps= 1;
		%let MarkerSet= "All_Chromosomes";
		%let KVars1= _rau:;
		
		%FitMixedModel;
		
		%let NumGps= &OrigNumGps;

		%put exiterror_fit= &exiterror_fit;
		%if &exiterror_fit %then %do;
			%let exiterror=1;
			%goto exit;
		%end;
	
	%end;
	
	%let did=%sysfunc(open(cpg_all));
	%let AnnGrpVar=%sysfunc(varnum(&did,&AnnoGroup));
	%let rc=%sysfunc(close(&did));
	
	%if &AnnGrpVar %then %do;
		%if %sysevalf(%length(&AnnoGroup) >= 30) %then %do;
			%let AnnGrpNew= _%trim(%left(%substr(&AnnoGroup, 1, 30)))_;
		%end;
		%else %do;
			%let AnnGrpNew= _%trim(%left(&AnnoGroup))_;
		%end;
		%put AnnGrpNew= &AnnGrpNew;
	%end;
	
	/*data OutLib.gh_cpg_all1; set cpg_all; run;*/	
	
	data cpg_all;
		set cpg_all;
		_ord_=_n_;
		if _n_ = 1 then _count_ = 1;
		if upcase(covparm) ^= "VARIANCE" then do;
			_count_ = 1;
		end;
		else do;
			covparm= "SNPs";
			_lgord1_= _count_;
			_count_= _count_ + 1;
		end;
		retain _count_;
		drop _count_;
		label covparm= ' ';
		rename covparm= Source_Variance %if &AnnGrpVar %then %do; &AnnoGroup= &AnnGrpNew %end;;
	run;
	
	/*data OutLib.gh_cpg_all2; set cpg_all; run;*/	
	
	proc sql Undo_Policy=None;
		%if %upcase(&TraitType)= CONTINUOUS %then %do;
			create table cpg_all(drop= TraitName) as
			select a.*, b.*
			from cpg_all as a left join Sigma2Traitst as b
			on a.Trait=b.TraitName
			order by _ord_;
		%end;
		
		create table cpg_all(drop= _lgord1_ _lgord0_) as
		select a.*, b.*
		from cpg_all as a left join LGroup as b
		on a._lgord1_=b._lgord0_
		order by _ord_;	
	quit;
	
	/*data OutLib.gh_cpg_all3; set cpg_all; run;*/
		
	%let FlagHeritabilityError= 0;
	data cpg_all;
		retain Trait MarkerSet %if %length(&AnnoGroup) %then &AnnoGroup;;
		set cpg_all(drop= _ord_);
		if Source_Variance = "SNPs" then do;
			Heritability= Estimate/Phenotypic_Variance;
			if Heritability > 1 then do;
				call symputx("FlagHeritabilityError", 1);
			end;
		end;
		%if %length(&AnnoGroup) %then %do;
			if MarkerSet = "All_Chromosomes" then &AnnoGroup= "";
		%end;
		%if %length(&AnnoGroup) and &NumGps = 1 %then drop &AnnoGroup;;
		%if %upcase(&TraitType) ^= CONTINUOUS %then %do;
			drop Heritability Phenotypic_Variance;
		%end;
	run;
	%put FlagHeritabilityError= &FlagHeritabilityError;
	
	data OutLib.&GH_H2OutDataName;
		set cpg_all;
	run;
	
	data OutLib.&GH_FSOutDataName;
		set ma_a; 
	run;
	
	data OutLib.&GH_PEOutDataName;
		set pag_all; 
	run;
	
	%*let exiterror=1;
	%*goto exit;
	
	%put ClientOutPath= &ClientOutPath;
	%let wrapvars= 110;
			
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&GH_Script);

	data _null_;
		file "&GH_Script" lrecl=32767;
		put '//!';
		put 'wait(0.5);';
		put "Names Default to Here(1);";
		put "if(host is(Windows),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, windows) ),";
		put "	include( ConvertFilePath(""$LS_HOME/Generator/DialogHelperFunctions.jsl"", absolute, POSIX) ));";
		put "	";
		put "Here:DataList = {};";
		put "Here:TabButtonList = {};";
		put "Here:TabCodeList = {};";
		put "Here:TabAfterCodeList = {};";
		put "Here:TabOpenList = {};";
		put "Here:VarAllLGFunc = Function({},{default local},";
		put "	If(IsEmpty(Here:H2dat),";
		put " 	Here:H2dat = open(""&ClientOutPath.&GH_H2OutDataName..sas7bdat"",invisible);";
		put " ,"; 
		put "		Current Data Table(Here:H2dat);";
		put "	);";
		put "   Here:vlb_allg= VListBox(";
		put "	Here:olb_allg= OutLineBox(""Genomic Variance"",";
		put "	Here:VarGB_All= Graph Builder(";
		put "			Show Footer( 0 ),";
		put "			Show Control Panel( 0 ),";
		put "			Variables( X( :Trait ), Y( :Estimate ), Color( :Trait ) ),";
		put "			Elements( Bar( X, Y, Legend( 11 ) ) ),";
		put "			Where( :MarkerSet == ""All_Chromosomes"" & :Source_Variance == ""SNPs""),";
		put "			SendToReport(";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					OutlineBox,";
		put "					{Set Title( ""Total Genomic Variance"" )}";
		put "				),";
		put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
		put "				Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Total Genomic Variance"" )} )";
		put "			)";
		put "			//Local Data Filter(";
		put "			//	Add Filter(";
		put "			//		columns( :MarkerSet ),";
		put "			//		Where( :MarkerSet == ""All_Chromosomes"" ),";
		put "			//		Display( :MarkerSet, Size( 202, 38 ), List Display )";
		put "			//	)";
		put "			//)";
		put "		),";
		put "		),";
		put "		try(Here:olb_allg[TextBox(1)]<<Delete;);";
		put "	),";
		put ");";
		%if &NumGps > 1 %then %do;
			put "Here:VarByLGFunc = Function({},{default local},";
			put "	If(IsEmpty(Here:H2dat),";
			put " 	Here:H2dat = open(""&ClientOutPath.&GH_H2OutDataName..sas7bdat"",invisible);";
			put " ,"; 
			put "		Current Data Table(Here:H2dat);";
			put "	);";	
			put "   Here:hlb_byg= HListBox(";
			put "		Here:olb_byg= OutLineBox("""",";
			put "			Here:VarGB_By= Graph Builder(";
			put "				Show Footer(0),";
			put "				Show Control Panel( 0 ),";
			put "				Variables(";
			put "					X( :Trait ),";
			put "					X( :&AnnoGroup, Position( 1 ) ),";
			put "					Y( :Estimate ),";
			put "					Color( :&AnnoGroup )";
			put "				),";
			put "				Elements( Bar( X( 1 ), X( 2 ), Y, Legend( 20 ) ) ),";
			put "				Where( :MarkerSet == ""By_Chromosome"" & :Source_Variance == ""SNPs""),";
			put "				SendToReport(";
			put "					Dispatch(";
			put "						{},";
			put "						""Graph Builder"",";
			put "						OutlineBox,";
			put "						{Set Title( ""Partitioned Genomic Variance"" )}";
			put "					),";
			put "					Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "					Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Partitioned Genomic Variance"" )} )";
			put "				)";
			put "				//Local Data Filter(";
			put "				//	Add Filter(";
			put "				//		columns( :Source_Variance ),";
			put "				//		Where( :Source_Variance == ""SNPs"" ),";
			put "				//		Display( :Source_Variance, Size( 202, 57 ), List Display )";
			put "				//	)";
			put "				//)";
			put "			),";
			put "		),";
			put "		try(Here:olb_byg[TextBox(1)]<<Delete;);";
			put "	);";
			put "	Here:olb_allg << Append(Here:hlb_byg);";
			put ");";
		%end;
		put "InsertInto(DataList,{{""Here:H2dat""}});";
		put "InsertInto(TabButtonList, ""Genomic Variance"");";
		put "InsertInto(TabCodelist,expr(Here:VarAllLGFunc()));";
		%if &NumGps > 1 %then %do;
			put "InsertInto(TabAfterCodeList, expr(Here:VarByLGFunc()));";
		%end;
		%else %do;
			put "InsertInto(TabAfterCodeList, """");";
		%end;
		put "InsertInto(TabOpenList,1);";
		%if %upcase(&TraitType) = CONTINUOUS %then %do;
			put "Here:H2AllLGFunc = Function({},{default local},";
			put "	If(IsEmpty(Here:H2dat),";
			put " 	Here:H2dat = open(""&ClientOutPath.&GH_H2OutDataName..sas7bdat"",invisible);";
			put " ,"; 
			put "		Current Data Table(Here:H2dat);";
			put "	);";
			put "   Here:vlb_allh2= VListBox(";
			put "	Here:olb_allh2= OutLineBox(""Genomic Heritability"",";
			put "	Here:H2GB_All= Graph Builder(";
			put "			Show Footer( 0 ),";
			put "			Show Control Panel( 0 ),";
			put "			Variables( X( :Trait ), Y( :Heritability ), Color( :Trait ) ),";
			put "			Elements( Bar( X, Y, Legend( 11 ) ) ),";
			put "			Where( :MarkerSet == ""All_Chromosomes"" & :Source_Variance == ""SNPs""),";
			put "			SendToReport(";
			put "				Dispatch(";
			put "					{},";
			put "					""Graph Builder"",";
			put "					OutlineBox,";
			put "					{Set Title( ""Total Genomic Heritability"" )}";
			put "				),";
			put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "				Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Total Genomic Heritability"" )} )";
			put "			)";
			put "			//Local Data Filter(";
			put "			//	Add Filter(";
			put "			//		columns( :MarkerSet ),";
			put "			//		Where( :MarkerSet == ""All_Chromosomes"" ),";
			put "			//		Display( :MarkerSet, Size( 202, 38 ), List Display )";
			put "			//	)";
			put "			//)";
			put "		),";
			put "		),";
			put "		try(Here:olb_allh2[TextBox(1)]<<Delete;);";
			put "	),";
			put ");";
			%if &NumGps > 1 %then %do;
				put "Here:H2ByLGFunc = Function({},{default local},";
				put "	If(IsEmpty(Here:H2dat),";
				put " 	Here:H2dat = open(""&ClientOutPath.&GH_H2OutDataName..sas7bdat"",invisible);";
				put " ,"; 
				put "		Current Data Table(Here:H2dat);";
				put "	);";	
				put "   Here:hlb_byh2= HListBox(";
				put "		Here:olb_byh2= OutLineBox("""",";
				put "			Here:H2GB_By= Graph Builder(";
				put "				Show Footer(0),";
				put "				Show Control Panel( 0 ),";
				put "				Variables(";
				put "					X( :Trait ),";
				put "					X( :&AnnoGroup, Position( 1 ) ),";
				put "					Y( :Heritability ),";
				put "					Color( :&AnnoGroup )";
				put "				),";
				put "				Elements( Bar( X( 1 ), X( 2 ), Y, Legend( 20 ) ) ),";
				put "				Where( :MarkerSet == ""By_Chromosome"" & :Source_Variance == ""SNPs""),";
				put "				SendToReport(";
				put "					Dispatch(";
				put "						{},";
				put "						""Graph Builder"",";
				put "						OutlineBox,";
				put "						{Set Title( ""Partitioned Genomic Heritability"" )}";
				put "					),";
				put "					Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
				put "					Dispatch( {}, ""Y title"", TextEditBox, {Set Text( ""Partitioned Genomic Heritability"" )} )";
				put "				)";
				put "				//Local Data Filter(";
				put "				//	Add Filter(";
				put "				//		columns( :Source_Variance ),";
				put "				//		Where( :Source_Variance == ""SNPs"" ),";
				put "				//		Display( :Source_Variance, Size( 202, 57 ), List Display )";
				put "				//	)";
				put "				//)";
				put "			),";
				put "		),";
				put "		try(Here:olb_byh2[TextBox(1)]<<Delete;);";
				put "	);";
				put "	Here:olb_allh2 << Append(Here:hlb_byh2);";
				put ");";
			%end;
			put "InsertInto(DataList,{{""Here:H2dat""}});";
			put "InsertInto(TabButtonList, ""Genomic Heritability"");";
			put "InsertInto(TabCodelist,expr(Here:H2AllLGFunc()));";
			%if &NumGps > 1 %then %do;
				put "InsertInto(TabAfterCodeList, expr(Here:H2ByLGFunc()));";
			%end;
			%else %do;
				put "InsertInto(TabAfterCodeList, """");";
			%end;
			put "InsertInto(TabOpenList,1);";
		%end;
	run;

	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &GH_ScriptName, "&GH_H2OutDataName..sas7bdat", text/plain, yes);
	%PackageInsertDataset(Parameter Estimates data from &ProcessName, outlib, &GH_PEOutDataName, _type=wide);
	%PackageInsertDataset(Fit Statistics data from &ProcessName, outlib, &GH_FSOutDataName, _type=wide);
	%if %upcase(&OutAddMatrix)= YES %then %do;
		%PackageInsertDataset(Additive Relationship Matrix data from &ProcessName, outlib, &GH_ADOutDataName, _type=wide);
	%end;
	%if %upcase(&OutRootAddMatrix)= YES %then %do;
		%PackageInsertDataset(Rooted Additive Relationship Matrix data from &ProcessName, outlib, &GH_RAOutDataName, _type=wide);	
	%end;
	
	%if &FlagHeritabilityError or &Flag_EverNonConvergence %then %do;
		%global jsl_after_tabbedreport;
		%let jsl_after_tabbedreport = &clientoutpath.&ProcessName Message for &OutName..jsl;	
	
		%global GH_ScriptErr;
		%PrepOutFile(GH_ScriptErr,GH_ScriptErrName,&ProcessName Message for &OutName..jsl);	
	%end;	
	
	%if &FlagHeritabilityError %then %do;
		* put JSL statements;
		data _null_;
			file "&GH_ScriptErr";
			put '//!';
			put "Names Default to Here(1);";
			put 'wait(0.5);';
				put "Here:_nw00_= New Window(""&ProcessName Warning Message"",showmenu(0),showtoolbars(0),";
				put " TextBox(""Some heritability estimates were greater than the unit. ""|| ";
				put "			""Check the Convergency_Message and Convergency_Status columns in the data ""|| ";
				put "			""table associated with Genomic Heritability tab for more details. \!N\!N"",";
				put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
				put " ),";
				put " );";
			put "if(!IsEmpty(Here:TabWinList),";
			put "    InsertInto(Here:TabWinList,0);";
			put "    Here:TabWinList[Here:nwcounter]=Here:_nw00_;";
			put "    Here:nwcounter++; ";		
			put "	);";
		run;	
	%end;
	
	%if &Flag_EverNonConvergence %then %do;	
		data _null_;
			%if &FlagHeritabilityError %then %do;
				file "&GH_ScriptErr" mod lrecl=32767;		
			%end;
			%else %do;
				file "&GH_ScriptErr" lrecl=32767;	
				put '//!';
			%end;
			put "Names Default to Here(1);";
			put 'wait(0.5);';
			put "Here:_nw01_= New Window(""&ProcessName Warning Message"",showmenu(0),showtoolbars(0),";
			put " TextBox(""   PROC &WhichProc did not converge. ""|| ";
			put "			""Check the Convergency_Message and Convergency_Status columns in the data ""|| ";
			put "			""table associated with Genomic Heritability tab for more details. \!N\!N"",";
			put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400)";
			put " ),";
			put " );";		
			put "if(!IsEmpty(Here:TabWinList),";
			put "    InsertInto(Here:TabWinList,0);";
			put "    Here:TabWinList[Here:nwcounter]=_nw01_;";
			put "    Here:nwcounter++; ";		
			put "	);";
		run;
	%end;
	
	%if &FlagHeritabilityError or &Flag_EverNonConvergence %then %do;
		%PackageInsertFile(jsl code from &ProcessName, &OutPath, &GH_ScriptErr, , text/plain, yes, no);	
	%end;

	%if %symexist(_dsimk_) %then %do;
		%let _rc_ = %sysfunc(close(&_dsimk_));
	%end;		

	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
	
	%exitcheck:;
%mend;

* run the macro;
%GenomicHeritability;
