/*
JMP Genomics Analytical Process

Name: Haplotype Q-K Mixed Model
Author: Wendy Czika (implemented haplotype trend regression)
	Luciano Silva (implemented haplotype Q-K mixed model)

Description:
Performs "haplotype Q-K mixed model" using output 
data set from PROC HAPLOTYPE containing 
individuals' probabilities for haplotype pairs to 
test for association with a quantitative trait
using PROC MIXED, a binary trait with PROC LOGISTIC or PROC GLIMMIX, 
or an ordinal trait using PROC GLIMMIX. See the HAPLOTYPE procedure 
in the SAS/Genetics User's Guide and the REG 
procedure in the SAS/STAT User's Guide for more
information.
*/

/*
-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData TraitVars IDVars IndIDVar ByVars WhereStmt 
CombineBelow HapFreqData FreqVar DropBelow /*RegMethod*/ 
/*CensorVar CensorValues*/ Covar ClassVar Perms SeedPerm
/*EventTrait*/ OutName OutPath ServerOutPath MacroPath 
HaplotypeQKMixedModel_OutData ModelOut /*SaveEach BuildCombined*/ 
HaplotypeQKMixedModel_Frame HaplotypeQKMixedModel_Body 
HaplotypeQKMixedModel_Contents HaplotypeQKMixedModel_Script

QKInData IndIDMerg QVars ListQVars QSumto1 PedID KVars ListKVars
CompressedK FixCovParms TraitType QKClassVars FixedEff
RandEff RandOpts ParmsStmt DDFM
UseLowerB GMethod MultiMarkerModel Priors SelectMethod Adaptive
StopCriterion SLEntry SLStay CVPMethod CVKFold Seed
MaxStep WeightVar LockIn FitSelModelOnUnrotatedData SLStaySelModel
SelModelProfile Conv MTC PValAdj Tau Bandwidth Alpha
OnlySig OutputResid OutputPred OutputFitStat

AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoWindow AnnoLoc AnnoWhereStmt
;

*ProcessBody;

%macro HapQKMixedModel;

	%let ByVars=; /*set to empty because it is not used in this process*/
	%let Covar=;/*set to empty because it is not used in this process*/
	%let ClassVar=; /*set to empty because it is not used in this process*/

	%let LockIn = 0;
	%let Priors = Proportional;
	%let WeightVar=;
	%let StrataVars= ;
	%let Odds= No;
	%let pe= 0;
	%let covp= 1;
	%let OutputRSquare= No;
	%let EstMethod= No;
	%let pest= 1;
	
	%let ie=0;
	%let IntEffBuild= ;
	%let IntEff= ;
	
	/*S1199692: clear title to avoid showing titles from other processes*/
	title;

	*initialization and argument checking;
	%global ProcessName;
	%let ProcessName = HaplotypeQKMixedModel; 

	%global FullAPName;
	%let FullAPName=%str(Haplotype Q-K Mixed Model);

	%include "&MacroPath./UtilityMacros.sas" / nosource;
	%include "&MacroPath./GenMacros.sas" / nosource; /*Haplotype QK*/

	%GetOrigFileName(InData,OrigInData);

	%PathName(&InData);
	%let InData = &TmpName;
	libname InLib "&TmpPath";
	%if &exiterror=1 %then %goto exit;

	%let HFData=;
	%if %length(&HapFreqData) %then %do;
		%PathName(&HapFreqData);
		%if &exiterror=1 %then %goto exit;
		libname HFLib "&TmpPath";
		%let HFData = HFLib.&TmpName;

		data FreqReqVars; 
			set &HFData;
			where 0;
			keep WindowID Haplotype &FreqVar;
		run;

		%let _tid_ = %sysfunc(open(FreqReqVars));
		%let _nFreqReqVars_ = %sysfunc(attrn(&_tid_,NVARS));
		%let rc = %sysfunc(close(&_tid_));

		%if &_nFreqReqVars_ < 3 %then %do;
			%put ERROR: The Haplotype Frequency data set does not have the required variables: WindowID, &FreqVar, and Haplotype.;
			%let exiterror = 1;
			%goto exit;
		%end;

		%if %length(&WhereStmt) %then %do;
			%let hfds= hfdat_sub;
			data &hfds;
				set &HFData;
				haplotype=tranwrd(haplotype,'-','_');
				where &WhereStmt; 
			run;
			
			%Nobsvars(&hfds);
			%if &nobs=0 %then %do;
				%put ERROR: There are no observations in the Haplotype Frequency Data Set to be analyzed.  Check that you have entered a valid Where Clause.;
				%let exiterror = 1;
				%goto exit;
			%end;
		%end;
		%else %do;
			%let hfds= &HFData;
		%end;
	%end;

	* check name for OutName;
	%CheckDataName(_name=OutName,_default=&OrigInData);
	%if %length(&OutName)>26 %then %let OutName=%substr(&OutName,1,26);

	%if %length(&ByVars) %then %do;
	   proc sort data=InLib.&InData out=a;
		by &ByVars;
	   run;
	   %let ds = a;
	%end;
	%else %let ds = InLib.&InData;
	
	%if %length(&WhereStmt) %then %do;
		data ds_sub;
			set &ds;
			where &WhereStmt;
		run;
	
		%Nobsvars(ds_sub);
		%if &nobs=0 %then %do;
			%put ERROR: There are no observations in the data set to be analyzed.  Check that you have entered a valid Where Clause.;
			%goto exit;
		%end;
		%let ds= ds_sub;
	%end;
		
	%global Nbyval NByVars ByVarsOnly Jby OrigNByVal NonTraitByVars JNonTraitByVars lastby;
	%if %length(&ByVars) %then %do;
		%let temp=%sysfunc(tranwrd(&ByVars,DESCENDING, ));
		%let ByVarsOnly=%sysfunc(compbl(&temp)); 
		%let JBy=%sysfunc(translate(&ByVarsOnly,","," "));
		%let i=1;
		%do %while (%length(%scan(&ByVarsOnly, &i))>0);
			%let i=%eval(&i+1);
		%end;
		%let NByVars = %eval(&i-1);
		data countby;
			set &ds;
			by &ByVars;
			%let lastby=%scan(&ByVarsOnly, &NByVars);
			if first.&lastby;
		run;
		%let dsid=%sysfunc(open(countby));
		%let Nbyval=%sysfunc(attrn(&dsid,NOBS));
		%let ret=%sysfunc(close(&dsid));	
		%if &Nbyval=1 and &ProcessName ne %str(ImputedSNPTraitAssociation)%then %do;
			%let ByVars=;
			%let ByVarsOnly=;
			%let JBy=;
		%end;
	%end;
	%else %do;
	 %let Nbyval=1;
	 %let ByVarsOnly=;
	 %let NByVars = 0;
	%end;

	%let OrigNByVal = &NByVal;
	%let NonTraitByVars = &ByVars;

	%let b=1;
	%do %while(%length(%scan(&NonTraitByVars,&b)));
		%let JNonTraitByVars = &JNonTraitByVars :%scan(&NonTraitByVars,&b),;
		%let b=%eval(&b+1);
	%end;	

	data HapReqVars; 
		set &ds;
		where 0;
		keep WindowID IndivID Haplotype1 Haplotype2 Prob;
	run;

	%let _tid_ = %sysfunc(open(HapReqVars));
	%let _nHapReqVars_ = %sysfunc(attrn(&_tid_,NVARS));
	%let rc = %sysfunc(close(&_tid_));

	%if &_nHapReqVars_ < 5 %then %do;
		%put ERROR: The Phased Haplotype data set does not have the required variables: WindowID, IndivID, Haplotype1, Haplotype2, Prob.;
		%let exiterror = 1;
		%goto exit;
	%end;

	%if &ModelOut = HTML %then %do;
		%PrepHTML(HaplotypeQKMixedModel_body, HaplotypeQKMixedModel_frame, HaplotypeQKMixedModel_contents,
			&OutName._htr_body.html,&OutName._htr_frame.html,&OutName._htr_contents.html);

		* setup for html output;
		ods html 
			body = "&HaplotypeQKMixedModel_Body" (url="&bodyfile")
			frame = "&HaplotypeQKMixedModel_Frame"  (url="&framefile")
			contents = "&HaplotypeQKMixedModel_Contents" (url="&contentsfile")
		;
		ods listing close;
		%let OutputType = HTML;
		%let Listing= Yes;
	%end;
	%else %if &ModelOut = Text %then %do;
		%let OutputType = Listing;
		%let Listing= Yes;
	%end;
	%else %do;
		%let OutputType = None;
		ods exclude all;
		ods noresults;
		%let Listing= No;
	%end;

	data out1;
		set &ds;
		Haplotype=tranwrd(haplotype1,'-','_');
		drop haplotype1;
		/*
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
		*/
	run;

	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exit;
	%end;

	data out2;
		set &ds;
		haplotype=tranwrd(haplotype2,'-','_');
		drop haplotype2;
		/*
		%if %length(&WhereStmt) %then %do;
			where &WhereStmt;
		%end;
		*/
	run;

	data outnew;
		set out1 out2;
	run;
	%if &syserr>4 %then %goto exit;
	
	%if %length(&QKInData) %then %do;

		%PathName(&QKInData);
		%if &exiterror=1 %then %goto exit;
		libname QKLib "&TmpPath";
		%let QKData = QKLib.&TmpName;	

		%if %upcase(&FixCovParms)=YES and (%upcase(&DDFM)=SATTERTHWAITE or %upcase(&DDFM)=KENWARDROGER) %then %do;
			%put ERROR: You cannot select SATTERTHWAITE or KENWARDROGER for the Denominator Degrees of Freedom Method when the Fix Covariance Parameters checkbox is checked.;
			%let exiterror=1;
			%goto exit;
		%end;

		%if %length(&ListQVars) %then %do;
			data qvars;
				set &QKData(keep=&ListQVars obs=1);
			run;

			%let dsid=%sysfunc(open(qvars,i));
			%do i=1 %to %sysfunc(attrn(&dsid,nvars));
				%let QVars=&QVars %sysfunc(varname(&dsid,&i));
			%end;
			%let rc = %sysfunc(close(&dsid));
		%end;	

		%if %upcase(&QSumto1)=YES %then %do;
			/* Remove last Q var since sum to 1 */
			%let OrigQVars = &QVars;
			%let QVars = ;
			%let w=1;
			%do %while(%length(%scan(&OrigQVars,%eval(&w+1))));
				%let QVars = &QVars %scan(&OrigQVars,&w);
				%let w=%eval(&w+1);
			%end;
			%if %length(&QVars)=0 %then %do;
				%let QVars = &OrigQVars;
				%put WARNING: There is only one Q Matrix variable selected so the Columns of Q Matrix Sum to 1 checkbox is ignored.;
			%end;
		%end;

		%let FixedEff = &QVars &FixedEff; /*ListQVars has already been included into QVars*/

		%if %length(&ListKVars) %then %do;
			data Kvars;
				set &QKData(keep=&ListKVars obs=1);
			run;

			%let dsid=%sysfunc(open(kvars,i));
			%do i=1 %to %sysfunc(attrn(&dsid,nvars));
				%let KVars=&KVars %sysfunc(varname(&dsid,&i));
			%end;
			%let rc = %sysfunc(close(&dsid));
		%end;
		
		%let qkds= &QKData;
	%end;

	%let kmodel= 0;
	%if %length(&KVars) %then %let kmodel= 1;

	%if &Perms>0 %then %do;
		%if %upcase(&TraitType)=BINARY %then %do;
			/* If trait var is character & binary, convert it to 0/1 */

			%if %length(&QKInData)=0 %then %do;
				%let ds_tmp= outnew;
			%end;
			%else %do;
				data qkdat;
					set &qkds;
				run;
				
				%let qkds= qkdat;
				%let ds_tmp= &qkds;
			%end;
			
			%let TraitVarsNum=;
					
			%let ti=1;
			%do %while(%length(%scan(&TraitVars,&ti))>0);			
				%let TraitVar = %scan(&TraitVars,&ti);
					
				%let dsid=%sysfunc(open(&ds_tmp,i));
				%let trnum=%sysfunc(varnum(&dsid,&TraitVar));
				%let tlen=%sysfunc(varlen(&dsid,&trnum));
				%let ttype=%sysfunc(vartype(&dsid,&trnum));
				%let rc=%sysfunc(close(&dsid));

				%if &ttype=C %then %do;
					%CheckBinaryChar(&ds_tmp,&TraitVar,_len=&tlen);

					%if &nvalue=2 %then %do;
						%let tnum=%substr(%trim(&TraitVar),1,28)_num;
						%put Converting &TraitVar to numeric 0/1 variable &tnum.;
						data &ds_tmp;
							set &ds_tmp;
							if &TraitVar="&cval1" then &tnum=0;
							else if &TraitVar="&cval2" then &tnum=1;
							else &tnum=.;
						run;
						
						%let TraitVarsNum= &TraitVarsNum &tnum;
					%end;
					%else %do;
						%put ERROR: Trait Variable &TraitVar selected is not binary.;
						%goto exit;
					%end;
				%end;
				%else %do;
					%let TraitVarsNum= &TraitVarsNum &TraitVar;
				%end;
				%let ti=%eval(&ti+1);
			%end;
			%let TraitVars= &TraitVarsNum;
		%end;
	%end;/*perm*/

	proc sort data=outnew;
		by WindowID haplotype;
	run;

	proc sort data=outnew out=winvals nodupkey;
		by WindowID;
	run;

	data _null_;
		set winvals end=last;
		call symput('winval'||strip(_n_),strip(WindowID));
		if last then call symput('Nwinvals',strip(_n_));
		call symput("wintype",vtype(WindowID));
	run;

	%*let IDVars = IndivID &IDVars;
	%let IDVars = &IndIDVar &IDVars;

	* some windows may not contain the full data set, so assemble it here;
	proc sort data=outnew nodupkey
		out=fulldat(keep=&ByVars &IDVars %if %length(&QKInData)=0 %then &TraitVars; &Covar &ClassVar);
		by &ByVars &IDVars;
	run;
	%NObsVars(fulldat);
	%let nobsfull = &nobs;

	/*data OutLib.outnew; set outnew; run;*/

	/* Just so don't have to add WindowID everywhere */
	%let ByVars0 = &ByVars;
	%let ByVars = WindowID &ByVars;
	
	%global SNPTraitAssoc_HMG SNPTraitAssoc_HMS;
	%PrepOutDS(SNPTraitAssoc_HMG,SNPTraitAssoc_HMGName,&OutName._hmg);
	%PrepOutDS(SNPTraitAssoc_HMS,SNPTraitAssoc_HMSName,&OutName._hms);

	data OutLib.&SNPTraitAssoc_HMGName;
		delete;
	run;

	data OutLib.&SNPTraitAssoc_HMSName;
		delete;
	run;

	*begin: from qkmixedmodel.sas*;
	%global SNPTraitAssoc_HGPE SNPTraitAssoc_HGCP SNPTraitAssoc_HGOR SNPTraitAssoc_HGOP
		SNPTraitAssoc_HSPE;
	%let SNPTraitAssoc_HGPE=;
	%let SNPTraitAssoc_HGCP=;
	%let SNPTraitAssoc_HGOR=;
	%let SNPTraitAssoc_HGOP=;
	%let SNPTraitAssoc_HSPE=;

	%PrepOutDS(SNPTraitAssoc_HGPE,SNPTraitAssoc_HGPEName,&OutName._HGPE);
	%PrepOutDS(SNPTraitAssoc_HGCP,SNPTraitAssoc_HGCPName,&OutName._HGCP);
	%if %upcase(&OutputResid)=YES %then %PrepOutDS(SNPTraitAssoc_HGOR,SNPTraitAssoc_HGORName,&OutName._HGOR);
	%if %upcase(&OutputPred)=YES %then %PrepOutDS(SNPTraitAssoc_HGOP,SNPTraitAssoc_HGOPName,&OutName._HGOP);
	%PrepOutDS(SNPTraitAssoc_HSPE,SNPTraitAssoc_HSPEName,&OutName._HSPE);

	/* Table name from all but MIXED */
	%global ParmEst EffName;
	%let ParmEst=ParameterEstimates;
	%let EffName=Effect;

	%let WhichProc=MIXED;
	%let TraitOpt= ;
	%let Stat=F; 
	%let Gtable=Tests3;
	%let CStmt=contrast;
	%let Ctable=Contrasts;
		%let StatC = FValue;
		%let ProbC = ProbF;
		%let StatCLabel = F-Statistic;
		%let ProbCLabel = Prob > F;
		%let ProbC_empLabel = Empirical Prob > F;
		%let ConVarC=Label;			
	%let usestrata=0;
	%let ModOption=%quote(/ htype=3 ddfm=&DDFM s); 
	%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption residual outp=outres);
	%let ClassOpt=;
	%*let teststmt=%str('genotypes' Genotype 1);
	%let ProcOption= ;
	%let ParmEst = SolutionF;
	%let EffName = Effect;
	%if %length(&PedID) %then %let QKClassVars = &QKClassVars &PedID;

	/* Need a separate check for OR when using GLIMMIX since not creating OR data set */
	%let AnyOdds=0;
	%if %upcase(&TraitType)=COUNT %then %do;
		%let WhichProc=GLIMMIX; 
		%let ModOption=%quote(/ link=log dist=p htype=3 ddfm=&DDFM s);
		%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid predicted=Pred);
		%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
		%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=Pred);
		%let ParmEst = ParameterEstimates;
	%end;
	%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
		%if %length(&RandEff)>0 or %length(&KVars)>0 %then %do;
			%let WhichProc=GLIMMIX;
			%let ProcOption=%str(Method=&GMethod);		
			%if %upcase(&TraitType)=NOMINAL %then %let TraitOpt=(Ref=FIRST);
			%else %let TraitOpt=(Event=LAST);		
			%if %upcase(&EstMethod)=YES %then %let ProcOption=%quote(&ProcOption EMPIRICAL);
			%let Stat=F;     
			%let Gtable=Tests3;
			%let CStmt=contrast;
			%let Ctable=Contrasts;
				%let StatC = FValue;
				%let ProbC = ProbF;
				%let StatCLabel = F-Statistic;
				%let ProbCLabel = Prob > F;
				%let ProbC_empLabel = Empirical Prob > F;
				%let ConVarC=Label;
			%let EStmt=estimate;				
			%let Etable=Estimates;		
			%let ParmEst = ParameterEstimates;
			%if %upcase(&DDFM)=CONTAIN or %upcase(&DDFM)=BETWITHIN %then %let DDFM=RESIDUAL;
			%if %upcase(&TraitType)=BINARY %then %let ModOption=%quote(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
			%else %if %upcase(&TraitType)=NOMINAL %then %do;
				%*let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=betwithin s);
				%let ModOption=%quote(/ link=glogit dist=mult htype=3 ddfm=&DDFM s);
				%let QKClassVars = &QKClassVars &TraitVars;
			%end;
			%else %if %upcase(&TraitType)=ORDINAL %then %let ModOption=%quote(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
			%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid predicted=Pred);
			%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resid=Resid);
			%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=Pred);
			%*let teststmt=%str('genotypes' Genotype 1);		
			%if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %let lsmi=0;
		%end;
		%else %do;
			%let WhichProc=LOGISTIC;
			%let TraitOpt=(Event=LAST);		
			%let Stat=ChiSq;
			%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
			%let CStmt=contrast;
			%let Ctable=ContrastTest;
				%let StatC = WaldChisq;
				%let ProbC = ProbChiSq;
				%let StatCLabel = ChiSq-Statistic;
				%let ProbCLabel = Prob > ChiSq;
				%let ProbC_empLabel = Empirical Prob > ChiSq;
				%let ConVarC=Contrast;
			%let Otable=OddsRatiosWald;
			%let usestrata=1;
			%if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit);
			%else %if %length(StrataVars)>0 %then %let ModOption=/ GCONV=1E-6; 
			%else %let ModOption =;
			%if %upcase(&OutputResid)=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid predicted=pred);
			%else %if %upcase(&OutputResid)=YES and %upcase(&OutputPred)^=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres resdev=Resid);
			%else %if %upcase(&OutputResid)^=YES and %upcase(&OutputPred)=YES %then %let ModOption = %quote(&ModOption) %str(; output out=outres predicted=pred);
			%let ClassOpt = %str(/ cprefix=8);
			%*let teststmt=%str('genotypes' Genotype 1);
			%let ParmEst = ParameterEstimates;
			%let EffName = Variable;
			%let lsmi=0;
		%end;
	%end;

	%if (%upcase(&WhichProc)= MIXED or %upcase(&WhichProc)= GLIMMIX) %then %do;
		%if %upcase(&UseLowerB)=YES %then %do;
			%if %length(&ParmsStmt) %then %do;
				%if %index(&ParmsStmt,%str(/)) %then %do;
					%let ParmsStmt = %quote(&ParmsStmt lowerb=0);
				%end;
				%else %do;
					%let ParmsStmt = %quote(&ParmsStmt / lowerb=0);
				%end;
			%end;
			%else %let ParmsStmt = %str( / lowerb=0);
		%end;
	%end;
	%else %let ParmsStmt=;
	*end: from qkmixedmodel.sas*;

	*begin: from snptraitassocmacro.sas*;
	%let FixedEffVars = ;
	%let i = 1;
	%do %while(%length(%scan(&FixedEff,&i,'*| ')));
		%let FixedEffVars = &FixedEffVars %scan(&FixedEff,&i,'*| ');
		%let i=%eval(&i+1);
	%end;

	%let RandEffVars = ;
	%let i = 1;
	%do %while(%length(%scan(&RandEff,&i,'*| ')));
		%let RandEffVars = &RandEffVars %scan(&RandEff,&i,'*| ');
		%let i=%eval(&i+1);
	%end;

	%if &i>1 %then %do;
		/* If just one random effect variable, use alternative random statement */
		%if &i=2 and %length(&RandOpts)=0 and %length(&ParmsStmt)=0 and %index(&RandEffVars,%str(-))=0 %then %do;

			proc sort data=&qkds out=dssort;
				by &RandEffVars;
			run;

			%if %index(%upcase(&QKClassVars),%upcase(&RandEffVars))>0 %then %do;
				%let NumRandEff = %trim(%substr(&RandEffVars,1,28)_num);
				data numrandeff;
					set dssort;
					by &RandEffVars;
					if first.&RandEffVars then &NumRandEff+1;
				run;

				%let QKClassVars = %sysfunc(tranwrd(%upcase(&QKClassVars),%upcase(&RandEffVars),%str()));
				%let RandEffVars = &NumRandEff;
				%let qkds = numrandeff;
			%end;
			%else %let qkds=dssort;	
			%let RandEff = %str(intercept / subject=&RandEffVars);
		%end;
		%else %do;
			%let foundsub=0;
			%let RandEff = %str(&RandEff / &RandOpts);
			/* Need to see if there's a SUBJECT= variable that needs to be kept */
			%let subind=%index(&RandOpts,sub=)+4;
			%if &subind=4 %then %do;
				%let subind=%index(&RandOpts,subject=)+8;
				%if &subind>8 %then %let foundsub=1;
			%end;
			%else %let foundsub=1;
			%if &foundsub=1 %then %let RandEffVars = &RandEffVars %scan(%substr(&RandOpts,&subind),1);
		%end;
	%end;

	%if &syserr>4 %then %do;
		%let exiterror=1;
		%goto exit;
	%end;	

	%let AllQKDatVars=;
	%let ModelVars= ;
	%if %length(&QKInData) %then %do;
		%let AllQKDatVars= &IndIDMerg &TraitVars &QKClassVars &FixedEffVars &RandEffVars &KVars;
		%let ModelVars= &FixedEffVars &RandEffVars;
		
		data qkdat;
			set &qkds;			
			_ord_ = _n_;
			keep &AllQKDatVars _ord_;

		run;
		%let qkds= qkdat;

		/*data OutLib.qkds; set &qkds; run;*/

		%Nobsvars(&qkds);
		%if &kmodel=1 %then %do;
			%if (%length(&PedID)=0 and %upcase(&CompressedK)=NO) %then %do;
				data _null_;
					set &qkds(obs=1);
					array ks{*} &KVars;
					call symputx("NumKVars",dim(ks));
				run;

				%if &nobs ne &NumKVars %then %do;
					%put ERROR: The number of columns in the K Matrix must be the same as the number of observations in the Q and K Data Set.;
					%let exiterror=1;
					%goto exit;
				%end;
			%end;
		%end;
	%end;

	/*begin S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/
	%let ClassVarsTmp= &QKClassVars;
	%if %symexist(TraitVar) %then %do;
		%if %length(&TraitVar) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&TraitVar), %str()));
		%end;
	%end;

	%if %symexist(PedID) %then %do;
		%if %length(&PedID) %then %do;
			%let ClassVarsTmp= %sysfunc(tranwrd(%upcase(&ClassVarsTmp), %upcase(&PedID), %str()));
		%end;
	%end;
	
	%put ClassVarsTmp= &ClassVarsTmp;

	%let NeedConstant=0;
	%if %length(&ClassVarsTmp) = 0 %then %do;
		/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
		table for Trend test when no Class var is present*/
		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			%let NeedConstant=1;
		%end;
		%else 
			%let NeedConstant=0;
	%end;
	%else %if %length(&ClassVarsTmp) %then %do;

		/*Checking if QKClassVars have been included into any of the variables that goes into the model statement*/
		%let IntEffVars= ;
		%let FixRandEffVars= &FixedEffVars &RandEffVars &IntEffVars;
		%put FixRandEffVars= &FixRandEffVars;		

		%let j=1;
		%do %while(%length(%scan(&ClassVarsTmp, &j, %str( ))));
			%let j= %sysevalf(&j + 1);
			%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
		%end;
		%let ncin= %sysevalf(&j - 1);

		%macro CheckClassVars();
			%global ncout ClassVarsOut FoundAtLeastOne_flag;
			%let ncout= 0;
			%let ClassVarsOut= ;
			%let FoundAtLeastOne_flag= 0;

			%let FixRandEffVarsOrig= &FixRandEffVars;

			%if %length(&FixRandEffVars) = 0 %then %do;
				%let ClassVarsOut= &ClassVarsTmp;
				%let FoundAtLeastOne_flag= 0;
			%end;		
			%else %do;
				%let j=1;
				%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%do %while(%length(&currclass));
					%put currclass= &currclass;
					%let k=1;
					%let currvar= %scan(&FixRandEffVars, &k, %str( ));
					%if %length(&currvar) %then %do;
						%let CandidateClassVarOut= ;
						%let found_flag= 0;
						%do %while(%length(&currvar));
							%put &currclass &currvar;
							%if %upcase(&currclass) = %upcase(&currvar) %then %do;
								%*put YES;
								%let found_flag= 1;
								%let FoundAtLeastOne_flag= 1;
								%let FixRandEffVarsTmp=;
								%let l=1;
								%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%do %while(%length(&currvartmp));
									%if &l ^= &k %then %do;
										%let FixRandEffVarsTmp= &FixRandEffVarsTmp &currvartmp;
									%end;
									%let l= %sysevalf(&l + 1);
									%let currvartmp= %scan(&FixRandEffVars, &l, %str( ));
								%end;
								%let FixRandEffVars= &FixRandEffVarsTmp;
								%goto nextclassvar;
							%end;
							%else %do;
								%*put NO;
								%let CandidateClassVarOut= &currclass;
							%end;

							%let k= %sysevalf(&k + 1);
							%let currvar= %scan(&FixRandEffVars, &k, %str( ));
						%end;

						%if &found_flag = 0 %then %do;
							%let ClassVarsOut= &ClassVarsOut &CandidateClassVarOut;
						%end;
					%end;
					%else %do;
						%let ClassVarsOut= &ClassVarsOut &currclass;
					%end;

					%nextclassvar:;
					%let j= %sysevalf(&j + 1);
					%let currclass= %scan(&ClassVarsTmp, &j, %str( ));
				%end;
				%let FixRandEffVars= &FixRandEffVarsOrig;
				%put ClassVarsOut= &ClassVarsOut;
				%put FixRandEffVars= &FixRandEffVars;
			%end;

			%if %length(&ClassVarsOut) > 0 %then %do;
				%let ncout=1;
				%do %while(%length(%scan(&ClassVarsOut, &ncout, %str( ))));
					%let ncout= %sysevalf(&ncout + 1);
				%end;
				%let ncout= %sysevalf(&ncout - 1);
			%end;
		%mend CheckClassVars;
		%CheckClassVars();

		%put ncout= &ncout;
		%if &ncout %then %do;
			%if &ncout = 1 %then %do;
				%let ErrString0= Class Variable &ClassVarsOut has neither been specified as fixed, random nor interaction effect. Please, review the Model Variables Tab.;
			%end;
			%else %if &ncout > 1 %then %do;
				%let ErrString0= Class Variables &ClassVarsOut have neither been specified as fixed, random nor interaction effects. Please, review the Model Variables Tab.;
			%end;

			%if %length(&ProcessName) > 22 %then %do;
				%let ProcessNameShort= %substr(&ProcessName, 1, 22);
			%end;
			%else %do;
				%let ProcessNameShort= &ProcessName;
			%end;

			%let ScErr0= &ProcessNameShort._ScErr;
			%*put ScriptErr= &ScErr0;
			%global &ScErr0;
			%PrepOutFile(&ScErr0,&ScErr0.Name,saserror.jsl);
			%*put ScriptErr= &&&ScErr0;
			%*put ScriptErrName= &&&ScErr0.Name;

			data _null_;
				file "&&&ScErr0" lrecl=32767 ;
				put '//!' ;
				put "Names Default to Here(1);" ;
				put "new Window(""ERROR running &FullAPName.""," ;
				put "vlb=VListBox(" ; 
				put "textBox(""The following SAS error ocurred in process &FullAPName:""),";
				put "errtxt = TextEditBox(""\!N""||""";
				put "&ErrString0";
				put """||""\!N"",WhiteBoxStyle(0)),";
				put "errtxt << Font Color(3)," ; 
				put "errtxt << Set Wrap(1000)," ;
				/*put "TextBox(""Please refer to the SAS log displayed in the JMP log window for further details.\!N\!N""),";*/
				put ")" ;
				put ");";
			run;
			%*PackageInsertFile(jsl code from &ProcessName, &OutPath, &&&ScErr0.Name, no, no, yes);

			%put ERROR: &ErrString0;
			%let exiterror=1;
			%goto exitm;
		%end;

		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			/* Need to do this for Proc Logistic/Surveylogistic/PHREG so it will produce Type3 
			table for Trend test when no Class var is neither included as fixed nor random effect*/
			%let FixRandEffVars= &FixedEffVars &RandEffVars;
			%put FixRandEffVars0= &FixRandEffVars;
			%CheckClassVars();
			%put FoundAtLeastOne_flag= &FoundAtLeastOne_flag;
			%if &FoundAtLeastOne_flag = 0 %then %do;
				%let NeedConstant=1;
			%end;
			%else %do;
				%let NeedConstant=0;
			%end;
		%end;
	%end;
	%put NeedConstant= &NeedConstant;
	/*end S1206799: Luciano added this check to verify that when class variables are specified, they are specified at least
	in one of the effects model: fixed, random, or interaction.*/

	%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
		data outrest_o_all;
			length _Trait_ $32;
			delete;
		run;
	%end;

	data ta_o_all;
		length Effect $32;
		delete;
	run;

	%if %Upcase(&Odds)=YES  %then %do; 
		data or_o_all;
			delete;
		run;
	%end;

	data pat_o_all;
		length Effect $32;
		delete;
	run;

	data cpt_o_all;
		length Effect $32;
		delete;
	run;	
	
	data pat_s_all;
		length Effect $32;
		delete;
	run;
	*end: from snptraitassocmacro.sas*;
	
	%global global_flag single_flag;
	%let global_flag= 0;
	%let single_flag= 0;
	
	%let firstwind= 1;
	%do w=1 %to &Nwinvals;

		%put;
		%put Processing window &w of &Nwinvals....;
		
		%let flag_globalstatus= 0;

		data currwin;
			set outnew;
			%if &wintype=C %then %do;
				where WindowID="&&winval&w";
			%end;
			%else %do;
				where WindowID=&&winval&w;
			%end;
		run;

		%let Rare=;
		%if %length(&HFData) %then %do;

			data hf_;
				set &hfds;
				haplotype=tranwrd(haplotype,'-','_');
				%if &wintype=C %then %do;
					where WindowID="&&winval&w";
				%end;
				%else %do;
					where WindowID=&&winval&w;
				%end;
			run;

			proc sort data=hf_;
				by haplotype;
			run;

			/*data outlib.currwin; set currwin; run;*/
			/*data outlib.hf_; set hf_; run;*/

			data outnew_cut;
				merge currwin hf_(in=keep);
				by haplotype;
				if keep;
				if missing(IndivID) then delete;
			run;

			proc sort data=outnew_cut;
				by descending &FreqVar haplotype;
			run;

			/*data outlib.outnew_cut; set outnew_cut; run;*/

			data outnew2;
				set outnew_cut end=last;
				Orig_Haplotype= Haplotype;
				lagh=lag(haplotype);
				if &FreqVar>=&CombineBelow then do;
					if haplotype ne lagh then num+1;
						Hapname=compress("H_&w._"||num,' ');
					end;
				else do;
					Hapname="H_&w._Rare";
					call symput("Rare","Rare");
					Haplotype='Rare';
				end;
				if last then do;
					call symput('numhaps', strip(num));
				end;
			run;	

			/*data OutLib.outnew2; set outnew2; run;*/

			%if %length(&Rare) %then %do;
				%let numhaps=%eval(&numhaps+1);
			%end;

			%if &numhaps=0 %then %do;
				%put WARNING: There are no haplotypes with a frequency greater than the cutoff specified for window &&winval&w...;
				%goto nextwindow;
			%end;	
		%end;
		%else %do;
			data outnew2;
				set currwin end=last;
				lagh=lag(haplotype);
				if haplotype ne lagh then num+1;
				Hapname=compress("H_&w._"||num,' ');        
				if last then do;
					call symput('numhaps', strip(num));
				end;
			run;
		%end;

		%put numhaps = &numhaps;
		%if &numhaps=1 %then %do;
			%put WARNING: Haplotypes are constant for window &&winval&w...;
			%goto nextwindow;
		%end;	

		proc sort data=outnew2;
			by &ByVars &IDVars hapname;
		run;

		/*data outlib.outnew2; set outnew2; run;*/
		
		%if %length(&HFData) %then %do;
			%put HereHFDATA;
			
			proc sort data=outnew2 out=hapnames_ho(keep=Haplotype Hapname &FreqVar Orig_Haplotype) nodupkey;
				by hapname Orig_Haplotype;
			run;	
			
			data hapnames_ho;
				set hapnames_ho;
				by hapname Orig_Haplotype;
				if first.hapname then HapNameFreq=&FreqVar;	
				else if (index(hapname, 'Rare')) then HapNameFreq+&FreqVar;
				if last.hapname;
				drop Orig_Haplotype;
			run;
			
			proc sort data=hapnames_ho out=hapnames(keep=Haplotype Hapname &FreqVar HapNameFreq) nodupkey;
				by hapname;
			run;
		%end;
		%else %do;
			proc sort data=outnew2 out=hapnames(keep=Haplotype Hapname) nodupkey;
				by hapname;
			run;
		%end;
		
		/*data OutLib.hapnames; set hapnames; run;*/

		data outt;
			set outnew2;
			by &ByVars &IDVars hapname;
			if first.hapname then totprob=prob/2;
			else totprob+prob/2;
			if last.hapname;
		run;

		/*data outlib.outt0; set outt; run;*/

		options label;
		proc transpose data=outt out=outreg(drop=_NAME_) ;
			id hapname;
			idlabel haplotype;
			var totprob;
			by notsorted &ByVars &IDVars %if %length(&QKInData)=0 %then &TraitVars; &Covar &ClassVar;
		run;

		*make sure data set is complete;
		%NObsVars(outreg);
		%if &nobs < &nobsfull %then %do;
			data t1;
				set fulldat;
				%if &wintype=C %then %do;
					WindowID = "&&winval&w";
				%end;
				%else %do;
					WindowID = &&winval&w;
				%end;
			run;
			data outreg;
				merge t1 outreg;
				by &ByVars &IDVars;
			run;		
		%end;
		
		%if %length(&HFData) %then %do;
			proc sort data=hapnames out=hapnamesInitial;
				by descending &FreqVar hapname;
			run; 
		
			data _null_; 
				set hapnamesInitial end=last;
				call symputx("hapInitial"||compress(_n_),compress(hapname));
				if last then call symputx("nhapsInitial",compress(_n_));
			run;
		%end;
		
		data outreg;
			%if %length(&HFData) %then %do;
				retain 
				%do i=1 %to &nhapsInitial;
					&&hapInitial&i
				%end;
				;
			%end;
			set outreg;
			array h{&numhaps} h_:;
			do i=1 to &numhaps;
				if h{i}=. then h{i}=0; 
			end;
			keep &ByVars &IDVars %if %length(&QKInData)=0 %then &TraitVars; h_: &Covar &ClassVar;
		run;

		/*data outlib.outreg; set outreg; run;*/

		%if &DropBelow > 0 %then %do;
			data outreg1;
				set outreg;
				drop h_:;
			run;
			data outreg2;
				set outreg;
				keep h_:;
			run;
			proc transpose data=outreg2 out=outreg2t;
				var h_:;
			run;
			data outreg2t;
				set outreg2t;
				array col{*} col:;
				nnz = 0;
				do i=1 to dim(col);
					if col[i] > 0 then nnz = nnz+1;
				end;
				freq = nnz/&nobsfull;
				if freq < &DropBelow then delete;
			run;
			data _null_;
				set outreg2t end=last;
				call symputx("hap"||compress(_n_),compress(_name_));
				if last then call symputx("nhaps",compress(_n_));
			run;
			%if &nhaps < &numhaps %then %do;
				%let numhaps = &nhaps;
				%put numhaps after dropping hapolotypes with frequency below &DropBelow = &numhaps;

				%if &numhaps = 0 %then %do;
					%put WARNING: There are no haplotypes with a frequency greater than the cutoff specified for window &&winval&w...;
					%goto nextwindow;
				%end;
				%else %if &numhaps = 1 %then %do;
					%put WARNING: Haplotypes are constant for window &&winval&w...;
					%goto nextwindow;
				%end;	

				proc transpose data=outreg2t out=outreg2tt(drop=_name_);
					var col:;
				run;
				data outreg;
					merge outreg1 outreg2tt;
				run;
			%end;
		%end;
		%else %do;
			data outreg2;
				set outreg;
				keep h_:;
				if (0);
			run;
			%let tid = %sysfunc(open(outreg2));	
			%do i=1 %to &numhaps;
				%let hap&i = %sysfunc(varname(&tid,&i)); 
			%end;
			%let tid = %sysfunc(close(&tid));
		%end;
		
		/*begin new qk*/
		%if %length(&QKInData) %then %do;
		
			proc sql UNDO_POLICY= NONE;
				create table outreg_qk as
				select *
				from outreg as b right join &qkds as a   
				on a.&IndIDMerg = b.&IndIDVar
				order by _ord_;
			quit;

			/*data OutLib.outreg_qk&w; set outreg_qk; run;*/

			proc datasets noprint nolist;
				delete outreg;
				change outreg_qk= outreg;
			quit;
		%end;		
		/*end new qk*/
	
		%if &Perms>0 %then %do;
			%if %length(&SeedPerm)=0 %then %let SeedPerm=0;

			%put TraitVarsDrop= &TraitVars;
			%let DropStmt = %quote(missing(%scan(&TraitVars,1)));
			%let ti=2;
			%do %while(%length(%scan(&TraitVars,&ti))>0);
				%let DropStmt = %quote(&DropStmt and missing(%scan(&TraitVars,&ti)));
				%let ti=%eval(&ti+1);
			%end;	
			%put DropStmt= &DropStmt;
			
			/*data OutLib.poutreg; set outreg; run;*/

			/*
			data p(keep=constant1 &ByVars &TraitVars &Covar)
				%*if %length(&ClassVars) %then pclass(keep=&ClassVars); 
			*/
			data p(keep=constant1 &ByVars &TraitVars)
				%if %length(&KVars &QVars &ModelVars) %then pother(keep= &KVars &QVars &ModelVars); /*ClassVars are already included into ModelVars*/
				;
				set outreg;
				if &DropStmt then delete;	
				constant1=1;
			run;		

			data o;
				set outreg;
				if &DropStmt then delete;	
				*keep h_&w._1-h_&w._&numhaps;
				keep 
					%do i=1 %to &numhaps; 
						&&hap&i 
						/*S1147948: making sure that H_1_Rare is kept in the data set if it is present*/
					%end;
				; 
			run;

			options nonotes;
			data operms;
				set %do i=1 %to &Perms; o %end;;
			run;
			options notes;

			%if &OutputType ^= None %then %do;
				ods &OutputType exclude all;
			%end;
			
			/*data OutLib.p; set p; run;*/

			proc multtest data=p outsamp=perms_ds perm n=&perms seed=&SeedPerm;
				class constant1;
				*test mean(&TraitVars &Covar &ModelVars);
				test mean(&TraitVars);
				by &ByVars;
			run;

			proc sort data=perms_ds;
				by _sample_ &ByVars;
			run;
			
			/*data OutLib.perms_ds0; set perms_ds; run;*/

			/*
			%if %length(&ClassVars) %then %do;
				data OutLib.pclass; set pclass; run;
				data perms_ds;
					set perms_ds;
					set pclass point=_obs_;
					obs=_obs_;  
				run;
				
				data OutLib.perms_ds1; set perms_ds; run;
			%end;
			*/
			
			%if %length(&KVars &QVars &ModelVars) %then %do;
				
				/*data OutLib.pother; set pother; run;*/
				
				data perms_ds;
					set perms_ds;
					set pother point=_obs_;
					obs=_obs_;
				run;
				
				/*data OutLib.perms_ds2; set perms_ds; run;*/							
			%end;

			data allp;
				merge perms_ds operms;
			run;
			
			/*data OutLib.pall; set allp; run;*/
			
			%if &OutputType ^= None %then %do;
				ods &OutputType;
			%end;
			
		%end; /*perm*/

		%let nhaps = &numhaps;

		/*begin new qk*/
		/* Need to do this for Proc Logistic so it will produce Type3 table when no Class var */
		%if (%index(&WhichProc,%str(LOGISTIC))>0 or %upcase(&WhichPROC)=%str(PHREG)) %then %do;
			%if &NeedConstant %then %do;
				data outreg;
					retain constant 0;
					set outreg;
				run;
				/*%let NeedConstant=1;*/
				
				%if &Perms>0 %then %do;
					data allp;
						retain constant 0;
						set allp;
					run;
				%end;
			%end;
		%end;

		data OverallTest&&winval&w;
			delete;
		run;

		data SingleTests&&winval&w;
			delete;
		run;
		/*end new qk*/

		/* Loop over traits */
		%if &w=1 %then %do;
			%global NTrait;
			%let NTrait= 0;
		%end;
		
		%let ti=1;
		%do %while(%length(%scan(&TraitVars,&ti))>0);
			%let ParmsStmt&ti= &ParmsStmt;

			%let TraitVar = %scan(&TraitVars,&ti);

			%if %upcase(&TraitType)=BINARY %then %do;
				proc sort data=outreg out=isbin nodupkey;
					by &TraitVar;
					where missing(&TraitVar)=0; 
				run;

				%NObsVars(isbin);
				%if &nobs ^= 2 %then %do;
					%put ERROR: The trait variable &TraitVar is non-binary.  This variable will not be used.;
					%goto nexttrait;
				%end;
			%end;

			/*data OutLib.outreg&w; set outreg; run;*/

			/*begin new qk*/
			%if %symexist(FixCovParms) %then %do;
				%if &w=1 and &pe=0 and %upcase(&FixCovParms)=YES and (%length(&KVars) or %length(&RandEff)) %then %do;
					%if %upcase(&Listing)=YES %then %do;
						ods listing close;
					%end;	
					ods output CovParms = fixcp;
					proc &WhichProc data=outreg &ProcOption;
						class &QKClassVars &ClassOpt;
						model &TraitVar &TraitOpt = &FixedEff &ModOption;
						%if &kmodel=1 %then %do;
							random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
						%end;
						%if %length(&RandEff)>0 %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;
						
						%if %length(&ByVars) %then %do; 
							by &ByVars;
						%end;
					quit;					

					data _null_;
						set fixcp end=_l;
						call symput('varcomp'||strip(_n_),strip(Estimate));
						if _l then call symputx("numcp",_n_);
					run;

					%let ParmsStmt= ;
					%let hold= %str(hold=);
					%do cp=1 %to &numcp;
						%let ParmsStmt = &ParmsStmt &&varcomp&cp;
						%let hold = %quote(&hold &cp,);
					%end;
					%let ParmsStmt&ti= %quote(&ParmsStmt / &hold);
					%put ParmsStmt&ti= &&ParmsStmt&ti;

					/*data outlib.fixcp; set fixcp; run;*/
					
					%if %upcase(&Listing)=YES %then %do;
						ods listing;
					%end;	
				%end;
			%end;

			data ta; /* kemicl - added to ensure data is not repeated if PROC does not produce estimates */
				delete;
			run;
			
			data ct;
				delete;
			run;

			%if %Upcase(&Odds)=YES  %then %do; 
				data ot;
					delete;
				run;
			%end;

			proc datasets nolist;
				delete outres;
			quit;
			
			/*data OutLib.outreg; set outreg; run;*/

			/** Overall test **/
			title "Overall Test for Window= &&winval&w";

			options nonotes;
			/*kemicl - Added odds ratio options and Random effect ability with discrete trait. 
			 If Logistic Procedure - enable OddsRatio statement and corresponding ODS output table 
			 If GLIMMIX Procedure - exponentiate estimate from Contrast Statement and retain in ODS output table
			 */
			ods output &Gtable=ta
				 %if &pe=0 %then %do;
					&ParmEst= pat
					&CTable= ct
					%if &WhichProc ne SURVEYREG %then %do;
						nobs = tnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
					%end;
					%if (&kmodel=1 or %length(&RandEff)) and &covp %then CovParms=cp;
					%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 

					/*begin: Luciano*/
					%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
						rsquare= trsq;
					/*end: Luciano*/	
					%if %upcase(&OutputFitStat)=YES %then 
						FitStatistics= fst;
				%end;
			;
			
			proc &WhichProc data=outreg &ProcOption	%if %Upcase(&Odds)=YES and ^%length(&RandEff &KVars) %then alpha=&Alpha;;
				class %if &NeedConstant=1 %then constant; &QKClassVars &ClassOpt;
				model &TraitVar &TraitOpt = %if &NeedConstant=1 %then constant; &FixedEff %do i=1 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then &&hap&i; %end; &ModOption;
				%if &kmodel=1 %then %do;
					random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
				%end;

				%if %length(&RandEff) %then %do;
					random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
				%end;

				%if %length(&&ParmsStmt&ti) %then %do;
					parms &&ParmsStmt&ti;
				%end;					

				%if %length(&StrataVars)>0 %then %do;
					strata &StrataVars;
				%end;

				%if %Upcase(&Odds)=YES %then %do;
					%if ^%length(&RandEff &KVars) %then %do;
						%if ^%index(%upcase(&hap1), RARE) %then %do;
							oddsratio &hap1 %do i=2 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then , &&hap&i; %end;; 
						%end;
						%else %do;
							oddsratio &hap2 %do i=3 %to &nhaps;, &&hap&i %end;;
						%end;
					%end;
					%else %do;
						%if ^%index(%upcase(&hap1), RARE) %then %do;
							&EStmt 'Overall' &hap1 1 %do i=2 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then , &&hap&i 1; %end; / alpha=&Alpha exp cl; 
						%end;
						%else %do;
							&EStmt 'Overall' &hap2 1 %do i=3 %to &nhaps;, &&hap&i 1 %end; / alpha=&Alpha exp cl;
						%end;
					%end;
				%end;

				%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
					id &IndIDMerg;
				%end;

				%if ^%index(%upcase(&hap1), RARE) %then %do;
					contrast 'Overall' &hap1 1 %do i=2 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then , &&hap&i 1; %end;; 
				%end;
				%else %do;
					contrast 'Overall' &hap2 1 %do i=3 %to &nhaps;, &&hap&i 1 %end;;
				%end;

				%if %length(&ByVars) %then %do; 
					by &ByVars;
				%end;
			quit;
			options notes;
			%if &syserr>4 %then %goto exit;
			
			/* kemicl - added to catch error when all effects are not estimable */	
			/*data OutLib.ct_over&w; set ct; run;*/
			
			%NObsVars(ct);
			%if &nobs %then %do;
				data ct;
					set ct;
					if not missing(&ProbC);
				run;
				%NObsVars(ct);
			%end;
		
			%if &nobs=0 %then %do;
				%put WARNING: PROC &WhichProc generated no estimates for trait &TraitVar in window &&winval&w...  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
				%put WARNING: Overall test could not be performed for Trait &TraitVar in Window &&winval&w...;
				%let haperr=1;
			%end;	
			%else %do;
				%let haperr=0;
				data ta_o;
					*length Dependent $32;
					length Trait $32;
					set ta;
					Trait=propcase("&TraitVar");
					*Dependent=propcase("&TraitVar");
					*Stat=&Stat;
					*ProbGlobalHap=&Prob;
					*keep &ByVars Dependent Stat ProbGlobalHap;
					*label Stat="&StatLabel" ProbGlobalHap="&ProbLabel";
				run;	

				/*data OutLib.ta_0o&w; set ta_o; run;*/

				data ta;
					delete;
				run;

				data pat_o;
					*length Dependent $32;
					length Trait $32;
					set pat;
					Trait=propcase("&TraitVar");
					*Dependent=propcase("&TraitVar");
					*Stat=&Stat;
					*ProbGlobalHap=&Prob;
					*keep &ByVars Dependent Stat ProbGlobalHap;
					*label Stat="&StatLabel" ProbGlobalHap="&ProbLabel";
				run;	
				
				/*data OutLib.pat_0o&w; set pat_o; run;*/
				
				data ma_o;
					*length Dependent $32;
					length Trait $32;					
					set ct;
					Trait=propcase("&TraitVar");
					*Dependent=propcase("&TraitVar");
					/*Stat=&StatC;*/
					&StatC._Global=&StatC;
					/*ProbGlobalHap=&ProbC;*/
					&ProbC._Global=&ProbC;
					*keep &ByVars Dependent Stat ProbGlobalHap;
					drop &StatC &ProbC;
					/*label Stat="&StatCLabel" ProbGlobalHap="&ProbCLabel";*/
				run;	
			
				/*data OutLib.ma_o0_&w; set ma_o; run;*/
				
				data ct;
					delete;
				run;
				
				%if (&kmodel=1 or %length(&RandEff)) and &covp %then %do;

					data cp_o;
						*length Dependent $32;
						length Trait $32;
						set cp;
						Trait=propcase("&TraitVar");
						*Dependent=propcase("&TraitVar");
						*Stat=&Stat;
						*ProbGlobalHap=&Prob;
						*keep &ByVars Dependent Stat ProbGlobalHap;
						*label Stat="&StatLabel" ProbGlobalHap="&ProbLabel";
					run;	

					/*data OutLib.cp_o&w; set cp; run;*/

					data cp;
						delete;
					run;
				%end;

				%if %upcase(&TraitType)=BINARY %then %do;

					proc transpose data=rp out=rp(drop=_name_) prefix=NumTrait;
						by &ByVars;
						var count;
						id %if %index(%upcase(&WhichProc),LOGISTIC) %then Outcome; %else &TraitVar;;
					run;

					%let datatomerge=rp;
				%end;
				%else %let datatomerge=;	

				%if &WhichProc ne SURVEYREG %then %do;
					data ta_o;
						merge tnobs(keep=&ByVars nobsused) &datatomerge ta_o;
						by &ByVars;
						rename nobsused=SampleSize;
					run;
					
					data pat_o;
						merge tnobs(keep=&ByVars nobsused) &datatomerge pat_o;
						by &ByVars;
						rename nobsused=SampleSize;
					run;
				%end;

				/*begin: including RSquare for logistic regression. Luciano*/
				%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then %do;
					data ta_o;
						merge ta_o trsq(keep= nValue1 nValue2 &ByVars);
						by &ByVars;
						rename nValue1= RSquare_Global nValue2=Rescaled_RSquare_Global;
					run;
				%end;
				/*end: Luciano*/

				/*data OutLib.ta_1o&w; set ta_o; run;*/

				%if %upcase(&OutputFitStat)=YES %then %do;
					%put WhichProc= &WhichProc;
					/*data OutLib.fst0; set fst; run;*/
					/*data OutLib.ta; set ta; run;*/

					proc transpose data=fst(
						%if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
							rename=(
								criterion= descr 
								interceptandcovariates= value
							)
						%end;
						%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
							rename=(
								criterion= descr  
								withcovariates= value
							)
						%end;
						)
						out=fst(drop= _name_); 
						var value;
						id descr;
						by &ByVars;
					run;

					/*data OutLib.fst1; set fst; run;*/

					data ta_o;
						merge ta_o fst;
						by &ByVars;
						rename
						%if %index(%upcase(&WhichProc),MIXED) %then %do;
							_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Global
							AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Global
							AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Global
							BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Global
						%end;
						%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
							%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
								or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
									_2_Log_Likelihood= _2_Log_Likelihood_Global
									AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
									AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
									BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
									CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
									HQIC__smaller_is_better_= HQIC_smaller_is_better_Global											
								%end;
							%end;
							%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 		
								_2_Log_Likelihood= _2_Log_Likelihood_Global
								AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
								AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
								BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
								CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
								HQIC__smaller_is_better_= HQIC_smaller_is_better_Global
								Pearson_Chi_Square= Pearson_ChiSquare_Global
								Pearson_Chi_Square___DF= Pearson_ChiSq_DF_Ratio_Global											
							%end;
							%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
								%end;
								%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
									_2_Log_Likelihood= _2_Log_Likelihood_Global
									AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
									AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
									BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
									CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
									HQIC__smaller_is_better_= HQIC_smaller_is_better_Global
								%end;	
							%end;
							%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global											
								%end;
							%end;										
						%end;
						%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
							_2_LOG_L= _2_LOG_L_Global
							AIC= AIC_Global
							SBC= SBC_Global
						%end;
						%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
							AIC= AIC_Global
							SC= SC_Global
							_2_Log_L= _2_Log_L_Global
						%end;
						;
					run;

					/*data OutLib.ta_2o&w; set ta_o; run;*/
					
					data ma_o;
						merge ma_o fst;
						by &ByVars;
						rename
						%if %index(%upcase(&WhichProc),MIXED) %then %do;
							_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Global
							AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Global
							AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Global
							BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Global
						%end;
						%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
							%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
								or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
									_2_Log_Likelihood= _2_Log_Likelihood_Global
									AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
									AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
									BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
									CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
									HQIC__smaller_is_better_= HQIC_smaller_is_better_Global											
								%end;
							%end;
							%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 		
								_2_Log_Likelihood= _2_Log_Likelihood_Global
								AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
								AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
								BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
								CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
								HQIC__smaller_is_better_= HQIC_smaller_is_better_Global
								Pearson_Chi_Square= Pearson_ChiSquare_Global
								Pearson_Chi_Square___DF= Pearson_ChiSq_DF_Ratio_Global											
							%end;
							%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
								%end;
								%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
									_2_Log_Likelihood= _2_Log_Likelihood_Global
									AIC___smaller_is_better_= AIC_Smaller_is_Better_Global
									AICC__smaller_is_better_= AICC_Smaller_is_Better_Global
									BIC___smaller_is_better_= BIC_Smaller_is_Better_Global
									CAIC__smaller_is_better_= CAIC_smaller_is_better_Global
									HQIC__smaller_is_better_= HQIC_smaller_is_better_Global
								%end;	
							%end;
							%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
								%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
									_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global
								%end;
								%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
									_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Global
									Generalized_Chi_Square= Generalized_ChiSquare_Global
									Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Global											
								%end;
							%end;										
						%end;
						%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
							_2_LOG_L= _2_LOG_L_Global
							AIC= AIC_Global
							SBC= SBC_Global
						%end;
						%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
							AIC= AIC_Global
							SC= SC_Global
							_2_Log_L= _2_Log_L_Global
						%end;
						;
					run;

					/*data OutLib.ma_o1_&w; set ma_o; run;*/					
				%end;

				%if (&ie=0 or %sysfunc(indexw(%upcase(&QKClassVars),%upcase(%scan(&IntEff,&ie))))=0) and %upcase(&TraitType) ne NOMINAL %then %do;
					data ta_o;
						%if &ie=0 %then %do;
							merge 
								pat_o(where=(
									%let first= 1;
									%do i=1 %to &nhaps; 
										%if &first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											&EffName="&&hap&i"
											%let first=0;
										%end;
										%else %if ^&first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											or &EffName="&&hap&i"
										%end;
									%end;
									) 
									keep=&EffName Estimate Stderr
								)
								ta_o(where=(
									%let first= 1;
									%do i=1 %to &nhaps;
										%if &first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											effect ? "&&hap&i"
											%let first= 0;
										%end;
										%else %if ^&first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											or effect ? "&&hap&i"
										%end;
									%end;
									)
								);
						%end;
						%else %do;
							merge 
								pat_o(where=(
									%let first= 1;
									%do i=1 %to &nhaps;
										%if &first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											&EffName ? "&&hap&i*" or &EffName ? "*&&hap&i"
											%let first= 0;
										%end;
										%else %if ^&first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											or &EffName ? "&&hap&i*" or &EffName ? "*&&hap&i"
										%end;
									%end;
									) 
									keep=&EffName Estimate Stderr
								)
								ta_o(where=(
									%let first= 1;
									%do i=1 %to &nhaps;
										%if &first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											Effect ? "&&hap&i*" or Effect ? "*&&hap&i"
											%let first= 0;
										%end;
										%else %if ^&first and ^%index(%upcase(&&hap&i), RARE) %then %do;
											or Effect ? "&&hap&i*" or Effect ? "*&&hap&i" 
										%end;
									%end;
									)
								);									
						%end;
						%if %quote(&EffName)=%str(Variable) %then drop &EffName;;
						rename Estimate = Estimate_Global StdErr= StdErr_Global;
					run;

					%let volcanoplot = 1;

					/*data OutLib.ta_3o&w; set ta_o; run;*/
				%end;

				data ta_o;  
					length Trait $32;
					set ta_o;
					%if &ie>0 %then %do;
						if orig=1
							%do i=1 %to &nhaps;
								 or index(Effect,"&&hap&i*")>0 or index(Effect,"*&&hap&i")>0
							%end;
						;
						/* To make sure interactions in same order for Global and Single tests */
						%do i=1 %to &nhaps; 
							if index(Effect,"&&hap&i*")>1 then do;
								Effect=cats("&&hap&i*",tranwrd(Effect,"&&hap&i*",""));
							end;
							else if index(Effect,"&&hap&i")>1 then do;
								Effect=cats("&&hap&i*",tranwrd(Effect,"*&&hap&i",""));
							end;
						%end;					
					%end;
					%else %do;

						%let first= 1;
						where
						%do i=1 %to &nhaps;
							%if &first and ^%index(%upcase(&&hap&i), RARE) %then %do;
								 index(Effect,"&&hap&i")>0
								 %let first= 0;
							%end;
							%else %if ^&first and ^%index(%upcase(&&hap&i), RARE) %then %do;
								 or index(Effect,"&&hap&i")>0
							%end;
						%end;
						;
					%end;				
				run;

				data ta_o_all;  
					length Trait $32;
					set ta_o_all(in=orig) ta_o;				
					if not orig then Trait= propcase("&TraitVar");
					label Prob&Stat=' '; 
				run;

				/*data OutLib.ta_o_all0_&w; set ta_o_all; run; */ 

				%if &ie=0 %then %do;
					%if %Upcase(&Odds)=YES  %then %do; 

						data or_o_all;
							set or_o_all or;
						run;

						data ta_o_all;
							length Trait $32;
							set ta_o_all; 
							set or_o_all;
							%if ^%length(&RandEff &KVars) %then %do;	
								drop unit;
							%end;
							%else %do;
								drop alpha lower upper;
							%end;							
						run;

						/*data OutLib.ta_o_all1_&w; set ta_o_all; run;*/  
					 %end;

					 %if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
						%if %sysfunc(exist(outres)) %then %do;
							data outrest_o_all;
								set outrest_o_all(in=orig) outres;
								if orig=0 then do;
									_Trait_=propcase("&TraitVar");
								end;
							run;		
						%end;

						/*data OutLib.outrest_o_all; set outrest_o_all; run;*/
					%end;

				%end;

				%if &pest=1 %then %do;
					data pat_o_all;	
						%if %length(&IntEffBuild) %then %do;
							length Interaction $32;
						%end;
						set pat_o_all(in=orig) pat_o;
						if orig=0 then do;
							Trait=propcase("&TraitVar");
							%if %length(&IntEffBuild) %then %do;
								Interaction="&Interaction";
							%end;
						end;
					run;

					/*data OutLib.pat_o_all; set pat_o_all; run;*/
				%end;

				%if (&kmodel=1 or %length(&RandEff)) and &covp %then %do;
					data cpt_o_all;
						%if %length(&IntEffBuild) %then %do;
							length Interaction $32;
						%end;										
						set cpt_o_all(in=orig) cp_o;
						if orig=0 then do;
							Trait=propcase("&TraitVar");
							%if %length(&IntEffBuild) %then %do;
								Interaction="&Interaction";
							%end;
						end;
					run;

					/*data OutLib.cpt_o_all; set cpt_o_all; run;*/
				%end;			
			%end;		 
			/*end new qk*/

			/** Single haplotype tests **/
			%let flag_sh= 0;
			%do i=1 %to &numhaps;

				/*begin new qk*/
				title "Test for Haplotype &&hap&i";

				data ta;
					delete;
				run;
				
				data ct;
					delete;
				run;

				options nonotes;
				/*kemicl - Added odds ratio options and Random effect ability with discrete trait. 
				 If Logistic Procedure - enable OddsRatio statement and corresponding ODS output table 
				 If GLIMMIX Procedure - exponentiate estimate from Contrast Statement and retain in ODS output table
				 */
				ods output &Gtable=ta
					 %if &pe=0 %then %do;
						&ParmEst= pat
						&CTable= ct
						%if &WhichProc ne SURVEYREG %then %do;
							nobs = tnobs %if &WhichProc ne PHREG %then (where=(Label ? "Observations Used"));
						%end;
						%if (&kmodel=1 or %length(&RandEff)) and &covp %then CovParms=cp;
						%if %upcase(&TraitType)=BINARY %then responseprofile=rp; 

						/*begin: Luciano*/
						%if %upcase(&OutputRSquare)=YES and %index(%upcase(&WhichProc),LOGISTIC) %then
							rsquare= trsq;
						/*end: Luciano*/	
						%if %upcase(&OutputFitStat)=YES %then 
							FitStatistics= fst;
					%end;
				;
				proc &WhichProc data=outreg &ProcOption %if %Upcase(&Odds)=YES and ^%length(&RandEff &KVars) %then alpha=&Alpha;;
					class %if &NeedConstant=1 %then constant; &QKClassVars &ClassOpt;
					model &TraitVar &TraitOpt = %if &NeedConstant=1 %then constant; &FixedEff &&hap&i &ModOption;
					%if &kmodel=1 %then %do;
						random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
					%end;

					%if %length(&RandEff) %then %do;
						random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
					%end;

					%if %length(&&ParmsStmt&ti) %then %do;
						parms &&ParmsStmt&ti;
					%end;					

					%if %length(&StrataVars)>0 %then %do;
						strata &StrataVars;
					%end;

					%if %Upcase(&Odds)=YES %then %do;
						%if ^%length(&RandEff &KVars) %then %do;
							oddsratio &&hap&i; 
						%end;
						%else %do;
							&EStmt "&&hap&i" &&hap&i 1 / alpha=&Alpha exp cl;
						%end;
					%end;

					contrast "&&hap&i" &&hap&i 1;
					
					%if (%upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES) and %length(&RandEff) %then %do;
						id &IndIDMerg;
					%end;

					%if %length(&ByVars) %then %do; 
						by &ByVars;
					%end;
				quit;	
				options notes;
				
				%NObsVars(ct);
				%if &nobs %then %do;
					data ct;
						set ct;
						if not missing(&ProbC);
					run;
					%NObsVars(ct);
				%end;
			
				%if &nobs=0 %then %do;
					%put WARNING: PROC &WhichProc generated no estimates for Haplotype &&hap&i and Trait &TraitVar in Window &&winval&w...  There may be excessive missing data or the contrast may be nonestimable. Please see the log for more details.;
					%put WARNING: Haplotype test could not be performed for Haplotype &&hap&i and Trait &TraitVar in Window &&winval&w...;
					%let sherr&i=1;
				%end;
				%else %do;				
					%let sherr&i=0;
					%let flag_sh= 1;
					data ta_s&i;
						*length Dependent $32;
						length Trait $32;
						set ta;
						Trait=propcase("&TraitVar");
						*Dependent=propcase("&TraitVar");
						*Stat=&Stat;
						*ProbGlobalHap=&Prob;
						*keep &ByVars Dependent Stat ProbGlobalHap &ConVarC;
						*label Stat="&StatLabel" ProbGlobalHap="&ProbLabel";
					run;	

					/*data OutLib.ta_s&i; set ta_s&i;run;*/

					data ta;
						delete;
					run;
					
					data pat_s&i;
						*length Dependent $32;
						length Trait $32;
						set pat;
						Trait=propcase("&TraitVar");
						*Dependent=propcase("&TraitVar");
						*Stat=&Stat;	
						*ProbGlobalHap=&Prob;
						*keep &ByVars Dependent Stat ProbGlobalHap;
						*label Stat="&StatLabel" ProbGlobalHap="&ProbLabel";
						rename Estimate= Estimate_Single; /*GenMacro.sas volcano plot needs this variable*/
					run;	

					/*data OutLib.pat_s&i; set pat_s&i; run;*/
					
					data ma_s&i;
						*length Dependent $32;
						length Trait $32;
						set ct;
						Trait=propcase("&TraitVar");
						*Dependent=propcase("&TraitVar");
						/*Stat=&StatC;*/
						&StatC._Single=&StatC;
						/*ProbTrendHap=&ProbC;*/
						&ProbC._Single=&ProbC;
						*ProbGlobalHap=&ProbC;
						*keep &ByVars Dependent Stat ProbGlobalHap &ConVarC;
						drop &StatC &ProbC;
						*label Stat="&StatCLabel" ProbGlobalHap="&ProbCLabel";
						/*label Stat="&StatCLabel" ProbTrendHap="&ProbCLabel";*/
					run;
					
					data ma_s&i;
						merge ma_s&i pat_s&i(where= (&EffName= "&&hap&i"));
					run;
					
					data ct;
						delete;
					run;

					/*data OutLib.ma_s&i.&w; set ma_s&i; run;*/
					
					%if %upcase(&OutputFitStat)=YES %then %do;
						%put WhichProc= &WhichProc;

						proc transpose data=fst(
							%if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
								rename=(
									criterion= descr 
									interceptandcovariates= value
								)
							%end;
							%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
								rename=(
									criterion= descr  
									withcovariates= value
								)
							%end;
							)
							out=fst(drop= _name_); 
							var value;
							id descr;
							by &ByVars;
						run;

						/*data OutLib.fst_s1; set fst; run;*/

						data ma_s&i;
							merge ma_s&i fst;
							by &ByVars;
							rename
							%if %index(%upcase(&WhichProc),MIXED) %then %do;
								_2_Res_Log_Likelihood= _2_Res_Log_Likelihood_Single
								AIC__Smaller_is_Better_= AIC_Smaller_is_Better_Single
								AICC__Smaller_is_Better_= AICC_Smaller_is_Better_Single
								BIC__Smaller_is_Better_= BIC_Smaller_is_Better_Single
							%end;
							%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
								%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
									or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
									%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
										_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Single
										Generalized_Chi_Square= Generalized_ChiSquare_Single
										Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Single
									%end;
									%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
										_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Single
										Generalized_Chi_Square= Generalized_ChiSquare_Single
										Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Single
									%end;
									%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
										_2_Log_Likelihood= _2_Log_Likelihood_Single
										AIC___smaller_is_better_= AIC_Smaller_is_Better_Single
										AICC__smaller_is_better_= AICC_Smaller_is_Better_Single
										BIC___smaller_is_better_= BIC_Smaller_is_Better_Single
										CAIC__smaller_is_better_= CAIC_smaller_is_better_Single
										HQIC__smaller_is_better_= HQIC_smaller_is_better_Single											
									%end;
								%end;
								%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 		
									_2_Log_Likelihood= _2_Log_Likelihood_Single
									AIC___smaller_is_better_= AIC_Smaller_is_Better_Single
									AICC__smaller_is_better_= AICC_Smaller_is_Better_Single
									BIC___smaller_is_better_= BIC_Smaller_is_Better_Single
									CAIC__smaller_is_better_= CAIC_smaller_is_better_Single
									HQIC__smaller_is_better_= HQIC_smaller_is_better_Single
									Pearson_Chi_Square= Pearson_ChiSquare_Single
									Pearson_Chi_Square___DF= Pearson_ChiSq_DF_Ratio_Single											
								%end;
								%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
									%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
										_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Single
									%end;
									%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
										_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Single
									%end;
									%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
										_2_Log_Likelihood= _2_Log_Likelihood_Single
										AIC___smaller_is_better_= AIC_Smaller_is_Better_Single
										AICC__smaller_is_better_= AICC_Smaller_is_Better_Single
										BIC___smaller_is_better_= BIC_Smaller_is_Better_Single
										CAIC__smaller_is_better_= CAIC_smaller_is_better_Single
										HQIC__smaller_is_better_= HQIC_smaller_is_better_Single
									%end;	
								%end;
								%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
									%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
										_2_Res_Log_Pseudo_Likelihood= _2_Res_Log_Pseud_Likel_Single
										Generalized_Chi_Square= Generalized_ChiSquare_Single
										Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Single
									%end;
									%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
										_2_Log_Pseudo_Likelihood= _2_Log_Pseud_Likel_Single
										Generalized_Chi_Square= Generalized_ChiSquare_Single
										Gener__Chi_Square___DF= Gener_ChiSq_DF_Ratio_Single											
									%end;
								%end;										
							%end;
							%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
								_2_LOG_L= _2_LOG_L_Single
								AIC= AIC_Single
								SBC= SBC_Single
							%end;
							%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
								AIC= AIC_Single
								SC= SC_Single
								_2_Log_L= _2_Log_L_Single
							%end;
							;
						run;

						/*data OutLib.ma_s_fit&i; set ma_s&i;  run;*/				

					%end;
									
					%if &pest=1 %then %do;
						data pat_s_all;	
							%if %length(&IntEffBuild) %then %do;
								length Interaction $32;
							%end;
							set pat_s_all(in=orig) pat_s&i;
							if orig=0 then do;
								Trait=propcase("&TraitVar");
								%if %length(&IntEffBuild) %then %do;
									Interaction="&Interaction";
								%end;
							end;
						run;
	
						/*data OutLib.pat_s_all; set pat_s_all; run;*/
					%end;
					
				%end;	
				/*end new qk*/	
			%end;

			%if &Perms>0 %then %do;		
				%if %length(&ByVars) %then %do;
					%let i=1;
					%do %while(%length(%scan(&ByVars,&i))>0);
						%let i=%eval(&i+1);
					%end;
					%let lastby=%scan(&ByVars,%eval(&i-1));
				%end;

				data ma_all;
					delete;
				run;
				
				data c_perm;
					delete;
				run;

				%if &haperr=1 %then %goto skipoverall;

				title "Permutations for Overall Test";
				%if &OutputType ^= None %then %do;
					ods &OutputType exclude all;
				%end;
				ods output &CTable=c_perm;		
				proc &WhichProc data=allp &ProcOption;

					class %if &NeedConstant=1 %then constant; &QKClassVars &ClassOpt;

					model &TraitVar &TraitOpt = %if &NeedConstant=1 %then constant; &FixedEff %do i=1 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then &&hap&i; %end; &ModOption;

					%if &kmodel=1 %then %do;
						random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
					%end;

					%if %length(&RandEff) %then %do;
						random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
					%end;

					%if %length(&&ParmsStmt&ti) %then %do;
						parms &&ParmsStmt&ti;
					%end;					

					%if %length(&StrataVars)>0 %then %do;
						strata &StrataVars;
					%end;

					%if ^%index(%upcase(&hap1), RARE) %then %do;
						contrast 'Overall' &hap1 1 %do i=2 %to &nhaps; %if ^%index(%upcase(&&hap&i), RARE) %then , &&hap&i 1; %end;; 
					%end;
					%else %do;
						contrast 'Overall' &hap2 1 %do i=3 %to &nhaps;, &&hap&i 1 %end;;
					%end;

					by _sample_ &ByVars;
				quit;
				
				%NobsVars(c_perm);
				%if &nobs %then %do;

					%if %length(&ByVars) %then %do;
						proc sort data=c_perm;
							by &ByVars _sample_;
						run;
					%end;

					data c_perm;
						*length Dependent $32;
						length Trait $32;
						set c_perm;
						*Dependent=propcase("&TraitVar");
						Trait=propcase("&TraitVar");
					run;

					/*data OutLib.c_perm0; set c_perm; run;*/
					/*data OutLib.c_ma_o0; set ma_o; run;*/

					data c_perm;
						merge c_perm ma_o;
						*by Dependent &ByVars;
						by Trait &ByVars;
						drop df ss ms probf;
					run;

					/*data OutLib.c_perm1; set c_perm; run;*/

					options label;
					data c_perm;
						set c_perm end=last;
						%if %length(&ByVars) %then %do;
							by &ByVars;
							if first.&lastby then Prob=0;
						%end;
						*if &StatC > Stat then Prob+1;
						if &StatC > &StatC._Global then Prob+1;
						%if %length(&ByVars) %then %do;
							if last.&lastby then do;
								Prob_emp = Prob / &Perms;
								output;
							end;			 	
						%end;
						%else %do;
							if last then do;
								Prob_emp = Prob / &Perms;
								output;
							end;
						%end;
						*keep &ByVars Dependent Stat ProbGlobalHap Prob_emp;
						drop &StatC &ProbC Prob _sample_;
						label Prob_emp="&ProbC_empLabel";
					run;

					/*data OutLib.c_perm2; set c_perm; run;*/
					/*data OutLib.c_OverallTest&&winval&w; set OverallTest&&winval&w; run;*/

					data OverallTest&&winval&w;
						set OverallTest&&winval&w c_perm(in= b);
						if b then GlobalTest_Status = 0;
					run;
					
					%let flag_globalstatus= 1;
				
				%end;

				%skipoverall:

				%do i=1 %to &numhaps;
					%if &&sherr&i=1 %then %goto skipsh;
					
					data c_perm;
						delete;
					run;					

					title "Permutations for Haplotype &&hap&i (Trait=&TraitVar)";
					%if &OutputType ^= None %then %do;
						ods &OutputType exclude all;
					%end;
					*ods output &Ctable=ct;
					ods output &Ctable=c_perm;
					proc &WhichProc data=allp &ProcOption;
						class %if &NeedConstant=1 %then constant; &QKClassVars &ClassOpt;
						model &TraitVar &TraitOpt = %if &NeedConstant=1 %then constant; &FixedEff &&hap&i &ModOption;
						%if &kmodel=1 %then %do;
							random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
						%end;

						%if %length(&RandEff) %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;

						%if %length(&&ParmsStmt&ti) %then %do;
							parms &&ParmsStmt&ti;
						%end;					

						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;

						contrast "&&hap&i" &&hap&i 1;

						by _sample_ &ByVars;
					quit;
					
					%NobsVars(c_perm);
					%if &nobs %then %do;

						%if %length(&ByVars) %then %do;
							proc sort data=c_perm;
								by &ByVars _sample_;
							run;
						%end;

						data c_perm;
							*length Dependent $32;
							length Trait $32;
							set c_perm;
							*Dependent=propcase("&TraitVar");
							Trait=propcase("&TraitVar");
						run;

						/*data OutLib.c_c_perm; set c_perm; run;*/

						options label;
						data ma_s&i;
							merge ma_s&i c_perm end=last;
							*by Dependent &ByVars;
							by Trait &ByVars;
							%if %length(&ByVars) %then %do;
								if first.&lastby then Prob=0;
							%end;
							*if &StatC > Stat then Prob+1;
							if &StatC > &StatC._Single then Prob+1;
							%if %length(&ByVars) %then %do;
								if last.&lastby then do;
									Prob_emp = Prob / &Perms;
									output;
								end;			 	
							%end;
							%else %do;
								if last then do;
									Prob_emp = Prob / &Perms;
									output;
								end;
							%end;
							*keep &ByVars Dependent Stat ProbGlobalHap Prob_emp &ConVarC;
							drop &StatC &ProbC Prob _sample_;
							label Prob_emp="&ProbC_empLabel";
						run;

						/*data OutLib.c1_ma_s&i; set ma_s&i; run;*/

						data c_perm;
							delete;
						run;

						data ma_all;
							length Hapname &ConVarC $16;
							set ma_all ma_s&i(in=new);
							if new then Hapname=strip(&ConVarC);
						run;
					%end;

					%skipsh:;
				%end;

				%if &flag_sh %then %do;
					proc sort data=ma_all;
						by hapname;
					run;
	
					data ma_all;
						*length Dependent $32;
						length Trait $32;
						merge hapnames ma_all;
						by hapname;
						if Prob_Emp=. then Prob_Emp=0;
						if missing(Trait) then Trait= propcase("&TraitVar");
						drop &ConVarC;
					run;
	
					proc sort data=ma_all; 
						by &ByVars %if %length(&HFData) %then descending &FreqVar; haplotype;
					run;
				%end;
				
				/*data OutLib.ma_all_perm; set ma_all; run;*/
				
				%if &OutputType ^= None %then %do;
					ods &OutputType;
				%end;
			%end; /*perm*/
			%else %do;
				/*begin new qk*/
				/*data OutLib.OverallTest0_&&winval&w; set OverallTest&&winval&w; run;*/
				%let flag_globalstatus= 1;
				
				%if &haperr=0 %then %do;
					data OverallTest&&winval&w;
						set OverallTest&&winval&w ma_o;
						GlobalTest_Status = 0;
					run;
				%end;
				%else %do;
					
					data OverallTest&&winval&w;
						length Trait %if &wintype=C %then WindowID; $32;
						Trait= propcase("&TraitVar");
						%if &wintype=C %then %do;
							WindowID= "&&winval&w";
						%end;
						%else %do;
							WindowID= &&winval&w;
						%end;
						GlobalTest_Status = 1;
					run;
				%end;
				
				/*data OutLib.OverallTest1_&&winval&w; set OverallTest&&winval&w; run;*/

				data ma_all; 
					length Hapname &ConVarC $32;
					set %do i=1 %to &numhaps; %if &&sherr&i=0 %then ma_s&i; %end;;
					Hapname=strip(&ConVarC);
					drop &ConVarC;
				run;

				proc sort data=ma_all;
					by hapname;
				run;

				/*data OutLib.ma_all0; set ma_all; run;*/

				data ma_all;
					*length Dependent $32;
					length Trait $32;
					merge hapnames ma_all;
					by hapname;
					*if missing(dependent) then delete;
					*if missing(Trait) then delete;
					if missing(Trait) then Trait= propcase("&TraitVar");
				run;

				proc sort data=ma_all; 
					by &ByVars %if %length(&HFData) %then descending &FreqVar; haplotype;
				run;

				/*data OutLib.ma_all1; set ma_all; run;*/
				/*end new qk*/
			%end;
			
			/*data OutLib.ma_all_f; set ma_all; run;*/

			data SingleTests&&winval&w;
				set SingleTests&&winval&w ma_all;
			run;

			%if &w=1 %then %do;
				%let NTrait= %eval(&NTrait + 1);
				%global _tv&NTrait;
				%let _tv&NTrait = &TraitVar;
			%end;
			
			%nexttrait:;
			%let ti=%eval(&ti+1);
		%end;
		
		%if &w=1 %then %do;
			%put NTrait= &NTrait;
		%end;
		/* End loop over traits */

		/*begin new qk*/
		%if &flag_globalstatus %then %do;
			data OverallTest_Temp;
				set OverallTest&&winval&w;
				where GlobalTest_Status = 0;
			run;
			%Nobsvars(OverallTest_Temp);
		%end;
		%else %do;
			%Nobsvars(OverallTest&&winval&w);
		%end;
		%let nobs1=&nobs;
		%put nobs1=&nobs1;
		%Nobsvars(SingleTests&&winval&w);
		%let nobs2=&nobs;
		%put nobs2=&nobs2;		

		%if %eval(&nobs1+&nobs2)>0 %then %do;
			%if &nobs1>0 %then %do;
				%let global_flag= 1;
			
				/*
				title "Global Test Statistic - Adjusted for Any Covariates (WindowID=&&winval&w)";
				proc print data=OverallTest&&winval&w(drop=WindowID) label; run;
				*/

				data OutLib.&SNPTraitAssoc_HMGName;
					set OutLib.&SNPTraitAssoc_HMGName OverallTest&&winval&w(in=new);
					%if &wintype=C %then %do;
						if new then WindowID = "&&winval&w";				
					%end;
					%else %do;
						if new then WindowID = &&winval&w;
					%end;	
					if new then _TempVar_= 1;
				run;
			%end;

			%if &nobs2>0 %then %do;
				%let single_flag= 1;
				
				/*
				title "Single Haplotype Test Statistics - Adjusted for Any Covariates (WindowID=&&winval&w)";
				proc print data=SingleTests&&winval&w(drop=WindowID) label; run;
				*/
				
				%if &firstwind %then %do;
					%let nobs2_cum= 0;
					%let firstwind= 0;
				%end;
				
				/*data OutLib.SingleTests_&&winval&w; set SingleTests&&winval&w; run;*/
				
				data OutLib.&SNPTraitAssoc_HMSName;
					set OutLib.&SNPTraitAssoc_HMSName(in= old) SingleTests&&winval&w(in=new);
					%if &wintype=C %then %do;
						if new then WindowID = "&&winval&w";				
					%end;
					%else %do;
						if new then WindowID = &&winval&w;
					%end;
					if new then _TempVar_= 1;
				run;
				
				/*data OutLib.SNPTraitAssoc_HMS_00; set &SNPTraitAssoc_HMS; run;*/
				
				%let nobs2_cum= %eval(&nobs2_cum + &nobs2);
			%end;

			proc datasets nolist;
				delete OverallTest&&winval&w SingleTests&&winval&w;
			quit;		
		%end;	
		/*end new qk*/

		%nextwindow:;
	%end;	

	%if &ModelOut = None %then %do;
		ods exclude none;
		ods results;
	%end;
	%else %if &ModelOut = HTML %then %do;
		ods html close;
		%PackageInsertHtml(html from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile);
		ods listing;
	%end;

	/*begin new qk*/
	%PrepOutFile(HaplotypeQKMixedModel_Script,HaplotypeQKMixedModel_ScriptName,&ProcessName Script for &OutName..jsl);
	%global _mainjslfile_;
	%let _mainjslfile_ = %quote(&HaplotypeQKMixedModel_Script);

	%global maxlenAG NumGps AnnoOrganism AnnoAccession_All AnnoData 
		ANNOACCESSION GpType /*JBy NonTraitByVars
		Nbyval ByVarsOnly NByVars OrigNbyval JNONTRAITBYVARS*/
		SGroupn1 HQKDepData nCallstoJSPV HaploTest;
		
	%if &NTrait > 1 %then %do;
		%let ByVars= Trait &ByVars0;
	%end;
	%else %do;
		%let ByVars= &ByVars0;
	%end;
 		
 	%if &NTrait>1 %then %do;
 		/* This stuff for Venns */
 		%if %length(&ByVars)>0 %then %do;
 			%let ByVars = Trait &ByVars;
 			%let ByVarsOnly = Trait &ByVarsOnly;
 			%let JBy = %str(Trait,&JBy);
 			%let JByOdds= &JBy; 
 		%end;
 		%else %do;
 			%let ByVars = Trait;
 			%let ByVarsOnly = Trait;
 			%let JBy=Trait;
 			%let JByOdds= &JBy; 
 		%end;
 		%let Nbyval = %eval(&Nbyval*&Ntrait);
	%end;		
				
	%let MTCOrig= &MTC;
	%let ConvOrig= &Conv;
			
	**begin: output statistics from global haplotype tests**;
	%put global_flag= &global_flag;
	%if &global_flag %then %do;
		%if &pest=1 %then %do;
			data &SNPTraitAssoc_HGPE;
				set pat_o_all;
			run;

			%PackageInsertDataset(Parameter estimates data for the global tests from &ProcessName, OutLib, &SNPTraitAssoc_HGPEName);
		%end;

		%if (&kmodel=1 or %length(&RandEff)) and &covp %then %do;
			data &SNPTraitAssoc_HGCP;
				set cpt_o_all;
			run;

			%PackageInsertDataset(Covariance parameter data for the global tests from &ProcessName, OutLib, &SNPTraitAssoc_HGCPName);
		%end;

		%if %upcase(&OutputResid)=YES or %upcase(&OutputPred)=YES %then %do;
			proc sort data=outrest_o_all;
				by &IndIDMerg WindowID _Trait_;
			run;	
		%end;

		%if %upcase(&OutputResid)=YES %then %do;
			proc transpose data=outrest_o_all out =outrestt(drop= _name_ _LABEL_) let
				%if &NTrait>1 %then %do;
					prefix=Res_ delimiter=_Trt_
				%end;
				%else %do;
					prefix=Resid_
				%end;
				;
				var resid;
				id WindowID %if &NTrait>1 %then _Trait_;;
				by &IndIDMerg;
			run;

			data &SNPTraitAssoc_HGOR;
				set outrestt;
			*	 merge othervars outrestt;
			*	 by IndIDMerg;
			run;

			%PackageInsertDataset(Residual data for the global tests from &ProcessName, OutLib, &SNPTraitAssoc_HGORName);
		%end;

		%if %upcase(&OutputPred)=YES %then %do;
			proc transpose data=outrest_o_all out=outpredtt(drop= _name_ _LABEL_) let
				%if &NTrait>1 %then %do;
					prefix=Pred_ delimiter=_Trt_
				%end;
				%else %do;
					prefix=Pred_
				%end;
				;
				var pred;
				id WindowID %if &NTrait>1 %then _Trait_;;
				by &IndIDMerg;
			run;

			data &SNPTraitAssoc_HGOP;
				set outpredtt;
				drop _LABEL_;
			*	 merge othervars outpredtt;
			*	 by IndIDMerg;
			run;

			%PackageInsertDataset(Predicted data for the global tests from &ProcessName, OutLib, &SNPTraitAssoc_HGOPName);			
		%end;
		/*end new qk*/	

		%if %upcase(&OutputFitStat)= NO %then %do;
			%global PlotFitVars;
			%let PlotFitVars=0;
		%end;
		%else %if %upcase(&OutputFitStat)=YES %then %do;	
			%global PlotFitVars Group1OverlayVars Group2OverlayVars Group3OverlayVars;
			%let PlotFitVars=1;
			%let Group1OverlayVars=;
			%let Group2OverlayVars=;
			%let Group3OverlayVars=;

			** Global test **;
			%if %index(%upcase(&WhichProc),MIXED) %then %do;
				%let Group1OverlayVars= _2_Res_Log_Likelihood_Global AIC_Smaller_is_Better_Global AICC_Smaller_is_Better_Global BIC_Smaller_is_Better_Global;
			%end; 
			%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
				%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
					or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
					%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
						%let Group1OverlayVars= _2_Res_Log_Pseud_Likel_Global;
						%let Group2OverlayVars= Generalized_ChiSquare_Global;
						%let Group3OverlayVars= Gener_ChiSq_DF_Ratio_Global;					
					%end;
					%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
						%let Group1OverlayVars= _2_Log_Pseud_Likel_Global;
						%let Group2OverlayVars= Generalized_ChiSquare_Global;
						%let Group3OverlayVars= Gener_ChiSq_DF_Ratio_Global;
					%end;
					%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
						%let Group1OverlayVars= _2_Log_Likelihood_Global AIC_Smaller_is_Better_Global AICC_Smaller_is_Better_Global BIC_Smaller_is_Better_Global CAIC_smaller_is_better_Global HQIC_smaller_is_better_Global;
					%end;
				%end;
				%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 
					%let Group1OverlayVars= _2_Log_Likelihood_Global AIC_Smaller_is_Better_Global AICC_Smaller_is_Better_Global BIC_Smaller_is_Better_Global CAIC_smaller_is_better_Global HQIC_smaller_is_better_Global;	
					%let Group2OverlayVars= Pearson_ChiSquare_Global;	
					%let Group3OverlayVars= Pearson_ChiSq_DF_Ratio_Global;	
				%end;
				%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
					%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
						%let Group1OverlayVars= _2_Res_Log_Pseud_Likel_Global;
					%end;
					%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
						%let Group1OverlayVars= _2_Log_Pseud_Likel_Global;
					%end;
					%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
						%let Group1OverlayVars= _2_Log_Likelihood_Global AIC_Smaller_is_Better_Global AICC_Smaller_is_Better_Global BIC_Smaller_is_Better_Global CAIC_smaller_is_better_Global HQIC_smaller_is_better_Global;
					%end;
				%end;
				%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
					%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
						%let Group1OverlayVars= _2_Res_Log_Pseud_Likel_Global;
						%let Group2OverlayVars= Generalized_ChiSquare_Global;
						%let Group3OverlayVars= Gener_ChiSq_DF_Ratio_Global;
					%end;
					%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
						%let Group1OverlayVars= _2_Log_Pseud_Likel_Global;
						%let Group2OverlayVars= Generalized_ChiSquare_Global;
						%let Group3OverlayVars= Gener_ChiSq_DF_Ratio_Global;										
					%end;
				%end;
			%end;
			%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
				%let Group1OverlayVars= _2_LOG_L_Global AIC_Global SBC_Global;
			%end;
			%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
				%let Group1OverlayVars= AIC_Global SC_Global _2_Log_L_Global;
			%end;

			%put PlotFitVars= &PlotFitVars; 
			%put Group1OverlayVars= &Group1OverlayVars; 
			%put Group2OverlayVars= &Group2OverlayVars; 
			%put Group3OverlayVars= &Group3OverlayVars; 
		%end;	

		/*data OutLib.SNPTraitAssoc_HMG0; set &SNPTraitAssoc_HMG; run;*/
		
		%if %length(&ByVars0) or &NTrait > 1 %then %do;
			proc sort data= OutLib.&SNPTraitAssoc_HMGName;
				by Trait &ByVars0 WindowID;
			run;
		%end;

		/*
		%*let ByVars= Trait;
		%if &NTrait > 1 %then %do;
			%let ByVars= Trait &ByVars0;
		%end;
		%else %do;
			%let ByVars= &ByVars0;
		%end;
		*/
		%let MTC= &MTCOrig;
		%let Conv= &ConvOrig;		
		%let HTests= %sysfunc(substr(&ProbC, 5))_Global; /*GlobalHap*/
		%put HTests= &HTests;
		%Psmooth(&HTests, &SNPTraitAssoc_HMG, &SNPTraitAssoc_HMGName);
		%if (&syserr>4) %then %goto exit;
		%let PlotVarsGlobal= &PlotVars;
		%put PlotVarsGlobal= &PlotVarsGlobal;
		%let NPVars= &NPVars;

		/*data OutLib.SNPTraitAssoc_HMG1; set &SNPTraitAssoc_HMG; run;*/
		
		%let maxlenAG= 0;
		%let NumGps= 1;
		%let AnnoOrganism= ;
		%let AnnoAccession_All= ;
		%let AnnoGroup= _TempVar_;
		%let AnnoLoc= WindowID;
		%let AnnoLabel= &ConVarC;
		/*
		%*let ByVars= Trait &ByVars0;
		%let JBy= &ByVars;
		%let NbyVal= &NTrait;
		%let ByVarsOnly=;
		%let NByVars= 0;
		%let OrigNbyval= 1;
		%let JNONTRAITBYVARS=;
		*/
		/*
 		%if &NTrait>1 %then %do;
 			** This stuff for Venns *;
 			%if %length(&ByVars)>0 %then %do;
 				%let ByVars = Trait &ByVars;
 				%let ByVarsOnly = Trait &ByVarsOnly;
 				%let JBy = %str(Trait,&JBy);
 				%let JByOdds= &JBy; 
 			%end;
 			%else %do;
 				%let ByVars = Trait;
 				%let ByVarsOnly = Trait;
 				%let JBy=Trait;
 				%let JByOdds= &JBy; 
 			%end;
 			%let Nbyval = %eval(&Nbyval*&Ntrait);
		%end;	
		*/
		%let nCallstoJSPV= 0;
		%let HQKDepData=;
		%let HaploTest= Global;
		%JSLPvalMap(Haplotype Q-K, &HaplotypeQKMixedModel_Script, &HaplotypeQKMixedModel_ScriptName, &SNPTraitAssoc_HMGName, mod=0, _summchart=1, _volplot=0); 
	%end;
	%else %do;
		proc datasets library=outlib nolist;
			delete &SNPTraitAssoc_HMGName;
		quit;
	%end;
	%let HQKDepData_all= &HQKDepData;
	
	**begin: output statistics from single haplotype tests**;
	/*data OutLib.SNPTraitAssoc_HMS_01; set &SNPTraitAssoc_HMS; run;*/

	%let modsingle= 0;	
	%if &global_flag %then %do;
		data _null_;
			file "&HaplotypeQKMixedModel_Script" mod lrecl=32767;
			put "////// JSL Single Starts Here //////";
		run;
		%let modsingle= 1;
	%end;
	
	/*data OutLib.SNPTraitAssoc_HMS_0; set &SNPTraitAssoc_HMS; run;*/
	
	proc sort data=&SNPTraitAssoc_HMS;
		by WindowID %if %length(&HFData) %then descending &FreqVar; Hapname Trait;
	run;
	
	options label; /*must use options label here, otherwise labels won't appear in the output file*/
	data &SNPTraitAssoc_HMS;
		set &SNPTraitAssoc_HMS;
		by WindowID %if %length(&HFData) %then descending &FreqVar; Hapname Trait;
		if _n_ = 1 then do;	
			_HaplotypeID_= 0;
		end;
		if first.Hapname then _HaplotypeID_= _HaplotypeID_ + 1;
		retain _HaplotypeID_;
		%if %length(&QKClassVars) %then %do;
			drop &QKClassVars;
		%end;
		/*%if %length(&HFData) %then drop &FreqVar;;*/
		%if &Perms>0 %then %do;
			label Prob_emp="&ProbC_empLabel"; /*must put label here, otherwise it won't appear in the output file*/
		%end;
	run;
	
	/*data OutLib.SNPTraitAssoc_HMS_1; set &SNPTraitAssoc_HMS; run;*/
	
	%if %upcase(&OutputFitStat)= NO %then %do;
		%global PlotFitVars;
		%let PlotFitVars=0;
	%end;
	%else %if %upcase(&OutputFitStat)=YES %then %do;	
		
		%global PlotFitVars Group1OverlayVars Group2OverlayVars Group3OverlayVars;
		%let PlotFitVars=1;
		%let Group1OverlayVars=;
		%let Group2OverlayVars=;
		%let Group3OverlayVars=;
				
		** Single test **;
		
		%if %index(%upcase(&WhichProc),MIXED) %then %do;
			%let Group1OverlayVars= &Group1OverlayVars _2_Res_Log_Likelihood_Single AIC_Smaller_is_Better_Single AICC_Smaller_is_Better_Single BIC_Smaller_is_Better_Single;
		%end; 
		%else %if %index(%upcase(&WhichProc),GLIMMIX) %then %do;
			%if (%upcase(&TraitType)=COUNT and %length(&RandEff)>0) 
				or (%upcase(&TraitType)=BINARY and %length(&RandEff)>0) %then %do;
				%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Res_Log_Pseud_Likel_Single;
					%let Group2OverlayVars= &Group2OverlayVars Generalized_ChiSquare_Single;
					%let Group3OverlayVars= &Group3OverlayVars Gener_ChiSq_DF_Ratio_Single;					
				%end;
				%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Log_Pseud_Likel_Single;
					%let Group2OverlayVars= &Group2OverlayVars Generalized_ChiSquare_Single;
					%let Group3OverlayVars= &Group3OverlayVars Gener_ChiSq_DF_Ratio_Single;
				%end;
				%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Log_Likelihood_Single AIC_Smaller_is_Better_Single AICC_Smaller_is_Better_Single BIC_Smaller_is_Better_Single CAIC_smaller_is_better_Single HQIC_smaller_is_better_Single;
				%end;
			%end;
			%else %if %upcase(&TraitType)=COUNT and %length(&RandEff)=0 %then %do; 
				%let Group1OverlayVars= &Group1OverlayVars _2_Log_Likelihood_Single AIC_Smaller_is_Better_Single AICC_Smaller_is_Better_Single BIC_Smaller_is_Better_Single CAIC_smaller_is_better_Single HQIC_smaller_is_better_Single;	
				%let Group2OverlayVars= &Group2OverlayVars Pearson_ChiSquare_Single;	
				%let Group3OverlayVars= &Group3OverlayVars Pearson_ChiSq_DF_Ratio_Single;	
			%end;
			%else %if %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
				%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Res_Log_Pseud_Likel_Single;
				%end;
				%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Log_Pseud_Likel_Single;
				%end;
				%else %if %upcase(&GMethod)=QUAD or %upcase(&GMethod)=LAPLACE %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Log_Likelihood_Single AIC_Smaller_is_Better_Single AICC_Smaller_is_Better_Single BIC_Smaller_is_Better_Single CAIC_smaller_is_better_Single HQIC_smaller_is_better_Single;
				%end;
			%end;
			%else %if (%upcase(&TraitType)=BINARY and &kmodel=1) %then %do;
				%if %upcase(&GMethod)=RSPL or %upcase(&GMethod)=RMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Res_Log_Pseud_Likel_Single;
					%let Group2OverlayVars= &Group2OverlayVars Generalized_ChiSquare_Single;
					%let Group3OverlayVars= &Group3OverlayVars Gener_ChiSq_DF_Ratio_Single;
				%end;
				%else %if %upcase(&GMethod)=MSPL or %upcase(&GMethod)=MMPL %then %do;
					%let Group1OverlayVars= &Group1OverlayVars _2_Log_Pseud_Likel_Single;
					%let Group2OverlayVars= &Group2OverlayVars Generalized_ChiSquare_Single;
					%let Group3OverlayVars= &Group3OverlayVars Gener_ChiSq_DF_Ratio_Single;										
				%end;
			%end;
		%end;
		%else %if %index(%upcase(&WhichProc),PHREG) %then %do;
			%let Group1OverlayVars= &Group1OverlayVars _2_LOG_L_Single AIC_Single SBC_Single;
		%end;
		%else %if %index(%upcase(&WhichProc),LOGISTIC) %then %do;
			%let Group1OverlayVars= &Group1OverlayVars AIC_Single SC_Single _2_Log_L_Single;
		%end;
		
		%put PlotFitVars= &PlotFitVars; 
		%put Group1OverlayVars= &Group1OverlayVars; 
		%put Group2OverlayVars= &Group2OverlayVars; 
		%put Group3OverlayVars= &Group3OverlayVars; 
	%end;
	%*let PlotFitVars=0; /*when 0, prevents from plotting fit stats for single tests */
	
	%if %length(&ByVars0) or &NTrait > 1 %then %do;
		proc sort data= OutLib.&SNPTraitAssoc_HMSName;
			by Trait &ByVars0 WindowID %if %length(&HFData) %then descending &FreqVar; haplotype;
		run;
	%end;
	
	/*
	%*let ByVars= Trait;
	%if &NTrait > 1 %then %do;
		%let ByVars= Trait &ByVars0;
	%end;
	%else %do;
		%let ByVars= &ByVars0;
	%end;
	*/
	%let MTC= &MTCOrig;
	%let Conv= &ConvOrig;
	%let HTests= %sysfunc(substr(&ProbC, 5))_Single; /*TrendHap*/;
	%put HTests= &HTests;
	%Psmooth(&HTests, &SNPTraitAssoc_HMS, &SNPTraitAssoc_HMSName);
	%if (&syserr>4) %then %goto exit;
	%let PlotVarsSingle= &PlotVars;
	%put PlotVarsSingle= &PlotVarsSingle;
	%let NPVars= &NPVars;
	
	%let maxlenAG= 0;
	%let NumGps= 1;
	%let AnnoOrganism= ;
	%let AnnoAccession_All= ;
	%let AnnoGroup= _TempVar_;
	%let AnnoLoc= _HaplotypeID_;
	%let AnnoLabel= Haplotype;
	/*
	%*let ByVars= Trait &ByVars0;
	%let JBy= &ByVars;
	%let NbyVal= &NTrait;
	%let ByVarsOnly=;
 	%let NByVars= 0;
 	%let OrigNbyval= 1;
 	%let JNONTRAITBYVARS=;
 	*/
 	/*
 	%if &NTrait>1 %then %do;
 		** This stuff for Venns *;
 		%if %length(&ByVars)>0 %then %do;
 			%let ByVars = Trait &ByVars;
 			%let ByVarsOnly = Trait &ByVarsOnly;
 			%let JBy = %str(Trait,&JBy);
 			%let JByOdds= &JBy; 
 		%end;
 		%else %do;
 			%let ByVars = Trait;
 			%let ByVarsOnly = Trait;
 			%let JBy=Trait;
 			%let JByOdds= &JBy; 
 		%end;
 		%let Nbyval = %eval(&Nbyval*&Ntrait);
	%end; 
	*/
 	%let nCallstoJSPV= 1; 
 	%let HaploTest= Single;
	%JSLPvalMap(Haplotype Q-K, &HaplotypeQKMixedModel_Script, &HaplotypeQKMixedModel_ScriptName, &SNPTraitAssoc_HMSName, mod=&modsingle, _summchart=1, _volplot=1);
	
	%if &global_flag %then %do; 
		%let HQKDepData_all= &HQKDepData_all, &HQKDepData;
	%end;
	%else %do;
		%let HQKDepData_all= &HQKDepData;
	%end;
	
	%if &pest=1 %then %do;
		data &SNPTraitAssoc_HSPE;
			set pat_s_all;
		run;
		
		%PackageInsertDataset(Parameter estimates data for the single tests from &ProcessName, OutLib, &SNPTraitAssoc_HSPEName);
	%end;
	
	%put HQKDepData_all= &HQKDepData_all;
	%PackageInsertFile(jsl code from &ProcessName, &OutPath, &HaplotypeQKMixedModel_ScriptName, "&HQKDepData_all", text/plain, yes);
	
	/*S1199692: clear title to avoid showing titles on other processes*/
	title;

	%exit:;
	%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend ;

* run the macro;
%HapQKMixedModel;
