/*
JMP Genomics Analytical Process

Name: Marker-Trait Association
Author: Wendy Czika

Description:
		Marker-Trait Association tests for
        association between various types of traits
        and marker genotypes or alleles.  Two
        types of analyses can performed: an
        ANOVA based on marker genotypes or a
        regression testing for a linear trend of
        marker alleles.  Adjustments can be made
        for quantitative covariates and random effects
        or for some trait types, strata variables.
        P-values from these tests, with adjustments
        applied if requested, are plotted along
        the marker map.  See the GLIMMIX, LOGISTIC,
        and PHREG procedures in the SAS/STAT User's
        Guide for more information.
        
History:
	Kelci  08-26-2008	Modify
	
	 Added Odds Ratio Calculations.  Restructured Output table 
	 to avoid errors from missing estimates.
	 
	Luciano 05-09-2013	Modify
	Include a new column (ReasonNoConvergence) in the output SAS 
	dataset to indicate when PROC LOGISTIC did not converge. 
*/

/*

-----------------------------------------------------------
Copyright (C) 2004 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InData AlleleVars ByVars WhereStmt ListMarkerVars ListTraitVars MarkerFmt GenoDelim TraitVar TraitType EventTrait RefTrait CensorVar CensorValues ClassVars 
FixedEff RandEff RandOpts ParmsStmt IntEff StrataVars  AnnoInData AnnoLabel AnnoMName AnnoGroup AnnoLoc AnnoAccession AnnoWhereStmt 
PedID KVars ListKVars CompressedK FixCovParms UseLowerB
UseDomCoding QTests Odds DDFM GMethod EstMethod Conv MTC  PValAdj Tau Bandwidth Alpha OnlySig 
OutPath ServerOutPath OutName MacroPath MarkerTraitAssoc_OutData MarkerTraitAssocScript QKMatrix;
 
*ProcessBody;

%macro MarkerTraitAssociation;

%include "&MacroPath./GenMacros.sas" / nosource;
%GenInit(MarkerTraitAssociation,28);
%if &exiterror=1 %then %goto exit;
%global FullAPName;
%let FullAPName=%str(Marker-Trait Association);

%let ReorderingVars = ; * set this macro to empty to avoid certain conflict when call %GenSubset;
	
%PrepOutDS(MarkerTraitAssoc_OutData,MarkerTraitAssoc_OutDataName,&OutName._mta);

%PrepOutFile(MarkerTraitAssocScript,MarkerTraitAssocScriptName,&ProcessName Script for &OutName..jsl);

%if %length(&ListMarkerVars &AlleleVars)=0 %then %do;
 %put ERROR: Marker variables must be specified in either the Marker Variables or List-Style Specification of Marker Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%if %length(&ListTraitVars &TraitVar)=0 %then %do;
 %put ERROR: One or more trait variables must be specified in either the Trait Variables or List-Style Specification of Trait Variables parameters.;
 %let exiterror=1;
 %goto exit;
%end;

%global OldAlleleVars;
%let OldAlleleVars = &AlleleVars;
%let AlleleVars = &AlleleVars &ListMarkerVars;

%let TraitVar = &TraitVar &ListTraitVars;

%if (%length(&QTests)=0) %then %let QTests=Genotype Trend;
%if %upcase(&TraitType)=SURVIVAL %then %do;
	%let QTests=Trend;
%end;
%let geno=0;
%if (%index(%upcase(&QTests),GENOTYPE)>0) %then %let geno=1;
%let trend=0;
%if (%index(%upcase(&QTests),TREND)>0) %then %let trend=1;

%PrepMapandBy;
run;
%if &exiterror=1 %then %goto exit;

ods html close;
ods output AlleleFreq=af;
ods listing close;
proc allele data=&ds &GenoCol prefix=g;
 var &AlleleVars;
 %if %upcase(&MarkerFmt)=%str(NUMERIC GENOTYPES) %then %do;
		format &AlleleVars numtochar.;
 %end;	   	
 %if %length(&WhereStmt) %then %do;
  where &WhereStmt;
 %end;
run;

%if &syserr>4 %then %goto exit;

ods listing;

data af;
 set af;
 by Locus notsorted;
 if first.Locus then ac=1;
 else ac+1;
 mnum = substr(locus,2)+0;
 call symput(compress('m'||left(mnum)||'a'||left(ac)),trim(Allele));
 if last.Locus then call symput('Numa'||left(mnum),left(ac));
run;

/*S1234607: Check for monomorphic markers and if any is present, then issue an error message.
Otherwise, the code below has an if statement (if(&ncol>0)) that checks for polymorphic 
markers in order to create the genotypes data set, and for monomorphic markers the 
genotypes data set will not be created, therefore, generation error*/  
%put nummark= &nummark;
%let nmono= 0;
%let imono= ;
%do i= 1 %to &nummark;
	%if %symexist(Numa&i) %then %do;
		%*put Numa= &&Numa&i;
		%if &&Numa&i < 2 %then %do;		
			%let nmono= %sysevalf(&nmono + 1);
			%let imono= &imono &i;
		%end;
	%end;
%end;

%put nmono= &nmono; 
%put imono= &imono; 
%if &nmono %then %do;
	data mknames;
		set &ds;
		keep &AlleleVars;
	run;
   	%let _dsid_=%sysfunc(open(mknames,i));
	
	%let monoList= ;
	%if %qupcase(&MarkerFmt)= ALLELES %then %do;
		%do i=1 %to &nmono;
			%let j= %scan(&imono,&i,' ');
			%let j= %sysevalf(&j*2 - 1);
			%let k= %sysevalf(&j + 1);
			%let monoList= &monoList %sysfunc(varname(&_dsid_,&j))%str(-)%sysfunc(varname(&_dsid_,&k));
		%end;	
		
		%if &nmono < 2 %then %do;
			%put ERROR: This is one monomorphic marker in the Input SAS Data Set. Please, remove this marker by either using the Filter to Include Markers in the Annotation tab, or running the Marker Properties process under the Genetic Marker Statistics menu. The monomorphic marker pair is: &monoList..;
		%end;
		%else %do;
			%let monoList= %sysfunc(tranwrd(&monoList, %str( ), %str(, )));
			%put ERROR: There are monomorphic markers in the Input SAS Data Set. Please, remove these markers by either using the Filter to Include Markers in the Annotation tab, or running the Marker Properties process under the Genetic Marker Statistics menu. The monomorphic marker pairs are: &monoList..;
		%end;
	%end;
	%else %do;
		%do i=1 %to &nmono;
			%let j= %scan(&imono,&i,' ');
			%let monoList= &monoList %sysfunc(varname(&_dsid_,&j));
		%end;	
		%if &nmono < 2 %then %do;
			%put ERROR: This is one monomorphic marker in the Input SAS Data Set. Please, remove this marker by either using the Filter to Include Markers in the Annotation tab, or running the Marker Properties process under the Genetic Marker Statistics menu. The monomorphic marker name is: &monoList..;
		%end;
		%else %do;
			%let monoList= %sysfunc(tranwrd(&monoList, %str( ), %str(, )));
			%put ERROR: There are monomorphic markers in the Input SAS Data Set. Please, remove these markers by either using the Filter to Include Markers in the Annotation tab, or running the Marker Properties process under the Genetic Marker Statistics menu. The monomorphic marker names are: &monoList..;
		%end;
	%end;
	%let _rc_ = %sysfunc(close(&_dsid_));	
	%let exiterror= 1;
	%goto exit;
%end;

data _null_;
 set mapdata;
 call symput('Locus'||left(_n_),strip(&AnnoLabel));
run;

%global SNPTraitAssoc_PEG SNPTraitAssoc_PET SNPTraitAssoc_CPG SNPTraitAssoc_CPT;
%let SNPTraitAssoc_PEG=;
%let SNPTraitAssoc_PET=;
%let SNPTraitAssoc_CPG=;
%let SNPTraitAssoc_CPT=;
%if &geno=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PEG,SNPTraitAssoc_PEGName,&OutName._peg);
	%PrepOutDS(SNPTraitAssoc_CPG,SNPTraitAssoc_CPGName,&OutName._cpg);

%end;
%if &trend=1 %then %do;
	%PrepOutDS(SNPTraitAssoc_PET,SNPTraitAssoc_PETName,&OutName._pet);
	%PrepOutDS(SNPTraitAssoc_CPT,SNPTraitAssoc_CPTName,&OutName._cpt);
%end;

%if %length(&ListKVars) %then %do;
	data Kvars;
		set &ds(keep=&ListKVars obs=1);
	run;
	
   %let dsid=%sysfunc(open(kvars,i));
   %do i=1 %to %sysfunc(attrn(&dsid,nvars));
		%let KVars=&KVars %sysfunc(varname(&dsid,&i));
	%end;
	%let rc = %sysfunc(close(&dsid));
%end;	
%if %length(&KVars) %then %do;
	%let kmodel=1;
	%let odds=No;
%end;
%else %let kmodel=0;

%let WhichProc=MIXED;
%let Stat=F; 
%let Gtable=Tests3;
%let TStmt=contrast;
%let Ttable=Contrasts;
%let usestrata=0;
%if %length(&DDFM)=0 %then %let DDFM=residual;
%let ModOption=%str(/ htype=3 ddfm=&DDFM s); 
%let ClassOpt=;
%let ProcOption= ;
%let ParmEst = SolutionF;
%let EffName = Effect;

/* Need a separate check for OR when using GLIMMIX since not creating OR data set */
%let AnyOdds=0;
%if %upcase(&TraitType)=COUNT %then %do;
	%let WhichProc=GLIMMIX; 
	%let ModOption=%str(/ link=log dist=p htype=3 ddfm=&DDFM s);
	%let ParmEst = ParameterEstimates;
%end;
%else %if %upcase(&TraitType)=BINARY or %upcase(&TraitType)=NOMINAL or %upcase(&TraitType)=ORDINAL %then %do;
	%let ParmEst = ParameterEstimates;
	%if %length(&RandEff &KVars)>0 %then %do;
		%let WhichProc=GLIMMIX;
		%let ProcOption=%str(Method=&GMethod);		
		%if %upcase(&EstMethod)=YES %then %let ProcOption=%quote(&ProcOption EMPIRICAL);
				
		%if %upcase(&TraitType)=BINARY %then %let ModOption=%str(/ link=logit dist=binary htype=3 ddfm=&DDFM s);
		%else %if %upcase(&TraitType)=NOMINAL %then %do;
			%let ModOption=%str(/ link=glogit dist=mult htype=3 ddfm=betwithin s);
			%let ClassVars = &ClassVars &TraitVar;
		%end;
		%else %if %upcase(&TraitType)=ORDINAL %then %let ModOption=%str(/ link=clogit dist=mult htype=3 ddfm=&DDFM s);
	%end;
	%else %do;
		%let WhichProc=LOGISTIC;
		%let Stat=ChiSq;
		%*let Gtable=Type3;
		%let Gtable=ModelANOVA; /*S1146207: The logistic output formerly named Type3 is now named ModelANOVA*/
		%let TStmt=contrast;
		%let Ttable=ContrastTest;
		%let Otable=OddsRatiosWald;
		%let usestrata=1;
		%if %upcase(&TraitType)=NOMINAL %then %let ModOption=%str(/ link=glogit);
		%else %if %length(StrataVars)>0 %then %let ModOption=/ GCONV=1E-6; 
		%else %let ModOption =;
		%let ClassOpt = %str(/ cprefix=8);
		%let EffName = Variable;
	%end;
%end; 
%else %if %upcase(&TraitType)=SURVIVAL %then %do;
	%let WhichProc=PHREG;
	%let Stat=ChiSq;
	%let TStmt=test;
	%let Ttable=TestStmts;
	%let ModOption =;
	%let usestrata=1;
	%let ParmEst = ParameterEstimates;  
	%let EffName = Parameter;
%end;
%if &usestrata=0 %then %let StrataVars=;

%if %upcase(&UseLowerB)=YES and &kmodel %then %do;
	%if %length(&ParmsStmt) %then %do;
		%if %index(&ParmsStmt,%str(/)) %then %do;
			%let ParmsStmt = %quote(&ParmsStmt lowerb=0);
		%end;
		%else %do;
			%let ParmsStmt = %quote(&ParmsStmt / lowerb=0);
		%end;
	%end;
	%else %let ParmsStmt = %str( / lowerb=0);
%end;

data ta_all;
 delete;
run;

data ma_a;
	length Effect $32;
  delete;
run;

data or_all;         /*kemicl - added for odds ratio table when requested with no random effects */
  delete;
run;

data pag_all;
	length &EffName $32;
	delete;
run;
	
data pat_all;
	length &EffName $32;
	delete;
run;

data cpg_all;
	length Trait $32;
	delete;
run;

data cpt_all;
	length Trait $32;
	delete;
run;

data ncg_all;			/* sbrlsi - collect markers that did not converged in genotype test*/
	length Marker Trait $32;
	delete; 
run;

data nct_all;			/* sbrlsi - collect markers that did not converged in trend test*/
	length Marker Trait $32;
	delete; 
run;

data a1;
	set &ds(obs=1);
run;

proc transpose data=a1 out=at;
	var &TraitVar;
run;

%if &syserr %then %goto exit;

%NObsVars(at);
%global Ntrait;
%let Ntrait = &nobs;

%do i=1 %to &Ntrait; 
	%global _tv&i;
%end; 

proc sort data=at;
	by _name_;
run;

data _null_;
	set at;
	length is$ 16;
	is = "_tv" || strip(_n_);
	call symput(is,strip(_name_));
run;

%let FixedEffVars = ;
%let i = 1;
%do %while(%length(%scan(&FixedEff,&i,'*| ')));
	%let FixedEffVars = &FixedEffVars %scan(&FixedEff,&i,'*| ');
	%let i=%eval(&i+1);
%end;

%let RandEffVars = ;
%let i = 1;
%do %while(%length(%scan(&RandEff,&i,'*| ')));
	%let RandEffVars = &RandEffVars %scan(&RandEff,&i,'*| ');
	%let i=%eval(&i+1);
%end;

%let RandOpts=%lowcase(&RandOpts);

%if &Kmodel %then %do;
	%if (%length(&PedID)=0 and %upcase(&CompressedK)=NO) %then %do;
		data _null_;
			set &ds(obs=1);
			array ks{*} &KVars;
			call symputx("NumKVars",dim(ks));
		run;
		%Nobsvars(&ds);
	
		%if &nobs ne &NumKVars %then %do;
			%put ERROR: The number of columns in the K Matrix must be the same as the number of observations in the input data set.;
			%let exiterror=1;
			%goto exitm;
		%end;
	%end;
%end;

%if &i>1 %then %do;
	%if %qlowcase(&RandEffVars)=%str(intercept) %then %do;
		%let RandEffVars=;
		/* Setting i=3 just to stay out of IF below */
		%let i=3;
	%end;
	/* If just one random effect variable, use alternative random statement */
	%if &i=2 and %length(&RandOpts)=0 and %length(&ParmsStmt)=0 and %index(&RandEffVars,%str(-))=0 %then %do;

		proc sort data=&ds out=dssort;
			by &ByVars &RandEffVars;
		run;
		%if %index(%upcase(&ClassVars),%upcase(&RandEffVars))>0 %then %do;
			%let NumRandEff = %trim(%substr(&RandEffVars,1,28)_num);
			data numrandeff;
				set dssort;
				by &ByVars &RandEffVars;
				if first.&RandEffVars then &NumRandEff+1;
			run;

			%let ClassVars = %sysfunc(tranwrd(%upcase(&ClassVars),%upcase(&RandEffVars),%str()));
			%let RandEffVars = &NumRandEff;
			%let ds = numrandeff;
		%end;
		%else %let ds=dssort;		
		%let RandEff = %str(intercept / subject=&RandEffVars);
	%end;
	%else %do;
		%let foundsub=0;
		%let RandEff = %str(&RandEff / &RandOpts); %* Need slash for nominal trait to do group= option;
		/* Need to see if there's a SUBJECT= variable that needs to be kept */
		%let subind=%index(&RandOpts,sub=)+4;
		%if &subind=4 %then %do;
			%let subind=%index(&RandOpts,subject=)+8;
			%if &subind>8 %then %let foundsub=1;
		%end;
		%else %let foundsub=1;
		%if &foundsub=1 %then %let RandEffVars = &RandEffVars %scan(%substr(&RandOpts,&subind),1);
	%end;
%end;

%let IntEffVars=;
%let i = 1;
%do %while(%length(%scan(&IntEff,&i,'* ')));
	%let IntEffVars = &IntEffVars %scan(&IntEff,&i,'* ');
	%let i=%eval(&i+1);
%end;

%let IntEffBuild=;
%let i = 1;
%do %while(%length(%scan(&IntEff,&i,' ')));
	%let IntEffBuild = &IntEffBuild Genotype*%scan(&IntEff,&i,' ');
	%let i=%eval(&i+1);
%end;
%let NumInt = &i;

%if &syserr %then %goto exit;

%let OrigTraitVar = &TraitVar;

options DKROCOND=nowarn;

%let Flag_NonConvergenceG = 0; /*Luciano*/
%let Flag_NonConvergenceT = 0; /*Luciano*/
%let Flag_NoEstimatesG = 0;
%let Flag_NoEstimatesT = 0;

%do i=1 %to &nummark;
 
	%if %upcase(&UseDomCoding)=YES %then %do;
		/* Use all columns for dominant coding since no linear dependence */
		%if %symexist(Numa&i) %then %let ncol=%eval(&&Numa&i);
		%else %let ncol=0;
	%end;
	%else %do;
		%if %symexist(Numa&i) %then %let ncol=%eval(&&Numa&i-1);
		%else %let ncol=0;
	%end;
	%let maxalen=1;

	%if (&ncol>0) %then %do;
		%let Alist = &&m&i.a1;
		%let avtmp = &&m&i.a1;
		%Naming(_SingleVar=avtmp);
		%let Alistv = &avtmp;		
		%if %length(&&m&i.a1) > &maxalen %then %let maxalen=%length(&&m&i.a1);
		%if (&ncol>1) %then %do;
			%do j=2 %to &ncol;
				%if %length(&&m&i.a&j) > &maxalen %then %let maxalen=%length(&&m&i.a&j);
				%let Alist= &Alist &&m&i.a&j;
				%let avtmp = &&m&i.a&j;
				%Naming(_SingleVar=avtmp);
				%let Alistv = &Alistv &avtmp;				
			%end;
		%end;

		%let all1 = 1;
		data genotypes;
			%if %upcase(&MarkerFmt) ne %str(NUMERIC GENOTYPES) %then %do;
				length Genotype $%eval(2*&maxalen + 1) %if %length(&GenoCol) %then _a_1 _a_2 $&maxalen;;
			%end;
			set &ds; 
			array _g_{*} &AlleleVars;
			%if %length(&GenoCol) %then %do;
				%if %length(&GenoDelim) %then %do;
					_a_1=scan(_g_{&i},1,"&GenoDelim");
					_a_2=trim(scan(_g_{&i},2,"&GenoDelim"));
				%end;
				%else %do;
				/*nondelimited genotype*/
					_a_1=substr(_g_{&i},1,1);
					_a_2=substr(_g_{&i},2,1);
				%end;				
			%end;
			%else %do;
				_a_1=_g_{%eval(2*&i-1)}; 
				_a_2=_g_{%eval(2*&i)}; 				
			%end;
			%if %length(&WhereStmt) %then %do;
				where &WhereStmt;
			%end;
			array QuantGenotype{&ncol};
			%if %upcase(&MarkerFmt) ne %str(NUMERIC GENOTYPES) %then %do;
				array _x_{&ncol};
				array _y_{&ncol};
				if missing(_a_1) or missing(_a_2) then do;
					do j=1 to &ncol;
						QuantGenotype{j}=.;
					end;
					Genotype="";
				end;
				else do;
					do j=1 to &ncol;
						if (_a_1=scan("&Alist",j,' ')) then _x_{j}=1;
						else _x_{j}=0;    	
						if (_a_2=scan("&Alist",j,' ')) then _y_{j}=1;		
						else _y_{j}=0;
					end;	
					if _a_1 ge _a_2 then Genotype=cats(_a_1,"/",_a_2);
					else Genotype=cats(_a_2,"/",_a_1);
					*Genotype=0;
					do j=1 to &ncol;
						%if %upcase(&UseDomCoding)=YES %then %do;
							QuantGenotype{j}=(_x_{j}+_y_{j} > 0);
						%end;
						%else %do;
							QuantGenotype{j}=_x_{j}+_y_{j};
						%end;
					*	Genotype + QuantGenotype{j}*(3**(j-1));
					end;
				end;
			%end;
			/* Already have numeric genotypes - don't need to loop over alleles - must be SNP*/
			%else %do;
				if _g_{&i} in (0,1,2) then do;
					%if %upcase(&UseDomCoding)=YES %then %do;
						if _g_{&i} > 0 then QuantGenotype{1}=1;
						else QuantGenotype{1}=0;
						%if &ncol>1 %then %do;
							if _g_{&i} < 2 then QuantGenotype{2}=1;
							else QuantGenotype{2}=0;		
						%end;
					%end;
					%else %do;
						QuantGenotype{1}=_g_{&i};
					%end;
					Genotype = _g_{&i};
				end;
				else do;
					QuantGenotype{1}=.;
					%if &ncol>1 and %upcase(&UseDomCoding)=YES %then QuantGenotype{2}=.;;
					Genotype=.;
				end;		
			%end;
			%if %upcase(&UseDomCoding)=YES %then %do;
				/* Check if all rows sum to 1 (all homozygous) - if so, remove last column */
				checksum = sum(of quantgenotype:);
				if checksum ne 1 and checksum ne . then call symputx("all1",checksum);
			%end;
			keep &ByVarsOnly &OrigTraitVar &ClassVars &FixedEffVars &RandEffVars &KVars &PedID &IntEffVars &StrataVars &CensorVar QuantGenotype1-QuantGenotype&ncol Genotype;
			rename 
				%do j=1 %to &ncol;
					%let qg&j = QuantGenotype_%scan(&Alistv,&j);
					QuantGenotype&j = &&qg&j
				%end;
			;
		run;
		
		%if %upcase(&UseDomCoding)=YES and &all1 %then %let ncol = %eval(&ncol - 1);	
	%end;

	title "Marker &&Locus&i";
	%do t=1 %to &NTrait;
		%let TraitVar = &&_tv&t;
		%if %upcase(&TraitType)=BINARY %then %do;
			%if %length(&EventTrait)>0 %then %let TraitVarm=%str(&TraitVar(event="%sysfunc(dequote(&EventTrait))"));
			%else %let TraitVarm=%str(&TraitVar(event=LAST));
		%end;
		%else %if %upcase(&TraitType)=NOMINAL %then %do;
			%if %length(&RefTrait)>0 %then %let TraitVarm=%str(&TraitVar(ref="%sysfunc(dequote(&RefTrait))"));
			%else %let TraitVarm=%str(&TraitVar(ref=FIRST));
		%end;  
		%else %let TraitVarm=&TraitVar;
		
		%if (%length(&KVars) or %length(&RandEff)) and %upcase(&FixCovParms)=YES %then %do;
			%if &i=1 %then %do;
				ods output CovParms = fixcp;
				ods listing close;
				proc &WhichProc data=genotypes &ProcOption namelen=32;
					class &ClassVars &ClassOpt;
					model &TraitVarm = &FixedEff &ModOption;
					%if &kmodel=1 %then %do;
						random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
					%end;
					%if %length(&RandEff)>0 %then %do;
						random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
					%end;
					%if %length(&ParmsStmt) %then %do;
						parms &ParmsStmt;
					%end;
				quit;		

				data _null_;
					set fixcp end=_l;
					call symput('varcomp'||strip(_n_),strip(Estimate));
					if _l then call symputx("numcp",_n_);
				run;
				%let ParmsStmt&t = ;
				%let hold= %str(hold=);

				%do cp=1 %to &numcp;
					%let ParmsStmt&t = &&ParmsStmt&t &&varcomp&cp;
					%let hold = %quote(&hold &cp,);
				%end;
				%let ParmsStmt&t = %quote(&&ParmsStmt&t / &hold);
				ods listing;

			%end;
		%end;
		%else %let ParmsStmt&t = &ParmsStmt;

		%let ie=0;
		%let Interaction=;		
		%do %until(%length(%scan(&IntEffBuild,&ie,' '))=0);

			data ma;                       
				/* kemicl - reset dataset ma to empty for estimates to avoid errors when estimates don't converge */
				length Effect $32;
				delete;
			run;
			
			data ta;			 /* kemicl - reset dataset ta and or to empty for estimates to avoid errors when estimates don't converge */                      
				length Effect $32;
				delete;
			run;		

			data or;
				delete;
			run;
			
			%if (&ncol>0) %then %do;
				title2 "Trait &TraitVar";

				%if (&geno=1) %then %do;
				
					%if %sysfunc(exist(pag))=1 %then %do;
						proc datasets nolist;
							delete pag;
						quit;
					%end;
				
					title3 "Genotype Test &Interaction";
					ods output &Gtable=ma  
						&ParmEst=pag
						%if %length(&RandEff &KVars) %then CovParms=cpg;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then ConvergenceStatus= ConvStatus2 (where=(status=1));; /*--- Luciano ---*/
					proc &WhichProc data=genotypes &ProcOption namelen=32;
						class Genotype &ClassVars &ClassOpt;
						model &TraitVarm = &FixedEff Genotype &Interaction &ModOption;
						%if &kmodel=1 %then %do;
							random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
						%end;
						%if %length(&RandEff)>0 %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;
						%if %length(&&ParmsStmt&t) %then %do;
							parms &&ParmsStmt&t;
						%end;
						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;
						%if %length(&ByVars) %then %do; 
							by &ByVars;
						%end;
					quit;
					
					/*---begin: Luciano ---*/
					%NObsVars(ma);
					%if &NObs %then %do;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then %do;
							%NObsVars(ConvStatus2);
							%if &NObs %then %do;

								%let Flag_NonConvergenceG = 1;
								options label;
								
								/*data OutLib.mt_ConvStatus2; set ConvStatus2; run;*/								

								/*
								data ma;
									merge ma ConvStatus2(drop=NullModel 
												rename=(Status=StatusConvG
													Reason=ReasonNonConvG)
												);
									by &ByVars;
									label StatusConvG= "Status Non Convergence Genotype Test";
									label ReasonNonConvG= "Reason Non Convergence Genotype Test";
									retain StatusConvG ReasonNonConvG;
								run;
								*/

								data ma;
									set ma;
									_ord_= _n_;
								run;	
								
								/*data OutLib.mt_ma0; set ma; run;*/

								proc sql UNDO_POLICY= NONE;
									create table ma(drop= _ord_) as
									select * 
									from ma as a left join ConvStatus2(
												%if %upcase(&whichproc) = LOGISTIC %then drop=NullModel;
												rename=(Status=StatusConvG
													Reason=ReasonNonConvG)
												) as b
									on a.effect
									%if %length(&ByVars) %then %do;
										%let _k_=1;
										%do %while(%length(%scan(&ByVars, &_k_, ' ')));
											and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
											%let _k_= %sysevalf(&_k_ + 1);
										%end;
									%end;
									order by _ord_
									;
								quit;

								/*data OutLib.mt_ma1; set ma; run;*/
														
								/*
								data pag;
									merge pag ConvStatus2(drop=NullModel 
												rename=(Status=StatusConvG
													Reason=ReasonNonConvG)
												);
									by &ByVars;
									label StatusConvG= "Status Non Convergence Genotype Test";
									label ReasonNonConvG= "Reason Non Convergence Genotype Test";
								run;
								*/

								data pag;
									set pag;
									_ord_= _n_;
								run;
								
								/*data OutLib.mt_pag0; set pag; run;*/

								proc sql UNDO_POLICY= NONE;
									create table pag(drop= _ord_) as
									select * 
									from pag as a left join ConvStatus2(
												%if %upcase(&whichproc) = LOGISTIC %then drop=NullModel; 
												rename=(Status=StatusConvG
													Reason=ReasonNonConvG)
												) as b
									/*on a.Variable*/
									on a.&EffName
									%if %length(&ByVars) %then %do;
										%let _k_=1;
										%do %while(%length(%scan(&ByVars, &_k_, ' ')));
											and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
											%let _k_= %sysevalf(&_k_ + 1);
										%end;
									%end;
									order by _ord_
									;
								quit;

								/*data OutLib.mt_pag1; set pag; run;*/

								%if %length(&RandEff &KVars) %then %do;
									/*data OutLib.mt_cpg0; set cpg; run;*/

									/*
									data cpg;
										merge cpg ConvStatus2(drop=NullModel 
												rename=(Status=StatusConvG
													Reason=ReasonNonConvG)
												);
										by &ByVars;
										label StatusConvG= "Status Non Convergence Genotype Test";
										label ReasonNonConvG= "Reason Non Convergence Genotype Test";
									run;
									*/

									data cpg;
										set cpg;
										_ord_= _n_;
									run;

									proc sql UNDO_POLICY= NONE;
										create table cpg(drop= _ord_) as
										select * 
										from cpg as a left join ConvStatus2(
													%if %upcase(&whichproc) = LOGISTIC %then drop=NullModel; 
													rename=(Status=StatusConvG
														Reason=ReasonNonConvG)
													) as b
										on a.CovParm 
										%if %length(&ByVars) %then %do;
											%let _k_=1;
											%do %while(%length(%scan(&ByVars, &_k_, ' ')));
												and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
												%let _k_= %sysevalf(&_k_ + 1);
											%end;
										%end;
										order by _ord_
										;
									quit;

									/*data OutLib.mt_cpg1; set cpg; run;*/
								%end;
							%end;
						%end;		
					%end;
					%else %do;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then %do;
							%NObsVars(ConvStatus2);
							%if &NObs %then %do;
								%let Flag_NoEstimatesG = 1;
								
								/*data OutLib.mt_ma0; set ma; run;*/
								/*data OutLib.mt_ConvStatus2; set ConvStatus2; run;*/								
						
								data ncg_all;
									length Marker Trait $32;
									set ncg_all(in=orig) ConvStatus2(
										%if %upcase(&whichproc) = LOGISTIC %then drop=NullModel; 
											rename=(Status=StatusConvG
											Reason=ReasonNonConvG)
										);
									if not orig then do;
										Marker= "&&Locus&i";
										Trait=  "&TraitVar";
									end;
								run;
						
								/*data OutLib.mt_ncg_all; set ncg_all; run;*/
							%end;
						%end;
					%end;/*---end: Luciano ---*/
				%end;

				%if (&trend=1 and &ie=0) %then %do;
					%if %index(&WhichProc,PHREG)=0 %then %let teststmt=%str('genotypes' &qg1 1);
					%else %let teststmt=&qg1;
					
					%if (&ncol>1) %then %do;
						%do k=2 %to &ncol;
							%if %index(&WhichProc,PHREG)>0 %then %let teststmt=&teststmt, &&qg&k;
							%else %let teststmt=%str(&teststmt, &&qg&k 1);
						%end;
					%end;

					%if %Upcase(&Odds)=YES and &ncol=1 and ^%length(&RandEff &KVars) %then %do;
						%let oddstmt=&qg1;    
					%end;
					%else %if %Upcase(&Odds)=YES and &ncol=1 and %length(&RandEff &KVars) %then %do;
						%let AnyOdds=1;    
					%end;					

					title3 "Trend Test";
					
					%if %sysfunc(exist(pe))=1 %then %do;
						proc datasets nolist;
							delete pe;
						quit;
					%end;

					ods output &Ttable=ta 
						&ParmEst=pe
						%if %Upcase(&Odds)=YES and &ncol=1 and ^%length(&RandEff &KVars) %then &Otable=OR; /*kemicl - added OR table to output odds ratios for Logistic */
						%if %length(&RandEff &KVars) %then CovParms=cpt;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then ConvergenceStatus= ConvStatus3 (where=(status=1));; /*--- Luciano ---*/
					proc &WhichProc data=genotypes &ProcOption namelen=32;
						%if %length(&ClassVars) %then %do;
							class &ClassVars;
						%end;
						%if &ncol>1 %then %do;
							model &TraitVarm %if(%upcase(&TraitType)=SURVIVAL) %then *&CensorVar(&CensorValues); = &FixedEff %do k=1 %to &ncol; &&qg&k %end; &ModOption;
						%end;
						%else %do;
							model &TraitVarm %if(%upcase(&TraitType)=SURVIVAL) %then *&CensorVar(&CensorValues); = &FixedEff &qg1 &ModOption;
						%end;
						%if &kmodel=1 %then %do;
							random &KVars / type=toep(1) %if %length(&PedID) %then subject=&PedID;;
						%end;						
						
						%if %length(&RandEff) %then %do;
							random &RandEff %if %upcase(&TraitType)=NOMINAL %then group=&TraitVar;;
						%end;
						%if %length(&&ParmsStmt&t) %then %do;
							parms &&ParmsStmt&t;
						%end;
			
						%if %Upcase(&Odds)=YES and &ncol=1 and ^%length(&RandEff &KVars) %then %do;            /* kemicl - added to estimate odds ratios in Logistic Procedure */
							oddsratio %unquote(&oddstmt);
						%end;
						%if %length(&StrataVars)>0 %then %do;
							strata &StrataVars;
						%end;
						&TStmt &teststmt %if %Upcase(&Odds)=YES and &ncol=1 and %length(&RandEff &KVars) %then / alpha=&Alpha exp cl; ;
						%if %length(&ByVars) %then %do; 
							by &ByVars;
						%end;
					run;
					
					/*---begin: Luciano ---*/
					%NObsVars(ta);
					%if &NObs %then %do;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then %do;
							%NObsVars(ConvStatus3);
							%if &NObs %then %do;

								%let Flag_NonConvergenceT = 1;
								options label;

								/*data OutLib.mt_ta0; set ta; run;*/

								/*
								data ta;
									merge ta ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												);
									by &ByVars;
									label StatusConvT= "Status Non Convergence Trend Test";
									label ReasonNonConvT= "Reason Non Convergence Trend Test";
								run;
								*/

								data ta;
									set ta;
									_ord_= _n_;
								run;

								proc sql UNDO_POLICY= NONE;
									create table ta(drop= _ord_) as
									select * 
									from ta as a left join ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												) as b
									on a.Contrast 
									%if %length(&ByVars) %then %do;
										%let _k_=1;
										%do %while(%length(%scan(&ByVars, &_k_, ' ')));
											and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
											%let _k_= %sysevalf(&_k_ + 1);
										%end;
									%end;
									order by _ord_
									;
								quit;

								/*
								data pe;
									merge pe ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												);
									by &ByVars;
									label StatusConvT= "Status Non Convergence Trend Test";
									label ReasonNonConvT= "Reason Non Convergence Trend Test";
								run;
								*/


								data pe;
									set pe;
									_ord_= _n_;
								run;

								proc sql UNDO_POLICY= NONE;
									create table pe(drop= _ord_) as
									select * 
									from pe as a left join ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												) as b
									/*on a.variable*/
									on a.&EffName
									%if %length(&ByVars) %then %do;
										%let _k_=1;
										%do %while(%length(%scan(&ByVars, &_k_, ' ')));
											and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
											%let _k_= %sysevalf(&_k_ + 1);
										%end;
									%end;
									order by _ord_
									;
								quit;

								%if %Upcase(&Odds)=YES and &ncol=1 and ^%length(&RandEff &KVars) %then %do;
									/*
									data OR;
										merge OR ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												);
										by &ByVars;
										label StatusConvT= "Status Non Convergence Trend Test";
										label ReasonNonConvT= "Reason Non Convergence Trend Test";
									run;
									*/

									data OR;
										set OR;
										_ord_= _n_;
									run;

									proc sql UNDO_POLICY= NONE;
										create table OR(drop= _ord_) as
										select * 
										from OR as a left join ConvStatus3(drop=NullModel 
													rename=(Status=StatusConvT
														Reason=ReasonNonConvT)
													) as b
										on a.effect 
										%if %length(&ByVars) %then %do;
											%let _k_=1;
											%do %while(%length(%scan(&ByVars, &_k_, ' ')));
												and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
												%let _k_= %sysevalf(&_k_ + 1);
											%end;
										%end;
										order by _ord_
										;
									quit;
								%end;

								%if %length(&RandEff &KVars) %then %do;

									/*
									data cpt;
										merge cpt ConvStatus3(drop=NullModel 
												rename=(Status=StatusConvT
													Reason=ReasonNonConvT)
												);
										by &ByVars;
										label StatusConvT= "Status Non Convergence Trend Test";
										label ReasonNonConvT= "Reason Non Convergence Trend Test";
									run;	
									*/


									data cpt;
										set cpt;
										_ord_= _n_;
									run;

									proc sql UNDO_POLICY= NONE;
										create table cpt(drop= _ord_) as
										select * 
										from cpt as a left join ConvStatus3(drop=NullModel 
													rename=(Status=StatusConvT
														Reason=ReasonNonConvT)
													) as b
										on a.CovParm 
										%if %length(&ByVars) %then %do;
											%let _k_=1;
											%do %while(%length(%scan(&ByVars, &_k_, ' ')));
												and a.%scan(&ByVars, &_k_, ' ')= b.%scan(&ByVars, &_k_, ' ')
												%let _k_= %sysevalf(&_k_ + 1);
											%end;
										%end;
										order by _ord_
										;
									quit;
								%end;
							%end;
						%end;
					%end;
					%else %do;
						%if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then %do;
							%NObsVars(ConvStatus3);
							%if &NObs %then %do;
								%let Flag_NoEstimatesT = 1;
								
								/*data OutLib.mt_ConvStatus3; set ConvStatus3; run;*/
								/*data OutLib.mt_ta0; set ta; run;*/
								
								data nct_all;
									length Marker Trait $32;
									set nct_all(in= orig) ConvStatus3(
											%if %upcase(&whichproc) = LOGISTIC %then drop=NullModel; 
											rename=(Status=StatusConvT
											Reason=ReasonNonConvT)
										);
									if not orig then do;
										Marker= "&&Locus&i";
										Trait=  "&TraitVar";
									end;
								run;
						
								/*data OutLib.mt_nct_all; set nct_all; run;*/					
							%end;
						%end;
					%end;
					/*---end: Luciano ---*/
				%end;
				/* wehass comment out    	%if (&syserr>4) %then %goto exit;*/
			%end;
			
			title3;
			%if (&geno=1) %then %do;
			
				%if %sysfunc(exist(pag))=0 and %sysfunc(exist(cpg))=0 %then %goto skip1;

				%if %sysfunc(exist(pag)) %then %do;
					data ma_a; 
						length Trait $32;
						set ma_a(in=orig) ma;
						%if &ie>0 %then %do;
							if orig=1 or (index(Effect,"Geno")>0 and length(Effect)>11); 
						%end;
						%else %do;
							if index(Effect,"Geno")>0; 
						%end;
						if not orig then do;
							Trait = "&TraitVar";
							_order_=&i;
						end;
						label Prob&Stat=' ';
					run;
					
					/*data OutLib.mt_ma_a; set ma_a; run;*/

					data pag_all;	
						%if %length(&IntEffBuild) %then %do;
							length Interaction $32;
						%end;
						set pag_all(in=orig) pag;
						if orig=0 then do;
							Trait="&TraitVar";
							%if %length(&IntEffBuild) %then %do;
								Interaction="&Interaction";
							%end;
							_order_=&i;
						end;
					run;		
				%end;
				
				%if %length(&RandEff &KVars) and %sysfunc(exist(cpg)) %then %do;
					data cpg_all;
						%if %length(&IntEffBuild) %then %do;
							length Interaction $32;
						%end;					
						set cpg_all(in=orig) cpg;
						if orig=0 then do;
							Trait="&TraitVar";
							_order_=&i;
							%if %length(&IntEffBuild) %then %do;
								Interaction="&Interaction";
							%end;
						end;
					run;
				%end;				
			%end;
			
			%skip1:

			%if (&trend=1 and &ie=0) %then %do;      
				
				%if %sysfunc(exist(pe))=0 and %sysfunc(exist(cpt))=0 %then %goto skip;
						
				%if %sysfunc(exist(pe)) %then %do;
					%if &ncol=1 %then %do;

						data ta;
							length Allele $32;
							Allele=substr("&qg1",15);
							set pe(where=(&EffName contains"QuantGenotype") keep=&EffName Estimate StdErr);
							set ta;
							drop &EffName;
							rename Estimate = Estimate_Trend StdErr = StdErr_Trend;
						run;
					%end;

					data ta_all;  
						length Trait $32;
						set ta_all(in=orig) ta;
						if not orig then do;
							Trait = "&TraitVar";
							_order_=&i;
							Effect = "Genotype";
						end;
						label Prob&Stat=' ';
						drop label contrast statement;
					run;

					data pat_all;	
						set pat_all(in=orig) pe;
						if orig=0 then do;
							Trait="&TraitVar";
							_order_=&i;
						end;
					run;	
				%end;
				
				%if %length(&RandEff &KVars) and %sysfunc(exist(cpt)) %then %do;
					data cpt_all;
						set cpt_all(in=orig) cpt;
						if orig=0 then do;
							Trait="&TraitVar";
							_order_=&i;

						end;
					run;
				%end;				

				/*kemicl - Added new dataset to contain odds ratio computation for Logistic Procedure - to merge in to final dataset */
				%if %Upcase(&Odds)=YES and &ncol=1 and ^%length(&RandEff &KVars) %then %do; 
					data or_all;
						length Trait $32;
						set or_all(in=orig) or;
						if not orig then do;
							Trait = "&TraitVar";
							_order_=&i;
							Log_OddsRatio_Trend=log(OddsRatioEst);
						end;
						drop effect unit;
					run;
				/* */		 
				%end;
			%end;		
			
			%skip:

			%let ie=%eval(&ie+1);   
			%let Interaction=%scan(&IntEffBuild,&ie,' ');
		%end; /* end loop over interactions */
	%end;  /* end loop over traits */
%end; /* end loop over markers */

%if %Upcase(&Odds)=YES and ^%length(&RandEff &KVars) %then %do;
	%Nobsvars(or_all);
	%if &nobs=0 %then %let Odds=NO;
%end;

%put Flag_NoEstimatesG= &Flag_NoEstimatesG;
%put Flag_NoEstimatesT= &Flag_NoEstimatesT;

%let SkipPvalues= 0;
%if (&geno=1) %then %do;	/* Kemicl added to throw error if nothing is estimable */
	%NObsVars(ma_a);
	%if ^&nobs %then %do;
		%if &Flag_NoEstimatesG = 0 %then %do;
			%put ERROR:  PROC &WhichProc generated no estimates for the Genotype Test.  There may be excessive missing data or the contrast may be nonestimable.;
			%if (&trend ne 1) %then %do;		
				%let exiterror = 1;
				%goto exit;
			%end;
			%else %do;
				%let geno=0;
				%let QTests=Trend;
			%end;
		%end;
		%else %do;
			%put WARNING:  PROC &WhichProc generated no estimates for the Genotype Test.  There may be excessive missing data or the contrast may be nonestimable.;
			%if (&trend ne 1) %then %do;
				%let SkipPvalues= 1;
				%goto NoneConv;
			%end;
			%else %do;
				%let geno=0;
				%let QTests=Trend;
			%end;		
		%end;
	%end;
%end;

%if (&trend=1) %then %do;
	%NObsVars(ta_all);
	%if ^&nobs %then %do;
		%if &Flag_NoEstimatesT = 0 %then %do;
			%put ERROR:  PROC &WhichProc generated no estimates for the Trend Test.  There may be excessive missing data or the contrast may be nonestimable.;
			%if (&geno ne 1) %then %do;
				%let exiterror = 1;
				%goto exit;
			%end;
			%else %do;
				%let trend=0;
				%let QTests=Genotype;
				%let Odds = No;
			%end;	
		%end;
		%else %do;
			%put WARNING:  PROC &WhichProc generated no estimates for the Trend Test.  There may be excessive missing data or the contrast may be nonestimable.;
			%if (&geno ne 1) %then %do;
				%let SkipPvalues= 1;
				%goto NoneConv;
			%end;
			%else %do;
				%let trend=0;
				%let QTests=Genotype;
				%let Odds = No;
			%end;
		%end;
	%end;
%end;

%if %Upcase(&Odds)=YES and ^%length(&RandEff &KVars) %then %do;      /* kemicl - merge in OR table for Odds ratio when no Random effect */
	 data ta_all;
	 	merge ta_all or_all;
	 	by _order_ ;
	 run;
%end;							      

%if %Upcase(&Odds)=YES and %length(&RandEff &KVars)  %then %do;       /* kemicl -Rename Odds Ratio estimates when there is a Random effect */
	data ta_all;
		set ta_all;
		%if &AnyOdds=1 %then %do;
			Log_OddsRatio_Trend=log(ExpEstimate);
			rename ExpEstimate=OddsRatioEst ExpUpper=UpperCL ExpLower=LowerCL;
		%end;
		%else %do;
			%let Odds=NO;
			drop LowerCL UpperCL Log_Oddsratio_Trand Oddsratio_Trend;
		%end;
		drop stderr contrast label estimate alpha lower upper;
	run;
%end; 

%let FillByVars= &ByVars;          /* kemicl - Create new macro to keep original By variables to build output table */

%global JByOdds;		   /* kemicl - Create new macro JByOdds for Odds Ratio plots to not include interaction eff */
%let JByOdds= &ByVars;

%if &NTrait>1 %then %do;
	/* This stuff for Venns */
	%if %length(&IntEff)>0 %then %do;
		%let NonTraitByVars=&NonTraitByVars Effect;
		%let JNonTraitByVars = &JNonTraitByVars :Effect;
		%let OrigNbyval = %eval(&OrigNbyval*&NumInt);
	%end;
	
	%if %length(&ByVars)>0 %then %do;
	 %let ByVars = Trait &ByVars;
	 %let ByVarsOnly = Trait &ByVarsOnly;
	 %let JBy = %str(Trait,&JBy);
	 %let JByOdds= &JBy; 
	%end;
	%else %do;
	 %let ByVars = Trait;
	 %let ByVarsOnly = Trait;
	 %let JBy=Trait;
	 %let JByOdds= &JBy; 
	%end;
	%let Nbyval = %eval(&Nbyval*&Ntrait);
%end;

/* kemicl - new format for output data - build effect table */

%if (&geno=1) %then %do;
  proc sort data=Ma_a out=effect_map(keep=Effect Trait &FillByVars) nodupkey;
 	by Effect &Byvars;
  run;
%end;

%else %do;
  proc sort data=Ta_all out=effect_map(keep=Effect Trait &FillByVars) nodupkey;
	by Effect &Byvars;
  run;
%end;

data effect_map;
 	set effect_map;
 	do e=1 to &nummark;
 		output;
 	end;
 	rename e=_order_;
 run;
 	
proc sort data=effect_map;
	by _order_;
run;

data mapdata2;                     
	merge mapdata effect_map;
	by _order_;
run;

%if &NTrait>1 or %length(&IntEff)>0 or %length(&FillByVars)>0 %then %do;
		
		proc sort data=mapdata2;
			by _order_ &ByVars Effect ;
		run;
%end;

%if (&geno=1) %then %do;

	%if &NTrait>1 or %length(&IntEff)>0 or %length(&FillByVars)>0 %then %do;
		proc sort data=ma_a;
			by _order_ &ByVars Effect ;
		run;
	%end;

	data ma_a;					
		merge mapdata2 ma_a;                               /* kemicl - restructured to use mapdata2 and rename DF - from SNPTrait*/
		by _order_ &ByVars Effect;
		%if %index(%lowcase(&TraitType),continuous) %then %do;
			rename NumDF=NumDF_Genotype DenDF=DenDF_Genotype FValue=FValue_Genotype Prob&Stat=ProbGenotype ; 	
		%end;
		%else %if %Upcase(&Stat)=T %then %do;       /* kemicl - added to rename Genotype F test correctly*/  
			if FValue<0 then ProbF=.;
			rename NumDF=NumDF_Genotype DenDF=DenDF_Genotype FValue=FValue_Genotype ProbF=ProbGenotype; 	
		%end;
		%else %do;
			rename DF=DF_Genotype WaldChiSq=WaldChiSq_Genotype Prob&Stat=ProbGenotype;
		%end;		
	run;
	
	proc sort data=ma_a;
		by &AnnoGroup &AnnoLoc &ByVars Effect;
 	run;

%end;

%if (&trend=1) %then %do;
	%if &NTrait>1 or %length(&IntEff)>0 or %length(&FillByVars)>0 %then %do;
		proc sort data=ta_all;
			by _order_ &ByVars Effect;
		run;
	%end;
	
	data ta_all;                                        /* kemicl - restructured to use mapdata2 and rename DF - from SNPTrait*/  
		merge mapdata2(where=(Effect="Genotype")) ta_all;
		by _order_ &ByVars ;
		/* %if &TraitType=SContinuous %then drop effect;; */        	
		%if %index(%lowcase(&TraitType),continuous) %then %do;
			rename NumDF=NumDF_Trend DenDF=DenDF_Trend FValue=FValue_Trend Prob&Stat=ProbTrend;
		%end;
		%else %if %Upcase(&Stat)=T %then %do;                  /* kemicl - added to convert t test to F test */
			if DF=. then NumDF_Trend=.;
			else NumDF_Trend=1;
			DenDF_Trend=DF;
			FValue_Trend=tValue**2;
			ProbTrend=Prob&Stat;
			drop DF tValue Prob&Stat;
		%end;	
		%else %do;
			rename DF=DF_Trend WaldChiSq=WaldChiSq_Trend Prob&Stat=ProbTrend;
		%end;
		%if %Upcase(&Odds)=YES %then %do;
				rename OddsRatioEst=OddsRatio_Trend;
		%end;		
	run; 
	
	proc sort data=ta_all;
  		by &AnnoGroup &AnnoLoc &ByVars;
 	run;

%end;

%if &geno=1 %then %do;
	%if &NTrait>1 or %length(&IntEff)>0 %then %do;
		proc sort data=pag_all;
			by _order_;
		run;
		
		%if %length(&RandEff &KVars) %then %do;
			proc sort data=cpg_all;
				by _order_;
			run;
		%end;		
	%end;
	data &SNPTraitAssoc_PEG;
		merge mapdata pag_all;
		by _order_;
		drop _order_ _ESTTYPE_;
	run;
	
	%PackageInsertDataset(Genotype parameter estimate data from &ProcessName, OutLib, &SNPTraitAssoc_PEGName,_type=other);
	
	%if %length(&RandEff &KVars) %then %do;
		data &SNPTraitAssoc_CPG;
			merge mapdata cpg_all;
			by _order_;
			drop _order_;
		run;

		%PackageInsertDataset(Genotype covariance parameter data from &ProcessName, OutLib, &SNPTraitAssoc_CPGName, _type=other);
		
	%end;	
%end;
	
%if &trend=1 %then %do;
	%if &NTrait>1 or %length(&IntEff)>0 %then %do;
		proc sort data=pat_all;
			by _order_;
		run;
		
		%if %length(&RandEff &KVars) %then %do;
			proc sort data=cpt_all;
				by _order_;
			run;
		%end;			
	%end;
		
	data &SNPTraitAssoc_PET;
		merge mapdata pat_all;
		by _order_;
		drop _order_ _ESTTYPE_;
	run;	

	%PackageInsertDataset(Trend parameter estimate data from &ProcessName, OutLib, &SNPTraitAssoc_PETName, _type=other);
	
	%if %length(&RandEff &KVars) %then %do;
		data &SNPTraitAssoc_CPT;
			merge mapdata cpt_all;
			by _order_;
			drop _order_;
		run;

		%PackageInsertDataset(Trend covariance parameter data from &ProcessName, OutLib, &SNPTraitAssoc_CPTName, _type=other);
		
	%end;		
			
%end;

/* kemicl - added from SNPTrait */ 
%let mergeds=;
%if &geno=1 %then %let mergeds=ma_a;
%if &trend=1 %then %let mergeds = &mergeds ta_all;

data ma_all;
	merge &mergeds;
  	by &AnnoGroup &AnnoLoc &ByVars Effect;
run;

%if %Upcase(&Stat)=T %then %let Stat=F;    /* kemicl - Change this due to different Stats used with Proc GLIMMIX for output plots*/

title;
title2;
title3;

%let PlotVar1=%scan(&QTests,1);
%let PlotVar2=%scan(&QTests,2);

%if %length(&IntEff)>0 %then %do;
	%if %length(&ByVars)>0 %then %do;	
		%let ByVars=&ByVars Effect;
		%let JBy = %str(&JBy,Effect);
	%end;
	%else %do;
	 %let ByVars = Effect;
	 %let JBy=Effect;
	%end;
	%let Nbyval = %eval(&Nbyval*&NumInt);
%end;

/*data OutLib.mt_ma_all; set ma_all; run;*/

proc sort data=ma_all out=&MarkerTraitAssoc_OutData;
 by &ByVars &AnnoGroup &AnnoLoc;
run;

/*begin: Luciano*/
%NoneConv:;
%put Flag_NoEstimatesG= &Flag_NoEstimatesG;
%put Flag_NoEstimatesT= &Flag_NoEstimatesT;
%*if %upcase(&whichproc) = LOGISTIC or %upcase(&whichproc) = GLIMMIX %then %do;
%if (&Flag_NonConvergenceG = 1 or &Flag_NonConvergenceT=1) or (&Flag_NoEstimatesG = 1 or &Flag_NoEstimatesT = 1) %then %do;
	%global SNPTraitAssoc_NCT;
	%let SNPTraitAssoc_NCT=;
	%PrepOutDS(SNPTraitAssoc_NCT,SNPTraitAssoc_NCTName,&OutName._nct);
			
	%if &SkipPvalues = 0 %then %do;
		%if &Flag_NoEstimatesG = 1 %then %do;
			proc sql UNDO_POLICY= NONE;
				create table &MarkerTraitAssoc_OutData as
				select * 
				from &MarkerTraitAssoc_OutData as a full join ncg_all as b
				on a.Marker= b.Marker
				;
			quit;
			
			/*
			data &MarkerTraitAssoc_OutData;
				set &MarkerTraitAssoc_OutData ncg_all;
			run;
			*/
			%let Flag_NonConvergenceG= 1;
		%end;
	
		%if &Flag_NoEstimatesT = 1 %then %do;
			proc sql UNDO_POLICY= NONE;
				create table &MarkerTraitAssoc_OutData as
				select * 
				from &MarkerTraitAssoc_OutData as a full join nct_all as b
				on a.Marker= b.Marker
				;
			quit;
			
			/*
			data &MarkerTraitAssoc_OutData;
				set &MarkerTraitAssoc_OutData nct_all;
			run;
			*/
			%let Flag_NonConvergenceT= 1;
		%end;
		
		%if &Flag_NoEstimatesG = 1 or &Flag_NoEstimatesT = 1 %then %do;
			proc sort data=&MarkerTraitAssoc_OutData;
				by &ByVars &AnnoGroup &AnnoLoc;
			run;
		%end;
	
		%if (&Flag_NonConvergenceG = 1 or &Flag_NonConvergenceT = 1) %then %do;			
			%let NCFwhere = ;
			%if &Flag_NonConvergenceG %then %do;
				%let NCFwhere = %nrbquote(StatusConvG = 1);
				%if &Flag_NonConvergenceT %then %do;
					%let NCFwhere = %nrbquote(&NCFwhere or StatusConvT = 1);
				%end;
			%end;
			%else %do;
				%let NCFwhere = %nrbquote(StatusConvT =1);
			%end;
		
			data &SNPTraitAssoc_NCT;
				set &MarkerTraitAssoc_OutData;
				where %unquote(&NCFwhere);
			run;
		
			%PackageInsertDataset(Non-convergent Tests from &ProcessName, outlib, &OutName._nct, yes, yes, _type=wide);
		%end;
	%end;
	%else %do;
		
		data &SNPTraitAssoc_NCT;
			set 
				%if &Flag_NoEstimatesG = 1 %then ncg_all;
				%if &Flag_NoEstimatesT = 1 %then nct_all;
			;
		run;
	
		%PackageInsertDataset(Non-convergent Tests from &ProcessName, outlib, &OutName._nct, yes, no, no, _type=wide);		

		%global jsl_after_tabbedreport;
		%let jsl_after_tabbedreport = &clientoutpath.&ProcessName Message for &OutName..jsl;
	
		* create JSL file;
		%global SkipPvaluesScript;
		%PrepOutFile(SkipPvaluesScript,SkipPvaluesScriptName,&ProcessName Message for &OutName..jsl);
		
		%let upcase_nct= %upcase(&SNPTraitAssoc_NCTName);
		
		options noquotelenmax;
		data _null_;
			file "&SkipPvaluesScript" lrecl=32767;
			put "//!";
			put "Names Default to Here(1);";
			put "wait(0.5);";		
			put "Here:_nwMT00_= New Window(""&ProcessName Warning Message"",showmenu(0),showtoolbars(0),";
			%if &Flag_NoEstimatesG and &Flag_NoEstimatesT = 0 %then %do;
				put " TextBox(""NOTE: PROC &WhichProc generated no estimates for the Genotype Test because of convergence problems. ""|| ";
			%end;
			%else %if &Flag_NoEstimatesG = 0 and &Flag_NoEstimatesT %then %do;
				put " TextBox(""NOTE: PROC &WhichProc generated no estimates for the Trend Test because of convergence problems. ""|| ";
			%end;
			%else %if &Flag_NoEstimatesG and &Flag_NoEstimatesT %then %do;
				put " TextBox(""NOTE: PROC &WhichProc generated no estimates for the Genotype and Trend Tests because of convergence problems. ""|| ";
			%end;
			put "		""A data set named &upcase_nct..sas7bdat containing the Non-convergent Tests with respective reason for non-convergency is included in the output folder.""|| ";
			put "		""\!N\!N"",";
			put "		<<Font Color(5),<<Set Font Size(10),<<Set Wrap(400),";
			put " ),";					
			put ");";
			put "if(!IsEmpty(Here:TabWinList),";
			put "    InsertInto(Here:TabWinList,0);";
			put "    Here:TabWinList[Here:nwcounter]=Here:_nwMT00_;";
			put "    Here:nwcounter++; ";		
			put "	);";
		run;
	
		%PackageInsertFile(jsl code from &ProcessName, &OutPath, &SkipPvaluesScriptName, ,text/plain, yes, yes);	
		%goto exit;
	%end;
%end;
/*end: Luciano*/

%if %length(&PlotVar2)>4 %then %let QTests=&PlotVar1 &PlotVar2;
%else %let QTests = &PlotVar1;

%Psmooth(&QTests, &MarkerTraitAssoc_OutData, &MarkerTraitAssoc_OutDataName);

 %if (&syserr>4) %then %goto exit;
 %PackageInsertDataset(P-value data from &ProcessName, outlib, &MarkerTraitAssoc_OutDataName, yes, yes, _type=anno);
 
%global KVars;
%let KVars=;

%if &NByVal > 1 or %length(&IntEff) %then %let ssr=0;
%else %let ssr=1;
%JSLPvalMap(Marker-Trait Association, &MarkerTraitAssocScript, &MarkerTraitAssocScriptName, &MarkerTraitAssoc_OutDataName, _subsetdd=&ssr);

%exit:;

%*goto exitcheck; /*checking code run from SAS: Luciano*/

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%exitcheck:; /*checking code run from SAS: Luciano*/

%mend;

* run the macro;
%MarkerTraitAssociation;

